@@ -1,22 +1,18 @@
 /*
- *  $Id: datachooser.c 23637 2021-05-05 07:32:51Z yeti-dn $
- *  Copyright (C) 2006-2019 David Necas (Yeti), Petr Klapetek.
+ *  $Id: datachooser.c 24088 2021-09-07 12:12:49Z yeti-dn $
+ *  Copyright (C) 2006-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -79,8 +75,8 @@ typedef struct {
     GwyDataWatchEventType event_type;
 } GwyDataChooserEvent;
 
-/* To avoid "row-changed" when values are actually filled in.  Filling rows
- * inside a cell renderer causes an obscure Gtk+ crash. */
+/* To avoid "row-changed" when values are actually filled in.  Filling rows inside a cell renderer causes an obscure
+ * Gtk+ crash. */
 typedef struct {
     GdkPixbuf *thumb;
     gchar *name;
@@ -146,15 +142,13 @@ gwy_app_data_id_get_type(void)
 /**
  * gwy_app_data_id_new:
  * @datano: Numeric identifier of data container.  Zero is used for none.
- * @id: Numeric identifier of a specific data item, such as channel or graph
- *      number.  Value -1 is used for none.
+ * @id: Numeric identifier of a specific data item, such as channel or graph number.  Value -1 is used for none.
  *
  * Creates a new data identifier on heap.
  *
  * This is mostly useful for language bindings.
  *
- * Returns: Newly allocated data id.  It must be freed with
- *          gwy_app_data_id_free().
+ * Returns: Newly allocated data id.  It must be freed with gwy_app_data_id_free().
  *
  * Since: 2.47
  **/
@@ -176,8 +170,7 @@ gwy_app_data_id_new(gint datano,
  *
  * This is mostly useful for language bindings.
  *
- * Returns: Newly allocated data id.  It must be freed with
- *          gwy_app_data_id_free().
+ * Returns: Newly allocated data id.  It must be freed with gwy_app_data_id_free().
  *
  * Since: 2.47
  **/
@@ -266,16 +259,10 @@ gwy_data_chooser_init(GwyDataChooser *chooser)
     GtkTreeIter iter;
     Proxy *proxy;
 
-    chooser->store = gtk_list_store_new(MODEL_NCOLUMNS,
-                                        GWY_TYPE_CONTAINER,
-                                        G_TYPE_INT,
-                                        G_TYPE_POINTER);
-    chooser->filter = gtk_tree_model_filter_new(GTK_TREE_MODEL(chooser->store),
-                                                NULL);
+    chooser->store = gtk_list_store_new(MODEL_NCOLUMNS, GWY_TYPE_CONTAINER, G_TYPE_INT, G_TYPE_POINTER);
+    chooser->filter = gtk_tree_model_filter_new(GTK_TREE_MODEL(chooser->store), NULL);
     filter = GTK_TREE_MODEL_FILTER(chooser->filter);
-    gtk_tree_model_filter_set_visible_func(filter,
-                                           gwy_data_chooser_is_visible, chooser,
-                                           NULL);
+    gtk_tree_model_filter_set_visible_func(filter, gwy_data_chooser_is_visible, chooser, NULL);
 
     /* Create `none' row */
     proxy = g_new0(Proxy, 1);
@@ -297,8 +284,7 @@ gwy_data_chooser_init(GwyDataChooser *chooser)
 /**
  * gwy_data_chooser_set_active:
  * @chooser: A data chooser.
- * @data: Container to select, %NULL to select none (if the chooser contains
- *        `none' item).
+ * @data: Container to select, %NULL to select none (if the chooser contains `none' item).
  * @id: Id of particular data to select in @data.
  *
  * Selects a data in a data chooser.
@@ -353,8 +339,7 @@ gwy_data_chooser_set_active(GwyDataChooser *chooser,
  *
  * Gets the selected item in a data chooser.
  *
- * Returns: The container selected data lies in, %NULL if nothing is selected
- *          or `none' item is selected.
+ * Returns: The container selected data lies in, %NULL if nothing is selected or `none' item is selected.
  **/
 GwyContainer*
 gwy_data_chooser_get_active(GwyDataChooser *chooser,
@@ -417,8 +402,7 @@ gwy_data_chooser_set_active_id(GwyDataChooser *chooser,
  *
  * Gets the selected item in a data chooser as numerical identifiers.
  *
- * Returns: %TRUE if any actual data item is selected.  %FALSE is nothing is
- *          selected or the ‘none’ item is selected.
+ * Returns: %TRUE if any actual data item is selected.  %FALSE is nothing is selected or the ‘none’ item is selected.
  *
  * Since: 2.41
  **/
@@ -478,11 +462,10 @@ gwy_data_chooser_is_visible(GtkTreeModel *model,
  *
  * Sets the filter applied to a data chooser.
  *
- * The display of an item corresponding to no data is controlled by
- * gwy_data_chooser_set_none(), @filter function is only called for real data.
+ * The display of an item corresponding to no data is controlled by gwy_data_chooser_set_none(), @filter function is
+ * only called for real data.
  *
- * Use gwy_data_chooser_refilter() to update the list if the filter depends on
- * external state and that changes.
+ * Use gwy_data_chooser_refilter() to update the list if the filter depends on external state and that changes.
  **/
 void
 gwy_data_chooser_set_filter(GwyDataChooser *chooser,
@@ -507,10 +490,9 @@ gwy_data_chooser_set_filter(GwyDataChooser *chooser,
  *
  * Gets the tree model filter used in a data chooser.
  *
- * In general, you should not access the filter directly.  An exception being
- * gtk_tree_model_filter_refilter() when the filtering functions given in
- * gwy_data_chooser_set_filter() depends on external state and that state
- * changes.  However, gwy_data_chooser_refilter() is usually more useful.
+ * In general, you should not access the filter directly.  An exception being gtk_tree_model_filter_refilter() when
+ * the filtering functions given in gwy_data_chooser_set_filter() depends on external state and that state changes.
+ * However, gwy_data_chooser_refilter() is usually more useful.
  *
  * Returns: The #GtkTreeModelFilter object used by the chooser.
  **/
@@ -527,13 +509,11 @@ gwy_data_chooser_get_filter(GwyDataChooser *chooser)
  *
  * Reruns the filter function of a data chooser.
  *
- * This function is useful when the filtering functions given in
- * gwy_data_chooser_set_filter() depends on external state and that state
- * changes.
+ * This function is useful when the filtering functions given in gwy_data_chooser_set_filter() depends on external
+ * state and that state changes.
  *
- * If the currently selected item becomes filtered out the chooser selects the
- * no-data item if enabled.  If the no-data item is disabled but the list
- * contains some items then an arbitrary item is selected.  At present, this
+ * If the currently selected item becomes filtered out the chooser selects the no-data item if enabled.  If the
+ * no-data item is disabled but the list contains some items then an arbitrary item is selected.  At present, this
  * means the first item in the list.
  *
  * Since: 2.41
@@ -562,8 +542,8 @@ gwy_data_chooser_refilter(GwyDataChooser *chooser)
  *
  * Gets the label of the item corresponding to no data.
  *
- * Returns: The label corresponding to no data, an empty string for the default
- *          label and %NULL if the chooser does not display the no-data item.
+ * Returns: The label corresponding to no data, an empty string for the default label and %NULL if the chooser does
+ *          not display the no-data item.
  **/
 const gchar*
 gwy_data_chooser_get_none(GwyDataChooser *chooser)
@@ -576,9 +556,8 @@ gwy_data_chooser_get_none(GwyDataChooser *chooser)
 /**
  * gwy_data_chooser_set_none:
  * @chooser: A data chooser.
- * @none: Label to use for item corresponding to no data. Passing %NULL,
- *        disables such an item, an empty string enables it with
- *        the default label.
+ * @none: Label to use for item corresponding to no data. Passing %NULL, disables such an item, an empty string
+ *        enables it with the default label.
  *
  * Sets the label of the item corresponding to no data.
  **/
@@ -593,9 +572,7 @@ gwy_data_chooser_set_none(GwyDataChooser *chooser,
     gwy_assign_string(&chooser->none_label, none);
 
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(chooser->store), &iter);
-    gtk_tree_model_get(GTK_TREE_MODEL(chooser->store), &iter,
-                       MODEL_COLUMN_PROXY, &proxy,
-                       -1);
+    gtk_tree_model_get(GTK_TREE_MODEL(chooser->store), &iter, MODEL_COLUMN_PROXY, &proxy, -1);
     gwy_assign_string(&proxy->name,
                       chooser->none_label && *chooser->none_label
                       ? chooser->none_label
@@ -637,8 +614,7 @@ gwy_data_chooser_remove_events(GwyDataChooser *chooser)
     chooser->update_id = 0;
 }
 
-/* Returns TRUE if we find the item exactly, otherwise it finds the position
- * where to insert the item at */
+/* Returns TRUE if we find the item exactly, otherwise it finds the position where to insert the item at */
 static gboolean
 gwy_data_chooser_find_data(GwyDataChooser *chooser,
                            GwyContainer *container,
@@ -687,32 +663,26 @@ gwy_data_chooser_process_events(gpointer user_data)
         gint position;
         Proxy *proxy;
 
-        found = gwy_data_chooser_find_data(chooser, event->container, event->id,
-                                           &position);
-        gwy_debug("id %u, type %u, container %p, found %d",
-                  event->id, event->event_type, event->container, found);
+        found = gwy_data_chooser_find_data(chooser, event->container, event->id, &position);
+        gwy_debug("id %u, type %u, container %p, found %d", event->id, event->event_type, event->container, found);
 
         if (found) {
             if (event->event_type == GWY_DATA_WATCH_EVENT_ADDED) {
-                g_warning("Attempted to add an item already present %p, %d.",
-                          event->container, event->id);
+                g_warning("Attempted to add an item already present %p, %d.", event->container, event->id);
                 event->event_type = GWY_DATA_WATCH_EVENT_CHANGED;
             }
         }
         else {
             if (event->event_type == GWY_DATA_WATCH_EVENT_CHANGED) {
-                g_warning("Attempted to change an item not present yet %p, %d.",
-                          event->container, event->id);
+                g_warning("Attempted to change an item not present yet %p, %d.", event->container, event->id);
                 event->event_type = GWY_DATA_WATCH_EVENT_ADDED;
             }
             if (event->event_type == GWY_DATA_WATCH_EVENT_REMOVED) {
-                g_warning("Attempted to remove a nonexistent item %p, %d.",
-                          event->container, event->id);
+                g_warning("Attempted to remove a nonexistent item %p, %d.", event->container, event->id);
                 continue;
             }
         }
 
-
         if (event->event_type == GWY_DATA_WATCH_EVENT_ADDED) {
             proxy = g_new0(Proxy, 1);
             gtk_list_store_insert_with_values(chooser->store, &iter, position,
@@ -727,21 +697,18 @@ gwy_data_chooser_process_events(gpointer user_data)
             GWY_OBJECT_UNREF(proxy->thumb);
             g_free(proxy->name);
             proxy->name = NULL;
-            /* XXX XXX XXX: We cannot emit "row-changed" on the tree model here
-             * for some reason.  If we do, we get CRITICAL message
+            /* XXX XXX XXX: We cannot emit "row-changed" on the tree model here for some reason.  If we do, we get
+             * CRITICAL message
              *
              * gtk_menu_attach: assertion 'left_attach < right_attach' failed
              *
-             * because property "right_attach" of the combo box menu is -1 now
-             * (like unattached?), it gets typecast to guint as G_MAXUINT and
-             * things only go downhill from there.
+             * because property "right_attach" of the combo box menu is -1 now (like unattached?), it gets typecast to
+             * guint as G_MAXUINT and things only go downhill from there.
              *
-             * This does not occur in Fedora, but does in Ubuntu.  No RedHat
-             * patch seems relevant, so who knows...
+             * This does not occur in Fedora, but does in Ubuntu.  No RedHat patch seems relevant, so who knows...
              *
-             * Work around by pretending the row did not change (we can do that
-             * because the tree model only contains a pointer) and asking the
-             * widget to redraw itself. */
+             * Work around by pretending the row did not change (we can do that because the tree model only contains
+             * a pointer) and asking the widget to redraw itself. */
             gtk_widget_queue_draw(GTK_WIDGET(chooser));
         }
         else if (event->event_type == GWY_DATA_WATCH_EVENT_REMOVED) {
@@ -795,10 +762,9 @@ gwy_data_chooser_receive_event(GwyContainer *data,
         }
         else if (event_type == GWY_DATA_WATCH_EVENT_CHANGED) {
             if (event->event_type == GWY_DATA_WATCH_EVENT_REMOVED)
-                g_warning("Got event CHANGED after REMOVED on %p, %d.",
-                          data, id);
-            /* Keep the item type.   ADDED is as good as CHANGED for the update
-             * processing and it permits removal when a later REMOVED comes. */
+                g_warning("Got event CHANGED after REMOVED on %p, %d.", data, id);
+            /* Keep the item type.   ADDED is as good as CHANGED for the update processing and it permits removal when
+             * a later REMOVED comes. */
         }
         else {
             g_assert_not_reached();
@@ -809,8 +775,7 @@ gwy_data_chooser_receive_event(GwyContainer *data,
         event->container = data;
         event->id = id;
         event->event_type = event_type;
-        /* Use g_list_prepend() for efficiency; we revert the order when we
-         * process the items. */
+        /* Use g_list_prepend() for efficiency; we revert the order when we process the items. */
         chooser->events = g_list_prepend(chooser->events, event);
     }
 
@@ -871,8 +836,7 @@ gwy_data_chooser_render_name(G_GNUC_UNUSED GtkCellLayout *layout,
     }
     g_object_set(renderer,
                  "text", proxy->name,
-                 "style",
-                 proxy->is_none ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL,
+                 "style", proxy->is_none ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL,
                  NULL);
 }
 
@@ -894,8 +858,7 @@ gwy_data_chooser_render_icon(G_GNUC_UNUSED GtkCellLayout *layout,
                            MODEL_COLUMN_CONTAINER, &container,
                            MODEL_COLUMN_ID, &id,
                            -1);
-        proxy->thumb = chooser->get_thumbnail(container, id,
-                                              ICON_SIZE, ICON_SIZE);
+        proxy->thumb = chooser->get_thumbnail(container, id, ICON_SIZE, ICON_SIZE);
         g_object_unref(container);
     }
     g_object_set(renderer, "pixbuf", proxy->thumb, NULL);
@@ -912,8 +875,7 @@ gwy_data_chooser_channels_setup_watcher(GwyDataChooser *chooser)
 {
     gint id;
 
-    id = gwy_app_data_browser_add_channel_watch(gwy_data_chooser_receive_event,
-                                                chooser);
+    id = gwy_app_data_browser_add_channel_watch(gwy_data_chooser_receive_event, chooser);
     chooser->watcher_id = id;
 }
 
@@ -922,8 +884,8 @@ gwy_data_chooser_channels_setup_watcher(GwyDataChooser *chooser)
  *
  * Creates a data chooser for data channels.
  *
- * Returns: A new channel chooser.  Nothing may be assumed about the type and
- *          properties of the returned widget as they can change in the future.
+ * Returns: A new channel chooser.  Nothing may be assumed about the type and properties of the returned widget as
+ *          they can change in the future.
  **/
 GtkWidget*
 gwy_data_chooser_new_channels(void)
@@ -943,16 +905,12 @@ gwy_data_chooser_new_channels(void)
 
     renderer = gtk_cell_renderer_pixbuf_new();
     gtk_cell_layout_pack_start(layout, renderer, FALSE);
-    gtk_cell_layout_set_cell_data_func(layout, renderer,
-                                       gwy_data_chooser_render_icon,
-                                       chooser, NULL);
+    gtk_cell_layout_set_cell_data_func(layout, renderer, gwy_data_chooser_render_icon, chooser, NULL);
 
     renderer = gtk_cell_renderer_text_new();
     g_object_set(renderer, "xalign", 0.0, "style-set", TRUE, NULL);
     gtk_cell_layout_pack_start(layout, renderer, TRUE);
-    gtk_cell_layout_set_cell_data_func(layout, renderer,
-                                       gwy_data_chooser_render_name,
-                                       chooser, NULL);
+    gtk_cell_layout_set_cell_data_func(layout, renderer, gwy_data_chooser_render_name, chooser, NULL);
 
     gwy_data_chooser_choose_whatever(chooser);
     gwy_data_chooser_channels_setup_watcher(chooser);
@@ -971,8 +929,8 @@ gwy_data_chooser_new_channels(void)
  *
  * Creates a data chooser for volume data.
  *
- * Returns: A new volume chooser.  Nothing may be assumed about the type and
- *          properties of the returned widget as they can change in the future.
+ * Returns: A new volume chooser.  Nothing may be assumed about the type and properties of the returned widget as they
+ *          can change in the future.
  *
  * Since: 2.33
  **/
@@ -993,16 +951,12 @@ gwy_data_chooser_new_volumes(void)
 
     renderer = gtk_cell_renderer_pixbuf_new();
     gtk_cell_layout_pack_start(layout, renderer, FALSE);
-    gtk_cell_layout_set_cell_data_func(layout, renderer,
-                                       gwy_data_chooser_render_icon,
-                                       chooser, NULL);
+    gtk_cell_layout_set_cell_data_func(layout, renderer, gwy_data_chooser_render_icon, chooser, NULL);
 
     renderer = gtk_cell_renderer_text_new();
     g_object_set(renderer, "xalign", 0.0, "style-set", TRUE, NULL);
     gtk_cell_layout_pack_start(layout, renderer, TRUE);
-    gtk_cell_layout_set_cell_data_func(layout, renderer,
-                                       gwy_data_chooser_render_name,
-                                       chooser, NULL);
+    gtk_cell_layout_set_cell_data_func(layout, renderer, gwy_data_chooser_render_name, chooser, NULL);
 
     gwy_data_chooser_choose_whatever(chooser);
     /* FIXME: Needs data browser support. */
@@ -1021,8 +975,7 @@ static gchar*
 get_graph_title(GwyContainer *data, gint id)
 {
     GQuark quark = gwy_app_get_graph_key_for_id(id);
-    GwyGraphModel *gmodel = (GwyGraphModel*)gwy_container_get_object(data,
-                                                                     quark);
+    GwyGraphModel *gmodel = (GwyGraphModel*)gwy_container_get_object(data, quark);
     gchar *s, *title;
 
     g_return_val_if_fail(GWY_IS_GRAPH_MODEL(gmodel), NULL);
@@ -1038,8 +991,7 @@ gwy_data_chooser_graphs_setup_watcher(GwyDataChooser *chooser)
 {
     gint id;
 
-    id = gwy_app_data_browser_add_graph_watch(gwy_data_chooser_receive_event,
-                                              chooser);
+    id = gwy_app_data_browser_add_graph_watch(gwy_data_chooser_receive_event, chooser);
     chooser->watcher_id = id;
 }
 
@@ -1048,8 +1000,8 @@ gwy_data_chooser_graphs_setup_watcher(GwyDataChooser *chooser)
  *
  * Creates a data chooser for graphs.
  *
- * Returns: A new graph chooser.  Nothing may be assumed about the type and
- *          properties of the returned widget as they can change in the future.
+ * Returns: A new graph chooser.  Nothing may be assumed about the type and properties of the returned widget as they
+ *          can change in the future.
  *
  * Since: 2.41
  **/
@@ -1071,16 +1023,12 @@ gwy_data_chooser_new_graphs(void)
 
     renderer = gtk_cell_renderer_pixbuf_new();
     gtk_cell_layout_pack_start(layout, renderer, FALSE);
-    gtk_cell_layout_set_cell_data_func(layout, renderer,
-                                       gwy_data_chooser_render_icon,
-                                       chooser, NULL);
+    gtk_cell_layout_set_cell_data_func(layout, renderer, gwy_data_chooser_render_icon, chooser, NULL);
 
     renderer = gtk_cell_renderer_text_new();
     g_object_set(renderer, "xalign", 0.0, "style-set", TRUE, NULL);
     gtk_cell_layout_pack_start(layout, renderer, TRUE);
-    gtk_cell_layout_set_cell_data_func(layout, renderer,
-                                       gwy_data_chooser_render_name,
-                                       chooser, NULL);
+    gtk_cell_layout_set_cell_data_func(layout, renderer, gwy_data_chooser_render_name, chooser, NULL);
 
     gwy_data_chooser_choose_whatever(chooser);
     gwy_data_chooser_graphs_setup_watcher(chooser);
@@ -1099,8 +1047,8 @@ gwy_data_chooser_new_graphs(void)
  *
  * Creates a data chooser for XYZ data.
  *
- * Returns: A new XYZ data chooser.  Nothing may be assumed about the type and
- *          properties of the returned widget as they can change in the future.
+ * Returns: A new XYZ data chooser.  Nothing may be assumed about the type and properties of the returned widget as
+ *          they can change in the future.
  *
  * Since: 2.45
  **/
@@ -1121,16 +1069,12 @@ gwy_data_chooser_new_xyzs(void)
 
     renderer = gtk_cell_renderer_pixbuf_new();
     gtk_cell_layout_pack_start(layout, renderer, FALSE);
-    gtk_cell_layout_set_cell_data_func(layout, renderer,
-                                       gwy_data_chooser_render_icon,
-                                       chooser, NULL);
+    gtk_cell_layout_set_cell_data_func(layout, renderer, gwy_data_chooser_render_icon, chooser, NULL);
 
     renderer = gtk_cell_renderer_text_new();
     g_object_set(renderer, "xalign", 0.0, "style-set", TRUE, NULL);
     gtk_cell_layout_pack_start(layout, renderer, TRUE);
-    gtk_cell_layout_set_cell_data_func(layout, renderer,
-                                       gwy_data_chooser_render_name,
-                                       chooser, NULL);
+    gtk_cell_layout_set_cell_data_func(layout, renderer, gwy_data_chooser_render_name, chooser, NULL);
 
     gwy_data_chooser_choose_whatever(chooser);
     /* FIXME: Needs data browser support. */
@@ -1139,6 +1083,53 @@ gwy_data_chooser_new_xyzs(void)
     return (GtkWidget*)chooser;
 }
 
+/*****************************************************************************
+ *
+ * Lawns.
+ *
+ *****************************************************************************/
+
+/**
+ * gwy_data_chooser_new_curve_maps:
+ *
+ * Creates a data chooser for curve map data.
+ *
+ * Returns: A new curve map data chooser.  Nothing may be assumed about the type and properties of the returned widget
+ *          as they can change in the future.
+ *
+ * Since: 2.60
+ **/
+GtkWidget*
+gwy_data_chooser_new_curve_maps(void)
+{
+    GwyDataChooser *chooser;
+    GtkCellRenderer *renderer;
+    GtkCellLayout *layout;
+
+    chooser = (GwyDataChooser*)g_object_new(GWY_TYPE_DATA_CHOOSER, NULL);
+    chooser->kind = GWY_PAGE_CURVE_MAPS;
+    chooser->get_ids = gwy_app_data_browser_get_curve_map_ids;
+    chooser->get_title = gwy_app_get_lawn_title;
+    chooser->get_thumbnail = gwy_app_get_curve_map_thumbnail;
+    gwy_app_data_browser_foreach(gwy_data_chooser_fill, chooser);
+    layout = GTK_CELL_LAYOUT(chooser);
+
+    renderer = gtk_cell_renderer_pixbuf_new();
+    gtk_cell_layout_pack_start(layout, renderer, FALSE);
+    gtk_cell_layout_set_cell_data_func(layout, renderer, gwy_data_chooser_render_icon, chooser, NULL);
+
+    renderer = gtk_cell_renderer_text_new();
+    g_object_set(renderer, "xalign", 0.0, "style-set", TRUE, NULL);
+    gtk_cell_layout_pack_start(layout, renderer, TRUE);
+    gtk_cell_layout_set_cell_data_func(layout, renderer, gwy_data_chooser_render_name, chooser, NULL);
+
+    gwy_data_chooser_choose_whatever(chooser);
+    /* FIXME: Needs data browser support. */
+    /* gwy_data_chooser_curve_maps_setup_watcher(chooser); */
+
+    return (GtkWidget*)chooser;
+}
+
 /************************** Documentation ****************************/
 
 /**
@@ -1146,27 +1137,22 @@ gwy_data_chooser_new_xyzs(void)
  * @title: GwyDataChooser
  * @short_description: Data object choosers
  *
- * #GwyDataChooser is an base data object chooser class.  Choosers for
- * particular data objects can be created with functions like
- * gwy_data_chooser_new_channels() or gwy_data_chooser_new_volumes() and then
- * manipulated through #GwyDataChooser interface.
+ * #GwyDataChooser is an base data object chooser class.  Choosers for particular data objects can be created with
+ * functions like gwy_data_chooser_new_channels() or gwy_data_chooser_new_volumes() and then manipulated through
+ * #GwyDataChooser interface.
  *
- * The widget type used to implement choosers is not a part of the interface
- * and may be subject of future changes.  In any case #GwyDataChooser has
- * a <code>"changed"</code> signal emitted when the selected item changes.
+ * The widget type used to implement choosers is not a part of the interface and may be subject of future changes.  In
+ * any case #GwyDataChooser has a <code>"changed"</code> signal emitted when the selected item changes.
  *
- * It is possible to offer only data objects matching some criteria.  For
- * example to offer only data fields compatible with another data field,
- * one can use:
+ * It is possible to offer only data objects matching some criteria.  For example to offer only data fields compatible
+ * with another data field, one can use:
  * <informalexample><programlisting>
  * GtkWidget *chooser;
  * GwyDataField *model;
  * <!-- Hello, gtk-doc! -->
  * model = ...;
  * chooser = gwy_data_chooser_new_channels(<!-- Hello, gtk-doc! -->);
- * gwy_data_chooser_set_filter(GWY_DATA_CHOOSER(chooser),
- *                             compatible_field_filter, model,
- *                             NULL);
+ * gwy_data_chooser_set_filter(GWY_DATA_CHOOSER(chooser), compatible_field_filter, model, NULL);
  * </programlisting></informalexample>
  * where the filter function looks like
  * <informalexample><programlisting>
@@ -1181,12 +1167,11 @@ gwy_data_chooser_new_xyzs(void)
  *     quark = gwy_app_get_data_key_for_id(id);
  *     data_field = gwy_container_get_object(data, quark);
  *     model = GWY_DATA_FIELD(user_data);
- *     return !gwy_data_field_check_compatibility
- *                          (data_field, model,
- *                           GWY_DATA_COMPATIBILITY_RES
- *                           | GWY_DATA_COMPATIBILITY_REAL
- *                           | GWY_DATA_COMPATIBILITY_LATERAL
- *                           | GWY_DATA_COMPATIBILITY_VALUE);
+ *     return !gwy_data_field_check_compatibility(data_field, model,
+ *                                                GWY_DATA_COMPATIBILITY_RES
+ *                                                | GWY_DATA_COMPATIBILITY_REAL
+ *                                                | GWY_DATA_COMPATIBILITY_LATERAL
+ *                                                | GWY_DATA_COMPATIBILITY_VALUE);
  * }
  * </programlisting></informalexample>
  **/
@@ -1205,13 +1190,12 @@ gwy_data_chooser_new_xyzs(void)
 /**
  * GwyAppDataId:
  * @datano: Numeric identifier of data container.  Zero is used for none.
- * @id: Numeric identifier of a specific data item, such as channel or graph
- *      number.  Value -1 is used for none.
+ * @id: Numeric identifier of a specific data item, such as channel or graph number.  Value -1 is used for none.
  *
  * Auxiliary structure representing one data item in an open file.
  *
- * The data container number can be obtained with gwy_app_data_browser_get()
- * and used to look up the container with gwy_app_data_browser_get_number().
+ * The data container number can be obtained with gwy_app_data_browser_get() and used to look up the container with
+ * gwy_app_data_browser_get_number().
  *
  * Since: 2.41
  **/
@@ -1221,7 +1205,7 @@ gwy_data_chooser_new_xyzs(void)
  *
  * Initialiser for #GwyAppDataId that corresponds to no data.
  *
- * The macro would be typically used
+ * The macro would be typically used in initialisation as
  * |[
  * GwyAppDataId dataid = GWY_APP_DATA_ID_NONE;
  * ]|
@@ -1229,4 +1213,4 @@ gwy_data_chooser_new_xyzs(void)
  * Since: 2.41
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
