@@ -1,5 +1,5 @@
 /*
- *  $Id: dialog.c 24601 2022-02-15 15:54:06Z yeti-dn $
+ *  $Id: dialog.c 24974 2022-08-31 11:48:49Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -58,6 +58,8 @@ struct _GwyDialogPrivate {
 
 static void     gwy_dialog_finalize              (GObject *gobject);
 static void     gwy_dialog_destroy               (GtkObject *gtkobject);
+static void     gwy_dialog_response              (GtkDialog *gtkdialog,
+                                                  gint response);
 static void     everything_has_changed           (GwyDialog *dialog);
 static void     look_for_instant_updates_param   (GwyDialog *dialog,
                                                   GwyParamTable *partable);
@@ -69,8 +71,6 @@ static void     update_preview_button_sensitivity(GwyDialog *dialog);
 static void     handle_instant_updates_enabled   (GwyDialog *dialog);
 static gboolean preview_gsource                  (gpointer user_data);
 static void     preview_immediately              (GwyDialog *dialog);
-static void     dialog_response                  (GwyDialog *dialog,
-                                                  gint response);
 static void     reset_all_parameters             (GwyDialog *dialog);
 
 static guint param_table_param_changed = 0;
@@ -82,9 +82,11 @@ gwy_dialog_class_init(GwyDialogClass *klass)
 {
     GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
     GtkObjectClass *gtkobject_class = GTK_OBJECT_CLASS(klass);
+    GtkDialogClass *dialog_class = GTK_DIALOG_CLASS(klass);
 
     gobject_class->finalize = gwy_dialog_finalize;
     gtkobject_class->destroy = gwy_dialog_destroy;
+    dialog_class->response = gwy_dialog_response;
 
     g_type_class_add_private(klass, sizeof(GwyDialogPrivate));
 }
@@ -109,7 +111,6 @@ gwy_dialog_finalize(GObject *gobject)
     GwyDialog *dialog = (GwyDialog*)gobject;
     GwyDialogPrivate *priv = dialog->priv;
 
-    /* TODO TODO TODO */
     g_ptr_array_free(priv->tables, TRUE);
 
     G_OBJECT_CLASS(gwy_dialog_parent_class)->finalize(gobject);
@@ -157,9 +158,6 @@ gwy_dialog_new(const gchar *title)
     if (title)
         gtk_window_set_title(GTK_WINDOW(dialog), title);
 
-    /* The class handler is run after.  We do not want that. */
-    g_signal_connect(dialog, "response", G_CALLBACK(dialog_response), NULL);
-
     return dialog;
 }
 
@@ -523,9 +521,15 @@ gwy_dialog_run(GwyDialog *dialog)
         else if (response == GWY_RESPONSE_UPDATE)
             preview_immediately(dialog);
         else if (response == GWY_RESPONSE_RESET) {
-            /* Do not handle GWY_RESPONSE_RESET here.  It would be run after everything and that is too late.  Modules
-             * do not want to run their own precious reset handlers and us come later and break everything.  We need
-             * to reset first and then the module corrects whatever needs special treatment.  */
+            /* Do not handle GWY_RESPONSE_RESET here.  It would be run after everything and that is too late.  The
+             * order of execution is:
+             * 1. "response" handler we could connect upon construction (without after), but do not currently,
+             * 2. "response" handler the module connects (without after),
+             * 3. class handler gtkdialog_class->response(),
+             * 4. "response" handler the module connects with G_CONNECT_AFTER, and
+             * 5. here.
+             * Do the auto-reset in the class handler. Modules can then easily fix up the param tables either before
+             * or after our handler (or both), as they see fit. */
         }
         else {
             /* The caller may also add its own uncommon response types, so do not assume we can handle everything. */
@@ -808,8 +812,11 @@ preview_immediately(GwyDialog *dialog)
 }
 
 static void
-dialog_response(GwyDialog *dialog, gint response)
+gwy_dialog_response(GtkDialog *gtkdialog, gint response)
 {
+    GwyDialog *dialog = GWY_DIALOG(gtkdialog);
+
+    gwy_debug("\"response\" class handler\n");
     if (response == GWY_RESPONSE_RESET)
         reset_all_parameters(dialog);
 }
@@ -845,8 +852,12 @@ reset_all_parameters(GwyDialog *dialog)
  *
  * #GwyDialog also handles the state change logic typical for Gwyddion data processing modules.  The function
  * gwy_dialog_run() only exits when the dialog finishes with some kind of conclusion what to do next, see
- * #GwyDialogOutcome.  Other response can either be handled completely automatically, for instance GWY_RESPONSE_RESET
- * and GWY_RESPONSE_UPDATE, or in cooperation with the module which connects to the GtkDialog::response signal.
+ * #GwyDialogOutcome.  Other response can either be handled completely automatically, for instance %GWY_RESPONSE_RESET
+ * and %GWY_RESPONSE_UPDATE, or in cooperation with the module which connects to the GtkDialog::response signal.
+ *
+ * If you need to prepare the parameter tables before the automatic reset, connect to the signal normally and it will
+ * be run before the #GwyDialog handler. If you need to touch up the tables afterwards connect your handler with
+ * %G_CONNECT_AFTER.
  **/
 
 /**
