@@ -1,22 +1,18 @@
 /*
- *  $Id: file.c 24417 2021-10-24 08:36:28Z yeti-dn $
- *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
+ *  $Id: file.c 24584 2022-02-05 10:48:18Z yeti-dn $
+ *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -57,10 +53,8 @@ static gchar *current_dir = NULL;
  *
  * Warning: This function is probably temporary.
  *
- * Returns: A string in GLib file name encoding that should not be modified
- *          neither freed, valid only until next call to
- *          gwy_app_set_current_directory().  It ends with a
- *          %G_DIR_SEPARATOR_S.
+ * Returns: A string in GLib file name encoding that should not be modified neither freed, valid only until next call
+ *          to gwy_app_set_current_directory().  It ends with a %G_DIR_SEPARATOR_S.
  **/
 const gchar*
 gwy_app_get_current_directory(void)
@@ -77,8 +71,8 @@ gwy_app_get_current_directory(void)
 
 /**
  * gwy_app_set_current_directory:
- * @directory: The directory to set, or a filename to take directory part
- *             from, it must be an absolute path.  In GLib file name encoding.
+ * @directory: The directory to set, or a filename to take directory part from, it must be an absolute path.  In GLib
+ *             file name encoding.
  *
  * Sets what the app should use as `current directory'.
  *
@@ -119,20 +113,17 @@ gwy_app_set_current_directory(const gchar *directory)
  * gwy_app_file_load:
  * @filename_utf8: Name of file to load, in UTF-8.
  * @filename_sys: Name of file to load, in GLib encoding.
- * @name: File type to open file as, but normally %NULL to automatically
- *        detect from file contents.
+ * @name: File type to open file as, but normally %NULL to automatically detect from file contents.
  *
  * Loads a file into application (a high-level function).
  *
  * At least one of @filename_utf8, @filename_sys must be non-%NULL.
  *
- * The file is loaded in interactive mode, modules can ask for user input.
- * Upon a successful load all necessary setup tasks are performed.  If the
- * load fails, an error dialog is presented.
+ * The file is loaded in interactive mode, modules can ask for user input. Upon a successful load all necessary setup
+ * tasks are performed.  If the load fails, an error dialog is presented.
  *
- * Returns: Container of the just loaded file on success, %NULL on failure.
- *          The caller does not own the reference, the container is only
- *          owned by the data browser.
+ * Returns: Container of the just loaded file on success, %NULL on failure. The caller does not own the reference, the
+ *          container is only owned by the data browser.
  **/
 GwyContainer*
 gwy_app_file_load(const gchar *filename_utf8,
@@ -142,8 +133,7 @@ gwy_app_file_load(const gchar *filename_utf8,
     GwyContainer *data;
 
     data = gwy_app_file_load_real(filename_utf8, filename_sys, name, TRUE);
-    /* gwy_app_file_add_loaded() takes a reference therefore we can release
-     * the initial one */
+    /* gwy_app_file_add_loaded() takes a reference therefore we can release the initial one */
     if (data) {
         g_object_unref(data);
         warn_broken_load_func(name, data);
@@ -163,9 +153,7 @@ find_file_func_author(gpointer hkey, gpointer hvalue, gpointer user_data)
     l = gwy_module_get_functions(modname);
     while (l) {
         const gchar *funcname = (const gchar*)l->data;
-        if (g_str_has_prefix(funcname, "file::")
-            && gwy_strequal(funcname + sizeof("file::") - 1,
-                            ffdata->funcname)) {
+        if (g_str_has_prefix(funcname, "file::") && gwy_strequal(funcname + sizeof("file::") - 1, ffdata->funcname)) {
             ffdata->modname = modname;
             ffdata->modinfo = modinfo;
             return;
@@ -207,21 +195,19 @@ warn_broken_load_func(const gchar *name,
     gwy_module_foreach(find_file_func_author, &ffdata);
     g_return_if_fail(ffdata.modinfo && ffdata.modname);
 
-    dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_WARNING,
-                                    GTK_BUTTONS_OK,
+    dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
                                     _("Imported data are likely incorrect."));
 
-    gtk_message_dialog_format_secondary_text
-        (GTK_MESSAGE_DIALOG(dialog),
-         _("Import support for files of type\n\n"
-           "%s\n\n"
-           "is incomplete due to the lack of documentation, "
-           "testing files and/or people willing to help with the testing.\n\n"
-           "If you can help to improve the import please contact the "
-           "author of module %s-%s:\n\n"
-           "%s"),
-         _(description), ffdata.modname, ffdata.modinfo->version,
-         ffdata.modinfo->author);
+    gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
+                                             _("Import support for files of type\n\n"
+                                               "%s\n\n"
+                                               "is incomplete due to the lack of documentation, "
+                                               "testing files and/or people willing to help with the testing.\n\n"
+                                               "If you can help to improve the import please contact the "
+                                               "author of module %s-%s:\n\n"
+                                               "%s"),
+                                             _(description), ffdata.modname, ffdata.modinfo->version,
+                                             ffdata.modinfo->author);
 
     g_signal_connect(dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
     gtk_widget_show_all(dialog);
@@ -241,8 +227,7 @@ gwy_app_file_load_real(const gchar *filename_utf8,
 
     g_return_val_if_fail(filename_utf8 || filename_sys, NULL);
     if (!filename_sys) {
-        filename_sys = g_filename_from_utf8(filename_utf8, -1,
-                                            NULL, NULL, NULL);
+        filename_sys = g_filename_from_utf8(filename_utf8, -1, NULL, NULL, NULL);
         if (!filename_sys) {
             g_warning("FIXME: file name not convertible to system encoding");
             return NULL;
@@ -251,23 +236,18 @@ gwy_app_file_load_real(const gchar *filename_utf8,
     }
 
     if (!filename_utf8) {
-        filename_utf8 = g_filename_to_utf8(filename_sys, -1,
-                                           NULL, NULL, NULL);
+        filename_utf8 = g_filename_to_utf8(filename_sys, -1, NULL, NULL, NULL);
         free_utf8 = TRUE;
     }
 
     _gwy_app_log_start_message_capture();
     if (name)
-        data = gwy_file_func_run_load(name, filename_sys,
-                                      GWY_RUN_INTERACTIVE, &err);
+        data = gwy_file_func_run_load(name, filename_sys, GWY_RUN_INTERACTIVE, &err);
     else
-        data = gwy_file_load_with_func(filename_sys, GWY_RUN_INTERACTIVE,
-                                       &name, &err);
+        data = gwy_file_load_with_func(filename_sys, GWY_RUN_INTERACTIVE, &name, &err);
 
     if (data) {
-        gwy_data_validate(data,
-                          GWY_DATA_VALIDATE_CORRECT
-                          | GWY_DATA_VALIDATE_NO_REPORT);
+        gwy_data_validate(data, GWY_DATA_VALIDATE_CORRECT | GWY_DATA_VALIDATE_NO_REPORT);
         if (_gwy_app_enforce_graph_abscissae_order(data)) {
             if (!gwy_stramong(name, "gwyfile", NULL)) {
                 g_warning("Module %s did not import file %s with graph curve "
@@ -282,31 +262,23 @@ gwy_app_file_load_real(const gchar *filename_utf8,
         if (do_add_loaded)
             _gwy_app_log_discard_captured_messages();
 
-        if (err && !g_error_matches(err,
-                                    GWY_MODULE_FILE_ERROR,
-                                    GWY_MODULE_FILE_ERROR_CANCELLED)) {
+        if (err && !g_error_matches(err, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_CANCELLED)) {
             gchar *filename_disp = g_filename_display_basename(filename_utf8);
             GString *message = g_string_new(NULL);
 
-            dialog = gtk_message_dialog_new(NULL, 0,
-                                            GTK_MESSAGE_ERROR,
-                                            GTK_BUTTONS_CLOSE,
-                                            _("Opening of `%s' failed"),
-                                            filename_disp);
+            dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+                                            _("Opening of `%s' failed"), filename_disp);
             g_free(filename_disp);
             g_string_append(message, err->message);
             g_string_append(message, "\n\n");
-            g_string_append_printf(message,
-                                   _("Full file path: %s."), filename_utf8);
+            g_string_append_printf(message, _("Full file path: %s."), filename_utf8);
             if (name) {
                 g_string_append(message, "\n\n");
                 g_string_append_printf(message, _("Loaded using: %s."), name);
             }
-            gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
-                                                     "%s", message->str);
+            gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), "%s", message->str);
             g_string_free(message, TRUE);
-            g_signal_connect(dialog, "response",
-                             G_CALLBACK(gtk_widget_destroy), NULL);
+            g_signal_connect(dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
             gtk_widget_show_all(dialog);
             gtk_window_present(GTK_WINDOW(dialog));
             g_clear_error(&err);
@@ -343,8 +315,7 @@ gwy_app_file_add_loaded(GwyContainer *data,
         set_filename(data, filename_utf8);
 
     gwy_app_data_browser_add(data);
-    gwy_app_data_browser_reset_visibility(data,
-                                          GWY_VISIBILITY_RESET_DEFAULT);
+    gwy_app_data_browser_reset_visibility(data, GWY_VISIBILITY_RESET_DEFAULT);
     _gwy_app_data_browser_add_messages(data);
 
     if (filename_utf8)
@@ -414,9 +385,7 @@ gwy_app_file_open_or_merge(gboolean merge)
                 warn_broken_load_func(name, data);
                 g_object_unref(data);
 
-                gwy_app_data_browser_get_current(GWY_APP_CONTAINER,
-                                                 &current_data,
-                                                 NULL);
+                gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &current_data, NULL);
                 _gwy_app_data_browser_add_messages(current_data);
             }
             else
@@ -438,15 +407,14 @@ gwy_app_file_open_or_merge(gboolean merge)
  * @data: Data to write.
  * @filename_utf8: Name of file to write data to, in UTF-8.
  * @filename_sys: Name of file to write data to, in GLib encoding.
- * @name: File type to save file as, but normally %NULL to automatically detect
- *        from file name.
+ * @name: File type to save file as, but normally %NULL to automatically detect from file name.
  *
  * Writes container to a file (a high-level function).
  *
  * At least one of @filename_utf8, @filename_sys must be non-%NULL.
  *
- * The file is saved in interactive mode, modules can ask for user input.
- * If the write fails, an error dialog is presented.
+ * The file is saved in interactive mode, modules can ask for user input. If the write fails, an error dialog is
+ * presented.
  *
  * Returns: %TRUE on success.
  **/
@@ -463,8 +431,7 @@ gwy_app_file_write(GwyContainer *data,
     GError *err = NULL;
     gint id;
 
-    /* If the @data is the current container, make thumbnail from the current
-     * channel.
+    /* If the @data is the current container, make thumbnail from the current channel.
      * FIXME: Needs new data browser functions to do this less hackishly. */
     gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &container,
                                      GWY_APP_DATA_FIELD_ID, &id,
@@ -474,8 +441,7 @@ gwy_app_file_write(GwyContainer *data,
 
     g_return_val_if_fail(filename_utf8 || filename_sys, FALSE);
     if (!filename_sys) {
-        filename_sys = g_filename_from_utf8(filename_utf8, -1,
-                                            NULL, NULL, NULL);
+        filename_sys = g_filename_from_utf8(filename_utf8, -1, NULL, NULL, NULL);
         if (!filename_sys) {
             g_warning("FIXME: file name not convertible to system encoding");
             return FALSE;
@@ -484,27 +450,23 @@ gwy_app_file_write(GwyContainer *data,
     }
 
     if (!filename_utf8) {
-        filename_utf8 = g_filename_to_utf8(filename_sys, -1,
-                                           NULL, NULL, NULL);
+        filename_utf8 = g_filename_to_utf8(filename_sys, -1, NULL, NULL, NULL);
         free_utf8 = TRUE;
     }
 
     if (name) {
         saveok = gwy_file_func_get_operations(name);
-        if (saveok & GWY_FILE_OPERATION_SAVE
-            && gwy_file_func_run_save(name, data, filename_sys,
-                                      GWY_RUN_INTERACTIVE, &err))
+        if (saveok & GWY_FILE_OPERATION_SAVE && gwy_file_func_run_save(name, data, filename_sys,
+                                                                       GWY_RUN_INTERACTIVE, &err))
             saveok = GWY_FILE_OPERATION_SAVE;
-        else if (saveok & GWY_FILE_OPERATION_EXPORT
-                 && gwy_file_func_run_export(name, data, filename_sys,
-                                             GWY_RUN_INTERACTIVE, &err))
+        else if (saveok & GWY_FILE_OPERATION_EXPORT && gwy_file_func_run_export(name, data, filename_sys,
+                                                                                GWY_RUN_INTERACTIVE, &err))
             saveok = GWY_FILE_OPERATION_EXPORT;
         else
             saveok = 0;
     }
     else
-        saveok = gwy_file_save_with_func(data, filename_sys,
-                                         GWY_RUN_INTERACTIVE, &name, &err);
+        saveok = gwy_file_save_with_func(data, filename_sys, GWY_RUN_INTERACTIVE, &name, &err);
 
     switch (saveok) {
         case GWY_FILE_OPERATION_SAVE:
@@ -517,31 +479,23 @@ gwy_app_file_write(GwyContainer *data,
         break;
 
         default:
-        if (err && !g_error_matches(err,
-                                    GWY_MODULE_FILE_ERROR,
-                                    GWY_MODULE_FILE_ERROR_CANCELLED)) {
+        if (err && !g_error_matches(err, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_CANCELLED)) {
             gchar *filename_disp = g_filename_display_basename(filename_utf8);
             GString *message = g_string_new(NULL);
 
-            dialog = gtk_message_dialog_new(NULL, 0,
-                                            GTK_MESSAGE_ERROR,
-                                            GTK_BUTTONS_CLOSE,
-                                            _("Saving of `%s' failed"),
-                                            filename_disp);
+            dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+                                            _("Saving of `%s' failed"), filename_disp);
             g_free(filename_disp);
             g_string_append(message, err->message);
             g_string_append(message, "\n\n");
-            g_string_append_printf(message,
-                                   _("Full file path: %s."), filename_utf8);
+            g_string_append_printf(message, _("Full file path: %s."), filename_utf8);
             if (name) {
                 g_string_append(message, "\n\n");
                 g_string_append_printf(message, _("Saved using: %s."), name);
             }
-            gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
-                                                     "%s", message->str);
+            gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), "%s", message->str);
             g_string_free(message, TRUE);
-            g_signal_connect(dialog, "response",
-                             G_CALLBACK(gtk_widget_destroy), NULL);
+            g_signal_connect(dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
             gtk_widget_show_all(dialog);
             gtk_window_present(GTK_WINDOW(dialog));
             g_clear_error(&err);
@@ -564,8 +518,8 @@ gwy_app_file_write(GwyContainer *data,
  *
  * Saves current data to a file (very high-level app function).
  *
- * May fall back to gwy_app_file_save_as() when current data has no file name
- * associated with it, or the format it was loaded from is not saveable.
+ * May fall back to gwy_app_file_save_as() when current data has no file name associated with it, or the format it was
+ * loaded from is not saveable.
  **/
 void
 gwy_app_file_save(void)
@@ -609,13 +563,11 @@ gwy_app_file_save_as(void)
 
     dialog = _gwy_app_file_chooser_get(GTK_FILE_CHOOSER_ACTION_SAVE);
     /* XXX: UTF-8 conversion missing */
-    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog),
-                                        gwy_app_get_current_directory());
+    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());
     if (filename_sys) {
         filename_sys = g_path_get_basename(filename_sys);
         filename_utf8 = g_filename_to_utf8(filename_sys, -1, NULL, NULL, NULL);
-        gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog),
-                                          filename_utf8);
+        gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), filename_utf8);
         g_free(filename_utf8);
         g_free(filename_sys);
     }
@@ -652,9 +604,7 @@ gwy_app_file_close(void)
 {
     GwyContainer *current_data;
 
-    gwy_app_data_browser_get_current(GWY_APP_CONTAINER,
-                                     &current_data,
-                                     NULL);
+    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &current_data, NULL);
     g_return_if_fail(current_data);
     gwy_app_data_browser_remove(current_data);
 }
@@ -672,13 +622,10 @@ gboolean
 gwy_app_file_confirm_overwrite(GtkWidget *chooser)
 {
     GtkWidget *dialog, *toplevel;
-    gchar *filename_sys, *filename_utf8, *dirname_sys, *dirname_utf8,
-          *fullname_sys;
+    gchar *filename_sys, *filename_utf8, *dirname_sys, *dirname_utf8, *fullname_sys;
     gint response;
 
-    g_return_val_if_fail(gtk_file_chooser_get_action(GTK_FILE_CHOOSER(chooser))
-                         == GTK_FILE_CHOOSER_ACTION_SAVE,
-                         FALSE);
+    g_return_val_if_fail(gtk_file_chooser_get_action(GTK_FILE_CHOOSER(chooser)) == GTK_FILE_CHOOSER_ACTION_SAVE, FALSE);
     fullname_sys = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
     if (!fullname_sys)
         return FALSE;
@@ -691,17 +638,14 @@ gwy_app_file_confirm_overwrite(GtkWidget *chooser)
     dirname_utf8 = g_filename_to_utf8(dirname_sys, -1, NULL, NULL, NULL);
 
     toplevel = gtk_widget_get_toplevel(chooser);
-    dialog = gtk_message_dialog_new(GTK_WINDOW(toplevel),
-                                    GTK_DIALOG_DESTROY_WITH_PARENT,
-                                    GTK_MESSAGE_QUESTION,
-                                    GTK_BUTTONS_YES_NO,
+    dialog = gtk_message_dialog_new(GTK_WINDOW(toplevel), GTK_DIALOG_DESTROY_WITH_PARENT,
+                                    GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
                                     _("File `%s' already exists.  Replace?"),
                                     filename_utf8);
-    gtk_message_dialog_format_secondary_text
-        (GTK_MESSAGE_DIALOG(dialog),
-         _("The file already exists in `%s'.  "
-           "Replacing it will overwrite its contents."),
-         dirname_utf8);
+    gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
+                                             _("The file already exists in `%s'.  "
+                                               "Replacing it will overwrite its contents."),
+                                             dirname_utf8);
     gtk_window_set_title(GTK_WINDOW(dialog), _("Replace File?"));
 
     g_free(fullname_sys);
@@ -723,22 +667,17 @@ gwy_app_file_confirm_overwrite(GtkWidget *chooser)
  * @title: file
  * @short_description: High level file functions
  *
- * High-level functions gwy_app_file_load() and gwy_app_file_write()
- * wrap low-level file handling functions from
- * <link linkend="libgwymodule-gwymodule-file">gwymodule-file</link>
- * and handle registration of loaded containers in data browser, showing
- * windows, remembering file types, or the differenced between save and export.
+ * High-level functions gwy_app_file_load() and gwy_app_file_write() wrap low-level file handling functions from <link
+ * linkend="libgwymodule-gwymodule-file">gwymodule-file</link> and handle registration of loaded containers in data
+ * browser, showing windows, remembering file types, or the differenced between save and export.
  *
- * They are complemented by application-level functions gwy_app_file_open(),
- * gwy_app_file_save(), and gwy_app_file_save_as() that perform the
- * corresponding operations of File menu.  These are probably not of general
+ * They are complemented by application-level functions gwy_app_file_open(), gwy_app_file_save(), and
+ * gwy_app_file_save_as() that perform the corresponding operations of File menu.  These are probably not of general
  * interest.
  *
- * Beside that, functions to maintain application-level idea of current
- * directory are provided: gwy_app_get_current_directory(),
- * gwy_app_set_current_directory().  They should be used in place of system
- * chdir() which has various unwanted side-effect, like change of the directory
- * where core is dumped on segfault.
+ * Beside that, functions to maintain application-level idea of current directory are provided:
+ * gwy_app_get_current_directory(), gwy_app_set_current_directory().  They should be used in place of system chdir()
+ * which has various unwanted side-effect, like change of the directory where core is dumped on segfault.
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
