@@ -1,5 +1,5 @@
 /*
- *  $Id: filelist.c 24417 2021-10-24 08:36:28Z yeti-dn $
+ *  $Id: filelist.c 24709 2022-03-21 17:31:45Z yeti-dn $
  *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -234,18 +234,18 @@ gwy_app_recent_file_list_new(void)
 
     gcontrols.prune = gwy_stock_like_button_new(_("Clean U_p"), GTK_STOCK_FIND);
     gtk_box_pack_start(GTK_BOX(buttonbox), gcontrols.prune, TRUE, TRUE, 0);
-    gtk_tooltips_set_tip(gwy_app_get_tooltips(), gcontrols.prune,
-                         _("Remove entries of files that no longer exist"), NULL);
+    gtk_widget_set_tooltip_text(gcontrols.prune,
+                                _("Remove entries of files that no longer exist"));
     g_signal_connect_swapped(gcontrols.prune, "clicked", G_CALLBACK(gwy_app_recent_file_list_prune), &gcontrols);
 
     button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
     gtk_box_pack_start(GTK_BOX(buttonbox), button, TRUE, TRUE, 0);
-    gtk_tooltips_set_tip(gwy_app_get_tooltips(), button, _("Close file list"), NULL);
+    gtk_widget_set_tooltip_text(button, _("Close file list"));
     g_signal_connect_swapped(button, "clicked", G_CALLBACK(gtk_widget_destroy), gcontrols.window);
 
     gcontrols.open = gtk_button_new_from_stock(GTK_STOCK_OPEN);
     gtk_box_pack_start(GTK_BOX(buttonbox), gcontrols.open, TRUE, TRUE, 0);
-    gtk_tooltips_set_tip(gwy_app_get_tooltips(), gcontrols.open, _("Open selected file"), NULL);
+    gtk_widget_set_tooltip_text(gcontrols.open, _("Open selected file"));
     g_signal_connect_swapped(gcontrols.open, "clicked", G_CALLBACK(gwy_app_recent_file_list_open), list);
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
     gtk_widget_set_sensitive(gcontrols.open, gtk_tree_selection_get_selected(selection, NULL, NULL));
