@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyappfilechooser.c 24709 2022-03-21 17:31:45Z yeti-dn $
+ *  $Id: gwyappfilechooser.c 24828 2022-05-18 07:57:22Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -242,21 +242,17 @@ _gwy_app_file_chooser_get(GtkFileChooserAction action)
     g_object_add_weak_pointer(G_OBJECT(chooser), (gpointer*)instance);
 
     gtk_dialog_add_button(dialog, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-    switch (action) {
-        case GTK_FILE_CHOOSER_ACTION_OPEN:
+    if (action == GTK_FILE_CHOOSER_ACTION_OPEN) {
         chooser->prefix = "/app/file/load";
         gtk_dialog_add_button(dialog, GTK_STOCK_OPEN, GTK_RESPONSE_OK);
         gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(chooser), TRUE);
-        break;
-
-        case GTK_FILE_CHOOSER_ACTION_SAVE:
+    }
+    else if (action == GTK_FILE_CHOOSER_ACTION_SAVE) {
         chooser->prefix = "/app/file/save";
         gtk_dialog_add_button(dialog, GTK_STOCK_SAVE, GTK_RESPONSE_OK);
-        break;
-
-        default:
+    }
+    else {
         g_assert_not_reached();
-        break;
     }
 
     gwy_help_add_to_window(GTK_WINDOW(dialog), "managing-files", NULL, GWY_HELP_DEFAULT);
@@ -442,27 +438,23 @@ gwy_app_file_chooser_add_type_list(GwyAppFileChooser *chooser)
 
     store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
     gtk_list_store_append(store, &iter);
-    switch (action) {
-        case GTK_FILE_CHOOSER_ACTION_SAVE:
+    if (action == GTK_FILE_CHOOSER_ACTION_SAVE) {
         gtk_list_store_set(store, &iter,
                            COLUMN_FILETYPE, "",
                            COLUMN_LABEL, _("Automatic by extension"),
                            -1);
         gwy_app_file_chooser_add_types(store, GWY_FILE_OPERATION_SAVE, FALSE);
         gwy_app_file_chooser_add_types(store, GWY_FILE_OPERATION_EXPORT, FALSE);
-        break;
-
-        case GTK_FILE_CHOOSER_ACTION_OPEN:
+    }
+    else if (action == GTK_FILE_CHOOSER_ACTION_OPEN) {
         gtk_list_store_set(store, &iter,
                            COLUMN_FILETYPE, "",
                            COLUMN_LABEL, _("Automatically detected"),
                            -1);
         gwy_app_file_chooser_add_types(store, GWY_FILE_OPERATION_LOAD, TRUE);
-        break;
-
-        default:
+    }
+    else {
         g_assert_not_reached();
-        break;
     }
 
     chooser->expander = gtk_expander_new(NULL);
@@ -496,12 +488,13 @@ gwy_app_file_chooser_add_type_list(GwyAppFileChooser *chooser)
 
     selection = gtk_tree_view_get_selection(treeview);
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
+    gtk_tree_selection_select_iter(selection, &iter);  /* This relies on iter being set above. */
     g_signal_connect_swapped(selection, "changed", G_CALLBACK(gwy_app_file_chooser_type_changed), chooser);
 
     if (action == GTK_FILE_CHOOSER_ACTION_OPEN) {
         /* Gtk+ file chooser filter model is completely antagonistic to any parametric filter mechanism.  Also we
          * cannot set the filter to NULL because it assumes we always choose a filter from some list.  So (1) we need
-         * an explicit no_filter object that does nothing (2) we cannot combine filters, we need a single
+         * an explicit no_filter object that does nothing (2) since we cannot combine filters we need a single
          * monster-filter function that does everything (3) despite this, there is no ‘refilter’ function, so we must
          * set filter to no_filter and then back to filter to refilter. */
         chooser->filter = gtk_file_filter_new();
