@@ -1,5 +1,5 @@
 /*
- *  $Id: gwymoduleutils-file.c 21435 2018-09-11 12:39:53Z yeti-dn $
+ *  $Id: gwymoduleutils-file.c 23978 2021-08-13 14:48:02Z yeti-dn $
  *  Copyright (C) 2007-2016 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -812,6 +812,34 @@ gwy_file_xyz_import_log_add(GwyContainer *data,
     add_import_log(data, id, gwy_app_xyz_log_add, filetype, filename);
 }
 
+/**
+ * gwy_file_curve_map_import_log_add:
+ * @data: A data container.
+ * @id: Volume data id.
+ * @filetype: File type, i.e. the name of the function importing the data
+ *            (without any "file::" prefix).  Since 2.38 it is possible to
+ *            pass %NULL to fill the name of the currently running file type
+ *            function automatically.
+ * @filename: Name of the imported file.  If it is not valid UTF-8, it will be
+ *            converted to UTF-8 using g_filename_to_utf8().  Failing even
+ *            that, non-ASCII characters will be escaped.
+ *
+ * Logs the import of volume data from third-party file.
+ *
+ * This is a convenience wrapper for gwy_app_curve_map_log_add().  The source id
+ * will be set to -1.  The file name will be added to function arguments.
+ *
+ * Since: 2.60
+ **/
+void
+gwy_file_curve_map_import_log_add(GwyContainer *data,
+                                  gint id,
+                                  const gchar *filetype,
+                                  const gchar *filename)
+{
+    add_import_log(data, id, gwy_app_curve_map_log_add, filetype, filename);
+}
+
 static void
 add_import_log(GwyContainer *data,
                gint id,
