@@ -1,5 +1,5 @@
 /*
- *  $Id: gwymoduleutils.c 24672 2022-03-15 12:32:09Z yeti-dn $
+ *  $Id: gwymoduleutils.c 25037 2022-09-22 12:25:33Z yeti-dn $
  *  Copyright (C) 2007-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -843,6 +843,46 @@ gwy_app_wait_preview_data_field(GwyDataField *dfield,
     return preview;
 }
 
+/**
+ * gwy_run_subdialog:
+ * @dialog: A data dialog to be run as modal.
+ *
+ * Runs a modal dialog transient for another another dialog, trying to work around bugs.
+ *
+ * If both dialogs are modal, bad things can happen in some graphical environments, such as the second being hidden
+ * under the first one and inaccessible. This function attempts to work around this by temporarily setting the parent
+ * dialog (the one @dialog is transient for) non-modal.
+ *
+ * If the dialog is not transient for anything or its parent is not modal it is just run normally.
+ *
+ * When the function resturns only the dialog has been destroyed.
+ *
+ * Returns: The response of @dialog.
+ *
+ * Since: 2.62
+ **/
+gint
+gwy_run_subdialog(GtkDialog *dialog)
+{
+    GtkWindow *parent;
+    gint response;
+
+    g_return_val_if_fail(GTK_IS_DIALOG(dialog), GTK_RESPONSE_NONE);
+    parent = gtk_window_get_transient_for(GTK_WINDOW(dialog));
+    if (parent && GTK_IS_WINDOW(parent) && gtk_window_get_modal(parent))
+        gtk_window_set_modal(GTK_WINDOW(parent), FALSE);
+    else
+        parent = NULL;
+
+    response = gtk_dialog_run(dialog);
+    if (parent)
+        gtk_window_set_modal(GTK_WINDOW(parent), TRUE);
+    if (response != GTK_RESPONSE_NONE)
+        gtk_widget_destroy(GTK_WIDGET(dialog));
+
+    return response;
+}
+
 /**
  * gwy_require_image_same_units:
  * @field: The data field to check.
@@ -1163,6 +1203,116 @@ gwy_param_active_page_link_to_notebook(GwyParams *params,
     g_signal_connect(notebook, "switch-page", G_CALLBACK(active_page_changed), params);
 }
 
+static void
+expander_expanded(GtkExpander *expander,
+                  G_GNUC_UNUSED GParamSpec *pspec,
+                  GwyParams *params)
+{
+    gint id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(expander), "gwy-expander-param-id"));
+
+    gwy_params_set_boolean(params, id, gtk_expander_get_expanded(expander));
+}
+
+/**
+ * gwy_create_expander_with_param:
+ * @label: Expander label markup. Usually boldface Options.
+ * @params: A parameter value set.
+ * @id: Parameter identifier.
+ *
+ * Creates an expander linked to a boolean parameter.
+ *
+ * This is a convenience function for making an expander whose state is remembered in settings. The parameter
+ * identified by @id must be a boolean parameter. The expander initial state will be set from the parameter and,
+ * conversely, the parameter will be updated according to the expander state.
+ *
+ * Returns: A newly created #GtkExpander widget.
+ *
+ * Since: 2.62
+ **/
+GtkWidget*
+gwy_create_expander_with_param(const gchar *label,
+                               GwyParams *params,
+                               gint id)
+{
+    GtkWidget *expander;
+
+    expander = gtk_expander_new(label);
+    gtk_expander_set_use_markup(GTK_EXPANDER(expander), TRUE);
+    gtk_expander_set_expanded(GTK_EXPANDER(expander), gwy_params_get_boolean(params, id));
+    g_object_set_data(G_OBJECT(expander), "gwy-expander-param-id", GINT_TO_POINTER(id));
+    g_signal_connect(expander, "notify::expanded", G_CALLBACK(expander_expanded), params);
+
+    return expander;
+}
+
+static gboolean
+item_is_visible(GtkTreeModel *model, GtkTreeIter *iter, gpointer default_item)
+{
+    gpointer item;
+
+    gtk_tree_model_get(model, iter, 0, &item, -1);
+    return item != default_item;
+}
+
+/**
+ * gwy_create_inventory_model_without_default:
+ * @store: An inventory store.
+ *
+ * Creates a tree model filter around an inventory store which hides the default item.
+ *
+ * The model can be used to create resource lists without the default item if it is special and should not be shown to
+ * users. Remember you need to translate iters between the models using
+ * gtk_tree_model_filter_convert_child_iter_to_iter() and gtk_tree_model_filter_convert_iter_to_child_iter().
+ *
+ * Returns: A newly created #GtkTreeModelFilter wrapping @store.
+ *
+ * Since: 2.62
+ **/
+GtkTreeModel*
+gwy_create_inventory_model_without_default(GwyInventoryStore *store)
+{
+    GtkTreeModel *filtermodel;
+    gpointer default_item;
+
+    g_return_val_if_fail(GWY_IS_INVENTORY_STORE(store), NULL);
+    default_item = gwy_inventory_get_default_item(gwy_inventory_store_get_inventory(store));
+    filtermodel = gtk_tree_model_filter_new(GTK_TREE_MODEL(store), NULL);
+    gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(filtermodel), &item_is_visible, default_item, NULL);
+
+    return filtermodel;
+}
+
+/**
+ * gwy_select_in_filtered_inventory_treeeview:
+ * @treeview: A filtered tree view showing an inventory.
+ * @name: Item name to select.
+ *
+ * Selects an item in filtered tree view showing an inventory.
+ *
+ * Usually the tree view would show a #GtkTreeModelFilter model created by
+ * gwy_create_inventory_model_without_default(). However, this function works also for other filter models around
+ * a #GwyInventoryStore.
+ *
+ * Since: 2.62
+ **/
+void
+gwy_select_in_filtered_inventory_treeeview(GtkTreeView *treeview,
+                                           const gchar *name)
+{
+    GtkTreeModelFilter *filtermodel;
+    GwyInventoryStore *store;
+    GtkTreeIter storeiter, filteriter;
+    GtkTreeSelection *selection;
+
+    filtermodel = GTK_TREE_MODEL_FILTER(gtk_tree_view_get_model(treeview));
+    store = GWY_INVENTORY_STORE(gtk_tree_model_filter_get_model(filtermodel));
+    if (gwy_inventory_store_get_iter(store, name, &storeiter)
+        && gtk_tree_model_filter_convert_child_iter_to_iter(filtermodel, &filteriter, &storeiter)) {
+        selection = gtk_tree_view_get_selection(treeview);
+        gtk_tree_selection_select_iter(selection, &filteriter);
+    }
+}
+
 /************************** Documentation ****************************/
 
 /**
