@@ -1,6 +1,6 @@
 /*
- *  $Id: gwyresultsexport.c 24709 2022-03-21 17:31:45Z yeti-dn $
- *  Copyright (C) 2017-2021 David Necas (Yeti).
+ *  $Id: gwyresultsexport.c 24881 2022-08-12 09:02:19Z yeti-dn $
+ *  Copyright (C) 2017-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -37,6 +37,8 @@ struct _GwyResultsExportPrivate {
     GtkWidget *format;
     GtkWidget *machine;
     gchar *title;
+    GwyResultsFormatFunc format_func;
+    gpointer format_data;
     gboolean updating;
 };
 
@@ -224,27 +226,33 @@ format_changed(GtkComboBox *combo, GwyResultsExport *rexport)
     g_signal_emit(rexport, results_export_signals[FORMAT_CHANGED], 0);
 }
 
+static gchar*
+format_report(GwyResultsExport *rexport)
+{
+    ResultsExportPriv *priv = rexport->priv;
+
+    if (priv->format_func)
+        return priv->format_func(priv->format_data);
+    if (priv->results)
+        return gwy_results_create_report(rexport->priv->results, rexport->format);
+    return NULL;
+}
+
 static void
 gwy_results_export_save_impl(GwyResultsExport *rexport)
 {
-    ResultsExportPriv *priv = rexport->priv;
     GtkWindow *window;
     const gchar *title;
     gchar *text;
 
-    if (!priv->results)
+    if (!(text = format_report(rexport)))
         return;
 
-    if (rexport->priv->title)
-        title = rexport->priv->title;
-    else
-        title = _("Save Results to File");
-
-    text = gwy_results_create_report(rexport->priv->results, rexport->format);
     if ((window = (GtkWindow*)gtk_widget_get_toplevel(GTK_WIDGET(rexport)))) {
         if (!GTK_IS_WINDOW(window))
             window = NULL;
     }
+    title = rexport->priv->title ? rexport->priv->title : _("Save Results to File");
     gwy_save_auxiliary_data(title, window, -1, text);
     g_free(text);
 }
@@ -252,16 +260,14 @@ gwy_results_export_save_impl(GwyResultsExport *rexport)
 static void
 gwy_results_export_copy_impl(GwyResultsExport *rexport)
 {
-    ResultsExportPriv *priv = rexport->priv;
     GtkClipboard *clipboard;
     GdkDisplay *display;
     gchar *text;
 
-    if (!priv->results)
+    if (!(text = format_report(rexport)))
         return;
 
     display = gtk_widget_get_display(GTK_WIDGET(rexport));
-    text = gwy_results_create_report(priv->results, rexport->format);
     clipboard = gtk_clipboard_get_for_display(display, GDK_SELECTION_CLIPBOARD);
     gtk_clipboard_set_text(clipboard, text, -1);
     g_free(text);
@@ -470,6 +476,30 @@ gwy_results_export_get_results(GwyResultsExport *rexport)
     return rexport->priv->results;
 }
 
+/**
+ * gwy_results_export_set_format_func:
+ * @rexport: Controls for result set export.
+ * @func: The formatting function.
+ * @user_data: Data passed to @func.
+ *
+ * Sets the report formatting function for result set export controls.
+ *
+ * Setting the formatting function allows automated handling of Copy and Save buttons, even when there is not
+ * #GwyResults object. It is intended to be used with %GWY_RESULTS_EXPORT_FIXED_FORMAT. However, it takes precedence
+ * even when a #GwyResults object is set.
+ *
+ * Since: 2.62
+ **/
+void
+gwy_results_export_set_format_func(GwyResultsExport *rexport,
+                                   GwyResultsFormatFunc func,
+                                   gpointer user_data)
+{
+    g_return_if_fail(GWY_IS_RESULTS_EXPORT(rexport));
+    rexport->priv->format_func = func;
+    rexport->priv->format_data = user_data;
+}
+
 /**
  * gwy_results_export_set_title:
  * @rexport: Controls for result set export.
@@ -789,4 +819,17 @@ gwy_results_fill_lawn_curve(GwyResults *results,
  * @short_description: Controls for value set export
  **/
 
+/**
+ * GwyResultsFormatFunc:
+ * @user_data: Data passed to gwy_results_export_set_format_func().
+ *
+ * The type of result formatting function.
+ *
+ * Returns: Formatted report as a newly allocated string. #GwyResultsExport will free it using g_free() after saving
+ *          it to a file or copying to the clipboard.
+ *
+ * Since: 2.62
+ **/
+typedef gchar* (*GwyResultsFormatFunc)(gpointer user_data);
+
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
