@@ -1,5 +1,5 @@
 /*
- *  $Id: log.c 23947 2021-08-10 11:21:05Z yeti-dn $
+ *  $Id: log.c 25040 2022-09-22 12:32:22Z yeti-dn $
  *  Copyright (C) 2014-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -879,7 +879,7 @@ format_arg(gpointer hkey, gpointer hvalue, gpointer user_data)
     else if (G_VALUE_HOLDS_BOOLEAN(gvalue))
         formatted = g_strdup_printf("%s=%s", name, g_value_get_boolean(gvalue) ? "True" : "False");
     else if (G_VALUE_HOLDS_STRING(gvalue)) {
-        gchar *s = g_strescape(g_value_get_string(gvalue), NULL);
+        gchar *s = gwy_utf8_strescape(g_value_get_string(gvalue), NULL);
         formatted = g_strdup_printf("%s=\"%s\"", name, s);
         g_free(s);
     }
