@@ -1,5 +1,5 @@
 /*
- *  $Id: logging.c 21696 2018-11-27 21:39:57Z yeti-dn $
+ *  $Id: logging.c 22396 2019-08-18 06:38:30Z yeti-dn $
  *  Copyright (C) 2016-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -107,7 +107,7 @@ gwy_app_setup_logging(GwyAppLoggingFlags flags)
 {
     static const gchar *our_domains[] = {
         "Gwyddion", "GwyProcess", "GwyDraw", "Gwydgets", "GwyModule", "GwyApp",
-        "Module", NULL
+        "Module", "Gwyddion-Program", NULL
     };
     static const gchar *other_domains[] = {
         "GLib", "GLib-GObject", "GLib-GIO", "GModule", "GThread",
