@@ -1,22 +1,18 @@
 /*
- *  $Id: meta.c 23208 2021-03-08 15:55:47Z yeti-dn $
- *  Copyright (C) 2003-2013 David Necas (Yeti), Petr Klapetek.
+ *  $Id: meta.c 24417 2021-10-24 08:36:28Z yeti-dn $
+ *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -42,6 +38,7 @@ typedef enum {
     BROWSER_DATA_CHANNEL,
     BROWSER_DATA_VOLUME,
     BROWSER_DATA_XYZ,
+    BROWSER_DATA_CURVE_MAP,
 } BrowserDataType;
 
 typedef struct {
@@ -105,11 +102,10 @@ static void             gwy_meta_save_items           (MetadataBrowser *browser)
  *
  * Shows a simple metadata browser for a channel.
  *
- * If the metadata browser is already shown for this channel it is just raised
- * and given focus.  Otherwise, a new window is created.
+ * If the metadata browser is already shown for this channel it is just raised and given focus.  Otherwise, a new
+ * window is created.
  *
- * Returns: The metadata browser (owned by the library).  Usually, you can
- *          ignore the return value.
+ * Returns: The metadata browser (owned by the library).  Usually, you can ignore the return value.
  *
  * Since: 2.32
  **/
@@ -127,11 +123,10 @@ gwy_app_metadata_browser_for_channel(GwyContainer *data,
  *
  * Shows a simple metadata browser for volume data.
  *
- * If the metadata browser is already shown for this volume data it is just
- * raised and given focus.  Otherwise, a new window is created.
+ * If the metadata browser is already shown for this volume data it is just raised and given focus.  Otherwise, a new
+ * window is created.
  *
- * Returns: The metadata browser (owned by the library).  Usually, you can
- *          ignore the return value.
+ * Returns: The metadata browser (owned by the library).  Usually, you can ignore the return value.
  *
  * Since: 2.32
  **/
@@ -149,11 +144,10 @@ gwy_app_metadata_browser_for_volume(GwyContainer *data,
  *
  * Shows a simple metadata browser for XYZ data.
  *
- * If the metadata browser is already shown for this XYZ data it is just
- * raised and given focus.  Otherwise, a new window is created.
+ * If the metadata browser is already shown for this XYZ data it is just raised and given focus.  Otherwise, a new
+ * window is created.
  *
- * Returns: The metadata browser (owned by the library).  Usually, you can
- *          ignore the return value.
+ * Returns: The metadata browser (owned by the library).  Usually, you can ignore the return value.
  *
  * Since: 2.45
  **/
@@ -164,6 +158,27 @@ gwy_app_metadata_browser_for_xyz(GwyContainer *data,
     return get_metadata_browser(data, BROWSER_DATA_XYZ, id);
 }
 
+/**
+ * gwy_app_metadata_browser_for_curve_map:
+ * @data: A data container.
+ * @id: Id of curve map data in @data to show metadata for.
+ *
+ * Shows a simple metadata browser for curve map data.
+ *
+ * If the metadata browser is already shown for this curve map data it is just raised and given focus.  Otherwise,
+ * a new window is created.
+ *
+ * Returns: The metadata browser (owned by the library).  Usually, you can ignore the return value.
+ *
+ * Since: 2.60
+ **/
+GtkWidget*
+gwy_app_metadata_browser_for_curve_map(GwyContainer *data,
+                                       gint id)
+{
+    return get_metadata_browser(data, BROWSER_DATA_CURVE_MAP, id);
+}
+
 static GtkWidget*
 get_metadata_browser(GwyContainer *data,
                      BrowserDataType type,
@@ -193,54 +208,44 @@ metadata_browser_new(void)
     browser = g_new0(MetadataBrowser, 1);
     gwy_meta_browser_construct(browser);
     browser->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    gwy_help_add_to_window(GTK_WINDOW(browser->window), "metadata", NULL,
-                           GWY_HELP_DEFAULT);
+    gwy_help_add_to_window(GTK_WINDOW(browser->window), "metadata", NULL, GWY_HELP_DEFAULT);
 
     gtk_widget_size_request(browser->treeview, &request);
     request.width = MAX(request.width, 320);
     request.height = MAX(request.height, 400);
     gtk_window_set_default_size(GTK_WINDOW(browser->window),
                                 MIN(request.width + 24, 2*gdk_screen_width()/3),
-                                MIN(request.height + 32,
-                                    2*gdk_screen_height()/3));
+                                MIN(request.height + 32, 2*gdk_screen_height()/3));
     gwy_app_add_main_accel_group(GTK_WINDOW(browser->window));
 
     vbox = gtk_vbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(browser->window), vbox);
 
     scroll = gtk_scrolled_window_new(NULL, NULL);
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
-                                   GTK_POLICY_AUTOMATIC,
-                                   GTK_POLICY_AUTOMATIC);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);
     gtk_container_add(GTK_CONTAINER(scroll), browser->treeview);
-    browser->destroy_id
-        = g_signal_connect_swapped(browser->window, "destroy",
-                                   G_CALLBACK(gwy_meta_destroy), browser);
+    browser->destroy_id = g_signal_connect_swapped(browser->window, "destroy", G_CALLBACK(gwy_meta_destroy), browser);
 
     hbox = gtk_hbox_new(TRUE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
     browser->save = gwy_stock_like_button_new(_("_Export"), GTK_STOCK_SAVE);
     gtk_box_pack_start(GTK_BOX(hbox), browser->save, TRUE, TRUE, 0);
-    g_signal_connect_swapped(browser->save, "clicked",
-                             G_CALLBACK(gwy_meta_save_items), browser);
+    g_signal_connect_swapped(browser->save, "clicked", G_CALLBACK(gwy_meta_save_items), browser);
     gtk_widget_set_sensitive(browser->save, FALSE);
 
     browser->new = gwy_stock_like_button_new(_("_New"), GTK_STOCK_NEW);
     gtk_box_pack_start(GTK_BOX(hbox), browser->new, TRUE, TRUE, 0);
-    g_signal_connect_swapped(browser->new, "clicked",
-                             G_CALLBACK(gwy_meta_new_item), browser);
+    g_signal_connect_swapped(browser->new, "clicked", G_CALLBACK(gwy_meta_new_item), browser);
 
     browser->delete = gwy_stock_like_button_new(_("_Delete"), GTK_STOCK_DELETE);
     gtk_box_pack_start(GTK_BOX(hbox), browser->delete, TRUE, TRUE, 0);
-    g_signal_connect_swapped(browser->delete, "clicked",
-                             G_CALLBACK(gwy_meta_delete_item), browser);
+    g_signal_connect_swapped(browser->delete, "clicked", G_CALLBACK(gwy_meta_delete_item), browser);
 
     browser->close = gwy_stock_like_button_new(_("_Close"), GTK_STOCK_CLOSE);
     gtk_box_pack_start(GTK_BOX(hbox), browser->close, TRUE, TRUE, 0);
-    g_signal_connect_swapped(browser->close, "clicked",
-                             G_CALLBACK(gtk_widget_destroy), browser->window);
+    g_signal_connect_swapped(browser->close, "clicked", G_CALLBACK(gtk_widget_destroy), browser->window);
 
     return browser;
 }
@@ -287,12 +292,13 @@ gwy_meta_update_title(MetadataBrowser *browser,
         dataname = gwy_app_get_brick_title(data, id);
     else if (type == BROWSER_DATA_XYZ)
         dataname = gwy_app_get_surface_title(data, id);
+    else if (type == BROWSER_DATA_CURVE_MAP)
+        dataname = gwy_app_get_lawn_title(data, id);
     else {
         g_return_if_reached();
     }
 
-    title = g_strdup_printf(_("Metadata of %s (%s)"),
-                            dataname, g_get_application_name());
+    title = g_strdup_printf(_("Metadata of %s (%s)"), dataname, g_get_application_name());
     gtk_window_set_title(GTK_WINDOW(browser->window), title);
     g_free(title);
     g_free(dataname);
@@ -323,6 +329,8 @@ gwy_meta_switch_data(MetadataBrowser *browser,
         quark = gwy_app_get_brick_meta_key_for_id(id);
     else if (type == BROWSER_DATA_XYZ)
         quark = gwy_app_get_surface_meta_key_for_id(id);
+    else if (type == BROWSER_DATA_CURVE_MAP)
+        quark = gwy_app_get_lawn_meta_key_for_id(id);
     else {
         g_return_val_if_reached(NULL);
     }
@@ -335,8 +343,7 @@ gwy_meta_switch_data(MetadataBrowser *browser,
 
         fixlist = NULL;
         store = gtk_list_store_new(1, G_TYPE_UINT);
-        gwy_container_foreach(meta, NULL,
-                              (GHFunc)(gwy_meta_browser_add_line), &fixlist);
+        gwy_container_foreach(meta, NULL, (GHFunc)(gwy_meta_browser_add_line), &fixlist);
         /* Commit UTF-8 fixes found by gwy_meta_browser_add_line() */
         for (l = fixlist; l; l = g_slist_next(l)) {
             FixupData *fd = (FixupData*)l->data;
@@ -344,9 +351,7 @@ gwy_meta_switch_data(MetadataBrowser *browser,
             if (fd->isok || fd->value) {
                 if (!fd->isok)
                     gwy_container_set_string(meta, fd->quark, fd->value);
-                gtk_list_store_insert_with_values(store, &iter, G_MAXINT,
-                                                  META_KEY, fd->quark,
-                                                  -1);
+                gtk_list_store_insert_with_values(store, &iter, G_MAXINT, META_KEY, fd->quark, -1);
             }
             else
                 gwy_container_remove(meta, fd->quark);
@@ -367,32 +372,23 @@ gwy_meta_switch_data(MetadataBrowser *browser,
     GWY_SIGNAL_HANDLER_DISCONNECT(browser->meta, browser->changed_id);
     if (browser->meta) {
         g_object_set_data(G_OBJECT(browser->meta), "metadata-browser", NULL);
-        g_object_weak_unref(G_OBJECT(browser->meta),
-                            (GWeakNotify)&gwy_meta_data_finalized,
-                            browser);
+        g_object_weak_unref(G_OBJECT(browser->meta), (GWeakNotify)&gwy_meta_data_finalized, browser);
     }
     browser->meta = meta;
     g_object_set_data(G_OBJECT(browser->meta), "metadata-browser", browser);
-    g_object_weak_ref(G_OBJECT(browser->meta),
-                      (GWeakNotify)&gwy_meta_data_finalized,
-                      browser);
+    g_object_weak_ref(G_OBJECT(browser->meta), (GWeakNotify)&gwy_meta_data_finalized, browser);
 
-    browser->changed_id = g_signal_connect(meta, "item-changed",
-                                           G_CALLBACK(gwy_meta_item_changed),
-                                           browser);
+    browser->changed_id = g_signal_connect(meta, "item-changed", G_CALLBACK(gwy_meta_item_changed), browser);
 
     treeview = GTK_TREE_VIEW(browser->treeview);
     gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store));
     g_object_unref(store);
 
     /* Search column must set whenver the model changes */
-    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), 0,
-                                    gwy_meta_sort_func, NULL, NULL);
-    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store), META_KEY,
-                                         GTK_SORT_ASCENDING);
+    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), 0, gwy_meta_sort_func, NULL, NULL);
+    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store), META_KEY, GTK_SORT_ASCENDING);
     gtk_tree_view_set_search_column(treeview, META_KEY);
-    gtk_tree_view_set_search_equal_func(treeview, gwy_meta_equal_func,
-                                        browser, NULL);
+    gtk_tree_view_set_search_equal_func(treeview, gwy_meta_equal_func, browser, NULL);
 
     gwy_meta_update_title(browser, data, type, id);
 
@@ -425,16 +421,11 @@ gwy_meta_browser_construct(MetadataBrowser *browser)
     for (i = 0; i < G_N_ELEMENTS(columns); i++) {
         renderer = gtk_cell_renderer_text_new();
         g_object_set(renderer, "editable", TRUE, "editable-set", TRUE, NULL);
-        column = gtk_tree_view_column_new_with_attributes(_(columns[i].title),
-                                                          renderer, NULL);
-        gtk_tree_view_column_set_cell_data_func(column, renderer,
-                                                gwy_meta_browser_cell_renderer,
-                                                browser, NULL);
+        column = gtk_tree_view_column_new_with_attributes(_(columns[i].title), renderer, NULL);
+        gtk_tree_view_column_set_cell_data_func(column, renderer, gwy_meta_browser_cell_renderer, browser, NULL);
         gtk_tree_view_append_column(treeview, column);
-        g_object_set_data(G_OBJECT(renderer), "column",
-                          GUINT_TO_POINTER(columns[i].id));
-        g_signal_connect(renderer, "edited",
-                         G_CALLBACK(gwy_meta_cell_edited), browser);
+        g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(columns[i].id));
+        g_signal_connect(renderer, "edited", G_CALLBACK(gwy_meta_cell_edited), browser);
     }
 
     selection = gtk_tree_view_get_selection(treeview);
@@ -541,8 +532,8 @@ gwy_meta_browser_add_line(gpointer hkey,
     quark = GPOINTER_TO_UINT(hkey);
     val = g_value_get_string(value);
 
-    /* Theoretically, modules should assure metadata are in UTF-8 when it's
-     * stored to container.  But in practice we cannot rely on it. */
+    /* Theoretically, modules should assure metadata are in UTF-8 when it's stored to container.  But in practice we
+     * cannot rely on it. */
     fd = g_new0(FixupData, 1);
     fd->quark = quark;
     if (g_utf8_validate(val, -1, NULL)) {
@@ -552,14 +543,12 @@ gwy_meta_browser_add_line(gpointer hkey,
     else if ((s = g_convert(val, -1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL)))
         fd->value = s;
     else {
-        g_warning("Bogus metadata <%s> at key <%s>",
-                  val, g_quark_to_string(quark));
+        g_warning("Bogus metadata <%s> at key <%s>", val, g_quark_to_string(quark));
         g_free(fd);
         return;
     }
 
-    /* The same applies to markup validity.  Fix invalid markup by taking it
-     * literally. */
+    /* The same applies to markup validity.  Fix invalid markup by taking it literally. */
     if (!pango_parse_markup(fd->value, -1, 0, NULL, NULL, NULL, NULL)) {
         s = g_markup_escape_text(fd->value, -1);
         if (!fd->isok)
@@ -595,8 +584,7 @@ gwy_meta_item_changed(GwyContainer *container,
         return;
     }
 
-    gtk_list_store_insert_with_values(store, &iter, G_MAXINT, META_KEY, quark,
-                                      -1);
+    gtk_list_store_insert_with_values(store, &iter, G_MAXINT, META_KEY, quark, -1);
     gwy_meta_focus_iter(browser, &iter);
 }
 
@@ -604,11 +592,9 @@ static void
 gwy_meta_new_item(MetadataBrowser *browser)
 {
     static const gchar *whatever[] = {
-        "angary", "bistere", "couchant", "dolerite", "envoy", "figwort",
-        "gudgeon", "hidalgo", "ictus", "jibbah", "kenosis", "logie",
-        "maser", "nephology", "ozalid", "parallax", "reduit", "savate",
-        "thyristor", "urate", "versicle", "wapentake", "xystus",
-        "yogh", "zeugma",
+        "angary", "bistere", "couchant", "dolerite", "envoy", "figwort", "gudgeon", "hidalgo", "ictus", "jibbah",
+        "kenosis", "logie", "maser", "nephology", "ozalid", "parallax", "reduit", "savate", "thyristor", "urate",
+        "versicle", "wapentake", "xystus", "yogh", "zeugma",
     };
     GtkTreeIter iter;
     static GQuark quark = 0;
@@ -659,9 +645,7 @@ gwy_meta_destroy(MetadataBrowser *browser)
 {
     GWY_SIGNAL_HANDLER_DISCONNECT(browser->meta, browser->changed_id);
     g_object_set_data(G_OBJECT(browser->meta), "metadata-browser", NULL);
-    g_object_weak_unref(G_OBJECT(browser->meta),
-                        (GWeakNotify)&gwy_meta_data_finalized,
-                        browser);
+    g_object_weak_unref(G_OBJECT(browser->meta), (GWeakNotify)&gwy_meta_data_finalized, browser);
     g_free(browser);
 }
 
@@ -727,19 +711,13 @@ gwy_meta_save_items(MetadataBrowser *browser)
         do {
             gtk_tree_model_get(model, &iter, META_KEY, &name, -1);
             value = gwy_container_get_string(browser->meta, name);
-            g_string_append_printf(str_to_save, "%s %s\n",
-                                   g_quark_to_string(name),
-                                   value);
+            g_string_append_printf(str_to_save, "%s %s\n", g_quark_to_string(name), value);
         } while (gtk_tree_model_iter_next(model, &iter));
     }
 
-    gwy_save_auxiliary_data(_("Save Metadata"),
-                            GTK_WINDOW(browser->window),
-                            -1,
-                            str_to_save->str);
+    gwy_save_auxiliary_data(_("Save Metadata"), GTK_WINDOW(browser->window), -1, str_to_save->str);
 
     g_string_free(str_to_save, TRUE);
-
 }
 
 /************************** Documentation ****************************/
@@ -750,4 +728,4 @@ gwy_meta_save_items(MetadataBrowser *browser)
  * @short_description: Metadata browser
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
