@@ -1,5 +1,5 @@
 /*
- *  $Id: param-def.c 24645 2022-03-08 14:26:25Z yeti-dn $
+ *  $Id: param-def.c 25075 2022-10-11 13:47:20Z yeti-dn $
  *  Copyright (C) 2021-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -39,33 +39,33 @@ struct _GwyParamDefPrivate {
 #endif
 };
 
-static void   gwy_param_def_finalize    (GObject *gobject);
-static void   gwy_param_def_append      (GwyParamDef *pardef,
-                                         gint id,
-                                         const gchar *name,
-                                         const gchar *desc,
-                                         GwyParamDefItem *item);
-static void   gwy_param_def_append_enum (GwyParamDef *pardef,
-                                         gint id,
-                                         const gchar *name,
-                                         const gchar *desc,
-                                         GType gtype,
-                                         const GwyEnum *values,
-                                         gint nvalues,
-                                         gint default_value);
-static void   gwy_param_def_append_flags(GwyParamDef *pardef,
-                                         gint id,
-                                         const gchar *name,
-                                         const gchar *desc,
-                                         GType gtype,
-                                         const GwyEnum *values,
-                                         gint nvalues,
-                                         guint default_value);
-static gchar* rectify_string            (const gchar *value,
-                                         GwyParamStringFlags flags,
-                                         GwyRectifyStringFunc rectify);
-static gint   find_param_def            (GwyParamDefPrivate *priv,
-                                         gint id);
+static void         gwy_param_def_finalize       (GObject *gobject);
+static void         gwy_param_def_append         (GwyParamDef *pardef,
+                                                  gint id,
+                                                  const gchar *name,
+                                                  const gchar *desc,
+                                                  GwyParamDefItem *item);
+static void         gwy_param_def_append_enum    (GwyParamDef *pardef,
+                                                  gint id,
+                                                  const gchar *name,
+                                                  const gchar *desc,
+                                                  GType gtype,
+                                                  const GwyEnum *values,
+                                                  gint nvalues,
+                                                  gint default_value);
+static void         gwy_param_def_append_flags   (GwyParamDef *pardef,
+                                                  gint id,
+                                                  const gchar *name,
+                                                  const gchar *desc,
+                                                  GType gtype,
+                                                  const GwyEnum *values,
+                                                  gint nvalues,
+                                                  guint default_value);
+static gchar*       rectify_string               (const gchar *value,
+                                                  GwyParamStringFlags flags,
+                                                  GwyRectifyStringFunc rectify);
+static gint         find_param_def               (GwyParamDefPrivate *priv,
+                                                  gint id);
 
 G_DEFINE_TYPE(GwyParamDef, gwy_param_def, G_TYPE_OBJECT);
 
@@ -163,6 +163,57 @@ gwy_param_def_get_function_name(GwyParamDef *pardef)
     return pardef->priv->function_name;
 }
 
+/**
+ * gwy_param_def_get_param_name:
+ * @pardef: A set of parameter definitions.
+ * @id: Parameter identifier.
+ *
+ * Gets the name of an already defined parameter in a set of parameter definitions.
+ *
+ * See gwy_param_def_get_param_label() for parameter labels.
+ *
+ * Returns: The parameter name, as a string owned by @pardef. It may be %NULL if the parameter is not stored to
+ * settings.
+ *
+ * Since: 2.62
+ **/
+const gchar*
+gwy_param_def_get_param_name(GwyParamDef *pardef,
+                             gint id)
+{
+    const GwyParamDefItem *item;
+
+    g_return_val_if_fail(GWY_IS_PARAM_DEF(pardef), NULL);
+    item = _gwy_param_def_item(pardef, _gwy_param_def_index(pardef, id));
+    g_return_val_if_fail(item, NULL);
+    return item->name;
+}
+
+/**
+ * gwy_param_def_get_param_label:
+ * @pardef: A set of parameter definitions.
+ * @id: Parameter identifier.
+ *
+ * Gets the label of an already defined parameter in a set of parameter definitions.
+ *
+ * See gwy_param_def_get_param_name() for parameter names (keys).
+ *
+ * Returns: The parameter name, as a string owned by @pardef. It may be %NULL if the parameter has no label.
+ *
+ * Since: 2.62
+ **/
+const gchar*
+gwy_param_def_get_param_label(GwyParamDef *pardef,
+                              gint id)
+{
+    const GwyParamDefItem *item;
+
+    g_return_val_if_fail(GWY_IS_PARAM_DEF(pardef), NULL);
+    item = _gwy_param_def_item(pardef, _gwy_param_def_index(pardef, id));
+    g_return_val_if_fail(item, NULL);
+    return item->desc;
+}
+
 static gint
 count_enum_values(const GwyEnum *values, gint nvalues)
 {
@@ -201,6 +252,8 @@ find_enum_value(const GwyEnum *values, gint nvalues, gint value)
  *
  * See the introduction for @name and @desc lifetime rules.
  *
+ * See gwy_param_def_add_enum() for predefined enums.
+ *
  * Since: 2.59
  **/
 void
@@ -262,6 +315,8 @@ gwy_param_def_add_gwyflags(GwyParamDef *pardef,
  *
  * See the introduction for @name and @desc lifetime rules.
  *
+ * See gwy_param_def_add_gwyenum() for enums defined by a #GwyEnum table.
+ *
  * Since: 2.59
  **/
 void
@@ -301,6 +356,10 @@ gwy_param_def_add_enum(GwyParamDef *pardef,
         values = gwy_distance_transform_type_get_enum();
         standard_desc = _("_Distance type");
     }
+    else if (enum_gtype == GWY_TYPE_GRAPH_CURVE_TYPE) {
+        values = gwy_graph_curve_type_get_enum();
+        standard_desc = _("Plot _style");
+    }
     else {
         /* We could play with GEnumClass here but why.  It should be implemented if it is used. */
         standard_desc = g_type_name(enum_gtype);
@@ -1411,10 +1470,11 @@ _gwy_param_def_use(GwyParamDef *pardef, GwyParams *params)
      * use binary search. */
     priv->is_used = TRUE;
 #ifdef DEBUG_USERS
-    /* There are valid use cases for multiple GwyParams instances created for one GwyParamDef.  But they do not occur
-     * during normal Gwyddion operation.  So enable this check during development. */
+    /* When presets are implemented using GwyParams then multiple GwyParams instances created for one GwyParamDef
+     * are common.  Keep enabled this check during development but we may want a mechanism to exclude presets. */
     if (!g_slist_find(priv->users, params)) {
-        if (priv->users) {
+        /* XXX: Hardcoded warning suppression for known modules with GwyParamResource presets. */
+        if (priv->users && !gwy_stramong(priv->function_name, "rawfile", NULL)) {
             g_warning("Parameter definitions for %s are used multiple times.  "
                       "Check module function %s; it is probably leaking GwyParams objects!",
                       priv->function_name, priv->function_name);
