@@ -1,5 +1,5 @@
 /*
- *  $Id: param-def.h 24645 2022-03-08 14:26:25Z yeti-dn $
+ *  $Id: param-def.h 24840 2022-06-03 11:43:26Z yeti-dn $
  *  Copyright (C) 2021-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -57,6 +57,10 @@ GwyParamDef* gwy_param_def_new                   (void);
 void         gwy_param_def_set_function_name     (GwyParamDef *pardef,
                                                   const gchar *name);
 const gchar* gwy_param_def_get_function_name     (GwyParamDef *pardef)                 G_GNUC_PURE;
+const gchar* gwy_param_def_get_param_name        (GwyParamDef *pardef,
+                                                  gint id)                             G_GNUC_PURE;
+const gchar* gwy_param_def_get_param_label       (GwyParamDef *pardef,
+                                                  gint id)                             G_GNUC_PURE;
 void         gwy_param_def_add_gwyenum           (GwyParamDef *pardef,
                                                   gint id,
                                                   const gchar *name,
