@@ -1,6 +1,6 @@
 /*
- *  $Id: param-internal.h 24330 2021-10-11 15:45:25Z yeti-dn $
- *  Copyright (C) 2021 David Necas (Yeti).
+ *  $Id: param-internal.h 24941 2022-08-25 18:51:02Z yeti-dn $
+ *  Copyright (C) 2021-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -21,6 +21,7 @@
 #define __GWY_PARAM_INTERNAL_H__
 
 #include "dialog.h"
+#include "gwyplaintool.h"
 
 G_BEGIN_DECLS
 
@@ -214,6 +215,10 @@ G_GNUC_INTERNAL
 void _gwy_param_table_set_parent_dialog(GwyParamTable *partable,
                                         GwyDialog *dialog);
 
+G_GNUC_INTERNAL
+void _gwy_param_table_set_parent_plain_tool(GwyParamTable *partable,
+                                            GwyPlainTool *plain_tool);
+
 G_GNUC_INTERNAL
 void _gwy_param_table_proceed(GwyParamTable *partable);
 
@@ -223,6 +228,21 @@ void _gwy_dialog_param_table_update_started(GwyDialog *dialog);
 G_GNUC_INTERNAL
 void _gwy_dialog_param_table_update_finished(GwyDialog *dialog);
 
+G_GNUC_INTERNAL
+void _gwy_plain_tool_param_table_update_started(GwyPlainTool *plain_tool);
+
+G_GNUC_INTERNAL
+void _gwy_plain_tool_param_table_update_finished(GwyPlainTool *plain_tool);
+
+G_GNUC_INTERNAL
+void _gwy_params_dump_to_string(GwyParams *params,
+                                GString *str);
+
+G_GNUC_INTERNAL
+void _gwy_params_parse_string(GwyParams *params,
+                              const gchar *text,
+                              GHashTable *parmap);
+
 G_GNUC_UNUSED
 static inline gboolean
 param_type_is_data_id(GwyParamType type)
