@@ -1,5 +1,5 @@
 /*
- *  $Id: params.c 24672 2022-03-15 12:32:09Z yeti-dn $
+ *  $Id: params.c 25038 2022-09-22 12:26:38Z yeti-dn $
  *  Copyright (C) 2021-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,6 +16,7 @@
  */
 
 #include "config.h"
+#include <stdlib.h>
 #include "libgwyddion/gwymacros.h"
 #include "libprocess/gwyprocesstypes.h"
 #include "libprocess/gwyprocessenums.h"
@@ -109,6 +110,9 @@ static guint       get_flags_internal    (GwyParams *params,
                                           GType type);
 static gboolean    reset_param_value     (GwyParams *params,
                                           guint i);
+static gboolean    assign_parameter_value(GwyParamValue *value,
+                                          const GwyParamDefItem *def,
+                                          GwyParamValue *src);
 static gpointer    get_data_object       (GwyParams *params,
                                           gint id,
                                           GwyParamType param_type,
@@ -159,6 +163,8 @@ gwy_params_finalize(GObject *gobject)
 
         if (type == GWY_PARAM_STRING || type == GWY_PARAM_RESOURCE)
             g_free(values[i].s);
+        else if (param_type_is_curve_no(type))
+            g_free(values[i].cu.s);
         else if (type == GWY_PARAM_UNIT) {
             GWY_OBJECT_UNREF(values[i].si.cached_unit);
             g_free(values[i].si.s);
@@ -222,17 +228,84 @@ gwy_params_duplicate(GwyParams *params)
     /* Allocated values must be handled specially, but everything else can be just a bitwise copy. */
     gwy_assign(values, priv->values, n);
     for (i = 0; i < n; i++) {
-        const GwyParamDefItem *def = _gwy_param_def_item(priv->def, i);
+        GwyParamType type = _gwy_param_def_item(priv->def, i)->type;
 
-        if (def->type == GWY_PARAM_STRING)
+        if (type == GWY_PARAM_STRING || type == GWY_PARAM_RESOURCE)
             values[i].s = g_strdup(values[i].s);
-        else if (def->type == GWY_PARAM_UNIT)
+        else if (param_type_is_curve_no(type))
+            values[i].cu.s = g_strdup(values[i].cu.s);
+        else if (type == GWY_PARAM_UNIT) {
             values[i].si.s = g_strdup(values[i].si.s);
+            if (values[i].si.cached_unit)
+                values[i].si.cached_unit = gwy_si_unit_duplicate(values[i].si.cached_unit);
+        }
     }
 
     return copy;
 }
 
+/**
+ * gwy_params_assign:
+ * @params: A set of parameter values.
+ * @source: A set of parameter values.
+ *
+ * Assigns values to a set of parameter values from another one.
+ *
+ * The parameter sets must have compatible definitions. Usually, the #GwyParamDef of @source is the same as the one of
+ * @params, a subset, or a superset. In other words, parameters existing only in one objects are ignored. Usually
+ * a #GwyParamDef superset is created by defining one parameter set, then defining another one exactly the same way
+ * and adding a few more parameters.
+ *
+ * Parameters are identified by id. Both parameter ids and names must match. However, a %NULL name is considered to
+ * match any name – if the ids are equal. Of course, parameter types must also match.
+ *
+ * Since: 2.62
+ **/
+void
+gwy_params_assign(GwyParams *params,
+                  GwyParams *source)
+{
+    GwyParamsPrivate *priv, *spriv;
+    GwyParamValue *values, *svalues;
+    guint k, sn;
+
+    g_return_if_fail(GWY_IS_PARAMS(params));
+    g_return_if_fail(GWY_IS_PARAMS(source));
+
+    priv = params->priv;
+    spriv = source->priv;
+    values = priv->values;
+    svalues = spriv->values;
+    sn = spriv->nvalues;
+    g_return_if_fail(priv->def);
+    g_return_if_fail(spriv->def);
+
+    for (k = 0; k < sn; k++) {
+        /* Look up the parameter by id. There may be a use case for a more loose function identfying parameters by
+         * name, but it would probably be a different function. */
+        const GwyParamDefItem *def, *srcdef = _gwy_param_def_item(spriv->def, k);
+        gint id = srcdef->id;
+        gint i = _gwy_param_def_index(priv->def, id);
+
+        if (i < 0) {
+            //g_warning("Destination GwyParams has no parameter with id %d.", id);
+            continue;
+        }
+        def = _gwy_param_def_item(priv->def, i);
+        if (srcdef->type != def->type) {
+            g_warning("Destination parameter %d type is %d, but source's is %d.", id, def->type, srcdef->type);
+            continue;
+        }
+        if (srcdef->name && def->name && !gwy_strequal(srcdef->name, def->name)) {
+            g_warning("Destination parameter %d is called %s, but source's is %s.", id, def->name, srcdef->name);
+            continue;
+        }
+        /* We do not check if the parameter definitions match exactly. The set-parameter function will still spit out
+         * a warning if the value is not valid. So we currently just ignore the return value. */
+        assign_parameter_value(values + i, def, svalues + k);
+    }
+}
+
 /**
  * gwy_params_set_def:
  * @params: A parameter value set.
@@ -1639,6 +1712,8 @@ gwy_params_set_resource(GwyParams *params,
  *
  * Gets the resource object of a resource name parameter.
  *
+ * Use gwy_params_get_string() to get the resource name instead of the object.
+ *
  * Returns: The resource object corresponding to the parameter value.  A default value is returned if the resource
  *          does not exist.
  *
@@ -1937,6 +2012,45 @@ reset_param_value(GwyParams *params, guint i)
     g_return_val_if_reached(FALSE);
 }
 
+static gboolean
+assign_parameter_value(GwyParamValue *value, const GwyParamDefItem *def, GwyParamValue *src)
+{
+    GwyParamType type = def->type;
+
+    if (type == GWY_PARAM_BOOLEAN)
+        return set_boolean_value(value, src->b);
+    if (type == GWY_PARAM_INT)
+        return set_int_value(value, def, src->i);
+    if (type == GWY_PARAM_ENUM)
+        return set_enum_value(value, def, src->i);
+    if (type == GWY_PARAM_FLAGS)
+        return set_flags_value(value, def, src->u);
+    if (type == GWY_PARAM_REPORT_TYPE)
+        return set_report_type_value(value, def, src->u);
+    if (type == GWY_PARAM_RANDOM_SEED)
+        return set_random_seed_value(value, def, src->i);
+    if (type == GWY_PARAM_ACTIVE_PAGE)
+        return set_int_value(value, def, src->i);
+    if (type == GWY_PARAM_DOUBLE)
+        return set_double_value(value, def, src->d);
+    if (type == GWY_PARAM_STRING)
+        return set_string_value(value, def, src->s);
+    if (type == GWY_PARAM_UNIT)
+        return set_unit_value(value, def, src->si.s);
+    if (type == GWY_PARAM_RESOURCE)
+        return set_resource_value(value, def, src->s);
+    if (type == GWY_PARAM_COLOR)
+        return set_color_value(value, def, src->c);
+    if (param_type_is_data_id(type))
+        return set_data_id_value(value, src->di);
+    if (param_type_is_curve_no(type)) {
+        if (src->cu.use_string)
+            return set_curve_string_value(value, def, src->cu.s);
+        return set_curve_int_value(value, def, src->cu.i);
+    }
+    g_return_val_if_reached(FALSE);
+}
+
 /* This also works for curve parameters, by getting the parent object. */
 static gpointer
 get_data_object(GwyParams *params, gint id,
@@ -1999,6 +2113,255 @@ ensure_data_ids(void)
     return data_ids;
 }
 
+static void
+append_escaped_string(GString *str, const gchar *s)
+{
+    gchar *escaped = gwy_utf8_strescape(s, NULL);
+
+    g_string_append_c(str, '"');
+    g_string_append(str, escaped);
+    g_string_append_c(str, '"');
+    g_free(escaped);
+}
+
+static void
+append_double(GString *str, gdouble d)
+{
+    gchar dbuf[G_ASCII_DTOSTR_BUF_SIZE];
+
+    g_ascii_dtostr(dbuf, sizeof(dbuf), d);
+    g_string_append(str, dbuf);
+}
+
+static gchar*
+replace_suffix(const gchar *s, const gchar *suffix, const gchar *newsuffix)
+{
+    guint len = strlen(s), slen = strlen(suffix), nlen;
+    gchar *t;
+
+    if (slen > len || strcmp(s + len - slen, suffix))
+        return NULL;
+
+    nlen = strlen(newsuffix);
+    t = g_new(gchar, len+1 + nlen - slen);
+    memcpy(t, s, len-slen);
+    memcpy(t + len-slen, newsuffix, nlen+1);
+    return t;
+}
+
+void
+_gwy_params_dump_to_string(GwyParams *params, GString *str)
+{
+    GwyParamsPrivate *priv;
+    guint i;
+
+    g_return_if_fail(GWY_IS_PARAMS(params));
+    priv = params->priv;
+    g_assert(priv->nvalues == _gwy_param_def_size(priv->def));
+    for (i = 0; i < priv->nvalues; i++) {
+        const GwyParamDefItem *def = _gwy_param_def_item(priv->def, i);
+        const GwyParamValue *value = priv->values + i;
+        const gchar *name = def->name;
+        GwyParamType type = def->type;
+        gchar *t;
+
+        /* Silent skips. Parameters without name are not meant to be stored. Data id parameters are not mean to be
+         * stored either – but there is nothing wrong with having them. */
+        if (!name || param_type_is_data_id(type))
+            continue;
+
+        /* Warninigs. */
+        if (!gwy_strisident(name, "-_/", NULL)) {
+            g_warning("Parameter %s has non-identifier name.", name);
+            continue;
+        }
+
+        g_string_append(str, name);
+        g_string_append_c(str, ' ');
+
+        if (type == GWY_PARAM_BOOLEAN)
+            g_string_append_printf(str, "%d", value->b);
+        else if (type == GWY_PARAM_INT || type == GWY_PARAM_ENUM
+                 || type == GWY_PARAM_RANDOM_SEED || type == GWY_PARAM_ACTIVE_PAGE)
+            g_string_append_printf(str, "%d", value->i);
+        else if (type == GWY_PARAM_FLAGS)
+            g_string_append_printf(str, "%u", value->u);
+        else if (type == GWY_PARAM_REPORT_TYPE)
+            g_string_append_printf(str, "%u", value->rt);
+        else if (param_type_is_curve_no(type)) {
+            if (value->cu.use_string)
+                append_escaped_string(str, value->cu.s);
+            else
+                g_string_append_printf(str, "%d", value->cu.i);
+        }
+        else if (type == GWY_PARAM_DOUBLE)
+            append_double(str, value->d);
+        else if (type == GWY_PARAM_STRING || type == GWY_PARAM_RESOURCE)
+            append_escaped_string(str, value->s);
+        else if (type == GWY_PARAM_UNIT)
+            append_escaped_string(str, value->si.s);
+        else if (type == GWY_PARAM_COLOR) {
+            append_double(str, value->c.r);
+            g_string_append_c(str, ' ');
+            append_double(str, value->c.g);
+            g_string_append_c(str, ' ');
+            append_double(str, value->c.b);
+
+            /* XXX: This is how we historically did it. */
+            if (def->def.c.has_alpha) {
+                if ((t = replace_suffix(name, "_color", "_opacity"))) {
+                    g_string_append_printf(str, "\n%s ", t);
+                    append_double(str, value->c.a);
+                    g_free(t);
+                }
+                else {
+                    g_warning("Cannot figure out corresponding opacity name for %s.", name);
+                }
+            }
+        }
+        else {
+            g_assert_not_reached();
+            g_string_append(str, "0");
+        }
+        g_string_append_c(str, '\n');
+    }
+}
+
+static gboolean
+unescape_string(gchar *s, gchar **target)
+{
+    guint len = (s ? strlen(s) : 0);
+    gchar *unescaped;
+
+    if (len < 2 || s[0] != '"' || s[len-1] != '"')
+        return FALSE;
+
+    s[len-1] = '\0';
+    unescaped = g_strcompress(s+1);
+    gwy_assign_string(target, unescaped);
+    g_free(unescaped);
+
+    return TRUE;
+}
+
+void
+_gwy_params_parse_string(GwyParams *params, const gchar *text, GHashTable *parmap)
+{
+    gchar *s, *line, *p;
+    guint i, n;
+    GwyParamsPrivate *priv;
+
+    g_return_if_fail(GWY_IS_PARAMS(params));
+    priv = params->priv;
+    g_assert(priv->nvalues == _gwy_param_def_size(priv->def));
+
+    p = s = g_strdup(text);
+    n = priv->nvalues;
+    while ((line = gwy_str_next_line(&p))) {
+        gchar *key, *val, *end, *altkey = NULL, *ustr = NULL;
+        const GwyParamDefItem *def;
+        GwyParamValue *parvalue;
+        GwyParamType type;
+
+        g_strstrip(line);
+        key = line;
+        if (!*key)
+            continue;
+        val = strchr(key, ' ');
+        if (val) {
+            *val = '\0';
+            val++;
+            g_strstrip(val);
+        }
+        if (!val || !*val) {
+            g_warning("Missing val for `%s'.", key);
+            continue;
+        }
+
+        i = GPOINTER_TO_UINT(g_hash_table_lookup(parmap, (gpointer)key));
+        if (!i) {
+            if ((altkey = replace_suffix(key, "_opacity", "_color")))
+                i = GPOINTER_TO_UINT(g_hash_table_lookup(parmap, (gpointer)altkey));
+            if (!i) {
+                g_warning("Unknown parameter `%s'.", key);
+                GWY_FREE(altkey);
+                continue;
+            }
+        }
+        i--;
+        g_return_if_fail(i < n);
+
+        def = _gwy_param_def_item(priv->def, i);
+        parvalue = priv->values + i;
+        type = def->type;
+
+        if (type == GWY_PARAM_BOOLEAN)
+            set_boolean_value(parvalue, !!atoi(val));
+        else if (type == GWY_PARAM_INT || type == GWY_PARAM_ACTIVE_PAGE)
+            set_int_value(parvalue, def, atoi(val));
+        else if (type == GWY_PARAM_ENUM)
+            set_enum_value(parvalue, def, atoi(val));
+        else if (type == GWY_PARAM_RANDOM_SEED)
+            set_random_seed_value(parvalue, def, atoi(val));
+        else if (type == GWY_PARAM_FLAGS) {
+            guint u;
+            if (sscanf(val, "%u", &u))
+                set_flags_value(parvalue, def, u);
+        }
+        else if (type == GWY_PARAM_REPORT_TYPE)
+            set_report_type_value(parvalue, def, atoi(val));
+        else if (param_type_is_data_id(type)) {
+            g_warning("Trying to restore a data id type parameter %s.", def->name);
+        }
+        else if (param_type_is_curve_no(type)) {
+            if (unescape_string(val, &ustr)) {
+                set_curve_string_value(parvalue, def, ustr);
+                GWY_FREE(ustr);
+            }
+            else
+                set_curve_int_value(parvalue, def, atoi(val));
+        }
+        else if (type == GWY_PARAM_DOUBLE)
+            set_double_value(parvalue, def, g_ascii_strtod(val, NULL));
+        else if (type == GWY_PARAM_STRING || type == GWY_PARAM_RESOURCE || type == GWY_PARAM_UNIT) {
+            if (unescape_string(val, &ustr)) {
+                if (type == GWY_PARAM_STRING)
+                    set_string_value(parvalue, def, ustr);
+                else if (type == GWY_PARAM_RESOURCE)
+                    set_resource_value(parvalue, def, ustr);
+                else if (type == GWY_PARAM_UNIT)
+                    set_unit_value(parvalue, def, ustr);
+            }
+            GWY_FREE(ustr);
+        }
+        else if (type == GWY_PARAM_COLOR) {
+            GwyRGBA rgba = parvalue->c;
+
+            if (altkey)
+                rgba.a = g_ascii_strtod(val, NULL);
+            else {
+                rgba.r = g_ascii_strtod(val, &end);
+                while (g_ascii_isspace(*end))
+                    end++;
+                val = end;
+                rgba.g = g_ascii_strtod(val, &end);
+                while (g_ascii_isspace(*end))
+                    end++;
+                val = end;
+                rgba.b = g_ascii_strtod(val, &end);
+            }
+            set_color_value(parvalue, def, rgba);
+        }
+        else {
+            g_assert_not_reached();
+        }
+
+        GWY_FREE(altkey);
+    }
+
+    g_free(s);
+}
+
 /**
  * SECTION:params
  * @title: GwyParams
