@@ -1,22 +1,18 @@
 /*
- *  $Id: settings.c 23986 2021-08-16 13:15:36Z yeti-dn $
+ *  $Id: settings.c 24546 2021-11-26 10:44:01Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -53,9 +49,9 @@ static gboolean gwy_gl_ok = FALSE;
  *
  * Gets the Gwyddion settings.
  *
- * The settings are a #GwyContainer automatically loaded at program startup
- * and saved ad its exit.  For storing persistent module data you should
- * use <literal>"/module/YOUR_MODULE_NAME/"</literal> prefix.
+ * The program settings are a #GwyContainer automatically loaded at program startup and saved ad its exit.  For
+ * storing persistent module data manually you should use <literal>"/module/YOUR_MODULE_NAME/"</literal> prefix.
+ * However, in common cases you should use #GwyParamDef and #GwyParams which can handle the dirty work themselves.
  *
  * Returns: The settings as a #GwyContainer.
  **/
@@ -91,8 +87,7 @@ gwy_app_settings_free(void)
  *
  * Saves the settings.
  *
- * Use gwy_app_settings_get_settings_filename() to obtain a suitable default
- * filename.
+ * Use gwy_app_settings_get_settings_filename() to obtain a suitable default filename.
  *
  * Returns: Whether it succeeded.
  **/
@@ -119,15 +114,12 @@ gwy_app_settings_save(const gchar *filename,
     g_return_val_if_fail(GWY_IS_CONTAINER(settings), FALSE);
     fh = gwy_fopen(filename, "w");
     if (!fh) {
-        g_set_error(error,
-                    GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_FILE,
-                    _("Cannot open file for writing: %s."),
-                    g_strerror(errno));
+        g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_FILE,
+                    _("Cannot open file for writing: %s."), g_strerror(errno));
         return FALSE;
     }
     if (fputs(magic_header, fh) == EOF) {
-        g_set_error(error,
-                    GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_FILE,
+        g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_FILE,
                     _("Cannot write to file: %s."), g_strerror(errno));
         fclose(fh);
         return FALSE;
@@ -137,8 +129,7 @@ gwy_app_settings_save(const gchar *filename,
     pa = gwy_container_serialize_to_text(settings);
     for (i = 0; i < pa->len; i++) {
         if (fputs((gchar*)pa->pdata[i], fh) == EOF) {
-            g_set_error(error,
-                        GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_FILE,
+            g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_FILE,
                         _("Cannot write to file: %s."), g_strerror(errno));
             while (i < pa->len)
                 g_free(pa->pdata[i]);
@@ -161,9 +152,8 @@ gwy_app_settings_save(const gchar *filename,
  *
  * Loads settings file.
  *
- * Returns: Whether it succeeded.  In either case you can call
- *          gwy_app_settings_get() then to obtain either the loaded settings
- *          or the old ones (if failed), or an empty #GwyContainer.
+ * Returns: Whether it succeeded.  In either case you can call gwy_app_settings_get() then to obtain either the loaded
+ *          settings or the old ones (if failed), or an empty #GwyContainer.
  **/
 gboolean
 gwy_app_settings_load(const gchar *filename,
@@ -176,16 +166,14 @@ gwy_app_settings_load(const gchar *filename,
 
     gwy_debug("Loading settings from `%s'", filename);
     if (!g_file_get_contents(filename, &buffer, &size, &err)) {
-        g_set_error(error,
-                    GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_FILE,
+        g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_FILE,
                     _("Cannot read file contents: %s"), err->message);
         g_clear_error(&err);
         return FALSE;
     }
 
     if (!size || !buffer) {
-        g_set_error(error,
-                    GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_EMPTY,
+        g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_EMPTY,
                     _("File is empty."));
         g_free(buffer);
         return FALSE;
@@ -194,19 +182,16 @@ gwy_app_settings_load(const gchar *filename,
     gwy_strkill(buffer, "\r");
 #endif
     if (!g_str_has_prefix(buffer, magic_header)) {
-        g_set_error(error,
-                    GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_CORRUPT,
+        g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_CORRUPT,
                     _("File is corrupted, magic header does not match."));
         g_free(buffer);
         return FALSE;
     }
-    new_settings = gwy_container_deserialize_from_text(buffer
-                                                       + strlen(magic_header));
+    new_settings = gwy_container_deserialize_from_text(buffer + strlen(magic_header));
     g_free(buffer);
     if (!GWY_IS_CONTAINER(new_settings)) {
         GWY_OBJECT_UNREF(new_settings);
-        g_set_error(error,
-                    GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_CORRUPT,
+        g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_CORRUPT,
                     _("File is corrupted, deserialization failed."));
         return FALSE;
     }
@@ -217,8 +202,7 @@ gwy_app_settings_load(const gchar *filename,
 
     //FIXME: deleting "/module/logistic/thetas" stored in wrong
     //FIXME: format during 2.46 development cycle
-    gwy_container_remove_by_prefix(gwy_settings,
-                                   "/module/logistic/thetas");
+    gwy_container_remove_by_prefix(gwy_settings, "/module/logistic/thetas");
 
     return TRUE;
 }
@@ -258,11 +242,10 @@ create_config_dir_real(const gchar *cfgdir, GError **error)
     if (!ok) {
         gwy_debug("Trying to create user config directory %s", cfgdir);
         ok = !g_mkdir(cfgdir, 0700);
-        if (!ok)
-            g_set_error(error,
-                        GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_CFGDIR,
-                        _("Cannot create user config directory %s: %s"),
-                        cfgdir, g_strerror(errno));
+        if (!ok) {
+            g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_CFGDIR,
+                        _("Cannot create user config directory %s: %s"), cfgdir, g_strerror(errno));
+        }
     }
 
     if (ok) {
@@ -271,11 +254,8 @@ create_config_dir_real(const gchar *cfgdir, GError **error)
             gwy_debug("Trying to create user ui directory %s", ui_dir);
             ok = !g_mkdir(ui_dir, 0700);
             if (!ok) {
-                g_set_error(error,
-                            GWY_APP_SETTINGS_ERROR,
-                            GWY_APP_SETTINGS_ERROR_CFGDIR,
-                            _("Cannot create user ui directory %s: %s"),
-                            ui_dir, g_strerror(errno));
+                g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_CFGDIR,
+                            _("Cannot create user ui directory %s: %s"), ui_dir, g_strerror(errno));
             }
         }
         g_free(ui_dir);
@@ -288,11 +268,8 @@ create_config_dir_real(const gchar *cfgdir, GError **error)
             gwy_debug("Trying to create user module directory %s", moddirs[i]);
             ok = !g_mkdir(moddirs[i], 0700);
             if (!ok) {
-                g_set_error(error,
-                            GWY_APP_SETTINGS_ERROR,
-                            GWY_APP_SETTINGS_ERROR_CFGDIR,
-                            _("Cannot create user module directory %s: %s"),
-                            moddirs[i], g_strerror(errno));
+                g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_CFGDIR,
+                            _("Cannot create user module directory %s: %s"), moddirs[i], g_strerror(errno));
                 break;
             }
         }
@@ -336,37 +313,25 @@ gwy_app_settings_set_defaults(GwyContainer *settings)
         "Warmish-White";
 
     if (!gwy_container_contains_by_name(settings, "/mask/alpha")) {
-        gwy_container_set_double_by_name(settings, "/mask/red",
-                                         default_mask_color.r);
-        gwy_container_set_double_by_name(settings, "/mask/green",
-                                         default_mask_color.g);
-        gwy_container_set_double_by_name(settings, "/mask/blue",
-                                         default_mask_color.b);
-        gwy_container_set_double_by_name(settings, "/mask/alpha",
-                                         default_mask_color.a);
+        gwy_container_set_double_by_name(settings, "/mask/red", default_mask_color.r);
+        gwy_container_set_double_by_name(settings, "/mask/green", default_mask_color.g);
+        gwy_container_set_double_by_name(settings, "/mask/blue", default_mask_color.b);
+        gwy_container_set_double_by_name(settings, "/mask/alpha", default_mask_color.a);
     }
     if (!gwy_container_contains_by_name(settings, "/app/gradients/preferred"))
-        gwy_container_set_string_by_name(settings, "/app/gradients/preferred",
-                                         g_strdup(default_preferred_gradients));
-    if (!gwy_container_contains_by_name(settings, "/app/glmaterials/preferred"))
+        gwy_container_set_string_by_name(settings, "/app/gradients/preferred", g_strdup(default_preferred_gradients));
+    if (!gwy_container_contains_by_name(settings, "/app/glmaterials/preferred")) {
         gwy_container_set_string_by_name(settings, "/app/glmaterials/preferred",
                                          g_strdup(default_preferred_gl_materials));
-    if (!gwy_container_contains_by_name(settings,
-                                        "/app/toolbox/visible/graph"))
-        gwy_container_set_boolean_by_name(settings,
-                                          "/app/toolbox/visible/graph", TRUE);
-    if (!gwy_container_contains_by_name(settings,
-                                        "/app/toolbox/visible/proc"))
-        gwy_container_set_boolean_by_name(settings,
-                                          "/app/toolbox/visible/proc", TRUE);
-    if (!gwy_container_contains_by_name(settings,
-                                        "/app/toolbox/visible/tool"))
-        gwy_container_set_boolean_by_name(settings,
-                                          "/app/toolbox/visible/tool", TRUE);
-    if (!gwy_container_contains_by_name(settings,
-                                        "/app/toolbox/visible/zoom"))
-        gwy_container_set_boolean_by_name(settings,
-                                          "/app/toolbox/visible/zoom", TRUE);
+    }
+    if (!gwy_container_contains_by_name(settings, "/app/toolbox/visible/graph"))
+        gwy_container_set_boolean_by_name(settings, "/app/toolbox/visible/graph", TRUE);
+    if (!gwy_container_contains_by_name(settings, "/app/toolbox/visible/proc"))
+        gwy_container_set_boolean_by_name(settings, "/app/toolbox/visible/proc", TRUE);
+    if (!gwy_container_contains_by_name(settings, "/app/toolbox/visible/tool"))
+        gwy_container_set_boolean_by_name(settings, "/app/toolbox/visible/tool", TRUE);
+    if (!gwy_container_contains_by_name(settings, "/app/toolbox/visible/zoom"))
+        gwy_container_set_boolean_by_name(settings, "/app/toolbox/visible/zoom", TRUE);
 }
 
 static void
@@ -396,30 +361,24 @@ gwy_app_settings_gather(GwyContainer *settings)
 
     /* Preferred resources */
     preferred = g_ptr_array_new();
-    gwy_inventory_foreach(gwy_gradients(),
-                          &add_preferred_resource_name, preferred);
+    gwy_inventory_foreach(gwy_gradients(), &add_preferred_resource_name, preferred);
     g_ptr_array_add(preferred, NULL);
     gwy_container_set_string_by_name(settings, "/app/gradients/preferred",
-                                     g_strjoinv("\n",
-                                                (gchar**)preferred->pdata));
+                                     g_strjoinv("\n", (gchar**)preferred->pdata));
     g_ptr_array_set_size(preferred, 0);
-    gwy_inventory_foreach(gwy_gl_materials(),
-                          &add_preferred_resource_name, preferred);
+    gwy_inventory_foreach(gwy_gl_materials(), &add_preferred_resource_name, preferred);
     g_ptr_array_add(preferred, NULL);
     gwy_container_set_string_by_name(settings, "/app/glmaterials/preferred",
-                                     g_strjoinv("\n",
-                                                (gchar**)preferred->pdata));
+                                     g_strjoinv("\n", (gchar**)preferred->pdata));
     g_ptr_array_free(preferred, TRUE);
 
     /* Default resources */
     name = gwy_inventory_get_default_item_name(gwy_gradients());
     if (name)
-        gwy_container_set_string_by_name(settings, "/app/gradients/default",
-                                         g_strdup(name));
+        gwy_container_set_string_by_name(settings, "/app/gradients/default", g_strdup(name));
     name = gwy_inventory_get_default_item_name(gwy_gl_materials());
     if (name)
-        gwy_container_set_string_by_name(settings, "/app/glmaterials/default",
-                                         g_strdup(name));
+        gwy_container_set_string_by_name(settings, "/app/glmaterials/default", g_strdup(name));
 }
 
 /**
@@ -439,8 +398,7 @@ gwy_app_settings_apply(GwyContainer *settings)
     gboolean disabled;
 
     /* Preferred resources */
-    if (gwy_container_gis_string_by_name(settings, "/app/gradients/preferred",
-                                         &s)) {
+    if (gwy_container_gis_string_by_name(settings, "/app/gradients/preferred", &s)) {
         inventory = gwy_gradients();
         preferred = g_strsplit(s, "\n", 0);
         for (p = preferred; *p; p++) {
@@ -449,8 +407,7 @@ gwy_app_settings_apply(GwyContainer *settings)
         }
         g_strfreev(preferred);
     }
-    if (gwy_container_gis_string_by_name(settings, "/app/glmaterials/preferred",
-                                         &s)) {
+    if (gwy_container_gis_string_by_name(settings, "/app/glmaterials/preferred", &s)) {
         inventory = gwy_gl_materials();
         preferred = g_strsplit(s, "\n", 0);
         for (p = preferred; *p; p++) {
@@ -461,23 +418,17 @@ gwy_app_settings_apply(GwyContainer *settings)
     }
 
     /* Default resources */
-    if (gwy_container_gis_string_by_name(settings, "/app/gradients/default",
-                                         &s))
+    if (gwy_container_gis_string_by_name(settings, "/app/gradients/default", &s))
         gwy_inventory_set_default_item_name(gwy_gradients(), s);
-    if (gwy_container_gis_string_by_name(settings, "/app/glmaterials/default",
-                                         &s))
+    if (gwy_container_gis_string_by_name(settings, "/app/glmaterials/default", &s))
         gwy_inventory_set_default_item_name(gwy_gl_materials(), s);
 
     /* Globally disabled 3D view axes */
-    if (gwy_container_gis_boolean_by_name(settings, "/app/3d/axes/disable",
-                                          &disabled)
-        && disabled)
+    if (gwy_container_gis_boolean_by_name(settings, "/app/3d/axes/disable", &disabled) && disabled)
         gwy_3d_view_class_disable_axis_drawing(disabled);
 
     /* Globally disabled logging */
-    if (gwy_container_gis_boolean_by_name(settings, "/app/log/disable",
-                                          &disabled)
-        && disabled)
+    if (gwy_container_gis_boolean_by_name(settings, "/app/log/disable", &disabled) && disabled)
         gwy_log_set_enabled(FALSE);
 }
 
@@ -486,9 +437,8 @@ gwy_app_settings_apply(GwyContainer *settings)
  *
  * Returns a list of directories to search modules in.
  *
- * Returns: The list of module directories as a newly allocated array of
- *          newly allocated strings, to be freed with g_strfreev() when
- *          not longer needed.
+ * Returns: The list of module directories as a newly allocated array of newly allocated strings, to be freed with
+ *          g_strfreev() when not longer needed.
  **/
 gchar**
 gwy_app_settings_get_module_dirs(void)
@@ -511,10 +461,8 @@ gwy_app_settings_get_module_dirs(void)
 
     q = gwy_get_user_dir();
     for (i = 0; i < n; i++)
-        module_dirs[n+1 + i] = g_build_filename(q, "modules", module_types[i],
-                                                NULL);
+        module_dirs[n+1 + i] = g_build_filename(q, "modules", module_types[i], NULL);
     module_dirs[2*n + 1] = g_build_filename(q, "modules", NULL);
-
     module_dirs[2*n + 2] = NULL;
 
     return module_dirs;
@@ -566,16 +514,13 @@ gwy_app_settings_get_recent_file_list_filename(void)
 
 /**
  * gwy_app_gl_init:
- * @argc: Address of the argc parameter of main(). Passed to
- *        gtk_gl_init_check().
- * @argv: Address of the argv parameter of main(). Passed to
- *        gtk_gl_init_check().
+ * @argc: Address of the argc parameter of main(). Passed to gtk_gl_init_check().
+ * @argv: Address of the argv parameter of main(). Passed to gtk_gl_init_check().
  *
  * Checks for working OpenGL and initializes it.
  *
- * When OpenGL support is not compiled in, this function does not do anything.
- * When OpenGL is supported, it calls gtk_gl_init_check() and
- * gwy_widgets_gl_init() (if the former succeeeds).
+ * When OpenGL support is not compiled in, this function does not do anything. When OpenGL is supported, it calls
+ * gtk_gl_init_check() and gwy_widgets_gl_init() (if the former succeeeds).
  *
  * Returns: %TRUE if OpenGL initialization succeeeded.
  **/
@@ -595,9 +540,8 @@ gwy_app_gl_init(G_GNUC_UNUSED int *argc,
  *
  * Returns OpenGL availability.
  *
- * Returns: The return value is the same as the return value of
- *          gwy_app_gl_init() which needs to be called prior to this function
- *          (until then, the return value is always %FALSE).
+ * Returns: The return value is the same as the return value of gwy_app_gl_init() which needs to be called prior to
+ *          this function (until then, the return value is always %FALSE).
  **/
 gboolean
 gwy_app_gl_is_ok(void)
@@ -620,8 +564,7 @@ gwy_app_settings_error_quark(void)
     static GQuark error_domain = 0;
 
     if (!error_domain)
-        error_domain
-            = g_quark_from_static_string("gwy-app-settings-error-quark");
+        error_domain = g_quark_from_static_string("gwy-app-settings-error-quark");
 
     return error_domain;
 }
@@ -632,37 +575,32 @@ gwy_app_settings_error_quark(void)
  * SECTION:settings
  * @title: settings
  * @short_description: Application and module settings
+ * @see_also: #GwyParamDef, #GwyParams -- a higher level module settings interface
  *
- * All application and module settings are stored in a one big #GwyContainer
- * which can be obtained by gwy_app_settings_get(). Then you can use
- * #GwyContainer functions to get and save settings.
+ * All application and module settings are stored in a one big #GwyContainer which can be obtained by
+ * gwy_app_settings_get(). Then you can use #GwyContainer functions to get and save settings.
  *
- * The rest of the setting manipulating functions is normally useful only in
- * main application.
+ * The rest of the setting manipulating functions is normally useful only in the main application.
  **/
 
 /**
  * GWY_APP_SETTINGS_ERROR:
  *
- * Error domain for application settings operations. Errors in this domain will
- * be from the #GwyAppSettingsError enumeration. See #GError for information
- * on error domains.
+ * Error domain for application settings operations. Errors in this domain will be from the #GwyAppSettingsError
+ * enumeration. See #GError for information on error domains.
  **/
 
 /**
  * GwyAppSettingsError:
  * @GWY_APP_SETTINGS_ERROR_FILE: Settings file is not readable or writable.
  * @GWY_APP_SETTINGS_ERROR_CORRUPT: Settings file contents is corrupted.
- * @GWY_APP_SETTINGS_ERROR_CFGDIR: User configuration directory is not
- *                                 readable or writable or it does not exist
- *                                 and its creation failed.
- * @GWY_APP_SETTINGS_ERROR_EMPTY: Settings file is empty (Since 2.45).  This
- *                                was previously reported as
- *                                %GWY_APP_SETTINGS_ERROR_CORRUPT but empty
- *                                settings files can be overwritten without
+ * @GWY_APP_SETTINGS_ERROR_CFGDIR: User configuration directory is not readable or writable or it does not exist and
+ *                                 its creation failed.
+ * @GWY_APP_SETTINGS_ERROR_EMPTY: Settings file is empty (Since 2.45).  This was previously reported as
+ *                                %GWY_APP_SETTINGS_ERROR_CORRUPT but empty settings files can be overwritten without
  *                                losing anything.
  *
  * Error codes returned by application settings functions.
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
