@@ -1,22 +1,18 @@
 /*
- *  $Id: validate.c 21151 2018-06-15 08:18:55Z yeti-dn $
- *  Copyright (C) 2007-2016 David Necas (Yeti).
+ *  $Id: validate.c 23968 2021-08-12 15:39:50Z yeti-dn $
+ *  Copyright (C) 2007-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -48,6 +44,7 @@ typedef struct {
     GArray *spectra;
     GArray *volumes;
     GArray *xyzs;
+    GArray *curve_maps;
 } GwyDataValidationInfo;
 
 static GwyDataValidationFailure*
@@ -110,9 +107,7 @@ check_utf8(const gchar *string,
         return TRUE;
 
     *errors = g_slist_prepend(*errors,
-                              FAIL(GWY_DATA_ERROR_NON_UTF8_STRING, key,
-                                   _("byte at position %d"),
-                                   end - string));
+                              FAIL(GWY_DATA_ERROR_NON_UTF8_STRING, key, _("byte at position %d"), end - string));
     return FALSE;
 }
 
@@ -131,9 +126,7 @@ check_ascii_key(const guchar *strkey,
 
 fail:
     *errors = g_slist_prepend(*errors,
-                              FAIL(GWY_DATA_ERROR_KEY_CHARACTERS, key,
-                                   _("byte at position %d"),
-                                   end - strkey));
+                              FAIL(GWY_DATA_ERROR_KEY_CHARACTERS, key, _("byte at position %d"), end - strkey));
     return FALSE;
 }
 
@@ -157,10 +150,8 @@ check_type(GValue *gvalue,
     /* Expecting object but found a simple type */
     if (G_UNLIKELY(!G_VALUE_HOLDS(gvalue, G_TYPE_OBJECT))) {
         *errors = g_slist_prepend(*errors,
-                                  FAIL(GWY_DATA_ERROR_ITEM_TYPE, key,
-                                       _("%s instead of %s"),
-                                       g_type_name(G_VALUE_TYPE(gvalue)),
-                                       g_type_name(type)));
+                                  FAIL(GWY_DATA_ERROR_ITEM_TYPE, key, _("%s instead of %s"),
+                                       g_type_name(G_VALUE_TYPE(gvalue)), g_type_name(type)));
         return FALSE;
     }
 
@@ -170,10 +161,8 @@ check_type(GValue *gvalue,
         return TRUE;
 
     *errors = g_slist_prepend(*errors,
-                              FAIL(GWY_DATA_ERROR_ITEM_TYPE, key,
-                                   _("%s instead of %s"),
-                                   g_type_name(G_VALUE_TYPE(gvalue)),
-                                   g_type_name(type)));
+                              FAIL(GWY_DATA_ERROR_ITEM_TYPE, key, _("%s instead of %s"),
+                                   g_type_name(G_VALUE_TYPE(gvalue)), g_type_name(type)));
     return FALSE;
 }
 
@@ -195,15 +184,12 @@ validate_item_pass1(gpointer hash_key,
     strkey = g_quark_to_string(key);
     check_ascii_key(strkey, key, errors);
     if (strkey[0] != GWY_CONTAINER_PATHSEP)
-        *errors = g_slist_prepend(*errors,
-                                  FAIL(GWY_DATA_ERROR_KEY_FORMAT, key, NULL));
+        *errors = g_slist_prepend(*errors, FAIL(GWY_DATA_ERROR_KEY_FORMAT, key, NULL));
 
     id = _gwy_app_analyse_data_key(strkey, &type, &len);
     if (type == KEY_IS_NONE) {
         if (info->flags & GWY_DATA_VALIDATE_UNKNOWN)
-            *errors = g_slist_prepend(*errors,
-                                      FAIL(GWY_DATA_ERROR_KEY_UNKNOWN,
-                                           key, NULL));
+            *errors = g_slist_prepend(*errors, FAIL(GWY_DATA_ERROR_KEY_UNKNOWN, key, NULL));
         return;
     }
 
@@ -217,11 +203,9 @@ validate_item_pass1(gpointer hash_key,
         return;
     }
 
-    /* Items that must have data id.  While ids above 2^20 are technically
-     * valid, they indicate a bug or malice. */
+    /* Items that must have data id.  While ids above 2^20 are technically valid, they indicate a bug or malice. */
     if (id < 0 || id >= (1 << 20))
-        *errors = g_slist_prepend(*errors,
-                                  FAIL(GWY_DATA_ERROR_KEY_ID, key, NULL));
+        *errors = g_slist_prepend(*errors, FAIL(GWY_DATA_ERROR_KEY_ID, key, NULL));
 
     /* Types */
     switch (type) {
@@ -235,6 +219,7 @@ validate_item_pass1(gpointer hash_key,
         case KEY_IS_CALDATA:
         case KEY_IS_BRICK_PREVIEW:
         case KEY_IS_SURFACE_PREVIEW:
+        case KEY_IS_LAWN_PREVIEW:
         check_type(gvalue, GWY_TYPE_DATA_FIELD, key, errors);
         break;
 
@@ -258,15 +243,22 @@ validate_item_pass1(gpointer hash_key,
             g_array_append_val(info->xyzs, id);
         break;
 
+        case KEY_IS_LAWN:
+        if (check_type(gvalue, GWY_TYPE_LAWN, key, errors))
+            g_array_append_val(info->curve_maps, id);
+        break;
+
         case KEY_IS_CHANNEL_META:
         case KEY_IS_BRICK_META:
         case KEY_IS_SURFACE_META:
+        case KEY_IS_LAWN_META:
         check_type(gvalue, GWY_TYPE_CONTAINER, key, errors);
         break;
 
         case KEY_IS_CHANNEL_LOG:
         case KEY_IS_BRICK_LOG:
         case KEY_IS_SURFACE_LOG:
+        case KEY_IS_LAWN_LOG:
         check_type(gvalue, GWY_TYPE_STRING_LIST, key, errors);
         break;
 
@@ -278,6 +270,8 @@ validate_item_pass1(gpointer hash_key,
         case KEY_IS_BRICK_PREVIEW_PALETTE:
         case KEY_IS_SURFACE_TITLE:
         case KEY_IS_SURFACE_PREVIEW_PALETTE:
+        case KEY_IS_LAWN_TITLE:
+        case KEY_IS_LAWN_PREVIEW_PALETTE:
         check_type(gvalue, G_TYPE_STRING, key, errors);
         break;
 
@@ -298,17 +292,20 @@ validate_item_pass1(gpointer hash_key,
         case KEY_IS_DATA_VIEW_SCALE:
         case KEY_IS_BRICK_VIEW_SCALE:
         case KEY_IS_SURFACE_VIEW_SCALE:
+        case KEY_IS_LAWN_VIEW_SCALE:
         case KEY_IS_GRAPH_VIEW_SCALE:
         case KEY_IS_3D_VIEW_SCALE:
         check_type(gvalue, G_TYPE_DOUBLE, key, errors);
         break;
 
         case KEY_IS_REAL_SQUARE:
+        case KEY_IS_LAWN_REAL_SQUARE:
         case KEY_IS_DATA_VISIBLE:
         case KEY_IS_GRAPH_VISIBLE:
         case KEY_IS_SPECTRA_VISIBLE:
         case KEY_IS_BRICK_VISIBLE:
         case KEY_IS_SURFACE_VISIBLE:
+        case KEY_IS_LAWN_VISIBLE:
         check_type(gvalue, G_TYPE_BOOLEAN, key, errors);
         break;
 
@@ -321,8 +318,7 @@ validate_item_pass1(gpointer hash_key,
         break;
 
         default:
-        g_warning("Key type %u of %s not handled in validate_item()",
-                  type, strkey);
+        g_warning("Key type %u of %s not handled in validate_item()", type, strkey);
         break;
     }
 }
@@ -384,9 +380,7 @@ validate_item_pass2(gpointer hash_key,
         case KEY_IS_DATA_VIEW_SCALE:
         if (!in_array(info->channels, id)) {
             *errors = g_slist_prepend(*errors,
-                                      FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA,
-                                           key,
-                                           _("no channel %d exists for %s"),
+                                      FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA, key, _("no channel %d exists for %s"),
                                            id, strkey));
         }
         break;
@@ -396,9 +390,7 @@ validate_item_pass2(gpointer hash_key,
         case KEY_IS_GRAPH_VIEW_SIZE:
         if (!in_array(info->graphs, id)) {
             *errors = g_slist_prepend(*errors,
-                                      FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA,
-                                           key,
-                                           _("no graph %d exists for %s"),
+                                      FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA, key, _("no graph %d exists for %s"),
                                            id, strkey));
         }
         break;
@@ -406,9 +398,7 @@ validate_item_pass2(gpointer hash_key,
         case KEY_IS_SPECTRA_VISIBLE:
         if (!in_array(info->spectra, id)) {
             *errors = g_slist_prepend(*errors,
-                                      FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA,
-                                           key,
-                                           _("no spectra %d exists for %s"),
+                                      FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA, key, _("no spectra %d exists for %s"),
                                            id, strkey));
         }
         break;
@@ -422,9 +412,7 @@ validate_item_pass2(gpointer hash_key,
         case KEY_IS_BRICK_VIEW_SCALE:
         if (!in_array(info->volumes, id)) {
             *errors = g_slist_prepend(*errors,
-                                      FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA,
-                                           key,
-                                           _("no brick %d exists for %s"),
+                                      FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA, key, _("no brick %d exists for %s"),
                                            id, strkey));
         }
         break;
@@ -438,9 +426,21 @@ validate_item_pass2(gpointer hash_key,
         case KEY_IS_SURFACE_VIEW_SCALE:
         if (!in_array(info->xyzs, id)) {
             *errors = g_slist_prepend(*errors,
-                                      FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA,
-                                           key,
-                                           _("no surface %d exists for %s"),
+                                      FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA, key, _("no surface %d exists for %s"),
+                                           id, strkey));
+        }
+        break;
+
+        case KEY_IS_LAWN_VISIBLE:
+        case KEY_IS_LAWN_TITLE:
+        case KEY_IS_LAWN_PREVIEW:
+        case KEY_IS_LAWN_PREVIEW_PALETTE:
+        case KEY_IS_LAWN_META:
+        case KEY_IS_LAWN_LOG:
+        case KEY_IS_LAWN_VIEW_SCALE:
+        if (!in_array(info->curve_maps, id)) {
+            *errors = g_slist_prepend(*errors,
+                                      FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA, key, _("no lawn %d exists for %s"),
                                            id, strkey));
         }
         break;
@@ -494,9 +494,7 @@ check_ref_count(GObject *object,
 
     s = format_stack(object, stack);
     *errors = g_slist_prepend(*errors,
-                              FAIL(GWY_DATA_ERROR_REF_COUNT, key,
-                                   _("ref_count is %d for %s"),
-                                   object->ref_count, s));
+                              FAIL(GWY_DATA_ERROR_REF_COUNT, key, _("ref_count is %d for %s"), object->ref_count, s));
     g_free(s);
 
     return FALSE;
@@ -525,10 +523,8 @@ validate_item_pass3(gpointer hash_key,
     check_ref_count(object, key, info->stack, errors);
 
     PUSH(info->stack, typename);
-    if (GWY_IS_CONTAINER(object)) {
-        gwy_container_foreach((GwyContainer*)object, NULL,
-                              &validate_item_pass3, info);
-    }
+    if (GWY_IS_CONTAINER(object))
+        gwy_container_foreach((GwyContainer*)object, NULL, &validate_item_pass3, info);
     else if (GWY_IS_DATA_FIELD(object)) {
         GwyDataField *data_field = (GwyDataField*)object;
 
@@ -570,6 +566,11 @@ validate_item_pass3(gpointer hash_key,
         }
         POP(info->stack);
     }
+    else if (GWY_IS_LAWN(object)) {
+        GwyLawn *lawn = (GwyLawn*)object;
+
+        CRFC(lawn->si_unit_xy, "si-unit-xy");
+    }
     POP(info->stack);
 }
 
@@ -593,27 +594,20 @@ gwy_data_correct(GwyContainer *data,
 
 /**
  * gwy_data_validate:
- * @data: Data container.  It should not be managed by the data browser (yet)
- *        if flags contain %GWY_DATA_VALIDATE_CORRECT (because things can
- *        in principle break during the correction) or
- *        %GWY_DATA_VALIDATE_REF_COUNT (because the application took some
- *        references).
- * @flags: Validation flags.  Some influence what is checked, some determine
- *         what to do when problems are found.
+ * @data: Data container.  It should not be managed by the data browser (yet) if flags contain
+ *        %GWY_DATA_VALIDATE_CORRECT (because things can in principle break during the correction) or
+ *        %GWY_DATA_VALIDATE_REF_COUNT (because the application took some references).
+ * @flags: Validation flags.  Some influence what is checked, some determine what to do when problems are found.
  *
  * Checks the contents of a data file.
  *
- * If %GWY_DATA_VALIDATE_CORRECT is given in @flags, correctable problems are
- * corrected.  At present correctable problems are those that can be fixed
- * by removal of the offending data.
+ * If %GWY_DATA_VALIDATE_CORRECT is given in @flags, correctable problems are corrected.  At present correctable
+ * problems are those that can be fixed by removal of the offending data.
  *
- * Returns: List of errors found, free
- *          with gwy_data_validation_failure_list_free().
- *          The list is independent on whether %GWY_DATA_VALIDATE_CORRECT is
- *          given in flags, even though the offending items may be no longer
- *          exist in the container after correction.  If
- *          %GWY_DATA_VALIDATE_NO_REPORT is present in flags, %NULL is always
- *          returned.
+ * Returns: List of errors found, free with gwy_data_validation_failure_list_free(). The list is independent on
+ *          whether %GWY_DATA_VALIDATE_CORRECT is given in flags, even though the offending items may be no longer
+ *          exist in the container after correction.  If %GWY_DATA_VALIDATE_NO_REPORT is present in flags, %NULL is
+ *          always returned.
  *
  * Since: 2.9
  **/
@@ -624,8 +618,7 @@ gwy_data_validate(GwyContainer *data,
     GwyDataValidationInfo info;
     GSList *errors;
 
-    if ((flags & GWY_DATA_VALIDATE_NO_REPORT)
-        && !(flags & GWY_DATA_VALIDATE_CORRECT)) {
+    if ((flags & GWY_DATA_VALIDATE_NO_REPORT) && !(flags & GWY_DATA_VALIDATE_CORRECT)) {
         g_warning("Neither report no correction asked for, "
                   "validation is useless.");
         return NULL;
@@ -638,6 +631,7 @@ gwy_data_validate(GwyContainer *data,
     info.spectra = g_array_new(FALSE, FALSE, sizeof(gint));
     info.volumes = g_array_new(FALSE, FALSE, sizeof(gint));
     info.xyzs = g_array_new(FALSE, FALSE, sizeof(gint));
+    info.curve_maps = g_array_new(FALSE, FALSE, sizeof(gint));
 
     gwy_container_foreach(data, NULL, &validate_item_pass1, &info);
     gwy_container_foreach(data, NULL, &validate_item_pass2, &info);
@@ -655,6 +649,7 @@ gwy_data_validate(GwyContainer *data,
     g_array_free(info.spectra, TRUE);
     g_array_free(info.volumes, TRUE);
     g_array_free(info.xyzs, TRUE);
+    g_array_free(info.curve_maps, TRUE);
 
     if (flags & GWY_DATA_VALIDATE_NO_REPORT) {
         gwy_data_validation_failure_list_free(errors);
@@ -670,8 +665,7 @@ gwy_data_validate(GwyContainer *data,
  *
  * Describes a data validation error type.
  *
- * Returns: Error description as an untranslated string owned by the
- *          library.
+ * Returns: Error description as an untranslated string owned by the library.
  *
  * Since: 2.9
  **/
@@ -745,24 +739,20 @@ _gwy_app_enforce_graph_abscissae_order(GwyContainer *data)
  * @title: Validate
  * @short_description: Check data sanity and consistency
  *
- * A #GwyContainer can be used to represent all sorts of data.  However,
- * Gwyddion, the application, follows certain conventions in data organization.
- * Function gwy_data_validate() checks whether the data actually follows them.
- * This includes but is not limited to representability of keys in ASCII,
- * conformance to the key naming convention, types of objects and other items
- * corresponding to expectation, string values being valid UTF-8 and no stray
- * secondary data.
+ * A #GwyContainer can be used to represent all sorts of data.  However, Gwyddion, the application, follows certain
+ * conventions in data organization. Function gwy_data_validate() checks whether the data actually follows them. This
+ * includes but is not limited to representability of keys in ASCII, conformance to the key naming convention, types
+ * of objects and other items corresponding to expectation, string values being valid UTF-8 and no stray secondary
+ * data.
  **/
 
 /**
  * GwyDataValidationFailure:
  * @error: Error type.
  * @key: Key of the problematic data item.
- * @details: Error details, may be %NULL for some types of @error.  This is
- *           a dynamically allocated string owned by the caller, however, he
- *           normally frees the complete errors lists with
- *           gwy_data_validation_failure_list_free() which frees these fields
- *           too.
+ * @details: Error details, may be %NULL for some types of @error.  This is a dynamically allocated string owned by
+ *           the caller, however, he normally frees the complete errors lists with
+ *           gwy_data_validation_failure_list_free() which frees these fields too.
  *
  * Information about one data validate error.
  *
@@ -773,20 +763,15 @@ _gwy_app_enforce_graph_abscissae_order(GwyContainer *data)
 
 /**
  * GwyDataError:
- * @GWY_DATA_ERROR_KEY_FORMAT: Key format is invalid (e.g. does not start
- *                             with %GWY_CONTAINER_PATHSEP).
- * @GWY_DATA_ERROR_KEY_CHARACTERS: Key contains unprintable characters or
- *                                 characters not representable in ASCII.
- * @GWY_DATA_ERROR_KEY_UNKNOWN: Key does not correspond to any data item known
- *                              to this version of Gwyddion.
+ * @GWY_DATA_ERROR_KEY_FORMAT: Key format is invalid (e.g. does not start with %GWY_CONTAINER_PATHSEP).
+ * @GWY_DATA_ERROR_KEY_CHARACTERS: Key contains unprintable characters or characters not representable in ASCII.
+ * @GWY_DATA_ERROR_KEY_UNKNOWN: Key does not correspond to any data item known to this version of Gwyddion.
  * @GWY_DATA_ERROR_KEY_ID: Key corresponds to a data item with bogus id number.
- * @GWY_DATA_ERROR_ITEM_TYPE: Wrong item type (for instance an integer at key
- *                            <literal>"/0/data"</literal>).
+ * @GWY_DATA_ERROR_ITEM_TYPE: Wrong item type (for instance an integer at key <literal>"/0/data"</literal>).
  * @GWY_DATA_ERROR_NON_UTF8_STRING: String value is not valid UTF-8.
  * @GWY_DATA_ERROR_REF_COUNT: Reference count is higher than 1.
- * @GWY_DATA_ERROR_STRAY_SECONDARY_DATA: Secondary data item (e.g. mask,
- *                                       selection or visibility) without a
- *                                       corresponding valid primary data item.
+ * @GWY_DATA_ERROR_STRAY_SECONDARY_DATA: Secondary data item (e.g. mask, selection or visibility) without
+ *                                       a corresponding valid primary data item.
  *
  * Type of data validation errors.
  *
@@ -795,25 +780,21 @@ _gwy_app_enforce_graph_abscissae_order(GwyContainer *data)
 
 /**
  * GwyDataValidateFlags:
- * @GWY_DATA_VALIDATE_UNKNOWN: Report all unknown keys as
- *                             %GWY_DATA_ERROR_KEY_UNKNOWN errors.  Note while
- *                             a data item unknown to the current version of
- *                             Gwyddion can come from a newer version therefore
- *                             it can be in certain sense valid.
- * @GWY_DATA_VALIDATE_REF_COUNT: Report all object items with reference count
- *                               higher than 1 as %GWY_DATA_ERROR_REF_COUNT
- *                               errors.  Obviously this makes sense only with
- *                               `fresh' data containers.
+ * @GWY_DATA_VALIDATE_UNKNOWN: Report all unknown keys as %GWY_DATA_ERROR_KEY_UNKNOWN errors.  Note while a data item
+ *                             unknown to the current version of Gwyddion can come from a newer version therefore it
+ *                             can be in certain sense valid.
+ * @GWY_DATA_VALIDATE_REF_COUNT: Report all object items with reference count higher than 1 as
+ *                               %GWY_DATA_ERROR_REF_COUNT errors.  Obviously this makes sense only with `fresh' data
+ *                               containers.
  * @GWY_DATA_VALIDATE_ALL: All above flags combined.
  * @GWY_DATA_VALIDATE_CORRECT: Attempt to correct problems.
  * @GWY_DATA_VALIDATE_NO_REPORT: Do not report problems.
  *
  * Flags controlling gwy_data_validate() behaviour.
  *
- * Note passing @GWY_DATA_VALIDATE_NO_REPORT is allowed only if
- * @GWY_DATA_VALIDATE_CORRECT is present too.
+ * Note passing @GWY_DATA_VALIDATE_NO_REPORT is allowed only if @GWY_DATA_VALIDATE_CORRECT is present too.
  *
  * Since: 2.9
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
