@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyddion.c 24403 2021-10-22 12:56:03Z yeti-dn $
+ *  $Id: gwyddion.c 24555 2021-12-02 15:31:19Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -50,6 +50,7 @@ typedef enum {
     MODE_NORMAL,
     MODE_CHECK,
     MODE_IDENTIFY,
+    MODE_CONVERT_TO_GWY,
 } GwyAppMode;
 
 typedef struct {
@@ -60,6 +61,8 @@ typedef struct {
     gboolean log_to_console;
     GwyAppMode mode;
     GwyAppRemoteType remote;
+    gchar *disabled_modules;
+    gchar *convert_outfilename;
 } GwyAppOptions;
 
 static gboolean open_command_line_files         (gint n,
@@ -71,14 +74,16 @@ static gint     identify_command_line_files     (gint n,
                                                  gchar **args);
 static gint     check_command_line_files        (gint n,
                                                  gchar **args);
+static gint     convert_command_line_files      (gint n,
+                                                 gchar **args,
+                                                 const gchar *outname);
 static void     print_help_and_exit             (void);
 static void     print_version_and_exit          (void);
-static void     extend_disabled_modules_arg     (const gchar *arg,
-                                                 gchar **modules_to_block);
+static void     extend_disabled_modules_arg     (gchar **modules_to_block,
+                                                 const gchar *arg);
 static void     process_preinit_options         (int *argc,
                                                  char ***argv,
-                                                 GwyAppOptions *options,
-                                                 gchar **modules_to_block);
+                                                 GwyAppOptions *options);
 static void     debug_time                      (GTimer *timer,
                                                  const gchar *task);
 static void     setup_locale_from_win32_registry(void);
@@ -87,7 +92,8 @@ static void     warn_broken_settings_file       (GtkWidget *parent,
                                                  const gchar *reason);
 static void     check_broken_modules            (GtkWidget *parent);
 static void     gwy_app_init                    (int *argc,
-                                                 char ***argv);
+                                                 char ***argv,
+                                                 gboolean is_gui);
 static void     gwy_app_set_window_icon         (void);
 static void     gwy_app_check_version           (void);
 
@@ -95,6 +101,7 @@ static GwyAppOptions app_options = {
     FALSE, FALSE, FALSE,
     LOG_TO_FILE_DEFAULT, LOG_TO_CONSOLE_DEFAULT,
     MODE_NORMAL, GWY_APP_REMOTE_DEFAULT,
+    NULL, NULL,
 };
 
 # ifdef __MINGW64__
@@ -106,8 +113,7 @@ main(int argc, char *argv[])
 {
     GtkWidget *toolbox;
     gchar **module_dirs;
-    gchar *settings_file, *recent_file_file, *accel_file;
-    gchar *modules_to_block = NULL;
+    gchar *settings_file, *recent_file_file = NULL, *accel_file = NULL;
     gboolean has_settings, settings_ok = FALSE;
     gboolean opening_files = FALSE, show_tips = FALSE;
     GwyContainer *settings;
@@ -118,7 +124,7 @@ main(int argc, char *argv[])
     gwy_threads_set_enabled(TRUE);
     timer = g_timer_new();
 
-    process_preinit_options(&argc, &argv, &app_options, &modules_to_block);
+    process_preinit_options(&argc, &argv, &app_options);
     gwy_app_setup_logging((app_options.log_to_file ? GWY_APP_LOGGING_TO_FILE : 0)
                           | (app_options.log_to_console ? GWY_APP_LOGGING_TO_CONSOLE : 0));
     gwy_app_check_version();
@@ -147,10 +153,12 @@ main(int argc, char *argv[])
     gwy_app_settings_create_config_dir(NULL);
     debug_time(timer, "init");
     setup_locale_from_win32_registry();
-    gtk_init(&argc, &argv);
-    debug_time(timer, "gtk_init()");
-    gwy_remote_do(app_options.remote, argc - 1, argv + 1);
-    gwy_app_init(&argc, &argv);
+    if (app_options.mode == MODE_NORMAL) {
+        gtk_init(&argc, &argv);
+        debug_time(timer, "gtk_init()");
+        gwy_remote_do(app_options.remote, argc - 1, argv + 1);
+    }
+    gwy_app_init(&argc, &argv, app_options.mode == MODE_NORMAL);
     debug_time(timer, "gwy_app_init()");
 
     settings_file = gwy_app_settings_get_settings_filename();
@@ -160,19 +168,21 @@ main(int argc, char *argv[])
     gwy_app_splash_start(!app_options.no_splash && app_options.mode == MODE_NORMAL);
     debug_time(timer, "create splash");
 
-    accel_file = g_build_filename(gwy_get_user_dir(), "ui", "accel_map", NULL);
-    gtk_accel_map_load(accel_file);
-    debug_time(timer, "load accel map");
+    if (app_options.mode == MODE_NORMAL) {
+        accel_file = g_build_filename(gwy_get_user_dir(), "ui", "accel_map", NULL);
+        gtk_accel_map_load(accel_file);
+        debug_time(timer, "load accel map");
 
-    gwy_app_splash_set_message(_("Loading document history"));
-    recent_file_file = gwy_app_settings_get_recent_file_list_filename();
-    gwy_app_recent_file_list_load(recent_file_file);
-    debug_time(timer, "load document history");
+        gwy_app_splash_set_message(_("Loading document history"));
+        recent_file_file = gwy_app_settings_get_recent_file_list_filename();
+        gwy_app_recent_file_list_load(recent_file_file);
+        debug_time(timer, "load document history");
 
-    gwy_app_splash_set_message_prefix(_("Registering "));
-    gwy_app_splash_set_message(_("stock items"));
-    gwy_stock_register_stock_items();
-    debug_time(timer, "register stock items");
+        gwy_app_splash_set_message_prefix(_("Registering "));
+        gwy_app_splash_set_message(_("stock items"));
+        gwy_stock_register_stock_items();
+        debug_time(timer, "register stock items");
+    }
 
     gwy_app_splash_set_message(_("color gradients"));
     gwy_resource_class_load(g_type_class_peek(GWY_TYPE_GRADIENT));
@@ -199,10 +209,10 @@ main(int argc, char *argv[])
      *    function during registration, so it is its internal init)
      * Running gwyddion --disable-modules=pygwy,fitsfile can take module registration down to ~6 ms. */
     gwy_app_splash_set_message(_("Registering modules"));
-    if (app_options.mode == MODE_IDENTIFY)
-        extend_disabled_modules_arg("rawfile", &modules_to_block);
-    block_modules(modules_to_block);
-    GWY_FREE(modules_to_block);
+    if (app_options.mode != MODE_NORMAL)
+        extend_disabled_modules_arg(&app_options.disabled_modules, "rawfile");
+    block_modules(app_options.disabled_modules);
+    GWY_FREE(app_options.disabled_modules);
 
     module_dirs = gwy_app_settings_get_module_dirs();
     gwy_module_register_modules((const gchar**)module_dirs);
@@ -225,6 +235,11 @@ main(int argc, char *argv[])
         debug_time(timer, "check files");
         return !!nfailures;
     }
+    if (app_options.mode == MODE_CONVERT_TO_GWY) {
+        gint nfailures = convert_command_line_files(argc - 1, argv + 1, app_options.convert_outfilename);
+        debug_time(timer, "convert files");
+        return !!nfailures;
+    }
 
     /* Toolbox creation is one of the most time-consuming parts of startup.
      *
@@ -307,10 +322,7 @@ main(int argc, char *argv[])
 
 /* There are things we want to set up before GTK+ init. */
 static void
-process_preinit_options(int *argc,
-                        char ***argv,
-                        GwyAppOptions *options,
-                        gchar **modules_to_block)
+process_preinit_options(int *argc, char ***argv, GwyAppOptions *options)
 {
     gboolean ignore, handled;
     const gchar *arg;
@@ -334,14 +346,22 @@ process_preinit_options(int *argc,
             print_version_and_exit();
         else if (gwy_strequal(arg, "--no-splash"))
             options->no_splash = TRUE;
-        else if (gwy_strequal(arg, "--remote-existing"))
+        else if (gwy_strequal(arg, "--remote-existing")) {
+            options->mode = MODE_NORMAL;
             options->remote = GWY_APP_REMOTE_EXISTING;
-        else if (gwy_strequal(arg, "--remote-new"))
+        }
+        else if (gwy_strequal(arg, "--remote-new")) {
+            options->mode = MODE_NORMAL;
             options->remote = GWY_APP_REMOTE_NEW;
-        else if (gwy_strequal(arg, "--remote-query"))
+        }
+        else if (gwy_strequal(arg, "--remote-query")) {
+            options->mode = MODE_NORMAL;
             options->remote = GWY_APP_REMOTE_QUERY;
-        else if (gwy_strequal(arg, "--new-instance"))
+        }
+        else if (gwy_strequal(arg, "--new-instance")) {
+            options->mode = MODE_NORMAL;
             options->remote = GWY_APP_REMOTE_NONE;
+        }
         else if (gwy_strequal(arg, "--debug-objects")) {
             /* Silently ignore it. */
         }
@@ -361,8 +381,12 @@ process_preinit_options(int *argc,
             options->mode = MODE_CHECK;
         else if (gwy_strequal(arg, "--identify"))
             options->mode = MODE_IDENTIFY;
+        else if (g_str_has_prefix(arg, "--convert-to-gwy=")) {
+            options->mode = MODE_CONVERT_TO_GWY;
+            gwy_assign_string(&options->convert_outfilename, strchr(arg, '=') + 1);
+        }
         else if (g_str_has_prefix(arg, "--disable-modules="))
-            extend_disabled_modules_arg(arg + strlen("--disable-modules="), modules_to_block);
+            extend_disabled_modules_arg(&options->disabled_modules, strchr(arg, '=') + 1);
         else
             handled = FALSE;
 
@@ -376,41 +400,41 @@ process_preinit_options(int *argc,
 static void
 print_help_and_exit(void)
 {
-    puts(
-"Usage: gwyddion [OPTIONS...] FILES...\n"
-"An SPM data visualization and analysis tool, written with Gtk+.\n"
-        );
-    puts(
-"Gwyddion options:\n"
-" -h, --help                 Print this help and terminate.\n"
-" -v, --version              Print version info and terminate.\n"
-"     --no-splash            Don't show splash screen.\n"
-"     --remote-query         Check if a Gwyddion instance is already running.\n"
-"     --remote-new           Load FILES to a running instance or run a new one.\n"
-"     --remote-existing      Load FILES to a running instance or fail.\n"
-"     --new-instance         Run a new instance, ignoring any already running.\n"
-"     --identify             Identify and print the type of SPM data FILES.\n"
-"     --check                Check FILES, print problems and terminate.\n"
-"     --disable-gl           Disable OpenGL, including any availability checks.\n"
-"     --log-to-file          Write messages to file set in GWYDDION_LOGFILE.\n"
-"     --no-log-to-file       Do not write messages to any file.\n"
-"     --log-to-console       Print messages to console\n"
-"     --no-log-to-console    Do not print messages to console.\n"
-"     --disable-modules=MODNAME1,MODNAME2,...\n"
-"                            Prevent registration of given modules.\n"
-"     --startup-time         Measure time of startup tasks.\n"
-        );
-    puts(
-"Gtk+ and Gdk options:\n"
-"     --display=DISPLAY      Set X display to use.\n"
-"     --screen=SCREEN        Set X screen to use.\n"
-"     --sync                 Make X calls synchronous.\n"
-"     --name=NAME            Set program name as used by the window manager.\n"
-"     --class=CLASS          Set program class as used by the window manager.\n"
-"     --gtk-module=MODULE    Load an additional Gtk module MODULE.\n"
-"They may be other Gtk+, Gdk, and GtkGLExt options, depending on platform, on\n"
-"how it was compiled, and on loaded modules.  Please see Gtk+ documentation.\n"
-        );
+    puts("Usage: gwyddion [OPTIONS...] FILES...\n"
+         "An SPM data visualization and analysis tool, written with Gtk+.\n");
+    puts("Interaction with other instances:\n"
+         "     --remote-query         Check if a Gwyddion instance is already running.\n"
+         "     --remote-new           Load FILES to a running instance or run a new one.\n"
+         "     --remote-existing      Load FILES to a running instance or fail.\n"
+         "     --new-instance         Run a new instance, ignoring any already running.\n"
+         "Any of these options also implicitly selects the normal GUI mode.\n");
+    puts("Non-GUI operations:\n"
+         "     --identify             Identify and print the type of SPM data FILES.\n"
+         "     --check                Check FILES, print problems and terminate.\n"
+         "     --convert-to-gwy=OUTFILE.gwy\n"
+         "                            Read FILES, merge them and write a GWY file.\n"
+         " -h, --help                 Print this help and terminate.\n"
+         " -v, --version              Print version info and terminate.\n");
+    puts("Logging:\n"
+         "     --log-to-file          Write messages to file set in GWYDDION_LOGFILE.\n"
+         "     --no-log-to-file       Do not write messages to any file.\n"
+         "     --log-to-console       Print messages to console.\n"
+         "     --no-log-to-console    Do not print messages to console.\n");
+    puts("Miscellaneous options:\n"
+         "     --no-splash            Don't show splash screen.\n"
+         "     --disable-gl           Disable OpenGL, including any availability checks.\n"
+         "     --disable-modules=MODNAME1,MODNAME2,...\n"
+         "                            Prevent registration of given modules.\n"
+         "     --startup-time         Measure time of startup tasks.\n");
+    puts("Gtk+ and Gdk options:\n"
+         "     --display=DISPLAY      Set X display to use.\n"
+         "     --screen=SCREEN        Set X screen to use.\n"
+         "     --sync                 Make X calls synchronous.\n"
+         "     --name=NAME            Set program name as used by the window manager.\n"
+         "     --class=CLASS          Set program class as used by the window manager.\n"
+         "     --gtk-module=MODULE    Load an additional Gtk module MODULE.\n"
+         "They may be other Gtk+, Gdk, and GtkGLExt options, depending on platform, on\n"
+         "how it was compiled, and on loaded modules.  Please see Gtk+ documentation.\n");
     puts("Please report bugs to <" PACKAGE_BUGREPORT ">.");
     exit(0);
 }
@@ -431,7 +455,7 @@ print_version_and_exit(void)
 }
 
 static void
-extend_disabled_modules_arg(const gchar *arg, gchar **modules_to_block)
+extend_disabled_modules_arg(gchar **modules_to_block, const gchar *arg)
 {
     if (!strlen(arg))
         return;
@@ -650,7 +674,6 @@ identify_command_line_files(gint n, gchar **args)
 {
     gint i, nfailures;
 
-    /* FIXME: cwd is in GLib encoding. And args? */
     for (i = nfailures = 0; i < n; i++) {
         const gchar *filename = args[i], *name;
         gint score;
@@ -672,7 +695,6 @@ check_command_line_files(gint n, gchar **args)
 {
     gint i, nfailures;
 
-    /* FIXME: cwd is in GLib encoding. And args? */
     for (i = nfailures = 0; i < n; i++) {
         const gchar *filename = args[i];
         const gchar *name = NULL;
@@ -709,6 +731,89 @@ check_command_line_files(gint n, gchar **args)
     return nfailures;
 }
 
+static gboolean
+select_one(gint* (*get_ids)(GwyContainer*),
+           void (*select_object)(GwyContainer*, gint),
+           GwyContainer *data)
+{
+    gint *ids;
+    gint id;
+
+    ids = get_ids(data);
+    id = ids[0];
+    g_free(ids);
+    if (id < 0)
+        return FALSE;
+
+    select_object(data, id);
+    return TRUE;
+}
+
+static void
+select_anything(GwyContainer *data)
+{
+    if (!select_one(gwy_app_data_browser_get_data_ids, gwy_app_data_browser_select_data_field, data)
+        && !select_one(gwy_app_data_browser_get_graph_ids, gwy_app_data_browser_select_graph_model, data)
+        && !select_one(gwy_app_data_browser_get_volume_ids, gwy_app_data_browser_select_brick, data)
+        && !select_one(gwy_app_data_browser_get_xyz_ids, gwy_app_data_browser_select_surface, data)
+        && !select_one(gwy_app_data_browser_get_curve_map_ids, gwy_app_data_browser_select_lawn, data)) {
+        g_warning("Cannot find any data object to select in a file.");
+    }
+}
+
+static gint
+convert_command_line_files(gint n, gchar **args, const gchar *outname)
+{
+    GwyContainer *maindata = NULL;
+    GError *error = NULL;
+    gint i, nfailures;
+
+    gwy_app_data_browser_set_gui_enabled(FALSE);
+
+    for (i = nfailures = 0; i < n; i++) {
+        GwyContainer *data;
+        const gchar *filename = args[i];
+
+        if (!(data = gwy_file_load(filename, GWY_RUN_NONINTERACTIVE, &error))) {
+            if (!error)
+                g_printerr("%s: Loader failed to report error properly!\n", filename);
+            else {
+                g_printerr("%s: %s\n", filename, error->message);
+                g_clear_error(&error);
+            }
+            nfailures++;
+            continue;
+        }
+
+        if (maindata) {
+            gwy_debug("merge %s (i=%d, nfailures=%d)", filename, i, nfailures);
+            if (i - nfailures == 1)
+                select_anything(maindata);
+            gwy_app_data_browser_merge(data);
+            g_object_unref(data);
+        }
+        else {
+            gwy_debug("add %s (i=%d, nfailures=%d)", filename, i, nfailures);
+            maindata = data;
+            gwy_app_data_browser_add(maindata);
+        }
+
+    }
+
+    if (!maindata) {
+        g_printerr("Cannot write %s: No data.\n", outname);
+        return nfailures ? nfailures : 1;
+    }
+    if (!gwy_file_func_run_save("gwyfile", maindata, outname, GWY_RUN_NONINTERACTIVE, &error)) {
+        g_printerr("Cannot write %s: %s\n", outname, error->message);
+        g_clear_error(&error);
+        return 1;
+    }
+    g_object_unref(maindata);
+
+    return 0;
+}
+
 static void
 block_modules(const gchar *modules_to_block)
 {
@@ -748,12 +853,16 @@ show_tip_at_startup(G_GNUC_UNUSED gpointer user_data)
  * gwy_widgets_gl_init()).
  **/
 static void
-gwy_app_init(int *argc, char ***argv)
+gwy_app_init(int *argc, char ***argv, gboolean is_gui)
 {
     gwy_widgets_type_init();
     /* Dump core on Critical errors in development versions. */
     if (RELEASEDATE == 0 || sizeof(GWY_VERSION_STRING) > 9)
         g_log_set_always_fatal(G_LOG_LEVEL_CRITICAL);
+
+    if (!is_gui)
+        return;
+
     g_set_application_name(PACKAGE_NAME);
     if (!gwy_app_gl_disabled())
         gwy_app_gl_init(argc, argv);
