@@ -1,5 +1,5 @@
 /*
- *  $Id: toolbox-editor.c 23951 2021-08-10 12:18:16Z yeti-dn $
+ *  $Id: toolbox-editor.c 24795 2022-04-28 15:20:36Z yeti-dn $
  *  Copyright (C) 2016-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -1798,7 +1798,7 @@ create_gwy_icon_list(GtkWidget *widget)
     GWY_STOCK_CANTILEVER,
     GWY_STOCK_COLOR_RANGE,
     GWY_STOCK_COLOR_RANGE_ADAPTIVE,
-   GWY_STOCK_COLOR_RANGE_AUTO,
+    GWY_STOCK_COLOR_RANGE_AUTO,
     GWY_STOCK_COLOR_RANGE_FIXED,
     GWY_STOCK_COLOR_RANGE_FULL,
     GWY_STOCK_CONVOLUTION,
@@ -1813,6 +1813,7 @@ create_gwy_icon_list(GtkWidget *widget)
     GWY_STOCK_DATA_MEASURE,
     GWY_STOCK_DECONVOLVE,
     GWY_STOCK_DISCONNECTED,
+    GWY_STOCK_DISPLACEMENT_FIELD,
     GWY_STOCK_DISTANCE,
     GWY_STOCK_DISTANCE_TRANSFORM,
     GWY_STOCK_DISTRIBUTION_ANGLE,
@@ -1846,8 +1847,10 @@ create_gwy_icon_list(GtkWidget *widget)
     GWY_STOCK_GL_MATERIAL,
     GWY_STOCK_GRADIENT_HORIZONTAL,
     GWY_STOCK_GRADIENT_VERTICAL,
+    GWY_STOCK_GRAIN_BOUNDING_BOX,
     GWY_STOCK_GRAIN_CORRELATION,
     GWY_STOCK_GRAIN_EXSCRIBED_CIRCLE,
+    GWY_STOCK_GRAIN_INSCRIBED_BOX,
     GWY_STOCK_GRAIN_INSCRIBED_CIRCLE,
     GWY_STOCK_GRAINS,
     GWY_STOCK_GRAINS_EDGE,
@@ -1920,6 +1923,7 @@ create_gwy_icon_list(GtkWidget *widget)
     GWY_STOCK_MASK_INVERT,
     GWY_STOCK_MASK_LINE,
     GWY_STOCK_MASK_MORPH,
+    GWY_STOCK_MASK_NOISIFY,
     GWY_STOCK_MASK_PAINT_DRAW,
     GWY_STOCK_MASK_PAINT_ERASE,
     GWY_STOCK_MASK_RECT_EXCLUSIVE,
@@ -1946,6 +1950,7 @@ create_gwy_icon_list(GtkWidget *widget)
     GWY_STOCK_NULL_OFFSETS,
     GWY_STOCK_PALETTES,
     GWY_STOCK_PATH_LEVEL,
+    GWY_STOCK_PERSPECTIVE_DISTORT,
     GWY_STOCK_POINTER_MEASURE,
     GWY_STOCK_POLY_DISTORT,
     GWY_STOCK_POLYNOM,
@@ -2032,6 +2037,7 @@ create_gwy_icon_list(GtkWidget *widget)
     GWY_STOCK_XYZIZE,
     GWY_STOCK_ZERO_MAXIMUM,
     GWY_STOCK_ZERO_MEAN,
+    GWY_STOCK_ZERO_UNDER_MASK,
     GWY_STOCK_ZOOM_1_1,
     GWY_STOCK_ZOOM_FIT,
     GWY_STOCK_ZOOM_IN,
