@@ -1,5 +1,5 @@
 /*
- *  $Id: gwycontainer.c 23909 2021-08-02 12:26:16Z yeti-dn $
+ *  $Id: gwycontainer.c 25041 2022-09-22 12:34:41Z yeti-dn $
  *  Copyright (C) 2003,2004,2014 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -26,6 +26,7 @@
 
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwydebugobjects.h>
+#include "gwyutils.h"
 #include "gwycontainer.h"
 #include "gwyserializable.h"
 
@@ -2399,7 +2400,7 @@ hash_text_serialize_func(gpointer hkey, gpointer hvalue, gpointer hdata)
         break;
 
         case G_TYPE_STRING:
-        s = g_strescape(g_value_get_string(value), NULL);
+        s = gwy_utf8_strescape(g_value_get_string(value), NULL);
         v = g_strdup_printf("\"%s\" string \"%s\"", k, s);
         g_free(s);
         break;
