@@ -1,5 +1,5 @@
 /*
- *  $Id: gwymath.c 24790 2022-04-28 13:41:13Z yeti-dn $
+ *  $Id: gwymath.c 25074 2022-10-11 13:46:47Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -750,12 +750,10 @@ gwy_math_tridiag_solve_rewrite(gint n,
 static inline void
 order_3(gdouble *array)
 {
-    if (array[1] < array[0])
-        DSWAP(array[0], array[1]);
+    GWY_ORDER(gdouble, array[0], array[1]);
     if (array[2] < array[1]) {
         DSWAP(array[1], array[2]);
-        if (array[1] < array[0])
-            DSWAP(array[0], array[1]);
+        GWY_ORDER(gdouble, array[0], array[1]);
     }
 }
 
@@ -769,8 +767,7 @@ kth_rank_simple(gsize n, gdouble *array, gsize k)
         return array[0];
 
     if (n == 2) {
-        if (array[1] < array[0])
-            DSWAP(array[0], array[1]);
+        GWY_ORDER(gdouble, array[0], array[1]);
         return array[k];
     }
 
@@ -804,8 +801,7 @@ kth_rank_simple(gsize n, gdouble *array, gsize k)
     }
 
     if (k == 1) {
-        if (array[1] < array[0])
-            DSWAP(array[0], array[1]);
+        GWY_ORDER(gdouble, array[0], array[1]);
         a = array[0];
         b = array[1];
 
@@ -827,8 +823,7 @@ kth_rank_simple(gsize n, gdouble *array, gsize k)
     }
 
     if (k == n-2) {
-        if (array[n-2] > array[n-1])
-            DSWAP(array[n-1], array[n-2]);
+        GWY_ORDER(gdouble, array[n-1], array[n-2]);
         a = array[n-1];
         b = array[n-2];
 
@@ -947,12 +942,10 @@ gwy_math_kth_rank(gsize n, gdouble *array, gsize k)
 
         /* Find median of lo, middle and hi items; swap into position lo */
         middle = (lo + hi)/2;
-        if (array[middle] > array[hi])
-            DSWAP(array[middle], array[hi]);
-        if (array[lo] > array[hi])
-            DSWAP(array[lo], array[hi]);
-        if (array[middle] > array[lo])
-            DSWAP(array[middle], array[lo]);
+
+        GWY_ORDER(gdouble, array[middle], array[hi]);
+        GWY_ORDER(gdouble, array[lo], array[hi]);
+        GWY_ORDER(gdouble, array[middle], array[lo]);
 
         /* Swap low item (now in position middle) into position (lo+1) */
         DSWAP(array[middle], array[lo+1]);
@@ -1678,8 +1671,21 @@ interpolate_parabolic(const GwyXY *xy, gdouble *x)
     return TRUE;
 }
 
-static int
-compare_double(const void *a, const void *b)
+/**
+ * gwy_compare_double:
+ * @a: Pointer to a double.
+ * @b: Pointer to a double.
+ *
+ * Compares two double values, given as pointers.
+ *
+ * This function is suitable as #GCompareFunc and can be also used with plain qsort().
+ *
+ * It should only be used to sort normal numbers. The behaviour for NaNs is undefined.
+ *
+ * Since: 2.62
+ **/
+gint
+gwy_compare_double(gconstpointer a, gconstpointer b)
 {
     const gdouble *da = (const gdouble*)a;
     const gdouble *db = (const gdouble*)b;
@@ -1746,8 +1752,7 @@ gwy_math_find_minimum_1d(GwyRealFunc function,
     guint i, imin, n, iter;
     gboolean at_left_edge = FALSE, at_right_edge = FALSE;
 
-    if (b < a)
-        GWY_SWAP(gdouble, a, b);
+    GWY_ORDER(gdouble, a, b);
     if (b-a < 1.2e-16*(fabs(a) + fabs(b)))
         return 0.5*(a + b);
 
@@ -1816,7 +1821,7 @@ gwy_math_find_minimum_1d(GwyRealFunc function,
             xy[i].y = function(xy[i].x, user_data);
             gwy_debug("point %.16g, value %.16g", xy[i].x, xy[i].y);
         }
-        qsort(xy, n, sizeof(GwyXY), compare_double);
+        qsort(xy, n, sizeof(GwyXY), gwy_compare_double);
 
         if (!find_min_in_array(xy, n, &imin))
             return xy[imin].x;
