@@ -1,5 +1,5 @@
 /*
- *  $Id: gwynlfit.c 24758 2022-04-06 12:18:50Z yeti-dn $
+ *  $Id: gwynlfit.c 24952 2022-08-26 15:24:56Z yeti-dn $
  *  Copyright (C) 2000-2003 Martin Siler.
  *  Copyright (C) 2004-2020 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
@@ -729,6 +729,8 @@ gwy_math_nlfit_fit_real(GwyNLFitter *nlfit,
 
     /* Parameter errors computation */
     if (nlfit->eval) {
+        gboolean eeval;
+
         /* Fix the case of exactly zero derivatives.
          * XXX: Can we do something about the error then? */
         gwy_assign(a, save_a, covar_size);
@@ -740,7 +742,8 @@ gwy_math_nlfit_fit_real(GwyNLFitter *nlfit,
         }
 
         /* If we fail just slightly, try again. */
-        if (!gwy_math_choleski_invert(n_var_param, save_a)) {
+        eeval = gwy_math_choleski_invert(n_var_param, save_a);
+        if (!eeval) {
             gwy_assign(save_a, a, covar_size);
             for (i = 0; i < n_var_param; i++) {
                 guint diag = i*(i + 3)/2;
@@ -749,9 +752,10 @@ gwy_math_nlfit_fit_real(GwyNLFitter *nlfit,
                     save_a[diag] = 1.0;
                 save_a[diag] *= 1.0001;
             }
+            eeval = gwy_math_choleski_invert(n_var_param, save_a);
         }
 
-        if (gwy_math_choleski_invert(n_var_param, save_a)) {
+        if (eeval) {
             /* Stretch the matrix to span over fixed params too.  This is
              * actually a bit silly and callers may want to undo it later. */
             nlfit->covar = covar = g_new(gdouble, nparam*(nparam + 1)/2);
@@ -1275,9 +1279,8 @@ gwy_math_nlfit_get_correlations(GwyNLFitter *nlfit, gint par1, gint par2)
 
     if (par1 == par2)
         return 1.0;
-    if (par1 < par2)
-        GWY_SWAP(gint, par1, par2);
 
+    GWY_ORDER(gint, par2, par1);
     t = SLi(nlfit->covar, par1, par1) * SLi(nlfit->covar, par2, par2);
     if (t == 0) {
         g_warning("Zero element in covar matrix");
