@@ -1,22 +1,18 @@
 /*
- *  $Id: gwyresults.h 22091 2019-05-27 07:51:09Z yeti-dn $
- *  Copyright (C) 2017 David Necas (Yeti).
+ *  $Id: gwyresults.h 24606 2022-02-16 14:45:59Z yeti-dn $
+ *  Copyright (C) 2017-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef __GWY_RESULTS_H__
@@ -87,6 +83,9 @@ void         gwy_results_add_value_str          (GwyResults *results,
 void         gwy_results_add_value_x            (GwyResults *results,
                                                  const gchar *id,
                                                  const gchar *label);
+void         gwy_results_add_value_y            (GwyResults *results,
+                                                 const gchar *id,
+                                                 const gchar *label);
 void         gwy_results_add_value_z            (GwyResults *results,
                                                  const gchar *id,
                                                  const gchar *label);
@@ -188,4 +187,4 @@ G_END_DECLS
 
 #endif /* __GWY_RESULTS_H__ */
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
