@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyutils.c 24600 2022-02-15 12:28:54Z yeti-dn $
+ *  $Id: gwyutils.c 25035 2022-09-22 12:23:34Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1607,6 +1607,78 @@ gwy_utf16_to_utf8(const gunichar2 *str,
     return retval;
 }
 
+/**
+ * gwy_utf8_strescape:
+ * @source: A string to escape.
+ * @exceptions: A string containing characters to not escape, or %NULL.
+ *
+ * Escapes special characters in a string.
+ *
+ * Escapes the special characters '\b', '\f', '\n', '\r', '\t', '\v', '\' and '"' in the string source by inserting
+ * a '\' before them. Additionally all characters in the range 0x01-0x1F (everything below SPACE), the character 0x7F,
+ * characters 0xC0 and 0xC1, and all characters in the range 0xF5-0xFF are replaced with a '\' followed by their octal
+ * representation. Characters supplied in exceptions are not escaped.
+ *
+ * In essence, this functions differs from g_strescape() by preserving valid UTF-8 strings, unless they contain ‘odd’
+ * characters. Passing strings which are not valid UTF-8 is possible, but generally not advised.
+ *
+ * Function g_strcompress() does the reverse conversion.
+ *
+ * Returns: A newly allocated string with special characters escaped.
+ *
+ * Since: 2.62
+ **/
+gchar*
+gwy_utf8_strescape(const gchar *source,
+                   const gchar *exceptions)
+{
+    static const gchar stdescapes[] = { 'b', 't', 'n', 'v', 'f', 'r' };
+    const guchar *p = (guchar*)source;
+    guchar *dest, *q;
+    guchar c, excmap[256];
+
+    g_return_val_if_fail(source, NULL);
+
+    /* Each source byte needs maximally four destination chars (\777) */
+    q = dest = g_malloc(4*strlen(source) + 1);
+
+    if (exceptions && *exceptions) {
+        guchar *e = (guchar*)exceptions;
+
+        gwy_clear(excmap, 256);
+        do {
+            excmap[*(e++)] = 1;
+        } while (*e);
+    }
+    else
+        exceptions = NULL;
+
+    while ((c = *p)) {
+        if (exceptions && excmap[c])
+            *(q++) = c;
+        else if (c >= '\b' && c <= '\r') {
+            *(q++) = '\\';
+            *(q++) = stdescapes[c - '\b'];
+        }
+        else if (c == '\\' || c == '"') {
+            *(q++) = '\\';
+            *(q++) = c;
+        }
+        else if (c < ' ' || c == 0x7f || c == 0xc0 || c == 0xc1 || c >= 0xf5) {
+            *(q++) = '\\';
+            *(q++) = '0' + ((c >> 6) & 0x07);
+            *(q++) = '0' + ((c >> 3) & 0x07);
+            *(q++) = '0' + (c & 0x07);
+        }
+        else
+            *(q++) = c;
+        p++;
+    }
+    *q = 0;
+
+    return dest;
+}
+
 /**
  * gwy_assign_string:
  * @target: Pointer to target string, typically a struct field.
@@ -2028,6 +2100,26 @@ gwy_fprintf(FILE        *file,
  * @y: A variable of type @t to swap with @y.
  *
  * Swaps two variables (more precisely lhs and rhs expressions) of type @t in a single statement.
+ *
+ * Both @x and @y are evaluated multiple times as both lhs and rhs expressions. If their evaluation has any side
+ * effect the result is undefined.
+ */
+
+/**
+ * GWY_ORDER:
+ * @t: A C type.
+ * @x: A variable of type @t to possibly swap with @x.
+ * @y: A variable of type @t to possibly swap with @y.
+ *
+ * Ensures ascending order of two variables (more precisely lhs and rhs expressions) of type @t in a single statement.
+ *
+ * The macro makes @y not larger than @x. They may be still equal or their relation can be undefined (if either is
+ * NaN).
+ *
+ * Both @x and @y are evaluated multiple times as both lhs and rhs expressions. If their evaluation has any side
+ * effect the result is undefined.
+ *
+ * Since: 2.62
  */
 
 /**
