@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyutils.h 24011 2021-08-17 15:05:29Z yeti-dn $
+ *  $Id: gwyutils.h 25035 2022-09-22 12:23:34Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -59,7 +59,7 @@ gchar*       gwy_strkill               (gchar *s,
 gchar*       gwy_strreplace            (const gchar *haystack,
                                         const gchar *needle,
                                         const gchar *replacement,
-                                        gsize maxrepl);
+                                        gsize maxrepl)              G_GNUC_MALLOC;
 gint         gwy_strdiffpos            (const gchar *s1,
                                         const gchar *s2);
 gboolean     gwy_strisident            (const gchar *s,
@@ -84,7 +84,7 @@ gboolean     gwy_file_abandon_contents (guchar *buffer,
 gchar*       gwy_find_self_dir         (const gchar *dirname);
 const gchar* gwy_get_user_dir          (void);
 const gchar* gwy_get_home_dir          (void);
-gchar*       gwy_canonicalize_path     (const gchar *path);
+gchar*       gwy_canonicalize_path     (const gchar *path)          G_GNUC_MALLOC;
 gboolean     gwy_filename_ignore       (const gchar *filename_sys);
 gchar*       gwy_sgettext              (const gchar *msgid);
 gchar*       gwy_str_next_line         (gchar **buffer);
@@ -110,7 +110,9 @@ void         gwy_convert_raw_data      (gconstpointer data,
 guint        gwy_raw_data_size         (GwyRawDataType datatype);
 gchar*       gwy_utf16_to_utf8         (const gunichar2 *str,
                                         glong len,
-                                        GwyByteOrder byteorder);
+                                        GwyByteOrder byteorder)     G_GNUC_MALLOC;
+gchar*       gwy_utf8_strescape        (const gchar *source,
+                                        const gchar *exceptions)    G_GNUC_MALLOC;
 gboolean     gwy_assign_string         (gchar **target,
                                         const gchar *newvalue);
 void         gwy_object_set_or_reset   (gpointer object,
