@@ -1,5 +1,5 @@
 /*
- *  $Id: gwy3dlabel.c 21680 2018-11-26 10:39:39Z yeti-dn $
+ *  $Id: gwy3dlabel.c 23205 2021-03-08 15:35:13Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -22,6 +22,7 @@
 #include "config.h"
 #include <string.h>
 #include <libgwyddion/gwymacros.h>
+#include <libgwyddion/gwyutils.h>
 #include <libgwyddion/gwyserializable.h>
 #include <libgwyddion/gwydebugobjects.h>
 #include <libgwydgets/gwy3dlabel.h>
@@ -320,8 +321,7 @@ gwy_3d_label_clone_real(GObject *source,
 
     if (!gwy_strequal(label->default_text, clone->default_text)) {
         g_warning("Trying to change construction-only property by cloning");
-        g_free(clone->default_text);
-        clone->default_text = g_strdup(label->default_text);
+        gwy_assign_string(&clone->default_text, label->default_text);
     }
 
     g_object_freeze_notify(copy);
