@@ -1,5 +1,5 @@
 /*
- *  $Id: gwycoloraxis.c 21680 2018-11-26 10:39:39Z yeti-dn $
+ *  $Id: gwycoloraxis.c 24535 2021-11-22 17:15:18Z yeti-dn $
  *  Copyright (C) 2003-2014 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -988,22 +988,19 @@ void
 gwy_color_axis_set_si_unit(GwyColorAxis *axis,
                            GwySIUnit *unit)
 {
-    gboolean not_equal;
+    gboolean not_equal = TRUE;
     GwySIUnit *old;
 
     g_return_if_fail(GWY_IS_COLOR_AXIS(axis));
     g_return_if_fail(GWY_IS_SI_UNIT(unit));
 
-    if (axis->siunit == unit)
-        return;
-
-    not_equal = !gwy_si_unit_equal(unit, axis->siunit);
-
-    old = axis->siunit;
-    g_object_ref(unit);
-    axis->siunit = unit;
-    GWY_OBJECT_UNREF(old);
-
+    if (axis->siunit != unit) {
+        not_equal = !gwy_si_unit_equal(unit, axis->siunit);
+        old = axis->siunit;
+        g_object_ref(unit);
+        axis->siunit = unit;
+        GWY_OBJECT_UNREF(old);
+    }
     if (not_equal)
         gwy_color_axis_changed(axis);
 }
