@@ -1,5 +1,5 @@
 /*
- *  $Id: gwycombobox.c 24329 2021-10-11 15:08:54Z yeti-dn $
+ *  $Id: gwycombobox.c 24952 2022-08-26 15:24:56Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -363,8 +363,7 @@ gwy_combo_box_metric_unit_make_enum(gint from,
 
     from = from/3;
     to = (to + 2)/3;
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     n = (to - from) + 1;
     entries = g_new(GwyEnum, n + 1);
