@@ -1,6 +1,6 @@
 /*
- *  $Id: gwygrapharea.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003-2006,2015 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwygrapharea.c 21782 2019-01-03 12:56:50Z yeti-dn $
+ *  Copyright (C) 2003-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -697,45 +697,41 @@ gwy_graph_area_draw_zoom(GdkDrawable *drawable,
                          GwyGraphArea *area)
 {
     gint xmin, ymin, xmax, ymax, n_of_zooms;
-    gdouble selection_zoomdata[4];
+    gdouble sel_zoomdata[4];
+    GtkWidget *widget;
 
     n_of_zooms = gwy_selection_get_data(area->zoomdata, NULL);
 
     if (n_of_zooms != 1)
         return;
 
-    gwy_selection_get_object(area->zoomdata, 0, selection_zoomdata);
+    gwy_selection_get_object(area->zoomdata, 0, sel_zoomdata);
 
-    if (selection_zoomdata[2] == 0 || selection_zoomdata[3] == 0) return;
+    if (sel_zoomdata[2] == 0 || sel_zoomdata[3] == 0) return;
     gdk_gc_set_function(gc, GDK_INVERT);
+    widget = GTK_WIDGET(area);
 
-    if (selection_zoomdata[2] < 0) {
-        xmin = data_to_scr_x(GTK_WIDGET(area), selection_zoomdata[0]
-                             + selection_zoomdata[2]);
-        xmax = data_to_scr_x(GTK_WIDGET(area), selection_zoomdata[0]);
+    if (sel_zoomdata[2] < 0) {
+        xmin = data_to_scr_x(widget, sel_zoomdata[0] + sel_zoomdata[2]);
+        xmax = data_to_scr_x(widget, sel_zoomdata[0]);
     }
     else {
-        xmin = data_to_scr_x(GTK_WIDGET(area), selection_zoomdata[0]);
-        xmax = data_to_scr_x(GTK_WIDGET(area), selection_zoomdata[0]
-                             + selection_zoomdata[2]);
+        xmin = data_to_scr_x(widget, sel_zoomdata[0]);
+        xmax = data_to_scr_x(widget, sel_zoomdata[0] + sel_zoomdata[2]);
     }
 
-    if (selection_zoomdata[3] > 0) {
-        ymin = data_to_scr_y(GTK_WIDGET(area), selection_zoomdata[1]
-                             + selection_zoomdata[3]);
-        ymax = data_to_scr_y(GTK_WIDGET(area), selection_zoomdata[1]);
+    if (sel_zoomdata[3] > 0) {
+        ymin = data_to_scr_y(widget, sel_zoomdata[1] + sel_zoomdata[3]);
+        ymax = data_to_scr_y(widget, sel_zoomdata[1]);
     }
     else {
-        ymin = data_to_scr_y(GTK_WIDGET(area), selection_zoomdata[1]);
-        ymax = data_to_scr_y(GTK_WIDGET(area), selection_zoomdata[1]
-                             + selection_zoomdata[3]);
+        ymin = data_to_scr_y(widget, sel_zoomdata[1]);
+        ymax = data_to_scr_y(widget, sel_zoomdata[1] + sel_zoomdata[3]);
     }
 
     gdk_draw_rectangle(drawable, area->gc, 0,
-                       xmin,
-                       ymin,
-                       xmax - xmin,
-                       ymax - ymin);
+                       xmin, ymin,
+                       xmax - xmin, ymax - ymin);
 
     gdk_gc_set_function(area->gc, GDK_COPY);
 }
@@ -1562,28 +1558,6 @@ gwy_graph_area_draw_child_rectangle(GwyGraphArea *area)
     gdk_gc_set_function(area->gc, GDK_COPY);
 }
 
-static gdouble
-scr_to_data_x(GtkWidget *widget, gint scr)
-{
-    GwyGraphArea *area;
-    GwyGraphModel *model;
-    gboolean lg;
-
-    area = GWY_GRAPH_AREA(widget);
-    model = area->graph_model;
-
-    scr = CLAMP(scr, 0, widget->allocation.width-1);
-    g_object_get(model, "x-logarithmic", &lg, NULL);
-    if (!lg)
-        return area->x_min + scr*(area->x_max
-                                  - area->x_min) /(widget->allocation.width-1);
-    else
-        return pow(10, log10(area->x_min)
-                   + scr*(log10(area->x_max)
-                                - log10(area->x_min))/(widget->allocation.width-1));
-
-}
-
 static void
 gwy_graph_area_model_notify(GwyGraphArea *area,
                             GParamSpec *pspec,
@@ -1680,25 +1654,47 @@ gwy_graph_area_curve_data_changed(GwyGraphArea *area,
         gtk_widget_queue_draw(GTK_WIDGET(area));
 }
 
+static gdouble
+scr_to_data_x(GtkWidget *widget, gint scr)
+{
+    GwyGraphArea *area;
+    GwyGraphModel *model;
+    gdouble xmin, xmax;
+    gint w;
+    gboolean lg;
+
+    area = GWY_GRAPH_AREA(widget);
+    model = area->graph_model;
+    w = widget->allocation.width;
+    xmin = area->x_min;
+    xmax = area->x_max;
+
+    scr = CLAMP(scr, 0, w-1);
+    g_object_get(model, "x-logarithmic", &lg, NULL);
+    if (!lg)
+        return xmin + scr*(xmax - xmin)/(w-1);
+    return exp(log(xmin) + scr*log(xmax/xmin)/(w-1));
+}
+
 static gint
 data_to_scr_x(GtkWidget *widget, gdouble data)
 {
     GwyGraphArea *area;
     GwyGraphModel *model;
+    gdouble xmin, xmax;
+    gint w;
     gboolean lg;
 
     area = GWY_GRAPH_AREA(widget);
     model = area->graph_model;
+    w = widget->allocation.width;
+    xmin = area->x_min;
+    xmax = area->x_max;
 
     g_object_get(model, "x-logarithmic", &lg, NULL);
     if (!lg)
-        return (data - area->x_min)
-            /((area->x_max
-               - area->x_min)/(widget->allocation.width-1));
-    else
-        return (log10(data) - log10(area->x_min))
-            /((log10(area->x_max)
-               - log10(area->x_min))/(widget->allocation.width-1));
+        return (data - xmin)/(xmax - xmin)*(w-1);
+    return log(data/xmin)/log(xmax/xmin)*(w-1);
 }
 
 static gdouble
@@ -1706,23 +1702,21 @@ scr_to_data_y(GtkWidget *widget, gint scr)
 {
     GwyGraphArea *area;
     GwyGraphModel *model;
+    gdouble ymin, ymax;
+    gint h;
     gboolean lg;
 
     area = GWY_GRAPH_AREA(widget);
     model = area->graph_model;
+    h = widget->allocation.height;
+    ymin = area->y_min;
+    ymax = area->y_max;
 
-    scr = CLAMP(scr, 0, widget->allocation.height-1);
+    scr = CLAMP(scr, 0, h-1);
     g_object_get(model, "y-logarithmic", &lg, NULL);
     if (!lg)
-        return area->y_min
-            + (widget->allocation.height - scr)*
-            (area->y_max - area->y_min)
-            /(widget->allocation.height-1);
-    else
-        return pow(10, log10(area->y_min)
-                   + (widget->allocation.height - scr)*
-                   (log10(area->y_max) - log10(area->y_min))
-                   /(widget->allocation.height-1));
+        return ymin + (h - scr)*(ymax - ymin)/(h-1);
+    return exp(log(ymin) + (h - scr)*log(ymax/ymin)/(h-1));
 }
 
 static gint
@@ -1730,21 +1724,20 @@ data_to_scr_y(GtkWidget *widget, gdouble data)
 {
     GwyGraphArea *area;
     GwyGraphModel *model;
+    gdouble ymin, ymax;
+    gint h;
     gboolean lg;
 
     area = GWY_GRAPH_AREA(widget);
     model = area->graph_model;
+    h = widget->allocation.height;
+    ymin = area->y_min;
+    ymax = area->y_max;
+
     g_object_get(model, "y-logarithmic", &lg, NULL);
     if (!lg)
-        return widget->allocation.height
-            - (data - area->y_min)
-            /((area->y_max
-               - area->y_min)/((gdouble)widget->allocation.height-1));
-    else
-        return widget->allocation.height
-            - (log10(data) - log10(area->y_min))
-            /((log10(area->y_max)
-               - log10(area->y_min))/((gdouble)widget->allocation.height-1));
+        return h - (data - ymin)/(ymax - ymin)*(h-1);
+    return h - log(data/ymin)/log(ymax/ymin)*(h-1);
 }
 
 static void
