@@ -1,5 +1,5 @@
 /*
- *  $Id: gwygraphcurvemodel.c 24589 2022-02-11 16:49:14Z yeti-dn $
+ *  $Id: gwygraphcurvemodel.c 25077 2022-10-11 13:48:33Z yeti-dn $
  *  Copyright (C) 2005-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -757,19 +757,6 @@ find_out_ordering_type(GwyGraphCurveModel *gcmodel)
     return 0;
 }
 
-static int
-compare_double(gconstpointer a, gconstpointer b)
-{
-    const double da = *(const double*)a;
-    const double db = *(const double*)b;
-
-    if (da < db)
-        return -1;
-    if (da > db)
-        return 1;
-    return 0;
-}
-
 /**
  * gwy_graph_curve_model_enforce_order:
  * @gcmodel: A graph curve model.
@@ -816,7 +803,7 @@ gwy_graph_curve_model_enforce_order(GwyGraphCurveModel *gcmodel)
         bothdata[2*i + 0] = xdata[i];
         bothdata[2*i + 1] = ydata[i];
     }
-    qsort(bothdata, n, 2*sizeof(gdouble), compare_double);
+    qsort(bothdata, n, 2*sizeof(gdouble), gwy_compare_double);
     for (i = 0; i < n; i++) {
         xdata[i] = bothdata[2*i + 0];
         ydata[i] = bothdata[2*i + 1];
