@@ -1,5 +1,5 @@
 /*
- *  $Id: gwygraphmodel.c 21680 2018-11-26 10:39:39Z yeti-dn $
+ *  $Id: gwygraphmodel.c 22978 2020-11-30 12:54:18Z yeti-dn $
  *  Copyright (C) 2004-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1547,6 +1547,8 @@ gwy_graph_model_set_units_from_data_field(GwyGraphModel *gmodel,
     gwy_si_unit_power_multiply(xyunit, power_xy_in_y,
                                zunit, power_z_in_y,
                                gmodel->y_unit);
+    g_object_notify(G_OBJECT(gmodel), "si-unit-x");
+    g_object_notify(G_OBJECT(gmodel), "si-unit-y");
 }
 
 /**
@@ -2414,7 +2416,6 @@ ascii_name(const gchar *s)
     }
 
     as[n] = '\0';
-    g_printerr("<%s>\n", as);
     return as;
 }
 
