@@ -1,5 +1,5 @@
 /*
- *  $Id: gwygraphwindow.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: gwygraphwindow.c 24710 2022-03-21 17:35:20Z yeti-dn $
  *  Copyright (C) 2004-2016 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -57,8 +57,6 @@ static void     gwy_graph_window_resize          (GwyGraphWindow *graphwindow,
                                                   gint zoomtype);
 static void     gwy_graph_window_measure_finished(GwyGraphWindow *graphwindow,
                                                   gint response);
-static void     gwy_graph_window_set_tooltip     (GtkWidget *widget,
-                                                  const gchar *tip_text);
 static void     graph_title_changed              (GwyGraphWindow *graphwindow);
 static void   gwy_graph_window_curves_row_activated(GwyGraphWindow *graphwindow,
                                                     GtkTreePath *path,
@@ -193,7 +191,7 @@ gwy_graph_window_new(GwyGraph *graph)
                                                GTK_ICON_SIZE_LARGE_TOOLBAR));
     gtk_box_pack_start(GTK_BOX(hbox), graphwindow->button_measure_points,
                        FALSE, FALSE, 0);
-    gwy_graph_window_set_tooltip(graphwindow->button_measure_points,
+    gtk_widget_set_tooltip_text(graphwindow->button_measure_points,
                                  _("Measure distances in graph"));
     g_signal_connect_swapped(graphwindow->button_measure_points, "clicked",
                              G_CALLBACK(gwy_graph_window_measure),
@@ -205,7 +203,7 @@ gwy_graph_window_new(GwyGraph *graph)
                                                GTK_ICON_SIZE_LARGE_TOOLBAR));
     gtk_box_pack_start(GTK_BOX(hbox), graphwindow->button_zoom_in,
                        FALSE, FALSE, 0);
-    gwy_graph_window_set_tooltip(graphwindow->button_zoom_in,
+    gtk_widget_set_tooltip_text(graphwindow->button_zoom_in,
                                  _("Zoom in by mouse selection"));
     g_signal_connect_swapped(graphwindow->button_zoom_in, "toggled",
                              G_CALLBACK(gwy_graph_window_zoom_in),
@@ -217,7 +215,7 @@ gwy_graph_window_new(GwyGraph *graph)
                                                GTK_ICON_SIZE_LARGE_TOOLBAR));
     gtk_box_pack_start(GTK_BOX(hbox), graphwindow->button_zoom_to_fit,
                        FALSE, FALSE, 0);
-    gwy_graph_window_set_tooltip(graphwindow->button_zoom_to_fit,
+    gtk_widget_set_tooltip_text(graphwindow->button_zoom_to_fit,
                                  _("Zoom out to full curve"));
     g_signal_connect_swapped(graphwindow->button_zoom_to_fit, "clicked",
                              G_CALLBACK(gwy_graph_window_zoom_to_fit),
@@ -229,7 +227,7 @@ gwy_graph_window_new(GwyGraph *graph)
                                                GTK_ICON_SIZE_LARGE_TOOLBAR));
     gtk_box_pack_start(GTK_BOX(hbox), graphwindow->button_x_log,
                        FALSE, FALSE, 0);
-    gwy_graph_window_set_tooltip(graphwindow->button_x_log,
+    gtk_widget_set_tooltip_text(graphwindow->button_x_log,
                                  _("Toggle logarithmic x axis"));
     g_signal_connect_swapped(graphwindow->button_x_log, "clicked",
                              G_CALLBACK(gwy_graph_window_x_log),
@@ -241,7 +239,7 @@ gwy_graph_window_new(GwyGraph *graph)
                                                GTK_ICON_SIZE_LARGE_TOOLBAR));
     gtk_box_pack_start(GTK_BOX(hbox), graphwindow->button_y_log,
                        FALSE, FALSE, 0);
-    gwy_graph_window_set_tooltip(graphwindow->button_y_log,
+    gtk_widget_set_tooltip_text(graphwindow->button_y_log,
                                  _("Toggle logarithmic y axis"));
     g_signal_connect_swapped(graphwindow->button_y_log, "clicked",
                              G_CALLBACK(gwy_graph_window_y_log),
@@ -329,7 +327,6 @@ gwy_graph_window_get_graph_curves(GwyGraphWindow *graphwindow)
     return graphwindow->curves;
 }
 
-/*XXX: the "tips" line below is hard to understand */
 /**
  * gwy_graph_window_class_set_tooltips:
  * @tips: #GtkTooltips object #GwyGraphWindow should use for setting tooltips.
@@ -337,6 +334,8 @@ gwy_graph_window_get_graph_curves(GwyGraphWindow *graphwindow)
  *
  * Sets the tooltips object to use for adding tooltips to graph window parts.
  *
+ * This function does not do anything useful.  Do not use it.
+ *
  * This is a class method.  It affects only newly created graph windows,
  * existing graph windows will continue to use the tooltips they were
  * constructed with.
@@ -365,6 +364,8 @@ gwy_graph_window_class_set_tooltips(GtkTooltips *tips)
  *
  * Gets the tooltips object used for adding tooltips to Graph window parts.
  *
+ * This function does not do anything useful.  Do not use it.
+ *
  * Returns: The #GtkTooltips object. (Do not free).
  **/
 GtkTooltips*
@@ -706,14 +707,6 @@ gwy_graph_window_y_log(GwyGraphWindow *graphwindow)
     g_object_set(model, "y-logarithmic", !state, NULL);
 }
 
-static void
-gwy_graph_window_set_tooltip(GtkWidget *widget,
-                             const gchar *tip_text)
-{
-    if (tooltips)
-        gtk_tooltips_set_tip(tooltips, widget, tip_text, NULL);
-}
-
 static void
 graph_title_changed(GwyGraphWindow *graphwindow)
 {
