@@ -1,22 +1,18 @@
 /*
- *  $Id: gwynullstore.c 21680 2018-11-26 10:39:39Z yeti-dn $
- *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwynullstore.c 24900 2022-08-18 16:06:57Z yeti-dn $
+ *  Copyright (C) 2005-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -30,13 +26,11 @@
         static const GInterfaceInfo gwy_tree_model_iface_info = { \
             (GInterfaceInitFunc)iface_init, NULL, NULL \
         }; \
-        g_type_add_interface_static(g_define_type_id, \
-                                    GTK_TYPE_TREE_MODEL, \
-                                    &gwy_tree_model_iface_info); \
+        g_type_add_interface_static(g_define_type_id, GTK_TYPE_TREE_MODEL, &gwy_tree_model_iface_info); \
     }
 
 static void              gwy_null_store_finalize       (GObject *object);
-static void            gwy_null_store_tree_model_init(GtkTreeModelIface *iface);
+static void              gwy_null_store_tree_model_init(GtkTreeModelIface *iface);
 static GtkTreeModelFlags gwy_null_store_get_flags      (GtkTreeModel *model);
 static gint              gwy_null_store_get_n_columns  (GtkTreeModel *model);
 static GType             gwy_null_store_get_column_type(GtkTreeModel *model,
@@ -67,9 +61,8 @@ static gboolean          gwy_null_store_iter_parent    (GtkTreeModel *model,
                                                         GtkTreeIter *iter,
                                                         GtkTreeIter *child);
 
-G_DEFINE_TYPE_EXTENDED
-    (GwyNullStore, gwy_null_store, G_TYPE_OBJECT, 0,
-     GWY_IMPLEMENT_TREE_MODEL(gwy_null_store_tree_model_init))
+G_DEFINE_TYPE_EXTENDED(GwyNullStore, gwy_null_store, G_TYPE_OBJECT, 0,
+                       GWY_IMPLEMENT_TREE_MODEL(gwy_null_store_tree_model_init))
 
 static void
 gwy_null_store_class_init(GwyNullStoreClass *klass)
@@ -300,8 +293,7 @@ gwy_null_store_new(guint n)
  *
  * Gets the number of imaginary rows in a null store.
  *
- * This is a convenience function, the same information can be obtained with
- * gtk_tree_model_iter_n_children().
+ * This is a convenience function, the same information can be obtained with gtk_tree_model_iter_n_children().
  *
  * Returns: The number of rows.
  **/
@@ -320,16 +312,14 @@ gwy_null_store_get_n_rows(GwyNullStore *store)
  *
  * Sets the number of imaginary rows in a null store.
  *
- * If the new number of rows is larger than the current one, rows will be
- * sequentially and virtually appended to the end of the store until the
- * requested number of rows is reached.
+ * If the new number of rows is larger than the current one, rows will be sequentially and virtually appended to the
+ * end of the store until the requested number of rows is reached.
  *
- * Similarly, if the new number of rows is smaller then the current one, rows
- * will be sequentially and virtually deleted from the end of the store until
- * the requested number of rows is reached.
+ * Similarly, if the new number of rows is smaller then the current one, rows will be sequentially and virtually
+ * deleted from the end of the store until the requested number of rows is reached.
  *
- * Note for radical changes it is usually more useful to disconnect the model
- * from its view(s), change the number of rows, and then reconnect.
+ * Note for radical changes it is usually more useful to disconnect the model from its view(s), change the number of
+ * rows, and then reconnect.
  **/
 void
 gwy_null_store_set_n_rows(GwyNullStore *store,
@@ -387,23 +377,19 @@ gwy_null_store_get_model(GwyNullStore *store)
  * gwy_null_store_set_model:
  * @store: A null store.
  * @model: Model pointer.
- * @destroy: Function to call on @model when it is replaced or the store is
- *           destroyed.
+ * @destroy: Function to call on @model when it is replaced or the store is destroyed.
  *
  * Sets the model pointer of a null store.
  *
- * While the virtual integers in #GwyNullStore can be used directly, a null
- * store typically serves as an adaptor for array-like structures and its rows
- * are used as indices to these structures.  This helper method provides means
- * to attach such a structure to a null store in the common case.
+ * While the virtual integers in #GwyNullStore can be used directly, a null store typically serves as an adaptor for
+ * array-like structures and its rows are used as indices to these structures.  This helper method provides means to
+ * attach such a structure to a null store in the common case.
  *
- * The store itself does not interpret nor access the attached data by any
- * means.  No signals are emitted in response to the model pointer change
- * either, particularly because it is expected to be set only once upon
- * creation (null stores are cheap).
+ * The store itself does not interpret nor access the attached data by any means.  No signals are emitted in response
+ * to the model pointer change either, particularly because it is expected to be set only once upon creation (null
+ * stores are cheap).
  *
- * You are free to keep the model pointer at %NULL if these functions do not
- * suit your needs.
+ * You are free to keep the model pointer at %NULL if these functions do not suit your needs.
  **/
 void
 gwy_null_store_set_model(GwyNullStore *store,
@@ -430,8 +416,8 @@ gwy_null_store_set_model(GwyNullStore *store,
  *
  * Emits "GtkTreeModel::row-changed" signal on a null store.
  *
- * This is a convenience method, with a bit more work the same effect can be
- * achieved with gtk_tree_model_row_changed().
+ * This is a convenience method, with a bit more work the same effect can be achieved with
+ * gtk_tree_model_row_changed().
  **/
 void
 gwy_null_store_row_changed(GwyNullStore *store,
@@ -452,6 +438,58 @@ gwy_null_store_row_changed(GwyNullStore *store,
     gtk_tree_path_free(path);
 }
 
+/**
+ * gwy_null_store_rows_changed:
+ * @store: A null store.
+ * @ifrom: The first row to emit "row-changed" on.
+ * @ito: The last row to emit "row-changed" on (inclusive).
+ *
+ * Emits "GtkTreeModel::row-changed" signal on a block of rows in a null store.
+ *
+ * This is a convenience method, with a bit more work the same effect can be achieved with
+ * gtk_tree_model_row_changed().
+ *
+ * It is possible to pass @ifrom larger than @ito. The signal is then emitted for the same rows, but in the opposite
+ * order.
+ *
+ * Since: 2.62
+ **/
+void
+gwy_null_store_rows_changed(GwyNullStore *store,
+                            guint ifrom,
+                            guint ito)
+{
+    GtkTreePath *path;
+    GtkTreeIter iter;
+    guint i;
+
+    g_return_if_fail(GWY_IS_NULL_STORE(store));
+    g_return_if_fail(ifrom < store->n);
+    g_return_if_fail(ito < store->n);
+
+    iter.stamp = store->stamp;
+
+    path = gtk_tree_path_new();
+    gtk_tree_path_append_index(path, ifrom);
+    if (ifrom <= ito) {
+        for (i = ifrom; i <= ito; i++) {
+            iter.user_data = GUINT_TO_POINTER(i);
+            gtk_tree_model_row_changed(GTK_TREE_MODEL(store), path, &iter);
+            if (i < ito)
+                gtk_tree_path_next(path);
+        }
+    }
+    else {
+        for (i = ifrom; i >= ito; i--) {
+            iter.user_data = GUINT_TO_POINTER(i);
+            gtk_tree_model_row_changed(GTK_TREE_MODEL(store), path, &iter);
+            if (i < ito)
+                gtk_tree_path_prev(path);
+        }
+    }
+    gtk_tree_path_free(path);
+}
+
 /**
  * gwy_null_store_iter_is_valid:
  * @store: A null store.
@@ -467,9 +505,7 @@ gwy_null_store_iter_is_valid(GwyNullStore *store,
 {
     g_return_val_if_fail(GWY_IS_NULL_STORE(store), FALSE);
 
-    if (!iter
-        || iter->stamp != store->stamp
-        || GPOINTER_TO_UINT(iter->user_data) >= store->n)
+    if (!iter || iter->stamp != store->stamp || GPOINTER_TO_UINT(iter->user_data) >= store->n)
         return FALSE;
 
     return TRUE;
@@ -483,22 +519,19 @@ gwy_null_store_iter_is_valid(GwyNullStore *store,
  * @short_description: GtkTreeModel wrapper around nothing
  * @see_also: #GwyInventoryStore -- #GtkTreeModel wrapper around #GwyInventory
  *
- * #GwyNullStore is a very simple class which pretends to be a #GtkTreeModel
- * with one column of type %G_TYPE_UINT whose values are equal to row numbers
- * (counted from 0).  In reality the column is purely virtual and the store
+ * #GwyNullStore is a very simple class which pretends to be a #GtkTreeModel with one column of type %G_TYPE_UINT
+ * whose values are equal to row numbers (counted from 0).  In reality the column is purely virtual and the store
  * always takes up only a small constant amount of memory.
  *
- * The purpose of #GwyNullStore is to provide a low-overhead #GtkTreeModel
- * interface for array-like (and other indexed) data structures.
+ * The purpose of #GwyNullStore is to provide a low-overhead #GtkTreeModel interface for array-like (and other
+ * indexed) data structures.
  *
- * A new null store can be created with gwy_null_store_new(), then number of
- * virtual rows can be controlled with gwy_null_store_set_n_rows().  For
- * convenience, a method to emit "row-changed" signal on a row by its index is
+ * A new null store can be created with gwy_null_store_new(), then number of virtual rows can be controlled with
+ * gwy_null_store_set_n_rows().  For convenience, a method to emit "row-changed" signal on a row by its index is
  * provided: gwy_null_store_row_changed().
  *
- * Since null stores often serve as wrappers around other data structures,
- * convenience methods to attach and obtain such a data are provided:
- * gwy_null_store_set_model(), gwy_null_store_get_model().
+ * Since null stores often serve as wrappers around other data structures, convenience methods to attach and obtain
+ * such a data are provided: gwy_null_store_set_model(), gwy_null_store_get_model().
  *
  * A simple example to create a multiplication table with null storage:
  * <informalexample><programlisting>
@@ -524,8 +557,8 @@ gwy_null_store_iter_is_valid(GwyNullStore *store,
  * gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
  * </programlisting></informalexample>
  *
- * The cell data function multiply<!-- -->() just multiplies the column number
- * with the number of (virtual) null store row:
+ * The cell data function multiply<!-- -->() just multiplies the column number with the number of (virtual) null store
+ * row:
  * <informalexample><programlisting>
  * static void
  * multiply(GtkTreeViewColumn *column,
@@ -549,4 +582,4 @@ gwy_null_store_iter_is_valid(GwyNullStore *store,
  * </programlisting></informalexample>
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
