@@ -1,5 +1,5 @@
 /*
- *  $Id: gwystock.h 23527 2021-04-18 09:53:47Z yeti-dn $
+ *  $Id: gwystock.h 24795 2022-04-28 15:20:36Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -51,6 +51,7 @@ G_BEGIN_DECLS
 #define GWY_STOCK_DATA_MEASURE                   "gwy_data_measure"
 #define GWY_STOCK_DECONVOLVE                     "gwy_deconvolve"
 #define GWY_STOCK_DISCONNECTED                   "gwy_disconnected"
+#define GWY_STOCK_DISPLACEMENT_FIELD             "gwy_displacement_field"
 #define GWY_STOCK_DISTANCE                       "gwy_distance"
 #define GWY_STOCK_DISTANCE_TRANSFORM             "gwy_distance_transform"
 #define GWY_STOCK_DISTRIBUTION_ANGLE             "gwy_distribution_angle"
@@ -84,8 +85,10 @@ G_BEGIN_DECLS
 #define GWY_STOCK_GL_MATERIAL                    "gwy_gl_material"
 #define GWY_STOCK_GRADIENT_HORIZONTAL            "gwy_gradient_horizontal"
 #define GWY_STOCK_GRADIENT_VERTICAL              "gwy_gradient_vertical"
+#define GWY_STOCK_GRAIN_BOUNDING_BOX             "gwy_grain_bounding_box"
 #define GWY_STOCK_GRAIN_CORRELATION              "gwy_grain_correlation"
 #define GWY_STOCK_GRAIN_EXSCRIBED_CIRCLE         "gwy_grain_exscribed_circle"
+#define GWY_STOCK_GRAIN_INSCRIBED_BOX            "gwy_grain_inscribed_box"
 #define GWY_STOCK_GRAIN_INSCRIBED_CIRCLE         "gwy_grain_inscribed_circle"
 #define GWY_STOCK_GRAINS                         "gwy_grains"
 #define GWY_STOCK_GRAINS_EDGE                    "gwy_grains_edge"
@@ -158,6 +161,7 @@ G_BEGIN_DECLS
 #define GWY_STOCK_MASK_INVERT                    "gwy_mask_invert"
 #define GWY_STOCK_MASK_LINE                      "gwy_mask_line"
 #define GWY_STOCK_MASK_MORPH                     "gwy_mask_morph"
+#define GWY_STOCK_MASK_NOISIFY                   "gwy_mask_noisify"
 #define GWY_STOCK_MASK_PAINT_DRAW                "gwy_mask_paint_draw"
 #define GWY_STOCK_MASK_PAINT_ERASE               "gwy_mask_paint_erase"
 #define GWY_STOCK_MASK_RECT_EXCLUSIVE            "gwy_mask_rect_exclusive"
@@ -184,6 +188,7 @@ G_BEGIN_DECLS
 #define GWY_STOCK_NULL_OFFSETS                   "gwy_null_offsets"
 #define GWY_STOCK_PALETTES                       "gwy_palettes"
 #define GWY_STOCK_PATH_LEVEL                     "gwy_path_level"
+#define GWY_STOCK_PERSPECTIVE_DISTORT            "gwy_perspective_distort"
 #define GWY_STOCK_POINTER_MEASURE                "gwy_pointer_measure"
 #define GWY_STOCK_POLY_DISTORT                   "gwy_poly_distort"
 #define GWY_STOCK_POLYNOM                        "gwy_polynom"
@@ -270,6 +275,7 @@ G_BEGIN_DECLS
 #define GWY_STOCK_XYZIZE                         "gwy_xyzize"
 #define GWY_STOCK_ZERO_MAXIMUM                   "gwy_zero_maximum"
 #define GWY_STOCK_ZERO_MEAN                      "gwy_zero_mean"
+#define GWY_STOCK_ZERO_UNDER_MASK                "gwy_zero_under_mask"
 #define GWY_STOCK_ZOOM_1_1                       "gwy_zoom_1_1"
 #define GWY_STOCK_ZOOM_FIT                       "gwy_zoom_fit"
 #define GWY_STOCK_ZOOM_IN                        "gwy_zoom_in"
