@@ -1,5 +1,5 @@
 /*
- *  $Id: gwymodule-file.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: gwymodule-file.c 23207 2021-03-08 15:42:36Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -969,12 +969,7 @@ gwy_file_type_info_set(GwyContainer *data,
 
     fti = gwy_file_type_info_get(data, TRUE);
     fti->name = g_quark_from_string(name);
-    /* filename_sys can be *physically* the same string as fti->filename_sys */
-    if (fti->filename_sys == filename_sys)
-        return;
-    if (fti->filename_sys)
-        g_free(fti->filename_sys);
-    fti->filename_sys = g_strdup(filename_sys);
+    gwy_assign_string(&fti->filename_sys, filename_sys);
 }
 
 static FileTypeInfo*
