@@ -1,22 +1,18 @@
 /*
- *  $Id: gwymoduleloader.c 20726 2018-01-17 08:09:43Z yeti-dn $
- *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwymoduleloader.c 23946 2021-08-10 11:16:22Z yeti-dn $
+ *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -104,12 +100,10 @@ gwy_module_error_quark(void)
  *
  * Registers all modules in given directories.
  *
- * It can be called several times (on different directories).  No errors are
- * reported, register modules individually with gwy_module_register_module()
- * to get registration errors.
+ * It can be called several times (on different directories).  No errors are reported, register modules individually
+ * with gwy_module_register_module() to get registration errors.
  *
- * If you need to prevent specific modules from loading use
- * gwy_module_disable_registration() beforehand.
+ * If you need to prevent specific modules from loading use gwy_module_disable_registration() beforehand.
  **/
 void
 gwy_module_register_modules(const gchar **paths)
@@ -168,8 +162,7 @@ gwy_module_failure_foreach_one(G_GNUC_UNUSED const gchar *key,
  *
  * Runs @function for each module that failed to register.
  *
- * It passes the failure info (#GwyModuleFailureInfo) as the data argument.
- * It should be modified.
+ * It passes the failure info (#GwyModuleFailureInfo) as the data argument. It should be modified.
  *
  * Since: 2.49
  **/
@@ -184,8 +177,7 @@ gwy_module_failure_foreach(GFunc function, gpointer data)
     g_return_if_fail(function);
     fdata.func = function;
     fdata.data = data;
-    g_hash_table_foreach(failures,
-                         (GHFunc)&gwy_module_failure_foreach_one, &fdata);
+    g_hash_table_foreach(failures, (GHFunc)&gwy_module_failure_foreach_one, &fdata);
 }
 
 static void
@@ -203,8 +195,8 @@ gwy_module_foreach_one(const gchar *name,
  *
  * Runs @function on each registered module.
  *
- * It passes module name as the key and pointer to module
- * info (#GwyModuleInfo) as the value.  Neither should be modified.
+ * It passes module name as the key and pointer to module info (#GwyModuleInfo) as the value.  Neither should be
+ * modified.
  **/
 void
 gwy_module_foreach(GHFunc function,
@@ -249,8 +241,8 @@ gwy_module_get_filename(const gchar *name)
  *
  * Returns list of names of functions a module implements.
  *
- * Returns: List of module function names, as a #GSList that is owned by
- *          module loader and must not be modified or freed.
+ * Returns: List of module function names, as a #GSList that is owned by module loader and must not be modified or
+ *          freed.
  **/
 GSList*
 gwy_module_get_functions(const gchar *name)
@@ -271,13 +263,11 @@ gwy_module_get_functions(const gchar *name)
 /**
  * gwy_module_register_module:
  * @name: Module file name to load, including full path and extension.
- * @error: Location to store error, or %NULL to ignore them.  Errors from
- *         #GwyModuleError domain can occur.
+ * @error: Location to store error, or %NULL to ignore them.  Errors from #GwyModuleError domain can occur.
  *
  * Loads a single module.
  *
- * This function also works with bundles.  The returned module info is for the
- * bundle and thus not of much use.
+ * This function also works with bundles.  The returned module info is for the bundle and thus not of much use.
  *
  * Returns: Module info on success, %NULL on failure.
  **/
@@ -343,8 +333,7 @@ gwy_module_do_register_module(const gchar *filename,
     mod = g_module_open(filename, G_MODULE_BIND_LAZY);
 
     if (!mod) {
-        g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_OPEN,
-                    "Cannot open module: %s", g_module_error());
+        g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_OPEN, "Cannot open module: %s", g_module_error());
         gwy_module_register_fail(err, error, modname, filename);
         g_free(modname);
         return NULL;
@@ -354,10 +343,8 @@ gwy_module_do_register_module(const gchar *filename,
     /* Sanity checks on the module before registration is attempted. */
     ok = TRUE;
     currenly_registered_module = modname;
-    if (!g_module_symbol(mod, "_gwy_module_query", (gpointer)&query)
-        || !query) {
-        g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_QUERY,
-                    "Module contains no query function");
+    if (!g_module_symbol(mod, "_gwy_module_query", (gpointer)&query) || !query) {
+        g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_QUERY, "Module contains no query function");
         gwy_module_register_fail(err, error, modname, filename);
         ok = FALSE;
     }
@@ -365,16 +352,14 @@ gwy_module_do_register_module(const gchar *filename,
     if (ok) {
         mod_info = query();
         /* register_module_with_info() always consumes (or frees) modname */
-        ok = register_module_with_info(mods, mod_info, filename, modname, FALSE,
-                                       error);
+        ok = register_module_with_info(mods, mod_info, filename, modname, FALSE, error);
         if (ok) {
             gwy_debug("Making module `%s' resident.", filename);
             g_module_make_resident(mod);
         }
         else {
             if (!g_module_close(mod)) {
-                g_critical("Cannot unload module `%s': %s",
-                           filename, g_module_error());
+                g_critical("Cannot unload module `%s': %s", filename, g_module_error());
             }
         }
     }
@@ -401,8 +386,7 @@ register_module_bundle(GHashTable *mods, const GwyModuleInfo *bundle_info,
 
     if (!bundle_info->register_func) {
         g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_ABI,
-                    "Module bundle %s info has no registration function",
-                    bundlename);
+                    "Module bundle %s info has no registration function", bundlename);
         gwy_module_register_fail(err, error, bundlename, filename);
         return FALSE;
     }
@@ -411,8 +395,7 @@ register_module_bundle(GHashTable *mods, const GwyModuleInfo *bundle_info,
     records = register_bundle();
     if (!records) {
         g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_ABI,
-                    "Module bundle %s returned NULL module records",
-                    bundlename);
+                    "Module bundle %s returned NULL module records", bundlename);
         gwy_module_register_fail(err, error, bundlename, filename);
         return FALSE;
     }
@@ -434,8 +417,7 @@ register_module_bundle(GHashTable *mods, const GwyModuleInfo *bundle_info,
         mod_info = query();
         if (mod_info) {
             /* TODO: Must again check duplicit names, etc. */
-            if (register_module_with_info(mods, mod_info, filename,
-                                          modname, TRUE, NULL))
+            if (register_module_with_info(mods, mod_info, filename, modname, TRUE, NULL))
                 nok++;
         }
         else
@@ -449,8 +431,7 @@ register_module_bundle(GHashTable *mods, const GwyModuleInfo *bundle_info,
 
     /* FIXME: Could be report errors in more detail? */
     g_set_error(error, GWY_MODULE_ERROR, GWY_MODULE_ERROR_ABI,
-                "Module bundle %s did not successfully register any module",
-                bundlename);
+                "Module bundle %s did not successfully register any module", bundlename);
     return FALSE;
 }
 
@@ -465,8 +446,7 @@ register_module_with_info(GHashTable *mods, const GwyModuleInfo *mod_info,
     guint abi_version;
 
     if (!mod_info) {
-        g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_INFO,
-                    "Module info is NULL");
+        g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_INFO, "Module info is NULL");
         gwy_module_register_fail(err, error, modname, filename);
         ok = FALSE;
     }
@@ -477,16 +457,14 @@ register_module_with_info(GHashTable *mods, const GwyModuleInfo *mod_info,
             gwy_debug("bundle flag found on %s", filename);
             if (in_bundle) {
                 g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_NESTING,
-                            "Nested module bundles are insane "
-                            "and not supported.");
+                            "Nested module bundles are insane and not supported.");
                 gwy_module_register_fail(err, error, modname, filename);
                 g_free(modname);
                 return FALSE;
             }
             abi_version &= ~(GWY_MODULE_BUNDLE_FLAG);
             if (abi_version == GWY_MODULE_ABI_VERSION) {
-                ok = register_module_bundle(mods, mod_info,
-                                            filename, modname, error);
+                ok = register_module_bundle(mods, mod_info, filename, modname, error);
                 g_free(modname);
                 return ok;
             }
@@ -498,8 +476,7 @@ register_module_with_info(GHashTable *mods, const GwyModuleInfo *mod_info,
 
         if (!ok) {
             g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_ABI,
-                        "Module ABI version %d differs from %d",
-                        mod_info->abi_version, GWY_MODULE_ABI_VERSION);
+                        "Module ABI version %d differs from %d", mod_info->abi_version, GWY_MODULE_ABI_VERSION);
             gwy_module_register_fail(err, error, modname, filename);
         }
     }
@@ -512,8 +489,7 @@ register_module_with_info(GHashTable *mods, const GwyModuleInfo *mod_info,
               && mod_info->copyright && *mod_info->copyright
               && mod_info->date && *mod_info->date);
         if (!ok) {
-            g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_ABI,
-                        "Module info has missing/invalid fields");
+            g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_ABI, "Module info has missing/invalid fields");
             gwy_module_register_fail(err, error, modname, filename);
         }
     }
@@ -527,13 +503,11 @@ register_module_with_info(GHashTable *mods, const GwyModuleInfo *mod_info,
         iinfo->funcs = NULL;
         g_hash_table_insert(mods, (gpointer)iinfo->name, iinfo);
         if (!(ok = mod_info->register_func())) {
-            g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_REGISTER,
-                        "Module feature registration failed");
+            g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_REGISTER, "Module feature registration failed");
             gwy_module_register_fail(err, error, modname, filename);
         }
         if (ok && !iinfo->funcs) {
-            g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_REGISTER,
-                        "Module did not register any function");
+            g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_REGISTER, "Module did not register any function");
             gwy_module_register_fail(err, error, modname, filename);
             ok = FALSE;
         }
@@ -559,10 +533,8 @@ gwy_module_figure_out_name(const gchar *filename)
     for (i = 0; modname[i]; i++)
         modname[i] = g_ascii_tolower(modname[i]);
 
-    /* FIXME: On normal platforms module names have an extension, but if
-     * it doesn't, just get over it.  This can happen only with explicit
-     * gwy_module_register_module() as gwy_load_modules_in_dir() accepts
-     * only sane names. */
+    /* FIXME: On normal platforms module names have an extension, but if it doesn't, just get over it.  This can
+     * happen only with explicit gwy_module_register_module() as gwy_load_modules_in_dir() accepts only sane names. */
     if ((s = strchr(modname, '.')))
         *s = '\0';
 
@@ -575,18 +547,15 @@ gwy_module_check_module_name(const gchar *modname,
                              GError **error)
 {
     if (!modname || !*modname) {
-        g_set_error(error, GWY_MODULE_ERROR, GWY_MODULE_ERROR_NAME,
-                    "Module name is empty");
+        g_set_error(error, GWY_MODULE_ERROR, GWY_MODULE_ERROR_NAME, "Module name is empty");
         return FALSE;
     }
 
     if (!gwy_strisident(modname, "_-", NULL))
-        g_warning("Module name `%s' is not a valid identifier. "
-                  "It may be rejected in future.", modname);
+        g_warning("Module name `%s' is not a valid identifier. It may be rejected in future.", modname);
 
     if (g_hash_table_lookup(mods, modname)) {
-        g_set_error(error, GWY_MODULE_ERROR, GWY_MODULE_ERROR_DUPLICATE,
-                    "Module was already registered");
+        g_set_error(error, GWY_MODULE_ERROR, GWY_MODULE_ERROR_DUPLICATE, "Module was already registered");
         return FALSE;
     }
 
@@ -611,25 +580,19 @@ check_python_availability(void)
     gboolean ok = FALSE;
 
     gwy_clear(pythondir, sizeof(pythondir));
-    if (RegOpenKeyEx(HKEY_CURRENT_USER, TEXT(python_key), 0, KEY_READ,
-                     &reg_key) == ERROR_SUCCESS) {
-        if (RegQueryValueEx(reg_key, NULL, NULL, NULL, pythondir, &size) == ERROR_SUCCESS) {
+    if (RegOpenKeyEx(HKEY_CURRENT_USER, TEXT(python_key), 0, KEY_READ, &reg_key) == ERROR_SUCCESS) {
+        if (RegQueryValueEx(reg_key, NULL, NULL, NULL, pythondir, &size) == ERROR_SUCCESS)
             ok = TRUE;
-        }
         RegCloseKey(reg_key);
     }
-    if (!ok
-        && RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT(python_key), 0, KEY_READ,
-                        &reg_key) == ERROR_SUCCESS) {
-        if (RegQueryValueEx(reg_key, NULL, NULL, NULL,
-                            pythondir, &size) == ERROR_SUCCESS)
+    if (!ok && RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT(python_key), 0, KEY_READ, &reg_key) == ERROR_SUCCESS) {
+        if (RegQueryValueEx(reg_key, NULL, NULL, NULL, pythondir, &size) == ERROR_SUCCESS)
             ok = TRUE;
         RegCloseKey(reg_key);
     }
 
     if (!ok) {
-        g_message("Cannot get %s registry key, assuming no python 2.7.",
-                  python_key);
+        g_message("Cannot get %s registry key, assuming no python 2.7.", python_key);
         return FALSE;
     }
 #endif
@@ -703,8 +666,7 @@ gwy_module_pedantic_check(_GwyModuleInfoInternal *iinfo)
             p += 2;
 
             if (!g_str_has_prefix(p, "GwyLayer")) {
-                g_warning("Module `%s' registered layer function `%s' "
-                          "whose name has not the form `GwyLayerFoo'.",
+                g_warning("Module `%s' registered layer function `%s' whose name has not the form `GwyLayerFoo'.",
                           iinfo->name, p);
                 ok = FALSE;
             }
@@ -719,8 +681,7 @@ gwy_module_pedantic_check(_GwyModuleInfoInternal *iinfo)
             p += 2;
 
             if (!g_str_has_prefix(p, "GwyTool")) {
-                g_warning("Module `%s' registered tool function `%s' "
-                          "whose name has not the form `GwyToolFoo'.",
+                g_warning("Module `%s' registered tool function `%s' whose name has not the form `GwyToolFoo'.",
                           iinfo->name, p);
                 ok = FALSE;
             }
@@ -734,8 +695,7 @@ gwy_module_pedantic_check(_GwyModuleInfoInternal *iinfo)
         p += 2;
 
         if (!gwy_strequal(iinfo->name, p)) {
-            g_warning("Module `%s' registered only one function `%s' "
-                      "and its name differs from module name.  Usually, "
+            g_warning("Module `%s' registered only one function `%s' and its name differs from module name.  Usually, "
                       "these two names should be the same.",
                       iinfo->name, p);
             return FALSE;
@@ -760,13 +720,14 @@ gwy_module_get_rid_of(const gchar *modname)
         gboolean (*func)(const gchar*);
     }
     gro_funcs[] = {
-        { GWY_MODULE_PREFIX_PROC,   _gwy_process_func_remove },
-        { GWY_MODULE_PREFIX_FILE,   _gwy_file_func_remove },
-        { GWY_MODULE_PREFIX_GRAPH,  _gwy_graph_func_remove },
-        { GWY_MODULE_PREFIX_TOOL,   _gwy_tool_func_remove },
-        { GWY_MODULE_PREFIX_LAYER,  _gwy_layer_func_remove },
-        { GWY_MODULE_PREFIX_VOLUME, _gwy_volume_func_remove },
-        { GWY_MODULE_PREFIX_XYZ,    _gwy_xyz_func_remove },
+        { GWY_MODULE_PREFIX_PROC,   _gwy_process_func_remove, },
+        { GWY_MODULE_PREFIX_FILE,   _gwy_file_func_remove,    },
+        { GWY_MODULE_PREFIX_GRAPH,  _gwy_graph_func_remove,   },
+        { GWY_MODULE_PREFIX_TOOL,   _gwy_tool_func_remove,    },
+        { GWY_MODULE_PREFIX_LAYER,  _gwy_layer_func_remove,   },
+        { GWY_MODULE_PREFIX_VOLUME, _gwy_volume_func_remove,  },
+        { GWY_MODULE_PREFIX_XYZ,    _gwy_xyz_func_remove,     },
+        { GWY_MODULE_PREFIX_CMAP,   _gwy_cmap_func_remove,    },
     };
 
     _GwyModuleInfoInternal *iinfo;
@@ -802,11 +763,9 @@ gwy_module_get_rid_of(const gchar *modname)
 /**
  * gwy_module_init:
  *
- * Initializes the loadable module system, aborting if there's no support
- * for modules on the platform.
+ * Initializes the loadable module system, aborting if there's no support for modules on the platform.
  *
- * Must be called at most once.  It is automatically called on first
- * gwy_module_register_modules() call.
+ * Must be called at most once.  It is automatically called on first gwy_module_register_modules() call.
  **/
 static void
 gwy_module_init(void)
@@ -828,8 +787,7 @@ gwy_module_init(void)
  *
  * Returns information about one module.
  *
- * Returns: The module info, of %NULL if not found.  It must be considered
- *          constant and never modified or freed.
+ * Returns: The module info, of %NULL if not found.  It must be considered constant and never modified or freed.
  **/
 const GwyModuleInfo*
 gwy_module_lookup(const gchar *name)
@@ -849,10 +807,9 @@ gwy_module_lookup(const gchar *name)
  *
  * Prevents the registration of a module of given name.
  *
- * This function blocks future module registration using
- * gwy_module_register_modules().  Already loaded modules are unaffected.
- * The low-level module loading function gwy_module_register_module() always
- * attempts to load the module, even if blocked.
+ * This function blocks future module registration using gwy_module_register_modules().  Already loaded modules are
+ * unaffected. The low-level module loading function gwy_module_register_module() always attempts to load the module,
+ * even if blocked.
  *
  * Since: 2.48
  **/
@@ -865,8 +822,7 @@ gwy_module_disable_registration(const gchar *name)
     g_return_if_fail(name);
     blocked_modules = gwy_module_get_blocking_table(TRUE);
     quark = g_quark_from_string(name);
-    g_hash_table_insert(blocked_modules,
-                        GSIZE_TO_POINTER(quark), GINT_TO_POINTER(TRUE));
+    g_hash_table_insert(blocked_modules, GSIZE_TO_POINTER(quark), GINT_TO_POINTER(TRUE));
 }
 
 /**
@@ -875,8 +831,7 @@ gwy_module_disable_registration(const gchar *name)
  *
  * Unblocks the registration of a module of given name.
  *
- * This function influences future module registration.  Already loaded modules
- * are unaffected.
+ * This function influences future module registration.  Already loaded modules are unaffected.
  *
  * Since: 2.48
  **/
@@ -901,15 +856,13 @@ gwy_module_enable_registration(const gchar *name)
  *
  * Reports whether the registration of a module is enabled.
  *
- * If the registration of module @name was prevented using
- * gwy_module_disable_registration() and not subsequently re-enabled
- * using gwy_module_enabled_registration() this function returns %FALSE.
+ * If the registration of module @name was prevented using gwy_module_disable_registration() and not subsequently
+ * re-enabled using gwy_module_enabled_registration() this function returns %FALSE.
  *
- * The reported values only represents the current state of blocking.  A module
- * @name could have been loaded when it was not blocked.
+ * The reported values only represents the current state of blocking.  A module @name could have been loaded when it
+ * was not blocked.
  *
- * Returns: %TRUE if module @name can be registered; %FALSE when it is blocked
- *          from registration.
+ * Returns: %TRUE if module @name can be registered; %FALSE when it is blocked from registration.
  *
  * Since: 2.48
  **/
@@ -1000,52 +953,43 @@ gwy_module_get_blocking_table(gboolean do_create)
  *
  * The module query must be the ONLY exported symbol from a module.
  *
- * This macro does The Right Thing necessary to export module info in a way
- * Gwyddion understands it. Put %GWY_MODULE_QUERY with the module info
- * (#GwyModuleInfo) of your module as its argument on a line (with NO
- * semicolon after).
+ * This macro does The Right Thing necessary to export module info in a way Gwyddion understands it. Put
+ * %GWY_MODULE_QUERY with the module info (#GwyModuleInfo) of your module as its argument on a line (with NO semicolon
+ * after).
  *
- * If you write a module in C++ note the module query must have C linkage.
- * This is achieved by marking it <literal>extern "C"</literal>:
+ * If you write a module in C++ note the module query must have C linkage. This is achieved by marking it
+ * <literal>extern "C"</literal>:
  * |[
  * extern "C" {
  * GWY_MODULE_QUERY(module_info)
  * }
  * ]|
- * This has to be done manually in versions up to 2.24; since version 2.25
- * GWY_MODULE_QUERY() includes <literal>extern "C"</literal> automatically if
- * it is compiled using a C++ compiler.
+ * This has to be done manually in versions up to 2.24; since version 2.25 * GWY_MODULE_QUERY() includes
+ * <literal>extern "C"</literal> automatically if it is compiled using a C++ compiler.
  **/
 
 /**
  * GWY_MODULE_QUERY2:
  * @mod_info: The #GwyModuleInfo structure to return as module info.
- * @mod_name: Module name, given as C identifier corresponding to file name
- *            withough extension.
+ * @mod_name: Module name, given as C identifier corresponding to file name withough extension.
  *
  * The module query must be the ONLY exported symbol from a module.
  *
  * See %GWY_MODULE_QUERY for discussion.
  *
- * This macro is intended for modules that can be bundled together to one big
- * shared library for time and space optimization.  This mostly makes sense
- * only for modules included in Gwyddion itself as it contains hundreds of
+ * This macro is intended for modules that can be bundled together to one big shared library for time and space
+ * optimization.  This mostly makes sense only for modules included in Gwyddion itself as it contains hundreds of
  * modules.
  *
- * However, it safe to use in third party modules because if modules
- * are not bundled the macro behaves exactly as %GWY_MODULE_QUERY and the
- * @mod_name argument is just ignored.  The macro expansion differs only when
- * %GWY_MODULE_BUNDLING is defined when including the "gwymoduleloader.h"
- * header.
+ * However, it safe to use in third party modules because if modules are not bundled the macro behaves exactly as
+ * %GWY_MODULE_QUERY and the @mod_name argument is just ignored.  The macro expansion differs only when
+ * %GWY_MODULE_BUNDLING is defined when including the "gwymoduleloader.h" header.
  *
- * The module name @mod_name is given as a C identifier, not a string.  It
- * <emphasis>must</emphasis> be identical to the file name of the module if it
- * was installed standalone, with extension removed and with non-identifier
- * characters converted to underscores.  For instance module implemented in
- * "nt-mdt.c" and installed as "nt-mdt.so" or "nt-mdt.dll" (or with other
- * extensions, depending on the operating system) must pass
- * <literal>nt_mdt</literal>.  If it does not match the file name, linking will
- * fail with a confusing message when bundling is enabled.
+ * The module name @mod_name is given as a C identifier, not a string.  It <emphasis>must</emphasis> be identical to
+ * the file name of the module if it was installed standalone, with extension removed and with non-identifier
+ * characters converted to underscores.  For instance module implemented in "nt-mdt.c" and installed as "nt-mdt.so" or
+ * "nt-mdt.dll" (or with other extensions, depending on the operating system) must pass <literal>nt_mdt</literal>.  If
+ * it does not match the file name, linking will fail with a confusing message when bundling is enabled.
  *
  * Since: 2.49
  **/
@@ -1063,17 +1007,14 @@ gwy_module_get_blocking_table(gboolean do_create)
  *
  * Module registration function type.
  *
- * It actually runs particular featrue registration functions, like
- * gwy_file_func_register() and gwy_process_func_register().
+ * It actually runs particular featrue registration functions, like gwy_file_func_register() and
+ * gwy_process_func_register().
  *
- * If the module has a complex initialisation it may be safer to simply not
- * register any function but return %TRUE even if it fails to set up itself
- * correctly.  The module feature unregistration is somewhat crude and,
- * generally, unregisteration may lead to disaster when shared library
- * unloading has unexpected side effects.
+ * If the module has a complex initialisation it may be safer to simply not register any function but return %TRUE
+ * even if it fails to set up itself correctly.  The module feature unregistration is somewhat crude and, generally,
+ * unregisteration may lead to disaster when shared library unloading has unexpected side effects.
  *
- * Returns: Whether the registration succeeded.  When it returns %FALSE, the
- *          module and its features are unregistered.
+ * Returns: Whether the registration succeeded.  When it returns %FALSE, the module and its features are unregistered.
  **/
 
 /**
@@ -1081,9 +1022,8 @@ gwy_module_get_blocking_table(gboolean do_create)
  *
  * Module query function type.
  *
- * The module query function should be simply declared as
- * GWY_MODULE_QUERY(mod_info), where mod_info is module info struct for
- * the module.
+ * The module query function should be simply declared as GWY_MODULE_QUERY(mod_info), where mod_info is module info
+ * struct for the module.
  *
  * Returns: The module info struct.
  **/
@@ -1093,19 +1033,16 @@ gwy_module_get_blocking_table(gboolean do_create)
  *
  * Module bundle query function type.
  *
- * It returns an array of module records for all modules in the bundle,
- * terminated by {%NULL, %NULL}.
+ * It returns an array of module records for all modules in the bundle, terminated by {%NULL, %NULL}.
  *
  * Since: 2.49
  **/
 
 /**
  * GwyModuleInfo:
- * @abi_version: Gwyddion module ABI version, should be always
- *               #GWY_MODULE_ABI_VERSION.
- * @register_func: Module registration function (the function run by Gwyddion
- *                 module system, actually registering particular module
- *                 features).
+ * @abi_version: Gwyddion module ABI version, should be always #GWY_MODULE_ABI_VERSION.
+ * @register_func: Module registration function (the function run by Gwyddion module system, actually registering
+ * particular module features).
  * @blurb: Some module description.
  * @author: Module author(s).
  * @version: Module version.
@@ -1141,44 +1078,36 @@ gwy_module_get_blocking_table(gboolean do_create)
 /**
  * GwyRunType:
  * @GWY_RUN_NONE: None.
- * @GWY_RUN_NONINTERACTIVE: The function is run non-interactively, it must not
- *                          present any GUI and touch application state.
- * @GWY_RUN_INTERACTIVE: The function presents a modal user interfaces where
- *                       parameters can be adjusted, it returns after
- *                       finishing all operations.
- * @GWY_RUN_IMMEDIATE: Function is run immediately and uses parameter values
- *                     stored in the settings to reproduce previous run.
- *                     It can however present GUI error messages
- *                     or display progress, create new widndows, etc.
+ * @GWY_RUN_NONINTERACTIVE: The function is run non-interactively, it must not present any GUI and touch application
+ *                          state.
+ * @GWY_RUN_INTERACTIVE: The function presents a modal user interfaces where parameters can be adjusted, it returns
+ *                       after finishing all operations.
+ * @GWY_RUN_IMMEDIATE: Function is run immediately and uses parameter values stored in the settings to reproduce
+ *                     previous run. It can however present GUI error messages or display progress, create new
+ *                     windows, etc.
  * @GWY_RUN_MASK: The mask for all the run modes.
  *
  * Module function run-modes.
  *
- * Note @GWY_RUN_NONINTERACTIVE is only possible for file functions,
- * processing functions do not have a truly non-interactive interface yet and
- * therefore they should not claim they support it.  The closest match for
- * process functions is @GWY_RUN_IMMEDIATE.
+ * Note @GWY_RUN_NONINTERACTIVE is only possible for file functions, processing functions do not have a truly
+ * non-interactive interface yet and therefore they should not claim they support it.  The closest match for process
+ * functions is @GWY_RUN_IMMEDIATE.
  **/
 
 /**
  * GwyModuleError:
- * @GWY_MODULE_ERROR_NAME: Module has an invalid name.  It is recommended that
- *                         module names are valid C identifiers, possibly with
- *                         dashes instead of underscores, but only really
- *                         broken names are rejected.
- * @GWY_MODULE_ERROR_DUPLICATE: A module of the same name has already been
- *                              registered.
+ * @GWY_MODULE_ERROR_NAME: Module has an invalid name.  It is recommended that module names are valid C identifiers,
+ *                         possibly with dashes instead of underscores, but only really broken names are rejected.
+ * @GWY_MODULE_ERROR_DUPLICATE: A module of the same name has already been registered.
  * @GWY_MODULE_ERROR_OPEN: Calling g_module_open() on the module failed.
  * @GWY_MODULE_ERROR_QUERY: Module does not contain any query function.
- * @GWY_MODULE_ERROR_ABI: Module has different ABI version than
- *                        expected/supported; or required info field are
+ * @GWY_MODULE_ERROR_ABI: Module has different ABI version than expected/supported; or required info field are
  *                        missing.
  * @GWY_MODULE_ERROR_INFO: Module query function provided %NULL info.
- * @GWY_MODULE_ERROR_REGISTER: The registration function returned %FALSE; or
- *                             the module did not register any function.
+ * @GWY_MODULE_ERROR_REGISTER: The registration function returned %FALSE; or the module did not register any function.
  * @GWY_MODULE_ERROR_NESTING: Nested module bundle found.  (Since 2.49)
  *
  * Type of module loading and registration error.
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
