@@ -1,5 +1,5 @@
 /*
- *  $Id: arithmetic.c 23659 2021-05-10 17:29:09Z yeti-dn $
+ *  $Id: arithmetic.c 24347 2021-10-12 13:39:24Z rsleza $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -474,15 +474,9 @@ gwy_brick_check_compatibility(GwyBrick *brick1,
     g_return_val_if_fail(GWY_IS_BRICK(brick1), check);
     g_return_val_if_fail(GWY_IS_BRICK(brick2), check);
 
-    check_basic_properties(brick1->xres, brick2->xres,
-                           brick1->xreal, brick2->xreal,
-                           check, &result);
-    check_basic_properties(brick1->yres, brick2->yres,
-                           brick1->yreal, brick2->yreal,
-                           check, &result);
-    check_basic_properties(brick1->zres, brick2->zres,
-                           brick1->zreal, brick2->zreal,
-                           check, &result);
+    check_basic_properties(brick1->xres, brick2->xres, brick1->xreal, brick2->xreal, check, &result);
+    check_basic_properties(brick1->yres, brick2->yres, brick1->yreal, brick2->yreal, check, &result);
+    check_basic_properties(brick1->zres, brick2->zres, brick1->zreal, brick2->zreal, check, &result);
 
     if ((check & GWY_DATA_COMPATIBILITY_LATERAL)
         && (!units_are_equal(brick1->si_unit_x, brick2->si_unit_x)
@@ -490,8 +484,7 @@ gwy_brick_check_compatibility(GwyBrick *brick1,
             || !units_are_equal(brick1->si_unit_z, brick2->si_unit_z)))
         result |= GWY_DATA_COMPATIBILITY_LATERAL;
 
-    if (check & GWY_DATA_COMPATIBILITY_VALUE
-        && !units_are_equal(brick1->si_unit_w, brick2->si_unit_w))
+    if (check & GWY_DATA_COMPATIBILITY_VALUE && !units_are_equal(brick1->si_unit_w, brick2->si_unit_w))
         result |= GWY_DATA_COMPATIBILITY_VALUE;
 
     /* Z-calibration. */
@@ -507,6 +500,75 @@ gwy_brick_check_compatibility(GwyBrick *brick1,
     return result;
 }
 
+/**
+ * gwy_lawn_check_compatibility:
+ * @lawn1: A data lawn.
+ * @lawn2: Another data lawn.
+ * @check: The compatibility tests to perform.
+ *
+ * Checks whether two data lawns are compatible.
+ *
+ * Dimensions are checked only in the plane.  To check if the curve lengths match, use the
+ * %GWY_DATA_COMPATIBILITY_CURVELEN flag.  Use %GWY_DATA_COMPATIBILITY_NCURVES to check if the two lawns have the
+ * same number of curves.
+ *
+ * Returns: Zero if all tested properties are compatible.  Flags corresponding to failed tests if lawns are not
+ *          compatible.
+ *
+ * Since: 2.60
+ **/
+GwyDataCompatibilityFlags
+gwy_lawn_check_compatibility(GwyLawn *lawn1,
+                             GwyLawn *lawn2,
+                             GwyDataCompatibilityFlags check)
+{
+    GwyDataCompatibilityFlags result = 0;
+
+    g_return_val_if_fail(GWY_IS_LAWN(lawn1), check);
+    g_return_val_if_fail(GWY_IS_LAWN(lawn2), check);
+
+    check_basic_properties(lawn1->xres, lawn2->xres, lawn1->xreal, lawn2->xreal, check, &result);
+    check_basic_properties(lawn1->yres, lawn2->yres, lawn1->yreal, lawn2->yreal, check, &result);
+
+    if ((check & GWY_DATA_COMPATIBILITY_LATERAL) && !units_are_equal(lawn1->si_unit_xy, lawn2->si_unit_xy))
+        result |= GWY_DATA_COMPATIBILITY_LATERAL;
+
+    if (check & GWY_DATA_COMPATIBILITY_NCURVES && gwy_lawn_get_n_curves(lawn1) != gwy_lawn_get_n_curves(lawn2))
+        result |= GWY_DATA_COMPATIBILITY_NCURVES;
+
+    if (check & GWY_DATA_COMPATIBILITY_VALUE) {
+        if (gwy_lawn_get_n_curves(lawn1) != gwy_lawn_get_n_curves(lawn2))
+            result |= GWY_DATA_COMPATIBILITY_VALUE;
+        else {
+            gint i, n = gwy_lawn_get_n_curves(lawn1);
+
+            for (i = 0; i < n; i++) {
+                if (!units_are_equal(gwy_lawn_get_si_unit_curve(lawn1, i), gwy_lawn_get_si_unit_curve(lawn2, i))) {
+                    result |= GWY_DATA_COMPATIBILITY_VALUE;
+                    break;
+                }
+            }
+        }
+    }
+
+    if (check & GWY_DATA_COMPATIBILITY_CURVELEN) {
+        if (lawn1->xres != lawn2->xres || lawn1->yres != lawn2->yres)
+            result |= GWY_DATA_COMPATIBILITY_CURVELEN;
+        else {
+            gint i, xres = lawn1->xres, yres = lawn1->yres;
+            for (i = 0; i < xres*yres; i++) {
+                if (gwy_lawn_get_curve_length(lawn1, i % xres, i/xres)
+                    != gwy_lawn_get_curve_length(lawn2, i % xres, i/xres)) {
+                    result |= GWY_DATA_COMPATIBILITY_CURVELEN;
+                    break;
+                }
+            }
+        }
+    }
+
+    return result;
+}
+
 /**
  * gwy_data_field_check_compatibility_with_brick_xy:
  * @data_field: A two-dimensional data field.
@@ -589,9 +651,8 @@ gwy_data_line_check_compatibility_with_brick_z(GwyDataLine *data_line,
         return result;
     }
 
-    /* With Z-calibration we compare to @zcal.  The *values* of @zcal are the
-     * same thing as *coordinates* of @data_line.  So this is a mess and
-     * possibly not useful at all. */
+    /* With Z-calibration we compare to @zcal.  The *values* of @zcal are the same thing as *coordinates* of
+     * @data_line.  So this is a mess and possibly not useful at all. */
     g_assert(zcal->res == brick->zres);
     check_basic_properties(data_line->res, zcal->res, data_line->real, zcal->data[zcal->res-1] - zcal->data[0],
                            check, &result);
@@ -602,6 +663,38 @@ gwy_data_line_check_compatibility_with_brick_z(GwyDataLine *data_line,
     return result;
 }
 
+/**
+ * gwy_data_field_check_compatibility_with_lawn_xy:
+ * @data_field: A two-dimensional data field.
+ * @lawn: A lawn curve map object.
+ * @check: The compatibility tests to perform.
+ *
+ * Checks whether a data field is compatible with lawn in the XY-plane.
+ *
+ * Returns: Zero if all tested properties are compatible.  Flags corresponding to failed tests if the data objects are
+ *          not compatible.
+ *
+ * Since: 2.60
+ **/
+GwyDataCompatibilityFlags
+gwy_data_field_check_compatibility_with_lawn(GwyDataField *data_field,
+                                             GwyLawn *lawn,
+                                             GwyDataCompatibilityFlags check)
+{
+    GwyDataCompatibilityFlags result = 0;
+
+    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), check);
+    g_return_val_if_fail(GWY_IS_LAWN(lawn), check);
+
+    check_basic_properties(data_field->xres, lawn->xres, data_field->xreal, lawn->xreal, check, &result);
+    check_basic_properties(data_field->yres, lawn->yres, data_field->yreal, lawn->yreal, check, &result);
+
+    if ((check & GWY_DATA_COMPATIBILITY_LATERAL) && !units_are_equal(data_field->si_unit_xy, lawn->si_unit_xy))
+        result |= GWY_DATA_COMPATIBILITY_LATERAL;
+
+    return result;
+}
+
 static inline void
 fill_block(gdouble *data, guint len, gdouble value)
 {
