@@ -1,5 +1,5 @@
 /*
- *  $Id: correlation.c 24420 2021-10-24 11:17:25Z yeti-dn $
+ *  $Id: correlation.c 24832 2022-05-19 13:43:16Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -887,8 +887,7 @@ gwy_data_field_crosscorrelate_set_weights(GwyComputationState *state,
     g_return_if_fail(cstate->cs.state == GWY_COMPUTATION_STATE_INIT);
 
     gwy_data_field_fill(cstate->weights, 1.0);
-    gwy_fft_window_data_field(cstate->weights, GWY_ORIENTATION_HORIZONTAL, type);
-    gwy_fft_window_data_field(cstate->weights, GWY_ORIENTATION_VERTICAL, type);
+    gwy_data_field_fft_window(cstate->weights, type);
 }
 
 /**
