@@ -1,5 +1,5 @@
 /*
- *  $Id: dataline.c 24757 2022-04-06 11:41:24Z yeti-dn $
+ *  $Id: dataline.c 24952 2022-08-26 15:24:56Z yeti-dn $
  *  Copyright (C) 2003-2020 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -420,8 +420,7 @@ void
 gwy_data_line_resize(GwyDataLine *a, gint from, gint to)
 {
     g_return_if_fail(GWY_IS_DATA_LINE(a));
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
     g_return_if_fail(from >= 0 && to <= a->res);
     a->real *= (to - from)/((double)a->res);
     a->res = to - from;
@@ -1225,8 +1224,7 @@ gwy_data_line_part_fill(GwyDataLine *data_line,
     gint i;
 
     g_return_if_fail(GWY_IS_DATA_LINE(data_line));
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_if_fail(from >= 0 && to <= data_line->res);
 
@@ -1247,8 +1245,7 @@ gwy_data_line_part_clear(GwyDataLine *data_line,
                          gint from, gint to)
 {
     g_return_if_fail(GWY_IS_DATA_LINE(data_line));
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_if_fail(from >= 0 && to <= data_line->res);
 
@@ -1272,8 +1269,7 @@ gwy_data_line_part_add(GwyDataLine *data_line,
     gint i;
 
     g_return_if_fail(GWY_IS_DATA_LINE(data_line));
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_if_fail(from >= 0 && to <= data_line->res);
 
@@ -1298,8 +1294,7 @@ gwy_data_line_part_multiply(GwyDataLine *data_line,
     gint i;
 
     g_return_if_fail(GWY_IS_DATA_LINE(data_line));
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_if_fail(from >= 0 && to <= data_line->res);
 
@@ -1362,8 +1357,7 @@ gwy_data_line_part_threshold(GwyDataLine *a,
     gint i, tot = 0;
 
     g_return_val_if_fail(GWY_IS_DATA_LINE(a), 0);
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_val_if_fail(from >= 0 && to <= a->res, 0);
 
@@ -1593,9 +1587,7 @@ gwy_data_line_part_fit_polynom(GwyDataLine *data_line,
     g_return_val_if_fail(GWY_IS_DATA_LINE(data_line), NULL);
     g_return_val_if_fail(n >= 0, NULL);
     data = data_line->data;
-
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     sumx = g_new0(gdouble, 2*n+1);
     if (!coeffs)
@@ -1689,9 +1681,7 @@ gwy_data_line_part_subtract_polynom(GwyDataLine *data_line,
     g_return_if_fail(GWY_IS_DATA_LINE(data_line));
     g_return_if_fail(coeffs);
     g_return_if_fail(n >= 0);
-
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     for (i = from; i < to; i++) {
         val = 0.0;
