@@ -1,5 +1,5 @@
 /*
- *  $Id: dataline.h 22791 2020-05-03 08:55:57Z yeti-dn $
+ *  $Id: dataline.h 24757 2022-04-06 11:41:24Z yeti-dn $
  *  Copyright (C) 2003-2020 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -68,20 +68,19 @@ struct _GwyDataLineClass {
                                          G_OBJECT(dest), \
                                          GWY_TYPE_DATA_LINE)
 
-GType  gwy_data_line_get_type  (void) G_GNUC_CONST;
-
-GwyDataLine*   gwy_data_line_new                   (gint res,
-                                                    gdouble real,
-                                                    gboolean nullme);
-GwyDataLine*   gwy_data_line_new_alike             (GwyDataLine *model,
-                                                    gboolean nullme);
-void           gwy_data_line_data_changed          (GwyDataLine *data_line);
-GwyDataLine*   gwy_data_line_new_resampled (GwyDataLine *data_line,
-                                            gint res,
-                                            GwyInterpolationType interpolation);
-void           gwy_data_line_resample      (GwyDataLine *data_line,
-                                            gint res,
-                                            GwyInterpolationType interpolation);
+GType             gwy_data_line_get_type          (void)                                G_GNUC_CONST;
+GwyDataLine*      gwy_data_line_new               (gint res,
+                                                   gdouble real,
+                                                   gboolean nullme);
+GwyDataLine*      gwy_data_line_new_alike         (GwyDataLine *model,
+                                                   gboolean nullme);
+void              gwy_data_line_data_changed      (GwyDataLine *data_line);
+GwyDataLine*      gwy_data_line_new_resampled     (GwyDataLine *data_line,
+                                                   gint res,
+                                                   GwyInterpolationType interpolation);
+void              gwy_data_line_resample          (GwyDataLine *data_line,
+                                                   gint res,
+                                                   GwyInterpolationType interpolation);
 void              gwy_data_line_resize            (GwyDataLine *data_line,
                                                    gint from,
                                                    gint to);
@@ -163,6 +162,12 @@ void              gwy_data_line_subtract_lines    (GwyDataLine *result,
 void              gwy_data_line_multiply_lines    (GwyDataLine *result,
                                                    GwyDataLine *operand1,
                                                    GwyDataLine *operand2);
+void              gwy_data_line_linear_combination(GwyDataLine *result,
+                                                   gdouble coeff1,
+                                                   GwyDataLine *operand1,
+                                                   gdouble coeff2,
+                                                   GwyDataLine *operand2,
+                                                   gdouble constant);
 gint              gwy_data_line_threshold         (GwyDataLine *data_line,
                                                    gdouble threshval,
                                                    gdouble bottom,
