@@ -1,22 +1,18 @@
 /*
- *  $Id: filters.c 23661 2021-05-10 20:26:51Z yeti-dn $
- *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
+ *  $Id: filters.c 24518 2021-11-12 15:43:28Z yeti-dn $
+ *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <string.h>
@@ -68,8 +64,7 @@ gwy_data_field_normalize(GwyDataField *data_field)
         *p = (*p - min)/max;
 
     /* We can transform some stats */
-    data_field->cached &= CBIT(MIN) | CBIT(MAX) | CBIT(SUM) | CBIT(RMS)
-                          | CBIT(MED) | CBIT(ARF) | CBIT(ART);
+    data_field->cached &= CBIT(MIN) | CBIT(MAX) | CBIT(SUM) | CBIT(RMS) | CBIT(MED) | CBIT(ARF) | CBIT(ART);
     CVAL(data_field, MIN) = 0.0;
     CVAL(data_field, MAX) = 1.0;
     CVAL(data_field, SUM) /= (CVAL(data_field, SUM) - xres*yres*min)/max;
@@ -77,8 +72,7 @@ gwy_data_field_normalize(GwyDataField *data_field)
     CVAL(data_field, MED) = (CVAL(data_field, MED) - min)/max;
     CVAL(data_field, ART) = (CVAL(data_field, ART) - min)/max;
     CVAL(data_field, ARF) = (CVAL(data_field, ARF) - min)/max;
-    /* There is a transformation formula for MSQ, but it can be prone to ugly
-     * cancellation errors. */
+    /* There is a transformation formula for MSQ, but it can be prone to ugly cancellation errors. */
 }
 
 /**
@@ -89,13 +83,11 @@ gwy_data_field_normalize(GwyDataField *data_field)
  *
  * Transforms data in a data field with linear function to given range.
  *
- * When @range is positive, the new data range is (@offset, @offset+@range);
- * when @range is negative, the new data range is (@offset-@range, @offset).
- * In neither case the data are flipped, negative range only means different
+ * When @range is positive, the new data range is (@offset, @offset+@range); when @range is negative, the new data
+ * range is (@offset-@range, @offset). In neither case the data are flipped, negative range only means different
  * selection of boundaries.
  *
- * When @range is zero, this method is equivalent to
- * gwy_data_field_fill(@data_field, @offset).
+ * When @range is zero, this method is equivalent to gwy_data_field_fill(@data_field, @offset).
  **/
 void
 gwy_data_field_renormalize(GwyDataField *data_field,
@@ -119,8 +111,7 @@ gwy_data_field_renormalize(GwyDataField *data_field,
         return;
     }
 
-    if ((range > 0 && min == offset && min + range == max)
-        || (range < 0 && max == offset && min - range == max))
+    if ((range > 0 && min == offset && min + range == max) || (range < 0 && max == offset && min - range == max))
         return;
 
     /* The general case */
@@ -140,18 +131,15 @@ gwy_data_field_renormalize(GwyDataField *data_field,
             d[i] = (d[i] - min)/max*range + offset;
 
         /* We can transform stats */
-        data_field->cached &= CBIT(MIN) | CBIT(MAX) | CBIT(SUM) | CBIT(RMS)
-                              | CBIT(MED);
+        data_field->cached &= CBIT(MIN) | CBIT(MAX) | CBIT(SUM) | CBIT(RMS) | CBIT(MED);
         CVAL(data_field, MIN) = offset;
         CVAL(data_field, MAX) = offset + range;
         v = CVAL(data_field, SUM);
         CVAL(data_field, SUM) = (v - n*min)/max*range + offset*n;
         CVAL(data_field, RMS) = CVAL(data_field, RMS)/max*range;
-        CVAL(data_field, MED) = (CVAL(data_field, MED) - min)/max*range
-                                + offset;
+        CVAL(data_field, MED) = (CVAL(data_field, MED) - min)/max*range + offset;
         /* FIXME: we can recompute ARF and ART too */
-        /* There is a transformation formula for MSQ, but it can be prone to
-         * ugly cancellation errors. */
+        /* There is a transformation formula for MSQ, but it can be prone to ugly cancellation errors. */
     }
     else {
         min = max - min;
@@ -164,19 +152,15 @@ gwy_data_field_renormalize(GwyDataField *data_field,
             d[i] = (max - d[i])/min*range + offset;
 
         /* We can transform stats */
-        data_field->cached &= CBIT(MIN) | CBIT(MAX) | CBIT(SUM) | CBIT(RMS)
-                              | CBIT(MED);
+        data_field->cached &= CBIT(MIN) | CBIT(MAX) | CBIT(SUM) | CBIT(RMS) | CBIT(MED);
         CVAL(data_field, MIN) = offset + range;
         CVAL(data_field, MAX) = offset;
         v = CVAL(data_field, SUM);
-        CVAL(data_field, SUM) = (xres*yres*max - v)/min*range
-                                + offset*xres*yres;
+        CVAL(data_field, SUM) = (xres*yres*max - v)/min*range + offset*xres*yres;
         CVAL(data_field, RMS) = CVAL(data_field, RMS)/min*(-range);
-        CVAL(data_field, MED) = (max - CVAL(data_field, MED))/min*range
-                                + offset;
+        CVAL(data_field, MED) = (max - CVAL(data_field, MED))/min*range + offset;
         /* FIXME: we can recompute ARF and ART too */
-        /* There is a transformation formula for MSQ, but it can be prone to
-         * ugly cancellation errors. */
+        /* There is a transformation formula for MSQ, but it can be prone to ugly cancellation errors. */
     }
 }
 
@@ -190,25 +174,20 @@ gwy_data_field_renormalize(GwyDataField *data_field,
  * @range: New data interval size.
  * @offset: New data interval offset.
  *
- * Transforms data in a part of a data field with linear function to given
- * range.
+ * Transforms data in a part of a data field with linear function to given range.
  *
- * When @range is positive, the new data range is (@offset, @offset+@range);
- * when @range is negative, the new data range is (@offset-@range, @offset).
- * In neither case the data are flipped, negative range only means different
+ * When @range is positive, the new data range is (@offset, @offset+@range); when @range is negative, the new data
+ * range is (@offset-@range, @offset). In neither case the data are flipped, negative range only means different
  * selection of boundaries.
  *
- * When @range is zero, this method is equivalent to
- * gwy_data_field_fill(@data_field, @offset).
+ * When @range is zero, this method is equivalent to gwy_data_field_fill(@data_field, @offset).
  *
  * Since: 2.45
  **/
 void
 gwy_data_field_area_renormalize(GwyDataField *dfield,
-                                gint col,
-                                gint row,
-                                gint width,
-                                gint height,
+                                gint col, gint row,
+                                gint width, gint height,
                                 gdouble range,
                                 gdouble offset)
 {
@@ -231,16 +210,13 @@ gwy_data_field_area_renormalize(GwyDataField *dfield,
         return;
     }
 
-    gwy_data_field_area_get_min_max_mask(dfield, NULL, GWY_MASK_IGNORE,
-                                         col, row, width, height,
-                                         &min, &max);
+    gwy_data_field_area_get_min_max_mask(dfield, NULL, GWY_MASK_IGNORE, col, row, width, height, &min, &max);
     if (min == max) {
         gwy_data_field_area_fill(dfield, col, row, width, height, offset);
         return;
     }
 
-    if ((range > 0 && min == offset && min + range == max)
-        || (range < 0 && max == offset && min - range == max))
+    if ((range > 0 && min == offset && min + range == max) || (range < 0 && max == offset && min - range == max))
         return;
 
     /* The general case */
@@ -284,8 +260,8 @@ gwy_data_field_area_renormalize(GwyDataField *dfield,
  *
  * Tresholds values of a data field.
  *
- * Values smaller than @threshold are set to value @bottom, values higher
- * than @threshold or equal to it are set to value @top
+ * Values smaller than @threshold are set to value @bottom, values higher than @threshold or equal to it are set to
+ * value @top
  *
  * Returns: The total number of values above threshold.
  **/
@@ -316,8 +292,7 @@ gwy_data_field_threshold(GwyDataField *data_field,
     }
 
     /* We can precompute stats */
-    data_field->cached = CBIT(MIN) | CBIT(MAX) | CBIT(SUM) | CBIT(RMS)
-                         | CBIT(MED);
+    data_field->cached = CBIT(MIN) | CBIT(MAX) | CBIT(SUM) | CBIT(RMS) | CBIT(MED);
     if (tot == n)
         CVAL(data_field, MIN) = CVAL(data_field, MAX) = top;
     else if (tot == 0)
@@ -327,8 +302,7 @@ gwy_data_field_threshold(GwyDataField *data_field,
         CVAL(data_field, MAX) = MAX(top, bottom);
     }
     CVAL(data_field, SUM) = tot*top + (n - tot)*bottom;
-    CVAL(data_field, RMS) = (top - bottom)*(top - bottom)
-                            * tot/(gdouble)n * (n - tot)/(gdouble)n;
+    CVAL(data_field, RMS) = (top - bottom)*(top - bottom) * tot/(gdouble)n * (n - tot)/(gdouble)n;
     /* FIXME: may be incorrect for tot == n/2(?) */
     CVAL(data_field, MED) = tot > n/2 ? top : bottom;
 
@@ -349,8 +323,8 @@ gwy_data_field_threshold(GwyDataField *data_field,
  *
  * Tresholds values of a rectangular part of a data field.
  *
- * Values smaller than @threshold are set to value @bottom, values higher
- * than @threshold or equal to it are set to value @top
+ * Values smaller than @threshold are set to value @bottom, values higher than @threshold or equal to it are set to
+ * value @top
  *
  * Returns: The total number of values above threshold.
  **/
@@ -398,8 +372,7 @@ gwy_data_field_area_threshold(GwyDataField *data_field,
  *
  * Limits data field values to a range.
  *
- * Returns: The number of changed values, i.e., values that were outside
- *          [@bottom, @top].
+ * Returns: The number of changed values, i.e., values that were outside [@bottom, @top].
  **/
 gint
 gwy_data_field_clamp(GwyDataField *data_field,
@@ -433,8 +406,7 @@ gwy_data_field_clamp(GwyDataField *data_field,
         data_field->cached &= CBIT(MIN) | CBIT(MAX) | CBIT(MED);
         CVAL(data_field, MIN) = MAX(bottom, CVAL(data_field, MIN));
         CVAL(data_field, MAX) = MIN(top, CVAL(data_field, MAX));
-        if (CTEST(data_field, MED)
-            && (CVAL(data_field, MED) < bottom || CVAL(data_field, MED) > top))
+        if (CTEST(data_field, MED) && (CVAL(data_field, MED) < bottom || CVAL(data_field, MED) > top))
             data_field->cached &= ~CBIT(MED);
     }
 
@@ -453,8 +425,7 @@ gwy_data_field_clamp(GwyDataField *data_field,
  *
  * Limits values in a rectangular part of a data field to a range.
  *
- * Returns: The number of changed values, i.e., values that were outside
- *          [@bottom, @top].
+ * Returns: The number of changed values, i.e., values that were outside [@bottom, @top].
  **/
 gint
 gwy_data_field_area_clamp(GwyDataField *data_field,
@@ -489,7 +460,6 @@ gwy_data_field_area_clamp(GwyDataField *data_field,
                 tot++;
             }
         }
-
     }
     if (tot)
         gwy_data_field_invalidate(data_field);
@@ -501,37 +471,30 @@ gwy_data_field_area_clamp(GwyDataField *data_field,
  * gwy_data_field_area_gather:
  * @data_field: A data field.
  * @result: A data field to put the result to, it may be @data_field itself.
- * @buffer: A data field to use as a scratch area, its size must be at least
- *          @width*@height.  May be %NULL to allocate a private temporary
- *          buffer.
+ * @buffer: A data field to use as a scratch area, its size must be at least @width*@height.  May be %NULL to allocate
+ *          a private temporary buffer.
  * @col: Upper-left column coordinate.
  * @row: Upper-left row coordinate.
  * @width: Area width (number of columns).
  * @height: Area height (number of rows).
- * @hsize: Horizontal size of gathered area.  The area is centered around
- *         each sample if @hsize is odd, it extends one pixel more to the
- *         right if @hsize is even.
- * @vsize: Vertical size of gathered area.  The area is centered around
- *         each sample if @vsize is odd, it extends one pixel more down
- *         if @vsize is even.
- * @average: %TRUE to divide resulting sums by the number of involved samples
- *           to get averages instead of sums.
+ * @hsize: Horizontal size of gathered area.  The area is centered around each sample if @hsize is odd, it extends one
+ *         pixel more to the right if @hsize is even.
+ * @vsize: Vertical size of gathered area.  The area is centered around each sample if @vsize is odd, it extends one
+ *         pixel more down if @vsize is even.
+ * @average: %TRUE to divide resulting sums by the number of involved samples to get averages instead of sums.
  *
- * Sums or averages values in reactangular areas around each sample in a data
- * field.
+ * Sums or averages values in reactangular areas around each sample in a data field.
  *
- * When the gathered area extends out of calculation area, only samples from
- * their intersection are taken into the local sum (or average).
+ * When the gathered area extends out of calculation area, only samples from their intersection are taken into the
+ * local sum (or average).
  *
- * There are no restrictions on values of @hsize and @vsize with regard to
- * @width and @height, but they have to be positive.
+ * There are no restrictions on values of @hsize and @vsize with regard to @width and @height, but they have to be
+ * positive.
  *
- * The result is calculated by the means of two-dimensional rolling sums.
- * One one hand it means the calculation time depends linearly on
- * (@width + @hsize)*(@height + @vsize) instead of
- * @width*@hsize*@height*@vsize.  On the other hand it means absolute rounding
- * errors of all output values are given by the largest input values, that is
- * relative precision of results small in absolute value may be poor.
+ * The result is calculated by the means of two-dimensional rolling sums. One one hand it means the calculation time
+ * depends linearly on (@width + @hsize)*(@height + @vsize) instead of @width*@hsize*@height*@vsize.  On the other
+ * hand it means absolute rounding errors of all output values are given by the largest input values, that is relative
+ * precision of results small in absolute value may be poor.
  **/
 void
 gwy_data_field_area_gather(GwyDataField *data_field,
@@ -564,16 +527,14 @@ gwy_data_field_area_gather(GwyDataField *data_field,
     else
         buffer = gwy_data_field_new(width, height, 1.0, 1.0, FALSE);
 
-    /* Extension to the left and to the right (for asymmetric sizes extend
-     * to the right more) */
+    /* Extension to the left and to the right (for asymmetric sizes extend to the right more) */
     hs2m = (hsize - 1)/2;
     hs2p = hsize/2;
     vs2m = (vsize - 1)/2;
     vs2p = vsize/2;
 
     /* Row-wise sums */
-    /* FIXME: This is inefficient, split the inner loops to explicitly
-     * according to the conditions inside */
+    /* FIXME: This is inefficient, split the inner loops to explicitly according to the conditions inside */
     for (i = 0; i < height; i++) {
         srow = data_field->data + (i + row)*xres + col;
         drow = buffer->data + i*width;
@@ -584,8 +545,7 @@ gwy_data_field_area_gather(GwyDataField *data_field,
         for (j = 0; j <= m; j++)
             drow[0] += srow[j];
         for (j = 1; j < width/2; j++) {
-            v = ((j + hs2p < width ? srow[j + hs2p] : 0.0)
-                 - (j-1 - hs2m >= 0 ? srow[j-1 - hs2m] : 0.0));
+            v = ((j + hs2p < width ? srow[j + hs2p] : 0.0) - (j-1 - hs2m >= 0 ? srow[j-1 - hs2m] : 0.0));
             drow[j] = drow[j-1] + v;
         }
 
@@ -595,8 +555,7 @@ gwy_data_field_area_gather(GwyDataField *data_field,
         for (j = width-1; j >= m; j--)
             drow[width-1] += srow[j];
         for (j = width-2; j >= width/2; j--) {
-            v = ((j - hs2m >= 0 ? srow[j - hs2m] : 0.0)
-                 - (j+1 + hs2p < width ? srow[j+1 + hs2p] : 0.0));
+            v = ((j - hs2m >= 0 ? srow[j - hs2m] : 0.0) - (j+1 + hs2p < width ? srow[j+1 + hs2p] : 0.0));
             drow[j] = drow[j+1] + v;
         }
     }
@@ -729,9 +688,8 @@ gwy_data_field_area_gather(GwyDataField *data_field,
  *
  * Filters a rectangular part of a data field with mean filter of size @size.
  *
- * This method is a simple gwy_data_field_area_gather() wrapper, so the kernel
- * is square.  Use convolution gwy_data_field_area_ext_convolve() to perform a
- * mean filter with different, for instance circular, kernel.
+ * This method is a simple gwy_data_field_area_gather() wrapper, so the kernel is square.  Use convolution
+ * gwy_data_field_area_ext_convolve() to perform a mean filter with different, for instance circular, kernel.
  **/
 void
 gwy_data_field_area_filter_mean(GwyDataField *data_field,
@@ -739,9 +697,7 @@ gwy_data_field_area_filter_mean(GwyDataField *data_field,
                                 gint col, gint row,
                                 gint width, gint height)
 {
-    gwy_data_field_area_gather(data_field, data_field, NULL,
-                               size, size, TRUE,
-                               col, row, width, height);
+    gwy_data_field_area_gather(data_field, data_field, NULL, size, size, TRUE, col, row, width, height);
 }
 
 /**
@@ -751,17 +707,15 @@ gwy_data_field_area_filter_mean(GwyDataField *data_field,
  *
  * Filters a data field with mean filter of size @size.
  *
- * This method is a simple gwy_data_field_area_gather() wrapper, so the kernel
- * is square.  Use convolution gwy_data_field_area_ext_convolve() to perform a
- * mean filter with different, for instance circular, kernel.
+ * This method is a simple gwy_data_field_area_gather() wrapper, so the kernel is square.  Use convolution
+ * gwy_data_field_area_ext_convolve() to perform a mean filter with different, for instance circular, kernel.
  **/
 void
 gwy_data_field_filter_mean(GwyDataField *data_field,
                            gint size)
 {
     g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    gwy_data_field_area_filter_mean(data_field, size, 0, 0,
-                                    data_field->xres, data_field->yres);
+    gwy_data_field_area_filter_mean(data_field, size, 0, 0, data_field->xres, data_field->yres);
 }
 
 /**
@@ -802,12 +756,8 @@ gwy_data_field_area_filter_rms(GwyDataField *data_field,
         avg2->data[i] *= avg2->data[i];
 
     buffer = gwy_data_field_new_alike(avg2, FALSE);
-    gwy_data_field_area_gather(avg2, avg2, buffer,
-                               size, size, TRUE,
-                               0, 0, width, height);
-    gwy_data_field_area_gather(data_field, data_field, buffer,
-                               size, size, TRUE,
-                               col, row, width, height);
+    gwy_data_field_area_gather(avg2, avg2, buffer, size, size, TRUE, 0, 0, width, height);
+    gwy_data_field_area_gather(data_field, data_field, buffer, size, size, TRUE, col, row, width, height);
     g_object_unref(buffer);
 
     for (i = 0; i < height; i++) {
@@ -835,8 +785,7 @@ gwy_data_field_filter_rms(GwyDataField *data_field,
                           gint size)
 {
     g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    gwy_data_field_area_filter_rms(data_field, size, 0, 0,
-                                   data_field->xres, data_field->yres);
+    gwy_data_field_area_filter_rms(data_field, size, 0, 0, data_field->xres, data_field->yres);
 }
 
 /**
@@ -854,7 +803,7 @@ gwy_data_field_filter_canny(GwyDataField *data_field,
     GwyDataField *sobel_vertical;
     gint i, j, k;
     gint xres, yres, n;
-    gdouble angle;
+    gdouble angle, min, max;
     gint pass;
     gdouble *data, *svdata, *shdata;
 
@@ -876,9 +825,8 @@ gwy_data_field_filter_canny(GwyDataField *data_field,
 
     gwy_data_field_invalidate(data_field);
 
-    threshold = gwy_data_field_get_min(data_field)
-                + (gwy_data_field_get_max(data_field)
-                   - gwy_data_field_get_min(data_field))*threshold;
+    gwy_data_field_get_min_max(data_field, &min, &max);
+    threshold = min + (max - min)*threshold;
 
     /* We do not need sobel array more, so use sobel_horizontal to store data
      * results. */
@@ -893,13 +841,11 @@ gwy_data_field_filter_canny(GwyDataField *data_field,
             if (data[i*xres + j] > threshold) {
                 angle = atan2(svdata[i*xres + j], shdata[i*xres + j]);
 
-                if (angle < 0.3925 || angle > 5.8875
-                    || (angle > 2.7475 && angle < 3.5325)) {
+                if (angle < 0.3925 || angle > 5.8875 || (angle > 2.7475 && angle < 3.5325)) {
                     if (data[j + 1 + xres*i] > threshold)
                         pass = 1;
                 }
-                else if ((angle > 1.178 && angle < 1.9632)
-                         || (angle > 4.318 && angle < 5.1049)) {
+                else if ((angle > 1.178 && angle < 1.9632) || (angle > 4.318 && angle < 5.1049)) {
                     if (data[j + 1 + xres*(i + 1)] > threshold)
                         pass = 1;
                 }
@@ -924,16 +870,13 @@ gwy_data_field_filter_canny(GwyDataField *data_field,
 /**
  * gwy_data_field_filter_slope:
  * @data_field: A data field to apply the filter to.
- * @xder: Data field where the x-derivarive is to be stored, or %NULL if you
- *        are only interested in the y-derivarive.
- * @yder: Data field where the y-derivarive is to be stored, or %NULL if you
- *        are only interested in the x-derivarive.
+ * @xder: Data field where the x-derivarive is to be stored, or %NULL if you are only interested in the y-derivarive.
+ * @yder: Data field where the y-derivarive is to be stored, or %NULL if you are only interested in the x-derivarive.
  *
  * Calculates x and y derivaties for an entire field.
  *
- * The derivatives are calculated as the simple symmetrical differences (in
- * physical units, not pixel-wise), except at the edges where the differences
- * are one-sided.
+ * The derivatives are calculated as the simple symmetrical differences (in physical units, not pixel-wise), except at
+ * the edges where the differences are one-sided.
  *
  * Since: 2.37
  **/
@@ -1015,8 +958,7 @@ gwy_data_field_filter_slope(GwyDataField *data_field,
  *
  * Processes a data field with Gaussian step detection filter.
  *
- * The filter is a multi-directional combination of convolutions with
- * Gaussian multiplied by a signed step function.
+ * The filter is a multi-directional combination of convolutions with Gaussian multiplied by a signed step function.
  *
  * The resulting values correspond roughly to the step height around the pixel.
  *
@@ -1026,8 +968,7 @@ void
 gwy_data_field_filter_gauss_step(GwyDataField *data_field, gdouble sigma)
 {
     GwyDataField *extfield, *kernel, *freal, *fimag, *kreal, *kimag, *gauss;
-    guint ext, extx, exty, extxres, extyres, extn,
-          xres, yres, i, n, idir, ndirs = 6;
+    guint ext, extx, exty, extxres, extyres, extn, xres, yres, i, n, idir, ndirs = 6;
     gdouble *fre, *fim, *g, *k, *kre, *kim, *f, *d;
     gdouble gs;
 
@@ -1044,9 +985,7 @@ gwy_data_field_filter_gauss_step(GwyDataField *data_field, gdouble sigma)
     n = xres*yres;
     extx = gwy_fft_find_nice_size(xres + ext) - xres;
     exty = gwy_fft_find_nice_size(yres + ext) - yres;
-    extfield = gwy_data_field_extend(data_field,
-                                     extx/2, extx - extx/2,
-                                     exty/2, exty - exty/2,
+    extfield = gwy_data_field_extend(data_field, extx/2, extx - extx/2, exty/2, exty - exty/2,
                                      GWY_EXTERIOR_BORDER_EXTEND, 0.0, FALSE);
 
     extxres = extfield->xres;
@@ -1091,8 +1030,7 @@ gwy_data_field_filter_gauss_step(GwyDataField *data_field, gdouble sigma)
     gwy_data_field_fft_convolve(kernel, gauss);
     gwy_data_field_subtract_fields(extfield, extfield, kernel);
 
-    gwy_data_field_2dfft_raw(extfield, NULL, freal, fimag,
-                             GWY_TRANSFORM_DIRECTION_FORWARD);
+    gwy_data_field_2dfft_raw(extfield, NULL, freal, fimag, GWY_TRANSFORM_DIRECTION_FORWARD);
     fre = freal->data;
     fim = fimag->data;
 
@@ -1127,14 +1065,10 @@ gwy_data_field_filter_gauss_step(GwyDataField *data_field, gdouble sigma)
             private(i)
 #endif
         for (i = 0; i < extyres; i++) {
-            gdouble y = (i < (extyres + 1)/2
-                         ? i/sigma
-                         : (extyres-i)/(-sigma));
+            gdouble y = (i < (extyres + 1)/2 ? i/sigma : (extyres-i)/(-sigma));
             guint j, m = i*extxres;
             for (j = 0; j < extxres; j++, m++) {
-                gdouble x = (j < (extxres + 1)/2
-                             ? j/sigma
-                             : (extxres-j)/(-sigma));
+                gdouble x = (j < (extxres + 1)/2 ? j/sigma : (extxres-j)/(-sigma));
                 gdouble v = x*cphi + y*sphi;
 
                 if (G_UNLIKELY(fabs(v) < 1e-9))
@@ -1144,8 +1078,7 @@ gwy_data_field_filter_gauss_step(GwyDataField *data_field, gdouble sigma)
             }
         }
 
-        gwy_data_field_2dfft_raw(kernel, NULL, kreal, kimag,
-                                 GWY_TRANSFORM_DIRECTION_FORWARD);
+        gwy_data_field_2dfft_raw(kernel, NULL, kreal, kimag, GWY_TRANSFORM_DIRECTION_FORWARD);
 
 #ifdef _OPENMP
 #pragma omp parallel for if (gwy_threads_are_enabled()) default(none) \
@@ -1158,14 +1091,12 @@ gwy_data_field_filter_gauss_step(GwyDataField *data_field, gdouble sigma)
             kre[i] = re;
             kim[i] = im;
         }
-        gwy_data_field_2dfft_raw(kreal, kimag, kernel, NULL,
-                                 GWY_TRANSFORM_DIRECTION_BACKWARD);
+        gwy_data_field_2dfft_raw(kreal, kimag, kernel, NULL, GWY_TRANSFORM_DIRECTION_BACKWARD);
         for (i = 0; i < extn; i++)
             f[i] += k[i]*k[i];
     }
 
-    gwy_data_field_area_copy(extfield, data_field,
-                             extx/2, exty/2, xres, yres, 0, 0);
+    gwy_data_field_area_copy(extfield, data_field, extx/2, exty/2, xres, yres, 0, 0);
 
     d = data_field->data;
 #ifdef _OPENMP
@@ -1251,8 +1182,7 @@ gwy_data_field_area_filter_conservative(GwyDataField *data_field,
                 }
             }
 
-            hlpdata[i*width + j] = CLAMP(data[(i + row)*xres + j + col],
-                                         minval, maxval);
+            hlpdata[i*width + j] = CLAMP(data[(i + row)*xres + j + col], minval, maxval);
         }
     }
     /* fix bottom right corner for size == 2 */
@@ -1276,8 +1206,7 @@ gwy_data_field_filter_conservative(GwyDataField *data_field,
                                    gint size)
 {
     g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    gwy_data_field_area_filter_conservative(data_field, size, 0, 0,
-                                            data_field->xres, data_field->yres);
+    gwy_data_field_area_filter_conservative(data_field, size, 0, 0, data_field->xres, data_field->yres);
 }
 
 /* Return 8*number-of-neighbours + number-of-neighbour-segments.
@@ -1341,8 +1270,7 @@ pixel_thinnable(const gdouble *data, gint xres, gint k)
     if (neighval < 2 || neighval > 6)
         return FALSE;
 
-    /* We could pull the first parts into the table in neighbour_segments(),
-     * but I am currently too lazy. */
+    /* We could pull the first parts into the table in neighbour_segments(), but I am currently too lazy. */
     if (data[k + 1] > 0.0 && data[k - 1] > 0.0 && data[k + xres] > 0.0
         && neighbour_segments(data, xres, k+xres) % 8 == 1)
         return FALSE;
@@ -1397,9 +1325,8 @@ thinstep(GwyDataField *data_field,
     return ch;
 }
 
-/* XXX: Could be an alternative public function to gwy_data_field_thin().  It
- * is more aggressive and loses some branches, but this can be a good thing.
- * But for that it would be nice to implement it using a pixel queue so that
+/* XXX: Could be an alternative public function to gwy_data_field_thin().  It is more aggressive and loses some
+ * branches, but this can be a good thing.  But for that it would be nice to implement it using a pixel queue so that
  * we do not have to repeatedly scan the entire data field. */
 static void
 thin_data_field(GwyDataField *data_field)
@@ -1476,8 +1403,7 @@ kuwahara_block(const gdouble *a)
  * @width: Area width (number of columns).
  * @height: Area height (number of rows).
  *
- * Filters a rectangular part of a data field with a Kuwahara
- * (edge-preserving smoothing) filter.
+ * Filters a rectangular part of a data field with a Kuwahara (edge-preserving smoothing) filter.
  **/
 void
 gwy_data_field_area_filter_kuwahara(GwyDataField *data_field,
@@ -1537,8 +1463,7 @@ void
 gwy_data_field_filter_kuwahara(GwyDataField *data_field)
 {
     g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    gwy_data_field_area_filter_kuwahara(data_field, 0, 0,
-                                        data_field->xres, data_field->yres);
+    gwy_data_field_area_filter_kuwahara(data_field, 0, 0, data_field->xres, data_field->yres);
 }
 
 /**
@@ -1579,8 +1504,7 @@ gwy_data_field_shade(GwyDataField *data_field,
 #endif
     for (i = 0; i < yres; i++) {
         for (j = 0; j < xres; j++) {
-            v = (_gwy_data_field_xder(data_field, j, i)*cphi
-                 + _gwy_data_field_yder(data_field, j, i)*sphi);
+            v = _gwy_data_field_xder(data_field, j, i)*cphi + _gwy_data_field_yder(data_field, j, i)*sphi;
             tdata[j + xres*i] = -v;
             if (max < v)
                 max = v;
@@ -1639,10 +1563,8 @@ gwy_data_field_filter_harris(GwyDataField *x_gradient,
         return;
     }
 
-    mult = fabs(gwy_data_field_get_max(x_gradient)
-                - gwy_data_field_get_min(x_gradient));
-    mult += fabs(gwy_data_field_get_max(y_gradient)
-                 - gwy_data_field_get_min(y_gradient));
+    mult = (fabs(gwy_data_field_get_max(x_gradient) - gwy_data_field_get_min(x_gradient))
+            + fabs(gwy_data_field_get_max(y_gradient) - gwy_data_field_get_min(y_gradient)));
     mult = 1.0/(mult*mult);
 
     xx = result;
@@ -1690,14 +1612,9 @@ gwy_data_field_filter_harris(GwyDataField *x_gradient,
     }
 
     gwy_data_field_area_clear(result, 0, 0, xres, neighbourhood);
-    gwy_data_field_area_clear(result,
-                              0, neighbourhood,
-                              neighbourhood, yres-2*neighbourhood);
-    gwy_data_field_area_clear(result,
-                              xres-neighbourhood, neighbourhood,
-                              neighbourhood, yres-2*neighbourhood);
-    gwy_data_field_area_clear(result,
-                              0, yres-neighbourhood, xres, neighbourhood);
+    gwy_data_field_area_clear(result, 0, neighbourhood, neighbourhood, yres-2*neighbourhood);
+    gwy_data_field_area_clear(result, xres-neighbourhood, neighbourhood, neighbourhood, yres-2*neighbourhood);
+    gwy_data_field_area_clear(result, 0, yres-neighbourhood, xres, neighbourhood);
 
     /* xx = result */
     g_object_unref(xy);
@@ -1711,11 +1628,9 @@ gwy_data_field_filter_harris(GwyDataField *x_gradient,
  * @title: filters
  * @short_description: Convolution and other 2D data filters
  *
- * Filters are point-wise operations, such as thresholding, or generally local
- * operations producing a value based on the data in the vicinity of each
- * point: gradients, step detectors and convolutions.  Some simple common
- * point-wise operations, e.g. value inversion, are also found in base
- * #GwyDataField methods.
+ * Filters are point-wise operations, such as thresholding, or generally local operations producing a value based on
+ * the data in the vicinity of each point: gradients, step detectors and convolutions.  Some simple common point-wise
+ * operations, e.g. value inversion, are also found in base #GwyDataField methods.
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
