@@ -1,5 +1,5 @@
 /*
- *  $Id: grains-disttrans.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: grains-disttrans.c 23662 2021-05-10 20:30:09Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -507,8 +507,7 @@ average_octagonal_dt(GwyDataField *dfield, gboolean from_border)
     gwy_data_field_grain_simple_dist_trans(tmp,
                                            GWY_DISTANCE_TRANSFORM_OCTAGONAL84,
                                            from_border);
-    gwy_data_field_sum_fields(dfield, dfield, tmp);
-    gwy_data_field_multiply(dfield, 0.5);
+    gwy_data_field_linear_combination(dfield, 0.5, dfield, 0.5, tmp, 0.0);
     g_object_unref(tmp);
 }
 
