@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyprocessenums.h 21457 2018-09-23 20:11:09Z yeti-dn $
+ *  $Id: gwyprocessenums.h 24254 2021-10-06 11:25:18Z yeti-dn $
  *  Copyright (C) 2005-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -238,13 +238,15 @@ typedef enum {
 } GwyGrainQuantity;
 
 typedef enum {
-    GWY_DATA_COMPATIBILITY_RES     = 1 << 0,
-    GWY_DATA_COMPATIBILITY_REAL    = 1 << 1,
-    GWY_DATA_COMPATIBILITY_MEASURE = 1 << 2,
-    GWY_DATA_COMPATIBILITY_LATERAL = 1 << 3,
-    GWY_DATA_COMPATIBILITY_VALUE   = 1 << 4,
-    GWY_DATA_COMPATIBILITY_AXISCAL = 1 << 5,
-    GWY_DATA_COMPATIBILITY_ALL     = 0x003f
+    GWY_DATA_COMPATIBILITY_RES      = 1 << 0,
+    GWY_DATA_COMPATIBILITY_REAL     = 1 << 1,
+    GWY_DATA_COMPATIBILITY_MEASURE  = 1 << 2,
+    GWY_DATA_COMPATIBILITY_LATERAL  = 1 << 3,
+    GWY_DATA_COMPATIBILITY_VALUE    = 1 << 4,
+    GWY_DATA_COMPATIBILITY_AXISCAL  = 1 << 5,
+    GWY_DATA_COMPATIBILITY_NCURVES  = 1 << 6,
+    GWY_DATA_COMPATIBILITY_CURVELEN = 1 << 7,
+    GWY_DATA_COMPATIBILITY_ALL      = 0x00ff
 } GwyDataCompatibilityFlags;
 
 typedef enum {
