@@ -1,5 +1,5 @@
 /*
- *  $Id: inttrans.c 24765 2022-04-14 15:03:32Z yeti-dn $
+ *  $Id: inttrans.c 24844 2022-06-22 06:36:15Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -25,7 +25,6 @@
 #include <libprocess/simplefft.h>
 #include <libprocess/level.h>
 #include <libprocess/stats.h>
-#include <libprocess/cwt.h>
 #include "gwyprocessinternal.h"
 #include "gwyfftw.h"
 
@@ -58,6 +57,18 @@ static void zoom_fft_1d_do                (const gdouble *rein,
                                            gint m,
                                            gdouble f0,
                                            gdouble f1);
+static void zoom_fft_2d_do                (const gdouble *rein,
+                                           const gdouble *imin,
+                                           gint nx,
+                                           gint ny,
+                                           gdouble *reout,
+                                           gdouble *imout,
+                                           gint mx,
+                                           gint my,
+                                           gdouble fx0,
+                                           gdouble fy0,
+                                           gdouble fx1,
+                                           gdouble fy1);
 static void gwy_data_field_area_2dfft_real(GwyDataField *ra,
                                            GwyDataField *rb,
                                            GwyDataField *ib,
@@ -135,10 +146,6 @@ static void gwy_data_field_yfft_real_do   (GwyDataField *rin,
                                            GwyDataField *rout,
                                            GwyDataField *iout,
                                            GwyTransformDirection direction);
-static void gwy_data_field_mult_wav       (GwyDataField *real_field,
-                                           GwyDataField *imag_field,
-                                           gdouble scale,
-                                           Gwy2DCWTWaveletType wtype);
 static void gwy_level_simple              (gint n,
                                            gint stride,
                                            gdouble *data,
@@ -158,7 +165,7 @@ static void gwy_preserve_rms_simple       (gint nsrc,
  * @isrc: Imaginary input data line.
  * @rdest: Real output data line.  It will be resized to the size of the input data line.
  * @idest: Imaginary output data line.  It will be resized to the size of the input data line.
- * @windowing: Windowing mode.
+ * @windowing: Windowing type to use.
  * @direction: FFT direction.
  * @interpolation: Interpolation type. Ignored since 2.8 as no resampling is performed.
  * @preserverms: %TRUE to preserve RMS value while windowing.
@@ -192,7 +199,7 @@ gwy_data_line_fft(GwyDataLine *rsrc, GwyDataLine *isrc,
  * @idest: Imaginary output data line, it will be resized to @len.
  * @from: The index in input lines to start from (inclusive).
  * @len: Lenght of data line part, it must be at least 2.
- * @windowing: Windowing mode.
+ * @windowing: Windowing type to use.
  * @direction: FFT direction.
  * @interpolation: Interpolation type. Ignored since 2.8 as no resampling is performed.
  * @preserverms: %TRUE to preserve RMS value while windowing.
@@ -230,12 +237,12 @@ gwy_data_line_part_fft(GwyDataLine *rsrc, GwyDataLine *isrc,
 
     rbuf = gwy_data_line_part_extract(rsrc, from, len);
     gwy_level_simple(len, 1, rbuf->data, level);
-    gwy_fft_window(len, rbuf->data, windowing);
+    gwy_data_line_fft_window(rbuf, windowing);
 
     if (isrc) {
         ibuf = gwy_data_line_part_extract(isrc, from, len);
         gwy_level_simple(len, 1, ibuf->data, level);
-        gwy_fft_window(len, ibuf->data, windowing);
+        gwy_data_line_fft_window(ibuf, windowing);
         gwy_data_line_fft_do(rbuf, ibuf, rdest, idest, direction);
         if (preserverms) {
             gwy_preserve_rms_simple(len, 1, rsrc->data + from, isrc->data + from,
@@ -399,10 +406,11 @@ gwy_data_line_fft_real_do(GwyDataLine *rsrc,
  * indexed by integers and work with that.
  *
  * Frequency step of one corresponds to the normal DFT frequency step. Therefore, passing @f0=0, @f1=@n-1 (where @rsrc
- * has @n points) and @m=@n reproduces FFT, except more slowly. The result is normalised as raw FFT and the units
- * of the output data lines are unchanged.
+ * has @n points) and @m=@n reproduces the usual DFT, except more slowly. The result is normalised as raw FFT and the
+ * units of the output data lines are unchanged.
  *
- * The transform direction is always forward.
+ * The transform direction is always forward.  Windowing or other preprocessing need to be done separately beforehand.
+ * They would be usually once, but followed by any number of (Zoom) FFTs.
  *
  * Since: 2.61
  **/
@@ -434,33 +442,85 @@ gwy_data_line_zoom_fft(GwyDataLine *rsrc,
     rdest->off = idest->off = f0;
 }
 
+/**
+ * gwy_data_field_zoom_fft:
+ * @rsrc: Real input data field.
+ * @isrc: Imaginary input data field.  It can be %NULL for real-to-complex transform.
+ * @rdest: Real output data field.  It will be resized to @mx × @my samples.
+ * @idest: Imaginary output data field.  It will be resized to @mx × @my samples.
+ * @fx0: The first horizontal spatial frequency, measured in DFT frequency steps.
+ * @fy0: The first vertical spatial frequency, measured in DFT frequency steps.
+ * @fx1: The last horizontal spatial frequency, measured in DFT frequency steps.
+ * @fy1: The last vetical spatial frequency, measured in DFT frequency steps.
+ * @mx: The number of horizontal frequencies to compute. It must be at least 2.
+ * @my: The number of vertical frequencies to compute. It must be at least 2.
+ *
+ * Computes Zoom FFT of a data field.
+ *
+ * The output is DFTs, but computed for an arbitrary 2D Cartesian grid of frequencies along @x and @y. The frequencies
+ * do not have to be in any relation to the data sampling step.
+ *
+ * The top-left pixel of output corresponds exactly to (@fx0,@fy0) and the bottom right exactly to (@fx1,@fy1).  So
+ * the frequency sampling steps will be (@fx1 − @fx0)/(@mx − 1) and (@fy1 − @fy0)/(@my − 1), instead of the more usual
+ * division by @mx and @my. To follow the usual Gwyddion conventions, the output data field real size will be (@fx1
+ * − @fx0)/(@mx − 1)*@mx along @x, and similarly along @y. If it seems confusing, just take the output as indexed by
+ * integers and work with that.
+ *
+ * Frequency step of one corresponds to the normal DFT frequency step. Therefore, passing @fx0=0, @fx1=@xres–1,
+ * @fy0=0, @fy1=@yres–1 (where @rsrc has @xres × @yres points), @mx=@xres and @my=@yres reproduces the usual DFT,
+ * except more slowly. The result is normalised as raw FFT and the units of the output data fields are unchanged.
+ *
+ * The transform direction is always forward.  Windowing or other preprocessing need to be done separately beforehand.
+ * They would be usually once, but followed by any number of (Zoom) FFTs.
+ *
+ * Since: 2.62
+ **/
+void
+gwy_data_field_zoom_fft(GwyDataField *rsrc,
+                        GwyDataField *isrc,
+                        GwyDataField *rdest,
+                        GwyDataField *idest,
+                        gint mx,
+                        gint my,
+                        gdouble fx0,
+                        gdouble fy0,
+                        gdouble fx1,
+                        gdouble fy1)
+{
+    g_return_if_fail(GWY_IS_DATA_FIELD(rsrc));
+    g_return_if_fail(!isrc || GWY_IS_DATA_FIELD(isrc));
+    if (isrc) {
+        g_return_if_fail(!gwy_data_field_check_compatibility(rsrc, isrc, GWY_DATA_COMPATIBILITY_RES));
+    }
+    g_return_if_fail(GWY_IS_DATA_FIELD(rdest));
+    g_return_if_fail(GWY_IS_DATA_FIELD(idest));
+    g_return_if_fail(mx > 1);
+    g_return_if_fail(my > 1);
+
+    gwy_data_field_resample(rdest, mx, my, GWY_INTERPOLATION_NONE);
+    gwy_data_field_resample(idest, mx, my, GWY_INTERPOLATION_NONE);
+
+    zoom_fft_2d_do(rsrc->data, isrc ? isrc->data : NULL, rsrc->xres, rsrc->yres,
+                   rdest->data, idest->data, mx, my, fx0, fy0, fx1, fy1);
+
+    rdest->xreal = idest->xreal = (fx1 - fx0)*mx/(mx - 1.0);
+    rdest->yreal = idest->yreal = (fy1 - fy0)*my/(my - 1.0);
+    rdest->xoff = idest->xoff = fx0;
+    rdest->yoff = idest->yoff = fy0;
+}
+
+/* Precompute the factors w_k = exp(-2πik²/(ND)) */
 static void
-zoom_fft_1d_do(const gdouble *rein, const gdouble *imin, gint n,
-               gdouble *reout, gdouble *imout, gint m, gdouble f0, gdouble f1)
+make_chirp(fftw_complex *w, gint m, gint n, gint size, gdouble D)
 {
-    /* The range of chirp coefficient w indices is -n+1,-n+2,...,m-2,m-1.
-     * The range of data d indices is 0,1,...,n-1.
-     * We only need convolution result to be correct for indices 0,1,...,m-1.
-     * This means we do not have to pad, i.e. the minimum transform length is m+n-1. */
     gint minsize = m + n - 1;
-    gint size = gwy_fft_find_nice_size(minsize);
-    fftw_complex *x = gwy_fftw_new_complex(size);
-    fftw_complex *w = gwy_fftw_new_complex(size);
-    fftw_complex *fx = gwy_fftw_new_complex(size);
-    fftw_complex *fw = gwy_fftw_new_complex(size);
-    fftw_plan fplan, bplan;
-    gdouble q, D = (f1 - f0)/(m - 1);
     gint k, mm = MIN(m, n);
+    gdouble q = -G_PI/n*D;
 
-    fplan = gwy_fftw_plan_dft_1d(size, x, fw, FFTW_FORWARD, FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
-    bplan = gwy_fftw_plan_dft_1d(size, fx, x, FFTW_BACKWARD, FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
-
-    /* Precompute the factors w_k = exp(-2πik²/(ND)) */
     /* NB: Gwydion has swapped forward and backward signs! This basically means q has the opposite sign than it would
      * normally have. */
     gwycreal(w[0]) = 1.0;
     gwycimag(w[0]) = 0.0;
-    q = -G_PI/n*D;
     for (k = 1; k < mm; k++) {
         gdouble s, c;
         _gwy_sincos(q*k*k, &s, &c);
@@ -484,26 +544,66 @@ zoom_fft_1d_do(const gdouble *rein, const gdouble *imin, gint n,
         //w[size - k] = cexp(q*k*k*I);
     }
     gwy_clear(w + m, size - minsize);
+}
+
+static void
+make_cexp(fftw_complex *cs, gint n, gdouble q)
+{
+    gint k;
+
+    gwycreal(cs[0]) = 1.0;
+    gwycimag(cs[0]) = 0.0;
+    for (k = 1; k < n; k++) {
+        gdouble s, c;
+        _gwy_sincos(q*k, &s, &c);
+        gwycreal(cs[k]) = c;
+        gwycimag(cs[k]) = s;
+    }
+}
+
+#define cmulre(a,b) (gwycreal(a)*gwycreal(b) - gwycimag(a)*gwycimag(b))
+#define cmulim(a,b) (gwycimag(a)*gwycreal(b) + gwycreal(a)*gwycimag(b))
+#define cjmulre(a,b) (gwycreal(a)*gwycreal(b) + gwycimag(a)*gwycimag(b))
+#define cjmulim(a,b) (gwycimag(a)*gwycreal(b) - gwycreal(a)*gwycimag(b))
+
+static void
+zoom_fft_1d_do(const gdouble *rein, const gdouble *imin, gint n,
+               gdouble *reout, gdouble *imout, gint m, gdouble f0, gdouble f1)
+{
+    /* The range of chirp coefficient w indices is -n+1,-n+2,...,m-2,m-1.
+     * The range of data d indices is 0,1,...,n-1.
+     * We only need convolution result to be correct for indices 0,1,...,m-1.
+     * This means we do not have to pad, i.e. the minimum transform length is m+n-1. */
+    gint minsize = m + n - 1;
+    gint size = gwy_fft_find_nice_size(minsize);
+    fftw_complex *cs = g_new(fftw_complex, n);
+    fftw_complex *x = gwy_fftw_new_complex(size);
+    fftw_complex *w = gwy_fftw_new_complex(size);
+    fftw_complex *fx = gwy_fftw_new_complex(size);
+    fftw_complex *fw = gwy_fftw_new_complex(size);
+    fftw_plan fplan, bplan;
+    gdouble q, D = (f1 - f0)/(m - 1);
+    gint k;
+
+    fplan = gwy_fftw_plan_dft_1d(size, x, fw, FFTW_FORWARD, FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
+    bplan = gwy_fftw_plan_dft_1d(size, fx, x, FFTW_BACKWARD, FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
+
+    /* Precompute the factors w_k = exp(-2πik²/(ND)) */
+    make_chirp(w, m, n, size, D);
 
     /* Transform premultiplied data. */
-    gwycreal(x[0]) = rein[0];
-    gwycimag(x[0]) = imin ? imin[0] : 0.0;
-    q = 2.0*G_PI*f0/n;
-    for (k = 1; k < n; k++) {
-        gdouble s, c, tre, tim;
-        _gwy_sincos(q*k, &s, &c);
-        tre = c*gwycreal(w[size - k]) + s*gwycimag(w[size - k]);
-        tim = s*gwycreal(w[size - k]) - c*gwycimag(w[size - k]);
-        if (imin) {
-            gwycreal(x[k]) = tre*rein[k] - tim*imin[k];
-            gwycimag(x[k]) = tim*rein[k] + tre*imin[k];
-        }
-        else {
-            gwycreal(x[k]) = tre*rein[k];
-            gwycimag(x[k]) = tim*rein[k];
-        }
+    make_cexp(cs, n, 2.0*G_PI*f0/n);
+    for (k = 0; k < n; k++) {
+        fftw_complex t, d;
+        gwycreal(t) = cjmulre(cs[k], w[size - k]);
+        gwycimag(t) = cjmulim(cs[k], w[size - k]);
+        gwycreal(d) = rein[k];
+        gwycimag(d) = imin ? imin[k] : 0.0;
+        gwycreal(x[k]) = cmulre(t, d);
+        gwycimag(x[k]) = cmulim(t, d);
         //x[k] = cexp(q*k*I)*conj(w[size - k])*d[k];
     }
+    g_free(cs);
     gwy_clear(x + n, size - n);
     fftw_execute(fplan);
     gwy_assign(fx, fw, size);
@@ -515,9 +615,7 @@ zoom_fft_1d_do(const gdouble *rein, const gdouble *imin, gint n,
 
     /* Multiply and transform back. */
     for (k = 0; k < size; k++) {
-        gdouble tre, tim;
-        tre = gwycreal(fx[k])*gwycreal(fw[k]) - gwycimag(fx[k])*gwycimag(fw[k]);
-        tim = gwycimag(fx[k])*gwycreal(fw[k]) + gwycreal(fx[k])*gwycimag(fw[k]);
+        gdouble tre = cmulre(fx[k], fw[k]), tim = cmulim(fx[k], fw[k]);
         gwycreal(fx[k]) = tre;
         gwycimag(fx[k]) = tim;
         //fx[k] *= fw[k];
@@ -527,17 +625,134 @@ zoom_fft_1d_do(const gdouble *rein, const gdouble *imin, gint n,
     fftw_free(fx);
     fftw_free(fw);
 
-    /* And finally post-multiply by w. */
+    /* And finally post-multiply by conj(w). */
     q = 1.0/size/sqrt(n);
     for (k = 0; k < m; k++) {
-        reout[k] = q*(gwycreal(x[k])*gwycreal(w[k]) + gwycimag(x[k])*gwycimag(w[k]));
-        imout[k] = q*(gwycimag(x[k])*gwycreal(w[k]) - gwycreal(x[k])*gwycimag(w[k]));
+        reout[k] = q*cjmulre(x[k], w[k]);
+        imout[k] = q*cjmulim(x[k], w[k]);
         //f[k] = q*x[k]*conj(w[k]);
     }
     fftw_free(x);
     fftw_free(w);
 }
 
+static void
+zoom_fft_2d_do(const gdouble *rein, const gdouble *imin, gint nx, gint ny,
+               gdouble *reout, gdouble *imout, gint mx, gint my,
+               gdouble fx0, gdouble fy0, gdouble fx1, gdouble fy1)
+{
+    /* The range of chirp coefficient w indices is -n+1,-n+2,...,m-2,m-1.
+     * The range of data d indices is 0,1,...,n-1.
+     * We only need convolution result to be correct for indices 0,1,...,m-1.
+     * This means we do not have to pad, i.e. the minimum transform length is m+n-1. */
+    gint minxsize = mx + nx - 1;
+    gint minysize = my + ny - 1;
+    gint xsize = gwy_fft_find_nice_size(minxsize);
+    gint ysize = gwy_fft_find_nice_size(minysize);
+    gint size = xsize*ysize;
+    fftw_complex *w_x = g_new(fftw_complex, xsize);
+    fftw_complex *w_y = g_new(fftw_complex, ysize);
+    fftw_complex *cs_x = g_new(fftw_complex, nx);
+    fftw_complex *cs_y = g_new(fftw_complex, ny);
+    fftw_complex *x = gwy_fftw_new_complex(size);
+    fftw_complex *fx = gwy_fftw_new_complex(size);
+    fftw_complex *fw = gwy_fftw_new_complex(size);
+    fftw_plan fplan, bplan;
+    gdouble q, Dx = (fx1 - fx0)/(mx - 1), Dy = (fy1 - fy0)/(my - 1);
+    gint k, kx, ky;
+
+    fplan = gwy_fftw_plan_dft_2d(xsize, ysize, x, fw, FFTW_FORWARD, FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
+    bplan = gwy_fftw_plan_dft_2d(xsize, ysize, fx, x, FFTW_BACKWARD, FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
+
+    /* Precompute the factors w_k = exp(-2πik²/(ND))
+     * This part is entirely factorable. So we only compute 1D arrays along x and y and then basically make outer
+     * products. */
+    make_chirp(w_x, mx, nx, xsize, Dx);
+    make_chirp(w_y, my, ny, ysize, Dy);
+
+    /* Premultiply data.  We first combine cexp(qx*kx*I)*conj(wx[xsize - kx]) to a single x-factor, and similarly for
+     * y. We put it to cs[] because it is needed only here, whereas w[] will be still needed later. */
+    make_cexp(cs_x, nx, 2.0*G_PI*fx0/nx);
+    make_cexp(cs_y, ny, 2.0*G_PI*fy0/ny);
+    for (kx = 1; kx < nx; kx++) {
+        fftw_complex t;
+        gwycreal(t) = cjmulre(cs_x[kx], w_x[xsize - kx]);
+        gwycimag(t) = cjmulim(cs_x[kx], w_x[xsize - kx]);
+        gwycreal(cs_x[kx]) = gwycreal(t);
+        gwycimag(cs_x[kx]) = gwycimag(t);
+    }
+    for (ky = 1; ky < ny; ky++) {
+        fftw_complex t;
+        gwycreal(t) = cjmulre(cs_y[ky], w_y[ysize - ky]);
+        gwycimag(t) = cjmulim(cs_y[ky], w_y[ysize - ky]);
+        gwycreal(cs_y[ky]) = gwycreal(t);
+        gwycimag(cs_y[ky]) = gwycimag(t);
+    }
+    for (ky = 0; ky < ny; ky++) {
+        for (kx = 0; kx < nx; kx++) {
+            /* The arrays have different sizes! */
+            gint kkin = nx*ky + kx;
+            gint kkx = xsize*ky + kx;
+            fftw_complex t, d;
+            gwycreal(t) = cmulre(cs_x[kx], cs_y[ky]);
+            gwycimag(t) = cmulim(cs_x[kx], cs_y[ky]);
+            gwycreal(d) = rein[kkin];
+            gwycimag(d) = imin ? imin[kkin] : 0.0;
+            gwycreal(x[kkx]) = cmulre(t, d);
+            gwycimag(x[kkx]) = cmulim(t, d);
+            //x[ky,kx] = cexp(qx*kx*I)*cexp(qy*ky*I)*conj(wx[xsize - kx])*conj(wy[ysize - ky])*d[ky,kx];
+        }
+        gwy_clear(x + xsize*ky + nx, xsize - nx);
+    }
+    g_free(cs_x);
+    g_free(cs_y);
+    gwy_clear(x + xsize*ny, xsize*(ysize - ny));
+    fftw_execute(fplan);
+    gwy_assign(fx, fw, size);
+
+    /* Transform chirp w (must form it as an outer product of w_x and w_y). */
+    for (ky = 0; ky < ysize; ky++) {
+        for (kx = 0; kx < xsize; kx++) {
+            gint kk = xsize*ky + kx;
+            gwycreal(x[kk]) = cmulre(w_x[kx], w_y[ky]);
+            gwycimag(x[kk]) = cmulim(w_x[kx], w_y[ky]);
+            //x[ky,kx] = w[ky,kx]
+        }
+    }
+    fftw_execute(fplan);
+    fftw_destroy_plan(fplan);
+
+    /* Multiply and transform back. */
+    for (k = 0; k < size; k++) {
+        gdouble tre = cmulre(fx[k], fw[k]), tim = cmulim(fx[k], fw[k]);
+        gwycreal(fx[k]) = tre;
+        gwycimag(fx[k]) = tim;
+        //fx[k] *= fw[k];
+    }
+    fftw_execute(bplan);
+    fftw_destroy_plan(bplan);
+    fftw_free(fx);
+    fftw_free(fw);
+
+    /* And finally post-multiply by conj(w). The arrays have different sizes; we extract only a part of the result. */
+    q = 1.0/size/sqrt(nx*ny);
+    for (ky = 0; ky < my; ky++) {
+        for (kx = 0; kx < mx; kx++) {
+            gint kkx = xsize*ky + kx;
+            gint kkout = mx*ky + kx;
+            fftw_complex t;
+            gwycreal(t) = cmulre(w_x[kx], w_y[ky]);
+            gwycimag(t) = cmulim(w_x[kx], w_y[ky]);
+            reout[kkout] = q*cjmulre(x[kkx], t);
+            imout[kkout] = q*cjmulim(x[kkx], t);
+            //f[ky,kx] = q*x[ky,kx]*conj(w[ky,kx]);
+        }
+    }
+    fftw_free(x);
+    fftw_free(w_x);
+    fftw_free(w_y);
+}
+
 static void
 gwy_data_field_2dfft_prepare(GwyDataField *dfield,
                              gint level,
@@ -557,8 +772,7 @@ gwy_data_field_2dfft_prepare(GwyDataField *dfield,
         a = gwy_data_field_get_rms(dfield);
         *rms = hypot(*rms, a);
     }
-    gwy_fft_window_data_field(dfield, GWY_ORIENTATION_HORIZONTAL, windowing);
-    gwy_fft_window_data_field(dfield, GWY_ORIENTATION_VERTICAL, windowing);
+    gwy_data_field_fft_window(dfield, windowing);
 }
 
 /**
@@ -1326,9 +1540,8 @@ gwy_data_field_1dfft_raw(GwyDataField *rin,
 /**
  * gwy_data_field_area_xfft:
  * @ra: Real input data field.
- * @ia: Imaginary input data field.  It can be %NULL for real-to-complex
- *      transform which can be somewhat faster than complex-to-complex
- *      transform.
+ * @ia: Imaginary input data field.  It can be %NULL for real-to-complex transform which can be somewhat faster than
+ *      complex-to-complex transform.
  * @rout: Real output data field, it will be resized to area size.
  * @iout: Imaginary output data field, it will be resized to area size.
  * @col: Upper-left column coordinate.
@@ -1376,14 +1589,14 @@ gwy_data_field_area_xfft(GwyDataField *rin, GwyDataField *iin,
         for (k = 0; k < height; k++)
             gwy_level_simple(width, 1, rbuf->data + k*width, level);
     }
-    gwy_fft_window_data_field(rbuf, GWY_ORIENTATION_HORIZONTAL, windowing);
+    gwy_data_field_fft_window_1d(rbuf, GWY_ORIENTATION_HORIZONTAL, windowing);
 
     ibuf = gwy_data_field_area_extract(iin, col, row, width, height);
     if (level) {
         for (k = 0; k < height; k++)
             gwy_level_simple(width, 1, ibuf->data + k*width, level);
     }
-    gwy_fft_window_data_field(ibuf, GWY_ORIENTATION_HORIZONTAL, windowing);
+    gwy_data_field_fft_window_1d(ibuf, GWY_ORIENTATION_HORIZONTAL, windowing);
 
     gwy_data_field_xfft_do(rbuf, ibuf, rout, iout, direction);
 
@@ -1486,7 +1699,7 @@ gwy_data_field_area_xfft_real(GwyDataField *rin, GwyDataField *rout,
         for (k = 0; k < height; k++)
             gwy_level_simple(width, 1, rbuf->data + k*width, level);
     }
-    gwy_fft_window_data_field(rbuf, GWY_ORIENTATION_HORIZONTAL, windowing);
+    gwy_data_field_fft_window_1d(rbuf, GWY_ORIENTATION_HORIZONTAL, windowing);
 
     ibuf = gwy_data_field_new_alike(rbuf, FALSE);
 
@@ -1602,14 +1815,14 @@ gwy_data_field_area_yfft(GwyDataField *rin, GwyDataField *iin,
         for (k = 0; k < width; k++)
             gwy_level_simple(height, width, rbuf->data + k, level);
     }
-    gwy_fft_window_data_field(rbuf, GWY_ORIENTATION_VERTICAL, windowing);
+    gwy_data_field_fft_window_1d(rbuf, GWY_ORIENTATION_VERTICAL, windowing);
 
     ibuf = gwy_data_field_area_extract(iin, col, row, width, height);
     if (level) {
         for (k = 0; k < width; k++)
             gwy_level_simple(height, width, ibuf->data + k, level);
     }
-    gwy_fft_window_data_field(ibuf, GWY_ORIENTATION_VERTICAL, windowing);
+    gwy_data_field_fft_window_1d(ibuf, GWY_ORIENTATION_VERTICAL, windowing);
 
     gwy_data_field_yfft_do(rbuf, ibuf, rout, iout, direction);
 
@@ -1713,7 +1926,7 @@ gwy_data_field_area_yfft_real(GwyDataField *rin, GwyDataField *rout,
         for (k = 0; k < width; k++)
             gwy_level_simple(height, width, rbuf->data + k, level);
     }
-    gwy_fft_window_data_field(rbuf, GWY_ORIENTATION_VERTICAL, windowing);
+    gwy_data_field_fft_window_1d(rbuf, GWY_ORIENTATION_VERTICAL, windowing);
 
     ibuf = gwy_data_field_new_alike(rbuf, FALSE);
 
@@ -1885,92 +2098,6 @@ gwy_preserve_rms_simple(gint nsrc,
         *pdata *= q;
 }
 
-/**
- * gwy_data_field_mult_wav:
- * @real_field: A data field of real values
- * @imag_field: A data field of imaginary values
- * @scale: wavelet scale
- * @wtype: waveelt type
- *
- * multiply a complex data field (real and imaginary) with complex FT of spoecified wavelet at given scale.
- **/
-static void
-gwy_data_field_mult_wav(GwyDataField *real_field,
-                        GwyDataField *imag_field,
-                        gdouble scale,
-                        Gwy2DCWTWaveletType wtype)
-{
-    gint xres, yres, xresh, yresh;
-    gint i, j;
-    gdouble mval, val;
-
-    xres = real_field->xres;
-    yres = real_field->yres;
-    xresh = xres/2;
-    yresh = yres/2;
-
-    for (i = 0; i < yres; i++) {
-        for (j = 0; j < xres; j++) {
-            val = 1;
-            if (j < xresh) {
-                if (i < yresh)
-                    mval = hypot(j, i);
-                else
-                    mval = hypot(j, yres - i);
-            }
-            else {
-                if (i < yresh)
-                    mval = hypot(xres - j, i);
-                else
-                    mval = hypot(xres - j, yres - i);
-            }
-            val = gwy_cwt_wfunc_2d(scale, mval, xres, wtype);
-
-            real_field->data[j + i*xres] *= val;
-            imag_field->data[j + i*xres] *= val;
-        }
-    }
-}
-
-/**
- * gwy_data_field_cwt:
- * @data_field: A data field.
- * @interpolation: Interpolation type. Ignored since 2.8 as no resampling is performed.
- * @scale: Wavelet scale.
- * @wtype: Wavelet type.
- *
- * Computes a continuous wavelet transform (CWT) at given scale and using given wavelet.
- **/
-void
-gwy_data_field_cwt(GwyDataField *data_field,
-                   GwyInterpolationType interpolation,
-                   gdouble scale,
-                   Gwy2DCWTWaveletType wtype)
-{
-    GwyDataField *hlp_r, *hlp_i, *imag_field;
-
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-
-    hlp_r = gwy_data_field_new_alike(data_field, FALSE);
-    hlp_i = gwy_data_field_new_alike(data_field, FALSE);
-    imag_field = gwy_data_field_new_alike(data_field, TRUE);
-
-    gwy_data_field_2dfft(data_field, imag_field, hlp_r, hlp_i,
-                         GWY_WINDOWING_RECT, GWY_TRANSFORM_DIRECTION_FORWARD, interpolation,  /* ignored */
-                         FALSE, FALSE);
-    gwy_data_field_mult_wav(hlp_r, hlp_i, scale, wtype);
-
-    gwy_data_field_2dfft(hlp_r, hlp_i, data_field, imag_field,
-                         GWY_WINDOWING_RECT, GWY_TRANSFORM_DIRECTION_BACKWARD, interpolation,  /* ignored */
-                         FALSE, FALSE);
-
-    g_object_unref(hlp_r);
-    g_object_unref(hlp_i);
-    g_object_unref(imag_field);
-
-    gwy_data_field_invalidate(data_field);
-}
-
 static GwyDataLine*
 resample_dline_for_1d_fft_filter(GwyDataLine *dline, gint res,
                                  GwyInterpolationType interpolation)
@@ -2058,6 +2185,128 @@ gwy_data_field_fft_filter_1d(GwyDataField *data_field,
     g_object_unref(hlp_idfield);
 }
 
+/**
+ * gwy_data_line_fft_window:
+ * @line: A one-dimensional data line.
+ * @windowing: Windowing type to use.
+ *
+ * Performs windowing of a data line in preparation for FFT.
+ *
+ * Since: 2.62
+ **/
+void
+gwy_data_line_fft_window(GwyDataLine *line,
+                         GwyWindowingType windowing)
+{
+    g_return_if_fail(GWY_IS_DATA_LINE(line));
+    g_return_if_fail(windowing <= GWY_WINDOWING_KAISER25);
+    if (windowing <= GWY_WINDOWING_NONE)
+        return;
+    gwy_fft_window(line->res, line->data, windowing);
+}
+
+/**
+ * gwy_data_field_fft_window:
+ * @field: A two-dimensional data field.
+ * @windowing: Windowing type to use.
+ *
+ * Performs two-dimensional windowing of a data field in preparation for 2D FFT.
+ *
+ * The same windowing function is used row-wise and column-wise.
+ *
+ * Since: 2.62
+ **/
+void
+gwy_data_field_fft_window(GwyDataField *field,
+                          GwyWindowingType windowing)
+{
+    gint xres, yres, j, i;
+    gdouble *data, *wx = NULL, *wy = NULL;
+
+    g_return_if_fail(GWY_IS_DATA_FIELD(field));
+    g_return_if_fail(windowing <= GWY_WINDOWING_KAISER25);
+    if (windowing <= GWY_WINDOWING_NONE)
+        return;
+
+    xres = field->xres;
+    yres = field->yres;
+    data = field->data;
+    if (xres == yres) {
+        wx = wy = g_new(gdouble, xres);
+        for (i = 0; i < xres; i++)
+            wx[i] = 1.0;
+        gwy_fft_window(xres, wx, windowing);
+    }
+    else {
+        wx = g_new(gdouble, xres + yres);
+        wy = wx + xres;
+        for (i = 0; i < xres + yres; i++)
+            wx[i] = 1.0;
+        gwy_fft_window(xres, wx, windowing);
+        gwy_fft_window(yres, wy, windowing);
+    }
+
+    for (i = 0; i < yres; i++) {
+        gdouble qy = wy[i];
+        for (j = 0; j < xres; j++)
+            data[i*xres + j] *= qy*wx[j];
+    }
+
+    g_free(wx);
+    gwy_data_field_invalidate(field);
+}
+
+/**
+ * gwy_data_field_fft_window_1d:
+ * @field: A two-dimensional data field.
+ * @orientation: Windowing orientation (the same as corresponding FFT orientation).
+ * @windowing: Windowing type to use.
+ *
+ * Performs row-wise or column-wise windowing of a data field in preparation for 1D FFT.
+ *
+ * Since: 2.62
+ **/
+void
+gwy_data_field_fft_window_1d(GwyDataField *field,
+                             GwyOrientation orientation,
+                             GwyWindowingType windowing)
+{
+    gint xres, yres, wres, j, i;
+    gdouble *data, *w = NULL;
+
+    g_return_if_fail(GWY_IS_DATA_FIELD(field));
+    g_return_if_fail(windowing <= GWY_WINDOWING_KAISER25);
+    g_return_if_fail(orientation == GWY_ORIENTATION_HORIZONTAL || orientation == GWY_ORIENTATION_VERTICAL);
+    if (windowing <= GWY_WINDOWING_NONE)
+        return;
+
+    xres = field->xres;
+    yres = field->yres;
+    data = field->data;
+    wres = (orientation == GWY_ORIENTATION_VERTICAL ? yres : xres);
+    w = g_new(gdouble, wres);
+    for (i = 0; i < wres; i++)
+        w[i] = 1.0;
+    gwy_fft_window(wres, w, windowing);
+
+    if (orientation == GWY_ORIENTATION_VERTICAL) {
+        for (i = 0; i < yres; i++) {
+            gdouble q = w[i];
+            for (j = 0; j < xres; j++)
+                data[i*xres + j] *= q;
+        }
+    }
+    else {
+        for (i = 0; i < yres; i++) {
+            for (j = 0; j < xres; j++)
+                data[i*xres + j] *= w[j];
+        }
+    }
+
+    g_free(w);
+    gwy_data_field_invalidate(field);
+}
+
 /************************** Documentation ****************************/
 
 /**
