@@ -1,5 +1,5 @@
 /*
- *  $Id: level.c 22788 2020-05-02 21:17:17Z yeti-dn $
+ *  $Id: level.c 23189 2021-02-23 17:32:52Z yeti-dn $
  *  Copyright (C) 2003-2020 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -205,6 +205,7 @@ gwy_data_field_area_fit_plane_mask(GwyDataField *data_field,
                     sumxz += x*z;
                     sumyz += y*z;
                     n += cc;
+                    mrow++;
                 }
             }
             else if (masking == GWY_MASK_EXCLUDE) {
@@ -224,6 +225,7 @@ gwy_data_field_area_fit_plane_mask(GwyDataField *data_field,
                     sumxz += x*z;
                     sumyz += y*z;
                     n += cc;
+                    mrow++;
                 }
             }
             else {
@@ -1844,6 +1846,7 @@ gwy_data_field_area_fit_local_planes(GwyDataField *data_field,
     g_return_val_if_fail(size > 1, NULL);
     if (!nresults)
         return NULL;
+    g_return_val_if_fail(nresults > 0, NULL);
     g_return_val_if_fail(types, NULL);
     for (ri = 0; ri < nresults; ri++) {
         g_return_val_if_fail(types[ri] >= GWY_PLANE_FIT_A
