@@ -1,5 +1,5 @@
 /*
- *  $Id: linestats.c 22728 2020-01-31 12:09:57Z yeti-dn $
+ *  $Id: linestats.c 24952 2022-08-26 15:24:56Z yeti-dn $
  *  Copyright (C) 2003-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -285,8 +285,7 @@ gwy_data_line_part_get_max(GwyDataLine *a,
     gdouble max = -G_MAXDOUBLE;
 
     g_return_val_if_fail(GWY_IS_DATA_LINE(a), max);
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_val_if_fail(from >= 0 && to <= a->res, max);
 
@@ -315,8 +314,7 @@ gwy_data_line_part_get_min(GwyDataLine *a,
     gdouble min = G_MAXDOUBLE;
 
     g_return_val_if_fail(GWY_IS_DATA_LINE(a), min);
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_val_if_fail(from >= 0 && to <= a->res, min);
 
@@ -358,8 +356,7 @@ gwy_data_line_part_get_min_max(GwyDataLine *a,
     if (!max || !min)
         return;
 
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     for (i = from; i < to; i++) {
         if (min1 > a->data[i])
@@ -424,8 +421,7 @@ gwy_data_line_part_get_rms(GwyDataLine *a, gint from, gint to)
     gdouble avg;
 
     g_return_val_if_fail(GWY_IS_DATA_LINE(a), rms);
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_val_if_fail(from >= 0 && to <= a->res, rms);
 
@@ -505,8 +501,7 @@ gwy_data_line_part_get_ra(GwyDataLine *a, gint from, gint to)
     gdouble avg;
 
     g_return_val_if_fail(GWY_IS_DATA_LINE(a), ra);
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_val_if_fail(from >= 0 && to <= a->res, ra);
 
@@ -535,8 +530,7 @@ gwy_data_line_part_get_skew(GwyDataLine *a, gint from, gint to)
     gdouble avg;
 
     g_return_val_if_fail(GWY_IS_DATA_LINE(a), skew);
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_val_if_fail(from >= 0 && to <= a->res, skew);
 
@@ -574,8 +568,7 @@ gwy_data_line_part_get_kurtosis(GwyDataLine *a, gint from, gint to)
     gdouble avg;
 
     g_return_val_if_fail(GWY_IS_DATA_LINE(a), kurtosis);
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_val_if_fail(from >= 0 && to <= a->res, kurtosis);
 
@@ -633,8 +626,7 @@ gwy_data_line_part_get_tan_beta0(GwyDataLine *a, gint from, gint to)
     gdouble rms = 0.0;
 
     g_return_val_if_fail(GWY_IS_DATA_LINE(a), rms);
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_val_if_fail(from >= 0 && to <= a->res, rms);
 
@@ -672,8 +664,7 @@ gwy_data_line_part_get_variation(GwyDataLine *a, gint from, gint to)
     gdouble var = 0.0;
 
     g_return_val_if_fail(GWY_IS_DATA_LINE(a), var);
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_val_if_fail(from >= 0 && to <= a->res, var);
 
@@ -721,8 +712,7 @@ gwy_data_line_part_get_sum(GwyDataLine *a, gint from, gint to)
     gdouble sum = 0;
 
     g_return_val_if_fail(GWY_IS_DATA_LINE(a), sum);
-    if (to < from)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
 
     g_return_val_if_fail(from >= 0 && to <= a->res, sum);
 
@@ -925,8 +915,7 @@ gwy_data_line_distribution(GwyDataLine *data_line,
     g_return_if_fail(GWY_IS_DATA_LINE(distribution));
 
     /* Find reasonable binning */
-    if (ymin > ymax)
-        GWY_SWAP(gdouble, ymin, ymax);
+    GWY_ORDER(gdouble, ymin, ymax);
 
     /* if ymin == ymax == 0 use the full range */
     if (!ymin && !ymax) {
@@ -1214,8 +1203,7 @@ gwy_data_line_part_get_modus(GwyDataLine *data_line,
     g_return_val_if_fail(from >= 0 && to <= data_line->res, 0);
     g_return_val_if_fail(from != to, 0);
 
-    if (from > to)
-        GWY_SWAP(gint, from, to);
+    GWY_ORDER(gint, from, to);
     n = to - from;
 
     if (n == 1)
