@@ -1,5 +1,5 @@
 /*
- *  $Id: mfm.c 21625 2018-11-13 13:58:59Z yeti-dn $
+ *  $Id: mfm.c 22422 2019-08-20 12:17:42Z klapetek $
  *  Copyright (C) 2016-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -243,7 +243,7 @@ mfm_create_ztf_from_frequencies(GwyDataField *ztf, GwyDataField *freq,
     fdata = freq->data;
     n = ztf->xres * ztf->yres;
     for (i = 0; i < n; i++)
-        data[i] = exp(fdata[i]*zdiff);
+        data[i] = exp(2*M_PI*fdata[i]*zdiff);
     gwy_data_field_invalidate(ztf);
 }
 
@@ -267,7 +267,7 @@ mfm_perpendicular_create_field_mask(GwyDataField *fieldmask,
     data = fieldmask->data;
     n = fieldmask->xres*fieldmask->yres;
     for (i = 0; i < n; i++)
-        data[i] = exp(-2*M_PI*data[i]*height)*(1.0 - exp(-2*M_PI*data[i]*thickness)); //0.5 should not be there
+        data[i] = 0.5*exp(-2*M_PI*data[i]*height)*(1.0 - exp(-2*M_PI*data[i]*thickness)); //doubts about 0.5 before meff/hz split
     gwy_data_field_invalidate(fieldmask);
 }
 
@@ -323,7 +323,7 @@ gwy_data_field_mfm_perpendicular_stray_field(GwyDataField *mfield,
                                          FALSE);
     }
 
-    
+
     gwy_data_field_2dfft_raw(rea, NULL, reb, imb,
                              GWY_TRANSFORM_DIRECTION_FORWARD);
     mfm_perpendicular_create_field_mask(fieldmask, height, thickness);
@@ -333,7 +333,7 @@ gwy_data_field_mfm_perpendicular_stray_field(GwyDataField *mfield,
                              GWY_TRANSFORM_DIRECTION_BACKWARD);
     gwy_data_field_resample(out, mfield->xres, mfield->yres,
                             GWY_INTERPOLATION_NONE);
-                            
+
     gwy_data_field_copy(rea, out, TRUE);
 
     gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(out), "A/m");
@@ -346,6 +346,88 @@ gwy_data_field_mfm_perpendicular_stray_field(GwyDataField *mfield,
     GWY_OBJECT_UNREF(wallmask);
 }
 
+
+/**
+ * gwy_data_field_mfm_perpendicular_stray_field_angle_correction:
+ * @field: Field to be processed. It will be changed by the correction.
+ * @out: Cantilever angle in degrees.
+ * @orientation: Cantilever orientation with respect of the data.
+ *
+ * Performs correction of magnetic data for cantilever tilt.
+ *
+ * Since: 2.54
+ **/
+void
+gwy_data_field_mfm_perpendicular_stray_field_angle_correction(GwyDataField *field,
+                                                      gdouble angle,
+                                                      GwyOrientation orientation)
+{
+    GwyDataField *rea, *ima, *reb, *imb, *kx, *ky;
+    gdouble *rd, *id, *kxdata, *kydata, kval, kdirval, theta, ctheta, stheta;
+    gint n, i;
+    gdouble a, b, c, d;
+
+    g_return_if_fail(GWY_IS_DATA_FIELD(field));
+
+    rea = gwy_data_field_new_alike(field, TRUE);
+    reb = gwy_data_field_new_alike(field, TRUE);
+    ima = gwy_data_field_new_alike(field, TRUE);
+    imb = gwy_data_field_new_alike(field, TRUE);
+    kx = gwy_data_field_new_alike(field, FALSE);
+    ky = gwy_data_field_new_alike(field, FALSE);
+
+    gwy_data_field_copy(field, rea, FALSE);
+    gwy_data_field_2dfft_raw(rea, NULL, reb, imb,
+                             GWY_TRANSFORM_DIRECTION_FORWARD);
+
+    precaulcate_xy_frequency_fields(rea, kx, ky);
+    kxdata = kx->data;
+    kydata = ky->data;
+    rd = reb->data;
+    id = imb->data;
+
+    theta = angle*M_PI/180;
+    ctheta = cos(theta);
+    stheta = sin(theta);
+    n = reb->xres*reb->yres;
+    for (i = 0; i < n; i++) {
+
+        a = rd[i];
+        b = id[i];
+        kval = sqrt(kxdata[i]*kxdata[i] + kydata[i]*kydata[i]);
+
+        if (orientation == GWY_ORIENTATION_HORIZONTAL)
+            kdirval = kxdata[i];
+        else
+            kdirval = kydata[i];
+
+        if (kval == 0)
+            c = ctheta*ctheta;
+        else
+            c = ctheta*ctheta - kdirval*kdirval*stheta*stheta/kval/kval;
+
+        if (kval == 0)
+            d = 0;
+        else
+            d = 2*kdirval*stheta*ctheta/kval;
+
+        rd[i] = a*c - b*d;
+        id[i] = a*d + b*c;
+    }
+
+    gwy_data_field_2dfft_raw(reb, imb, rea, ima,
+                             GWY_TRANSFORM_DIRECTION_BACKWARD);
+    gwy_data_field_copy(rea, field, TRUE);
+
+    g_object_unref(rea);
+    g_object_unref(reb);
+    g_object_unref(ima);
+    g_object_unref(imb);
+    g_object_unref(kx);
+    g_object_unref(ky);
+}
+
+
 /**
  * gwy_data_field_mfm_perpendicular_medium_force:
  * @hz: Data field contaning the Z-component of the magnetic H field.
@@ -571,7 +653,7 @@ gwy_data_field_mfm_find_shift_z(GwyDataField *dfield,
 
 /**
  * gwy_data_field_mfm_parallel_medium:
- * @hfield: Resulting array. 
+ * @hfield: Resulting array.
  * @height: Height above surface.
  * @size_a: Left direction oriented area width.
  * @size_b: Right direction orientated area width.
@@ -700,7 +782,7 @@ gwy_data_field_mfm_parallel_medium(GwyDataField *hfield,
  *
  * Calculates magnetic field or its derivatives above a flat current line (stripe).
  * Results are added to the @hfield array, so it should be cleared if
- * function is run only once. 
+ * function is run only once.
  *
  * Since: 2.51
  **/
