@@ -1,5 +1,5 @@
 /*
- *  $Id: spectra.c 21680 2018-11-26 10:39:39Z yeti-dn $
+ *  $Id: spectra.c 23204 2021-03-08 15:31:59Z yeti-dn $
  *  Copyright (C) 2006-2018 Owain Davies, David Necas (Yeti), Petr Klapetek.
  *  E-mail: owain.davies@blueyonder.co.uk
  *          yeti@gwyddion.net, klapetek@gwyddion.net.
@@ -566,8 +566,7 @@ gwy_spectra_clone_real(GObject *source, GObject *copy)
     clone = GWY_SPECTRA(copy);
 
     /* Title */
-    g_free(clone->title);
-    clone->title = g_strdup(spectra->title);
+    gwy_assign_string(&clone->title, spectra->title);
 
     /* Remove any existing datalines in the clone */
     for (i = 0; i < clone->spectra->len; i++) {
@@ -1109,14 +1108,9 @@ void
 gwy_spectra_set_title(GwySpectra *spectra,
                       const gchar *title)
 {
-    gchar *old;
-
     g_return_if_fail(GWY_IS_SPECTRA(spectra));
 
-    old = spectra->title;
-    spectra->title = g_strdup(title);
-    g_free(old);
-
+    gwy_assign_string(&spectra->title, title);
     g_object_notify(G_OBJECT(spectra), "title");
 }
 
@@ -1151,19 +1145,8 @@ void
 gwy_spectra_set_spectrum_x_label(GwySpectra *spectra,
                                  const gchar *label)
 {
-    gchar *old;
-
     g_return_if_fail(GWY_IS_SPECTRA(spectra));
-
-    if (!label && !spectra->spec_xlabel)
-        return;
-    if (label && spectra->spec_xlabel && gwy_strequal(label,
-                                                      spectra->spec_xlabel))
-        return;
-
-    old = spectra->spec_xlabel;
-    spectra->spec_xlabel = g_strdup(label);
-    g_free(old);
+    gwy_assign_string(&spectra->spec_xlabel, label);
 }
 
 /**
@@ -1197,19 +1180,8 @@ void
 gwy_spectra_set_spectrum_y_label(GwySpectra *spectra,
                                  const gchar *label)
 {
-    gchar *old;
-
     g_return_if_fail(GWY_IS_SPECTRA(spectra));
-
-    if (!label && !spectra->spec_ylabel)
-        return;
-    if (label && spectra->spec_ylabel && gwy_strequal(label,
-                                                      spectra->spec_ylabel))
-        return;
-
-    old = spectra->spec_ylabel;
-    spectra->spec_ylabel = g_strdup(label);
-    g_free(old);
+    gwy_assign_string(&spectra->spec_ylabel, label);
 }
 
 /**
