@@ -1,5 +1,5 @@
 /*
- *  $Id: stats-acfpsdf.c 24752 2022-03-29 10:34:55Z yeti-dn $
+ *  $Id: stats-acfpsdf.c 24832 2022-05-19 13:43:16Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -971,8 +971,7 @@ gwy_data_field_area_2dpsdf_mask(GwyDataField *field,
             gwy_data_field_add(target, -avg);
         }
         rms = gwy_data_field_area_get_rms_mask(target, weights, masking, 0, 0, width, height);
-        gwy_fft_window_data_field(target, GWY_ORIENTATION_HORIZONTAL, windowing);
-        gwy_fft_window_data_field(target, GWY_ORIENTATION_VERTICAL, windowing);
+        gwy_data_field_fft_window(target, windowing);
 
         newrms = gwy_data_field_area_get_rms_mask(target, weights, masking, 0, 0, width, height);
         if (newrms > 0.0)
@@ -1031,10 +1030,7 @@ gwy_data_field_area_2dpsdf_mask(GwyDataField *field,
             gwy_data_field_add(target, -avg);
         }
         rms = gwy_data_field_get_rms(target);
-
-        gwy_fft_window_data_field(target, GWY_ORIENTATION_HORIZONTAL, windowing);
-        gwy_fft_window_data_field(target, GWY_ORIENTATION_VERTICAL, windowing);
-
+        gwy_data_field_fft_window(target, windowing);
         newrms = gwy_data_field_get_rms(target);
         if (newrms > 0.0)
             gwy_data_field_multiply(target, rms/newrms);
@@ -1200,7 +1196,7 @@ gwy_data_field_psdf_to_angular_spectrum(GwyDataField *psdf,
  *
  * The result of such averaging is an radial profile, starting from the disc centre.
  *
- * The function does not guarantee that @target_line will have exactly @nstats samples upon return.  A cmaller number
+ * The function does not guarantee that @target_line will have exactly @nstats samples upon return.  A smaller number
  * of samples than requested may be calculated for instance if either central or outer part of the disc is excluded by
  * masking.
  *
