@@ -1,5 +1,5 @@
 /*
- *  $Id: stats-sum.c 22790 2020-05-02 21:38:57Z yeti-dn $
+ *  $Id: stats-sum.c 23050 2021-01-11 11:22:44Z yeti-dn $
  *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -608,6 +608,201 @@ gwy_data_field_area_get_surface_area_mask(GwyDataField *data_field,
                                   col, row, width, height);
 }
 
+static gdouble
+calculate_rms_surface_slope(GwyDataField *dfield,
+                            GwyDataField *mask,
+                            GwyMaskingType mode,
+                            gint col, gint row,
+                            gint width, gint height)
+{
+    const gdouble *dataul, *maskul, *r, *m;
+    gint i, j, xres, n;
+    gdouble dx, dy, sumh, sumv, dz;
+
+    /* special cases */
+    if (!width || !height)
+        return 0.0;
+
+    xres = dfield->xres;
+    dx = gwy_data_field_get_dx(dfield);
+    dy = gwy_data_field_get_dy(dfield);
+    dataul = dfield->data + xres*row + col;
+    sumv = sumh = 0.0;
+    n = 0;
+
+    /* The gradient has two contributions, from horizontal derivatives and
+     * from vertical derivatives.  We could require both to be calculable for
+     * masked areas, but we do not.  Each gradient is calculate separately
+     * for each pixels where it is defined, regardless whether the orthogonal
+     * one can be calculated.  This can give uneven contributions for some
+     * odd mask shapes… */
+    if (mask && mode != GWY_MASK_IGNORE) {
+        maskul = mask->data + xres*row + col;
+#ifdef _OPENMP
+#pragma omp parallel for if(gwy_threads_are_enabled()) default(none) \
+            reduction(+:sumh,sumv,n) \
+            private(r,m,i,j,dz) \
+            shared(dataul,maskul,xres,height,width,mode)
+#endif
+        for (i = 0; i < height-1; i++) {
+            r = dataul + xres*i;
+            m = maskul + xres*i;
+            if (mode == GWY_MASK_INCLUDE) {
+                /* Inside. */
+                for (j = 0; j < width-1; j++) {
+                    if (m[j] > 0.0 && m[xres+j] > 0.0) {
+                        dz = r[xres+j] - r[j];
+                        sumv += dz*dz;
+                        n++;
+                    }
+                    if (m[j] > 0.0 && m[j+1] > 0.0) {
+                        dz = r[j+1] - r[j];
+                        sumh += dz*dz;
+                        n++;
+                    }
+                }
+                /* Last-column vertical gradient. */
+                if (m[j] > 0.0 && m[xres+j] > 0.0) {
+                    dz = r[xres+j] - r[j];
+                    sumv += dz*dz;
+                    n++;
+                }
+            }
+            else {
+                /* Inside. */
+                for (j = 0; j < width-1; j++) {
+                    if (m[j] <= 0.0 && m[xres+j] <= 0.0) {
+                        dz = r[xres+j] - r[j];
+                        sumv += dz*dz;
+                        n++;
+                    }
+                    if (m[j] <= 0.0 && m[j+1] <= 0.0) {
+                        dz = r[j+1] - r[j];
+                        sumh += dz*dz;
+                        n++;
+                    }
+                }
+                /* Last-column vertical gradient. */
+                if (m[j] <= 0.0 && m[xres+j] <= 0.0) {
+                    dz = r[xres+j] - r[j];
+                    sumv += dz*dz;
+                    n++;
+                }
+            }
+        }
+        /* Last-row horizontal gradient. */
+        i = height-1;
+        r = dataul + xres*i;
+        m = maskul + xres*i;
+        if (mode == GWY_MASK_INCLUDE) {
+            for (j = 0; j < width-1; j++) {
+                if (m[j] > 0.0 && m[j+1] > 0.0) {
+                    dz = r[j+1] - r[j];
+                    sumh += dz*dz;
+                    n++;
+                }
+            }
+        }
+        else {
+            for (j = 0; j < width-1; j++) {
+                if (m[j] <= 0.0 && m[j+1] <= 0.0) {
+                    dz = r[j+1] - r[j];
+                    sumh += dz*dz;
+                    n++;
+                }
+            }
+        }
+        if (!n)
+            return 0.0;
+    }
+    else {
+#ifdef _OPENMP
+#pragma omp parallel for if(gwy_threads_are_enabled()) default(none) \
+            reduction(+:sumh,sumv) \
+            private(r,i,j,dz) \
+            shared(dataul,xres,width,height)
+#endif
+        for (i = 0; i < height-1; i++) {
+            r = dataul + xres*i;
+            /* Inside. */
+            for (j = 0; j < width-1; j++) {
+                dz = r[xres+j] - r[j];
+                sumv += dz*dz;
+                dz = r[j+1] - r[j];
+                sumh += dz*dz;
+            }
+            /* Last-column vertical gradient. */
+            dz = r[xres+j] - r[j];
+            sumv += dz*dz;
+        }
+        /* Last-row horizontal gradient. */
+        i = height-1;
+        r = dataul + xres*i;
+        for (j = 0; j < width-1; j++) {
+            dz = r[j+1] - r[j];
+            sumh += dz*dz;
+        }
+        n = (width - 1)*height + width*(height - 1);
+    }
+
+    return sqrt(2.0*(sumh/(dx*dx) + sumv/(dy*dy))/n);
+}
+
+/**
+ * gwy_data_field_area_get_surface_slope_mask:
+ * @data_field: A data field.
+ * @mask: Mask specifying which values to take into account/exclude, or %NULL.
+ * @mode: Masking mode to use.  See the introduction for description of
+ *        masking modes.
+ * @col: Upper-left column coordinate.
+ * @row: Upper-left row coordinate.
+ * @width: Area width (number of columns).
+ * @height: Area height (number of rows).
+ *
+ * Computes root mean square surface slope (Sdq) of a rectangular part of
+ * a data field.
+ *
+ * Returns: The root mean square surface slope.
+ *
+ * Since: 2.58
+ **/
+gdouble
+gwy_data_field_area_get_surface_slope_mask(GwyDataField *data_field,
+                                           GwyDataField *mask,
+                                           GwyMaskingType mode,
+                                           gint col, gint row,
+                                           gint width, gint height)
+{
+    gdouble Sdq = 0.0;
+
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height)
+        || !_gwy_data_field_check_mask(data_field, &mask, &mode))
+        return Sdq;
+
+    return calculate_rms_surface_slope(data_field, mask, mode,
+                                       col, row, width, height);
+}
+
+/**
+ * gwy_data_field_get_surface_slope:
+ * @data_field: A data field.
+ *
+ * Computes root mean square surface slope (Sdq) of a data field.
+ *
+ * Returns: The root mean square surface slope.
+ *
+ * Since: 2.58
+ **/
+gdouble
+gwy_data_field_get_surface_slope(GwyDataField *data_field)
+{
+    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), 0.0);
+
+    return calculate_rms_surface_slope(data_field, NULL, GWY_MASK_IGNORE,
+                                       0, 0,
+                                       data_field->xres, data_field->yres);
+}
+
 /**
  * square_var1:
  * @z1: Z-value in first corner.
