@@ -1,5 +1,5 @@
 /*
- *  $Id: stats.h 22726 2020-01-31 10:55:32Z yeti-dn $
+ *  $Id: stats.h 23046 2021-01-08 12:21:05Z yeti-dn $
  *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -37,6 +37,7 @@ gdouble gwy_data_field_get_mean_square      (GwyDataField *data_field);
 gdouble gwy_data_field_get_sum              (GwyDataField *data_field);
 gdouble gwy_data_field_get_median           (GwyDataField *data_field);
 gdouble gwy_data_field_get_surface_area     (GwyDataField *data_field);
+gdouble gwy_data_field_get_surface_slope    (GwyDataField *data_field);
 gdouble gwy_data_field_get_variation        (GwyDataField *data_field);
 gdouble gwy_data_field_get_entropy          (GwyDataField *data_field);
 gdouble gwy_data_field_get_entropy_2d       (GwyDataField *xfield,
@@ -154,6 +155,13 @@ gdouble gwy_data_field_area_get_surface_area_mask(GwyDataField *data_field,
                                                   gint row,
                                                   gint width,
                                                   gint height);
+gdouble gwy_data_field_area_get_surface_slope_mask(GwyDataField *data_field,
+                                                   GwyDataField *mask,
+                                                   GwyMaskingType mode,
+                                                   gint col,
+                                                   gint row,
+                                                   gint width,
+                                                   gint height);
 gdouble gwy_data_field_area_get_mean_square (GwyDataField *data_field,
                                              GwyDataField *mask,
                                              GwyMaskingType mode,
