@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_fdfit.c 24495 2021-11-09 10:34:28Z yeti-dn $
+ *  $Id: cmap_fdfit.c 24531 2021-11-15 20:09:29Z klapetek $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -54,6 +54,12 @@ enum {
     PARAM_FUNCTION,
     WIDGET_FIT_PARAMETERS,
     PARAM_INFO,
+    PARAM_ESTIMATE,
+    PARAM_ADHESION,
+    PARAM_SEGMENT_ADHESION,
+    PARAM_SEGMENT_BASELINE,
+    PARAM_BASELINE_RANGE,
+
 };
 
 typedef struct {
@@ -65,6 +71,9 @@ typedef struct {
     gboolean *param_fixed;
     GwyDataField **result;
     GwyDataField *mask;
+    gint adhesion_index;
+    gdouble xmin;
+    gdouble xmax;
 } ModuleArgs;
 
 typedef struct {
@@ -85,6 +94,7 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table;
     GwyParamTable *table_fit;
+    GwyParamTable *table_optimize;
     GtkWidget *fit_param_table;
     GwyContainer *data;
     GwySelection *selection;
@@ -110,12 +120,17 @@ static void             param_changed           (ModuleGUI *gui,
                                                  gint id);
 static void             param_fit_changed       (ModuleGUI *gui,
                                                  gint id);
+static void             param_optimize_changed  (ModuleGUI *gui,
+                                                 gint id);
 static void             preview                 (gpointer user_data);
 static void             set_selection           (ModuleGUI *gui);
 static GtkWidget*       create_fit_table        (gpointer user_data);
 static void             point_selection_changed (ModuleGUI *gui,
                                                  gint id,
                                                  GwySelection *selection);
+static void             graph_selected          (GwySelection* selection, 
+                                                 gint i, 
+                                                 ModuleGUI *gui);
 static void             extract_one_curve       (GwyLawn *lawn,
                                                  GwyGraphCurveModel *gcmodel,
                                                  gint col,
@@ -135,6 +150,7 @@ static void             fit_one_curve           (GwyGraphCurveModel *gcmodel,
                                                  gdouble *error,
                                                  gint nsegments,
                                                  const gint *segments,
+                                                 gint adhesion_index,
                                                  gboolean *fitok);
 static void             do_fdfit                (const gdouble *xdata,
                                                  const gdouble *ydata,
@@ -143,6 +159,11 @@ static void             do_fdfit                (const gdouble *xdata,
                                                  const gint *segments,
                                                  gint segment,
                                                  gboolean segment_enabled,
+                                                 gboolean adhesion,
+                                                 gint adhesion_segment,
+                                                 gint baseline_segment,
+                                                 gdouble adhesion_range,
+                                                 gint adhesion_index,
                                                  gdouble from,
                                                  gdouble to,
                                                  gdouble *retparam,
@@ -211,6 +232,11 @@ define_module_params(void)
     gwy_param_def_add_double(paramdef, PARAM_RANGE_TO, "to", _("_To"), 0.0, 1.0, 1.0);
     gwy_param_def_add_lawn_segment(paramdef, PARAM_SEGMENT, "segment", NULL);
     gwy_param_def_add_boolean(paramdef, PARAM_ENABLE_SEGMENT, "enable_segment", NULL, FALSE);
+    gwy_param_def_add_boolean(paramdef, PARAM_ESTIMATE, "estimate", _("Run _estimate at each point"), FALSE);
+    gwy_param_def_add_boolean(paramdef, PARAM_ADHESION, "adhesion", _("Get adhesion directly"), FALSE);
+    gwy_param_def_add_lawn_segment(paramdef, PARAM_SEGMENT_ADHESION, "segment_adhesion", _("Adhesion data"));
+    gwy_param_def_add_lawn_segment(paramdef, PARAM_SEGMENT_BASELINE, "segment_baseline", _("Baseline data"));
+    gwy_param_def_add_double(paramdef, PARAM_BASELINE_RANGE, "baseline", _("Baseline _range"), 0.0, 0.5, 0.2);
 
     return paramdef;
 }
@@ -238,6 +264,9 @@ fdfit(GwyContainer *data, GwyRunType runtype)
     args.lawn = lawn;
     args.nsegments = gwy_lawn_get_n_segments(lawn);
     args.params = gwy_params_new_from_settings(define_module_params());
+    args.adhesion_index = -1;
+    args.xmin = 0;
+    args.xmax = 0;
     sanitise_params(&args);
 
     args.field = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
@@ -352,7 +381,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     area = gwy_graph_get_area(GWY_GRAPH(graph));
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gwy_graph_area_set_status(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XSEL);
-    gwy_graph_area_set_selection_editable(GWY_GRAPH_AREA(area), FALSE);
+    gwy_graph_area_set_selection_editable(GWY_GRAPH_AREA(area), TRUE);
     gui.graph_selection = gwy_graph_area_get_selection(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XSEL);
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
@@ -388,16 +417,31 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     gwy_dialog_add_param_table(dialog, table);
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
-
-
+   
+    table = gui.table_optimize = gwy_param_table_new(args->params);
+    gwy_param_table_append_checkbox(table, PARAM_ESTIMATE);
+    gwy_param_table_append_checkbox(table, PARAM_ADHESION);
+    if (args->nsegments) {
+        gwy_param_table_append_lawn_segment(table, PARAM_SEGMENT_ADHESION, args->lawn);
+        gwy_param_table_append_lawn_segment(table, PARAM_SEGMENT_BASELINE, args->lawn);
+    }
+    gwy_param_table_append_slider(table, PARAM_BASELINE_RANGE);
+    gwy_param_table_slider_set_factor(table, PARAM_BASELINE_RANGE, 100.0);
+    gwy_param_table_set_unitstr(table, PARAM_BASELINE_RANGE, "%");
+  
+    gwy_dialog_add_param_table(dialog, table);
+    gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
+ 
     g_signal_connect_swapped(gui.table, "param-changed", G_CALLBACK(param_changed), &gui);
     g_signal_connect_swapped(gui.table_fit, "param-changed", G_CALLBACK(param_fit_changed), &gui);
+    g_signal_connect_swapped(gui.table_optimize, "param-changed", G_CALLBACK(param_optimize_changed), &gui);
     g_signal_connect_swapped(gui.selection, "changed", G_CALLBACK(point_selection_changed), &gui);
     g_signal_connect_swapped(dialog, "response", G_CALLBACK(dialog_response), &gui);
+    g_signal_connect(gui.graph_selection, "changed", G_CALLBACK(graph_selected), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
     gwy_param_table_param_changed(gui.table_fit, PARAM_FUNCTION);
-
+    gwy_param_table_param_changed(gui.table_optimize, PARAM_ADHESION);
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.gmodel);
@@ -480,23 +524,17 @@ dialog_response(ModuleGUI *gui, gint response)
         //printf("fit single\n");
 
         nparams = gwy_nlfit_preset_get_nparams(gui->preset);
-        //printf("we have %d params\n", nparams);
-        //for (i = 0; i < nparams; i++)
-        //    printf("fix %d  value %g\n", args->param_fixed[i], args->fit_parameters[i]);
-
         nparams = CLAMP(nparams, 0, 1024);  // prevent silly GCC warning
 
-//        error = (gdouble *)malloc(nparams*sizeof(gdouble));
         error = g_new(gdouble, nparams);
         for (i = 0; i < nparams; i++) {
             error[i] = 0;
-            //printf("wtf %g %g %g %g\n", error[0], error[1], error[2], error[3]);
         }
 
         gc = gwy_graph_model_get_curve(gui->gmodel, 0);
         extract_one_curve(args->lawn, gc, col, row, params);
         fit_one_curve(gc, params, gui->preset, args->fit_parameters, args->param_fixed, error, args->nsegments,
-                      gwy_lawn_get_segments(args->lawn, col, row, NULL), &fitok);
+                      gwy_lawn_get_segments(args->lawn, col, row, NULL), args->adhesion_index, &fitok);
 
         for (i = 0; i < nparams; i++) {
            cntrl = &g_array_index(gui->param_controls, FitParamControl, i);
@@ -519,6 +557,39 @@ dialog_response(ModuleGUI *gui, gint response)
     }
 }
 
+static void
+graph_selected(GwySelection* selection, gint i, ModuleGUI *gui)
+{
+    ModuleArgs *args = gui->args;
+    gdouble range[2];
+    gdouble xfrom, xto, pfrom, pto;
+    gboolean have_range = TRUE;
+
+    g_return_if_fail(i <= 0);
+
+    if (gwy_selection_get_data(selection, NULL) <= 0)
+        have_range = FALSE;
+    else {
+        gwy_selection_get_object(selection, 0, range);
+        if (range[0] == range[1])
+            have_range = FALSE;
+    }
+    if (have_range) {
+        xfrom = MIN(range[0], range[1]);
+        xto = MAX(range[0], range[1]);
+    }
+    else {
+        xfrom = args->xmin;
+        xto = args->xmax;
+    }
+
+    pfrom = CLAMP((xfrom - args->xmin)/(args->xmax - args->xmin), 0.0, 1.0);
+    pto = CLAMP((xto - args->xmin)/(args->xmax - args->xmin), 0.0, 1.0);
+
+    gwy_param_table_set_double(gui->table, PARAM_RANGE_FROM, pfrom);
+    gwy_param_table_set_double(gui->table, PARAM_RANGE_TO, pto);
+}
+
 
 static void
 param_changed(ModuleGUI *gui, G_GNUC_UNUSED gint id)
@@ -541,8 +612,14 @@ param_fit_changed(ModuleGUI *gui, gint id)
         gui->args->fit_parameters = g_renew(gdouble, gui->args->fit_parameters, nparams);
         gui->args->param_fixed = g_renew(gboolean, gui->args->param_fixed, nparams);
 
-        for (i = 0; i < nparams; i++)  //FIXME, should we do this?
+        gui->args->adhesion_index = -1;
+        for (i = 0; i < nparams; i++) {
             gui->args->param_fixed[i] = 0;
+            
+            if (strcmp(gwy_nlfit_preset_get_param_name(gui->preset, i), "Fad") == 0) {
+                gui->args->adhesion_index = i;
+            }
+        }
 
         fit_param_table_resize(gui);
     }
@@ -560,6 +637,29 @@ fix_changed(GtkToggleButton *button, ModuleGUI *gui)
     gui->args->param_fixed[i] = fixed;
 }
 
+static void
+param_optimize_changed(ModuleGUI *gui, gint id)
+{
+    ModuleArgs *args = gui->args;
+    GwyParams *params = args->params;
+    FitParamControl *cntrl;
+    gboolean use_adhesion;
+
+    if (id == PARAM_ADHESION && args->nsegments) {
+        use_adhesion = gwy_params_get_boolean(params, PARAM_ADHESION);
+
+        gwy_param_table_set_sensitive(gui->table_optimize, PARAM_SEGMENT_ADHESION, use_adhesion);
+        gwy_param_table_set_sensitive(gui->table_optimize, PARAM_SEGMENT_BASELINE, use_adhesion);
+        gwy_param_table_set_sensitive(gui->table_optimize, PARAM_BASELINE_RANGE, use_adhesion);
+
+        if (gui->args->adhesion_index >= 0) {
+           cntrl = &g_array_index(gui->param_controls, FitParamControl, gui->args->adhesion_index);
+           gtk_widget_set_sensitive(cntrl->value, !use_adhesion);
+           if (!use_adhesion) fix_changed(GTK_TOGGLE_BUTTON(cntrl->fix), gui);
+        }
+    }
+}
+
 static void
 param_value_edited(GtkEntry *entry, ModuleGUI *gui)
 {
@@ -697,6 +797,8 @@ preview(gpointer user_data)
     extract_one_curve(args->lawn, gc, col, row, params);
     update_graph_model_props(gui->gmodel, args);
     gwy_graph_curve_model_get_x_range(gc, &xfrom, &xto);
+    args->xmin = xfrom;
+    args->xmax = xto;
     sel[0] = xfrom + from*(xto-xfrom);
     sel[1] = xfrom + to*(xto-xfrom);
 
@@ -716,6 +818,11 @@ execute(ModuleArgs *args, GtkWindow *window)
     gdouble to = gwy_params_get_double(params, PARAM_RANGE_TO);
     gboolean segment_enabled = args->nsegments ? gwy_params_get_boolean(params, PARAM_ENABLE_SEGMENT) : FALSE;
     gint segment = segment_enabled ? gwy_params_get_int(params, PARAM_SEGMENT) : -1;
+    gint adhesion = gwy_params_get_boolean(params, PARAM_ADHESION);
+    gint adhesion_segment = adhesion ? gwy_params_get_int(params, PARAM_SEGMENT_ADHESION) : -1;
+    gint baseline_segment = adhesion ? gwy_params_get_int(params, PARAM_SEGMENT_BASELINE) : -1;
+    gdouble baseline_range = gwy_params_get_double(params, PARAM_BASELINE_RANGE);
+    gboolean estimate = gwy_params_get_boolean(params, PARAM_ESTIMATE);
 
     GwyNLFitPreset *preset = gwy_inventory_get_item(gwy_fd_curve_presets(),
                                                     gwy_params_get_string(params, PARAM_FUNCTION));
@@ -768,9 +875,17 @@ execute(ModuleArgs *args, GtkWindow *window)
         cdx = cd + ndata*abscissa;
         cdy = cd + ndata*ordinate;
 
+        if (estimate) 
+            do_fdestimate(cdx, cdy, ndata, preset,
+                          gwy_lawn_get_segments(lawn, col, row, NULL),
+                          segment, segment_enabled,
+                          from, to, inits);
+
         do_fdfit(cdx, cdy, ndata, preset,
                  gwy_lawn_get_segments(lawn, col, row, NULL),
                  segment, segment_enabled,
+                 adhesion, adhesion_segment, baseline_segment, baseline_range,
+                 args->adhesion_index,
                  from, to, inits,
                  args->param_fixed,
                  NULL,
@@ -886,6 +1001,8 @@ do_fdfit(const gdouble *xdata, const gdouble *ydata,
          gint ndata, GwyNLFitPreset *preset,
          const gint *segments,
          gint segment, gboolean segment_enabled,
+         gboolean use_adhesion, gint adhesion_segment, gint baseline_segment, gdouble baseline_range,
+         gint adhesion_index,
          gdouble from, gdouble to,
          gdouble *fitparams, gboolean *fix, gdouble *error,
          gboolean *fitok)
@@ -895,6 +1012,10 @@ do_fdfit(const gdouble *xdata, const gdouble *ydata,
     gdouble *xf, *yf;
     GwyNLFitter *fitter;
     gint segment_from, segment_to;
+    gint baseline_from, baseline_to, adhesion_from, adhesion_to, nbaseline;
+    gdouble bfrom, bto;
+    const gdouble *yadata, *xbdata, *ybdata;
+    gdouble adhesion, baseline;
 
     //get the total range and fit range
     xmin = G_MAXDOUBLE;
@@ -922,8 +1043,57 @@ do_fdfit(const gdouble *xdata, const gdouble *ydata,
         segment_to = G_MAXINT;
     }
 
+    if (use_adhesion) {
+       //get baseline curve range
+       baseline_from = segments[2*baseline_segment];
+       baseline_to = segments[2*baseline_segment + 1];
+       adhesion_from = segments[2*adhesion_segment];
+       adhesion_to = segments[2*adhesion_segment + 1];
 
-    //determine number of points to fit,  FIXME: add segment here
+       xbdata = xdata + baseline_from;
+       ybdata = ydata + baseline_from;
+       yadata = ydata + adhesion_from;
+
+       bfrom = G_MAXDOUBLE;
+       bto = -G_MAXDOUBLE;
+       for (i = 0; i < (baseline_to - baseline_from); i++) {
+           if (xbdata[i] < bfrom)
+               bfrom = xbdata[i];
+           if (xbdata[i] > bto)
+               bto = xbdata[i];
+       }
+
+       //fit baseline - average value on the curve flat part
+       baseline = 0;
+       nbaseline = 0;
+       for (i = 0; i < (baseline_to - baseline_from); i++) {
+           if (xbdata[i] > (bto - baseline_range*(bto - bfrom))) {
+               baseline += ybdata[i];
+               nbaseline++;
+           }
+       }
+       if (nbaseline>0) baseline /= nbaseline;
+       else {
+           baseline = ybdata[baseline_to-baseline_from-1];
+       }
+
+       //get adhesion as minimum on the curve
+       adhesion = G_MAXDOUBLE;
+       for (i = 0; i < (adhesion_to - adhesion_from); i++) {
+           if (adhesion > yadata[i]) { 
+               adhesion = yadata[i]; 
+           }
+       }       
+       adhesion -= baseline;
+
+       if (adhesion_index >=0) {
+           fitparams[adhesion_index] = adhesion;
+           fix[adhesion_index] = 1;
+       }
+
+    }
+
+    //determine number of points to fit
     n = 0;
     for (i = 0; i < ndata; i++) {
         if (xdata[i] >= startval && xdata[i] < endval
@@ -966,21 +1136,29 @@ do_fdfit(const gdouble *xdata, const gdouble *ydata,
 static void
 fit_one_curve(GwyGraphCurveModel *gcmodel, GwyParams *params, GwyNLFitPreset *preset,
               gdouble *fitparams, gboolean *fix, gdouble *error, gint nsegments, const gint *segments,
+              gint adhesion_index,
               gboolean *fitok)
 {
     gdouble from = gwy_params_get_double(params, PARAM_RANGE_FROM);
     gdouble to = gwy_params_get_double(params, PARAM_RANGE_TO);
     gboolean segment_enabled = nsegments ? gwy_params_get_boolean(params, PARAM_ENABLE_SEGMENT) : FALSE;
     gint segment = segment_enabled ? gwy_params_get_int(params, PARAM_SEGMENT) : -1;
+    gint adhesion = gwy_params_get_boolean(params, PARAM_ADHESION);
+    gint adhesion_segment = adhesion ? gwy_params_get_int(params, PARAM_SEGMENT_ADHESION) : -1;
+    gint baseline_segment = adhesion ? gwy_params_get_int(params, PARAM_SEGMENT_BASELINE) : -1;
+    gdouble baseline_range = gwy_params_get_double(params, PARAM_BASELINE_RANGE);
 
     const gdouble *xdata = gwy_graph_curve_model_get_xdata(gcmodel);
     const gdouble *ydata = gwy_graph_curve_model_get_ydata(gcmodel);
     gint ndata = gwy_graph_curve_model_get_ndata(gcmodel);
 
-    do_fdfit(xdata, ydata, ndata, preset, segments, segment, segment_enabled, from, to, 
+    do_fdfit(xdata, ydata, ndata, preset, segments, segment, segment_enabled, 
+             adhesion, adhesion_segment, baseline_segment, baseline_range, adhesion_index,
+             from, to, 
              fitparams, fix, error, fitok);
 }
 
+
 static void
 estimate_one_curve(GwyGraphCurveModel *gcmodel, GwyParams *params, GwyNLFitPreset *preset,
                    gdouble *fitparams, gint nsegments, const gint *segments)
