@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_simplemech.c 24383 2021-10-15 10:07:39Z klapetek $
+ *  $Id: cmap_simplemech.c 24524 2021-11-14 16:53:23Z klapetek $
  *  Copyright (C) 2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -700,7 +700,8 @@ evaluate_curve(const gdouble *xdata, const gdouble *ydata,
             nbaseline++;
         }
     }
-    baseline /= nbaseline;
+    if (nbaseline>0) baseline /= nbaseline;
+    else baseline = yadata[nadata-1];
 
     //find zero force - point where we reach baseline force on approach curve when going from peak value
     xzero = xadata[nadata-1]; //safe defaults
