@@ -1,33 +1,27 @@
 /*
- *  $Id: aistfile.c 20674 2017-12-18 17:58:47Z yeti-dn $
- *  Copyright (C) 2009 David Necas (Yeti).
+ *  $Id: aistfile.c 24136 2021-09-15 12:21:10Z yeti-dn $
+ *  Copyright (C) 2009-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /*
  * Warning: Madness ahead.
  *
- * Files are saved and loaded using Qt serialization facilities but
- * inconsistently so, data stored by the Qt serialization layer are in
- * big-endian byte order (because people at TrollTech think it is more portable
- * or something) while AIST does not go that far as to think about byte order
- * so they dump stuff to QByteArrays in the native order which happens to be
- * little-endian because they run it on x86.  Bite me.
+ * Files are saved and loaded using Qt serialization facilities but inconsistently so, data stored by the Qt
+ * serialization layer are in big-endian byte order (because people at TrollTech think it is more portable or
+ * something) while AIST does not go that far as to think about byte order so they dump stuff to QByteArrays in the
+ * native order which happens to be little-endian because they run it on x86.  Bite me.
  */
 
 /**
@@ -122,7 +116,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports AIST-NT data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.5.1",
+    "0.5.3",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -154,35 +148,45 @@ aist_detect(const GwyFileDetectInfo *fileinfo,
     if (only_name)
         return (g_str_has_suffix(fileinfo->name_lowercase, EXTENSION)) ? 10 : 0;
 
-    /* The first byte is a boolean, it must be 0 or 1.  This is unfortunately
-     * the only quick means to filter-out non-AIST files. */
+    /* The first byte is a boolean, it must be 0 or 1.  This is unfortunately the only quick means to filter-out
+     * non-AIST files. */
     if (!read_qt_byte(&buf, &size, &is_data_node) || is_data_node > 1)
         return 0;
 
-    /* A silly detection method.  Try to start reading and check if the stuff
-     * looks like the nodes we expect.  Reading the strings looks dangerous.
-     * Fortnately we know buffer_len is small so it won't try to allocate
-     * 2GB of memory for the UTF-8 representation.  */
+    /* A silly detection method.  Try to start reading and check if the stuff looks like the nodes we expect.  Reading
+     * the strings looks dangerous. Fortnately we know buffer_len is small so it won't try to allocate 2GB of memory
+     * for the UTF-8 representation.  */
     if (is_data_node) {
         gchar *type = NULL;
         guint len;
 
-        if (read_qt_string(&buf, &size, &type)
-            && read_qt_int(&buf, &size, &len)) {
+        if (read_qt_string(&buf, &size, &type) && read_qt_int(&buf, &size, &len)) {
             if (gwy_stramong(type, "raster", "curve", "settings", NULL))
                 score = 85;
         }
         g_free(type);
     }
     else {
-        gchar *name = NULL;
-        guint nchildren;
+        gchar *p, *name = NULL;
+        guint nchildren, i, len, nascii;
+        gunichar c;
 
+        /* This is really weak.  Return a low score unless the string at least
+         * looks reasonable stringy. */
         if (read_qt_string(&buf, &size, &name)
             && read_qt_int(&buf, &size, &nchildren)
             && read_qt_byte(&buf, &size, &is_data_node)) {
-            if (is_data_node <= 1 && nchildren <= 0xff && strlen(name) <= 0xff)
-                score = 80;
+            if (is_data_node <= 1 && nchildren <= 0xff && strlen(name) <= 0xff) {
+                score = 40;
+                len = g_utf8_strlen(name, -1);
+                nascii = 0;
+                p = name;
+                for (i = 0; i < MIN(len, 20); i++) {
+                    c = g_utf8_get_char(p);
+                    nascii += (c >= 0x20 && c < 0x7f);
+                }
+                score += 30*nascii/(nascii + 1);
+            }
         }
         g_free(name);
     }
@@ -211,9 +215,8 @@ read_aist_common(const guchar **p, gsize *size,
     return TRUE;
 }
 
-/* Lateral units are a weird thing but they contain the actual units in
- * bracket somewhere.   If there are no brackes, try to interpret the entire
- * string as units -- I've been told this is the right thing.  */
+/* Lateral units are a weird thing but they contain the actual units in bracket somewhere.   If there are no brackes,
+ * try to interpret the entire string as units -- I've been told this is the right thing.  */
 static GwySIUnit*
 extract_units(const gchar *label,
               gdouble *q)
@@ -281,7 +284,6 @@ read_aist_raster(const guchar **p, gsize *size, AistContext *context)
     guint i, j, n, len;
     const guchar *data;
     gchar *s;
-    gchar key[32];
     gdouble *d;
     gdouble qxy, qz;
 
@@ -312,8 +314,7 @@ read_aist_raster(const guchar **p, gsize *size, AistContext *context)
     xyunit = extract_units(raster.xunits, &qxy);
     zunit = extract_units(raster.zunits, &qz);
     dfield = gwy_data_field_new(raster.xres, raster.yres,
-                                qxy*fabs(raster.right - raster.left),
-                                qxy*fabs(raster.bottom - raster.top),
+                                qxy*fabs(raster.right - raster.left), qxy*fabs(raster.bottom - raster.top),
                                 FALSE);
     gwy_data_field_set_si_unit_xy(dfield, xyunit);
     gwy_data_field_set_si_unit_z(dfield, zunit);
@@ -333,44 +334,36 @@ read_aist_raster(const guchar **p, gsize *size, AistContext *context)
         }
     }
 
-    g_snprintf(key, sizeof(key), "/%d/data", context->channel_id);
-    gwy_container_set_object_by_name(context->container, key, dfield);
+    gwy_container_set_object(context->container, gwy_app_get_data_key_for_id(context->channel_id), dfield);
     g_object_unref(dfield);
-    gwy_file_channel_import_log_add(context->container, context->channel_id,
-                                    NULL, context->filename);
+    gwy_file_channel_import_log_add(context->container, context->channel_id, NULL, context->filename);
 
     if ((s = strchr(raster.common.name, '[')))
         s = g_strchomp(g_strndup(raster.common.name, s - raster.common.name));
     else
         s = g_strdup(raster.common.name);
-    g_snprintf(key, sizeof(key), "/%d/data/title", context->channel_id);
-    gwy_container_set_string_by_name(context->container, key, s);
-
-    g_snprintf(key, sizeof(key), "/%d/mask", context->channel_id);
-
-    context->channel_id++;
+    gwy_container_set_string(context->container, gwy_app_get_data_title_key_for_id(context->channel_id), s);
 
     /* At this moment we consider the loading successful. */
     ok = TRUE;
 
-    /* The mask raster is byte-valued.   It contains nonzeroes in points that
-     * were measured (the opposite of how we normally create masks upon
-     * import). */
+    /* The mask raster is byte-valued.   It contains nonzeroes in points that were measured (the opposite of how we
+     * normally create masks upon import). */
     if (read_qt_byte_array(p, size, &len, &data)) {
         if (len == raster.xres*raster.yres) {
             mfield = make_mask_field(dfield, data);
             if (mfield) {
                 gwy_app_channel_remove_bad_data(dfield, mfield);
-                gwy_container_set_object_by_name(context->container, key,
-                                                 mfield);
+                gwy_container_set_object(context->container, gwy_app_get_mask_key_for_id(context->channel_id), mfield);
                 g_object_unref(mfield);
             }
         }
 
-        /* Here's something called view data.  It means the data was processed
-         * (by whatever means) hence we are not interesed in it. */
+        /* Here's something called view data.  It means the data was processed (by whatever means) hence we are not
+         * interesed in it. */
         read_qt_byte_array(p, size, &len, &data);
     }
+    context->channel_id++;
 
 fail:
     free_aist_common(&raster.common);
@@ -415,15 +408,13 @@ read_aist_curve(const guchar **p, gsize *size, AistContext *context)
     if (!read_qt_byte_array(p, size, &viewlen, &viewdata))
         goto fail;
 
-    if (!read_qt_string(p, size, &curve.xunits)
-        || !read_qt_string(p, size, &curve.yunits))
+    if (!read_qt_string(p, size, &curve.xunits) || !read_qt_string(p, size, &curve.yunits))
         goto fail;
 
     xunit = extract_units(curve.xunits, &qx);
     yunit = extract_units(curve.yunits, &qy);
 
-    /* The data are already stored as doubles in the correct order, so save
-     * work if also the endianess matches. */
+    /* The data are already stored as doubles in the correct order, so save work if also the endianess matches. */
     if (G_BYTE_ORDER == G_BIG_ENDIAN) {
         must_free = g_new(gdouble, 2*curve.res);
         xdata = must_free;
@@ -487,8 +478,7 @@ read_aist_data(const guchar **p, gsize *size, AistContext *context)
     gsize datasize;
     guint len;
 
-    if (!read_qt_string(p, size, &type)
-        || !read_qt_int(p, size, &len))
+    if (!read_qt_string(p, size, &type) || !read_qt_int(p, size, &len))
         goto fail;
 
     if (*size < len)
@@ -536,8 +526,7 @@ read_aist_tree(const guchar **p, gsize *size, AistContext *context)
             goto fail;
     }
 
-    if (!read_qt_string(p, size, &name)
-        || !read_qt_int(p, size, &nchildren))
+    if (!read_qt_string(p, size, &name) || !read_qt_int(p, size, &nchildren))
         goto fail;
 
     for (i = 0; i < nchildren; i++) {
@@ -593,8 +582,6 @@ aist_load(const gchar *filename,
 static gboolean
 read_qt_string(const guchar **p, gsize *size, gchar **value)
 {
-    const gunichar2 *utf16native;
-    gunichar2 *must_free;
     guint len;
 
     *value = NULL;
@@ -609,29 +596,13 @@ read_qt_string(const guchar **p, gsize *size, gchar **value)
     if (*size < len || len % sizeof(gunichar2))
         return FALSE;
 
-    if (!len) {
-        *value = g_strdup("");
-        return TRUE;
-    }
-
-    if (G_BYTE_ORDER == G_BIG_ENDIAN) {
-        utf16native = (const gunichar2*)(*p);
-        must_free = NULL;
-    }
-    else if (G_BYTE_ORDER == G_LITTLE_ENDIAN) {
-        utf16native = must_free = g_new(gunichar2, len/sizeof(gunichar2));
-        gwy_memcpy_byte_swap(*p, (guint8*)must_free,
-                             2, len/sizeof(gunichar2), 1);
-    }
-    *value = g_utf16_to_utf8(utf16native, len/sizeof(gunichar2), NULL, NULL,
-                             NULL);
+    *value = gwy_utf16_to_utf8((const gunichar2*)*p, len/sizeof(gunichar2), GWY_BYTE_ORDER_BIG_ENDIAN);
     gwy_debug("QString data: <%s>", *value);
-    g_free(must_free);
 
     *size -= len;
     *p += len;
 
-    return TRUE;
+    return !!*value;
 }
 
 static gboolean
@@ -694,8 +665,7 @@ read_qt_byte(const guchar **p, gsize *size, guint *value)
 
 /* NB: Returned value is a direct pointer to the input buffer.  Do not free! */
 static gboolean
-read_qt_byte_array(const guchar **p, gsize *size,
-                   guint *len, const guchar **value)
+read_qt_byte_array(const guchar **p, gsize *size, guint *len, const guchar **value)
 {
     *value = NULL;
     if (!read_qt_int(p, size, len))
@@ -716,4 +686,4 @@ read_qt_byte_array(const guchar **p, gsize *size,
     return TRUE;
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
