@@ -1,5 +1,5 @@
 /*
- *  $Id: assing-afm.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: assing-afm.c 22667 2019-11-10 10:26:25Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -52,26 +52,23 @@ typedef struct {
     gdouble range;
 } AFMFile;
 
-static gboolean      module_register       (void);
-static gint          aafm_detect           (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* aafm_load             (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static gboolean      read_binary_data      (guint res,
-                                            gdouble *data,
-                                            const guchar *buffer);
-static gboolean      aafm_export           (GwyContainer *data,
-                                            const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
+static gboolean      module_register(void);
+static gint          aafm_detect    (const GwyFileDetectInfo *fileinfo,
+                                     gboolean only_name);
+static GwyContainer* aafm_load      (const gchar *filename,
+                                     GwyRunType mode,
+                                     GError **error);
+static gboolean      aafm_export    (GwyContainer *data,
+                                     const gchar *filename,
+                                     GwyRunType mode,
+                                     GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Assing AFM data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.17",
+    "0.18",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -121,7 +118,7 @@ aafm_load(const gchar *filename,
     gsize size = 0;
     GError *err = NULL;
     AFMFile afmfile;
-    GwyDataField *dfield;
+    GwyDataField *dfield, *rawfield;
     gdouble min, max;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
@@ -151,11 +148,16 @@ aafm_load(const gchar *filename,
         afmfile.real = 1.0;
     }
 
-    dfield = gwy_data_field_new(afmfile.res, afmfile.res,
-                                afmfile.real, afmfile.real,
-                                FALSE);
-    read_binary_data(afmfile.res, gwy_data_field_get_data(dfield), p);
+    rawfield = gwy_data_field_new(afmfile.res, afmfile.res,
+                                  afmfile.real, afmfile.real,
+                                  FALSE);
+    gwy_convert_raw_data(p, afmfile.res*afmfile.res, 1,
+                         GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
+                         gwy_data_field_get_data(rawfield), 1.0, 0.0);
+    dfield = gwy_data_field_new_rotated_90(rawfield, FALSE);
+    g_object_unref(rawfield);
     p += 2*afmfile.res*afmfile.res;
+
     afmfile.range = gwy_get_gfloat_le(&p);
     gwy_data_field_get_min_max(dfield, &min, &max);
     if (min == max)
@@ -184,22 +186,6 @@ aafm_load(const gchar *filename,
     return container;
 }
 
-static gboolean
-read_binary_data(guint res,
-                 gdouble *data,
-                 const guchar *buffer)
-{
-    const guint16 *p = (const guint16*)buffer;
-    guint i, j;
-
-    for (i = 0; i < res*res; i++) {
-        j = (res - 1 - (i % res))*res + i/res;
-        data[j] = GINT16_FROM_LE(p[i]);
-    }
-
-    return TRUE;
-}
-
 static gboolean
 aafm_export(G_GNUC_UNUSED GwyContainer *data,
             const gchar *filename,
