@@ -1,5 +1,5 @@
 /*
- *  $Id: bcrfile.c 22642 2019-11-03 11:46:07Z yeti-dn $
+ *  $Id: bcrfile.c 24008 2021-08-17 14:28:39Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -140,7 +140,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Image Metrology BCR data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.16",
+    "0.17",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -214,26 +214,15 @@ bcrfile_load(const gchar *filename,
     gwy_debug("utf16: %d", utf16);
 
     if (utf16) {
-        gunichar2 *s;
-
         header_size = 2*HEADER_SIZE;
         find_header_size(buffer, &header_size);
-#if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
-        s = (gunichar2*)g_memdup(buffer, header_size);
-#endif
-#if (G_BYTE_ORDER == G_BIG_ENDIAN)
-        s = g_new(gunichar2, HEADER_SIZE);
-        gwy_memcpy_byte_swap(buffer, (guint8*)s, 2, HEADER_SIZE, 1);
-#endif
-        header = g_utf16_to_utf8(s, HEADER_SIZE, 0, 0, &err);
-        g_free(s);
+        header = gwy_utf16_to_utf8((const gunichar2*)buffer, header_size/2,
+                                   GWY_BYTE_ORDER_LITTLE_ENDIAN);
         if (!header) {
             g_set_error(error, GWY_MODULE_FILE_ERROR,
                         GWY_MODULE_FILE_ERROR_DATA,
-                        _("File header is not convertible from UTF-16: %s"),
-                        err->message);
+                        _("Cannot convert string from UTF-16."));
             gwy_file_abandon_contents(buffer, size, NULL);
-
             return NULL;
         }
 
