@@ -1,5 +1,5 @@
 /*
- *  $Id: dmefile.c 21783 2019-01-03 12:58:14Z yeti-dn $
+ *  $Id: dmefile.c 24040 2021-08-21 20:50:16Z yeti-dn $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -130,7 +130,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Danish Micro Engineering (DME) data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.5",
+    "0.5.1",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -174,10 +174,10 @@ dme_load(const gchar *filename,
     gsize size = 0;
     GError *err = NULL;
     GwyDataField *dfield = NULL;
-    guint i, j;
+    guint i;
     DMEFile dmefile;
     gdouble *data;
-    const gint16 *d16, *ls16;
+    const gint16 *ls16;
     GwySIUnit *siunit;
     gdouble q;
 
@@ -235,18 +235,18 @@ dme_load(const gchar *filename,
                                 Angstrom*dmefile.xreal, Angstrom*dmefile.yreal,
                                 FALSE);
     data = gwy_data_field_get_data(dfield);
-    d16 = (const gint16*)(buffer + dmefile.header_size);
     ls16 = (const gint16*)(buffer + dmefile.header_size)
            + dmefile.xres*dmefile.yres;
     for (i = 0; i < dmefile.yres; i++) {
         q = Angstrom * dmefile.height_scale_factor;
         q *= gwy_powi(2.0, ls16[i] & 0x0f);
         q *= dmefile.z_calibration;
-        for (j = 0; j < dmefile.xres; j++) {
-            data[i*dmefile.xres + (dmefile.xres-1 - j)]
-                = q*GINT16_FROM_LE(d16[i*dmefile.xres + j]);
-        }
+        gwy_convert_raw_data(buffer + dmefile.header_size + 2*i*dmefile.xres,
+                             dmefile.xres, 1,
+                             GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
+                             data + i*dmefile.xres, q, 0.0);
     }
+    gwy_data_field_invert(dfield, FALSE, TRUE, FALSE);
 
     gwy_file_abandon_contents(buffer, size, NULL);
 
