@@ -1,5 +1,5 @@
 /*
- *  $Id: err.h 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: err.h 23906 2021-08-02 08:58:59Z yeti-dn $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -55,6 +55,14 @@ err_OPEN_WRITE(GError **error)
                 _("Cannot open file for writing: %s."), g_strerror(errno));
 }
 
+static inline void
+err_OPEN_WRITE_GERROR(GError **error, GError **err)
+{
+    g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_IO,
+                _("Cannot open file for writing: %s."), (*err)->message);
+    g_clear_error(err);
+}
+
 static inline void
 err_WRITE(GError **error)
 {
