@@ -77,6 +77,7 @@ GwyModuleInfo* _gwy_module_query__nanosystemz(void);
 GwyModuleInfo* _gwy_module_query__nanotop(void);
 GwyModuleInfo* _gwy_module_query__netcdf(void);
 GwyModuleInfo* _gwy_module_query__nmmxyz(void);
+GwyModuleInfo* _gwy_module_query__nova_asc(void);
 GwyModuleInfo* _gwy_module_query__nt_mdt(void);
 GwyModuleInfo* _gwy_module_query__nxiifile(void);
 GwyModuleInfo* _gwy_module_query__oldmda(void);
@@ -196,6 +197,7 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__nanotop, "nanotop", },
   { _gwy_module_query__netcdf, "netcdf", },
   { _gwy_module_query__nmmxyz, "nmmxyz", },
+  { _gwy_module_query__nova_asc, "nova-asc", },
   { _gwy_module_query__nt_mdt, "nt-mdt", },
   { _gwy_module_query__nxiifile, "nxiifile", },
   { _gwy_module_query__oldmda, "oldmda", },
