@@ -1,6 +1,6 @@
 /*
- *  $Id: gdeffile.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2010 David Necas (Yeti).
+ *  $Id: gdeffile.c 22723 2020-01-30 15:54:47Z yeti-dn $
+ *  Copyright (C) 2010-2020 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -65,25 +65,29 @@ enum {
 /* XXX: String type seems unimplemented in the original as well. */
 typedef enum {
     /* our markers, not actually present in GDEF */
+    VAR_BOOLEAN     = -4,
     VAR_REAL        = -3,
     VAR_INTEGRAL    = -2,
     VAR_UNSPECIFIED = -1,
     /* GDEF types */
-    VAR_INTEGER   = 0,
-    VAR_FLOAT     = 1,
-    VAR_DOUBLE    = 2,
-    VAR_WORD      = 3,
-    VAR_DWORD     = 4,
-    VAR_CHAR      = 5,
-    VAR_STRING    = 6,
-    VAR_DATABLOCK = 7,
+    VAR_INTEGER     = 0,
+    VAR_FLOAT       = 1,
+    VAR_DOUBLE      = 2,
+    VAR_WORD        = 3,
+    VAR_DWORD       = 4,
+    VAR_CHAR        = 5,
+    VAR_STRING      = 6,
+    VAR_DATABLOCK   = 7,
+    VAR_BOOL        = 8,
     VAR_NVARS,
 } GDEFVariableType;
 
 typedef struct {
     gchar magic[4];
     guint version;
-    /* alignment 2 bytes here */
+    guchar alignment[2];
+    /* On-disk this is ‘32bit time_t’.  We assume it can be safely assigned to
+     * a larger integer when actual time_t is larger. */
     time_t creation_time;
     guint desc_len;
     gchar *description;
@@ -120,7 +124,7 @@ static GwyContainer* gdef_load      (const gchar *filename,
                                      GError **error);
 
 static const gsize type_sizes[VAR_NVARS] = {
-    4, 4, 8, 2, 4, 1, 0, 0
+    4, 4, 8, 2, 4, 1, 0, 0, 1
 };
 
 static GwyModuleInfo module_info = {
@@ -128,7 +132,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports DME GDEF data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.3",
+    "0.4",
     "David Nečas (Yeti)",
     "2010",
 };
@@ -187,8 +191,8 @@ gdef_read_header(GDEFHeader *header,
                     header->version/0x100, header->version % 0x100);
         return FALSE;
     }
-    *p += 2;
-    header->creation_time = gwy_get_guint32_le(p);
+    get_CHARARRAY(header->alignment, p);
+    header->creation_time = (time_t)gwy_get_guint32_le(p);
     gwy_debug("creation_time: %.24s", ctime(&header->creation_time));
     header->desc_len = gwy_get_guint32_le(p);
     gwy_debug("desc_len: %u", header->desc_len);
@@ -227,7 +231,8 @@ gdef_read_control_block(GDEFControlBlock *block,
     block->n_variables = gwy_get_guint32_le(p);
     block->n_data = gwy_get_guint32_le(p);
     block->next = GUINT_TO_POINTER(!!*((*p)++));
-    gwy_debug("n_variables: %u, n_data: %u, next: %u", block->n_variables, block->n_data, !!block->next);
+    gwy_debug("n_variables: %u, n_data: %u, next: %u",
+              block->n_variables, block->n_data, !!block->next);
     *p += 3;
 
     return TRUE;
@@ -406,9 +411,12 @@ gdef_find_variable(GDEFControlBlock *block,
             if (gwy_strequal(var->name, name)) {
                 if (type == VAR_UNSPECIFIED)
                     return var;
+                if (type == VAR_BOOLEAN && var->type == VAR_BOOL)
+                    return var;
                 if (type == VAR_INTEGRAL && (var->type == VAR_WORD
                                              || var->type == VAR_INTEGER
-                                             || var->type == VAR_DWORD))
+                                             || var->type == VAR_DWORD
+                                             || var->type == VAR_BOOL))
                     return var;
                 if (type == VAR_REAL && (var->type == VAR_FLOAT
                                          || var->type == VAR_DOUBLE))
@@ -432,9 +440,11 @@ gdef_get_int_var(GDEFVariable *var)
     if (var->type == VAR_WORD)
         return gwy_get_guint16_le(&p);
     if (var->type == VAR_DWORD)
-        return gwy_get_guint16_le(&p);
+        return gwy_get_guint32_le(&p);
     if (var->type == VAR_CHAR)
         return *p;
+    if (var->type == VAR_BOOL)
+        return *p;
 
     g_return_val_if_reached(0);
 }
@@ -583,6 +593,10 @@ gdef_read_channel_meta(GDEFControlBlock *block)
             s = g_strdup_printf("%ld", (glong)gdef_get_int_var(var));
             gwy_container_set_string_by_name(meta, var->name, s);
         }
+        else if (var->type == VAR_BOOL) {
+            const gchar *cs = gdef_get_int_var(var) ? "true" : "false";
+            gwy_container_set_const_string_by_name(meta, var->name, cs);
+        }
         else if (var->type == VAR_DOUBLE || var->type == VAR_FLOAT) {
             s = g_strdup_printf("%g", gdef_get_real_var(var));
             gwy_container_set_string_by_name(meta, var->name, s);
@@ -640,7 +654,9 @@ gdef_load(const gchar *filename,
         for (j = 0; j < block->n_variables; j++) {
             var = block->variables + j;
             if (var->type != VAR_DATABLOCK)
-                gwy_debug("%s 0x%08x", var->name, (guint)((const guchar*)var->data - buffer));
+                gwy_debug("%s 0x%08x",
+                          var->name,
+                          (guint)((const guchar*)var->data - buffer));
         }
 #endif
 
@@ -652,7 +668,8 @@ gdef_load(const gchar *filename,
             gwy_container_set_object(container, quark, dfield);
             g_object_unref(dfield);
 
-            var = gdef_find_variable(block, "SourceChannel", VAR_INTEGRAL, FALSE);
+            var = gdef_find_variable(block, "SourceChannel", VAR_INTEGRAL,
+                                     FALSE);
             if (var) {
                 title = gwy_enuml_to_string(gdef_get_int_var(var),
                                             "Light Level", 0,
