@@ -1,22 +1,18 @@
 /*
- *  $Id: gwytiff.h 22931 2020-08-27 16:14:27Z yeti-dn $
- *  Copyright (C) 2007-2020 David Necas (Yeti).
+ *  $Id: gwytiff.h 24567 2021-12-20 12:11:34Z yeti-dn $
+ *  Copyright (C) 2007-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <glib.h>
@@ -26,10 +22,9 @@
 /*
  * This is a rudimentary built-in TIFF reader.
  *
- * It is required to read some TIFF-based files because the software that
- * writes them is very creative with regard to the specification.  In other
- * words, we need to read some incorrect TIFFs too.  In particular, we do not
- * expect directories to be sorted and we accept bogus (nul) entries.
+ * It is required to read some TIFF-based files because the software that writes them is very creative with regard to
+ * the specification.  In other words, we need to read some incorrect TIFFs too.  In particular, we do not expect
+ * directories to be sorted and we accept bogus (nul) entries.
  *
  * Names starting GWY_TIFF, GwyTIFF and gwy_tiff are reserved.
  */
@@ -123,8 +118,7 @@ typedef enum {
 } GwyTIFFTag;
 
 /* Values of some standard tags.
- * Note only values interesting for us are enumerated.  Add more from the
- * standard if needed.  */
+ * Note only values interesting for us are enumerated.  Add more from the standard if needed.  */
 
 /* Baseline readers are required to implement NONE, HUFFMAN and PACKBITS.
  * PACKBITS seems to be used in the wild occasionally.
@@ -230,15 +224,12 @@ typedef struct {
     guint compression;
     /* Decompression (keeping track of current state). */
     GwyTIFFUnpackFunc unpack_func;
-    guchar *unpacked;       /* Buffer for unpacking, large enough to hold one
-                             * strip or tile. */
-    guint64 which_unpacked; /* Which strip or tile we have in unpacked[].
-                             * G_MAXUINT64 means none. */
+    guchar *unpacked;       /* Buffer for unpacking, large enough to hold one * strip or tile. */
+    guint64 which_unpacked; /* Which strip or tile we have in unpacked[]; G_MAXUINT64 means none. */
 } GwyTIFFImageReader;
 
-/* Parameters version and byteorder are inout.  If they are non-zero, the file
- * must match the specified value to be accepted.  In any case, they are set
- * to the true values on success. */
+/* Parameters version and byteorder are inout.  If they are non-zero, the file must match the specified value to be
+ * accepted.  In any case, they are set to the true values on success. */
 G_GNUC_UNUSED
 static const guchar*
 gwy_tiff_detect(const guchar *buffer,
@@ -285,13 +276,11 @@ gwy_tiff_detect(const guchar *buffer,
     return buffer;
 }
 
-/* By default compressed files are not allowed because no one saves SPM data
- * this way.
+/* By default compressed files are not allowed because no one saves SPM data this way.
  *
- * When files are not compressed gwy_tiff_read_image_row() can never fail
- * because we can easily check the sizes of everything before attempting to
- * read data.  Consequently, most GwyTIFF users do not need any error handling
- * after successfully creating a GwyTIFFImageReader. */
+ * When files are not compressed gwy_tiff_read_image_row() can never fail because we can easily check the sizes of
+ * everything before attempting to read data.  Consequently, most GwyTIFF users do not need any error handling after
+ * successfully creating a GwyTIFFImageReader. */
 G_GNUC_UNUSED static inline void
 gwy_tiff_allow_compressed(GwyTIFF *tiff,
                           gboolean setting)
@@ -299,6 +288,14 @@ gwy_tiff_allow_compressed(GwyTIFF *tiff,
     tiff->allow_compressed = setting;
 }
 
+static gpointer
+err_TIFF_REQUIRED_TAG(GError **error, GwyTIFFTag tag)
+{
+    g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
+                _("Required tag %u was not found."), tag);
+    return NULL;
+}
+
 static inline gboolean
 gwy_tiff_data_fits(const GwyTIFF *tiff,
                    guint64 offset,
@@ -369,8 +366,7 @@ gwy_tiff_scan_ifd(const GwyTIFF *tiff, guint64 offset,
 
     if (!gwy_tiff_data_fits(tiff, offset, ifdsize, 1)) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("TIFF directory %lu ended unexpectedly."),
-                    (gulong)tiff->dirs->len);
+                    _("TIFF directory %lu ended unexpectedly."), (gulong)tiff->dirs->len);
         return NULL;
     }
 
@@ -385,8 +381,7 @@ gwy_tiff_scan_ifd(const GwyTIFF *tiff, guint64 offset,
 
     if (!gwy_tiff_data_fits(tiff, offset + ifdsize, tagsize, nentries)) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("TIFF directory %lu ended unexpectedly."),
-                    (gulong)tiff->dirs->len);
+                    _("TIFF directory %lu ended unexpectedly."), (gulong)tiff->dirs->len);
         return NULL;
     }
 
@@ -419,28 +414,20 @@ gwy_tiff_ifd_is_vaild(const GwyTIFF *tiff, const GArray *tags, GError **error)
 
         entry = &g_array_index(tags, GwyTIFFEntry, j);
         if (tiff->version == GWY_TIFF_CLASSIC
-            && (entry->type == GWY_TIFF_LONG8
-                || entry->type == GWY_TIFF_SLONG8
-                || entry->type == GWY_TIFF_IFD8)) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
-                        _("BigTIFF data type %u was found in a classic "
-                          "TIFF."),
-                        entry->type);
+            && (entry->type == GWY_TIFF_LONG8 || entry->type == GWY_TIFF_SLONG8 || entry->type == GWY_TIFF_IFD8)) {
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
+                        _("BigTIFF data type %u was found in a classic TIFF."), entry->type);
             return FALSE;
         }
         p = entry->value;
         offset = tiff->get_length(&p);
         item_size = gwy_tiff_data_type_size(entry->type);
-        /* Uknown types are implicitly OK.  If we cannot read it we never
-         * read it by definition, so let the hell take what it refers to.
-         * This also means readers of custom types have to check the size
-         * themselves. */
+        /* Uknown types are implicitly OK.  If we cannot read it we never read it by definition, so let the hell take
+         * what it refers to. This also means readers of custom types have to check the size themselves. */
         if (item_size
             && entry->count > tiff->tagvaluesize/item_size
             && !gwy_tiff_data_fits(tiff, offset, item_size, entry->count)) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                         _("Invalid tag data positions were found."));
             return FALSE;
         }
@@ -529,11 +516,8 @@ gwy_tiff_load_impl(GwyTIFF *tiff,
         guint reserved0 = tiff->get_guint16(&p);
 
         if (bytesize != 8 || reserved0 != 0) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
-                        _("BigTIFF reserved fields are %u and %u "
-                          "instead of 8 and 0."),
-                          bytesize, reserved0);
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
+                        _("BigTIFF reserved fields are %u and %u instead of 8 and 0."), bytesize, reserved0);
             return FALSE;
         }
     }
@@ -603,8 +587,7 @@ gwy_tiff_sort_tags(GwyTIFF *tiff)
     gsize i;
 
     for (i = 0; i < tiff->dirs->len; i++)
-        g_array_sort((GArray*)g_ptr_array_index(tiff->dirs, i),
-                     gwy_tiff_tag_compare);
+        g_array_sort((GArray*)g_ptr_array_index(tiff->dirs, i), gwy_tiff_tag_compare);
 }
 
 static const GwyTIFFEntry*
@@ -700,9 +683,7 @@ gwy_tiff_get_uint(const GwyTIFF *tiff,
                   guint tag,
                   guint *retval)
 {
-    return gwy_tiff_get_uint_entry(tiff,
-                                   gwy_tiff_find_tag(tiff, dirno, tag),
-                                   retval);
+    return gwy_tiff_get_uint_entry(tiff, gwy_tiff_find_tag(tiff, dirno, tag), retval);
 }
 
 G_GNUC_UNUSED static gboolean
@@ -747,9 +728,7 @@ gwy_tiff_get_size(const GwyTIFF *tiff,
                   guint tag,
                   guint64 *retval)
 {
-    return gwy_tiff_get_size_entry(tiff,
-                                   gwy_tiff_find_tag(tiff, dirno, tag),
-                                   retval);
+    return gwy_tiff_get_size_entry(tiff, gwy_tiff_find_tag(tiff, dirno, tag), retval);
 }
 
 G_GNUC_UNUSED static gboolean
@@ -867,9 +846,7 @@ gwy_tiff_get_sint(const GwyTIFF *tiff,
                   guint tag,
                   gint *retval)
 {
-    return gwy_tiff_get_sint_entry(tiff,
-                                   gwy_tiff_find_tag(tiff, dirno, tag),
-                                   retval);
+    return gwy_tiff_get_sint_entry(tiff, gwy_tiff_find_tag(tiff, dirno, tag), retval);
 }
 
 G_GNUC_UNUSED static gboolean
@@ -910,9 +887,7 @@ gwy_tiff_get_bool(const GwyTIFF *tiff,
                   guint tag,
                   gboolean *retval)
 {
-    return gwy_tiff_get_bool_entry(tiff,
-                                   gwy_tiff_find_tag(tiff, dirno, tag),
-                                   retval);
+    return gwy_tiff_get_bool_entry(tiff, gwy_tiff_find_tag(tiff, dirno, tag), retval);
 }
 
 G_GNUC_UNUSED static gboolean
@@ -952,9 +927,7 @@ gwy_tiff_get_float(const GwyTIFF *tiff,
                    guint tag,
                    gdouble *retval)
 {
-    return gwy_tiff_get_float_entry(tiff,
-                                    gwy_tiff_find_tag(tiff, dirno, tag),
-                                    retval);
+    return gwy_tiff_get_float_entry(tiff, gwy_tiff_find_tag(tiff, dirno, tag), retval);
 }
 
 G_GNUC_UNUSED static gboolean
@@ -990,19 +963,15 @@ gwy_tiff_get_string(const GwyTIFF *tiff,
                     guint tag,
                     gchar **retval)
 {
-    return gwy_tiff_get_string_entry(tiff,
-                                     gwy_tiff_find_tag(tiff, dirno, tag),
-                                     retval);
+    return gwy_tiff_get_string_entry(tiff, gwy_tiff_find_tag(tiff, dirno, tag), retval);
 }
 
 /* Unpack a data segment compressed using the PackBits algorithm.
  *
- * Returns the number of bytes consumed, except on failure when zero is
- * returned.
+ * Returns the number of bytes consumed, except on failure when zero is returned.
  *
- * The caller must provide output buffer which can hold the entire segment.
- * Since TIFF forbids packing across row boundaries, we consider an error when
- * we do not stop exactly at the requested number of bytes. */
+ * The caller must provide output buffer which can hold the entire segment. Since TIFF forbids packing across row
+ * boundaries, we consider an error when we do not stop exactly at the requested number of bytes. */
 G_GNUC_UNUSED static inline guint
 gwy_tiff_unpack_packbits(const guchar *packed,
                          guint packedsize,
@@ -1056,8 +1025,7 @@ gwy_tiff_lzw_get_code(const guchar *packed, guint packedsize,
 
     *bitpos += nbits;
 
-    /* All our codes are larger than one byte so we always consume everything
-     * from the first byte. */
+    /* All our codes are larger than one byte so we always consume everything from the first byte. */
     x = ((0xff >> bi) & p[0]) << (nbits + bi - 8);
     if (nbits + bi <= 16)
         /* Another byte is enough. */
@@ -1066,8 +1034,8 @@ gwy_tiff_lzw_get_code(const guchar *packed, guint packedsize,
     /* Another byte is not enough, so consume it all. */
     x |= (p[1] << (nbits + bi - 16));
 
-    /* And with the next byte it is definitely enough because we can get at
-     * least 17 bits this way, but TIFF LZW needs at most 12. */
+    /* And with the next byte it is definitely enough because we can get at least 17 bits this way, but TIFF LZW needs
+     * at most 12. */
     return x | (p[2] >> (24 - nbits - bi));
 }
 
@@ -1200,19 +1168,16 @@ end:
             table_pos++;
         }
         else {
-            /* Any unseen code must be the next available.  Getting some other
-             * large number means things went awry.  This also covers getting
-             * G_MAXUINT from get-code. */
-            gwy_debug("random unseen large code %u (expecting %u)",
-                      code, table_pos);
+            /* Any unseen code must be the next available.  Getting some other large number means things went awry.
+             * This also covers getting G_MAXUINT from get-code. */
+            gwy_debug("random unseen large code %u (expecting %u)", code, table_pos);
             goto finalise;
         }
 
         if (table_pos == 511 || table_pos == 1023 || table_pos == 2047)
             nbits++;
         if (table_pos == 4095) {
-            gwy_debug("reached table pos 4095, so the next code would be "
-                      "13bit, even if it was CLEAR");
+            gwy_debug("reached table pos 4095, so the next code would be 13bit, even if it was CLEAR");
             goto finalise;
         }
         prev = code;
@@ -1240,25 +1205,18 @@ gwy_tiff_read_image_reader_sizes(const GwyTIFF *tiff,
     guint i;
 
     if (nvalues == 1) {
-        if (!gwy_tiff_get_size(tiff, reader->dirno, tag, values)) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
-                        _("Required tag %u was not found."), tag);
-            return FALSE;
-        }
+        if (!gwy_tiff_get_size(tiff, reader->dirno, tag, values))
+            return !!err_TIFF_REQUIRED_TAG(error, tag);
         return TRUE;
     }
 
     if (!(entry = gwy_tiff_find_tag(tiff, reader->dirno, tag))
-        || (entry->type != GWY_TIFF_LONG && entry->type != GWY_TIFF_LONG8)
+        || (entry->type != GWY_TIFF_SHORT && entry->type != GWY_TIFF_LONG && entry->type != GWY_TIFF_LONG8)
         || entry->count != nvalues) {
-        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Required tag %u was not found."), tag);
-        return FALSE;
+        return !!err_TIFF_REQUIRED_TAG(error, tag);
     }
 
-    /* Matching type ensured the tag data is at a valid position in the
-     * file. */
+    /* Matching type ensured the tag data is at a valid position in the file. */
     p = entry->value;
     i = tiff->get_guint32(&p);
     p = tiff->data + i;
@@ -1266,10 +1224,17 @@ gwy_tiff_read_image_reader_sizes(const GwyTIFF *tiff,
         for (l = 0; l < nvalues; l++)
             values[l] = tiff->get_guint32(&p);
     }
-    else {
+    else if (entry->type == GWY_TIFF_LONG8) {
         for (l = 0; l < nvalues; l++)
             values[l] = tiff->get_guint64(&p);
     }
+    else if (entry->type == GWY_TIFF_SHORT) {
+        for (l = 0; l < nvalues; l++)
+            values[l] = tiff->get_guint16(&p);
+    }
+    else {
+        g_return_val_if_reached(FALSE);
+    }
 
     return TRUE;
 }
@@ -1292,33 +1257,27 @@ gwy_tiff_init_image_reader_striped(const GwyTIFF *tiff,
         reader->unpack_func = gwy_tiff_unpack_lzw;
     else if (reader->compression != GWY_TIFF_COMPRESSION_NONE) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Compression type %u is not supported."),
-                    reader->compression);
+                    _("Compression type %u is not supported."), reader->compression);
         return FALSE;
     }
 
     nstrips = (reader->height + reader->strip_rows-1)/reader->strip_rows;
     reader->offsets = g_new(guint64, nstrips);
     reader->bytecounts = g_new(guint64, nstrips);
-    if (!gwy_tiff_read_image_reader_sizes(tiff, reader,
-                                          GWY_TIFFTAG_STRIP_OFFSETS,
+    if (!gwy_tiff_read_image_reader_sizes(tiff, reader, GWY_TIFFTAG_STRIP_OFFSETS,
                                           reader->offsets, nstrips, error))
         goto fail;
-    if (!gwy_tiff_read_image_reader_sizes(tiff, reader,
-                                          GWY_TIFFTAG_STRIP_BYTE_COUNTS,
+    if (!gwy_tiff_read_image_reader_sizes(tiff, reader, GWY_TIFFTAG_STRIP_BYTE_COUNTS,
                                           reader->bytecounts, nstrips, error))
         goto fail;
 
-    /* Validate strip offsets and sizes.  Strips are not padded so the last
-     * strip can be shorter.*/
-    reader->rowstride = (reader->bits_per_sample/8 * reader->samples_per_pixel
-                        * reader->width);
+    /* Validate strip offsets and sizes.  Strips are not padded so the last strip can be shorter.*/
+    reader->rowstride = (reader->bits_per_sample/8 * reader->samples_per_pixel * reader->width);
     ssize = reader->rowstride * reader->strip_rows;
     for (i = 0; i < nstrips; i++) {
         if (i == nstrips-1 && reader->height % reader->strip_rows)
             ssize = reader->rowstride * (reader->height % reader->strip_rows);
-        if ((reader->compression == GWY_TIFF_COMPRESSION_NONE
-             && ssize != reader->bytecounts[i])
+        if ((reader->compression == GWY_TIFF_COMPRESSION_NONE && ssize != reader->bytecounts[i])
             || reader->offsets[i] + reader->bytecounts[i] > tiff->size) {
             err_INVALID(error, "StripOffsets");
             goto fail;
@@ -1356,8 +1315,7 @@ gwy_tiff_init_image_reader_tiled(const GwyTIFF *tiff,
 
     if (reader->compression != GWY_TIFF_COMPRESSION_NONE) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Compression type %u is not supported."),
-                    reader->compression);
+                    _("Compression type %u is not supported."), reader->compression);
         return FALSE;
     }
 
@@ -1366,25 +1324,20 @@ gwy_tiff_init_image_reader_tiled(const GwyTIFF *tiff,
     ntiles = nhtiles*nvtiles;
     reader->offsets = g_new(guint64, ntiles);
     reader->bytecounts = g_new(guint64, ntiles);
-    if (!gwy_tiff_read_image_reader_sizes(tiff, reader,
-                                          GWY_TIFFTAG_TILE_OFFSETS,
+    if (!gwy_tiff_read_image_reader_sizes(tiff, reader, GWY_TIFFTAG_TILE_OFFSETS,
                                           reader->offsets, ntiles, error))
         goto fail;
-    if (!gwy_tiff_read_image_reader_sizes(tiff, reader,
-                                          GWY_TIFFTAG_TILE_BYTE_COUNTS,
+    if (!gwy_tiff_read_image_reader_sizes(tiff, reader, GWY_TIFFTAG_TILE_BYTE_COUNTS,
                                           reader->bytecounts, ntiles, error))
         goto fail;
 
-    /* Validate tile offsets and sizes.  Tiles are padded so size must be
-     * reserved for entire tiles.  The standard says the tile width must be
-     * a multiple of 16 so we can ignore alignment as only invalid files would
-     * need row padding. */
-    reader->rowstride = (reader->bits_per_sample/8 * reader->samples_per_pixel
-                         * reader->tile_width);
+    /* Validate tile offsets and sizes.  Tiles are padded so size must be reserved for entire tiles.  The standard
+     * says the tile width must be a multiple of 16 so we can ignore alignment as only invalid files would need row
+     * padding. */
+    reader->rowstride = (reader->bits_per_sample/8 * reader->samples_per_pixel * reader->tile_width);
     tsize = reader->rowstride * reader->tile_height;
     for (i = 0; i < ntiles; i++) {
-        if ((reader->compression == GWY_TIFF_COMPRESSION_NONE
-             && tsize != reader->bytecounts[i])
+        if ((reader->compression == GWY_TIFF_COMPRESSION_NONE && tsize != reader->bytecounts[i])
             || reader->offsets[i] + reader->bytecounts[i] > tiff->size) {
             err_INVALID(error, "TileOffsets");
             goto fail;
@@ -1417,41 +1370,27 @@ gwy_tiff_get_image_reader(const GwyTIFF *tiff,
     reader.which_unpacked = G_MAXUINT64;
 
     /* Required integer fields */
-    if (!gwy_tiff_get_size(tiff, dirno, GWY_TIFFTAG_IMAGE_WIDTH,
-                           &reader.width)) {
-        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Required tag %u was not found."),
-                    GWY_TIFFTAG_IMAGE_WIDTH);
-        return NULL;
-    }
-    if (!gwy_tiff_get_size(tiff, dirno, GWY_TIFFTAG_IMAGE_LENGTH,
-                           &reader.height)) {
-        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Required tag %u was not found."),
-                    GWY_TIFFTAG_IMAGE_LENGTH);
-        return NULL;
-    }
+    if (!gwy_tiff_get_size(tiff, dirno, GWY_TIFFTAG_IMAGE_WIDTH, &reader.width))
+        return (GwyTIFFImageReader*)err_TIFF_REQUIRED_TAG(error, GWY_TIFFTAG_IMAGE_WIDTH);
+    if (!gwy_tiff_get_size(tiff, dirno, GWY_TIFFTAG_IMAGE_LENGTH, &reader.height))
+        return (GwyTIFFImageReader*)err_TIFF_REQUIRED_TAG(error, GWY_TIFFTAG_IMAGE_LENGTH);
 
     /* The TIFF specs say this is required, but it seems to default to 1. */
-    if (!gwy_tiff_get_uint(tiff, dirno, GWY_TIFFTAG_SAMPLES_PER_PIXEL,
-                           &reader.samples_per_pixel))
+    if (!gwy_tiff_get_uint(tiff, dirno, GWY_TIFFTAG_SAMPLES_PER_PIXEL, &reader.samples_per_pixel))
         reader.samples_per_pixel = 1;
-    if (reader.samples_per_pixel == 0
-        || reader.samples_per_pixel > max_samples) {
+    if (reader.samples_per_pixel == 0 || reader.samples_per_pixel > max_samples) {
         err_UNSUPPORTED(error, "SamplesPerPixel");
         return NULL;
     }
 
     /* The TIFF specs say this is required, but it seems to default to 1. */
     bps = g_new(guint, reader.samples_per_pixel);
-    if (!gwy_tiff_get_uints(tiff, dirno, GWY_TIFFTAG_BITS_PER_SAMPLE,
-                            reader.samples_per_pixel, bps))
+    if (!gwy_tiff_get_uints(tiff, dirno, GWY_TIFFTAG_BITS_PER_SAMPLE, reader.samples_per_pixel, bps))
         reader.bits_per_sample = 1;
     else {
         for (i = 1; i < reader.samples_per_pixel; i++) {
             if (bps[i] != bps[i-1]) {
-                g_set_error(error, GWY_MODULE_FILE_ERROR,
-                            GWY_MODULE_FILE_ERROR_DATA,
+                g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                             _("Non-uniform bits per sample are unsupported."));
                 g_free(bps);
                 return NULL;
@@ -1461,35 +1400,26 @@ gwy_tiff_get_image_reader(const GwyTIFF *tiff,
     }
     g_free(bps);
 
-    /* The TIFF specs say this is required, but it seems to default to
-     * MAXINT.  Setting more reasonably RowsPerStrip = ImageLength achieves
-     * the same ends.  Also it is not required for tiled images. */
-    if (!gwy_tiff_get_size(tiff, dirno, GWY_TIFFTAG_ROWS_PER_STRIP,
-                           &reader.strip_rows))
+    /* The TIFF specs say this is required, but it seems to default to MAXINT.  Setting more reasonably
+     * RowsPerStrip = ImageLength achieves the same end.  Also it is not required for tiled images. */
+    if (!gwy_tiff_get_size(tiff, dirno, GWY_TIFFTAG_ROWS_PER_STRIP, &reader.strip_rows))
         reader.strip_rows = reader.height;
 
-    /*
-     * The data sample type (default is unsigned integer)
-     */
-    if (!gwy_tiff_get_uint(tiff, dirno, GWY_TIFFTAG_SAMPLE_FORMAT,
-                           &reader.sample_format))
+    /* The data sample type (default is unsigned integer). */
+    if (!gwy_tiff_get_uint(tiff, dirno, GWY_TIFFTAG_SAMPLE_FORMAT, &reader.sample_format))
         reader.sample_format = GWY_TIFF_SAMPLE_FORMAT_UNSIGNED_INTEGER;
 
     /* Integer fields specifying data in a format we do not support */
-    if (!gwy_tiff_get_uint(tiff, dirno, GWY_TIFFTAG_COMPRESSION,
-                           &reader.compression))
+    if (!gwy_tiff_get_uint(tiff, dirno, GWY_TIFFTAG_COMPRESSION, &reader.compression))
         reader.compression = GWY_TIFF_COMPRESSION_NONE;
 
-    if (!tiff->allow_compressed
-        && reader.compression != GWY_TIFF_COMPRESSION_NONE) {
+    if (!tiff->allow_compressed && reader.compression != GWY_TIFF_COMPRESSION_NONE) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Compression type %u is not supported."),
-                    reader.compression);
+                    _("Compression type %u is not supported."), reader.compression);
         return NULL;
     }
 
-    if (gwy_tiff_get_uint(tiff, dirno, GWY_TIFFTAG_PLANAR_CONFIG, &i)
-        && i != GWY_TIFF_PLANAR_CONFIG_CONTIGNUOUS) {
+    if (gwy_tiff_get_uint(tiff, dirno, GWY_TIFFTAG_PLANAR_CONFIG, &i) && i != GWY_TIFF_PLANAR_CONFIG_CONTIGNUOUS) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                     _("Planar configuration %u is not supported."), i);
         return NULL;
@@ -1507,8 +1437,7 @@ gwy_tiff_get_image_reader(const GwyTIFF *tiff,
         }
     }
     else if (reader.sample_format == GWY_TIFF_SAMPLE_FORMAT_FLOAT) {
-        if (reader.bits_per_sample != 32
-            && reader.bits_per_sample != 64) {
+        if (reader.bits_per_sample != 32 && reader.bits_per_sample != 64) {
             err_BPP(error, reader.bits_per_sample);
             return NULL;
         }
@@ -1519,22 +1448,17 @@ gwy_tiff_get_image_reader(const GwyTIFF *tiff,
         return NULL;
     }
 
-    /* Apparently in Zeiss SEM files, RowsPerStrip can be *anything* larger
-     * than the imager height. */
+    /* Apparently in Zeiss SEM files, RowsPerStrip can be *anything* larger than the imager height. */
     if (reader.strip_rows > reader.height)
         reader.strip_rows = reader.height;
 
-    if (err_DIMENSION(error, reader.width)
-        || err_DIMENSION(error, reader.height))
+    if (err_DIMENSION(error, reader.width) || err_DIMENSION(error, reader.height))
         return NULL;
 
-    /* If we can read the tile dimensions assume it is a tiled image and report
-     * possible errors as for a tiled image.  If the image contains just one
-     * of them ignore it (and report errors as for a non-tiled image). */
-    if (gwy_tiff_get_size(tiff, dirno,
-                          GWY_TIFFTAG_TILE_WIDTH, &reader.tile_width)
-        && gwy_tiff_get_size(tiff, dirno,
-                             GWY_TIFFTAG_TILE_LENGTH, &reader.tile_height)) {
+    /* If we can read the tile dimensions assume it is a tiled image and report possible errors as for a tiled image.
+     * If the image contains just one of them ignore it (and report errors as for a non-tiled image). */
+    if (gwy_tiff_get_size(tiff, dirno, GWY_TIFFTAG_TILE_WIDTH, &reader.tile_width)
+        && gwy_tiff_get_size(tiff, dirno, GWY_TIFFTAG_TILE_LENGTH, &reader.tile_height)) {
         reader.strip_rows = 0;
         if (!gwy_tiff_init_image_reader_tiled(tiff, &reader, error))
             return NULL;
@@ -1645,14 +1569,12 @@ gwy_tiff_read_image_row_striped(const GwyTIFF *tiff,
     p = tiff->data + reader->offsets[stripno];
     if (reader->unpack_func) {
         g_assert(reader->unpacked);
-        /* If we want a row from different stripe than current we unpack the
-         * stripe. */
+        /* If we want a row from different stripe than current we unpack the stripe. */
         if (stripno != reader->which_unpacked) {
             nstrips = (reader->height + nrows-1)/nrows;
             if (stripno == nstrips-1 && reader->height % nrows)
                 nrows = reader->height % nrows;
-            if (!reader->unpack_func(p, reader->bytecounts[stripno],
-                                     reader->unpacked, rowstride * nrows))
+            if (!reader->unpack_func(p, reader->bytecounts[stripno], reader->unpacked, rowstride * nrows))
                 return FALSE;
             reader->which_unpacked = stripno;
         }
@@ -1661,8 +1583,7 @@ gwy_tiff_read_image_row_striped(const GwyTIFF *tiff,
     }
     p += stripindex*rowstride + (bps/8)*channelno;
     skip = (reader->samples_per_pixel - 1)*bps/8;
-    gwy_tiff_reader_read_segment(tiff, sformat, bps, p, reader->width, skip,
-                                 q, z0, dest);
+    gwy_tiff_reader_read_segment(tiff, sformat, bps, p, reader->width, skip, q, z0, dest);
 
     return TRUE;
 }
@@ -1688,19 +1609,16 @@ gwy_tiff_read_image_row_tiled(const GwyTIFF *tiff,
     len = reader->tile_width;
     for (i = 0; i < nhtiles; i++) {
         l = vtileno*nhtiles + i;
-        p = tiff->data + (reader->offsets[l] + vtileindex*reader->rowstride
-                          + (bps/8)*channelno);
+        p = tiff->data + (reader->offsets[l] + vtileindex*reader->rowstride + (bps/8)*channelno);
         if (i == reader->tile_width-1 && reader->width % reader->tile_width)
             len = reader->width % reader->tile_width;
-        gwy_tiff_reader_read_segment(tiff, sformat, bps, p, len, skip,
-                                     q, z0, dest);
+        gwy_tiff_reader_read_segment(tiff, sformat, bps, p, len, skip, q, z0, dest);
         dest += len;
     }
 }
 
-/* If the file may be compressed (which needs to be explicitly allowed using
- * gwy_tiff_allow_compressed()) this function needs to be called with rowno
- * in a mononotonically increasing sequence.  Anything else can result in
+/* If the file may be compressed (which needs to be explicitly allowed using gwy_tiff_allow_compressed()) this
+ * function needs to be called with rowno in a mononotonically increasing sequence.  Anything else can result in
  * repeated unpacking data from the beginning and quadratic time complexity. */
 G_GNUC_UNUSED static inline gboolean
 gwy_tiff_read_image_row(const GwyTIFF *tiff,
@@ -1718,14 +1636,12 @@ gwy_tiff_read_image_row(const GwyTIFF *tiff,
     g_return_val_if_fail(channelno < reader->samples_per_pixel, FALSE);
     if (reader->strip_rows) {
         g_return_val_if_fail(!reader->tile_width, FALSE);
-        gwy_tiff_read_image_row_striped(tiff, reader, channelno, rowno,
-                                        q, z0, dest);
+        gwy_tiff_read_image_row_striped(tiff, reader, channelno, rowno, q, z0, dest);
     }
     else {
         g_return_val_if_fail(reader->tile_width, FALSE);
         g_return_val_if_fail(!reader->unpack_func, FALSE);
-        gwy_tiff_read_image_row_tiled(tiff, reader, channelno, rowno,
-                                      q, z0, dest);
+        gwy_tiff_read_image_row_tiled(tiff, reader, channelno, rowno, q, z0, dest);
     }
 
     return TRUE;
@@ -1791,8 +1707,7 @@ gwy_tiff_load(const gchar *filename,
     GwyTIFF *tiff;
 
     tiff = g_new0(GwyTIFF, 1);
-    if (gwy_tiff_load_impl(tiff, filename, error)
-        && gwy_tiff_tags_valid(tiff, error)) {
+    if (gwy_tiff_load_impl(tiff, filename, error) && gwy_tiff_tags_valid(tiff, error)) {
         gwy_tiff_sort_tags(tiff);
         return tiff;
     }
@@ -1801,4 +1716,4 @@ gwy_tiff_load(const gchar *filename,
     return NULL;
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin et columns=120 tw=118 ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
