@@ -1,21 +1,17 @@
 /*
- *  $Id: gxyzffile.c 21300 2018-08-08 14:08:32Z yeti-dn $
- *  Copyright (C) 2013-2016 David Necas (Yeti).
+ *  $Id: gxyzffile.c 25076 2022-10-11 13:48:10Z yeti-dn $
+ *  Copyright (C) 2013-2022 David Necas (Yeti).
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -51,7 +47,6 @@
 #include <libgwyddion/gwyutils.h>
 #include <libprocess/arithmetic.h>
 #include <libprocess/surface.h>
-#include <libgwydgets/gwydgetutils.h>
 #include <app/gwymoduleutils-file.h>
 #include <app/gwyapp.h>
 
@@ -61,63 +56,62 @@
 #define MAGIC_SIZE (sizeof(MAGIC)-1)
 #define EXTENSION ".gxyzf"
 
-typedef struct {
-    gboolean all_channels;
-} GXYZExportArgs;
+enum {
+    PARAM_ALL_CHANNELS,
 
-static gboolean      module_register              (void);
-static gint          gxyzf_detect                 (const GwyFileDetectInfo *fileinfo,
-                                                   gboolean only_name);
-static GwyContainer* gxyzf_load                   (const gchar *filename,
-                                                   GwyRunType mode,
-                                                   GError **error);
-static gboolean      gxyzf_export                 (GwyContainer *container,
-                                                   const gchar *filename,
-                                                   GwyRunType mode,
-                                                   GError **error);
-static gboolean      gxyzf_export_dialog          (GXYZExportArgs *args,
-                                                   GwyAppPage pageno,
-                                                   const gchar *title);
-static gboolean      gxyzf_export_data_fields     (const gchar *filename,
-                                                   GwyContainer *container,
-                                                   gint id,
-                                                   const GXYZExportArgs *args,
-                                                   GError **error);
-static gint*         gather_compatible_data_fields(GwyContainer *data,
-                                                   GwyDataField *dfield,
-                                                   guint *nchannels);
-static gboolean      gxyzf_export_surfaces        (const gchar *filename,
-                                                   GwyContainer *container,
-                                                   gint id,
-                                                   const GXYZExportArgs *args,
-                                                   GError **error);
-static gint*         gather_compatible_surfaces   (GwyContainer *data,
-                                                   GwySurface *surface,
-                                                   guint *nchannels);
-static gboolean      write_header                 (FILE *fh,
-                                                   guint nchannels,
-                                                   guint npoints,
-                                                   gchar **titles,
-                                                   GwySIUnit *xyunit,
-                                                   GwySIUnit **zunits,
-                                                   gint xres,
-                                                   gint yres,
-                                                   GError **error);
-static void          load_args                    (GwyContainer *settings,
-                                                   GXYZExportArgs *args);
-static void          save_args                    (GwyContainer *settings,
-                                                   const GXYZExportArgs *args);
-
-static const GXYZExportArgs gxyzf_defaults = {
-    FALSE,
+    INFO_CHANNEL,
 };
 
+typedef struct {
+    GwyParams *params;
+    GwyAppPage pageno;
+} ModuleArgs;
+
+static gboolean         module_register              (void);
+static GwyParamDef*     define_module_params         (void);
+static gint             gxyzf_detect                 (const GwyFileDetectInfo *fileinfo,
+                                                      gboolean only_name);
+static GwyContainer*    gxyzf_load                   (const gchar *filename,
+                                                      GwyRunType mode,
+                                                      GError **error);
+static gboolean         gxyzf_export                 (GwyContainer *container,
+                                                      const gchar *filename,
+                                                      GwyRunType mode,
+                                                      GError **error);
+static GwyDialogOutcome run_gui                      (ModuleArgs *args,
+                                                      const gchar *title);
+static gboolean         export_data_fields           (const gchar *filename,
+                                                      GwyContainer *container,
+                                                      gint id,
+                                                      const ModuleArgs *args,
+                                                      GError **error);
+static gint*            gather_compatible_data_fields(GwyContainer *data,
+                                                      GwyDataField *dfield,
+                                                      guint *nchannels);
+static gboolean         export_surfaces              (const gchar *filename,
+                                                      GwyContainer *container,
+                                                      gint id,
+                                                      const ModuleArgs *args,
+                                                      GError **error);
+static gint*            gather_compatible_surfaces   (GwyContainer *data,
+                                                      GwySurface *surface,
+                                                      guint *nchannels);
+static gboolean         write_header                 (FILE *fh,
+                                                      guint nchannels,
+                                                      guint npoints,
+                                                      gchar **titles,
+                                                      GwySIUnit *xyunit,
+                                                      GwySIUnit **zunits,
+                                                      gint xres,
+                                                      gint yres,
+                                                      GError **error);
+
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Gwyddion XYZ field files."),
     "Yeti <yeti@gwyddion.net>",
-    "2.0",
+    "2.1",
     "David Nečas (Yeti)",
     "2013",
 };
@@ -137,6 +131,21 @@ module_register(void)
     return TRUE;
 }
 
+static GwyParamDef*
+define_module_params(void)
+{
+    static GwyParamDef *paramdef = NULL;
+
+    if (paramdef)
+        return paramdef;
+
+    paramdef = gwy_param_def_new();
+    gwy_param_def_set_function_name(paramdef, gwy_file_func_current());
+    gwy_param_def_add_boolean(paramdef, PARAM_ALL_CHANNELS, "all-channels",
+                              _("Multi-channel file with all compatible data"), FALSE);
+    return paramdef;
+}
+
 static gint
 gxyzf_detect(const GwyFileDetectInfo *fileinfo,
              gboolean only_name)
@@ -144,8 +153,7 @@ gxyzf_detect(const GwyFileDetectInfo *fileinfo,
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 20 : 0;
 
-    if (fileinfo->file_size < MAGIC_SIZE
-        || memcmp(fileinfo->head, MAGIC, MAGIC_SIZE) != 0)
+    if (fileinfo->file_size < MAGIC_SIZE || memcmp(fileinfo->head, MAGIC, MAGIC_SIZE) != 0)
         return 0;
 
     return 100;
@@ -255,7 +263,6 @@ gxyzf_load(const gchar *filename,
     for (id = 0; id < nchan; id++) {
         GwySurface *surface;
         GwySIUnit *unit;
-        GQuark quark;
         gchar buf[24];
 
         for (i = 0; i < npoints; i++)
@@ -270,15 +277,12 @@ gxyzf_load(const gchar *filename,
         unit = gwy_surface_get_si_unit_xy(surface);
         gwy_si_unit_assign(unit, xyunit);
 
-        quark = gwy_app_get_surface_key_for_id(id);
-        gwy_container_set_object(container, quark, surface);
+        gwy_container_set_object(container, gwy_app_get_surface_key_for_id(id), surface);
         g_object_unref(surface);
 
         g_snprintf(buf, sizeof(buf), "Title%u", id+1);
-        if ((value = g_hash_table_lookup(hash, buf))) {
-            quark = gwy_app_get_surface_title_key_for_id(id);
-            gwy_container_set_const_string(container, quark, value);
-        }
+        if ((value = g_hash_table_lookup(hash, buf)))
+            gwy_container_set_const_string(container, gwy_app_get_surface_title_key_for_id(id), value);
         gwy_file_xyz_import_log_add(container, id, NULL, filename);
     }
 
@@ -304,29 +308,28 @@ gxyzf_export(G_GNUC_UNUSED GwyContainer *data,
              GwyRunType mode,
              GError **error)
 {
-    GXYZExportArgs args;
+    ModuleArgs args;
     GwyDataField *dfield;
     GwySurface *surface;
     gint fid, sid;
     const guchar *title = NULL;
-    GwyAppPage pageno;
-    gboolean ok;
+    GwyDialogOutcome outcome;
+    gboolean ok = FALSE;
 
     gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
                                      GWY_APP_DATA_FIELD_ID, &fid,
                                      GWY_APP_SURFACE, &surface,
                                      GWY_APP_SURFACE_ID, &sid,
-                                     GWY_APP_PAGE, &pageno,
+                                     GWY_APP_PAGE, &args.pageno,
                                      0);
 
-    /* Ensure at most one is set.  We produce an error if no exportable data
-     * type is available or both types are available but neither is active.
-     * When only one is available or one is active we assume that is what the
-     * user wants to export. */
+    /* Ensure at most one is set.  We produce an error if no exportable data type is available or both types are
+     * available but neither is active. When only one is available or one is active we assume that is what the user
+     * wants to export. */
     if (dfield && surface) {
-        if (pageno != GWY_PAGE_CHANNELS)
+        if (args.pageno != GWY_PAGE_CHANNELS)
             dfield = NULL;
-        if (pageno != GWY_PAGE_XYZS)
+        if (args.pageno != GWY_PAGE_XYZS)
             surface = NULL;
     }
     if (!dfield && !surface) {
@@ -335,96 +338,71 @@ gxyzf_export(G_GNUC_UNUSED GwyContainer *data,
     }
 
     if (dfield)
-        pageno = GWY_PAGE_CHANNELS;
+        args.pageno = GWY_PAGE_CHANNELS;
     if (surface)
-        pageno = GWY_PAGE_XYZS;
+        args.pageno = GWY_PAGE_XYZS;
 
-    load_args(gwy_app_settings_get(), &args);
+    args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (dfield) {
-        gwy_container_gis_string(data, gwy_app_get_data_title_key_for_id(fid),
-                                 &title);
-    }
-    if (surface) {
-        gwy_container_gis_string(data,
-                                 gwy_app_get_surface_title_key_for_id(sid),
-                                 &title);
-    }
+    if (dfield)
+        gwy_container_gis_string(data, gwy_app_get_data_title_key_for_id(fid), &title);
+    if (surface)
+        gwy_container_gis_string(data, gwy_app_get_surface_title_key_for_id(sid), &title);
 
     if (mode == GWY_RUN_INTERACTIVE) {
-        ok = gxyzf_export_dialog(&args, pageno, title);
-        save_args(gwy_app_settings_get(), &args);
-        if (!ok) {
+        outcome = run_gui(&args, title);
+        gwy_params_save_to_settings(args.params);
+        if (outcome == GWY_DIALOG_CANCEL) {
             err_CANCELLED(error);
-            return FALSE;
+            goto fail;
         }
     }
 
     if (dfield)
-        return gxyzf_export_data_fields(filename, data, fid, &args, error);
-    if (surface)
-        return gxyzf_export_surfaces(filename, data, sid, &args, error);
-
-    g_return_val_if_reached(FALSE);
-}
-
-static gboolean
-gxyzf_export_dialog(GXYZExportArgs *args,
-                    GwyAppPage pageno,
-                    const gchar *title)
-{
-    GtkWidget *dialog, *vbox, *label, *all_channels;
-    gchar *desc = NULL;
-    gint response;
-
-    dialog = gtk_dialog_new_with_buttons(_("Export GXYZF"), NULL, 0,
-                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                         GTK_STOCK_OK, GTK_RESPONSE_OK,
-                                         NULL);
-    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
-    gwy_help_add_to_file_dialog(GTK_DIALOG(dialog), GWY_HELP_DEFAULT);
-
-    vbox = gtk_vbox_new(FALSE, 2);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox, TRUE, TRUE, 0);
-    gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
-
-    if (pageno == GWY_PAGE_CHANNELS)
-        desc = g_strdup_printf("%s %s", _("Channel:"), title);
-    else if (pageno == GWY_PAGE_XYZS)
-        desc = g_strdup_printf("%s %s", _("XYZ data:"), title);
-    label = gtk_label_new(desc);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);
-    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 6);
-    g_free(desc);
-
-    gtk_box_pack_start(GTK_BOX(vbox), gwy_label_new_header(_("Options")),
-                       FALSE, FALSE, 0);
-
-    all_channels = gtk_check_button_new_with_mnemonic(_("Multi-channel "
-                                                        "file with all "
-                                                        "compatible data"));
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(all_channels),
-                                 args->all_channels);
-    gtk_box_pack_start(GTK_BOX(vbox), all_channels, FALSE, FALSE, 0);
-
-    gtk_widget_show_all(dialog);
-    response = gtk_dialog_run(GTK_DIALOG(dialog));
-    if (response != GTK_RESPONSE_NONE) {
-        args->all_channels
-            = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(all_channels));
-        gtk_widget_destroy(dialog);
+        ok = export_data_fields(filename, data, fid, &args, error);
+    else if (surface)
+        ok = export_surfaces(filename, data, sid, &args, error);
+    else {
+        g_assert_not_reached();
     }
 
-    return response == GTK_RESPONSE_OK;
+fail:
+    g_object_unref(args.params);
+    return ok;
+}
+
+static GwyDialogOutcome
+run_gui(ModuleArgs *args, const gchar *title)
+{
+    GwyDialog *dialog;
+    GwyParamTable *table;
+    gchar *desc = NULL;
+
+    dialog = GWY_DIALOG(gwy_dialog_new(_("Export GXYZF")));
+    gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
+
+    if (args->pageno == GWY_PAGE_CHANNELS)
+        desc = _("Channel");
+    else if (args->pageno == GWY_PAGE_XYZS)
+        desc = _("XYZ data");
+
+    table = gwy_param_table_new(args->params);
+    gwy_param_table_append_info(table, INFO_CHANNEL, desc);
+    gwy_param_table_info_set_valuestr(table, INFO_CHANNEL, title);
+    gwy_param_table_append_header(table, -1, _("Options"));
+    gwy_param_table_append_checkbox(table, PARAM_ALL_CHANNELS);
+    gwy_dialog_add_content(dialog, gwy_param_table_widget(table), FALSE, FALSE, 0);
+    gwy_dialog_add_param_table(dialog, table);
+
+    return gwy_dialog_run(dialog);
 }
 
 static gboolean
-gxyzf_export_data_fields(const gchar *filename,
-                         GwyContainer *container,
-                         gint id,
-                         const GXYZExportArgs *args,
-                         GError **error)
+export_data_fields(const gchar *filename,
+                   GwyContainer *container,
+                   gint id,
+                   const ModuleArgs *args,
+                   GError **error)
 {
     gdouble *ddbl = NULL;
     gint *ids = NULL;
@@ -435,7 +413,6 @@ gxyzf_export_data_fields(const gchar *filename,
     gdouble xreal, yreal, xoff, yoff;
     gchar **titles = NULL;
     GwySIUnit *xyunit, **zunits = NULL;
-    GQuark quark;
     FILE *fh;
 
     if (!(fh = gwy_fopen(filename, "wb"))) {
@@ -443,8 +420,7 @@ gxyzf_export_data_fields(const gchar *filename,
         return FALSE;
     }
 
-    quark = gwy_app_get_data_key_for_id(id);
-    dfield = gwy_container_get_object(container, quark);
+    dfield = gwy_container_get_object(container, gwy_app_get_data_key_for_id(id));
     g_return_val_if_fail(dfield, FALSE);
 
     xres = gwy_data_field_get_xres(dfield);
@@ -455,7 +431,7 @@ gxyzf_export_data_fields(const gchar *filename,
     yoff = gwy_data_field_get_yoffset(dfield);
     xyunit = gwy_data_field_get_si_unit_xy(dfield);
 
-    if (args->all_channels)
+    if (gwy_params_get_boolean(args->params, PARAM_ALL_CHANNELS))
         ids = gather_compatible_data_fields(container, dfield, &nchannels);
     else {
         nchannels = 1;
@@ -469,15 +445,13 @@ gxyzf_export_data_fields(const gchar *filename,
     titles = g_new0(gchar*, nchannels + 1);
     d = g_new0(const gdouble*, nchannels + 1);
     for (ci = 0; ci < nchannels; ci++) {
-        quark = gwy_app_get_data_key_for_id(ids[ci]);
-        other = gwy_container_get_object(container, quark);
+        other = gwy_container_get_object(container, gwy_app_get_data_key_for_id(ids[ci]));
         zunits[ci] = gwy_data_field_get_si_unit_z(other);
         d[ci] = gwy_data_field_get_data_const(other);
         titles[ci] = gwy_app_get_data_field_title(container, ids[ci]);
     }
 
-    if (!write_header(fh, nchannels, xres*yres, titles, xyunit, zunits,
-                      xres, yres, error))
+    if (!write_header(fh, nchannels, xres*yres, titles, xyunit, zunits, xres, yres, error))
         goto fail;
 
     npts = ((size_t)nchannels + 2)*xres*yres;
@@ -523,12 +497,10 @@ gather_compatible_data_fields(GwyContainer *container, GwyDataField *dfield,
     GwyDataField *other;
     gint *ids;
     guint ci, n = 0;
-    GQuark quark;
 
     ids = gwy_app_data_browser_get_data_ids(container);
     for (ci = 0; ids[ci] > -1; ci++) {
-        quark = gwy_app_get_data_key_for_id(ids[ci]);
-        other = gwy_container_get_object(container, quark);
+        other = gwy_container_get_object(container, gwy_app_get_data_key_for_id(ids[ci]));
         if (gwy_data_field_check_compatibility(dfield, other,
                                                GWY_DATA_COMPATIBILITY_RES
                                                | GWY_DATA_COMPATIBILITY_REAL
@@ -544,11 +516,11 @@ gather_compatible_data_fields(GwyContainer *container, GwyDataField *dfield,
 }
 
 static gboolean
-gxyzf_export_surfaces(const gchar *filename,
-                      GwyContainer *container,
-                      gint id,
-                      const GXYZExportArgs *args,
-                      GError **error)
+export_surfaces(const gchar *filename,
+                GwyContainer *container,
+                gint id,
+                const ModuleArgs *args,
+                GError **error)
 {
     gdouble *ddbl = NULL;
     gint *ids = NULL;
@@ -558,7 +530,6 @@ gxyzf_export_surfaces(const gchar *filename,
     const GwyXYZ **d;
     gchar **titles = NULL;
     GwySIUnit *xyunit, **zunits = NULL;
-    GQuark quark;
     FILE *fh;
 
     if (!(fh = gwy_fopen(filename, "wb"))) {
@@ -566,14 +537,13 @@ gxyzf_export_surfaces(const gchar *filename,
         return FALSE;
     }
 
-    quark = gwy_app_get_surface_key_for_id(id);
-    surface = gwy_container_get_object(container, quark);
+    surface = gwy_container_get_object(container, gwy_app_get_surface_key_for_id(id));
     g_return_val_if_fail(surface, FALSE);
 
     xyunit = gwy_surface_get_si_unit_xy(surface);
     n = gwy_surface_get_npoints(surface);
 
-    if (args->all_channels)
+    if (gwy_params_get_boolean(args->params, PARAM_ALL_CHANNELS))
         ids = gather_compatible_surfaces(container, surface, &nchannels);
     else {
         nchannels = 1;
@@ -587,15 +557,13 @@ gxyzf_export_surfaces(const gchar *filename,
     titles = g_new0(gchar*, nchannels + 1);
     d = g_new0(const GwyXYZ*, nchannels + 1);
     for (ci = 0; ci < nchannels; ci++) {
-        quark = gwy_app_get_surface_key_for_id(ids[ci]);
-        other = gwy_container_get_object(container, quark);
+        other = gwy_container_get_object(container, gwy_app_get_surface_key_for_id(ids[ci]));
         zunits[ci] = gwy_surface_get_si_unit_z(other);
         d[ci] = gwy_surface_get_data_const(other);
         titles[ci] = gwy_app_get_surface_title(container, ids[ci]);
     }
 
-    if (!write_header(fh, nchannels, n, titles, xyunit, zunits,
-                      0, 0, error))
+    if (!write_header(fh, nchannels, n, titles, xyunit, zunits, 0, 0, error))
         goto fail;
 
     npts = ((size_t)nchannels + 2)*n;
@@ -641,12 +609,10 @@ gather_compatible_surfaces(GwyContainer *container, GwySurface *surface,
     GwySurface *other;
     gint *ids;
     guint ci, n = 0;
-    GQuark quark;
 
     ids = gwy_app_data_browser_get_xyz_ids(container);
     for (ci = 0; ids[ci] > -1; ci++) {
-        quark = gwy_app_get_surface_key_for_id(ids[ci]);
-        other = gwy_container_get_object(container, quark);
+        other = gwy_container_get_object(container, gwy_app_get_surface_key_for_id(ids[ci]));
         if (!gwy_surface_xy_is_compatible(surface, other))
             continue;
 
@@ -712,24 +678,4 @@ write_header(FILE *fh, guint nchannels, guint npoints,
     return TRUE;
 }
 
-static const gchar all_channels_key[] = "/module/gxyzfile/all-channels";
-
-static void
-load_args(GwyContainer *settings,
-          GXYZExportArgs *args)
-{
-    *args = gxyzf_defaults;
-
-    gwy_container_gis_boolean_by_name(settings, all_channels_key,
-                                      &args->all_channels);
-}
-
-static void
-save_args(GwyContainer *settings,
-          const GXYZExportArgs *args)
-{
-    gwy_container_set_boolean_by_name(settings, all_channels_key,
-                                      args->all_channels);
-}
-
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
