@@ -1,5 +1,5 @@
 /*
- *  $Id: iso28600.c 21300 2018-08-08 14:08:32Z yeti-dn $
+ *  $Id: iso28600.c 22687 2019-11-21 12:53:47Z yeti-dn $
  *  Copyright (C) 2011-2016 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -245,8 +245,8 @@ static const ISO28600HeaderField header_fields[] = {
     { "Fast scan direction",                         21,  ISO28600_TEXT_LINE, },
     { "Slow scan axis",                              22,  ISO28600_ENUM,      },
     { "Slow scan direction",                         23,  ISO28600_TEXT_LINE, },
-    { "Number of discrete X coordintes in full map", 24,  ISO28600_INTEGER,   },
-    { "Number of discrete Y coordintes in full map", 25,  ISO28600_INTEGER,   },
+    { "Number of discrete X coordinates in full map", 24, ISO28600_INTEGER,   },
+    { "Number of discrete Y coordinates in full map", 25, ISO28600_INTEGER,   },
     { "Physical unit of X axis",                     26,  ISO28600_UNIT,      },
     { "Physical unit of Y axis",                     27,  ISO28600_UNIT,      },
     { "Field of view X",                             28,  ISO28600_REAL_NUM,  },
@@ -363,17 +363,17 @@ static const gchar header_fields_name[] =
     "ds\000Number of hours in advance of GMT\000scan information\000Scan mo"
     "de\000Scanning system\000Scanner type\000Fast scan axis\000Fast scan d"
     "irection\000Slow scan axis\000Slow scan direction\000Number of discret"
-    "e X coordintes in full map\000Number of discrete Y coordintes in full "
-    "map\000Physical unit of X axis\000Physical unit of Y axis\000Field of "
-    "view X\000Field of view Y\000Physical unit of X offset\000Physical uni"
-    "t of Y offset\000X offset\000Y offset\000Rotation angle\000Physical un"
-    "it of scan speed\000Scan speed\000Physical unit of scan rate\000Scan r"
-    "ate\000SPM technique\000Bias voltage contact\000Bias voltage\000Number"
-    " of set items\000Set parameters\000Units of set parameters\000Values o"
-    "f set parameters\000Calibration comments for set parameters\000Calibra"
-    "tions for set parameters\000environment description\000Environment mod"
-    "e\000Sample temperature\000Surroundings pressure\000Environment humidi"
-    "ty\000Comment (environment)\000probe description\000Probe identifier"
+    "e X coordinates in full map\000Number of discrete Y coordinates in ful"
+    "l map\000Physical unit of X axis\000Physical unit of Y axis\000Field o"
+    "f view X\000Field of view Y\000Physical unit of X offset\000Physical u"
+    "nit of Y offset\000X offset\000Y offset\000Rotation angle\000Physical "
+    "unit of scan speed\000Scan speed\000Physical unit of scan rate\000Scan"
+    " rate\000SPM technique\000Bias voltage contact\000Bias voltage\000Numb"
+    "er of set items\000Set parameters\000Units of set parameters\000Values"
+    " of set parameters\000Calibration comments for set parameters\000Calib"
+    "rations for set parameters\000environment description\000Environment m"
+    "ode\000Sample temperature\000Surroundings pressure\000Environment humi"
+    "dity\000Comment (environment)\000probe description\000Probe identifier"
     "\000Probe material\000Normal spring constant\000Resonance frequency"
     "\000Cantilever sensitvity\000Angle between probe and X axis\000Angle b"
     "etween probe vertical movement and Z axis in X azimuth\000Angle betwee"
@@ -430,110 +430,110 @@ header_fields[] = {
     { 366, 22, ISO28600_ENUM },
     { 381, 23, ISO28600_TEXT_LINE },
     { 401, 24, ISO28600_INTEGER },
-    { 445, 25, ISO28600_INTEGER },
-    { 489, 26, ISO28600_UNIT },
-    { 513, 27, ISO28600_UNIT },
-    { 537, 28, ISO28600_REAL_NUM },
-    { 553, 29, ISO28600_REAL_NUM },
-    { 569, 30, ISO28600_UNIT },
-    { 595, 31, ISO28600_UNIT },
-    { 621, 32, ISO28600_REAL_NUM },
-    { 630, 33, ISO28600_REAL_NUM },
-    { 639, 34, ISO28600_REAL_NUM },
-    { 654, 35, ISO28600_UNIT },
-    { 682, 36, ISO28600_REAL_NUM },
-    { 693, 37, ISO28600_UNIT },
-    { 720, 38, ISO28600_REAL_NUM },
-    { 730, 39, ISO28600_TEXT_LINE },
-    { 744, 40, ISO28600_ENUM },
-    { 765, 41, ISO28600_REAL_NUM },
-    { 778, 42, ISO28600_INTEGER },
-    { 798, 43, ISO28600_TEXT_LIST },
-    { 813, 44, ISO28600_UNITS },
-    { 837, 45, ISO28600_REAL_NUMS },
-    { 862, 46, ISO28600_TEXT_LIST },
-    { 902, 47, ISO28600_REAL_NUMS },
-    { 934, 48, ISO28600_FIXED },
-    { 958, 49, ISO28600_TEXT_LINE },
-    { 975, 50, ISO28600_REAL_NUM },
-    { 994, 51, ISO28600_REAL_NUM },
-    { 1016, 52, ISO28600_REAL_NUM },
-    { 1037, 53, ISO28600_TEXT_LINE },
-    { 1059, 54, ISO28600_FIXED },
-    { 1077, 55, ISO28600_TEXT_LINE },
-    { 1094, 56, ISO28600_TEXT_LINE },
-    { 1109, 57, ISO28600_REAL_NUM },
-    { 1132, 58, ISO28600_REAL_NUM },
-    { 1152, 59, ISO28600_REAL_NUM },
-    { 1174, 60, ISO28600_REAL_NUM },
-    { 1205, 61, ISO28600_REAL_NUM },
-    { 1267, 62, ISO28600_REAL_NUM },
-    { 1329, 63, ISO28600_TEXT_LINE },
-    { 1345, 64, ISO28600_FIXED },
-    { 1364, 65, ISO28600_TEXT_LINE },
-    { 1382, 66, ISO28600_TEXT_LINE },
-    { 1396, 67, ISO28600_TEXT_LINE },
-    { 1413, 68, ISO28600_FIXED },
-    { 1448, 69, ISO28600_TEXT_LINE },
-    { 1463, 70, ISO28600_UNIT },
-    { 1495, 71, ISO28600_TEXT_LINE },
-    { 1520, 72, ISO28600_FIXED },
-    { 1545, 73, ISO28600_TEXT_LINE },
-    { 1563, 74, ISO28600_ENUM },
-    { 1586, 75, ISO28600_TEXT_LINE },
-    { 1601, 76, ISO28600_UNIT },
-    { 1615, 77, ISO28600_REAL_NUM },
-    { 1630, 78, ISO28600_REAL_NUM },
-    { 1643, 79, ISO28600_REAL_NUM },
-    { 1662, 80, ISO28600_REAL_NUM },
-    { 1696, 81, ISO28600_INTEGER },
-    { 1725, 82, ISO28600_INTEGER },
-    { 1750, 83, ISO28600_TEXT_LIST },
-    { 1766, 84, ISO28600_UNITS },
-    { 1781, 85, ISO28600_REAL_NUMS },
-    { 1817, 86, ISO28600_TEXT_LINE },
-    { 1840, 87, ISO28600_FIXED },
-    { 1867, 88, ISO28600_ENUM },
-    { 1882, 89, ISO28600_TEXT_LINE },
-    { 1899, 90, ISO28600_TEXT_LINE },
-    { 1919, 91, ISO28600_TEXT_LINE },
-    { 1940, 92, ISO28600_TEXT_LINE },
-    { 1965, 93, ISO28600_FIXED },
-    { 1999, 94, ISO28600_INTEGER },
-    { 2023, 95, ISO28600_TEXT_LINE },
-    { 2040, 96, ISO28600_UNIT },
-    { 2063, 97, ISO28600_TEXT_LINE },
-    { 2088, 98, ISO28600_TEXT_LINE },
-    { 2105, 99, ISO28600_UNIT },
-    { 2128, 100, ISO28600_TEXT_LINE },
-    { 2153, 101, ISO28600_TEXT_LINE },
-    { 2170, 102, ISO28600_UNIT },
-    { 2193, 103, ISO28600_TEXT_LINE },
-    { 2218, 104, ISO28600_TEXT_LINE },
-    { 2235, 105, ISO28600_UNIT },
-    { 2258, 106, ISO28600_TEXT_LINE },
-    { 2283, 107, ISO28600_TEXT_LINE },
-    { 2300, 108, ISO28600_UNIT },
-    { 2323, 109, ISO28600_TEXT_LINE },
-    { 2348, 110, ISO28600_TEXT_LINE },
-    { 2365, 111, ISO28600_UNIT },
-    { 2388, 112, ISO28600_TEXT_LINE },
-    { 2413, 113, ISO28600_TEXT_LINE },
-    { 2430, 114, ISO28600_UNIT },
-    { 2453, 115, ISO28600_TEXT_LINE },
-    { 2478, 116, ISO28600_TEXT_LINE },
-    { 2495, 117, ISO28600_UNIT },
-    { 2518, 118, ISO28600_TEXT_LINE },
-    { 2543, 119, ISO28600_TEXT_LINE },
-    { 2575, 120, ISO28600_RESERVED },
-    { 2576, 121, ISO28600_RESERVED },
-    { 2577, 122, ISO28600_RESERVED },
-    { 2578, 123, ISO28600_RESERVED },
-    { 2579, 124, ISO28600_RESERVED },
-    { 2580, 125, ISO28600_RESERVED },
-    { 2581, 126, ISO28600_RESERVED },
-    { 2582, 127, ISO28600_RESERVED },
-    { 2583, 128, ISO28600_FIXED },
+    { 446, 25, ISO28600_INTEGER },
+    { 491, 26, ISO28600_UNIT },
+    { 515, 27, ISO28600_UNIT },
+    { 539, 28, ISO28600_REAL_NUM },
+    { 555, 29, ISO28600_REAL_NUM },
+    { 571, 30, ISO28600_UNIT },
+    { 597, 31, ISO28600_UNIT },
+    { 623, 32, ISO28600_REAL_NUM },
+    { 632, 33, ISO28600_REAL_NUM },
+    { 641, 34, ISO28600_REAL_NUM },
+    { 656, 35, ISO28600_UNIT },
+    { 684, 36, ISO28600_REAL_NUM },
+    { 695, 37, ISO28600_UNIT },
+    { 722, 38, ISO28600_REAL_NUM },
+    { 732, 39, ISO28600_TEXT_LINE },
+    { 746, 40, ISO28600_ENUM },
+    { 767, 41, ISO28600_REAL_NUM },
+    { 780, 42, ISO28600_INTEGER },
+    { 800, 43, ISO28600_TEXT_LIST },
+    { 815, 44, ISO28600_UNITS },
+    { 839, 45, ISO28600_REAL_NUMS },
+    { 864, 46, ISO28600_TEXT_LIST },
+    { 904, 47, ISO28600_REAL_NUMS },
+    { 936, 48, ISO28600_FIXED },
+    { 960, 49, ISO28600_TEXT_LINE },
+    { 977, 50, ISO28600_REAL_NUM },
+    { 996, 51, ISO28600_REAL_NUM },
+    { 1018, 52, ISO28600_REAL_NUM },
+    { 1039, 53, ISO28600_TEXT_LINE },
+    { 1061, 54, ISO28600_FIXED },
+    { 1079, 55, ISO28600_TEXT_LINE },
+    { 1096, 56, ISO28600_TEXT_LINE },
+    { 1111, 57, ISO28600_REAL_NUM },
+    { 1134, 58, ISO28600_REAL_NUM },
+    { 1154, 59, ISO28600_REAL_NUM },
+    { 1176, 60, ISO28600_REAL_NUM },
+    { 1207, 61, ISO28600_REAL_NUM },
+    { 1269, 62, ISO28600_REAL_NUM },
+    { 1331, 63, ISO28600_TEXT_LINE },
+    { 1347, 64, ISO28600_FIXED },
+    { 1366, 65, ISO28600_TEXT_LINE },
+    { 1384, 66, ISO28600_TEXT_LINE },
+    { 1398, 67, ISO28600_TEXT_LINE },
+    { 1415, 68, ISO28600_FIXED },
+    { 1450, 69, ISO28600_TEXT_LINE },
+    { 1465, 70, ISO28600_UNIT },
+    { 1497, 71, ISO28600_TEXT_LINE },
+    { 1522, 72, ISO28600_FIXED },
+    { 1547, 73, ISO28600_TEXT_LINE },
+    { 1565, 74, ISO28600_ENUM },
+    { 1588, 75, ISO28600_TEXT_LINE },
+    { 1603, 76, ISO28600_UNIT },
+    { 1617, 77, ISO28600_REAL_NUM },
+    { 1632, 78, ISO28600_REAL_NUM },
+    { 1645, 79, ISO28600_REAL_NUM },
+    { 1664, 80, ISO28600_REAL_NUM },
+    { 1698, 81, ISO28600_INTEGER },
+    { 1727, 82, ISO28600_INTEGER },
+    { 1752, 83, ISO28600_TEXT_LIST },
+    { 1768, 84, ISO28600_UNITS },
+    { 1783, 85, ISO28600_REAL_NUMS },
+    { 1819, 86, ISO28600_TEXT_LINE },
+    { 1842, 87, ISO28600_FIXED },
+    { 1869, 88, ISO28600_ENUM },
+    { 1884, 89, ISO28600_TEXT_LINE },
+    { 1901, 90, ISO28600_TEXT_LINE },
+    { 1921, 91, ISO28600_TEXT_LINE },
+    { 1942, 92, ISO28600_TEXT_LINE },
+    { 1967, 93, ISO28600_FIXED },
+    { 2001, 94, ISO28600_INTEGER },
+    { 2025, 95, ISO28600_TEXT_LINE },
+    { 2042, 96, ISO28600_UNIT },
+    { 2065, 97, ISO28600_TEXT_LINE },
+    { 2090, 98, ISO28600_TEXT_LINE },
+    { 2107, 99, ISO28600_UNIT },
+    { 2130, 100, ISO28600_TEXT_LINE },
+    { 2155, 101, ISO28600_TEXT_LINE },
+    { 2172, 102, ISO28600_UNIT },
+    { 2195, 103, ISO28600_TEXT_LINE },
+    { 2220, 104, ISO28600_TEXT_LINE },
+    { 2237, 105, ISO28600_UNIT },
+    { 2260, 106, ISO28600_TEXT_LINE },
+    { 2285, 107, ISO28600_TEXT_LINE },
+    { 2302, 108, ISO28600_UNIT },
+    { 2325, 109, ISO28600_TEXT_LINE },
+    { 2350, 110, ISO28600_TEXT_LINE },
+    { 2367, 111, ISO28600_UNIT },
+    { 2390, 112, ISO28600_TEXT_LINE },
+    { 2415, 113, ISO28600_TEXT_LINE },
+    { 2432, 114, ISO28600_UNIT },
+    { 2455, 115, ISO28600_TEXT_LINE },
+    { 2480, 116, ISO28600_TEXT_LINE },
+    { 2497, 117, ISO28600_UNIT },
+    { 2520, 118, ISO28600_TEXT_LINE },
+    { 2545, 119, ISO28600_TEXT_LINE },
+    { 2577, 120, ISO28600_RESERVED },
+    { 2578, 121, ISO28600_RESERVED },
+    { 2579, 122, ISO28600_RESERVED },
+    { 2580, 123, ISO28600_RESERVED },
+    { 2581, 124, ISO28600_RESERVED },
+    { 2582, 125, ISO28600_RESERVED },
+    { 2583, 126, ISO28600_RESERVED },
+    { 2584, 127, ISO28600_RESERVED },
+    { 2585, 128, ISO28600_FIXED },
 };
 #endif  /* }}} */
 
