@@ -1,6 +1,6 @@
 /*
- *  $Id: keyence.c 24459 2021-11-03 17:28:15Z yeti-dn $
- *  Copyright (C) 2015-2021 David Necas (Yeti).
+ *  $Id: keyence.c 24931 2022-08-24 15:13:25Z yeti-dn $
+ *  Copyright (C) 2015-2022 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -14,6 +14,16 @@
  *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+/**
+ * [FILE-MAGIC-FREEDESKTOP]
+ * <mime-type type="application/x-keyence-vk3">
+ *   <comment>Keyence VK3 profilometry data</comment>
+ *   <magic priority="80">
+ *     <match type="string" offset="0" value="VK3_"/>
+ *   </magic>
+ * </mime-type>
+ **/
+
 /**
  * [FILE-MAGIC-FREEDESKTOP]
  * <mime-type type="application/x-keyence-vk4">
@@ -36,6 +46,24 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-FREEDESKTOP]
+ * <mime-type type="application/x-keyence-vk7">
+ *   <comment>Keyence VK7 profilometry data</comment>
+ *   <magic priority="80">
+ *     <match type="string" offset="0" value="VK7">
+ *       <match type="string" offset="7" value="BM"/>
+ *     </match>
+ *   </magic>
+ * </mime-type>
+ **/
+
+/**
+ * [FILE-MAGIC-FILEMAGIC]
+ * # Keyence VK3.
+ * 0 string VK3_ Keyence profilometry VK3 data
+ **/
+
 /**
  * [FILE-MAGIC-FILEMAGIC]
  * # Keyence VK4.
@@ -46,13 +74,20 @@
  * [FILE-MAGIC-FILEMAGIC]
  * # Keyence VK6.
  * 0 string VK6
- * >7 string BM Keyence profilometry VK4 data
+ * >7 string BM Keyence profilometry VK7 data
+ **/
+
+/**
+ * [FILE-MAGIC-FILEMAGIC]
+ * # Keyence VK7.
+ * 0 string VK7
+ * >7 string BM Keyence profilometry VK7 data
  **/
 
 /**
  * [FILE-MAGIC-USERGUIDE]
- * Keyence profilometry VK4, VK6
- * .vk4, .vk6
+ * Keyence profilometry VK3, VK4, VK6, VK7
+ * .vk3, .vk4, .vk6, .vk7
  * Read
  **/
 
@@ -73,23 +108,32 @@
 #include "get.h"
 #include "err.h"
 
+#define MAGIC3 "VK3_"
+#define MAGIC3_SIZE (sizeof(MAGIC3)-1)
+
 #define MAGIC4 "VK4_"
 #define MAGIC4_SIZE (sizeof(MAGIC4)-1)
 
 #define MAGIC6 "VK6"
 #define MAGIC6_SIZE (sizeof(MAGIC6)-1)
 
+#define MAGIC7 "VK7"
+#define MAGIC7_SIZE (sizeof(MAGIC6)-1)
+
 #define MAGICBMP "BM"
 #define MAGICBMP_SIZE (sizeof(MAGICBMP)-1)
 
 #define MAGIC0 "\x00\x00\x00\x00"
 #define MAGIC0_SIZE (sizeof(MAGIC0)-1)
 
+#define EXTENSION3 ".vk3"
 #define EXTENSION4 ".vk4"
 #define EXTENSION6 ".vk6"
+#define EXTENSION7 ".vk7"
 
 #define Picometre (1e-12)
 
+/* VK3 and VK4 share many structures; we generally declare VK4 structures and use them also for VK3. */
 enum {
     KEYENCE4_HEADER_SIZE = 12,
     KEYENCE4_OFFSET_TABLE_SIZE = 72,
@@ -300,18 +344,25 @@ typedef struct {
 } Keyence6Meta;
 
 static gboolean      module_register     (void);
+static gint          keyence3_detect     (const GwyFileDetectInfo *fileinfo,
+                                          gboolean only_name);
 static gint          keyence4_detect     (const GwyFileDetectInfo *fileinfo,
                                           gboolean only_name);
+static GwyContainer* keyence3_load       (const gchar *filename,
+                                          GwyRunType mode,
+                                          GError **error);
 static GwyContainer* keyence4_load       (const gchar *filename,
                                           GwyRunType mode,
                                           GError **error);
 static GwyContainer* keyence4_load_membuf(const guchar *buffer,
                                           gsize size,
+                                          gint version,
                                           GError **error);
 static void          keyence4_free       (Keyence4File *kfile);
 static gboolean      read_header         (const guchar **p,
                                           gsize *size,
                                           Keyence4Header *header,
+                                          gint version,
                                           GError **error);
 static gboolean      read_offset_table   (const guchar **p,
                                           gsize *size,
@@ -352,6 +403,14 @@ static gint          keyence6_detect           (const GwyFileDetectInfo *fileinf
 static GwyContainer* keyence6_load             (const gchar *filename,
                                                 GwyRunType mode,
                                                 GError **error);
+static gint          keyence7_detect           (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static GwyContainer* keyence7_load             (const gchar *filename,
+                                                GwyRunType mode,
+                                                GError **error);
+static GwyContainer* keyence67_load_common     (const gchar *filename,
+                                                gint version,
+                                                GError **error);
 static GwyZipFile    make_temporary_zip_file   (const guchar *buffer,
                                                 gsize size,
                                                 const gchar *nametemplate,
@@ -378,9 +437,9 @@ static const gchar *gradientnames[3] = { "RGB-Red", "RGB-Green", "RGB-Blue" };
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
-    N_("Imports Keyence VK4 and VK6 files."),
+    N_("Imports Keyence VK3, VK4, VK6 and VK7 files."),
     "Yeti <yeti@gwyddion.net>",
-    "2.0",
+    "2.2",
     "David Nečas (Yeti)",
     "2015",
 };
@@ -390,6 +449,12 @@ GWY_MODULE_QUERY2(module_info, keyence)
 static gboolean
 module_register(void)
 {
+    gwy_file_func_register("keyence3",
+                           N_("Keyence VK3 data files (.vk3)"),
+                           (GwyFileDetectFunc)&keyence3_detect,
+                           (GwyFileLoadFunc)&keyence3_load,
+                           NULL,
+                           NULL);
     gwy_file_func_register("keyence4",
                            N_("Keyence VK4 data files (.vk4)"),
                            (GwyFileDetectFunc)&keyence4_detect,
@@ -403,10 +468,32 @@ module_register(void)
                            (GwyFileLoadFunc)&keyence6_load,
                            NULL,
                            NULL);
+    gwy_file_func_register("keyence7",
+                           N_("Keyence VK7 data files (.vk7)"),
+                           (GwyFileDetectFunc)&keyence7_detect,
+                           (GwyFileLoadFunc)&keyence7_load,
+                           NULL,
+                           NULL);
 #endif
     return TRUE;
 }
 
+static gint
+keyence3_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+{
+    gint score = 0;
+
+    if (only_name)
+        return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION3) ? 15 : 0;
+
+    if (fileinfo->buffer_len > MAGIC4_SIZE + KEYENCE4_HEADER_SIZE
+        && memcmp(fileinfo->head, MAGIC3, MAGIC3_SIZE) == 0
+        && memcmp(fileinfo->head + 8, MAGIC0, MAGIC0_SIZE) == 0)
+        score = 100;
+
+    return score;
+}
+
 static gint
 keyence4_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
@@ -437,8 +524,41 @@ keyence6_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 
     return 100;
 }
+
+static gint
+keyence7_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+{
+    if (only_name)
+        return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION7) ? 15 : 0;
+
+    if (fileinfo->buffer_len <= KEYENCE6_HEADER_SIZE + BMP_HEADER_SIZE
+        || memcmp(fileinfo->head, MAGIC7, MAGIC7_SIZE) != 0
+        || memcmp(fileinfo->head + KEYENCE6_HEADER_SIZE, MAGICBMP, MAGICBMP_SIZE) != 0)
+        return 0;
+
+    return 100;
+}
 #endif
 
+static GwyContainer*
+keyence3_load(const gchar *filename,
+              G_GNUC_UNUSED GwyRunType mode,
+              GError **error)
+{
+    GwyContainer *data;
+    GError *err = NULL;
+    guchar *buffer;
+    gsize size;
+
+    if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
+        err_GET_FILE_CONTENTS(error, &err);
+        return NULL;
+    }
+    data = keyence4_load_membuf(buffer, size, 3, error);
+    gwy_file_abandon_contents(buffer, size, NULL);
+    return data;
+}
+
 static GwyContainer*
 keyence4_load(const gchar *filename,
               G_GNUC_UNUSED GwyRunType mode,
@@ -453,7 +573,7 @@ keyence4_load(const gchar *filename,
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
     }
-    data = keyence4_load_membuf(buffer, size, error);
+    data = keyence4_load_membuf(buffer, size, 4, error);
     gwy_file_abandon_contents(buffer, size, NULL);
     return data;
 }
@@ -461,6 +581,7 @@ keyence4_load(const gchar *filename,
 static GwyContainer*
 keyence4_load_membuf(const guchar *buffer,
                      gsize size,
+                     gint version,
                      GError **error)
 {
     Keyence4File *kfile;
@@ -474,10 +595,11 @@ keyence4_load_membuf(const guchar *buffer,
     kfile->size = size;
     p = kfile->buffer;
 
-    if (!read_header(&p, &size, &kfile->header, error)
+    gwy_debug("format version %d", version);
+    if (!read_header(&p, &size, &kfile->header, version, error)
         || !read_offset_table(&p, &size, &kfile->offset_table, error)
         || !read_meas_conds(&p, &size, &kfile->meas_conds, error)
-        || !read_assembly_info(kfile, error)
+        || (version == 4 && !read_assembly_info(kfile, error))
         || !read_data_images(kfile, error)
         || !read_color_images(kfile, error)
         || !read_line_meas(kfile, error)
@@ -541,20 +663,29 @@ static gboolean
 read_header(const guchar **p,
             gsize *size,
             Keyence4Header *header,
+            gint version,
             GError **error)
 {
     gwy_debug("remaining size 0x%08lx", (gulong)*size);
     if (*size < KEYENCE4_HEADER_SIZE) {
-        err_TRUNCATED_PART(error, "Keyence4Header");
+        err_TRUNCATED_PART(error, version == 3 ? "Keyence3Header" : "Keyence4Header");
         return FALSE;
     }
 
     get_CHARARRAY(header->magic, p);
     get_CHARARRAY(header->dll_version, p);
     get_CHARARRAY(header->file_type, p);
-    if (memcmp(header->magic, MAGIC4, MAGIC4_SIZE) != 0 || memcmp(header->file_type, MAGIC0, MAGIC0_SIZE) != 0) {
-        err_FILE_TYPE(error, "Keyence VK4");
-        return FALSE;
+    if (version == 3) {
+        if (memcmp(header->magic, MAGIC3, MAGIC3_SIZE) != 0 || memcmp(header->file_type, MAGIC0, MAGIC0_SIZE) != 0) {
+            err_FILE_TYPE(error, "Keyence VK3");
+            return FALSE;
+        }
+    }
+    else {
+        if (memcmp(header->magic, MAGIC4, MAGIC4_SIZE) != 0 || memcmp(header->file_type, MAGIC0, MAGIC0_SIZE) != 0) {
+            err_FILE_TYPE(error, "Keyence VK4");
+            return FALSE;
+        }
     }
 
     *size -= KEYENCE4_HEADER_SIZE;
@@ -1201,6 +1332,22 @@ static GwyContainer*
 keyence6_load(const gchar *filename,
               G_GNUC_UNUSED GwyRunType mode,
               GError **error)
+{
+    return keyence67_load_common(filename, 6, error);
+}
+
+static GwyContainer*
+keyence7_load(const gchar *filename,
+              G_GNUC_UNUSED GwyRunType mode,
+              GError **error)
+{
+    return keyence67_load_common(filename, 7, error);
+}
+
+static GwyContainer*
+keyence67_load_common(const gchar *filename,
+                      gint version,
+                      GError **error)
 {
     GwyContainer *data = NULL, *meta = NULL;
     GError *err = NULL;
@@ -1216,16 +1363,27 @@ keyence6_load(const gchar *filename,
         return NULL;
     }
 
-    if (size <= KEYENCE6_HEADER_SIZE + BMP_HEADER_SIZE
-        || memcmp(buffer, MAGIC6, MAGIC6_SIZE) != 0
-        || memcmp(buffer + KEYENCE6_HEADER_SIZE, MAGICBMP, MAGICBMP_SIZE) != 0) {
-        err_FILE_TYPE(error, "Keyence VK6");
-        goto fail;
+    if (version == 7) {
+        if (size <= KEYENCE6_HEADER_SIZE + BMP_HEADER_SIZE
+            || memcmp(buffer, MAGIC7, MAGIC7_SIZE) != 0
+            || memcmp(buffer + KEYENCE6_HEADER_SIZE, MAGICBMP, MAGICBMP_SIZE) != 0) {
+            err_FILE_TYPE(error, "Keyence VK7");
+            goto fail;
+        }
+        p = buffer + MAGIC7_SIZE;
+    }
+    else {
+        if (size <= KEYENCE6_HEADER_SIZE + BMP_HEADER_SIZE
+            || memcmp(buffer, MAGIC6, MAGIC6_SIZE) != 0
+            || memcmp(buffer + KEYENCE6_HEADER_SIZE, MAGICBMP, MAGICBMP_SIZE) != 0) {
+            err_FILE_TYPE(error, "Keyence VK6");
+            goto fail;
+        }
+        p = buffer + MAGIC6_SIZE;
     }
 
     /* Check if the VK6 miniheader agrees with the BMP header on the BMP file size.  This is a strong indication we
      * are dealing with a VK6 file. */
-    p = buffer + MAGIC6_SIZE;
     vk6skip = gwy_get_guint32_le(&p);
     p = buffer + KEYENCE6_HEADER_SIZE + MAGICBMP_SIZE;
     bmpsize = gwy_get_guint32_le(&p);
@@ -1250,7 +1408,7 @@ keyence6_load(const gchar *filename,
         || !(vk4buf = gwyzip_get_file_content(zipfile, &vk4size, error)))
         goto fail;
 
-    if (!(data = keyence4_load_membuf(vk4buf, vk4size, error)))
+    if (!(data = keyence4_load_membuf(vk4buf, vk4size, 4, error)))
         goto fail;
 
     add_vk6_hdr_images(data, zipfile);
