@@ -1,5 +1,5 @@
 /*
- *  $Id: miffile.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: miffile.c 21783 2019-01-03 12:58:14Z yeti-dn $
  *  Copyright (C) 2010 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -700,7 +700,7 @@ mif_read_data_field(const MIFImageHeader *image_header,
         linecorrdata = g_new(gdouble, yres);
         d16 = (const gint16*)(buffer + block->offset + datasize);
         for (i = 0; i < yres; i++) {
-            linecorrdata[i] = pow(2.0, d16[i] - 13);
+            linecorrdata[i] = gwy_powi(2.0, d16[i] - 13);
             gwy_debug("corr[%u] %g", i, linecorrdata[i]);
         }
     }
