@@ -1,5 +1,5 @@
 /*
- *  $Id: mifile.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: mifile.c 25134 2022-11-02 14:00:39Z yeti-dn $
  *  Copyright (C) 2005 Chris Anderson, Molecular Imaging Corp.
  *  E-mail: sidewinder.asu@gmail.com
  *
@@ -260,7 +260,7 @@ mifile_load(const gchar *filename,
     if (!header_size)
          ok = FALSE;
 
-    gwy_debug("header_size: %i", header_size);
+    gwy_debug("header_size: %lu", header_size);
 
     /* Report error if file is invalid */
     if (!ok) {
@@ -508,6 +508,8 @@ find_data_start(const guchar *buffer, gsize size, MIDataType *data_type)
     else
         value = 0;
 
+    gwy_debug("data start at 0x%x", value);
+
     return value;
 }
 
