@@ -1,24 +1,20 @@
 /*
- *  $Id: nanonics.c 23930 2021-08-06 14:54:38Z yeti-dn $
+ *  $Id: nanonics.c 24542 2021-11-25 14:22:14Z yeti-dn $
  *  Copyright (C) 2009-2011 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
-
+//#define DEBUG 1
 /**
  * [FILE-MAGIC-FREEDESKTOP]
  * <mime-type type="application/x-nanonics-spm">
@@ -50,8 +46,10 @@
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwyutils.h>
 #include <libgwyddion/gwymath.h>
+#include <libgwyddion/gwyexpr.h>
 #include <libgwymodule/gwymodule-file.h>
 #include <libprocess/datafield.h>
+#include <libprocess/stats.h>
 #include <app/gwymoduleutils-file.h>
 #include <app/data-browser.h>
 
@@ -82,28 +80,28 @@ typedef struct {
     GHashTable **pagemeta;
 } NanonicsFile;
 
-static gboolean      module_register         (void);
-static gint          nanonics_detect         (const GwyFileDetectInfo *fileinfo,
-                                              gboolean only_name);
-static GwyContainer* nanonics_load           (const gchar *filename,
-                                              GwyRunType mode,
-                                              GError **error);
-static GHashTable*   nanonics_read_header    (gchar *text,
-                                              const gchar *name,
-                                              GError **error);
-static void          nanonics_parse_comment  (GHashTable *hash,
-                                              const gchar *comment);
-static GwyDataField* nanonics_read_data_field(const NanonicsFile *nfile,
-                                              guint id,
-                                              gboolean retrace,
-                                              const guchar *buffer);
+static gboolean      module_register          (void);
+static gint          nanonics_detect          (const GwyFileDetectInfo *fileinfo,
+                                               gboolean only_name);
+static GwyContainer* nanonics_load            (const gchar *filename,
+                                               GwyRunType mode,
+                                               GError **error);
+static GHashTable*   nanonics_read_header     (gchar *text,
+                                               const gchar *name,
+                                               GError **error);
+static void          nanonics_parse_comment   (GHashTable *hash,
+                                               const gchar *comment);
+static void          nanonics_read_data_fields(const NanonicsFile *nfile,
+                                               guint id,
+                                               const guchar *buffer,
+                                               GwyDataField **fields);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Nanonics NAN data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.4",
+    "0.5",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -130,8 +128,7 @@ nanonics_detect(const GwyFileDetectInfo *fileinfo,
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
 
-    if (fileinfo->buffer_len < MAGIC_SIZE
-        || memcmp(fileinfo->head, MAGIC, MAGIC_SIZE) != 0)
+    if (fileinfo->buffer_len < MAGIC_SIZE || memcmp(fileinfo->head, MAGIC, MAGIC_SIZE) != 0)
         return 0;
 
     return 80;
@@ -163,8 +160,7 @@ nanonics_load(const gchar *filename,
         goto fail;
     }
 
-    header = g_strndup(buffer + MAGIC_LINE_SIZE,
-                       MIN(size - MAGIC_LINE_SIZE, size_guess));
+    header = g_strndup(buffer + MAGIC_LINE_SIZE, MIN(size - MAGIC_LINE_SIZE, size_guess));
     if ((s = strstr(header, "HeaderLength="))) {
         header_size = g_ascii_strtoull(s + strlen("HeaderLength="), NULL, 10);
         if (header_size > size_guess && header_size <= size - MAGIC_LINE_SIZE) {
@@ -174,8 +170,7 @@ nanonics_load(const gchar *filename,
     }
     if (!(s = strstr(header, "-End Header-"))) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Expected header end marker ‘%s’ was not found."),
-                    "-End Header-");
+                    _("Expected header end marker ‘%s’ was not found."), "-End Header-");
         goto fail;
     }
     header_size = (s - header) + strlen("-End Header-");
@@ -197,17 +192,12 @@ nanonics_load(const gchar *filename,
     g_free(header);
     header = NULL;
 
-    if (!require_keys(nfile.meta, error,
-                      "HeaderLength", "DataLength",
-                      "ReF", "ReS", "WSF", "WSS",
-                      NULL))
+    if (!require_keys(nfile.meta, error, "HeaderLength", "DataLength", "ReF", "ReS", "WSF", "WSS", NULL))
         goto fail;
 
     /* Must explicitly specify base, the numbers often start with 0 */
-    nfile.header_size = strtol(g_hash_table_lookup(nfile.meta, "HeaderLength"),
-                               NULL, 10);
-    nfile.page_size = strtol(g_hash_table_lookup(nfile.meta, "DataLength"),
-                             NULL, 10);
+    nfile.header_size = strtol(g_hash_table_lookup(nfile.meta, "HeaderLength"), NULL, 10);
+    nfile.page_size = strtol(g_hash_table_lookup(nfile.meta, "DataLength"), NULL, 10);
 
     if (nfile.header_size != header_size + MAGIC_LINE_SIZE) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
@@ -220,9 +210,7 @@ nanonics_load(const gchar *filename,
         ndata = g_ascii_strtoull(s, NULL, 10);
         /* It should be only set if we did find any channel descriptions. */
         g_assert(ndata);
-        if (err_SIZE_MISMATCH(error,
-                              nfile.page_size*ndata,
-                              size - nfile.header_size, FALSE))
+        if (err_SIZE_MISMATCH(error, nfile.page_size*ndata, size - nfile.header_size, FALSE))
             goto fail;
         gwy_debug("ndata (from comment): %u", ndata);
     }
@@ -260,17 +248,12 @@ nanonics_load(const gchar *filename,
     /* Well, there is probably a stricter page header size lower bound than 4.
      * If there is a mismatch it can be an incomplete file.  Try to fix yres
      * if it looks sane and the data seem to consist of complete lines.  */
-    if (err_SIZE_MISMATCH(NULL, nfile.page_data_size + 4, nfile.page_size,
-                          FALSE)) {
-        void *p = gwy_memmem(buffer + nfile.header_size, nfile.page_size,
-                             END_CHANNEL_MAGIC, strlen(END_CHANNEL_MAGIC));
-        guint data_size = (nfile.page_size 
-                           - ((guchar*)p + strlen(END_CHANNEL_MAGIC)
-                              - (buffer + nfile.header_size)));
+    if (err_SIZE_MISMATCH(NULL, nfile.page_data_size + 4, nfile.page_size, FALSE)) {
+        void *p = gwy_memmem(buffer + nfile.header_size, nfile.page_size, END_CHANNEL_MAGIC, strlen(END_CHANNEL_MAGIC));
+        guint data_size = (nfile.page_size - ((guchar*)p + strlen(END_CHANNEL_MAGIC) - (buffer + nfile.header_size)));
         guint newyres = data_size/(sizeof(guint32)*nfile.xres);
 
-        gwy_debug("true data length is %u, trying to fix yres to %u",
-                  data_size, newyres);
+        gwy_debug("true data length is %u, trying to fix yres to %u", data_size, newyres);
         if (err_DIMENSION(error, newyres))
             goto fail;
 
@@ -279,8 +262,7 @@ nanonics_load(const gchar *filename,
         nfile.page_data_size = sizeof(guint32)*nfile.xres*nfile.yres;
         gwy_debug("fixed page data size: %u", nfile.page_data_size);
     }
-    if (err_SIZE_MISMATCH(error, nfile.page_data_size + 4, nfile.page_size,
-                          FALSE))
+    if (err_SIZE_MISMATCH(error, nfile.page_data_size + 4, nfile.page_size, FALSE))
         goto fail;
 
     nfile.page_header_size = nfile.page_size - nfile.page_data_size;
@@ -290,15 +272,13 @@ nanonics_load(const gchar *filename,
     for (i = 0; i < ndata; i++) {
         gwy_debug("reading page header %u", i);
         header = g_strndup(s + i*nfile.page_size, nfile.page_header_size);
-        if (!(nfile.pagemeta[i] = nanonics_read_header(header, "Channel Header",
-                                                       &err))) {
+        if (!(nfile.pagemeta[i] = nanonics_read_header(header, "Channel Header", &err))) {
             if (i == 0) {
                 g_propagate_error(error, err);
                 goto fail;
             }
             else {
-                g_warning("Cannot read the expected number of channels %u, "
-                          "failed after %u.", ndata, i);
+                g_warning("Cannot read the expected number of channels %u, failed after %u.", ndata, i);
                 ndata = i;
             }
         }
@@ -309,18 +289,17 @@ nanonics_load(const gchar *filename,
     container = gwy_container_new();
     s += nfile.page_header_size;
     for (i = 0; i < ndata; i++) {
+        GwyDataField *fields[2];
         guint j;
 
+        nanonics_read_data_fields(&nfile, i, s + i*nfile.page_size, fields);
         for (j = 0; j < 2; j++) {
-            GwyDataField *dfield;
             gchar *key, *title;
             GQuark quark;
 
-            dfield = nanonics_read_data_field(&nfile, i, j,
-                                              s + i*nfile.page_size);
             quark = gwy_app_get_data_key_for_id(2*i + j);
-            gwy_container_set_object(container, quark, dfield);
-            g_object_unref(dfield);
+            gwy_container_set_object(container, quark, fields[j]);
+            g_object_unref(fields[j]);
 
             if ((title = g_hash_table_lookup(nfile.pagemeta[i], "CHN"))) {
                 key = g_strconcat(g_quark_to_string(quark), "/title", NULL);
@@ -329,12 +308,10 @@ nanonics_load(const gchar *filename,
                     gwy_container_set_string_by_name(container, key, title);
                 }
                 else
-                    gwy_container_set_string_by_name(container, key,
-                                                     g_strdup(title));
+                    gwy_container_set_string_by_name(container, key, g_strdup(title));
                 g_free(key);
             }
-            gwy_file_channel_import_log_add(container, 2*i + j, NULL,
-                                            filename);
+            gwy_file_channel_import_log_add(container, 2*i + j, NULL, filename);
         }
     }
 
@@ -368,8 +345,7 @@ nanonics_read_header(gchar *text, const gchar *name, GError **error)
     marker = g_strdup_printf("-Start %s-", name);
     if (!gwy_strequal(line, marker)) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Expected header start marker ‘%s’ but found ‘%s’."),
-                    marker, line);
+                    _("Expected header start marker ‘%s’ but found ‘%s’."), marker, line);
         g_free(marker);
         return NULL;
     }
@@ -391,9 +367,7 @@ nanonics_read_header(gchar *text, const gchar *name, GError **error)
                 gwy_debug("comment: <%s>", comment->str);
                 g_string_erase(comment, 0, 1);
                 g_string_truncate(comment, comment->len-1);
-                g_hash_table_insert(hash,
-                                    g_strdup("comment"),
-                                    g_string_free(comment, FALSE));
+                g_hash_table_insert(hash, g_strdup("comment"), g_string_free(comment, FALSE));
                 comment = NULL;
             }
             continue;
@@ -443,8 +417,7 @@ nanonics_read_header(gchar *text, const gchar *name, GError **error)
 
     if (!line) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Expected header end marker ‘%s’ was not found."),
-                    marker);
+                    _("Expected header end marker ‘%s’ was not found."), marker);
         g_hash_table_destroy(hash);
         g_free(marker);
         return NULL;
@@ -475,11 +448,9 @@ nanonics_parse_comment(GHashTable *hash,
         }
         if (reading_channels) {
             if (sscanf(line, "%d )", &id) == 1) {
-                if (id != nchannels) {
-                    g_warning("Channel #%u has non-sequential number %u.",
-                              nchannels, id);
-                    id = nchannels;
-                }
+                gwy_debug("channel id %d", id);
+                /* Channel numbers can be only odd numbers or some other set.  Just number out channels sequentially
+                 * and do not make noise. */
                 line = strchr(line, ')') + 1;
                 fields = g_strsplit(line, ",", 0);
                 if (fields
@@ -487,17 +458,14 @@ nanonics_parse_comment(GHashTable *hash,
                     && g_str_has_prefix(fields[1], "Units:")
                     && g_str_has_prefix(fields[2], "Formula:")) {
                     value = g_strstrip(fields[0]);
-                    gwy_debug("Channel %u name: <%s>", id, value);
-                    g_hash_table_insert(hash, g_strdup_printf("Channel%u", id),
-                                        g_strdup(value));
+                    gwy_debug("Channel %u name: <%s>", nchannels, value);
+                    g_hash_table_insert(hash, g_strdup_printf("Channel%u", nchannels), g_strdup(value));
                     value = g_strstrip(fields[1] + strlen("Units:"));
-                    gwy_debug("Channel %u units: <%s>", id, value);
-                    g_hash_table_insert(hash, g_strdup_printf("Units%u", id),
-                                        g_strdup(value));
+                    gwy_debug("Channel %u units: <%s>", nchannels, value);
+                    g_hash_table_insert(hash, g_strdup_printf("Units%u", nchannels), g_strdup(value));
                     value = g_strstrip(fields[2] + strlen("Formula:"));
-                    gwy_debug("Channel %u formula: <%s>", id, value);
-                    g_hash_table_insert(hash, g_strdup_printf("Formula%u", id),
-                                        g_strdup(value));
+                    gwy_debug("Channel %u formula: <%s>", nchannels, value);
+                    g_hash_table_insert(hash, g_strdup_printf("Formula%u", nchannels), g_strdup(value));
                 }
                 g_strfreev(fields);
                 nchannels++;
@@ -505,6 +473,12 @@ nanonics_parse_comment(GHashTable *hash,
             }
             reading_channels = FALSE;
         }
+        if ((value = strstr(line, "Z axis"))) {
+            value += 6;
+            g_strstrip(value);
+            g_hash_table_insert(hash, g_strdup("Z_axis"), g_strdup(value));
+            continue;
+        }
         if ((value = strstr(line, ": "))) {
             *value = '\0';
             value += 2;
@@ -516,69 +490,136 @@ nanonics_parse_comment(GHashTable *hash,
     }
 
     if (nchannels)
-        g_hash_table_insert(hash, g_strdup("Number of channels"),
-                            g_strdup_printf("%u", nchannels));
+        g_hash_table_insert(hash, g_strdup("Number of channels"), g_strdup_printf("%u", nchannels));
 
     g_free(buffer);
 }
 
-static GwyDataField*
-nanonics_read_data_field(const NanonicsFile *nfile,
-                         guint id,
-                         gboolean retrace,
-                         const guchar *buffer)
+static gboolean
+get_double_value(GHashTable *hash, const gchar *name, gdouble *value)
+{
+    const gchar *s;
+    gchar *end;
+
+    if (!(s = g_hash_table_lookup(hash, name))) {
+        gwy_debug("cannot find %s", name);
+        return FALSE;
+    }
+
+    *value = g_ascii_strtod(s, &end);
+    if (end == s) {
+        gwy_debug("cannot parse %s", name);
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+static gdouble
+nanonics_read_formula(const NanonicsFile *nfile, guint id)
+{
+    const gchar *const var_names[] = { "x", "DZ", "div", "RDiv" };
+
+    gdouble var_values[G_N_ELEMENTS(var_names)];
+    guint var_positions[G_N_ELEMENTS(var_names)];
+    gdouble vars[G_N_ELEMENTS(var_names) + 1];
+    GwyExpr *expr;
+    GError *err = NULL;
+    const gchar *e;
+    gdouble c = 1.0;
+    gint i;
+
+    var_values[0] = 1.0;
+    if (!get_double_value(nfile->meta, "Z_axis", var_values+1) || !get_double_value(nfile->meta, "ZDiv", var_values+2))
+        return 1.0;
+    var_values[3] = 1.0;
+
+    if (!(e = g_hash_table_lookup(nfile->pagemeta[id], "CHF"))) {
+        gwy_debug("cannot find channel expression CHF");
+        return 1.0;
+    }
+
+    expr = gwy_expr_new();
+    if (!gwy_expr_compile(expr, e, &err)) {
+        gwy_debug("CHF %s does not compile: %s", e, err->message);
+        g_clear_error(&err);
+        goto end;
+    }
+
+    if (gwy_expr_resolve_variables(expr, G_N_ELEMENTS(var_names), var_names, var_positions)) {
+        gwy_debug("CHF %s contains unknown variables", e);
+        goto end;
+    }
+
+    for (i = 0; i < G_N_ELEMENTS(var_names); i++)
+        vars[var_positions[i]] = var_values[i];
+    c = gwy_expr_execute(expr, vars);
+
+end:
+    gwy_expr_free(expr);
+    return c;
+}
+
+static void
+nanonics_read_data_fields(const NanonicsFile *nfile,
+                          guint id,
+                          const guchar *buffer,
+                          GwyDataField **fields)
 {
     const gint16 *d16 = (const gint16*)buffer;
-    GwyDataField *dfield;
-    GwySIUnit *siunit;
     gint xres, yres, i, j, power10;
+    gdouble cmn = sizeof("This variable is not used unitinitialised");
+    gdouble cmx = sizeof("This variable is not used unitinitialised");
+    gdouble c, min, max;
+    gboolean have_minmax = FALSE;
     gdouble *d;
-    gchar *s;
+    const gchar *s;
 
     xres = nfile->xres;
     yres = nfile->yres;
-    dfield = gwy_data_field_new(xres, yres, nfile->xreal, nfile->yreal, FALSE);
-    d = gwy_data_field_get_data(dfield);
-
+    fields[0] = gwy_data_field_new(xres, yres, nfile->xreal, nfile->yreal, FALSE);
+    d = gwy_data_field_get_data(fields[0]);
     for (i = 0; i < yres; i++) {
-        if (retrace) {
-            for (j = 0; j < xres; j++) {
-                gint16 v = d16[(i + 1)*2*xres - 1 - j];
-                *(d++) = GINT16_FROM_BE(v);
-            }
+        for (j = 0; j < xres; j++) {
+            gint16 v = d16[i*2*xres + j];
+            *(d++) = GINT16_FROM_BE(v);
         }
-        else {
-            for (j = 0; j < xres; j++) {
-                gint16 v = d16[i*2*xres + j];
-                *(d++) = GINT16_FROM_BE(v);
-            }
+    }
+    fields[1] = gwy_data_field_new_alike(fields[0], FALSE);
+    d = gwy_data_field_get_data(fields[1]);
+    for (i = 0; i < yres; i++) {
+        for (j = 0; j < xres; j++) {
+            gint16 v = d16[(i + 1)*2*xres - 1 - j];
+            *(d++) = GINT16_FROM_BE(v);
         }
     }
 
-    siunit = gwy_data_field_get_si_unit_xy(dfield);
-    gwy_si_unit_set_from_string(siunit, "m");
+    /* Use first channel min, max and formula to calibrate data */
+    if (get_double_value(nfile->pagemeta[id], "CMX", &cmx) && get_double_value(nfile->pagemeta[id], "CMN", &cmn)) {
+        gwy_data_field_get_min_max(fields[0], &min, &max);
+        have_minmax = TRUE;
+    }
+    c = nanonics_read_formula(nfile, id);
 
     if ((s = g_hash_table_lookup(nfile->pagemeta[id], "CHU"))) {
-        /* Fix some verbose units
-         * XXX: Modifies the string in-place (can do since it's allocated but
-         * be careful). */
+        /* Fix some silly units */
         if (gwy_strequal(s, "Pi")) {
-            gwy_data_field_multiply(dfield, G_PI);
-            s[0] = '\0';
+            c *= G_PI;
+            s = NULL;
         }
-        if (g_str_has_suffix(s, "Volts"))
-            s[strlen(s) - strlen("Volts") + 1] = '\0';
-        else if (g_str_has_suffix(s, "Newton"))
-            s[strlen(s) - strlen("Newton") + 1] = '\0';
-
-        siunit = gwy_data_field_get_si_unit_z(dfield);
-        gwy_si_unit_set_from_string_parse(siunit, s, &power10);
-
-        if (power10)
-            gwy_data_field_multiply(dfield, pow10(power10));
     }
 
-    return dfield;
+    for (j = 0; j < 2; j++) {
+        GwyDataField *dfield = fields[j];
+
+        if (have_minmax) {
+            gwy_data_field_multiply(dfield, (cmx - cmn)/(max - min));
+            gwy_data_field_add(dfield, (cmx + cmn)/2);
+        }
+        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield), s, &power10);
+        gwy_data_field_multiply(dfield, c*pow10(power10));
+    }
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
