@@ -1,5 +1,5 @@
 /*
- *  $Id: nanonis-spec.c 21299 2018-08-08 13:32:01Z yeti-dn $
+ *  $Id: nanonis-spec.c 22881 2020-07-16 14:06:41Z yeti-dn $
  *  Copyright (C) 2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -97,7 +97,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Nanonis DAT spectrum files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.2",
+    "1.3",
     "David Nečas (Yeti)",
     "2018",
 };
@@ -144,7 +144,8 @@ dat_detect(const GwyFileDetectInfo *fileinfo,
     p += MAGIC1_SIZE;
 
     /* These fields seem universal. */
-    if (find_field_in_head(fileinfo, p, "Date")
+    if ((find_field_in_head(fileinfo, p, "Date")
+         || find_field_in_head(fileinfo, p, "Saved Date"))
         && find_field_in_head(fileinfo, p, "User")
         && (find_field_in_head(fileinfo, p, "X (m)")
             || find_field_in_head(fileinfo, p, "x (m)"))
@@ -193,6 +194,7 @@ dat_load(const gchar *filename,
     }
 
     for (i = j = 0; i < specset.template->ncolumns; i++) {
+        gwy_debug("[%u:%u] %p", i, j, specset.spectra[j]);
         if (!specset.spectra[i])
             continue;
 
@@ -235,15 +237,29 @@ check_compatibility(const DATSingleFile *datfile,
     const DATColumn *datcolumn, *tmplcolumn;
     guint i;
 
-    if (datfile->ncolumns != template->ncolumns)
+    if (datfile->ncolumns != template->ncolumns) {
+        gwy_debug("datfile->ncolumns(%u) != template->ncolumns(%u)",
+                  datfile->ncolumns, template->ncolumns);
         return FALSE;
+    }
     for (i = 0; i < template->ncolumns; i++) {
         datcolumn = datfile->columns + i;
         tmplcolumn = template->columns + i;
-        if (!gwy_strequal(datcolumn->name, tmplcolumn->name)
-            || !gwy_strequal0(datcolumn->ext, tmplcolumn->ext)
-            || !gwy_strequal0(datcolumn->unit, tmplcolumn->unit))
+        if (!gwy_strequal(datcolumn->name, tmplcolumn->name)) {
+            gwy_debug("[%u] datcolumn->name(%s) != tmplcolumn->name(%s)",
+                      i, datcolumn->name, tmplcolumn->name);
             return FALSE;
+        }
+        if (!gwy_strequal0(datcolumn->ext, tmplcolumn->ext)) {
+            gwy_debug("[%u] datcolumn->ext(%s) != tmplcolumn->ext(%s)",
+                      i, datcolumn->ext, tmplcolumn->ext);
+            return FALSE;
+        }
+        if (!gwy_strequal0(datcolumn->unit, tmplcolumn->unit)) {
+            gwy_debug("[%u], datcolumn->unit(%s) != tmplcolumn->unit(%s)",
+                      i, datcolumn->unit, tmplcolumn->unit);
+            return FALSE;
+        }
     }
     return TRUE;
 }
@@ -323,30 +339,26 @@ add_curves_to_spectra_set(DATSpectraSet *specset,
         specset->spectra = g_new0(GwySpectra*, ncolumns);
 
     reversed = FALSE;
-    /* Find the Bias column and use it as abscissa.  There are also Bias calc,
-     * Bias [bwd], but they should be all identical. */
-    for (i = 0; i < ncolumns; i++) {
-        if (!gwy_strequal(datfile->columns[i].name, "Bias"))
-            continue;
-
-        abscissa = datfile->columns + i;
-        real = data[i + ncolumns*(nrows - 1)];
-        off = data[i];
-        if (real < off) {
-            GWY_SWAP(gdouble, real, off);
-            reversed = TRUE;
-        }
-        real -= off;
-        xlabel = make_axis_label(abscissa);
-        break;
+    /* Use the first column as the abscissa. */
+    i = 0;
+    abscissa = datfile->columns + i;
+    real = data[i + ncolumns*(nrows - 1)];
+    off = data[i];
+    if (real < off) {
+        GWY_SWAP(gdouble, real, off);
+        reversed = TRUE;
     }
-    if (!abscissa)
+    real -= off;
+    xlabel = make_axis_label(abscissa);
+    if (!abscissa) {
+        gwy_debug("cannot find abscissa");
         return;
+    }
 
     /* Find ordinate columns. */
     for (i = 0; i < ncolumns; i++) {
         ordinate = datfile->columns + i;
-        if (g_str_has_prefix(ordinate->name, "Bias"))
+        if (gwy_strequal(ordinate->name, abscissa->name))
             continue;
 
         dline = gwy_data_line_new(nrows, real, FALSE);
