@@ -1,22 +1,18 @@
 /*
- *  $Id: nanonis.c 21958 2019-04-03 15:00:27Z yeti-dn $
+ *  $Id: nanonis.c 25057 2022-09-30 10:57:12Z yeti-dn $
  *  Copyright (C) 2006,2015-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -53,9 +49,8 @@
 #include <libgwyddion/gwyutils.h>
 #include <libprocess/datafield.h>
 #include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
+#include <app/gwyapp.h>
 #include <app/gwymoduleutils-file.h>
-#include <app/settings.h>
 
 #include "err.h"
 
@@ -64,6 +59,10 @@
 
 #define EXTENSION ".sxm"
 
+enum {
+    PARAM_PRESRRVE_COORDINATES
+};
+
 typedef enum {
     DIR_FORWARD  = 1 << 0,
     DIR_BACKWARD = 1 << 1,
@@ -93,39 +92,27 @@ typedef struct {
     gdouble yreal;
     gdouble xoff;
     gdouble yoff;
-    /* Set if the times are set to N/A or NaN, this seems to be done in slice
-     * files of 3D data.  We cannot trust direction filed then as it's set to
-     * `both' although the file contains one direction only. */
+    /* Set if the times are set to N/A or NaN, this seems to be done in slice files of 3D data.  We cannot trust
+     * direction filed then as it's set to `both' although the file contains one direction only. */
     gboolean bogus_scan_time;
 } SXMFile;
 
-typedef struct {
-    gboolean preserve_coordinates;
-} SXMArgs;
-
-static gboolean      module_register(void);
-static gint          sxm_detect     (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* sxm_load       (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
-static GwyContainer* sxm_build_meta (const SXMFile *sxmfile,
-                                     guint id);
-static void          sxm_load_args  (GwyContainer *container,
-                                     SXMArgs *args);
-static void          sxm_save_args  (GwyContainer *container,
-                                     const SXMArgs *args);
-
-static const SXMArgs sxm_defaults = {
-    FALSE,
-};
+static gboolean      module_register     (void);
+static GwyParamDef*  define_module_params(void);
+static gint          sxm_detect          (const GwyFileDetectInfo *fileinfo,
+                                          gboolean only_name);
+static GwyContainer* sxm_load            (const gchar *filename,
+                                          GwyRunType mode,
+                                          GError **error);
+static GwyContainer* sxm_build_meta      (const SXMFile *sxmfile,
+                                          guint id);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Nanonis SXM data files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.3",
+    "1.4",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -152,6 +139,21 @@ module_register(void)
     return TRUE;
 }
 
+static GwyParamDef*
+define_module_params(void)
+{
+    static GwyParamDef *paramdef = NULL;
+
+    if (paramdef)
+        return paramdef;
+
+    paramdef = gwy_param_def_new();
+    gwy_param_def_set_function_name(paramdef, gwy_file_func_current());
+    gwy_param_def_add_boolean(paramdef, PARAM_PRESRRVE_COORDINATES, "preserve_coordinates", "Preserve coordinates",
+                              FALSE);
+    return paramdef;
+}
+
 static gint
 sxm_detect(const GwyFileDetectInfo *fileinfo,
            gboolean only_name)
@@ -161,8 +163,7 @@ sxm_detect(const GwyFileDetectInfo *fileinfo,
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 20 : 0;
 
-    if (fileinfo->buffer_len > MAGIC_SIZE
-        && memcmp(fileinfo->head, MAGIC, MAGIC_SIZE) == 0)
+    if (fileinfo->buffer_len > MAGIC_SIZE && memcmp(fileinfo->head, MAGIC, MAGIC_SIZE) == 0)
         score = 100;
 
     return score;
@@ -297,8 +298,7 @@ sxm_read_tag(SXMFile *sxmfile,
 
         line = (gchar*)g_ptr_array_index(header_lines, lineno + 1);
         sxmfile->z_controller_values = split_line_in_place(line, '\t');
-        if (g_strv_length(sxmfile->z_controller_headers)
-            != g_strv_length(sxmfile->z_controller_values)) {
+        if (g_strv_length(sxmfile->z_controller_headers) != g_strv_length(sxmfile->z_controller_values)) {
             g_warning("The numbers of Z-CONTROLLER headers and values differ");
             sxm_free_z_controller(sxmfile);
         }
@@ -327,10 +327,8 @@ sxm_read_tag(SXMFile *sxmfile,
             || !gwy_strequal(columns[3], "Direction")
             || !gwy_strequal(columns[4], "Calibration")
             || !gwy_strequal(columns[5], "Offset")) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
-                        _("DATA_INFO does not contain the expected "
-                          "columns: %s."),
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
+                        _("DATA_INFO does not contain the expected columns: %s."),
                         "Channel Name Unit Direction Calibration Offset");
             g_free(columns);
             return 0;
@@ -349,8 +347,7 @@ sxm_read_tag(SXMFile *sxmfile,
                 continue;
             columns = split_line_in_place(line, '\t');
             if (g_strv_length(columns) < 6) {
-                g_set_error(error, GWY_MODULE_FILE_ERROR,
-                            GWY_MODULE_FILE_ERROR_DATA,
+                g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                             _("DATA_INFO line contains fewer than %d fields."),
                             6);
                 g_free(columns);
@@ -361,9 +358,7 @@ sxm_read_tag(SXMFile *sxmfile,
             di.channel = atoi(columns[0]);
             di.name = columns[1];
             di.unit = columns[2];
-            di.direction = gwy_string_to_enum(columns[3],
-                                              directions,
-                                              G_N_ELEMENTS(directions));
+            di.direction = gwy_string_to_enum(columns[3], directions, G_N_ELEMENTS(directions));
             if (di.direction == (SXMDirection)-1) {
                 err_INVALID(error, "Direction");
                 g_free(columns);
@@ -383,8 +378,7 @@ sxm_read_tag(SXMFile *sxmfile,
         g_array_free(data_info, FALSE);
     }
     else if (n) {
-        /* Generic tag.  We replace line ends with spaces since metadata are
-         * single-line strings... */
+        /* Generic tag.  We replace line ends with spaces since metadata are single-line strings... */
         if (n > 1)
             s = join_lines(header_lines, lineno, n);
         else
@@ -403,9 +397,10 @@ read_data_field(GwyContainer *container,
                 const SXMFile *sxmfile,
                 const SXMDataInfo *data_info,
                 SXMDirection dir,
-                SXMArgs *args,
+                GwyParams *params,
                 const guchar **p)
 {
+    gboolean preserve_coordinates = gwy_params_get_boolean(params, PARAM_PRESRRVE_COORDINATES);
     GwyContainer *meta;
     GwyDataField *dfield, *mfield = NULL;
     gdouble *data, *mdata;
@@ -413,9 +408,7 @@ read_data_field(GwyContainer *container,
     gchar *s;
     gboolean flip_vertically = FALSE, flip_horizontally = FALSE;
 
-    dfield = gwy_data_field_new(sxmfile->xres, sxmfile->yres,
-                                sxmfile->xreal, sxmfile->yreal,
-                                FALSE);
+    dfield = gwy_data_field_new(sxmfile->xres, sxmfile->yres, sxmfile->xreal, sxmfile->yreal, FALSE);
     /* This is correct for both preserved and non-preserved coordinates? */
     gwy_data_field_set_xoffset(dfield, sxmfile->xoff - 0.5*sxmfile->xreal);
     gwy_data_field_set_yoffset(dfield, sxmfile->yoff - 0.5*sxmfile->yreal);
@@ -448,35 +441,27 @@ read_data_field(GwyContainer *container,
     }
 
     gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield),
-                                data_info->unit);
-    gwy_container_set_object(container, gwy_app_get_data_key_for_id(*id),
-                             dfield);
+    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), data_info->unit);
+    gwy_container_set_object(container, gwy_app_get_data_key_for_id(*id), dfield);
 
     if (mfield) {
         gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(mfield), "m");
-        gwy_container_set_object(container, gwy_app_get_mask_key_for_id(*id),
-                                 mfield);
+        gwy_container_set_object(container, gwy_app_get_mask_key_for_id(*id), mfield);
     }
 
     if (!dir) {
-        gwy_container_set_const_string(container,
-                                       gwy_app_get_data_title_key_for_id(*id),
-                                       data_info->name);
+        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(*id), data_info->name);
     }
     else {
         gchar *title;
 
-        title = g_strdup_printf("%s (%s)", data_info->name,
-                                dir == DIR_BACKWARD ? "Backward" : "Forward");
-        gwy_container_set_string(container,
-                                 gwy_app_get_data_title_key_for_id(*id), title);
+        title = g_strdup_printf("%s (%s)", data_info->name, dir == DIR_BACKWARD ? "Backward" : "Forward");
+        gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(*id), title);
         /* Don't free title, container eats it */
     }
 
     if ((meta = sxm_build_meta(sxmfile, *id))) {
-        gwy_container_set_object(container,
-                                 gwy_app_get_data_meta_key_for_id(*id), meta);
+        gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(*id), meta);
         g_object_unref(meta);
     }
 
@@ -485,13 +470,11 @@ read_data_field(GwyContainer *container,
     if (dir == DIR_BACKWARD)
         flip_horizontally = TRUE;
 
-    /* With preserve_coordinates images are flipped vertically with respect to
-     * Nanonis software.  But it hopefully makes possible to put SPS points at
-     * correct positions.  We cannot do that when reading the spectra later
+    /* With preserve_coordinates images are flipped vertically with respect to Nanonis software.  But it hopefully
+     * makes possible to put SPS points at correct positions.  We cannot do that when reading the spectra later
      * because spectra do not know about ‘their’ images. */
     if ((s = g_hash_table_lookup(sxmfile->meta, "SCAN_DIR"))) {
-        if ((args->preserve_coordinates && gwy_strequal(s, "down"))
-            || (!args->preserve_coordinates && gwy_strequal(s, "up")))
+        if ((preserve_coordinates && gwy_strequal(s, "down")) || (!preserve_coordinates && gwy_strequal(s, "up")))
             flip_vertically = TRUE;
     }
 
@@ -514,9 +497,8 @@ sxm_load(const gchar *filename,
          GError **error)
 {
     SXMFile sxmfile;
-    SXMArgs args;
-    GwyContainer *settings;
     GwyContainer *container = NULL;
+    GwyParams *params;
     GPtrArray *header_lines;
     guchar *buffer = NULL;
     gsize size1 = 0, size = 0;
@@ -527,9 +509,6 @@ sxm_load(const gchar *filename,
     gint version;
     guint i;
 
-    settings = gwy_app_settings_get();
-    sxm_load_args(settings, &args);
-
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
@@ -598,8 +577,7 @@ sxm_load(const gchar *filename,
     if ((s = g_hash_table_lookup(sxmfile.meta, "NANONIS_VERSION")))
         version = atoi(s);
     else {
-        g_warning("Version is missing, assuming old files.  "
-                  "How it can happen, anyway?");
+        g_warning("Version is missing, assuming old files.  How it can happen, anyway?");
         version = 0;
     }
 
@@ -611,8 +589,7 @@ sxm_load(const gchar *filename,
             if (g_strv_length(columns) == 2
                 && gwy_strequal(columns[0], "FLOAT")
                 /* XXX: No matter what they say, the files seems to be BE */
-                && (gwy_strequal(columns[1], "LSBFIRST")
-                    || gwy_strequal(columns[1], "MSBFIRST"))) {
+                && (gwy_strequal(columns[1], "LSBFIRST") || gwy_strequal(columns[1], "MSBFIRST"))) {
                 size1 = sizeof(gfloat);
             }
             else {
@@ -715,11 +692,9 @@ sxm_load(const gchar *filename,
 
     /* Scan times, check for bogus values indicating generated slice files. */
     if (sxmfile.ok) {
-        if ((s = g_hash_table_lookup(sxmfile.meta, "ACQ_TIME"))
-            && gwy_strequal(s, "N/A"))
+        if ((s = g_hash_table_lookup(sxmfile.meta, "ACQ_TIME")) && gwy_strequal(s, "N/A"))
             sxmfile.bogus_scan_time = TRUE;
-        else if ((s = g_hash_table_lookup(sxmfile.meta, "SCAN_TIME"))
-                 && strncmp(s, "NaN", 3) == 0)
+        else if ((s = g_hash_table_lookup(sxmfile.meta, "SCAN_TIME")) && strncmp(s, "NaN", 3) == 0)
             sxmfile.bogus_scan_time = TRUE;
     }
 
@@ -752,6 +727,7 @@ sxm_load(const gchar *filename,
     }
 
     /* Read data */
+    params = gwy_params_new_from_settings(define_module_params());
     if (sxmfile.ok) {
         gint id = 0;
 
@@ -760,16 +736,11 @@ sxm_load(const gchar *filename,
             guint d = sxmfile.data_info[i].direction;
 
             if (d == DIR_BOTH) {
-                read_data_field(container, &id, filename,
-                                &sxmfile, sxmfile.data_info + i,
-                                DIR_FORWARD, &args, &p);
-                read_data_field(container, &id, filename,
-                                &sxmfile, sxmfile.data_info + i,
-                                DIR_BACKWARD, &args, &p);
+                read_data_field(container, &id, filename, &sxmfile, sxmfile.data_info + i, DIR_FORWARD, params, &p);
+                read_data_field(container, &id, filename, &sxmfile, sxmfile.data_info + i, DIR_BACKWARD, params, &p);
             }
             else if (d == DIR_FORWARD || d == DIR_BACKWARD) {
-                read_data_field(container, &id, filename,
-                                &sxmfile, sxmfile.data_info + i, d, &args, &p);
+                read_data_field(container, &id, filename, &sxmfile, sxmfile.data_info + i, d, params, &p);
             }
             else {
                 g_assert_not_reached();
@@ -782,9 +753,10 @@ sxm_load(const gchar *filename,
     g_hash_table_destroy(sxmfile.meta);
     g_free(header);
     gwy_file_abandon_contents(buffer, size, NULL);
-    /* This may seem pointless, but it creates the line in settings file so
-     * someone wishing to change it only has to change the value. */
-    sxm_save_args(settings, &args);
+    /* This may seem pointless, but it creates the line in settings file so someone wishing to change it only has to
+     * change the value. */
+    gwy_params_save_to_settings(params);
+    g_object_unref(params);
 
     return container;
 }
@@ -811,8 +783,7 @@ add_metadata(gpointer hkey,
 
     t = g_strsplit(key, ">", 0);
     key = g_strjoinv("::", t);
-    gwy_container_set_const_string_by_name(GWY_CONTAINER(user_data), key,
-                                           value);
+    gwy_container_set_const_string_by_name(GWY_CONTAINER(user_data), key, value);
     g_free(key);
     g_strfreev(t);
 }
@@ -832,16 +803,13 @@ sxm_build_meta(const SXMFile *sxmfile,
     if ((value = g_hash_table_lookup(hash, "REC_TIME")))
         gwy_container_set_string_by_name(meta, "Time", g_strdup(value));
     if ((value = g_hash_table_lookup(hash, "REC_TEMP")))
-        gwy_container_set_string_by_name(meta, "Temperature",
-                                         reformat_float("%g K", value));
+        gwy_container_set_string_by_name(meta, "Temperature", reformat_float("%g K", value));
     if ((value = g_hash_table_lookup(hash, "ACQ_TIME")))
-        gwy_container_set_string_by_name(meta, "Acquistion time",
-                                         reformat_float("%g s", value));
+        gwy_container_set_string_by_name(meta, "Acquistion time", reformat_float("%g s", value));
     if ((value = g_hash_table_lookup(hash, "SCAN_FILE")))
         gwy_container_set_string_by_name(meta, "File name", g_strdup(value));
     if ((value = g_hash_table_lookup(hash, "BIAS")))
-        gwy_container_set_string_by_name(meta, "Bias",
-                                         reformat_float("%g V", value));
+        gwy_container_set_string_by_name(meta, "Bias", reformat_float("%g V", value));
     if ((value = g_hash_table_lookup(hash, "SCAN_DIR")))
         gwy_container_set_string_by_name(meta, "Direction", g_strdup(value));
 
@@ -866,30 +834,4 @@ sxm_build_meta(const SXMFile *sxmfile,
     return NULL;
 }
 
-static const gchar preserve_coordinates_key[] = "/module/nanonis/preserve_coordinates";
-
-static void
-sxm_sanitize_args(SXMArgs *args)
-{
-    args->preserve_coordinates = !!args->preserve_coordinates;
-}
-
-static void
-sxm_save_args(GwyContainer *container, const SXMArgs *args)
-{
-    gwy_container_set_boolean_by_name(container, preserve_coordinates_key,
-                                      args->preserve_coordinates);
-}
-
-static void
-sxm_load_args(GwyContainer *container, SXMArgs *args)
-{
-    *args = sxm_defaults;
-
-    gwy_container_gis_boolean_by_name(container, preserve_coordinates_key,
-                                      &args->preserve_coordinates);
-
-    sxm_sanitize_args(args);
-}
-
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
