@@ -1,21 +1,17 @@
 /*
- *  $Id: nanoobserver.c 22001 2019-04-16 13:58:04Z yeti-dn $
+ *  $Id: nanoobserver.c 25056 2022-09-29 11:15:01Z yeti-dn $
  *  Copyright (C) 2012-2017 David Necas (Yeti).
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -76,6 +72,7 @@
 #define MAGIC133_0 "NAO_v133.txt"
 #define MAGIC133_1 "Data/Imaging.xml"
 #define MAGIC133_2 "Data/Spectro.xml"
+#define MAGIC133_S "Data/Streams/"
 
 #define BLOODY_UTF8_BOM "\xef\xbb\xbf"
 #define EXTENSION ".nao"
@@ -200,7 +197,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Reads Nano-Solution/NanoObserver .nao files."),
     "Yeti <yeti@gwyddion.net>",
-    "2.1",
+    "2.2",
     "David Nečas (Yeti)",
     "2012",
 };
@@ -249,10 +246,8 @@ nao123_detect(const GwyFileDetectInfo *fileinfo)
     GwyZipFile zipfile;
     gint score = 0;
 
-    /* It contains directory Scan so this should be somewehre near the begining
-     * of the file. */
-    if (gwy_memmem(fileinfo->head, fileinfo->buffer_len,
-                   MAGIC123_0, sizeof(MAGIC123_0)-1)) {
+    /* It contains directory Scan so this should be somewehre near the begining of the file. */
+    if (gwy_memmem(fileinfo->head, fileinfo->buffer_len, MAGIC123_0, sizeof(MAGIC123_0)-1)) {
         gwy_debug("found magic0 %s", MAGIC123_0);
     }
     else {
@@ -260,16 +255,13 @@ nao123_detect(const GwyFileDetectInfo *fileinfo)
         return 0;
     }
 
-    if (gwy_memmem(fileinfo->head, fileinfo->buffer_len,
-                   MAGIC123_1, sizeof(MAGIC123_1)-1)) {
+    if (gwy_memmem(fileinfo->head, fileinfo->buffer_len, MAGIC123_1, sizeof(MAGIC123_1)-1)) {
         gwy_debug("found magic1 %s", MAGIC123_1);
     }
-    else if (gwy_memmem(fileinfo->head, fileinfo->buffer_len,
-                        MAGIC123_2, sizeof(MAGIC123_2)-1)) {
+    else if (gwy_memmem(fileinfo->head, fileinfo->buffer_len, MAGIC123_2, sizeof(MAGIC123_2)-1)) {
         gwy_debug("found magic2 %s", MAGIC123_2);
     }
-    else if (gwy_memmem(fileinfo->head, fileinfo->buffer_len,
-                        MAGIC123_3, sizeof(MAGIC123_3)-1)) {
+    else if (gwy_memmem(fileinfo->head, fileinfo->buffer_len, MAGIC123_3, sizeof(MAGIC123_3)-1)) {
         gwy_debug("found magic3 %s", MAGIC123_3);
     }
     else {
@@ -298,35 +290,48 @@ nao133_detect(const GwyFileDetectInfo *fileinfo)
 {
     GwyZipFile zipfile;
     gint score = 0;
+    gboolean have_marker_file = TRUE, have_xml_file = TRUE;
 
-    /* It contains directory Scan so this should be somewehre near the begining
-     * of the file. */
-    if (gwy_memmem(fileinfo->head, fileinfo->buffer_len,
-                   MAGIC133_0, sizeof(MAGIC133_0)-1)) {
+    /* It contains directory Scan so this should be somewehre near the begining of the file. */
+    if (gwy_memmem(fileinfo->head, fileinfo->buffer_len, MAGIC133_0, sizeof(MAGIC133_0)-1)
+        || gwy_memmem(fileinfo->tail, fileinfo->buffer_len, MAGIC133_0, sizeof(MAGIC133_0)-1)) {
         gwy_debug("found magic0 %s", MAGIC133_0);
     }
     else {
-        gwy_debug("not found magic0 %s", MAGIC133_0);
-        return 0;
+        gwy_debug("not found magic0 %s, trying harder", MAGIC133_0);
+        have_marker_file = FALSE;
     }
 
-    if (gwy_memmem(fileinfo->head, fileinfo->buffer_len,
-                   MAGIC133_1, sizeof(MAGIC133_1)-1)) {
+    if (gwy_memmem(fileinfo->head, fileinfo->buffer_len, MAGIC133_1, sizeof(MAGIC133_1)-1)) {
         gwy_debug("found magic1 %s", MAGIC133_1);
     }
-    else if (gwy_memmem(fileinfo->head, fileinfo->buffer_len,
-                        MAGIC133_2, sizeof(MAGIC133_2)-1)) {
+    else if (gwy_memmem(fileinfo->head, fileinfo->buffer_len, MAGIC133_2, sizeof(MAGIC133_2)-1)) {
         gwy_debug("found magic2 %s", MAGIC133_2);
     }
     else {
-        gwy_debug("no magic filename fragment found.");
-        return 0;
+        gwy_debug("no magic filename fragment found, trying harder");
+        have_xml_file = FALSE;
+    }
+
+    if (!have_marker_file && !have_xml_file) {
+        gwy_debug("looking desperately for a data file name fragment");
+        if (!gwy_memmem(fileinfo->head, fileinfo->buffer_len, MAGIC133_S, sizeof(MAGIC133_S)-1)) {
+            gwy_debug("no typical filename found, giving up");
+            return 0;
+        }
+        gwy_debug("at least %s found, proceeding", MAGIC133_S);
     }
 
     /* We have to realy look inside. */
     if (!(zipfile = gwyzip_open(fileinfo->name, NULL)))
         return 0;
 
+    if (!gwyzip_locate_file(zipfile, MAGIC133_0, 1, NULL)) {
+        gwy_debug("marker file NAO_v133.txt not found in the archive");
+        gwyzip_close(zipfile);
+        return 0;
+    }
+
     if (gwyzip_locate_file(zipfile, "Data/Imaging.xml", 1, NULL)) {
         gwy_debug("found Data/Imaging.xml in the archive");
         score = 100;
@@ -336,8 +341,7 @@ nao133_detect(const GwyFileDetectInfo *fileinfo)
         score = 100;
     }
     else {
-        gwy_debug("cannot find neither Data/Imaging.xml nor Data/Spectro.xml "
-                  "in the archive");
+        gwy_debug("cannot find neither Data/Imaging.xml nor Data/Spectro.xml in the archive");
     }
     gwyzip_close(zipfile);
 
@@ -382,15 +386,14 @@ nao_load(const gchar *filename,
 static void
 add_meta(gpointer hkey, gpointer hvalue, gpointer user_data)
 {
-    gwy_container_set_string_by_name(GWY_CONTAINER(user_data),
-                                     (gchar*)hkey, g_strdup((gchar*)hvalue));
+    gwy_container_set_string_by_name(GWY_CONTAINER(user_data), (gchar*)hkey, g_strdup((gchar*)hvalue));
 }
 
 static GwyDataField*
 nao_read_field(GwyZipFile zipfile, NAOFile *naofile, guint id, GError **error)
 {
     gsize size, expected_size;
-    guint width, G_GNUC_UNUSED height, nscanlines, i, j;
+    guint width, G_GNUC_UNUSED height, nscanlines, i;
     guchar *buffer;
     const guchar *p;
     const gchar *units;
@@ -401,8 +404,7 @@ nao_read_field(GwyZipFile zipfile, NAOFile *naofile, guint id, GError **error)
         return NULL;
 
     if (size < 3*4 + 4 + 4) {
-        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Data block is truncated"));
+        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA, _("Data block is truncated"));
         g_free(buffer);
         return NULL;
     }
@@ -419,26 +421,21 @@ nao_read_field(GwyZipFile zipfile, NAOFile *naofile, guint id, GError **error)
         return NULL;
     }
 
-    field = gwy_data_field_new(width, nscanlines,
-                               naofile->xreal,
-                               naofile->yreal*nscanlines/naofile->yres,
-                               TRUE);
+    field = gwy_data_field_new(width, nscanlines, naofile->xreal, naofile->yreal*nscanlines/naofile->yres, TRUE);
     data = gwy_data_field_get_data(field);
 
     for (i = 0; i < nscanlines; i++) {
         guint lineno = gwy_get_guint32_le(&p);
-        gdouble *d;
 
         lineno = MIN(lineno, nscanlines-1);
-        d = data + (nscanlines-1 - lineno)*width;
-        for (j = width; j; j--)
-            *(d++) = gwy_get_gfloat_le(&p);
+        gwy_convert_raw_data(p, width, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN,
+                             data + (nscanlines-1 - lineno)*width, 1.0, 0.0);
+        p += width*sizeof(gfloat);
     }
 
     g_free(buffer);
 
-    /* Older versions of the format had human-readable units there but we
-     * disregard these. */
+    /* Older versions of the format had human-readable units there but we disregard these. */
     units = g_array_index(naofile->streams, NAOStream, id).units;
     gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), units);
     gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
@@ -466,8 +463,7 @@ nao123_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
 
         gwy_debug(element_name);
         if ((name = find_attribute(attribute_names, attribute_values, "Id"))
-            && (unit = find_attribute(attribute_names, attribute_values,
-                                      "Unit"))) {
+            && (unit = find_attribute(attribute_names, attribute_values, "Unit"))) {
             NAOStream stream;
             gwy_debug("Adding stream %s [%s]", name, unit);
             gwy_clear(&stream, 1);
@@ -510,10 +506,8 @@ nao123_text(G_GNUC_UNUSED GMarkupParseContext *context,
         g_strdelimit(name, "/", ' ');
         g_strstrip(value);
         if (strlen(value)) {
-            if (!naofile->hash) {
-                naofile->hash = g_hash_table_new_full(g_str_hash, g_str_equal,
-                                                      g_free, g_free);
-            }
+            if (!naofile->hash)
+                naofile->hash = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
             g_hash_table_replace(naofile->hash, name, value);
         }
         else
@@ -536,8 +530,7 @@ nao123_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
     guint len, id, channelno = 0;
     gboolean status;
 
-    if (!nao_parse_xml_header(zipfile, naofile, "Scan/Measure.xml", &parser,
-                              error))
+    if (!nao_parse_xml_header(zipfile, naofile, "Scan/Measure.xml", &parser, error))
         goto fail;
 
     status = gwyzip_first_file(zipfile, error);
@@ -545,8 +538,7 @@ nao123_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
         goto fail;
 
     container = gwy_container_new();
-    while (status && gwyzip_get_current_filename(zipfile, &filename_curr,
-                                                 NULL)) {
+    while (status && gwyzip_get_current_filename(zipfile, &filename_curr, NULL)) {
         if (g_str_has_prefix(filename_curr, "Scan/Data/")) {
             const gchar *dataname = filename_curr + strlen("Scan/Data/");
             NAOStream *stream = NULL;
@@ -565,8 +557,7 @@ nao123_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
                     break;
             }
             if (stream->dir) {
-                GwyDataField *dfield = nao_read_field(zipfile, naofile, id,
-                                                      error);
+                GwyDataField *dfield = nao_read_field(zipfile, naofile, id, error);
                 if (!dfield)
                     goto fail;
                 create_channel(naofile, dfield, stream, channelno, container);
@@ -603,9 +594,8 @@ nao133_imaging_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
     g_string_append(naofile->path, element_name);
     path = naofile->path->str;
 
-    /* The channels are a mess in 1.33.  They are listed three times but units
-     * are only given in the ‘view’ part, not as units of the data files
-     * themselves.  Try to piece the information toegether... */
+    /* The channels are a mess in 1.33.  They are listed three times but units are only given in the ‘view’ part, not
+     * as units of the data files themselves.  Try to piece the information toegether... */
     if (gwy_strequal(path, "/Imaging/ChannelList/ChannelData")) {
         const gchar *s;
         gwy_debug(element_name);
@@ -617,15 +607,14 @@ nao133_imaging_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         }
     }
     else if (gwy_stramong(path,
-                          "/Imaging/ChannelList/ChannelData/Left",
-                          "/Imaging/ChannelList/ChannelData/Right",
+                          "/Imaging/ChannelList/ChannelData/Left", "/Imaging/ChannelList/ChannelData/Right",
                           NULL)) {
         const gchar *filename;
+
         gwy_debug(element_name);
-        if (naofile->current_name &&
-            (filename = find_attribute(attribute_names, attribute_values,
-                                       "NaoSubFile"))) {
+        if (naofile->current_name && (filename = find_attribute(attribute_names, attribute_values, "NaoSubFile"))) {
             NAOStream stream;
+
             gwy_clear(&stream, 1);
             stream.name = g_strdup(naofile->current_name);
             stream.filename = g_strdelimit(g_strdup(filename), "\\", '/');
@@ -637,14 +626,14 @@ nao133_imaging_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
     }
     else if (gwy_strequal(path, "/Imaging/ImagingView/ChannelView")) {
         const gchar *name, *unit;
+
         gwy_debug(element_name);
         if ((name = find_attribute(attribute_names, attribute_values, "Name"))
-            && (unit = find_attribute(attribute_names, attribute_values,
-                                      "Unit"))) {
+            && (unit = find_attribute(attribute_names, attribute_values, "Unit"))) {
             n = naofile->streams->len;
             for (i = 0; i < n; i++) {
-                NAOStream *stream = &g_array_index(naofile->streams, NAOStream,
-                                                   i);
+                NAOStream *stream = &g_array_index(naofile->streams, NAOStream, i);
+
                 /* Use ChannelView unit only of there is no in ChannelData. */
                 if (gwy_strequal(stream->name, name) && !stream->units) {
                     gwy_debug("Found Unit on ChannelView");
@@ -654,16 +643,17 @@ nao133_imaging_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
             }
         }
     }
-    /* Spectra.  The GridMode attribute is only informational.  Each spectrum
-     * always has X and Y, and that is what we need. */
+    /* Spectra.  The GridMode attribute is only informational.  Each spectrum always has X and Y, and that is what we
+     * need. */
     else if (gwy_strequal(path, "/Imaging/FlexGrid/Locus")) {
         const gchar *filename, *x, *y;
+
         gwy_debug(element_name);
-        if ((filename = find_attribute(attribute_names, attribute_values,
-                                       "NaoSubFile"))
+        if ((filename = find_attribute(attribute_names, attribute_values, "NaoSubFile"))
             && (x = find_attribute(attribute_names, attribute_values, "X"))
             && (y = find_attribute(attribute_names, attribute_values, "Y"))) {
             NAOSpectrum spectrum;
+
             gwy_clear(&spectrum, 1);
             spectrum.filename = g_strdelimit(g_strdup(filename), "\\", '/');
             spectrum.x = atoi(x);
@@ -703,9 +693,8 @@ nao133_imaging_text(G_GNUC_UNUSED GMarkupParseContext *context,
     NAOFile *naofile = (NAOFile*)user_data;
     gchar *path = naofile->path->str;
 
-    /* The parameters are a mess in 1.33.  There is a nested plain text header
-     * inside a XML tag.  Why on Earth someone did that whey they already had
-     * everything represented in XML... */
+    /* The parameters are a mess in 1.33.  There is a nested plain text header inside a XML tag.  Why on Earth someone
+     * did that whey they already had everything represented in XML... */
     if (gwy_strequal(path, "/Imaging/ImagingParameters")) {
         GwyTextHeaderParser parser;
         GHashTable *hash;
@@ -743,8 +732,7 @@ nao133_imaging_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
     GwyDataField *dfield;
     guint id;
 
-    if (!nao_parse_xml_header(zipfile, naofile, "Data/Imaging.xml", &parser,
-                              error))
+    if (!nao_parse_xml_header(zipfile, naofile, "Data/Imaging.xml", &parser, error))
         goto fail;
 
     container = gwy_container_new();
@@ -760,11 +748,10 @@ nao133_imaging_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
     }
 
     for (id = 0; id < naofile->spectra->len; id++) {
-        NAOSpectrum *spectrum = &g_array_index(naofile->spectra, NAOSpectrum,
-                                               id);
-        /* Do not fail when a spectrum curve data are not found.  According to
-         * Scientec-CSI a spectrum NaoSubFile only means the spectrum was
-         * planned.  But it might not be actually measured. */
+        NAOSpectrum *spectrum = &g_array_index(naofile->spectra, NAOSpectrum, id);
+
+        /* Do not fail when a spectrum curve data are not found.  According to Scientec-CSI a spectrum NaoSubFile only
+         * means the spectrum was planned.  But it might not be actually measured. */
         if (!gwyzip_locate_file(zipfile, spectrum->filename, 1, NULL))
             continue;
         spectrum->path = naofile->path;
@@ -784,8 +771,8 @@ fail:
     return NULL;
 }
 
-/* The pure spectra case.  We do not know how to recalculate pixel coordinates
- * to real coordinates so we import the spectra as graph curves. */
+/* The pure spectra case.  We do not know how to recalculate pixel coordinates to real coordinates so we import the
+ * spectra as graph curves. */
 static GwyContainer*
 nao133_spectro_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
 {
@@ -809,8 +796,7 @@ nao133_spectro_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
     xtitle = g_hash_table_lookup(spectrum.hash, "SweepSignal");
 
     for (j = 0; j < spectrum.specdata->len; j++) {
-        NAOSpectrumData *specdata = &g_array_index(spectrum.specdata,
-                                                   NAOSpectrumData, j);
+        NAOSpectrumData *specdata = &g_array_index(spectrum.specdata, NAOSpectrumData, j);
         GwyDataLine *dline = create_dataline_for_spectrum(specdata, &spectrum);
         GwyGraphModel *gmodel = gwy_graph_model_new();
         GwyGraphCurveModel *gcmodel = gwy_graph_curve_model_new();
@@ -852,8 +838,7 @@ nao_parse_xml_header(GwyZipFile zipfile, NAOFile *naofile,
     guchar *content = NULL, *s;
     gboolean ok = FALSE;
 
-    if (!gwyzip_locate_file(zipfile, filename, 1, error)
-        || !(content = gwyzip_get_file_content(zipfile, NULL, error)))
+    if (!gwyzip_locate_file(zipfile, filename, 1, error) || !(content = gwyzip_get_file_content(zipfile, NULL, error)))
         return FALSE;
 
     gwy_strkill(content, "\r");
@@ -912,8 +897,7 @@ nao133_spectro_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         const gchar *name, *unit;
         gwy_debug(element_name);
         if ((name = find_attribute(attribute_names, attribute_values, "Name"))
-            && (unit = find_attribute(attribute_names, attribute_values,
-                                      "Unit"))) {
+            && (unit = find_attribute(attribute_names, attribute_values, "Unit"))) {
             spectrum->current_name = g_strdup(name);
             spectrum->current_unit = g_strdup(unit);
         }
@@ -923,11 +907,10 @@ nao133_spectro_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         const gchar *name, *capacity, *sizeused;
         gwy_debug(element_name);
         if ((name = find_attribute(attribute_names, attribute_values, "Name"))
-            && (capacity = find_attribute(attribute_names, attribute_values,
-                                          "Capacity"))
-            && (sizeused = find_attribute(attribute_names, attribute_values,
-                                          "SizeUsed"))) {
+            && (capacity = find_attribute(attribute_names, attribute_values, "Capacity"))
+            && (sizeused = find_attribute(attribute_names, attribute_values, "SizeUsed"))) {
             NAOSpectrumData specdata;
+
             gwy_clear(&specdata, 1);
             specdata.capacity = atoi(capacity);
             specdata.sizeused = atoi(sizeused);
@@ -957,8 +940,7 @@ nao133_spectro_end_element(G_GNUC_UNUSED GMarkupParseContext *context,
         GWY_FREE(spectrum->current_name);
         GWY_FREE(spectrum->current_unit);
     }
-    else if (gwy_strequal(path,
-                          "/Spectroscopy/SpectroData/ChannelData/PassData")) {
+    else if (gwy_strequal(path, "/Spectroscopy/SpectroData/ChannelData/PassData")) {
         spectrum->current_specdata_id = G_MAXUINT;
     }
     g_string_set_size(spectrum->path, len-1 - n);
@@ -974,8 +956,8 @@ nao133_spectro_text(G_GNUC_UNUSED GMarkupParseContext *context,
     NAOSpectrum *spectrum = (NAOSpectrum*)user_data;
     gchar *path = spectrum->path->str;
 
-    /* The per-spectrum metadata.  Unfortunately, there is no place in GWY
-     * files for them; but we use them to construct the GwySpectra. */
+    /* The per-spectrum metadata.  Unfortunately, there is no place in GWY files for them; but we use them to
+     * construct the GwySpectra. */
     if (gwy_strequal(path, "/Spectroscopy/SpectroParameters")) {
         GwyTextHeaderParser parser;
         GHashTable *hash;
@@ -997,14 +979,11 @@ nao133_spectro_text(G_GNUC_UNUSED GMarkupParseContext *context,
             g_free(spectro_parameters);
     }
     /* The actual data, represented as a list of plain text data values. */
-    else if (gwy_strequal(path,
-                          "/Spectroscopy/SpectroData/ChannelData/PassData")
+    else if (gwy_strequal(path, "/Spectroscopy/SpectroData/ChannelData/PassData")
              && spectrum->current_specdata_id != G_MAXUINT) {
-        NAOSpectrumData *specdata = &g_array_index(spectrum->specdata,
-                                                   NAOSpectrumData,
-                                                   spectrum->current_specdata_id);
-        gchar *end;
+        NAOSpectrumData *specdata = &g_array_index(spectrum->specdata, NAOSpectrumData, spectrum->current_specdata_id);
         GArray *values = g_array_new(FALSE, FALSE, sizeof(gdouble));
+        gchar *end;
 
         while (TRUE) {
             gdouble v = g_ascii_strtod(text, &end);
@@ -1077,29 +1056,24 @@ create_channel(NAOFile *naofile, GwyDataField *dfield,
                NAOStream *stream, guint channelno, GwyContainer *container)
 {
     GwyContainer *meta;
-    GQuark key;
     gchar *title;
 
-    key = gwy_app_get_data_key_for_id(channelno);
-    gwy_container_set_object(container, key, dfield);
+    gwy_container_set_object(container, gwy_app_get_data_key_for_id(channelno), dfield);
     g_object_unref(dfield);
 
-    key = gwy_app_get_data_title_key_for_id(channelno);
     title = g_strconcat(stream->name, " ", stream->dir, NULL);
-    gwy_container_set_string(container, key, title);
+    gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(channelno), title);
 
     if (naofile->meta) {
-        key = gwy_app_get_data_meta_key_for_id(channelno);
         meta = gwy_container_duplicate(naofile->meta);
-        gwy_container_set_object(container, key, meta);
+        gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(channelno), meta);
         g_object_unref(meta);
     }
-    gwy_file_channel_import_log_add(container, channelno,
-                                    NULL, naofile->filename);
+    gwy_file_channel_import_log_add(container, channelno, NULL, naofile->filename);
 }
 
-/* We must organise spectra differently than in the file: split them by channel
- * and direction but group different positions to one GwySpectra object. */
+/* We must organise spectra differently than in the file: split them by channel and direction but group different
+ * positions to one GwySpectra object. */
 static void
 create_spectra(NAOFile *naofile, GwyContainer *container)
 {
@@ -1110,16 +1084,13 @@ create_spectra(NAOFile *naofile, GwyContainer *container)
     if (!naofile->spectra || !naofile->spectra->len)
         return;
 
-    if (!gwy_container_gis_object(container, gwy_app_get_data_key_for_id(0),
-                                  (GObject**)&dfield)) {
-        g_warning("Cannot convert spectra pixel coordinates to real "
-                  "because there is no image.");
+    if (!gwy_container_gis_object(container, gwy_app_get_data_key_for_id(0), (GObject**)&dfield)) {
+        g_warning("Cannot convert spectra pixel coordinates to real because there is no image.");
     }
 
     sps = g_ptr_array_new();
     for (i = 0; i < naofile->spectra->len; i++) {
-        NAOSpectrum *spectrum = &g_array_index(naofile->spectra, NAOSpectrum,
-                                               i);
+        NAOSpectrum *spectrum = &g_array_index(naofile->spectra, NAOSpectrum, i);
         const gchar *xtitle;
 
         /* Spectrum planned, but not measured. */
@@ -1128,26 +1099,21 @@ create_spectra(NAOFile *naofile, GwyContainer *container)
 
         if (dfield) {
             spectrum->x = gwy_data_field_jtor(dfield, spectrum->x + 0.5);
-            spectrum->y = gwy_data_field_itor(dfield,
-                                              dfield->yres - 0.5 - spectrum->y);
+            spectrum->y = gwy_data_field_itor(dfield, dfield->yres - 0.5 - spectrum->y);
         }
 
         xtitle = g_hash_table_lookup(spectrum->hash, "SweepSignal");
         for (j = 0; j < spectrum->specdata->len; j++) {
-            NAOSpectrumData *specdata = &g_array_index(spectrum->specdata,
-                                                       NAOSpectrumData, j);
-            GwyDataLine *dline = create_dataline_for_spectrum(specdata,
-                                                              spectrum);
-            add_dline_to_spectra(sps, dline,
-                                 xtitle, specdata->name, specdata->dir,
-                                 spectrum->x, spectrum->y);
+            NAOSpectrumData *specdata = &g_array_index(spectrum->specdata, NAOSpectrumData, j);
+            GwyDataLine *dline = create_dataline_for_spectrum(specdata, spectrum);
+            add_dline_to_spectra(sps, dline, xtitle, specdata->name, specdata->dir, spectrum->x, spectrum->y);
         }
     }
 
     for (id = 0; id < sps->len; id++) {
         GwySpectra *spectra = (GwySpectra*)g_ptr_array_index(sps, id);
-        GQuark key = gwy_app_get_spectra_key_for_id(id);
-        gwy_container_set_object(container, key, spectra);
+
+        gwy_container_set_object(container, gwy_app_get_spectra_key_for_id(id), spectra);
         g_object_unref(spectra);
     }
     g_ptr_array_free(sps, TRUE);
@@ -1165,8 +1131,7 @@ create_dataline_for_spectrum(NAOSpectrumData *specdata, NAOSpectrum *spectrum)
     }
 
     dline = gwy_data_line_new(specdata->nvalues, fabs(real), FALSE);
-    gwy_assign(gwy_data_line_get_data(dline), specdata->values,
-               specdata->nvalues);
+    gwy_assign(gwy_data_line_get_data(dline), specdata->values, specdata->nvalues);
 
     if (real > 0.0)
         gwy_data_line_set_offset(dline, spectrum->sweep_from);
@@ -1175,10 +1140,8 @@ create_dataline_for_spectrum(NAOSpectrumData *specdata, NAOSpectrum *spectrum)
         gwy_data_line_set_offset(dline, spectrum->sweep_to);
     }
 
-    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_x(dline),
-                                spectrum->sweep_unit);
-    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_y(dline),
-                                specdata->unit);
+    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_x(dline), spectrum->sweep_unit);
+    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_y(dline), specdata->unit);
 
     return dline;
 }
@@ -1202,12 +1165,9 @@ add_dline_to_spectra(GPtrArray *sps, GwyDataLine *dline,
         specxlabel = gwy_spectra_get_spectrum_x_label(spectra);
         if (gwy_strequal(gwy_spectra_get_title(spectra), fullname)
             && gwy_strequal(gwy_spectra_get_spectrum_y_label(spectra), name)
-            && ((!specxlabel && !xtitle)
-                || (specxlabel && xtitle && gwy_strequal(specxlabel, xtitle)))
-            && gwy_si_unit_equal(gwy_data_line_get_si_unit_x(dline),
-                                 gwy_data_line_get_si_unit_x(firstspec))
-            && gwy_si_unit_equal(gwy_data_line_get_si_unit_y(dline),
-                                 gwy_data_line_get_si_unit_y(firstspec)))
+            && ((!specxlabel && !xtitle) || (specxlabel && xtitle && gwy_strequal(specxlabel, xtitle)))
+            && gwy_si_unit_equal(gwy_data_line_get_si_unit_x(dline), gwy_data_line_get_si_unit_x(firstspec))
+            && gwy_si_unit_equal(gwy_data_line_get_si_unit_y(dline), gwy_data_line_get_si_unit_y(firstspec)))
             break;
         spectra = NULL;
     }
@@ -1245,8 +1205,7 @@ find_size_and_resolution(NAOFile *naofile, GError **error)
             err_INVALID(error, "Resolution");
             return FALSE;
         }
-        if (err_DIMENSION(error, naofile->xres)
-            || err_DIMENSION(error, naofile->yres))
+        if (err_DIMENSION(error, naofile->xres) || err_DIMENSION(error, naofile->yres))
             return FALSE;
     }
     else {
@@ -1344,6 +1303,7 @@ nao_file_free(NAOFile *naofile)
     if (naofile->streams) {
         for (i = 0; i < naofile->streams->len; i++) {
             NAOStream *stream = &g_array_index(naofile->streams, NAOStream, i);
+
             g_free(stream->name);
             g_free(stream->units);
             g_free(stream->filename);
@@ -1355,12 +1315,12 @@ nao_file_free(NAOFile *naofile)
         GArray *spectra = naofile->spectra;
         for (i = 0; i < spectra->len; i++) {
             NAOSpectrum *spectrum = &g_array_index(spectra, NAOSpectrum, i);
+
             if (spectrum->specdata) {
                 GArray *specdatas = spectrum->specdata;
                 for (j = 0; j < specdatas->len; j++) {
-                    NAOSpectrumData *specdata = &g_array_index(specdatas,
-                                                               NAOSpectrumData,
-                                                               j);
+                    NAOSpectrumData *specdata = &g_array_index(specdatas, NAOSpectrumData, j);
+
                     g_free(specdata->dir);
                     g_free(specdata->name);
                     g_free(specdata->unit);
@@ -1391,4 +1351,4 @@ nao_file_free(NAOFile *naofile)
     GWY_OBJECT_UNREF(naofile->meta);
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
