@@ -1,5 +1,5 @@
 /*
- *  $Id: nanoscan.c 23651 2021-05-10 10:05:19Z yeti-dn $
+ *  $Id: nanoscan.c 24529 2021-11-15 16:55:33Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -44,7 +44,7 @@
 #include <libgwyddion/gwymath.h>
 #include <libgwyddion/gwyutils.h>
 #include <libprocess/stats.h>
-#include <libprocess/correct.h>
+#include <libprocess/grains.h>
 #include <libgwymodule/gwymodule-file.h>
 #include <app/gwymoduleutils-file.h>
 #include <app/data-browser.h>
@@ -190,7 +190,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports NanoScan XML files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.8",
+    "0.9",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -411,8 +411,10 @@ add_channel(GwyContainer *container,
             d[i] = 1.0;
     }
 
-    if (mask)
-        gwy_data_field_laplace_solve(dfield, mask, -1, 1.0);
+    if (mask) {
+        gwy_data_field_grains_invert(mask);
+        gwy_app_channel_remove_bad_data(dfield, mask);
+    }
 
     quark = gwy_app_get_data_key_for_id(id);
     gwy_container_set_object(container, quark, dfield);
