@@ -1,5 +1,5 @@
 /*
- *  $Id: nanoscope.c 24615 2022-02-21 16:06:34Z yeti-dn $
+ *  $Id: nanoscope.c 25134 2022-11-02 14:00:39Z yeti-dn $
  *  Copyright (C) 2004-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1091,8 +1091,13 @@ hash_to_lawn(GHashTable *hash,
         if (!zreshold && !zres2 && size == 2*xres*yres*zres*bpp)
             zres2 = zres;
         else {
-            err_SIZE_MISMATCH(error, xres*yres*zrestotal*bpp, size, TRUE);
-            goto fail;
+            if (xres*yres*zrestotal*bpp < size) {
+                g_warning("Data size mismatch %lu (computed) vs. %lu (actual).", xres*yres*zrestotal*bpp, size);
+            }
+            else {
+                err_SIZE_MISMATCH(error, xres*yres*zrestotal*bpp, size, TRUE);
+                goto fail;
+            }
         }
     }
 
@@ -1141,7 +1146,7 @@ hash_to_lawn(GHashTable *hash,
     if (nsegments == 1)
         nsegments = 0;
     gwy_debug("nsegments %d", nsegments);
-    lawn = gwy_lawn_new(yres, xres, xreal, yreal, 1, nsegments);
+    lawn = gwy_lawn_new(xres, yres, xreal, yreal, 1, nsegments);
     gwy_si_unit_assign(gwy_lawn_get_si_unit_xy(lawn), unitxy);
     gwy_si_unit_assign(gwy_lawn_get_si_unit_curve(lawn, 0), unitw);
     for (i = 0; i < nsegments; i++)
@@ -1753,13 +1758,15 @@ hash_to_curve(GHashTable *hash,
     GwyGraphModel *gmodel = NULL;
     GwyGraphCurveModel *gcmodel;
     GwySIUnit *unitz = NULL, *unitx = NULL;
-    gsize xres, bpp, qbpp, offset, size;
+    guint i, res, xres, xres2, xreshold, xrestotal, nsegments = 0;
+    gsize offset, size, bpp, qbpp;
     gdouble xreal, xoff, q = 1.0;
     gdouble *data;
-    gboolean size_ok, use_global, convert_to_force = FALSE;
+    gboolean size_ok = FALSE, use_global = FALSE, convert_to_force = FALSE;
     /* Call the curves Trace and Retrace when we do now know (often not correct).  Replace the titles with something
      * meaningful for specific types. */
-    const gchar *title0 = "Trace", *title1 = "Retrace";
+    gint segments[6];
+    const gchar *segment_labels[3] = { "Trace", "Retrace", "" };
 
     g_return_val_if_fail(base_type == NANOSCOPE_FILE_TYPE_FORCE_BIN, NULL);
 
@@ -1772,48 +1779,79 @@ hash_to_curve(GHashTable *hash,
     if (!(unitx = get_spec_abscissa_scale(hash, forcelist, scannerlist, scanlist, &xreal, &xoff, &spectype, error)))
         return NULL;
 
-    val = g_hash_table_lookup(hash, "Samps/line");
-    xres = (gsize)val->hard_value;
+    xres = get_samples_per_curve(hash, forcelist, &xreshold, &xres2);
+    xrestotal = xres + xres2 + xreshold;
+    gwy_debug("curve samples %u + %u + %u = %u", xres, xreshold, xres2, xrestotal);
 
     get_bpp_and_qbpp(hash, file_type, &bpp, &qbpp);
 
     if (!get_offset_and_size(hash, bufsize, &offset, &size, error))
         goto fail;
 
-    size_ok = FALSE;
-    use_global = FALSE;
-
-    /* Try channel size and local size */
-    if (!size_ok && size == 2*bpp*xres)
+    /* Try the full segment information from get_samples_per_curve(). If it fails try more and more desperate things,
+     * like channel size and local size */
+    if (!size_ok && size == bpp*xrestotal) {
+        gwy_debug("using xres + hxreshold + xres2 segmentation");
         size_ok = TRUE;
-
-    if (!size_ok && size == 2*bpp*gxres) {
-        size_ok = TRUE;
-        use_global = TRUE;
+        nsegments = 0;
+        /* NB: the data order is a bit confusing. */
+        if (xres) {
+            segment_labels[nsegments] = "Approach";
+            segments[2*nsegments + 0] = 0;
+            segments[2*nsegments + 1] = xres;
+            nsegments++;
+        }
+        if (xreshold) {
+            segment_labels[nsegments] = "Contact";
+            segments[2*nsegments + 0] = xres + xres2;
+            segments[2*nsegments + 1] = xrestotal;
+            nsegments++;
+        }
+        if (xres2) {
+            segment_labels[nsegments] = "Retract";
+            segments[2*nsegments + 0] = xres;
+            segments[2*nsegments + 1] = xres + xres2;
+            nsegments++;
+        }
     }
+    else {
+        xreshold = 0;
+        if (!size_ok && size == 2*bpp*xres)
+            size_ok = TRUE;
 
-    gwy_debug("size=%lu, xres=%lu, gxres=%lu, bpp=%lu", (gulong)size, (gulong)xres, (gulong)gxres, (gulong)bpp);
+        if (!size_ok && size == 2*bpp*gxres) {
+            size_ok = TRUE;
+            use_global = TRUE;
+        }
 
-    /* If they don't match exactly, try whether they at least fit inside */
-    if (!size_ok && size > bpp*MAX(2*xres, 2*gxres)) {
-        size_ok = TRUE;
-        use_global = (xres < gxres);
-    }
+        gwy_debug("size=%lu, xres=%lu, gxres=%lu, bpp=%lu", (gulong)size, (gulong)xres, (gulong)gxres, (gulong)bpp);
 
-    if (!size_ok && size > bpp*MIN(2*xres, 2*gxres)) {
-        size_ok = TRUE;
-        use_global = (xres > gxres);
+        /* If they don't match exactly, try whether they at least fit inside */
+        if (!size_ok && size > bpp*MAX(2*xres, 2*gxres)) {
+            size_ok = TRUE;
+            use_global = (xres < gxres);
+        }
+
+        if (!size_ok && size > bpp*MIN(2*xres, 2*gxres)) {
+            size_ok = TRUE;
+            use_global = (xres > gxres);
+        }
+
+        if (use_global && gxres)
+            xres = gxres;
+
+        nsegments = 2;
+        segments[0] = 0;
+        segments[1] = segments[2] = xres;
+        segments[3] = xrestotal = 2*xres;
     }
 
     if (!size_ok) {
-        err_SIZE_MISMATCH(error, bpp*xres, size, TRUE);
+        err_SIZE_MISMATCH(error, bpp*xrestotal, size, TRUE);
         goto fail;
     }
 
-    if (use_global && gxres)
-        xres = gxres;
-
-    if (err_DIMENSION(error, xres))
+    if (err_DIMENSION(error, xrestotal))
         goto fail;
 
     if (err_SIZE_MISMATCH(error, offset + size, bufsize, FALSE))
@@ -1827,8 +1865,7 @@ hash_to_curve(GHashTable *hash,
 
     val = g_hash_table_lookup(hash, "@4:Image Data");
     gwy_debug("channel=%s", val->hard_value_str);
-    if (spectype == NANOSCOPE_SPECTRA_FZ
-        && gwy_strequal(val->hard_value_str, "Deflection Error"))
+    if (spectype == NANOSCOPE_SPECTRA_FZ && gwy_strequal(val->hard_value_str, "Deflection Error"))
         convert_to_force = TRUE;
 
     if (!(unitz = get_spec_ordinate_scale(hash, scanlist, version, &q, &convert_to_force, qbpp, error)))
@@ -1839,72 +1876,58 @@ hash_to_curve(GHashTable *hash,
     if (spectype == NANOSCOPE_SPECTRA_IV) {
         g_object_set(gmodel,
                      "title", "I-V spectrum",
-                     "axis-label-bottom", "Voltage",
+                     "axis-label-bottom", gwy_si_unit_equal_string(unitx, NULL) ? "Sample number" : "Voltage",
                      "axis-label-left", val->hard_value_str,
                      NULL);
     }
-    else if (convert_to_force) {
-        title0 = "Extend";
-        title1 = "Retract";
+    else if (convert_to_force || spectype == NANOSCOPE_SPECTRA_FZ) {
         g_object_set(gmodel,
                      "title", "F-Z spectrum",
-                     "axis-label-bottom", "Distance",
+                     "axis-label-bottom", gwy_si_unit_equal_string(unitx, NULL) ? "Sample number" : "Distance",
                      "axis-label-left", "Force",
                      NULL);
-    }
-    else if (spectype == NANOSCOPE_SPECTRA_FZ) {
-        title0 = "Extend";
-        title1 = "Retract";
-        g_object_set(gmodel,
-                     "title", "F-Z spectrum",
-                     "axis-label-bottom", "Distance",
-                     "axis-label-left", val->hard_value_str,
-                     NULL);
+        if (convert_to_force)
+            g_object_set(gmodel, "axis-label-left", "Force", NULL);
+        else
+            g_object_set(gmodel, "axis-label-left", val->hard_value_str, NULL);
+
+        for (i = 0; i < nsegments; i++) {
+            if (gwy_strequal(segment_labels[i], "Trace"))
+                segment_labels[i] = "Extend";
+            else if (gwy_strequal(segment_labels[i], "Retrace"))
+                segment_labels[i] = "Retract";
+        }
     }
 
-    dline = gwy_data_line_new(xres, xreal, FALSE);
+    dline = gwy_data_line_new(1, xreal, FALSE);
     gwy_data_line_set_offset(dline, xoff);
     gwy_si_unit_assign(gwy_data_line_get_si_unit_x(dline), unitx);
     gwy_si_unit_assign(gwy_data_line_get_si_unit_y(dline), unitz);
-
-    data = gwy_data_line_get_data(dline);
-    gwy_debug("curve 1 at offset %lu", (gulong)offset);
-    if (!read_binary_data(xres, data, buffer + offset, bpp, qbpp, error)) {
-        GWY_OBJECT_UNREF(gmodel);
-        goto fail;
+    for (i = 0; i < nsegments; i++) {
+        res = segments[2*i+1] - segments[2*i];
+        gwy_data_line_resample(dline, res, GWY_INTERPOLATION_NONE);
+        data = gwy_data_line_get_data(dline);
+        gwy_debug("curve %d at offset %lu", i+1, (gulong)(offset + segments[2*i]*bpp));
+        if (!read_binary_data(res, data, buffer + offset + segments[2*i]*bpp, bpp, qbpp, error)) {
+            GWY_OBJECT_UNREF(gmodel);
+            goto fail;
+        }
+        gwy_debug("multiplying values by %g", q);
+        gwy_data_line_multiply(dline, q);
+        if (spectype == NANOSCOPE_SPECTRA_FZ)
+            gwy_data_line_invert(dline, TRUE, FALSE);
+        gcmodel = gwy_graph_curve_model_new();
+        gwy_graph_curve_model_set_data_from_dataline(gcmodel, dline, 0, 0);
+        g_object_set(gcmodel,
+                     "mode", GWY_GRAPH_CURVE_LINE,
+                     "color", gwy_graph_get_preset_color(i),
+                     "description", segment_labels[i],
+                     NULL);
+        gwy_graph_model_add_curve(gmodel, gcmodel);
+        g_object_unref(gcmodel);
+        if (!i)
+            gwy_graph_model_set_units_from_data_line(gmodel, dline);
     }
-    gwy_debug("multiplying values by %g", q);
-    gwy_data_line_multiply(dline, q);
-    if (spectype == NANOSCOPE_SPECTRA_FZ)
-        gwy_data_line_invert(dline, TRUE, FALSE);
-    gcmodel = gwy_graph_curve_model_new();
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, dline, 0, 0);
-    g_object_set(gcmodel,
-                 "mode", GWY_GRAPH_CURVE_LINE,
-                 "color", gwy_graph_get_preset_color(0),
-                 "description", title0,
-                 NULL);
-    gwy_graph_model_add_curve(gmodel, gcmodel);
-    g_object_unref(gcmodel);
-
-    gwy_debug("curve 2 at offset %lu", (gulong)(offset + bpp*xres));
-    if (!read_binary_data(xres, data, buffer + offset + bpp*xres, bpp, qbpp, error)) {
-        GWY_OBJECT_UNREF(gmodel);
-        goto fail;
-    }
-    gwy_data_line_multiply(dline, q);
-    if (spectype == NANOSCOPE_SPECTRA_FZ)
-        gwy_data_line_invert(dline, TRUE, FALSE);
-    gcmodel = gwy_graph_curve_model_new();
-    g_object_set(gcmodel,
-                 "mode", GWY_GRAPH_CURVE_LINE,
-                 "color", gwy_graph_get_preset_color(1),
-                 "description", title1,
-                 NULL);
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, dline, 0, 0);
-    gwy_graph_model_add_curve(gmodel, gcmodel);
-    g_object_unref(gcmodel);
-    gwy_graph_model_set_units_from_data_line(gmodel, dline);
 
 fail:
     GWY_OBJECT_UNREF(dline);
@@ -2014,6 +2037,19 @@ get_spec_abscissa_scale(GHashTable *hash,
     gchar *key, *end;
     gint q;
 
+    val = g_hash_table_lookup(hash, "X Data Type");
+    gwy_debug("have x data type: %d", !!val);
+    if (val && gwy_strequal(val->hard_value_str, "Time")) {
+        guint xres, xres2, xreshold;
+
+        xres = get_samples_per_curve(hash, forcelist, &xreshold, &xres2);
+        *xoff = 0.0;
+        *xreal = xres + xres2 + xreshold;
+        *spectype = NANOSCOPE_SPECTRA_FZ;
+
+        return gwy_si_unit_new(NULL);
+    }
+
     if (!(val = g_hash_table_lookup(forcelist, "@4:Ramp channel"))) {
         err_MISSING_FIELD(error, "Ramp channel");
         return NULL;
