@@ -1,22 +1,18 @@
 /*
- *  $Id: nrrdfile.c 22562 2019-10-10 14:45:41Z yeti-dn $
+ *  $Id: nrrdfile.c 25069 2022-10-05 14:03:46Z yeti-dn $
  *  Copyright (C) 2011-2019 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -179,7 +175,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports and exports nearly raw raster data (NRRD) files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.8",
+    "0.9",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -211,8 +207,7 @@ nrrdfile_detect(const GwyFileDetectInfo *fileinfo,
         && memcmp(fileinfo->head, MAGIC, MAGIC_SIZE) == 0
         && g_ascii_isdigit(fileinfo->head[MAGIC_SIZE])
         && (fileinfo->head[MAGIC_SIZE+1] == '\n'
-            || (fileinfo->head[MAGIC_SIZE+1] == '\r'
-                && fileinfo->head[MAGIC_SIZE+2] == '\n')))
+            || (fileinfo->head[MAGIC_SIZE+1] == '\r' && fileinfo->head[MAGIC_SIZE+2] == '\n')))
         return 100;
 
     return 0;
@@ -239,9 +234,8 @@ nrrdfile_load(const gchar *filename,
     GError *err = NULL;
     guint sizes[3] = { 1, 1, 1 };
     G_GNUC_UNUSED guint version;
-    guint dimension, xres = 0, yres = 0, zres = 0, header_size, i,
-          stride = 0, rowstride = 0, fieldstride = 0, chanaxis, xaxis, yaxis,
-          nchannels = 0;
+    guint dimension, xres = 0, yres = 0, zres = 0, nchannels = 0, stride = 0, rowstride = 0, fieldstride = 0,
+          chanaxis, xaxis, yaxis, header_size, i;
     gchar *value, *vkeyvalue, *vfield, *p, *line, *datafile = NULL;
     gchar **kinds = NULL;
     gboolean unix_eol, detached_header = FALSE;
@@ -251,7 +245,6 @@ nrrdfile_load(const gchar *filename,
     NRRDEncoding encoding;
     GwyRawDataType rawdatatype;
     GwyByteOrder byteorder;
-    GQuark quark;
 
     if (!g_file_get_contents(filename, (gchar**)&buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -265,18 +258,14 @@ nrrdfile_load(const gchar *filename,
     }
     if (memcmp(buffer, MAGIC, MAGIC_SIZE) != 0
         || !g_ascii_isdigit(buffer[MAGIC_SIZE])
-        || (buffer[MAGIC_SIZE+1] != '\n'
-            && (buffer[MAGIC_SIZE+1] != '\r'
-                || buffer[MAGIC_SIZE+2] != '\n'))) {
+        || (buffer[MAGIC_SIZE+1] != '\n' && (buffer[MAGIC_SIZE+1] != '\r' || buffer[MAGIC_SIZE+2] != '\n'))) {
         err_FILE_TYPE(error, "NRRD");
         goto fail;
     }
 
     version = buffer[MAGIC_SIZE] - '0';
     unix_eol = (buffer[MAGIC_SIZE+1] == '\n');
-    header_end = gwy_memmem(buffer, size,
-                            unix_eol ? "\n\n" : "\r\n\r\n",
-                            unix_eol ? 2 : 4);
+    header_end = gwy_memmem(buffer, size, unix_eol ? "\n\n" : "\r\n\r\n", unix_eol ? 2 : 4);
     if (header_end) {
         header_size = header_end - buffer + (unix_eol ? 2 : 4);
         first_byte = buffer[header_size];
@@ -297,9 +286,8 @@ nrrdfile_load(const gchar *filename,
         if (line[0] == '#')
             continue;
 
-        /* Fields and key-values are almost the same for us.  But do not put
-         * them to one hash table as we do not want key-values to override
-         * fields. */
+        /* Fields and key-values are almost the same for us.  But do not put them to one hash table as we do not want
+         * key-values to override fields. */
         vfield = strstr(line, ": ");
         vkeyvalue = strstr(line, ":=");
         if (vfield) {
@@ -311,8 +299,7 @@ nrrdfile_load(const gchar *filename,
             value = vkeyvalue ? vkeyvalue : NULL;
         }
         if (!hash) {
-            g_warning("Neither field nor key-value separator found on line: %s",
-                      line);
+            g_warning("Neither field nor key-value separator found on line: %s", line);
             continue;
         }
 
@@ -336,25 +323,21 @@ nrrdfile_load(const gchar *filename,
         goto fail;
     }
 
-    if (!require_keys(fields, error,
-                      "dimension", "encoding", "sizes", "type", NULL))
+    if (!require_keys(fields, error, "dimension", "encoding", "sizes", "type", NULL))
         goto fail;
 
-    if ((data_type = parse_data_type(g_hash_table_lookup(fields, "type")))
-        == (NRRDDataType)-1) {
+    if ((data_type = parse_data_type(g_hash_table_lookup(fields, "type"))) == (NRRDDataType)-1) {
         err_UNSUPPORTED(error, "type");
         goto fail;
     }
-    if ((encoding = parse_encoding(g_hash_table_lookup(fields, "encoding")))
-        == (NRRDEncoding)-1) {
+    if ((encoding = parse_encoding(g_hash_table_lookup(fields, "encoding"))) == (NRRDEncoding)-1) {
         err_UNSUPPORTED(error, "encoding");
         goto fail;
     }
     gwy_debug("data_type: %u, encoding: %u", data_type, encoding);
 
     /* TODO: Read 1D data as a graph. */
-    if (sscanf(g_hash_table_lookup(fields, "dimension"), "%u",
-               &dimension) != 1) {
+    if (sscanf(g_hash_table_lookup(fields, "dimension"), "%u", &dimension) != 1) {
         err_INVALID(error, "dimension");
         goto fail;
     }
@@ -363,13 +346,13 @@ nrrdfile_load(const gchar *filename,
                     _("Only two- and three-dimensional data are supported."));
         goto fail;
     }
-    if (!parse_uint_vector(g_hash_table_lookup(fields, "sizes"), dimension,
-                           &sizes[0], &sizes[1], &sizes[2])) {
+    if (!parse_uint_vector(g_hash_table_lookup(fields, "sizes"), dimension, &sizes[0], &sizes[1], &sizes[2])) {
         err_INVALID(error, "sizes");
         goto fail;
     }
     gwy_debug("sizes: %u, %u, %u", sizes[0], sizes[1], sizes[2]);
-    if (err_DIMENSION(error, sizes[0]) || err_DIMENSION(error, sizes[1])
+    if (err_DIMENSION(error, sizes[0])
+        || err_DIMENSION(error, sizes[1])
         || (dimension == 3 && err_DIMENSION(error, sizes[2])))
         goto fail;
 
@@ -380,13 +363,11 @@ nrrdfile_load(const gchar *filename,
 
     if (lineskip && nrrd_encoding_is_compressed(encoding)) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Non-zero lineskip is supported only for uncompressed "
-                      "files."));
+                    _("Non-zero lineskip is supported only for uncompressed files."));
         goto fail;
     }
 
-    if (!find_gwy_data_type(data_type, encoding,
-                            g_hash_table_lookup(fields, "endian"),
+    if (!find_gwy_data_type(data_type, encoding, g_hash_table_lookup(fields, "endian"),
                             &rawdatatype, &byteorder, error))
         goto fail;
 
@@ -405,8 +386,7 @@ nrrdfile_load(const gchar *filename,
         }
 
         data_buffer = load_detached_file(datafilepath, &data_size,
-                                         encoding == NRRD_ENCODING_GZIP,
-                                         encoding == NRRD_ENCODING_BZIP2,
+                                         encoding == NRRD_ENCODING_GZIP, encoding == NRRD_ENCODING_BZIP2,
                                          error);
         g_free(datafilepath);
         if (!data_buffer)
@@ -418,8 +398,7 @@ nrrdfile_load(const gchar *filename,
     }
     else {
         if (nrrd_encoding_is_compressed(encoding)) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                         _("Compression is supported only for detached files."));
             goto fail;
         }
@@ -464,66 +443,43 @@ nrrdfile_load(const gchar *filename,
         xres = sizes[xaxis];
         yres = sizes[yaxis];
         gwy_debug("xres: %u, yres: %u, nchannels: %u", xres, yres, nchannels);
-        gwy_debug("stride: %u, rowstride: %u, fieldstride: %u",
-                  stride, rowstride, fieldstride);
+        gwy_debug("stride: %u, rowstride: %u, fieldstride: %u", stride, rowstride, fieldstride);
     }
 
-    if (!(data_start = get_raw_data_pointer(data_buffer, &data_size,
-                                            sizes[0]*sizes[1]*sizes[2],
-                                            rawdatatype, &byteorder, encoding,
-                                            lineskip, byteskip,
-                                            &buffers_to_free,
-                                            error)))
+    if (!(data_start = get_raw_data_pointer(data_buffer, &data_size, sizes[0]*sizes[1]*sizes[2],
+                                            rawdatatype, &byteorder, encoding, lineskip, byteskip,
+                                            &buffers_to_free, error)))
         goto fail;
 
     container = gwy_container_new();
     meta = nrrd_make_meta(keyvalue);
 
     if (chanaxis == G_MAXUINT) {
-        GwyBrick *brick = read_raw_brick(xres, yres, zres,
-                                         rawdatatype, byteorder,
-                                         fields, data_start);
-        quark = gwy_app_get_brick_key_for_id(0);
-        gwy_container_set_object(container, quark, brick);
+        GwyBrick *brick = read_raw_brick(xres, yres, zres, rawdatatype, byteorder, fields, data_start);
+
+        gwy_container_set_object(container, gwy_app_get_brick_key_for_id(0), brick);
         g_object_unref(brick);
 
-        if ((value = g_hash_table_lookup(fields, "content"))) {
-            gwy_container_set_const_string_by_name(container, "/brick/0/title",
-                                                   value);
-        }
-
+        if ((value = g_hash_table_lookup(fields, "content")))
+            gwy_container_set_const_string(container, gwy_app_get_brick_title_key_for_id(0), value);
         if (meta)
-            gwy_container_set_object_by_name(container, "/brick/0/meta", meta);
+            gwy_container_set_object(container, gwy_app_get_brick_meta_key_for_id(0), meta);
     }
     else {
         for (i = 0; i < nchannels; i++) {
-            const guchar *chandata;
-            chandata = (data_start
-                        + i*fieldstride*gwy_raw_data_size(rawdatatype));
-            dfield = read_raw_data_field(xres, yres,
-                                         stride, rowstride,
-                                         rawdatatype, byteorder,
-                                         fields, chandata);
-            quark = gwy_app_get_data_key_for_id(i);
-            gwy_container_set_object(container, quark, dfield);
+            const guchar *chandata = data_start + i*fieldstride*gwy_raw_data_size(rawdatatype);
+
+            dfield = read_raw_data_field(xres, yres, stride, rowstride, rawdatatype, byteorder, fields, chandata);
+            gwy_container_set_object(container, gwy_app_get_data_key_for_id(i), dfield);
             g_object_unref(dfield);
 
-            if ((value = g_hash_table_lookup(fields, "content"))) {
-                gchar *key = g_strconcat(g_quark_to_string(quark), "/title",
-                                         NULL);
-                gwy_container_set_string_by_name(container, key,
-                                                 g_strdup(value));
-                g_free(key);
-            }
+            if ((value = g_hash_table_lookup(fields, "content")))
+                gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(i), g_strdup(value));
             if (meta) {
-                gchar *key = g_strdup(g_quark_to_string(quark));
                 GwyContainer *chanmeta = gwy_container_duplicate(meta);
 
-                g_assert(g_str_has_suffix(key, "/data"));
-                strcpy(key + strlen(key)-4, "meta");
-                gwy_container_set_object_by_name(container, key, chanmeta);
+                gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(i), chanmeta);
                 g_object_unref(chanmeta);
-                g_free(key);
             }
             gwy_file_channel_import_log_add(container, i, NULL, filename);
         }
@@ -548,8 +504,8 @@ normalise_field_name(gchar *name)
 {
     guint i, j;
 
-    /* Get rid of non-alnum characters, e.g. "sample units" → "sampleunits"
-     * and convert alphabetic characters to lowercase. */
+    /* Get rid of non-alnum characters, e.g. "sample units" → "sampleunits" and convert alphabetic characters to
+     * lowercase. */
     for (i = j = 0; name[i]; i++) {
         if (g_ascii_isalnum(name[i])) {
             name[j++] = g_ascii_tolower(name[i]);
@@ -736,8 +692,7 @@ load_detached_file(const gchar *datafile,
     gwy_debug("Loading detached <%s>", datafile);
     if (datafile) {
         if (strchr(datafile, ' ') || gwy_strequal(datafile, "LIST")) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                         _("Split detached data files are not supported."));
             return NULL;
         }
@@ -760,8 +715,7 @@ load_detached_file(const gchar *datafile,
             *size = MAX(2*prevsize, 4096);
             buffer = g_realloc(buffer, *size);
             toread = *size - prevsize;
-        } while ((readbytes = gzread(cfile, buffer + prevsize, toread))
-                 == toread);
+        } while ((readbytes = gzread(cfile, buffer + prevsize, toread)) == toread);
 
         if (readbytes < 0) {
             gint errcode;
@@ -769,10 +723,10 @@ load_detached_file(const gchar *datafile,
 
             if (errcode == Z_ERRNO)
                 err_READ(error);
-            else
-                g_set_error(error, GWY_MODULE_FILE_ERROR,
-                            GWY_MODULE_FILE_ERROR_DATA,
+            else {
+                g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                             _("Cannot read from file: %s."), message);
+            }
             gzclose(cfile);
             g_free(buffer);
             return NULL;
@@ -782,10 +736,8 @@ load_detached_file(const gchar *datafile,
         *size = prevsize + readbytes;
         gwy_debug("decompressed size: %lu", (gulong)*size);
 #else
-        g_set_error(error, GWY_MODULE_FILE_ERROR,
-                    GWY_MODULE_FILE_ERROR_SPECIFIC,
-                    _("Cannot decompress gzip-encoded data.  "
-                      "Zlib support was not built in."));
+        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_SPECIFIC,
+                    _("Cannot decompress gzip-encoded data.  Zlib support was not built in."));
 #endif
     }
     else if (bz2compressed) {
@@ -805,8 +757,7 @@ load_detached_file(const gchar *datafile,
             *size = MAX(2*prevsize, 4096);
             buffer = g_realloc(buffer, *size);
             toread = *size - prevsize;
-        } while ((readbytes = BZ2_bzread(cfile, buffer + prevsize, toread))
-                 == toread);
+        } while ((readbytes = BZ2_bzread(cfile, buffer + prevsize, toread)) == toread);
 
         if (readbytes < 0) {
             gint errcode;
@@ -814,10 +765,10 @@ load_detached_file(const gchar *datafile,
 
             if (errcode == BZ_IO_ERROR)  /* XXX: Is this right? */
                 err_READ(error);
-            else
-                g_set_error(error, GWY_MODULE_FILE_ERROR,
-                            GWY_MODULE_FILE_ERROR_DATA,
+            else {
+                g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                             _("Cannot read from file: %s."), message);
+            }
             BZ2_bzclose(cfile);
             g_free(buffer);
             return NULL;
@@ -827,10 +778,8 @@ load_detached_file(const gchar *datafile,
         *size = prevsize + readbytes;
         gwy_debug("decompressed size: %lu", (gulong)*size);
 #else
-        g_set_error(error, GWY_MODULE_FILE_ERROR,
-                    GWY_MODULE_FILE_ERROR_SPECIFIC,
-                    _("Cannot decompress bzip2-encoded data.  "
-                      "Bzip2 support was not built in."));
+        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_SPECIFIC,
+                    _("Cannot decompress bzip2-encoded data.  Bzip2 support was not built in."));
 #endif
     }
     else {
@@ -961,16 +910,14 @@ decode_hex(const guchar *encoded,
 
 fail:
     g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                _("Hex data contain fewer values (%u) "
-                  "than corresponds to the sizes (%u)."),
+                _("Hex data contain fewer values (%u) than corresponds to the sizes (%u)."),
                 (guint)(nitems-i), (guint)nitems);
 
     g_free(decoded);
     return NULL;
 }
 
-/* Handle also ASCII encoding as another representation of raw encoding to
- * unify all encodings. */
+/* Handle also ASCII encoding as another representation of raw encoding to unify all encodings. */
 static guchar*
 decode_text(const gchar *encoded,
             gsize nitems,
@@ -1082,14 +1029,16 @@ decode_text(const gchar *encoded,
     return decoded;
 
 fail:
-    if (!*p)
+    if (!*p) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Text data contain fewer values (%u) "
-                      "than corresponds to the sizes (%u)."),
+                    _("Text data contain fewer values (%u) than corresponds to the sizes (%u)."),
                     (guint)(nitems-i), (guint)nitems);
-    else
+    }
+    else {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Garbage after data sample #%u."), (guint)(nitems-i));
+                    _("Garbage after data sample #%u."),
+                    (guint)(nitems-i));
+    }
 
     g_free(decoded);
     return NULL;
@@ -1101,10 +1050,9 @@ fail:
  * 2. decompression (decoding)
  * 3. byte skipping
  *
- * However, we treat text and hex as decoding steps.  On the other hand, the
- * format uses actual gzip-compressed data, not just zlib-deflated and to use
- * gzopen() we need the compressed data as a detached file.  Fortunately, this
- * seems to be the common case.
+ * However, we treat text and hex as decoding steps.  On the other hand, the format uses actual gzip-compressed data,
+ * not just zlib-deflated and to use gzopen() we need the compressed data as a detached file.  Fortunately, this seems
+ * to be the common case.
  */
 static gconstpointer
 get_raw_data_pointer(gconstpointer base,
@@ -1135,8 +1083,7 @@ get_raw_data_pointer(gconstpointer base,
         return NULL;
     }
 
-    if (byteskip == -1 && (encoding == NRRD_ENCODING_TEXT
-                           || encoding == NRRD_ENCODING_HEX)) {
+    if (byteskip == -1 && (encoding == NRRD_ENCODING_TEXT || encoding == NRRD_ENCODING_HEX)) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                     _("Field byteskip cannot be -1 for text encodings."));
         return NULL;
@@ -1147,10 +1094,8 @@ get_raw_data_pointer(gconstpointer base,
         const guchar *p = memchr(base, '\n', *size);
 
         if (!p) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
-                        _("Field lineskip specifies more lines than there "
-                          "are in the file."));
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
+                        _("Field lineskip specifies more lines than there are in the file."));
             return NULL;
         }
         p++;
@@ -1190,17 +1135,14 @@ get_raw_data_pointer(gconstpointer base,
     /* Decoding for text formats. */
     g_assert(byteskip >= 0);   /* This must hold if we got here. */
     if (byteskip > *size) {
-        g_set_error(error, GWY_MODULE_FILE_ERROR,
-                    GWY_MODULE_FILE_ERROR_DATA,
-                    _("Field byteskip specifies more bytes than there "
-                      "are in the file."));
+        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
+                    _("Field byteskip specifies more bytes than there are in the file."));
         return NULL;
     }
     base = (gconstpointer)((const gchar*)base + byteskip);
     *size -= byteskip;
 
-    /* Buffers are always nul-terminated, passing the encoed size is not
-     * necessary. */
+    /* Buffers are always nul-terminated, passing the encoded size is not necessary. */
     if (encoding == NRRD_ENCODING_HEX)
         base = decode_hex(base, nitems, rawdatatype, size, error);
     else if (encoding == NRRD_ENCODING_TEXT) {
@@ -1273,9 +1215,13 @@ read_raw_data_field(guint xres, guint yres,
     }
 
     if ((value = g_hash_table_lookup(fields, "sampleunits"))) {
-        siunitz = gwy_si_unit_new_parse(value, &power10);
-        q *= pow10(power10);
-        z0 *= pow10(power10);
+        gchar *unitz;
+        if (parse_string_vector(value, 1, &unitz)) {
+            siunitz = gwy_si_unit_new_parse(unitz, &power10);
+            q *= pow10(power10);
+            z0 *= pow10(power10);
+            g_free(unitz);
+        }
     }
 
     if ((value = g_hash_table_lookup(fields, "units"))) {
@@ -1297,16 +1243,14 @@ read_raw_data_field(guint xres, guint yres,
     gwy_data_field_set_yoffset(dfield, yoff);
     d = gwy_data_field_get_data(dfield);
     for (i = 0; i < yres; i++)
-        gwy_convert_raw_data(data + i*rowstride, xres, stride,
-                             rawdatatype, byteorder,
-                             d + i*xres, q, z0);
+        gwy_convert_raw_data(data + i*rowstride, xres, stride, rawdatatype, byteorder, d + i*xres, q, z0);
 
     if (siunitxy) {
-        gwy_data_field_set_si_unit_xy(dfield, siunitxy);
+        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), siunitxy);
         g_object_unref(siunitxy);
     }
     if (siunitz) {
-        gwy_data_field_set_si_unit_z(dfield, siunitz);
+        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), siunitz);
         g_object_unref(siunitz);
     }
 
@@ -1320,10 +1264,8 @@ read_raw_brick(guint xres, guint yres, guint zres,
                const guchar *data)
 {
     GwyBrick *brick;
-    GwySIUnit *siunitz = NULL, *siunitx = NULL, *siunity = NULL,
-              *siunitw = NULL;
-    gdouble dx = 1.0, dy = 1.0, dz = 1.0, q = 1.0, w0 = 0.0,
-            xoff = 0.0, yoff = 0.0, zoff = 0.0;
+    GwySIUnit *siunitz = NULL, *siunitx = NULL, *siunity = NULL, *siunitw = NULL;
+    gdouble dx = 1.0, dy = 1.0, dz = 1.0, q = 1.0, w0 = 0.0, xoff = 0.0, yoff = 0.0, zoff = 0.0;
     gdouble *d;
     gint power10;
     gchar *value;
@@ -1374,9 +1316,13 @@ read_raw_brick(guint xres, guint yres, guint zres,
     }
 
     if ((value = g_hash_table_lookup(fields, "sampleunits"))) {
-        siunitw = gwy_si_unit_new_parse(value, &power10);
-        q *= pow10(power10);
-        w0 *= pow10(power10);
+        gchar *unitw;
+        if (parse_string_vector(value, 1, &unitw)) {
+            siunitw = gwy_si_unit_new_parse(unitw, &power10);
+            q *= pow10(power10);
+            w0 *= pow10(power10);
+            g_free(unitw);
+        }
     }
 
     if ((value = g_hash_table_lookup(fields, "units"))) {
@@ -1399,23 +1345,22 @@ read_raw_brick(guint xres, guint yres, guint zres,
     gwy_brick_set_yoffset(brick, yoff);
     gwy_brick_set_zoffset(brick, zoff);
     d = gwy_brick_get_data(brick);
-    gwy_convert_raw_data(data, xres*yres*zres, 1, rawdatatype, byteorder, d,
-                         q, w0);
+    gwy_convert_raw_data(data, xres*yres*zres, 1, rawdatatype, byteorder, d, q, w0);
 
     if (siunitx) {
-        gwy_brick_set_si_unit_x(brick, siunitx);
+        gwy_si_unit_assign(gwy_brick_get_si_unit_x(brick), siunitx);
         g_object_unref(siunitx);
     }
     if (siunity) {
-        gwy_brick_set_si_unit_y(brick, siunity);
+        gwy_si_unit_assign(gwy_brick_get_si_unit_y(brick), siunity);
         g_object_unref(siunity);
     }
     if (siunitz) {
-        gwy_brick_set_si_unit_z(brick, siunitz);
+        gwy_si_unit_assign(gwy_brick_get_si_unit_z(brick), siunitz);
         g_object_unref(siunitz);
     }
     if (siunitw) {
-        gwy_brick_set_si_unit_w(brick, siunitw);
+        gwy_si_unit_assign(gwy_brick_get_si_unit_w(brick), siunitw);
         g_object_unref(siunitw);
     }
 
@@ -1558,8 +1503,7 @@ parse_string_vector(const gchar *value,
         else
             values[i] = g_strndup(begin, value - begin);
 
-        /* Technically this means we do not require whitespace between an
-         * ending " and another starting ". */
+        /* Technically this means we do not require whitespace between an ending " and another starting ". */
         value++;
     }
 
@@ -1595,10 +1539,8 @@ split_per_axis_field(const gchar *value,
 
         if (quoted) {
             if (len < 2 || f[0] != '"' || f[len-1] != '"') {
-                g_set_error(error, GWY_MODULE_FILE_ERROR,
-                            GWY_MODULE_FILE_ERROR_DATA,
-                            _("Items of per-axis header field %s are not "
-                              "quoted."),
+                g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
+                            _("Items of per-axis header field %s are not quoted."),
                             name);
                 goto fail;
             }
@@ -1606,10 +1548,8 @@ split_per_axis_field(const gchar *value,
             f++;
         }
         if (n == nitems) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
-                        _("Per-axis header field %s contains too many "
-                          "items."),
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
+                        _("Per-axis header field %s contains too many items."),
                         name);
             goto fail;
         }
@@ -1649,6 +1589,7 @@ nrrdfile_export(G_GNUC_UNUSED GwyContainer *data,
         "axismaxs: %s %s\n"
         "units: \"%s\" \"%s\"\n"
         "sampleunits: \"%s\"\n"
+        "kinds: space space\n"
         "\n";
 
     GwyDataField *dfield;
@@ -1667,8 +1608,7 @@ nrrdfile_export(G_GNUC_UNUSED GwyContainer *data,
         return FALSE;
     }
 
-    /* The specification says both kind of EOLs are fine so write Unix EOLs
-     * everywhere. */
+    /* The specification says both kind of EOLs are fine so write Unix EOLs everywhere. */
     if (!(fh = gwy_fopen(filename, "wb"))) {
         err_OPEN_WRITE(error);
         return FALSE;
@@ -1681,10 +1621,8 @@ nrrdfile_export(G_GNUC_UNUSED GwyContainer *data,
     yreal = gwy_data_field_get_yreal(dfield);
     xoff = gwy_data_field_get_xoffset(dfield);
     yoff = gwy_data_field_get_yoffset(dfield);
-    unitxy = gwy_si_unit_get_string(gwy_data_field_get_si_unit_xy(dfield),
-                                    GWY_SI_UNIT_FORMAT_PLAIN);
-    unitz = gwy_si_unit_get_string(gwy_data_field_get_si_unit_z(dfield),
-                                   GWY_SI_UNIT_FORMAT_PLAIN);
+    unitxy = gwy_si_unit_get_string(gwy_data_field_get_si_unit_xy(dfield), GWY_SI_UNIT_FORMAT_PLAIN);
+    unitz = gwy_si_unit_get_string(gwy_data_field_get_si_unit_z(dfield), GWY_SI_UNIT_FORMAT_PLAIN);
 
     g_ascii_formatd(buf[0], sizeof(buf[0]), "%.8g", xoff);
     g_ascii_formatd(buf[1], sizeof(buf[1]), "%.8g", yoff);
@@ -1693,10 +1631,10 @@ nrrdfile_export(G_GNUC_UNUSED GwyContainer *data,
 
     /* Write in native endian. */
     gwy_fprintf(fh, header_format,
-            G_BYTE_ORDER == G_LITTLE_ENDIAN ? "little" : "big",
-            xres, yres,
-            buf[0], buf[1], buf[2], buf[3],
-            unitxy, unitxy, unitz);
+                G_BYTE_ORDER == G_LITTLE_ENDIAN ? "little" : "big",
+                xres, yres,
+                buf[0], buf[1], buf[2], buf[3],
+                unitxy, unitxy, unitz);
     g_free(unitz);
     g_free(unitxy);
 
@@ -1715,4 +1653,4 @@ nrrdfile_export(G_GNUC_UNUSED GwyContainer *data,
     return ok;
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
