@@ -1,5 +1,5 @@
 /*
- *  $Id: omicronflat.c 22562 2019-10-10 14:45:41Z yeti-dn $
+ *  $Id: omicronflat.c 24006 2021-08-17 14:16:36Z yeti-dn $
  *  Copyright (C) 2015-2019 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -357,7 +357,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Omicron flat files."),
     "Yeti <yeti@gwyddion.net>",
-    "2.3",
+    "2.5",
     "David Nečas (Yeti)",
     "2015",
 };
@@ -2488,7 +2488,6 @@ read_double(const guchar **p, gsize *size, gdouble *v, GError **error)
 static gboolean
 read_string(const guchar **p, gsize *size, gchar **v, GError **error)
 {
-    GError *err = NULL;
     guint len;
 
     if (!read_uint32(p, size, &len, error))
@@ -2505,12 +2504,11 @@ read_string(const guchar **p, gsize *size, gchar **v, GError **error)
         return FALSE;
     }
 
-    *v = g_utf16_to_utf8((gunichar2*)*p, len, NULL, NULL, &err);
+    *v = gwy_utf16_to_utf8((const gunichar2*)*p, len,
+                           GWY_BYTE_ORDER_LITTLE_ENDIAN);
     if (!*v) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Cannot convert string from UTF-16: %s"),
-                    err->message);
-        g_clear_error(&err);
+                    _("Cannot convert string from UTF-16."));
         return FALSE;
     }
 
