@@ -1,5 +1,5 @@
 /*
- *  $Id: opengps.c 20689 2017-12-21 18:59:14Z yeti-dn $
+ *  $Id: opengps.c 22001 2019-04-16 13:58:04Z yeti-dn $
  *  Copyright (C) 2014 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -133,7 +133,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Reads ISO 5436-2 OpenGPS .x3p files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.3",
+    "1.4",
     "David Nečas (Yeti)",
     "2014",
 };
@@ -180,7 +180,7 @@ x3p_detect(const GwyFileDetectInfo *fileinfo,
     /* We have to realy look inside.  And since main.xml is a popular name
      * for the main XML document within such files, we also have to see if
      * we find "ISO5436_2" somewehre near the begining of the file. */
-    if ((zipfile = gwyzip_open(fileinfo->name))) {
+    if ((zipfile = gwyzip_open(fileinfo->name, NULL))) {
         if (gwyzip_locate_file(zipfile, "main.xml", 1, NULL)
             && (content = gwyzip_get_file_content(zipfile, NULL, NULL))) {
             if (g_strstr_len(content, 4096, "ISO5436_2"))
@@ -202,13 +202,8 @@ x3p_load(const gchar *filename,
     X3PFile x3pfile;
     GwyZipFile zipfile;
 
-    zipfile = gwyzip_open(filename);
-    if (!zipfile) {
-        g_set_error(error, GWY_MODULE_FILE_ERROR,
-                    GWY_MODULE_FILE_ERROR_SPECIFIC,
-                    _("Minizip cannot open the file as a ZIP file."));
+    if (!(zipfile = gwyzip_open(filename, NULL)))
         return NULL;
-    }
 
     gwy_clear(&x3pfile, 1);
     if (!x3p_parse_main(zipfile, &x3pfile, error))
