@@ -1,5 +1,5 @@
 /*
- *  $Id: pixmap.c 24638 2022-03-04 16:19:08Z yeti-dn $
+ *  $Id: pixmap.c 25045 2022-09-23 13:20:48Z yeti-dn $
  *  Copyright (C) 2004-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -166,6 +166,7 @@ static gboolean pixmap_filter_xpm     (const GwyFileDetectInfo *fileinfo);
 static gboolean pixmap_filter_ras     (const GwyFileDetectInfo *fileinfo);
 static gboolean pixmap_filter_icns    (const GwyFileDetectInfo *fileinfo);
 static gboolean pixmap_filter_webp    (const GwyFileDetectInfo *fileinfo);
+static gboolean pixmap_filter_heif    (const GwyFileDetectInfo *fileinfo);
 
 static const GwyEnum map_types[PIXMAP_MAP_NTYPES] = {
     { N_("All channels"), PIXMAP_MAP_ALL,   },
@@ -183,23 +184,24 @@ static const GwyEnum map_types[PIXMAP_MAP_NTYPES] = {
 /* Use a whitelist of safe formats for which we have at least basic weed-out pre-detection function.  GdkPixbuf
  * loaders tend to accept any rubbish as their format and then crash completely surprised when it isn't. */
 static const PixmapKnownFormat known_formats[] = {
-    { "png",      N_("Portable Network Graphics (.png)"),       pixmap_filter_png,      },
-    { "jpeg",     N_("JPEG (.jpeg,.jpg)"),                      pixmap_filter_jpeg,     },
+    { "png",      N_("Portable Network Graphics (.png)"),          pixmap_filter_png,      },
+    { "jpeg",     N_("JPEG (.jpeg,.jpg)"),                         pixmap_filter_jpeg,     },
 #ifndef __WIN64
     /* The TIFF loader (supposedly GDI-based) crashes on Win64.  Unclear why.  TIFF is madness.  There is a fallback
      * GwyTIFF-based loader in hdrimage which will take over when we exclude the GdkPixbuf-based one.  */
-    { "tiff",     N_("TIFF (.tiff,.tif)"),                      pixmap_filter_tiff,     },
+    { "tiff",     N_("TIFF (.tiff,.tif)"),                         pixmap_filter_tiff,     },
 #endif
-    { "pnm",      N_("Portable Pixmap (.ppm,.pnm)"),            pixmap_filter_pnm,      },
-    { "bmp",      N_("Windows or OS2 Bitmap (.bmp)"),           pixmap_filter_bmp,      },
-    { "tga",      N_("TARGA (.tga,.targa)"),                    pixmap_filter_tga,      },
-    { "gif",      N_("Graphics Interchange Format GIF (.gif)"), pixmap_filter_gif,      },
-    { "jpeg2000", N_("JPEG 2000 (.jpx)"),                       pixmap_filter_jpeg2000, },
-    { "pcx",      N_("PCX (.pcx)"),                             pixmap_filter_pcx,      },
-    { "xpm",      N_("X Pixmap (.xpm)"),                        pixmap_filter_xpm,      },
-    { "ras",      N_("Sun raster image (.ras)"),                pixmap_filter_ras,      },
-    { "icns",     N_("Apple icon (.icns)"),                     pixmap_filter_icns,     },
-    { "webp",     N_("WebP (.webp)"),                           pixmap_filter_webp,     },
+    { "pnm",      N_("Portable Pixmap (.ppm,.pnm)"),               pixmap_filter_pnm,      },
+    { "bmp",      N_("Windows or OS2 Bitmap (.bmp)"),              pixmap_filter_bmp,      },
+    { "tga",      N_("TARGA (.tga,.targa)"),                       pixmap_filter_tga,      },
+    { "gif",      N_("Graphics Interchange Format GIF (.gif)"),    pixmap_filter_gif,      },
+    { "jpeg2000", N_("JPEG 2000 (.jpx)"),                          pixmap_filter_jpeg2000, },
+    { "pcx",      N_("PCX (.pcx)"),                                pixmap_filter_pcx,      },
+    { "xpm",      N_("X Pixmap (.xpm)"),                           pixmap_filter_xpm,      },
+    { "ras",      N_("Sun raster image (.ras)"),                   pixmap_filter_ras,      },
+    { "icns",     N_("Apple icon (.icns)"),                        pixmap_filter_icns,     },
+    { "webp",     N_("WebP (.webp)"),                              pixmap_filter_webp,     },
+    { "heif",     N_("High Efficiency Image File Format (.heif)"), pixmap_filter_heif,     },
 };
 
 /* List of PixmapFormatInfo for all formats.  Created in module_register() and never freed. */
@@ -211,7 +213,7 @@ static GwyModuleInfo module_info = {
     N_("Imports data from low-depth pixmap images (PNG, TIFF, JPEG, ...).  The set of available formats depends on "
        "available GDK pixbuf loaders."),
     "Yeti <yeti@gwyddion.net>",
-    "10.0",
+    "10.1",
     "David Nečas (Yeti)",
     "2004-2014",
 };
@@ -232,6 +234,8 @@ module_register(void)
         guint i;
 
         fmtname = gdk_pixbuf_format_get_name(pixbuf_format);
+        if (gwy_strequal(fmtname, "heif/avif"))
+            fmtname = "heif";
         gwy_debug("Found format %s", fmtname);
         for (i = 0; i < G_N_ELEMENTS(known_formats); i++) {
             if (gwy_strequal(fmtname, known_formats[i].name)) {
@@ -244,7 +248,7 @@ module_register(void)
             continue;
         }
 
-        gwy_debug("Format %s is known and whitelisted.  Proceeding.");
+        gwy_debug("Format %s is known and whitelisted.  Proceeding.", fmtname);
         format_info = g_new0(PixmapFormatInfo, 1);
         format_info->name = fmtname;
         format_info->pixbuf_format = pixbuf_format;
@@ -295,6 +299,7 @@ pixmap_detect(const GwyFileDetectInfo *fileinfo,
     GdkPixbufLoader *loader;
     GError *err = NULL;
     PixmapFormatInfo *format_info;
+    const gchar *loadername;
     gint score;
 
     if (only_name)
@@ -313,9 +318,10 @@ pixmap_detect(const GwyFileDetectInfo *fileinfo,
     if (!format_info->filter_func(fileinfo))
         return 0;
 
-    gwy_debug("Creating a loader for type %s", name);
-    loader = gdk_pixbuf_loader_new_with_type(name, NULL);
-    gwy_debug("Loader for type %s: %p", name, loader);
+    loadername = (gwy_strequal(name, "heif") ? "heif/avif" : name);
+    gwy_debug("Creating a loader for type %s", loadername);
+    loader = gdk_pixbuf_loader_new_with_type(loadername, NULL);
+    gwy_debug("Loader for type %s: %p", loadername, loader);
     if (!loader)
         return 0;
 
@@ -323,7 +329,7 @@ pixmap_detect(const GwyFileDetectInfo *fileinfo,
      * feed anything to it just accept the file is a TIFF and hope some other loader of a TIFF-based format will claim
      * it with a higher score. */
     score = 70;
-    if (gwy_strequal(name, "tiff")) {
+    if (gwy_strequal(loadername, "tiff")) {
         gwy_debug("Avoiding feeding data to TIFF loader, calling gdk_pixbuf_loader_close().");
         gdk_pixbuf_loader_close(loader, NULL);
         gwy_debug("Unreferencing the TIFF loader");
@@ -444,6 +450,7 @@ pixmap_load_pixbuf(const gchar *filename,
     GdkPixbufLoader *loader;
     GdkPixbuf *pixbuf;
     PixmapFormatInfo *format_info;
+    const gchar *loadername;
     GError *err = NULL;
     FILE *fh;
     guint n;
@@ -462,8 +469,9 @@ pixmap_load_pixbuf(const gchar *filename,
         return NULL;
     }
 
-    gwy_debug("Creating a loader for type %s", name);
-    loader = gdk_pixbuf_loader_new_with_type(name, &err);
+    loadername = (gwy_strequal(name, "heif") ? "heif/avif" : name);
+    gwy_debug("Creating a loader for type %s", loadername);
+    loader = gdk_pixbuf_loader_new_with_type(loadername, &err);
     if (!loader) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_SPECIFIC,
                     _("Cannot get pixbuf loader: %s."), err->message);
@@ -1040,4 +1048,14 @@ pixmap_filter_webp(const GwyFileDetectInfo *fileinfo)
             && !memcmp(fileinfo->head + 8, "WEBPVP8", 7));
 }
 
+static gboolean
+pixmap_filter_heif(const GwyFileDetectInfo *fileinfo)
+{
+    return (fileinfo->buffer_len >= 12
+            && !memcmp(fileinfo->head + 4, "ftyp", 4)
+            && (!memcmp(fileinfo->head + 8, "mif1", 4)
+                || !memcmp(fileinfo->head + 8, "heic", 4)
+                || !memcmp(fileinfo->head + 8, "heix", 4)));
+}
+
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
