@@ -1,22 +1,18 @@
 /*
- *  $Id: psia.c 23870 2021-06-23 11:53:07Z yeti-dn $
- *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
+ *  $Id: psia.c 24013 2021-08-17 15:18:43Z yeti-dn $
+ *  Copyright (C) 2006-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -124,8 +120,7 @@ typedef struct {
     gboolean logscale;
     gboolean square;
     /* Only in version 2+
-     * NB: This must be interpreted as the new version can have different data
-     * types. */
+     * NB: This must be interpreted as the new version can have different data types. */
     gdouble z_servo_gain;
     gdouble z_scanner_range;
     gchar *xy_voltage_mode;  /* [8] */
@@ -171,8 +166,7 @@ typedef struct {
     gboolean log_scale[PSIA_MAX_SPECTRO_CHANNEL];
     gboolean square[PSIA_MAX_SPECTRO_CHANNEL];
     /* Version 2+ only */
-    gint spec_point_per_x;   /* If volume_image && !spec_point_per_x, then
-                                points are arranged in a square matrix */
+    gint spec_point_per_x;   /* If volume_image && !spec_point_per_x, then points are arranged in a square matrix */
     gboolean has_reference_image;
     gdouble xreal;     /* Grid dimensions */
     gdouble yreal;
@@ -245,7 +239,7 @@ static GwyModuleInfo module_info = {
     module_register,
     N_("Imports Park Systems data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.8",
+    "0.9",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -277,8 +271,7 @@ psia_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
         return score;
 
     /* Weed out non-TIFFs */
-    if (!gwy_tiff_detect(fileinfo->head, fileinfo->buffer_len,
-                         &tversion, &byteorder))
+    if (!gwy_tiff_detect(fileinfo->head, fileinfo->buffer_len, &tversion, &byteorder))
         return 0;
 
     if ((tiff = gwy_tiff_load(fileinfo->name, NULL))
@@ -341,8 +334,7 @@ psia_load_tiff(GwyTIFF *tiff, GError **error)
     /* Data */
     entry = gwy_tiff_find_tag(tiff, 0, PSIA_TIFFTAG_Data);
     if (!entry) {
-        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Data tag is missing."));
+        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA, _("Data tag is missing."));
         return NULL;
     }
     p = entry->value;
@@ -351,8 +343,7 @@ psia_load_tiff(GwyTIFF *tiff, GError **error)
     data_len = entry->count;
     gwy_debug("data_len: %d", data_len);
     if (data_len + count > tiff->size) {
-        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("File is truncated."));
+        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA, _("File is truncated."));
         return NULL;
     }
 
@@ -381,9 +372,7 @@ psia_load_tiff(GwyTIFF *tiff, GError **error)
 
     gwy_tiff_get_string0(tiff, PSIA_TIFFTAG_Comments, &comment);
 
-    dfield = gwy_data_field_new(header.xres, header.yres,
-                                header.xreal, header.yreal,
-                                FALSE);
+    dfield = gwy_data_field_new(header.xres, header.yres, header.xreal, header.yreal, FALSE);
 
     siunit = gwy_si_unit_new("m");
     gwy_data_field_set_si_unit_xy(dfield, siunit);
@@ -412,8 +401,7 @@ psia_load_tiff(GwyTIFF *tiff, GError **error)
     gwy_app_channel_check_nonsquare(container, 0);
 
     if (header.source_name && *header.source_name)
-        gwy_container_set_string_by_name(container, "/0/data/title",
-                                         g_strdup(header.source_name));
+        gwy_container_set_const_string_by_name(container, "/0/data/title", header.source_name);
 
     meta = psia_get_metadata(&header, version);
     if (comment && *comment) {
@@ -422,8 +410,7 @@ psia_load_tiff(GwyTIFF *tiff, GError **error)
         comment = NULL;
     }
     g_free(comment);
-    gwy_container_set_string_by_name(meta, "Version",
-                                     g_strdup_printf("%08x", version));
+    gwy_container_set_string_by_name(meta, "Version", g_strdup_printf("%08x", version));
 
     gwy_container_set_object_by_name(container, "/0/meta", meta);
     g_object_unref(meta);
@@ -466,10 +453,8 @@ psia_read_data_field(GwyDataField *dfield,
 
     xres = gwy_data_field_get_xres(dfield);
     yres = gwy_data_field_get_yres(dfield);
-    gwy_convert_raw_data(p, xres*yres, 1, rawdatatype,
-                         GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield),
-                         q*z_scale, q*z0);
+    gwy_convert_raw_data(p, xres*yres, 1, rawdatatype, GWY_BYTE_ORDER_LITTLE_ENDIAN,
+                         gwy_data_field_get_data(dfield), q*z_scale, q*z0);
 }
 
 static guint
@@ -483,11 +468,9 @@ psia_read_image_header(const guchar *p,
 
     gwy_clear(header, 1);
 
-    if ((version == PSIA_VERSION1 && size < 356)
-        || (version == PSIA_VERSION2 && size < 580)) {
+    if ((version == PSIA_VERSION1 && size < 356) || (version == PSIA_VERSION2 && size < 580)) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Header is too short (only %lu bytes)."),
-                    (gulong)size);
+                    _("Header is too short (only %lu bytes)."), (gulong)size);
         return 0;
     }
 
@@ -495,23 +478,20 @@ psia_read_image_header(const guchar *p,
     gwy_clear(header, 1);
     header->image_type = gwy_get_guint32_le(&p);
     gwy_debug("image_type: %d", header->image_type);
-    if (header->image_type != PSIA_2D_MAPPED
-        && header->image_type != PSIA_SPECTROSCOPY) {
+    if (header->image_type != PSIA_2D_MAPPED && header->image_type != PSIA_SPECTROSCOPY) {
         err_NO_DATA(error);
         return 0;
     }
     header->source_name = psia_wchar_to_utf8(&p, 32);
     header->image_mode = psia_wchar_to_utf8(&p, 8);
-    gwy_debug("source_name: <%s>, image_mode: <%s>",
-              header->source_name, header->image_mode);
+    gwy_debug("source_name: <%s>, image_mode: <%s>", header->source_name, header->image_mode);
     header->lpf_strength = gwy_get_gdouble_le(&p);
     header->auto_flatten = gwy_get_guint32_le(&p);
     header->ac_track = gwy_get_guint32_le(&p);
     header->xres = gwy_get_guint32_le(&p);
     header->yres = gwy_get_guint32_le(&p);
     gwy_debug("xres: %d, yres: %d", header->xres, header->yres);
-    if (err_DIMENSION(error, header->xres)
-        || err_DIMENSION(error, header->yres))
+    if (err_DIMENSION(error, header->xres) || err_DIMENSION(error, header->yres))
         return 0;
 
     header->angle = gwy_get_gdouble_le(&p);
@@ -578,8 +558,7 @@ psia_read_image_header(const guchar *p,
     gwy_debug("data_type: %d", header->data_type);
     if (header->data_type == PSIA_DATA_INT16)
         bps = 2;
-    else if (header->data_type == PSIA_DATA_INT32
-             || header->data_type == PSIA_DATA_FLOAT)
+    else if (header->data_type == PSIA_DATA_INT32 || header->data_type == PSIA_DATA_FLOAT)
         bps = 4;
     else
         err_DATA_TYPE(error, header->data_type);
@@ -833,17 +812,11 @@ psia_wchar_to_utf8(const guchar **src,
                    guint len)
 {
     gchar *s;
-    gunichar2 *wstr;
-    guint i;
 
-    wstr = g_memdup(*src, 2*len);
-    for (i = 0; i < len; i++)
-        wstr[i] = GUINT16_FROM_LE(wstr[i]);
-    s = g_utf16_to_utf8(wstr, len, NULL, NULL, NULL);
-    g_free(wstr);
-    *src += 2*len;
+    s = gwy_utf16_to_utf8((const gunichar2*)*src, len, GWY_BYTE_ORDER_LITTLE_ENDIAN);
+    *src += len*sizeof(gunichar2);
 
-    return s;
+    return s ? s : g_strdup("");
 }
 
 static GwyContainer*
@@ -855,90 +828,59 @@ psia_get_metadata(PSIAImageHeader *header,
     meta = gwy_container_new();
 
     if (header->source_name && *header->source_name) {
-        gwy_container_set_string_by_name(meta, "Source name",
-                                         header->source_name);
+        gwy_container_set_string_by_name(meta, "Source name", header->source_name);
         header->source_name = NULL;
     }
     if (header->image_mode && *header->image_mode) {
-        gwy_container_set_string_by_name(meta, "Image mode",
-                                         header->image_mode);
+        gwy_container_set_string_by_name(meta, "Image mode", header->image_mode);
         header->image_mode = NULL;
     }
 
     gwy_container_set_string_by_name(meta, "Version",
                                      g_strdup_printf("%u.%u.%u",
-                                                     version >> 24,
-                                                     (version >> 12) & 0xfff,
-                                                     version & 0xfff));
-    gwy_container_set_string_by_name(meta, "Overscan",
-                                     g_strdup_printf("%g %%",
-                                                     100*header->overscan_rate));
-    gwy_container_set_string_by_name(meta, "Fast direction",
-                                     g_strdup(header->swap_xy ? "Y" : "X"));
-    gwy_container_set_string_by_name(meta, "Angle",
-                                     g_strdup_printf("%g°", header->angle));
+                                                     version >> 24, (version >> 12) & 0xfff, version & 0xfff));
+    gwy_container_set_string_by_name(meta, "Overscan", g_strdup_printf("%g %%", 100*header->overscan_rate));
+    gwy_container_set_string_by_name(meta, "Fast direction", g_strdup(header->swap_xy ? "Y" : "X"));
+    gwy_container_set_string_by_name(meta, "Angle", g_strdup_printf("%g°", header->angle));
     gwy_container_set_string_by_name(meta, "Scanning direction",
-                                     g_strdup(header->scan_up
-                                              ? "Bottom to top"
-                                              : "Top to bottom"));
+                                     g_strdup(header->scan_up ? "Bottom to top" : "Top to bottom"));
     gwy_container_set_string_by_name(meta, "Line direction",
-                                     g_strdup(header->forward
-                                              ? "Left to right"
-                                              : "Right to left"));
-    gwy_container_set_string_by_name(meta, "Sine scan",
-                                     g_strdup(header->sine_scan
-                                              ? "Yes"
-                                              : "No"));
-    gwy_container_set_string_by_name(meta, "Scan rate",
-                                     g_strdup_printf("%g s<sup>-1</sup>",
-                                                     header->scan_rate));
+                                     g_strdup(header->forward ? "Left to right" : "Right to left"));
+    gwy_container_set_string_by_name(meta, "Sine scan", g_strdup(header->sine_scan ? "Yes" : "No"));
+    gwy_container_set_string_by_name(meta, "Scan rate", g_strdup_printf("%g s<sup>-1</sup>", header->scan_rate));
     gwy_container_set_string_by_name(meta, "Set point",
-                                     g_strdup_printf("%g %s",
-                                                     header->set_point,
-                                                     header->set_point_unit));
-    gwy_container_set_string_by_name(meta, "Tip bias",
-                                     g_strdup_printf("%g V", header->tip_bias));
-    gwy_container_set_string_by_name(meta, "Sample bias",
-                                     g_strdup_printf("%g V",
-                                                     header->sample_bias));
+                                     g_strdup_printf("%g %s", header->set_point, header->set_point_unit));
+    gwy_container_set_string_by_name(meta, "Tip bias", g_strdup_printf("%g V", header->tip_bias));
+    gwy_container_set_string_by_name(meta, "Sample bias", g_strdup_printf("%g V", header->sample_bias));
 
     if (version == PSIA_VERSION1)
         return meta;
 
     if (header->xy_voltage_mode && *header->xy_voltage_mode) {
-        gwy_container_set_string_by_name(meta, "XY voltage mode",
-                                         header->xy_voltage_mode);
+        gwy_container_set_string_by_name(meta, "XY voltage mode", header->xy_voltage_mode);
         header->xy_voltage_mode = NULL;
     }
 
     if (header->z_voltage_mode && *header->z_voltage_mode) {
-        gwy_container_set_string_by_name(meta, "Z voltage mode",
-                                         header->z_voltage_mode);
+        gwy_container_set_string_by_name(meta, "Z voltage mode", header->z_voltage_mode);
         header->z_voltage_mode = NULL;
     }
 
     if (header->xy_servo_mode && *header->xy_servo_mode) {
-        gwy_container_set_string_by_name(meta, "XY servo mode",
-                                         header->xy_servo_mode);
+        gwy_container_set_string_by_name(meta, "XY servo mode", header->xy_servo_mode);
         header->xy_servo_mode = NULL;
     }
 
     if (header->cantilever_name && *header->cantilever_name) {
-        gwy_container_set_string_by_name(meta, "Cantilever",
-                                         header->cantilever_name);
+        gwy_container_set_string_by_name(meta, "Cantilever", header->cantilever_name);
         header->cantilever_name = NULL;
     }
 
-    gwy_container_set_string_by_name(meta, "Z scanner range",
-                                     g_strdup_printf("%g",
-                                                     header->z_scanner_range));
-    gwy_container_set_string_by_name(meta, "Z servo gain",
-                                     g_strdup_printf("%g",
-                                                     header->z_servo_gain));
-    gwy_container_set_string_by_name(meta, "Head tilt angle",
-                                     g_strdup_printf("%g°", header->head_rotation_angle));
+    gwy_container_set_string_by_name(meta, "Z scanner range", g_strdup_printf("%g", header->z_scanner_range));
+    gwy_container_set_string_by_name(meta, "Z servo gain", g_strdup_printf("%g", header->z_servo_gain));
+    gwy_container_set_string_by_name(meta, "Head tilt angle", g_strdup_printf("%g°", header->head_rotation_angle));
 
     return meta;
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
