@@ -1,5 +1,5 @@
 /*
- *  $Id: renishaw.c 23130 2021-01-31 21:09:44Z dn2010 $
+ *  $Id: renishaw.c 24585 2022-02-05 10:52:39Z yeti-dn $
  *  Copyright (C) 2014 Daniil Bratashov (dn2010), David Necas (Yeti)..
  *  Data structures and constants are copyright (c) 2011 Renishaw plc.
  *
@@ -257,26 +257,26 @@ typedef enum {
 
 /* FIXME: It is only a small subset of known predefined keys */
 typedef enum {
-	WDF_PSET_KEYS_PROPERTIES = 403,
-	WDF_PSET_KEYS_LABEL      = 411,
-	WDF_PSET_KEYS_MAPTYPE    = 412,
-	WDF_PSET_KEYS_DATALIST0  = 420,
-	WDF_PSET_KEYS_DATALIST1  = 421,
-	WDF_PSET_KEYS_DATALIST2  = 422,
-	WDF_PSET_KEYS_OPERATOR   = 430,
-	WDF_PSET_KEYS_TIME       = 431,
-	WDF_PSET_KEYS_VERSION    = 432
+    WDF_PSET_KEYS_PROPERTIES = 403,
+    WDF_PSET_KEYS_LABEL      = 411,
+    WDF_PSET_KEYS_MAPTYPE    = 412,
+    WDF_PSET_KEYS_DATALIST0  = 420,
+    WDF_PSET_KEYS_DATALIST1  = 421,
+    WDF_PSET_KEYS_DATALIST2  = 422,
+    WDF_PSET_KEYS_OPERATOR   = 430,
+    WDF_PSET_KEYS_TIME       = 431,
+    WDF_PSET_KEYS_VERSION    = 432
 } WdfPsetPredefinedKeys;
 
 static const GwyEnum wdf_predefined_keys[] = {
-	{ "Properties",          WDF_PSET_KEYS_PROPERTIES },
-	{ "Label",               WDF_PSET_KEYS_LABEL      },
-	{ "MapType",             WDF_PSET_KEYS_MAPTYPE    },
-	{ "DataList0",           WDF_PSET_KEYS_DATALIST0  },
-	{ "DataList1",           WDF_PSET_KEYS_DATALIST1  },
-	{ "DataList2",           WDF_PSET_KEYS_DATALIST2  },
-	{ "Operator",            WDF_PSET_KEYS_OPERATOR   },
-	{ "Time",                WDF_PSET_KEYS_TIME       },
+    { "Properties",          WDF_PSET_KEYS_PROPERTIES },
+    { "Label",               WDF_PSET_KEYS_LABEL      },
+    { "MapType",             WDF_PSET_KEYS_MAPTYPE    },
+    { "DataList0",           WDF_PSET_KEYS_DATALIST0  },
+    { "DataList1",           WDF_PSET_KEYS_DATALIST1  },
+    { "DataList2",           WDF_PSET_KEYS_DATALIST2  },
+    { "Operator",            WDF_PSET_KEYS_OPERATOR   },
+    { "Time",                WDF_PSET_KEYS_TIME       },
     { "Version",             WDF_PSET_KEYS_VERSION    },
 };
 
@@ -392,7 +392,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Renishaw WiRE data files (WDF)."),
     "Daniil Bratashov <dn2010@gmail.com>",
-    "0.11",
+    "0.12",
     "Daniil Bratashov (dn2010), David Necas (Yeti), Renishaw plc.",
     "2014",
 };
@@ -448,7 +448,7 @@ wdf_load(const gchar *filename,
     GwyDataLine *cal;
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
-    GwySIUnit *siunitx, *siunity, *siunitz, *siunitw, *siunitsp, 
+    GwySIUnit *siunitx, *siunity, *siunitz, *siunitw, *siunitsp,
               *siunitm;
     gdouble *ydata, *xdata, *data, *zdata = NULL, *xdatasp, *ydatasp;
     gint i, j, k, l, lsize, z, m;
@@ -725,9 +725,7 @@ wdf_load(const gchar *filename,
         gwy_graph_curve_model_set_data(gcmodel, xdata, ydata, zres);
         g_free(xdata);
         g_free(ydata);
-        if (!gwy_graph_curve_model_is_ordered(gcmodel)) {
-            gwy_graph_curve_model_enforce_order(gcmodel);
-        }
+        gwy_graph_curve_model_enforce_order(gcmodel);
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
         g_free(title);
@@ -777,9 +775,7 @@ wdf_load(const gchar *filename,
             g_free(title);
             gwy_graph_curve_model_set_data(gcmodel, xdata, ydata, zres);
             g_free(ydata);
-            if (!gwy_graph_curve_model_is_ordered(gcmodel)) {
-                gwy_graph_curve_model_enforce_order(gcmodel);
-            }
+            gwy_graph_curve_model_enforce_order(gcmodel);
             gwy_graph_model_add_curve(gmodel, gcmodel);
             g_object_unref(gcmodel);
         }
@@ -790,65 +786,63 @@ wdf_load(const gchar *filename,
         gwy_container_set_object_by_name(container, "/0/graph/graph/1",
                                          gmodel);
         g_object_unref(gmodel);
-        
+
         for (m = 0; m < filedata.nummaps; m++) { /* z maps */
-			zres = filedata.maps[m]->numpoints;
-			
+            zres = filedata.maps[m]->numpoints;
+
             if (gwy_container_gis_string_by_name(filedata.maps[m]->pset,
                                                  "/Label", &label)) {
                 title = g_strdup_printf("%s", label);
             }
-            else 			
-				title = g_strdup_printf("Map %d", m + 1);
+            else
+                title = g_strdup_printf("Map %d", m + 1);
 
-			siunitm = NULL;
+            siunitm = NULL;
             siunitm = gwy_si_unit_new_parse(unit, &power10m);
             mscale = pow10(power10m);
             if (!siunitm) {
                 siunitm = gwy_si_unit_new_parse("", &power10m);
                 mscale = 1.0;
             }
-            				
+
             gmodel = g_object_new(GWY_TYPE_GRAPH_MODEL,
                                   "title", title,
                                   "si-unit-x", siunitm,
-                                  "si-unit-y", siunitw,            
+                                  "si-unit-y", siunitw,
                                   NULL);
-			g_object_unref(siunitm);
-			xdata = g_new0(gdouble, zres);
-			for (i = 0; i < zres; i++) {
-				if (zdata) {
-					xdata[i] = zdata[i] * mscale;
-				}
-				else { 
-					xdata[i] = i;
-				}
-			}
+            g_object_unref(siunitm);
+            xdata = g_new0(gdouble, zres);
+            for (i = 0; i < zres; i++) {
+                if (zdata) {
+                    xdata[i] = zdata[i] * mscale;
+                }
+                else {
+                    xdata[i] = i;
+                }
+            }
             ydata = g_new(gdouble, zres);
             gwy_convert_raw_data(filedata.maps[m]->data,
                                  zres, 1,
                                  GWY_RAW_DATA_FLOAT,
                                  GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                                 ydata, wscale, 0.0);	
+                                 ydata, wscale, 0.0);
 
             gcmodel = g_object_new(GWY_TYPE_GRAPH_CURVE_MODEL,
                                    "description", title,
                                    "mode", GWY_GRAPH_CURVE_LINE,
-                                   NULL);            
+                                   NULL);
             // g_free(title);
             gwy_graph_curve_model_set_data(gcmodel, xdata, ydata, zres);
             g_free(xdata);
-            g_free(ydata);   
-            if (!gwy_graph_curve_model_is_ordered(gcmodel)) {
-                gwy_graph_curve_model_enforce_order(gcmodel);
-            }
+            g_free(ydata);
+            gwy_graph_curve_model_enforce_order(gcmodel);
             gwy_graph_model_add_curve(gmodel, gcmodel);
-            g_object_unref(gcmodel); 
+            g_object_unref(gcmodel);
             key = g_strdup_printf("/0/graph/graph/%d", m + 2);
             gwy_container_set_object_by_name(container, key, gmodel);
             g_object_unref(gmodel);
-		}
-		g_object_unref(siunitw);
+        }
+        g_object_unref(siunitw);
     }
     else { /* some kind of scan */
         gwy_debug("length = %d %d %d",
@@ -1296,9 +1290,7 @@ wdf_load(const gchar *filename,
                                        NULL);
                 gwy_graph_curve_model_set_data(gcmodel,
                                                xdata, ydata, zres);
-                if (!gwy_graph_curve_model_is_ordered(gcmodel)) {
-                    gwy_graph_curve_model_enforce_order(gcmodel);
-                }                                               
+                gwy_graph_curve_model_enforce_order(gcmodel);
                 gwy_graph_model_add_curve(gmodel, gcmodel);
                 g_object_unref(gcmodel);
                 g_free(title);
@@ -1507,7 +1499,7 @@ wdf_read_maparea_block(const guchar *buffer,
 }
 
 static void
-set_standard_keys(gpointer key, 
+set_standard_keys(gpointer key,
                   G_GNUC_UNUSED gpointer value, gpointer user_data)
 {
     WdfPsetData *pdata;
@@ -1539,7 +1531,7 @@ extract_metadata_from_values(gpointer key, gpointer value,
     gpointer value_value;
 
     pdata = (WdfPsetData *)user_data;
-    gwy_debug("value = %s", (char *)value); 
+    gwy_debug("value = %s", (char *)value);
     value_value = g_hash_table_lookup(pdata->values, key);
     if (value && value_value) {
         gwy_container_set_string_by_name(pdata->data,
