@@ -1,5 +1,5 @@
 /*
- *  $Id: rhk-sm3.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: rhk-sm3.c 24009 2021-08-17 14:36:26Z yeti-dn $
  *  Copyright (C) 2005,2008 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -235,7 +235,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports RHK Technology SM3 data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.15",
+    "0.16",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -282,9 +282,8 @@ static guchar*
 rhk_sm3_read_string(const guchar **buffer,
                     gsize len)
 {
-    gchar *s, *p;
-    guint i, n;
-    gunichar chr;
+    gchar *s;
+    guint n;
 
     if (len < 2)
         return NULL;
@@ -294,15 +293,9 @@ rhk_sm3_read_string(const guchar **buffer,
     if (len < 2*n)
         return NULL;
 
-    if (!n)
-        return g_strdup("");
-
-    p = s = g_new(gchar, 6*n + 1);
-    for (i = 0; i < n; i++) {
-        chr = gwy_get_guint16_le(buffer);
-        p += g_unichar_to_utf8(chr, p);
-    }
-    *p = '\0';
+    s = gwy_utf16_to_utf8((const gunichar2*)*buffer, n,
+                          GWY_BYTE_ORDER_LITTLE_ENDIAN);
+    *buffer += n*sizeof(gunichar2);
     g_strstrip(s);
     gwy_debug("String: <%s>", s);
 
