@@ -1,23 +1,19 @@
 /*
- *  $Id: spcfile.c 24585 2022-02-05 10:52:39Z yeti-dn $
+ *  $Id: spcfile.c 25052 2022-09-27 13:17:52Z yeti-dn $
  *  Copyright (C) 2017 Daniil Bratashov (dn2010), David Necas (Yeti)..
  *
  *  E-mail: dn2010@gwyddion.net
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -142,69 +138,69 @@ typedef enum {
 } SPCExperimentType;
 
 static const GwyEnum spc_xunits[] = {
-    {"",        SPC_XUNITS_ARBITRARY },
-    {"1/cm",    SPC_XUNITS_WAVENUMBER },
-    {"µm",      SPC_XUNITS_MICROMETER },
-    {"nm",      SPC_XUNITS_NANOMETER },
-    {"s",       SPC_XUNITS_SECS },
-    {"minutes", SPC_XUNITS_MINUTES },
-    {"Hz",      SPC_XUNITS_HERTZ },
-    {"kHz",     SPC_XUNITS_KILOHERTZ },
-    {"MHz",     SPC_XUNITS_MEGAHERTZ },
-    {"",        SPC_XUNITS_MASSNUMBER },
-    {"PPM",     SPC_XUNITS_PPM },
-    {"days",    SPC_XUNITS_DAYS },
-    {"years",   SPC_XUNITS_YEARS },
-    {"1/cm",    SPC_XUNITS_RAMAN_SHIFT },
-    {"eV",      SPC_XUNITS_EV },
-    {"",        SPC_XUNITS_TEXTLABEL },
-    {"Diode",   SPC_XUNITS_DIODE },
-    {"Channel", SPC_XUNITS_CHANNEL },
-    {"deg",     SPC_XUNITS_DEGREES },
-    {"°F",      SPC_XUNITS_DEGREES_F },
-    {"°C",      SPC_XUNITS_DEGREES_C },
-    {"°K",      SPC_XUNITS_DEGREES_K },
-    {"pt",      SPC_XUNITS_POINTS },
-    {"ms",      SPC_XUNITS_MILLISECS },
-    {"µs",      SPC_XUNITS_MICROSECS },
-    {"ns",      SPC_XUNITS_NANOSECS },
-    {"GHz",     SPC_XUNITS_GIGAHERTZ },
-    {"cm",      SPC_XUNITS_CM },
-    {"m",       SPC_XUNITS_METER },
-    {"mm",      SPC_XUNITS_MM },
-    {"hours",   SPC_XUNITS_HOURS },
-    {"",        SPC_XUNITS_DOUBLE_IGM }
+    { "",        SPC_XUNITS_ARBITRARY,   },
+    { "1/cm",    SPC_XUNITS_WAVENUMBER,  },
+    { "µm",      SPC_XUNITS_MICROMETER,  },
+    { "nm",      SPC_XUNITS_NANOMETER,   },
+    { "s",       SPC_XUNITS_SECS,        },
+    { "minutes", SPC_XUNITS_MINUTES,     },
+    { "Hz",      SPC_XUNITS_HERTZ,       },
+    { "kHz",     SPC_XUNITS_KILOHERTZ,   },
+    { "MHz",     SPC_XUNITS_MEGAHERTZ,   },
+    { "",        SPC_XUNITS_MASSNUMBER,  },
+    { "PPM",     SPC_XUNITS_PPM,         },
+    { "days",    SPC_XUNITS_DAYS,        },
+    { "years",   SPC_XUNITS_YEARS,       },
+    { "1/cm",    SPC_XUNITS_RAMAN_SHIFT, },
+    { "eV",      SPC_XUNITS_EV,          },
+    { "",        SPC_XUNITS_TEXTLABEL,   },
+    { "Diode",   SPC_XUNITS_DIODE,       },
+    { "Channel", SPC_XUNITS_CHANNEL,     },
+    { "deg",     SPC_XUNITS_DEGREES,     },
+    { "°F",      SPC_XUNITS_DEGREES_F,   },
+    { "°C",      SPC_XUNITS_DEGREES_C,   },
+    { "°K",      SPC_XUNITS_DEGREES_K,   },
+    { "pt",      SPC_XUNITS_POINTS,      },
+    { "ms",      SPC_XUNITS_MILLISECS,   },
+    { "µs",      SPC_XUNITS_MICROSECS,   },
+    { "ns",      SPC_XUNITS_NANOSECS,    },
+    { "GHz",     SPC_XUNITS_GIGAHERTZ,   },
+    { "cm",      SPC_XUNITS_CM,          },
+    { "m",       SPC_XUNITS_METER,       },
+    { "mm",      SPC_XUNITS_MM,          },
+    { "hours",   SPC_XUNITS_HOURS,       },
+    { "",        SPC_XUNITS_DOUBLE_IGM,  },
 };
 
 static const GwyEnum spc_yunits[] = {
-    {"", SPC_YUNITS_ARBITRARY },
-    {"", SPC_YUNITS_INTERFEROGRAM },
-    {"", SPC_YUNITS_ABSORBANCE },
-    {"", SPC_YUNITS_KUBELKA_MONK },
-    {"Counts", SPC_YUNITS_COUNTS },
-    {"V", SPC_YUNITS_VOLTS },
-    {"deg", SPC_YUNITS_DEGREES },
-    {"mA", SPC_YUNITS_MILLIAMPS },
-    {"mm", SPC_YUNITS_MILLIMETERS },
-    {"mV", SPC_YUNITS_MVOLTS },
-    {"", SPC_YUNITS_YLOGONEDIVR },
-    {"%", SPC_YUNITS_YPERCENT },
-    {"", SPC_YUNITS_YINTENSITY },
-    {"", SPC_YUNITS_YRELINTENSITY },
-    {"", SPC_YUNITS_YENERGY },
-    {"dB", SPC_YUNITS_YDECIBEL },
-    {"°F", SPC_YUNITS_YDEGREEF },
-    {"°C", SPC_YUNITS_YDEGREEC },
-    {"°K", SPC_YUNITS_YDEGREEK },
-    {"", SPC_YUNITS_YINDEXREFLECT },
-    {"", SPC_YUNITS_YEXTINCTIONCF },
-    {"", SPC_YUNITS_YREAL },
-    {"", SPC_YUNITS_YIMAGINARY },
-    {"", SPC_YUNITS_YCOMPLEX },
-    {"", SPC_YUNITS_YTRANSMISSION },
-    {"", SPC_YUNITS_YREFLECTANCE },
-    {"", SPC_YUNITS_YVALLEY },
-    {"", SPC_YUNITS_YEMISN },
+    { "",       SPC_YUNITS_ARBITRARY,     },
+    { "",       SPC_YUNITS_INTERFEROGRAM, },
+    { "",       SPC_YUNITS_ABSORBANCE,    },
+    { "",       SPC_YUNITS_KUBELKA_MONK,  },
+    { "Counts", SPC_YUNITS_COUNTS,        },
+    { "V",      SPC_YUNITS_VOLTS,         },
+    { "deg",    SPC_YUNITS_DEGREES,       },
+    { "mA",     SPC_YUNITS_MILLIAMPS,     },
+    { "mm",     SPC_YUNITS_MILLIMETERS,   },
+    { "mV",     SPC_YUNITS_MVOLTS,        },
+    { "",       SPC_YUNITS_YLOGONEDIVR,   },
+    { "%",      SPC_YUNITS_YPERCENT,      },
+    { "",       SPC_YUNITS_YINTENSITY,    },
+    { "",       SPC_YUNITS_YRELINTENSITY, },
+    { "",       SPC_YUNITS_YENERGY,       },
+    { "dB",     SPC_YUNITS_YDECIBEL,      },
+    { "°F",     SPC_YUNITS_YDEGREEF,      },
+    { "°C",     SPC_YUNITS_YDEGREEC,      },
+    { "°K",     SPC_YUNITS_YDEGREEK,      },
+    { "",       SPC_YUNITS_YINDEXREFLECT, },
+    { "",       SPC_YUNITS_YEXTINCTIONCF, },
+    { "",       SPC_YUNITS_YREAL,         },
+    { "",       SPC_YUNITS_YIMAGINARY,    },
+    { "",       SPC_YUNITS_YCOMPLEX,      },
+    { "",       SPC_YUNITS_YTRANSMISSION, },
+    { "",       SPC_YUNITS_YREFLECTANCE,  },
+    { "",       SPC_YUNITS_YVALLEY,       },
+    { "",       SPC_YUNITS_YEMISN,        },
 };
 
 typedef struct {
@@ -263,24 +259,24 @@ typedef struct {
     gchar*   reserved; // 4
 } SPCSubHeader;
 
-static gboolean      module_register (void);
-static gint          spc_detect      (const GwyFileDetectInfo *fileinfo,
-                                      gboolean only_name);
-static GwyContainer* spc_load        (const gchar *filename,
-                                      GwyRunType mode,
-                                      GError **error);
-static void     spc_read_main_header (const guchar *buffer,
-                                      SPCMainHeader *header,
-                                      GError **error);
-static void     spc_read_subheader   (const guchar *buffer,
-                                      SPCSubHeader *header);
+static gboolean      module_register     (void);
+static gint          spc_detect          (const GwyFileDetectInfo *fileinfo,
+                                          gboolean only_name);
+static GwyContainer* spc_load            (const gchar *filename,
+                                          GwyRunType mode,
+                                          GError **error);
+static void          spc_read_main_header(const guchar *buffer,
+                                          SPCMainHeader *header,
+                                          GError **error);
+static void          spc_read_subheader  (const guchar *buffer,
+                                          SPCSubHeader *header);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Thermo Fisher SPC files."),
     "Daniil Bratashov <dn2010@gwyddion.net>",
-    "0.2",
+    "0.3",
     "Daniil Bratashov (dn2010), David Necas (Yeti)",
     "2018",
 };
@@ -302,16 +298,14 @@ module_register(void)
 
 static gint
 spc_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+           gboolean only_name)
 {
     gint score = 0;
 
     if (only_name)
-        return (g_str_has_suffix(fileinfo->name_lowercase, EXTENSION))
-               ? 10 : 0;
+        return (g_str_has_suffix(fileinfo->name_lowercase, EXTENSION)) ? 10 : 0;
 
-    if ((fileinfo->buffer_len > 512)
-     && (*(fileinfo->head + 1) == 0x4B)) {
+    if ((fileinfo->buffer_len > 512) && (*(fileinfo->head + 1) == 0x4B)) {
         score = 20;
         if (g_str_has_suffix(fileinfo->name_lowercase, EXTENSION))
             score += 10;
@@ -339,7 +333,7 @@ spc_load(const gchar *filename,
     const gchar *unit;
     gdouble *xdata, *ydata;
     gdouble scale, xscale, yscale;
-    gint i, zres, power10x, power10y;
+    gint i, zres, power10x, power10y, npoints;
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
     gchar **axesstrings;
@@ -361,12 +355,19 @@ spc_load(const gchar *filename,
     spc_read_main_header(p, header, &err);
     p += 512;
     remaining -= 512;
-    if (remaining < header->point_number * sizeof(gfloat) + 32) {
+    npoints = header->point_number;
+    if (remaining < npoints*sizeof(gfloat) + 32) {
         err_TOO_SHORT(error);
         g_free(header);
         goto fail;
     }
 
+    /* We can only import a single spectrum. */
+    if (header->subfiles_number != 1) {
+        err_NO_DATA(error);
+        goto fail;
+    }
+
     container = gwy_container_new();
     xscale = 1.0;
 
@@ -374,120 +375,96 @@ spc_load(const gchar *filename,
     unit = gwy_enum_to_string(header->x_units, spc_xunits, 32);
     siunitx = gwy_si_unit_new_parse(unit, &power10x);
     xscale = pow10(power10x);
-    if (xscale == 0.0) {
+    if (xscale == 0.0)
         xscale = 1.0;
-    }
 
     gwy_debug("y units = %d", header->y_units);
     unit = gwy_enum_to_string(header->y_units, spc_yunits, 28);
     siunity = gwy_si_unit_new_parse(unit, &power10y);
     yscale = pow10(power10y);
-    if (yscale == 0.0) {
+    if (yscale == 0.0)
         yscale = 1.0;
-    }
 
-    xdata = g_new0(gdouble, header->point_number);
+    xdata = g_new0(gdouble, npoints);
     if (header->xy_file) {
         if (!header->x_for_all) {
-            gwy_convert_raw_data(p, header->point_number, 1,
-                                 GWY_RAW_DATA_FLOAT,
-                                 GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                                 xdata, xscale, 0.0);
-            p += header->point_number * sizeof(gfloat);
-            remaining -= header->point_number * sizeof(gfloat);
+            gwy_convert_raw_data(p, npoints, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN, xdata, xscale, 0.0);
+            p += npoints * sizeof(gfloat);
+            remaining -= npoints * sizeof(gfloat);
         }
     }
     else {
-        for (i = 0; i < header->point_number; i++) {
-            *(xdata + i) = xscale * (header->x_first
-                + (gdouble)i / (header->point_number - 1)
-                    * (header->x_last - header->x_first));
+        for (i = 0; i < npoints; i++) {
+            xdata[i] = xscale*(header->x_first + i/(npoints - 1.0) * (header->x_last - header->x_first));
         }
     }
 
-    if (header->subfiles_number == 1) { /* Single spectrum */
-        subheader = g_new0(SPCSubHeader, 1);
-        spc_read_subheader(p, subheader);
-        p += 32;
-        remaining -= 32;
-        zres = header->point_number;
+    subheader = g_new0(SPCSubHeader, 1);
+    spc_read_subheader(p, subheader);
+    p += 32;
+    remaining -= 32;
+    zres = npoints;
 
-        if (header->x_for_all) {
-            zres = subheader->point_number;
-            g_free(xdata);
-            xdata = g_new0(gdouble, zres);
-            gwy_debug("converting x data, remaining=%d", remaining);
-            gwy_convert_raw_data(p, zres, 1,
-                                 GWY_RAW_DATA_FLOAT,
-                                 GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                                 xdata, xscale, 0.0);
-            p += zres * sizeof(gfloat);
-            remaining -= zres * sizeof(gfloat);
-        }
-
-        ydata = g_new0(gdouble, zres);
-        gwy_debug("converting y data, remaining=%d", remaining);
-        if (header->exponent == 0x80) {
-            gwy_convert_raw_data(p, zres, 1,
-                                 GWY_RAW_DATA_FLOAT,
-                                 GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                                 ydata, yscale, 0.0);
-            p += zres * sizeof(gfloat);
-            remaining -= zres * sizeof(gfloat);
-        }
-        else if (header->precision16bit) {
-            scale = exp2(header->exponent)/65536.0 * yscale;
-            gwy_convert_raw_data(p, zres, 1,
-                                 GWY_RAW_DATA_SINT16,
-                                 GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                                 ydata, scale, 0.0);
-            p += zres * sizeof(gint16);
-            remaining -= zres * sizeof(gint16);
-        }
-        else {
-            scale = exp2(header->exponent)/4294967296.0 * yscale;
-            gwy_convert_raw_data(p, zres, 1,
-                                 GWY_RAW_DATA_SINT32,
-                                 GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                                 ydata, scale, 0.0);
-            p += zres * sizeof(gint32);
-            remaining -= zres * sizeof(gint32);
-        }
-
-        gmodel = g_object_new(GWY_TYPE_GRAPH_MODEL,
-                             "si-unit-x", siunitx,
-                             "si-unit-y", siunity,
-                              NULL);
-
-        if (header->custom_axis_labels) {
-            axesstrings = g_strsplit(header->custom_axis_strings,
-                                     " ", -1);
-            if (g_strv_length(axesstrings) >= 2) {
-                g_object_set(gmodel,
-                             "axis-label-bottom", axesstrings[0],
-                             "axis-label-left", axesstrings[1],
-                             NULL);
-            }
-            g_strfreev (axesstrings);
-        }
-
-        gcmodel = g_object_new(GWY_TYPE_GRAPH_CURVE_MODEL,
-                               "mode", GWY_GRAPH_CURVE_LINE,
-                               "color", gwy_graph_get_preset_color(0),
-                               NULL);
-        gwy_graph_curve_model_set_data(gcmodel, xdata, ydata,
-                                       zres);
+    if (header->x_for_all) {
+        zres = subheader->point_number;
         g_free(xdata);
-        g_free(ydata);
-        gwy_graph_curve_model_enforce_order(gcmodel);
-        gwy_graph_model_add_curve(gmodel, gcmodel);
-        g_object_unref(gcmodel);
-        gwy_container_set_object_by_name(container, "/0/graph/graph/1",
-                                         gmodel);
-        g_object_unref(gmodel);
-        g_free(subheader);
+        xdata = g_new0(gdouble, zres);
+        gwy_debug("converting x data, remaining=%d", remaining);
+        gwy_convert_raw_data(p, zres, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN, xdata, xscale, 0.0);
+        p += zres * sizeof(gfloat);
+        remaining -= zres * sizeof(gfloat);
     }
 
+    ydata = g_new0(gdouble, zres);
+    gwy_debug("converting y data, remaining=%d", remaining);
+    if (header->exponent == 0x80) {
+        gwy_convert_raw_data(p, zres, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN, ydata, yscale, 0.0);
+        p += zres * sizeof(gfloat);
+        remaining -= zres * sizeof(gfloat);
+    }
+    else if (header->precision16bit) {
+        scale = exp2(header->exponent)/65536.0 * yscale;
+        gwy_convert_raw_data(p, zres, 1, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN, ydata, scale, 0.0);
+        p += zres * sizeof(gint16);
+        remaining -= zres * sizeof(gint16);
+    }
+    else {
+        scale = exp2(header->exponent)/4294967296.0 * yscale;
+        gwy_convert_raw_data(p, zres, 1, GWY_RAW_DATA_SINT32, GWY_BYTE_ORDER_LITTLE_ENDIAN, ydata, scale, 0.0);
+        p += zres * sizeof(gint32);
+        remaining -= zres * sizeof(gint32);
+    }
+
+    gmodel = g_object_new(GWY_TYPE_GRAPH_MODEL,
+                          "si-unit-x", siunitx,
+                          "si-unit-y", siunity,
+                          NULL);
+
+    if (header->custom_axis_labels) {
+        axesstrings = g_strsplit(header->custom_axis_strings, " ", -1);
+        if (g_strv_length(axesstrings) >= 2) {
+            g_object_set(gmodel,
+                         "axis-label-bottom", axesstrings[0],
+                         "axis-label-left", axesstrings[1],
+                         NULL);
+        }
+        g_strfreev(axesstrings);
+    }
+
+    gcmodel = g_object_new(GWY_TYPE_GRAPH_CURVE_MODEL,
+                           "mode", GWY_GRAPH_CURVE_LINE,
+                           "color", gwy_graph_get_preset_color(0),
+                           NULL);
+    gwy_graph_curve_model_set_data(gcmodel, xdata, ydata, zres);
+    g_free(xdata);
+    g_free(ydata);
+    gwy_graph_curve_model_enforce_order(gcmodel);
+    gwy_graph_model_add_curve(gmodel, gcmodel);
+    g_object_unref(gcmodel);
+    gwy_container_set_object_by_name(container, "/0/graph/graph/1", gmodel);
+    g_object_unref(gmodel);
+    g_free(subheader);
+
     g_object_unref(siunitx);
     g_object_unref(siunity);
     g_free(header);
@@ -523,8 +500,7 @@ spc_read_main_header(const guchar *buffer,
     header->experiment_type_code = *(p++);
     gwy_debug("experiment type=%d", header->experiment_type_code);
 
-    if ((header->version != 0x4B)
-     || (header->experiment_type_code > 14)) {
+    if ((header->version != 0x4B) || (header->experiment_type_code > 14)) {
         err_FILE_TYPE(error, "Thermo Fisher SPC");
         return;
     }
@@ -543,17 +519,17 @@ spc_read_main_header(const guchar *buffer,
     header->posting_disposition = *(p++);
     header->date = gwy_get_guint32_le(&p);
 
-    header->resolution_description = (guchar *)p;
+    header->resolution_description = (guchar*)p;
     p += 9;
-    header->source_instrument = (guchar *)p;
+    header->source_instrument = (guchar*)p;
     p += 9;
     header->peak_points = gwy_get_guint16_le(&p);
     for (i = 0; i < 8; i++) {
         header->spare[i] = gwy_get_gfloat_le(&p);
     }
-    header->memo = (guchar *)p;
+    header->memo = (guchar*)p;
     p += 130;
-    header->custom_axis_strings = (guchar *)p;
+    header->custom_axis_strings = (guchar*)p;
     p += 30;
     header->logblock_offset = gwy_get_guint32_le(&p);
     header->file_modification = gwy_get_guint32_le(&p);
@@ -561,14 +537,14 @@ spc_read_main_header(const guchar *buffer,
     header->calibration_level = *(p++);
     header->submetod_sample_injection = gwy_get_guint16_le(&p);
     header->concentration_factor = gwy_get_gfloat_le(&p);
-    header->method_file = (guchar *)p;
+    header->method_file = (guchar*)p;
     p += 48;
     header->z_subfile_increment = gwy_get_gfloat_le(&p);
     header->w_planes = gwy_get_gint32_le(&p);
     gwy_debug("w planes= %d", header->w_planes);
     header->w_increment = gwy_get_gfloat_le(&p);
     header->w_units = *(p++);
-    header->reserved = (guchar *)p;
+    header->reserved = (guchar*)p;
     p += 187;
 }
 
@@ -583,8 +559,7 @@ spc_read_subheader(const guchar *buffer,
     flags = *(p++);
     header->subfile_changed = (flags & 1) ? TRUE : FALSE;
     header->do_not_use_peak_table = (flags & 8) ? TRUE : FALSE;
-    header->subfile_modified_by_arithmetic
-                                         = (flags & 128) ? TRUE : FALSE;
+    header->subfile_modified_by_arithmetic = (flags & 128) ? TRUE : FALSE;
     header->exponent = *(p++);
     header->subfile_index = gwy_get_guint16_le(&p);
     header->z_start_value = gwy_get_gfloat_le(&p);
@@ -598,4 +573,4 @@ spc_read_subheader(const guchar *buffer,
     p += 4;
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
