@@ -1,26 +1,21 @@
 /*
- *  $Id: spip-asc.c 22480 2019-09-13 14:37:31Z yeti-dn $
+ *  $Id: spip-asc.c 24698 2022-03-21 12:12:25Z yeti-dn $
  *  Copyright (C) 2009-2019 David Necas (Yeti).
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* FIXME: Not sure where these come from, but the files tend to bear
- * `created by SPIP'.  The field names resemble BCR, but the format is not
- * the same.  So let's call the format SPIP ASCII data... */
+/* FIXME: Not sure where these come from, but the files tend to bear `created by SPIP'.  The field names resemble BCR,
+ * but the format is not the same.  So let's call the format SPIP ASCII data... */
 /**
  * [FILE-MAGIC-FREEDESKTOP]
  * <mime-type type="application/x-spip-asc">
@@ -62,10 +57,9 @@
 
 #include "err.h"
 
-#define MAGIC_BARE "# File Format = ASCII"
-#define MAGIC1 MAGIC_BARE "\r\n"
-#define MAGIC2 MAGIC_BARE "\n"
-#define MAGIC1_SIZE (sizeof(MAGIC1)-1)
+#define MAGIC "# File Format = ASCII"
+#define MAGIC_SIZE (sizeof(MAGIC)-1)
+#define MAGIC2 "# Created by "
 #define MAGIC2_SIZE (sizeof(MAGIC2)-1)
 #define EXTENSION ".asc"
 
@@ -91,15 +85,15 @@ static gboolean      asc_export       (GwyContainer *data,
                                        GwyRunType mode,
                                        GError **error);
 static gchar*        asc_format_header(GwyContainer *data,
-                                       GwyDataField *dfield,
+                                       GwyDataField *field,
                                        gboolean *zunit_is_nm);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
-    N_("Imports SPIP ASC files."),
+    N_("Imports and exports SPIP ASC files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.6",
+    "0.7",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -120,18 +114,26 @@ module_register(void)
 }
 
 static gint
-asc_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+asc_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
+    gint seplen;
+
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
 
-    if (fileinfo->file_size < MAX(MAGIC1_SIZE, MAGIC2_SIZE)
-        || (memcmp(fileinfo->head, MAGIC1, MAGIC1_SIZE) != 0
-            && memcmp(fileinfo->head, MAGIC2, MAGIC2_SIZE) != 0))
+    if (fileinfo->file_size < MAGIC_SIZE + MAGIC2_SIZE + 4 || memcmp(fileinfo->head, MAGIC, MAGIC_SIZE) != 0)
         return 0;
 
-    return 100;
+    if (fileinfo->head[MAGIC_SIZE] == '\r')
+        seplen = (fileinfo->head[MAGIC_SIZE+1] == '\n' ? 2 : 1);
+    else if (fileinfo->head[MAGIC_SIZE] == '\n')
+        seplen = 1;
+    else
+        return 0;
+
+    /* Return nonzero score for files with matching first line, but a high score only for files with matching second
+     * line. */
+    return memcmp(fileinfo->head + MAGIC_SIZE + seplen, MAGIC2, MAGIC2_SIZE) ? 50 : 95;
 }
 
 static gboolean
@@ -171,7 +173,7 @@ asc_load(const gchar *filename,
 
     p = buffer;
     line = gwy_str_next_line(&p);
-    if (!gwy_strequal(line, MAGIC_BARE)) {
+    if (!gwy_strequal(line, MAGIC)) {
         err_FILE_TYPE(error, "SPIP ASCII data");
         goto fail;
     }
@@ -186,17 +188,12 @@ asc_load(const gchar *filename,
         g_propagate_error(error, err);
         goto fail;
     }
-    if (require_keys(hash, NULL,
-                     "x-pixels", "y-pixels", "x-length", "y-length",
-                     NULL)) {
+    if (require_keys(hash, NULL, "x-pixels", "y-pixels", "x-length", "y-length", NULL))
         container = read_image_data(hash, p, filename, error);
-    }
-    else if (require_keys(hash, NULL, "points", "length", NULL)) {
+    else if (require_keys(hash, NULL, "points", "length", NULL))
         container = read_graph_data(hash, buffer, p, filename, error);
-    }
-    else {
+    else
         err_NO_DATA(error);
-    }
 
 fail:
     g_free(buffer);
@@ -211,8 +208,7 @@ read_image_data(GHashTable *hash, gchar *p, const gchar *filename,
                 GError **error)
 {
     GwyContainer *container = NULL;
-    GwyDataField *dfield = NULL, *mfield = NULL;
-    GwySIUnit *unit;
+    GwyDataField *field = NULL, *mfield = NULL;
     gchar *value;
     gdouble xreal, yreal, q;
     gint i, xres, yres;
@@ -223,10 +219,8 @@ read_image_data(GHashTable *hash, gchar *p, const gchar *filename,
     if (err_DIMENSION(error, xres) || err_DIMENSION(error, yres))
         return NULL;
 
-    xreal = Nanometer * g_ascii_strtod(g_hash_table_lookup(hash, "x-length"),
-                                       NULL);
-    yreal = Nanometer * g_ascii_strtod(g_hash_table_lookup(hash, "y-length"),
-                                       NULL);
+    xreal = Nanometer * g_ascii_strtod(g_hash_table_lookup(hash, "x-length"), NULL);
+    yreal = Nanometer * g_ascii_strtod(g_hash_table_lookup(hash, "y-length"), NULL);
     /* Use negated positive conditions to catch NaNs */
     if (!((xreal = fabs(xreal)) > 0)) {
         g_warning("Real x size is 0.0, fixing to 1.0");
@@ -237,75 +231,60 @@ read_image_data(GHashTable *hash, gchar *p, const gchar *filename,
         yreal = 1.0;
     }
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-
-    unit = gwy_si_unit_new("m");
-    gwy_data_field_set_si_unit_xy(dfield, unit);
-    g_object_unref(unit);
+    field = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
+    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
 
     if ((value = g_hash_table_lookup(hash, "z-unit"))) {
         gint power10;
-
-        unit = gwy_si_unit_new_parse(value, &power10);
-        gwy_data_field_set_si_unit_z(dfield, unit);
-        g_object_unref(unit);
+        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(field), value, &power10);
         q = pow10(power10);
     }
     else if ((value = g_hash_table_lookup(hash, "Bit2nm"))) {
+        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), "m");
         q = Nanometer * g_ascii_strtod(value, NULL);
-        unit = gwy_si_unit_new("m");
-        gwy_data_field_set_si_unit_z(dfield, unit);
-        g_object_unref(unit);
     }
     else
         q = 1.0;
 
-    data = gwy_data_field_get_data(dfield);
+    data = gwy_data_field_get_data(field);
     value = p;
     for (i = 0; i < xres*yres; i++) {
         data[i] = q*g_ascii_strtod(value, &p);
         if (p == value && (!*p || g_ascii_isspace(*p))) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
-                        _("End of file reached when reading sample #%d of %d"),
-                        i, xres*yres);
-            g_object_unref(dfield);
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
+                        _("End of file reached when reading sample #%d of %d"), i, xres*yres);
+            g_object_unref(field);
             return NULL;
         }
         if (p == value) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
-                        _("Malformed data encountered when reading sample "
-                          "#%d of %d"),
-                        i, xres*yres);
-            g_object_unref(dfield);
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
+                        _("Malformed data encountered when reading sample #%d of %d"), i, xres*yres);
+            g_object_unref(field);
             return NULL;
         }
         value = p;
     }
 
     if ((value = g_hash_table_lookup(hash, "voidpixels")) && atoi(value)) {
-        mfield = gwy_data_field_new_alike(dfield, FALSE);
+        mfield = gwy_data_field_new_alike(field, FALSE);
         data = gwy_data_field_get_data(mfield);
         value = p;
         for (i = 0; i < xres*yres; i++) {
             data[i] = 1.0 - g_ascii_strtod(value, &p);
             value = p;
         }
-        if (!gwy_app_channel_remove_bad_data(dfield, mfield))
+        if (!gwy_app_channel_remove_bad_data(field, mfield))
             GWY_OBJECT_UNREF(mfield);
     }
 
     container = gwy_container_new();
-
-    gwy_container_set_object(container, gwy_app_get_data_key_for_id(0), dfield);
-
+    gwy_container_set_object(container, gwy_app_get_data_key_for_id(0), field);
+    g_object_unref(field);
     if (mfield) {
-        gwy_container_set_object(container, gwy_app_get_mask_key_for_id(0),
-                                 mfield);
+        gwy_container_set_object(container, gwy_app_get_mask_key_for_id(0), mfield);
         g_object_unref(mfield);
     }
-
+    gwy_app_channel_title_fall_back(container, 0);
     gwy_file_channel_import_log_add(container, 0, NULL, filename);
 
     return container;
@@ -335,9 +314,8 @@ read_graph_data(GHashTable *hash, gchar *buffer, gchar *p,
     if (err_DIMENSION(error, npoints))
         return NULL;
 
-    /* Unfortunately, the axes are given in the header in some random format,
-     * different from other header lines.  So the text header parser discards
-     * them and we have to extract them separately. */
+    /* Unfortunately, the axes are given in the header in some random format, different from other header lines.  So
+     * the text header parser discards them and we have to extract them separately. */
     if (!gwy_file_get_contents(filename, &buf2, &size2, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
@@ -352,8 +330,7 @@ read_graph_data(GHashTable *hash, gchar *buffer, gchar *p,
     header[p-buffer] = '\0';
     gwy_file_abandon_contents(buf2, size2, NULL);
 
-    regex = g_regex_new("^#\\s*X-Axis:\\s*(?P<xunit>[^;]*);"
-                        "\\s*Y-Axis:\\s*(?P<yunit>.*?)\\s*$",
+    regex = g_regex_new("^#\\s*X-Axis:\\s*(?P<xunit>[^;]*);\\s*Y-Axis:\\s*(?P<yunit>.*?)\\s*$",
                         G_REGEX_MULTILINE | G_REGEX_NO_AUTO_CAPTURE, 0, NULL);
     g_assert(regex);
 
@@ -394,10 +371,8 @@ read_graph_data(GHashTable *hash, gchar *buffer, gchar *p,
         line = end;
         y = g_ascii_strtod(line, &end);
         if (end == line) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
-                        _("Malformed data encountered when reading sample #%u"),
-                        i);
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
+                        _("Malformed data encountered when reading sample #%u"), i);
             g_free(xydata);
             g_object_unref(xunit);
             g_object_unref(yunit);
@@ -409,8 +384,7 @@ read_graph_data(GHashTable *hash, gchar *buffer, gchar *p,
     }
     if (i < npoints) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("End of file reached when reading sample #%u of %u"),
-                    i, npoints);
+                    _("End of file reached when reading sample #%u of %u"), i, npoints);
         g_free(xydata);
         g_object_unref(xunit);
         g_object_unref(yunit);
@@ -428,15 +402,13 @@ read_graph_data(GHashTable *hash, gchar *buffer, gchar *p,
         g_object_set(gmodel, "title", s, NULL);
         g_object_set(gcmodel, "description", s, NULL);
     }
-    gwy_graph_curve_model_set_data_interleaved(gcmodel,
-                                               (gdouble*)xydata, npoints);
+    gwy_graph_curve_model_set_data_interleaved(gcmodel, (gdouble*)xydata, npoints);
     g_free(xydata);
     gwy_graph_model_add_curve(gmodel, gcmodel);
     g_object_unref(gcmodel);
 
     container = gwy_container_new();
-    gwy_container_set_object(container, gwy_app_get_graph_key_for_id(1),
-                             gmodel);
+    gwy_container_set_object(container, gwy_app_get_graph_key_for_id(1), gmodel);
     g_object_unref(gmodel);
 
     return container;
@@ -448,16 +420,16 @@ asc_export(GwyContainer *data,
            G_GNUC_UNUSED GwyRunType mode,
            GError **error)
 {
-    GwyDataField *dfield;
+    GwyDataField *field;
     guint xres, i, n;
     gchar *header;
     const gdouble *d;
     gboolean zunit_is_nm;
     FILE *fh;
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield, 0);
+    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field, 0);
 
-    if (!dfield) {
+    if (!field) {
         err_NO_CHANNEL_EXPORT(error);
         return FALSE;
     }
@@ -467,13 +439,13 @@ asc_export(GwyContainer *data,
         return FALSE;
     }
 
-    header = asc_format_header(data, dfield, &zunit_is_nm);
+    header = asc_format_header(data, field, &zunit_is_nm);
     if (fputs(header, fh) == EOF)
         goto fail;
 
-    d = gwy_data_field_get_data_const(dfield);
-    xres = gwy_data_field_get_xres(dfield);
-    n = xres*gwy_data_field_get_yres(dfield);
+    d = gwy_data_field_get_data_const(field);
+    xres = gwy_data_field_get_xres(field);
+    n = xres*gwy_data_field_get_yres(field);
     for (i = 0; i < n; i++) {
         gchar buf[G_ASCII_DTOSTR_BUF_SIZE];
         gchar c;
@@ -506,7 +478,7 @@ fail:
 }
 
 static gchar*
-asc_format_header(GwyContainer *data, GwyDataField *dfield,
+asc_format_header(GwyContainer *data, GwyDataField *field,
                   gboolean *zunit_is_nm)
 {
     static const gchar asc_header_template[] =
@@ -525,20 +497,17 @@ asc_format_header(GwyContainer *data, GwyDataField *dfield,
 
     GwySIUnit *zunit;
     gchar *header, *zunit_str, *zunit_line;
-    gchar xreal_str[G_ASCII_DTOSTR_BUF_SIZE],
-          yreal_str[G_ASCII_DTOSTR_BUF_SIZE],
-          xoff_str[G_ASCII_DTOSTR_BUF_SIZE],
-          yoff_str[G_ASCII_DTOSTR_BUF_SIZE];
+    gchar xreal_str[G_ASCII_DTOSTR_BUF_SIZE], yreal_str[G_ASCII_DTOSTR_BUF_SIZE],
+          xoff_str[G_ASCII_DTOSTR_BUF_SIZE], yoff_str[G_ASCII_DTOSTR_BUF_SIZE];
     const guchar *filename = "NONE";
     gdouble xreal, yreal, xoff, yoff;
 
-    /* XXX: Gwyddion can have lateral dimensions as whatever we want.  But
-     * who knows about the SPIP ASC format... */
-    xreal = gwy_data_field_get_xreal(dfield)/Nanometer;
-    yreal = gwy_data_field_get_yreal(dfield)/Nanometer;
-    xoff = gwy_data_field_get_xoffset(dfield)/Nanometer;
-    yoff = gwy_data_field_get_yoffset(dfield)/Nanometer;
-    zunit = gwy_data_field_get_si_unit_z(dfield);
+    /* XXX: Gwyddion can have lateral dimensions as whatever we want.  But who knows about the SPIP ASC format... */
+    xreal = gwy_data_field_get_xreal(field)/Nanometer;
+    yreal = gwy_data_field_get_yreal(field)/Nanometer;
+    xoff = gwy_data_field_get_xoffset(field)/Nanometer;
+    yoff = gwy_data_field_get_yoffset(field)/Nanometer;
+    zunit = gwy_data_field_get_si_unit_z(field);
 
     g_ascii_dtostr(xreal_str, G_ASCII_DTOSTR_BUF_SIZE, xreal);
     g_ascii_dtostr(yreal_str, G_ASCII_DTOSTR_BUF_SIZE, yreal);
@@ -554,10 +523,8 @@ asc_format_header(GwyContainer *data, GwyDataField *dfield,
 
     header = g_strdup_printf(asc_header_template,
                              gwy_version_string(), filename,
-                             gwy_data_field_get_xres(dfield),
-                             gwy_data_field_get_yres(dfield),
-                             xreal_str, yreal_str, xoff_str, yoff_str,
-                             zunit_line);
+                             gwy_data_field_get_xres(field), gwy_data_field_get_yres(field),
+                             xreal_str, yreal_str, xoff_str, yoff_str, zunit_line);
 
     g_free(zunit_str);
     g_free(zunit_line);
@@ -565,4 +532,4 @@ asc_format_header(GwyContainer *data, GwyDataField *dfield,
     return header;
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
