@@ -1,40 +1,24 @@
 /*
- *  $Id: spml.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
+ *  $Id: spml.c 24776 2022-04-26 15:48:39Z yeti-dn $
+ *  Copyright (C) 2006-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
-/* TODO: check/write allocation/deallocation
- *       change GArray to GPtrArray where possible */
-
-/* CHANGELOG:
- * 2006/08/09
- * * atof() replaced by g_ascii_strtod() and set_locale() removed
- * 2006/06/07 - version 0.1.4
- * * corrected problem with non-square datachannel rotation
- * 2006/05/18 - version 0.1.3
- * * Corrected width and length of datafiels (rounding problem)
- * * TODO: what happend when axis or dimensions are nonequal?
- * 2006/05/07 - version 0.1.2
- * * Fixed offset of datafield (datafield is starting on location of axes start)
- * * Rotation of datafield (consult the meaning of axis)
- * * FIXME: Correct width and length of datafiels (rounding problem)
+/* TODO: Rewrite to report errors via GError and abort instead of printing g_warnings() and then continuing, probably
+ * towards a crash!
  *
- */
+ * check/write allocation/deallocation
+ * change GArray to GPtrArray where possible */
 
 /* XXX: The typical length of a XML declaration is about 40 bytes.  So while
  * there can be more stuff before <SPML than 60 bytes, we have to find a
@@ -108,7 +92,7 @@ static GwyModuleInfo module_info = {
     N_("Loads SPML (Scanning Probe Microscopy Markup Language) "
        "data files."),
     "Jan Hořák <xhorak@gmail.com>",
-    "0.1.6",
+    "0.2",
     "Jan Hořák",
     "2006",
 };
@@ -159,20 +143,15 @@ get_list_of_datachannels(const gchar *filename)
                     if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT
                         && strcmp(name, "DataChannelGroup") == 0) {
                         /* datachannelgroup, get and set name of datachannelgroup
-                           data_channel_group is dynammicaly allocated,
-                           and it must be freed only when returned list is going to
-                           be disposed because GList does not create
-                           copy of data_channel_group. */
+                           data_channel_group is dynammicaly allocated, and it must be freed only when returned list
+                           is going to be disposed because GList does not create copy of data_channel_group. */
                         data_channel_group = g_new(dataChannelGroup, 1);
-                        data_channel_group->name =
-                            xmlTextReaderGetAttribute(reader, "name");
+                        data_channel_group->name = xmlTextReaderGetAttribute(reader, "name");
                         data_channel_group->datachannels = NULL;
-                        gwy_debug("Switch to datachannelGroup '%s'",
-                                  data_channel_group->name);
+                        gwy_debug("Switch to datachannelGroup '%s'", data_channel_group->name);
                         state = IN_DATACHANNELGROUP;
                     }
-                    else if (xmlTextReaderNodeType(reader) ==
-                             XML_READER_TYPE_END_ELEMENT
+                    else if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_END_ELEMENT
                              && strcmp(name, "DataChannels") == 0) {
                         /* after datachannels, return possible? */
                         gwy_debug("Datachannels end, read complete.");
@@ -184,15 +163,13 @@ get_list_of_datachannels(const gchar *filename)
                         && strcmp(name, "DataChannel") == 0
                         && data_channel_group != NULL) {
                         /* datachannelgroup, get and set name of datachannelgroup */
-                        data_channel_group->datachannels =
-                            g_list_append(data_channel_group->datachannels,
-                                          xmlTextReaderGetAttribute(reader,
-                                                                    "name"));
+                        data_channel_group->datachannels = g_list_append(data_channel_group->datachannels,
+                                                                         xmlTextReaderGetAttribute(reader,
+                                                                                                   "name"));
                         gwy_debug("Read info about datachannel.");
                         /* stay in current state */
                     }
-                    else if (xmlTextReaderNodeType(reader) ==
-                             XML_READER_TYPE_END_ELEMENT
+                    else if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_END_ELEMENT
                              && strcmp(name, "DataChannelGroup") == 0
                              && data_channel_group != NULL) {
                         l = g_list_append(l, data_channel_group);
@@ -213,98 +190,72 @@ get_list_of_datachannels(const gchar *filename)
         xmlFreeTextReader(reader);
     }
     else {
-        g_warning("SPML: Unable to open %s!", filename);
+        g_warning("Unable to open %s!", filename);
         return NULL;
     }
     return l;
 }
 
 static int
-decode_data(double **data, const xmlChar * input, dataFormat data_format,
-            codingTypes coding, byteOrder byte_order, int max_input_len)
+decode_data(double **data, const xmlChar * input, gint data_format,
+            codingTypes coding, GwyByteOrder byte_order, int max_input_len)
 {
-    GArray *data_stream, *debase64_buf, *decoded_data;
+    GArray *data_stream, *debase64_buf;
+    gdouble *decoded_data = NULL;
     char *p, *end_ptr;
-    unsigned int i;
-    double val;
     int data_count = 0;
 
     gwy_debug("start.");
 
+    *data = NULL;
     if (input == NULL) {
-        g_warning("SPML: decode_data(): NULL input");
-        *data = NULL;
+        g_warning("NULL input");
         return 0;
     }
-    switch (coding) {
-        case ZLIB_COMPR_BASE64:
-            /* XXX: Is input guaranteed to be nul-terminated? */
-            decode_b64((const gchar*)input, &debase64_buf);
-            if (inflate_dynamic_array(debase64_buf, &data_stream) != 0) {
-                g_warning("Cannot inflate compressed data.");
-                g_array_free(debase64_buf, TRUE);
-                if (data_stream != NULL) {
-                    g_array_free(data_stream, TRUE);
-                }
-                *data = NULL;
-                return 0;
-            }
-            g_array_free(debase64_buf, TRUE);
-            break;
-        case BASE64:
-            /* XXX: Is input guaranteed to be nul-terminated? */
-            decode_b64((const gchar*)input, &data_stream);
-            break;
-        case ASCII:
-            p = (char *)input;
-            data_stream = g_array_new(FALSE, FALSE, sizeof(double));
-            while (p != NULL) {
-                double num;
 
-                if (*p == ' ' || *p == '\n' || *p == '\r' || *p == '\t') {
-                    p++;
-                    continue;
-                }
-                num = g_ascii_strtod(p, &end_ptr);
-                if (num == 0 && end_ptr == p) {
-                    g_warning("SPML: decode_data(): No conversion performed "
-                              "from ASCII string.");
-                    break;
-                }
-                g_array_append_val(data_stream, num);
-                p = end_ptr;
-                data_count++;
-            }
-
-            break;
-            /*/ TODO: */
-        case HEX:
-        case BINARY:
-            g_warning("SPML: decode_data(): Data coding 'HEX' and 'BINARY' "
-                      "not supported.");
-            break;
-        case UNKNOWN_CODING:
-            break;
+    if (data_format == UNKNOWN_DATAFORMAT) {
+        g_warning("Unknown data format");
+        return 0;
     }
+    if (data_format == STRING) {
+        g_warning("Data format 'String' not supported.");
+        return 0;
+    }
+
     if (coding == ASCII) {
+        p = (char *)input;
+        data_stream = g_array_new(FALSE, FALSE, sizeof(double));
+        while (p != NULL) {
+            double num;
+
+            if (*p == ' ' || *p == '\n' || *p == '\r' || *p == '\t') {
+                p++;
+                continue;
+            }
+            num = g_ascii_strtod(p, &end_ptr);
+            if (num == 0 && end_ptr == p) {
+                g_warning("No conversion performed from ASCII string.");
+                break;
+            }
+            g_array_append_val(data_stream, num);
+            p = end_ptr;
+            data_count++;
+        }
+
         /* we have already decoded data */
         if (max_input_len != -1 && data_count != max_input_len) {
-            /* not enough input data to fill array defined by length
-             * max_input_len */
-            g_warning("SPML: decode_data():\n"
-                      "Input has not the same length as declared in "
-                        "dimensions\n"
+            /* not enough input data to fill array defined by length max_input_len */
+            g_warning("Input has not the same length as declared in dimensions\n"
                       "(max:%d vs read:%d). Has the channel attribute\n"
-                      "'channelReadMethodName'? The channel may be  one\n"
+                      "'channelReadMethodName'? The channel may be one\n"
                       "dimensional data used for axis values but not as\n"
                       "a source of data for Gwyddion.",
                       max_input_len, data_count);
             g_array_free(data_stream, TRUE);
-            *data = NULL;
             return 0;
         }
         else {
-            *data = (double *)data_stream->data;
+            *data = (double*)data_stream->data;
             /* we can free dynamic array, but not */
             g_array_free(data_stream, FALSE);
             /* containing data. */
@@ -312,94 +263,48 @@ decode_data(double **data, const xmlChar * input, dataFormat data_format,
             return data_count;
         }
     }
-    decoded_data = g_array_new(FALSE, FALSE, sizeof(double));
-    p = data_stream->data;
-    i = 0;
-    switch (data_format) {
-        case FLOAT32:
-            while (i < data_stream->len) {
-                val = get_float32(&p, byte_order);
-                g_array_append_val(decoded_data, val);
-                data_count++;
-                i += sizeof(float);
+
+    if (coding == ZLIB_COMPR_BASE64) {
+        /* XXX: Is input guaranteed to be nul-terminated? */
+        decode_b64((const gchar*)input, &debase64_buf);
+        if (inflate_dynamic_array(debase64_buf, &data_stream) != 0) {
+            g_warning("Cannot inflate compressed data.");
+            g_array_free(debase64_buf, TRUE);
+            if (data_stream) {
+                g_array_free(data_stream, TRUE);
             }
-            break;
-        case FLOAT64:
-            while (i < data_stream->len) {
-                val = get_float64(&p, byte_order);
-                g_array_append_val(decoded_data, val);
-                i += sizeof(double);
-            }
-            break;
-        case INT8:
-            while (i < data_stream->len) {
-                val = get_int8(&p);
-                g_array_append_val(decoded_data, val);
-                i += sizeof(gint8);
-            }
-            break;
-        case INT16:
-            while (i < data_stream->len) {
-                val = get_int16(&p, byte_order);
-                g_array_append_val(decoded_data, val);
-                i += sizeof(gint16);
-            }
-            break;
-        case INT32:
-            while (i < data_stream->len) {
-                val = get_int32(&p, byte_order);
-                g_array_append_val(decoded_data, val);
-                i += sizeof(gint32);
-            }
-            break;
-        case UINT32:
-            while (i < data_stream->len) {
-                val = get_uint32(&p, byte_order);
-                g_array_append_val(decoded_data, val);
-                i += sizeof(guint32);
-            }
-            break;
-        case UINT8:
-            while (i < data_stream->len) {
-                val = get_uint8(&p);
-                g_array_append_val(decoded_data, val);
-                i += sizeof(guint8);
-            }
-            break;
-        case UINT16:
-            while (i < data_stream->len) {
-                val = get_uint16(&p, byte_order);
-                g_array_append_val(decoded_data, val);
-                i += sizeof(guint16);
-            }
-            break;
-        case STRING:
-            g_warning
-                ("SPML: decode_data(): Data format 'String' not supported.");
-            break;
-        case UNKNOWN_DATAFORMAT:
-            g_warning("SPML: decode_data(): Unknown dataformat.");
-            break;
+            return 0;
+        }
+        g_array_free(debase64_buf, TRUE);
     }
+    else if (coding == BASE64) {
+        /* XXX: Is input guaranteed to be nul-terminated? */
+        decode_b64((const gchar*)input, &data_stream);
+    }
+    else {
+        g_warning("Data coding not supported.");
+        return 0;
+    }
+
+    decoded_data = g_new(gdouble, data_stream->len);
+    data_count = data_stream->len/gwy_raw_data_size(data_format);
+    gwy_convert_raw_data(data_stream->data, data_count, 1, data_format, byte_order, decoded_data, 1.0, 0.0);
+
     g_array_free(data_stream, TRUE);
-    data_count = decoded_data->len;
     if (max_input_len != -1 && data_count != max_input_len) {
-        g_warning("SPML: decode_data():\n"
-                  "Input has not the same length as declared in dimensions\n"
+        g_warning("Input has not the same length as declared in dimensions\n"
                   "(max:%d vs read:%d). Has the channel attribute\n"
-                  "'channelReadMethodName'? The channel may be  one\n"
+                  "'channelReadMethodName'? The channel may be one\n"
                   "dimensional data used for axis values but not as\n"
-                  "a source of data for Gwyddion.", max_input_len, data_count);
-        g_array_free(decoded_data, TRUE);
+                  "a source of data for Gwyddion.",
+                  max_input_len, data_count);
+        g_free(decoded_data);
         *data = NULL;
         return 0;
     }
-    *data = (double *)decoded_data->data;
-    g_array_free(decoded_data, FALSE);  /* we can free dynamic array, but not */
-    /* containing data. */
+    *data = decoded_data;
     gwy_debug("Datacount: %d", data_count);
     return data_count;
-
 }
 
 static int
@@ -407,9 +312,9 @@ get_data(gboolean read_data_only, char *filename, char *datachannel_name,
          double *data[], int *dimensions[], char **unit,
          G_GNUC_UNUSED gboolean *scattered)
 {
-    dataFormat format = UNKNOWN_DATAFORMAT;
+    gint format = UNKNOWN_DATAFORMAT;
     codingTypes coding = UNKNOWN_CODING;
-    byteOrder byte_order = UNKNOWN_BYTEORDER;
+    GwyByteOrder byte_order = GWY_BYTE_ORDER_NATIVE;
     int ret, i;
     int state = 0;
     GArray *axes, *names, *units, *axis;
@@ -420,8 +325,7 @@ get_data(gboolean read_data_only, char *filename, char *datachannel_name,
     int out_len;
     int max_input_len = 1;
 
-    gwy_debug("read only: %d,  channel name==%s", read_data_only,
-              datachannel_name);
+    gwy_debug("read only: %d, channel name==%s", read_data_only, datachannel_name);
     *dimensions = NULL;
 
     reader = xmlReaderForFile(filename, NULL, 0);
@@ -440,22 +344,14 @@ get_data(gboolean read_data_only, char *filename, char *datachannel_name,
                 && strcmp(name, "DataChannel") == 0) {
                 id = xmlTextReaderGetAttribute(reader, "name");
                 if (id && strcmp(id, datachannel_name) == 0) {
-                    format =
-                        get_data_format(xmlTextReaderGetAttribute
-                                        (reader, "dataFormat"));
-                    coding =
-                        get_data_coding(xmlTextReaderGetAttribute
-                                        (reader, "coding"));
-                    byte_order =
-                        get_byteorder(xmlTextReaderGetAttribute
-                                      (reader, "byteOrder"));
+                    format = get_data_format(xmlTextReaderGetAttribute(reader, "dataFormat"));
+                    coding = get_data_coding(xmlTextReaderGetAttribute(reader, "coding"));
+                    byte_order = get_byteorder(xmlTextReaderGetAttribute(reader, "byteOrder"));
                     if (read_data_only == FALSE) {
                         /* when read_data_only is true, we skip dimensions array to prevent
                            infinite cycle when calling from get_axis to get non equidistant axis from
                            datachannel */
-                        read_method_name =
-                            xmlTextReaderGetAttribute(reader,
-                                                      "channelReadMethodName");
+                        read_method_name = xmlTextReaderGetAttribute(reader, "channelReadMethodName");
                         if (read_method_name == NULL) {
                             /* sequence of one dimensional values */
                             *dimensions = NULL;
@@ -463,13 +359,9 @@ get_data(gboolean read_data_only, char *filename, char *datachannel_name,
                         else {
                             g_free((char *)read_method_name);
                             /* get axes */
-                            data_dimension =
-                                get_axis(filename, datachannel_name, &axes,
-                                         &names, &units);
+                            data_dimension = get_axis(filename, datachannel_name, &axes, &names, &units);
                             if (data_dimension != 2) {
-                                g_warning
-                                    ("SPML: get_data(): Input data are in %d dimension(s).",
-                                     data_dimension);
+                                g_warning("Input data are in %d dimension(s).", data_dimension);
                             }
                             /* one dimension array */
                             if (data_dimension > 0) {
@@ -485,10 +377,8 @@ get_data(gboolean read_data_only, char *filename, char *datachannel_name,
                             }
                             /* free what get_axis allocated */
                             if (axes) {
-                                for (i = 0; i < axes->len; i++) {
-                                    g_array_free(g_array_index
-                                                 (axes, GArray *, i), TRUE);
-                                }
+                                for (i = 0; i < axes->len; i++)
+                                    g_array_free(g_array_index(axes, GArray *, i), TRUE);
                                 g_array_free(axes, TRUE);
                             }
                             if (names)
@@ -504,9 +394,7 @@ get_data(gboolean read_data_only, char *filename, char *datachannel_name,
 
                     *unit = xmlTextReaderGetAttribute(reader, "unit");
 
-                    gwy_debug
-                        ("Datachannel summary: format %d, coding %d, order %d.",
-                         format, coding, byte_order);
+                    gwy_debug("Datachannel summary: format %d, coding %d, order %d.", format, coding, byte_order);
                     state = 1;
                 }
                 if (id) {
@@ -516,17 +404,13 @@ get_data(gboolean read_data_only, char *filename, char *datachannel_name,
             else if (state == 1) {
                 value = xmlTextReaderConstValue(reader);
                 if (value == NULL) {
-                    g_warning("SPML: get_data(): No data available for "
-                              "datachannel '%s'", datachannel_name);
+                    g_warning("No data available for datachannel '%s'", datachannel_name);
                     *data = NULL;
                     break;
                 }
-                out_len =
-                    decode_data(data, value, format, coding, byte_order,
-                                max_input_len);
+                out_len = decode_data(data, value, format, coding, byte_order, max_input_len);
                 if (out_len == 0 || *data == NULL) {
-                    g_warning("SPML: get_data(): No input data available for"
-                              " datachannel '%s'.", datachannel_name);
+                    g_warning("No input data available for datachannel '%s'.", datachannel_name);
 
                 }
                 else if (read_data_only == TRUE) {
@@ -646,14 +530,13 @@ get_axis_datapoints(char *filename, xmlNodePtr axis_node)
     if (ch) {
         /* points are saved in datachannel */
         /* TODO: load points from datachannel */
-        num_of_dimensions =
-            get_data(TRUE, filename, ch, &data, &dimensions, &unit, &scattered);
+        num_of_dimensions = get_data(TRUE, filename, ch, &data, &dimensions, &unit, &scattered);
         if (num_of_dimensions == 1) {
             axes_values = g_array_new(TRUE, FALSE, sizeof(double));
             axes_values = g_array_append_vals(axes_values, data, dimensions[0]);
             return axes_values;
         }
-        g_warning("SPML: Loading scattered data.");
+        g_warning("Loading scattered data.");
         return NULL;
     }
     else {
@@ -710,7 +593,7 @@ get_axis(char *filename, char *datachannel_name, GArray ** axes,
     gwy_debug("datachannel name == %s", datachannel_name);
     doc = xmlParseFile(filename);
     if (doc == NULL) {
-        g_warning("SPML: get_axis(): Input file was not parsed successfully.");
+        g_warning("Input file was not parsed successfully.");
         *axes = NULL;
         *units = NULL;
         *names = NULL;
@@ -724,8 +607,7 @@ get_axis(char *filename, char *datachannel_name, GArray ** axes,
 
     if (axes_node == NULL || datachannels_node == NULL) {
         /* incomplete xml file */
-        g_warning("SPML: get_axis(): incomplete file, missing Axes or "
-                  "Datachannels tags.");
+        g_warning("incomplete file, missing Axes or Datachannels tags.");
         xmlFreeDoc(doc);
         *axes = NULL;
         *units = NULL;
@@ -738,9 +620,8 @@ get_axis(char *filename, char *datachannel_name, GArray ** axes,
     while (cur != NULL) {
         sub_node = get_node_ptr(cur, "DataChannel");
         while (sub_node != NULL) {
-            channelReadMethodName =
-                get_attribute_value_of_named_node(sub_node, datachannel_name,
-                                                  "channelReadMethodName");
+            channelReadMethodName = get_attribute_value_of_named_node(sub_node, datachannel_name,
+                                                                      "channelReadMethodName");
             if (channelReadMethodName) {
                 /* channelReadMethodName found, leave searching */
                 break;
@@ -756,7 +637,7 @@ get_axis(char *filename, char *datachannel_name, GArray ** axes,
     }
 
     if (channelReadMethodName == NULL) {
-        g_warning("SPML: get_axis(): Datachannel '%s' not found.",
+        g_warning("Datachannel '%s' not found.",
                   datachannel_name);
         xmlFreeDoc(doc);
         *axes = NULL;
@@ -776,11 +657,8 @@ get_axis(char *filename, char *datachannel_name, GArray ** axes,
                 sub_node = get_node_ptr(cur, "ReadAxis");
                 while (sub_node != NULL) {
                     read_method_name = xmlGetProp(sub_node, "name");
-                    if (read_method_name) {
-                        read_method_axes =
-                            g_array_append_val(read_method_axes,
-                                               read_method_name);
-                    }
+                    if (read_method_name)
+                        read_method_axes = g_array_append_val(read_method_axes, read_method_name);
                     sub_node = get_next_node_ptr(sub_node, "ReadAxis");
                 }
             }
@@ -791,8 +669,7 @@ get_axis(char *filename, char *datachannel_name, GArray ** axes,
     }
     if (g_array_index(read_method_axes, xmlChar *, 0) == NULL) {
         /* ReadMethod mentioned in selected DataChannel not found in ReadMethod section */
-        g_warning("SPML: get_axis(): ReadMethod '%s' for datachannel "
-                  "'%s' not found.",
+        g_warning("ReadMethod '%s' for datachannel '%s' not found.",
                   channelReadMethodName, datachannel_name);
         xmlFreeDoc(doc);
         *axes = NULL;
@@ -819,8 +696,7 @@ get_axis(char *filename, char *datachannel_name, GArray ** axes,
                       g_array_index(read_method_axes, xmlChar *, i)) != NULL;
                      i++) {
                     if (!xmlStrcmp(read_method_name, tmp_ch2)) {
-                        /* we found axis we are searching for
-                           add name to *names array */
+                        /* we found axis we are searching for add name to *names array */
                         tmp_ch = xmlGetProp(sub_node, "name");
                         g_array_append_val(*names, tmp_ch);
                         /* add units to *units array */
@@ -829,8 +705,7 @@ get_axis(char *filename, char *datachannel_name, GArray ** axes,
                             g_array_append_val(*units, tmp_ch);
                         }
                         else {
-                            g_warning
-                                ("SPML: get_axis(): unknown unit for axis.");
+                            g_warning("unknown unit for axis.");
                             tmp_ch = g_strdup("N/A");
                             g_array_append_val(*units, tmp_ch);
                         }
@@ -842,8 +717,7 @@ get_axis(char *filename, char *datachannel_name, GArray ** axes,
                             axis_count++;
                         }
                         else {
-                            g_warning
-                                ("SPML: get_axis(): Cannot compute or read axis data.");
+                            g_warning("Cannot compute or read axis data.");
                             /* g_array_free(*axes, TRUE); */
                             if (*units)
                                 g_array_free(*units, TRUE);
@@ -888,8 +762,7 @@ get_dimensions(GArray * axes)
     for (i = 0; i < axes->len; i++) {
         axis = g_array_index(axes, GArray *, i);
         if (axis->len > 1) {
-            width = (g_array_index(axis, double, 1) - g_array_index(axis, double, 0))
-                * axis->len;
+            width = (g_array_index(axis, double, 1) - g_array_index(axis, double, 0)) * axis->len;
             out_array = g_array_append_val(out_array, width);
             gwy_debug("Start: %e, step: %e, count: %d, width: %e",
                       g_array_index(axis, double, 0),
@@ -897,7 +770,8 @@ get_dimensions(GArray * axes)
                       axis->len,
                       width
                      );
-        } else {
+        }
+        else {
             g_array_free(out_array, TRUE);
             gwy_debug("Axis values count lesser than 2");
             return NULL;
@@ -950,15 +824,14 @@ spml_load(const gchar *filename)
             axes = axes_units = names = real_width_height = NULL;
 
             channel_name = sub_l->data;
-            gwy_debug("Channelgroup: %s, channelname: %s", dc_group->name,
-                      channel_name);
+            gwy_debug("Channelgroup: %s, channelname: %s", dc_group->name, channel_name);
             if (channel_name
                 /* get data and check it is 2 or more dimensional array */
-                && get_data(FALSE, (char *)filename, channel_name, &data,
-                            &dimensions, &unit, &scattered) >= 2 && data
+                && get_data(FALSE, (char *)filename, channel_name, &data, &dimensions, &unit, &scattered) >= 2
+                && data
                 /* get axes and check we get 2 or more axes */
-                && get_axis((char *)filename, channel_name, &axes, &axes_units,
-                            &names) >= 2 && axes
+                && get_axis((char *)filename, channel_name, &axes, &axes_units, &names) >= 2
+                && axes
                 /* get width and height of 2D acording to axes */
                 && (real_width_height = get_dimensions(axes)) != NULL
                 && real_width_height->len >= 2) {
@@ -967,26 +840,21 @@ spml_load(const gchar *filename)
                 y_dimen = g_array_index(real_width_height, double, 1);
 
                 /* parse unit and return power of 10 according to unit */
-                siunit = gwy_si_unit_new_parse(g_array_index(axes_units,
-                                                             xmlChar*, 0),
-                                               &x_power10);
+                siunit = gwy_si_unit_new_parse(g_array_index(axes_units, xmlChar*, 0), &x_power10);
                 g_object_unref(siunit);
 
-                siunit = gwy_si_unit_new_parse(g_array_index(axes_units,
-                                                             xmlChar*, 1),
-                                               &y_power10);
+                siunit = gwy_si_unit_new_parse(g_array_index(axes_units, xmlChar*, 1), &y_power10);
                 /* create and allocate datafield of given dimensions and given physical
                    dimensions */
                 if (rotate == 90) {
-                    dfield = gwy_data_field_new
-                                   (dimensions[1], dimensions[0],
-                                    y_dimen * pow10(y_power10),
-                                    x_dimen * pow10(x_power10), FALSE);
-                } else {
-                dfield = gwy_data_field_new
-                                   (dimensions[0], dimensions[1],
-                                    x_dimen * pow10(x_power10),
-                                    y_dimen * pow10(y_power10), FALSE);
+                    dfield = gwy_data_field_new(dimensions[1], dimensions[0],
+                                                y_dimen * pow10(y_power10), x_dimen * pow10(x_power10),
+                                                FALSE);
+                }
+                else {
+                    dfield = gwy_data_field_new(dimensions[0], dimensions[1],
+                                                x_dimen * pow10(x_power10), y_dimen * pow10(y_power10),
+                                                FALSE);
                 }
                 gwy_debug("X real width: %f", x_dimen * pow10(x_power10));
                 gwy_debug("X real_width_height: %f", x_dimen);
@@ -1033,21 +901,17 @@ spml_load(const gchar *filename)
                     object = gwy_container_new();
                 }
                 /* put datachannel into container */
-                gwy_channel_location = g_strdup_printf("/%i/data",
-                                                       channel_number);
-                gwy_container_set_object_by_name(object, gwy_channel_location,
-                                                 (GObject *)dfield);
+                gwy_channel_location = g_strdup_printf("/%i/data", channel_number);
+                gwy_container_set_object_by_name(object, gwy_channel_location, (GObject *)dfield);
 
                 /* set name of datachannel to store in container */
                 channel_key = g_strdup_printf("%s/title", gwy_channel_location);
-                gwy_container_set_string_by_name(object, channel_key,
-                                                 g_strdup(channel_name));
+                gwy_container_set_string_by_name(object, channel_key, g_strdup(channel_name));
                 g_free(channel_key);
                 g_free(gwy_channel_location);
                 g_object_unref(dfield);
 
-                gwy_file_channel_import_log_add(object, channel_number,
-                                                NULL, filename);
+                gwy_file_channel_import_log_add(object, channel_number, NULL, filename);
                 channel_number++;
             }
             /* Free possibly allocated memory */
@@ -1110,4 +974,4 @@ spml_detect(const GwyFileDetectInfo * fileinfo, gboolean only_name)
     return score;
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
