@@ -1,6 +1,6 @@
 /*
- *  $Id: surffile.c 21395 2018-09-06 15:49:20Z yeti-dn $
- *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
+ *  $Id: surffile.c 24038 2021-08-21 11:18:28Z yeti-dn $
+ *  Copyright (C) 2005-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -52,8 +52,6 @@
 #include <app/gwyapp.h>
 #include <glib/gstdio.h>
 #include <stdlib.h>
-
-
 #include "get.h"
 #include "err.h"
 
@@ -356,6 +354,7 @@ surffile_load(const gchar *filename,
         return NULL;
     }
 
+    gwy_clear(&surffile, 1);
     p = buffer;
 
     get_CHARARRAY(signature, &p);
@@ -527,8 +526,8 @@ surffile_load(const gchar *filename,
         break;
     }
 
-    gwy_data_field_multiply(surffile.dfield, pow10(coef));
-    surffile.ZOffset *=  pow10(coef);
+    gwy_data_field_multiply(surffile.dfield, pow10(coef)*surffile.dz);
+    surffile.ZOffset *= pow10(coef);
     gwy_data_field_add(surffile.dfield, surffile.ZOffset);
 
     container = gwy_container_new();
@@ -550,38 +549,34 @@ fill_data_fields(SurfFile *surffile,
                  const guchar *buffer,
                  GError **error)
 {
+    GwyRawDataType datatype;
+    GwyDataField *field;
     gdouble *data;
+    guint n;
 
-    surffile->dfield = gwy_data_field_new(surffile->xres,
-                                          surffile->yres,
-                                          surffile->xres*surffile->dx,
-                                          surffile->yres*surffile->dy,
-                                          FALSE);
+    n = surffile->xres*surffile->yres;
+    surffile->dfield = field = gwy_data_field_new(surffile->xres,
+                                                  surffile->yres,
+                                                  surffile->xres*surffile->dx,
+                                                  surffile->yres*surffile->dy,
+                                                  FALSE);
 
-    data = gwy_data_field_get_data(surffile->dfield);
-    switch (surffile->pointsize) {
-        case 16:
-        gwy_convert_raw_data(buffer, surffile->xres*surffile->yres, 1,
-                             GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                             data, surffile->dz, 0.0);
-        break;
-
-        case 32:
-        gwy_convert_raw_data(buffer, surffile->xres*surffile->yres, 1,
-                             GWY_RAW_DATA_SINT32, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                             data, surffile->dz, 0.0);
-        break;
-
-        default:
+    if (surffile->pointsize == 16)
+        datatype = GWY_RAW_DATA_SINT16;
+    else if (surffile->pointsize == 32)
+        datatype = GWY_RAW_DATA_SINT32;
+    else {
         err_BPP(error, surffile->pointsize);
         return FALSE;
-        break;
     }
+    data = gwy_data_field_get_data(field);
+    gwy_convert_raw_data(buffer, n, 1, datatype, GWY_BYTE_ORDER_LITTLE_ENDIAN,
+                         data, 1.0, 0.0);
 
-    gwy_data_field_set_si_unit_xy(surffile->dfield, surffile->xyunit);
+    gwy_data_field_set_si_unit_xy(field, surffile->xyunit);
     g_object_unref(surffile->xyunit);
 
-    gwy_data_field_set_si_unit_z(surffile->dfield, surffile->zunit);
+    gwy_data_field_set_si_unit_z(field, surffile->zunit);
     g_object_unref(surffile->zunit);
 
     return TRUE;
