@@ -1,5 +1,5 @@
 /*
- *  $Id: unisoku.c 22642 2019-11-03 11:46:07Z yeti-dn $
+ *  $Id: unisoku.c 23866 2021-06-16 19:17:41Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -144,7 +144,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Unisoku data files (two-part .hdr + .dat)."),
     "Yeti <yeti@gwyddion.net>",
-    "0.9",
+    "0.10",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -342,9 +342,11 @@ unisoku_read_header(gchar *buffer,
     ufile->unit_x = g_strdup(line);
 
     NEXT(buffer, line, error);
+    /* The log flags seem to be missing occasionally.  Do not abort when it
+     * happens. */
     if (unisoku_sscanf(line, "ddi",
                        &ufile->start_x, &ufile->end_x,
-                       &ufile->log_flag_x) != 3) {
+                       &ufile->log_flag_x) < 2) {
         err_INVALID(error, _("x scale parameters"));
         return FALSE;
     }
@@ -355,7 +357,7 @@ unisoku_read_header(gchar *buffer,
     NEXT(buffer, line, error);
     if (unisoku_sscanf(line, "ddii",
                        &ufile->start_y, &ufile->end_y,
-                       &ufile->ineq_flag, &ufile->log_flag_y) != 4) {
+                       &ufile->ineq_flag, &ufile->log_flag_y) < 3) {
         err_INVALID(error, _("y scale parameters"));
         return FALSE;
     }
@@ -379,7 +381,7 @@ unisoku_read_header(gchar *buffer,
     if (unisoku_sscanf(line, "ddddi",
                        &ufile->max_raw_z, &ufile->min_raw_z,
                        &ufile->max_z, &ufile->min_z,
-                       &ufile->log_flag_z) != 5) {
+                       &ufile->log_flag_z) < 4) {
         err_INVALID(error, _("z scale parameters"));
         return FALSE;
     }
