@@ -1,5 +1,5 @@
 /*
- *  $Id: win_stm.c 22625 2019-10-30 09:12:55Z yeti-dn $
+ *  $Id: win_stm.c 23822 2021-06-10 06:58:57Z yeti-dn $
  *  Copyright (C) 2014 Jeffrey J. Schwartz.
  *  E-mail: schwartz@physics.ucla.edu
  *
@@ -90,7 +90,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports WinSTM (.stm) files."),
     "Jeffrey J. Schwartz <schwartz@physics.ucla.edu>",
-    "0.5",
+    "0.6",
     "Jeffrey J. Schwartz",
     "April 2014",
 };
@@ -213,7 +213,7 @@ winSTM_load(const gchar *filename,
         g_snprintf(key, sizeof(key), "/%i/data", v);
         gwy_container_set_object_by_name(container, key, dfield);
         g_snprintf(key, sizeof(key), "/%i/data/title", v);
-        s = g_convert(Title, -1, "iso-8859-1", "utf-8", NULL, NULL, NULL);
+        s = g_convert(Title, -1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
         gwy_container_set_string_by_name(container, key, (guchar *)s);
         meta = gwy_container_new();
         gwy_container_set_string_by_name(meta, "Bias",
