@@ -1,6 +1,6 @@
 /*
- *  $Id: wrustfile.c 24453 2021-11-03 12:54:29Z yeti-dn $
- *  Copyright (C) 2021 David Necas (Yeti).
+ *  $Id: wrustfile.c 24756 2022-04-05 11:16:24Z yeti-dn $
+ *  Copyright (C) 2021-2022 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -67,7 +67,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports AFM files from Department of Nanometrology, WRUST."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "1.2",
     "David Nečas (Yeti)",
     "2021",
 };
@@ -129,7 +129,7 @@ dat_load(const gchar *filename,
     GwyContainer *container = NULL, *meta = NULL;
     GwyDataField *dfield = NULL;
     GwySIUnit *xunit = NULL, *yunit = NULL, *actzunit = NULL, *amplzunit = NULL, *voltunit = NULL;
-    gchar *line, *p, *value, *key, *sens, *title, *buffer = NULL;
+    gchar *line, *p, *s, *value, *key, *sens, *title, *buffer = NULL;
     GHashTable *hash = NULL;
     GRegex *regex;
     GMatchInfo *info = NULL;
@@ -155,9 +155,13 @@ dat_load(const gchar *filename,
             err_TRUNCATED_HEADER(error);
             goto fail;
         }
-
+        g_strstrip(line);
         len = strlen(line);
-        if (len < 2 || line[0] != '[' || line[len-1] != ']') {
+        /* Skip empty lines.  Apparently they can occur in some files. */
+        if (!len)
+            continue;
+
+        if (line[0] != '[' || line[len-1] != ']') {
             g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                         _("Invalid file header."));
             goto fail;
@@ -200,7 +204,10 @@ dat_load(const gchar *filename,
                       NULL))
         goto fail;
 
+    /* Older files can have just Liczba Linii. */
     xres = yres = atoi(g_hash_table_lookup(hash, "Liczba Linii"));
+    if ((s = g_hash_table_lookup(hash, "Liczba Kolumn")))
+        xres = atoi(s);
     if (err_DIMENSION(error, xres) || err_DIMENSION(error, yres))
         goto fail;
 
@@ -208,11 +215,6 @@ dat_load(const gchar *filename,
      * which is given like 1234 um/100V, which we do by multiplying by 100 back.  Together they give ×100/10 = ×10 */
     xreal = xscale * 10.0 * g_ascii_strtod(g_hash_table_lookup(hash, "RasterX"), NULL);
     yreal = yscale * 10.0 * g_ascii_strtod(g_hash_table_lookup(hash, "RasterY"), NULL);
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-
-    if (!gwy_si_unit_equal(yunit, xunit))
-        g_warning("X and Y units differ, using X");
-    gwy_si_unit_multiply(xunit, voltunit, gwy_data_field_get_si_unit_xy(dfield));
     /* Use negated positive conditions to catch NaNs */
     if (!((xreal = fabs(xreal)) > 0)) {
         g_warning("Real x size is 0.0, fixing to 1.0");
@@ -223,6 +225,12 @@ dat_load(const gchar *filename,
         yreal = 1.0;
     }
 
+    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
+
+    if (!gwy_si_unit_equal(yunit, xunit))
+        g_warning("X and Y units differ, using X");
+    gwy_si_unit_multiply(xunit, voltunit, gwy_data_field_get_si_unit_xy(dfield));
+
     q = amplz * actzsens;
     gwy_si_unit_multiply(gwy_si_unit_multiply(amplzunit, voltunit, amplzunit), actzunit,
                          gwy_data_field_get_si_unit_z(dfield));
