@@ -1,5 +1,5 @@
 /*
- *  $Id: wsffile.c 22662 2019-11-09 23:20:09Z yeti-dn $
+ *  $Id: wsffile.c 22967 2020-11-26 10:58:28Z yeti-dn $
  *  Copyright (C) 2011-2019 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,11 @@
  *         <match type="string" offset="0:240" value="\nLines in Y:"/>
  *     </match>
  *   </magic>
+ *   <magic priority="80">
+ *     <match type="string" offset="0:160" value="\rPixels in X:">
+ *         <match type="string" offset="0:240" value="\rLines in Y:"/>
+ *     </match>
+ *   </magic>
  *   <glob pattern="*.wsf"/>
  *   <glob pattern="*.WSF"/>
  * </mime-type>
@@ -64,9 +69,9 @@
 #define Micro (1e-6)
 #define Nano (1e-9)
 
-#define MAGIC1 "\nPixels in X:"
+#define MAGIC1 "Pixels in X:"
 #define MAGIC1_SIZE (sizeof(MAGIC1)-1)
-#define MAGIC2 "\nLines in Y:"
+#define MAGIC2 "Lines in Y:"
 #define MAGIC2_SIZE (sizeof(MAGIC2)-1)
 #define EXTENSION ".wsf"
 
@@ -83,7 +88,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports WSF ASCII files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.5",
+    "0.6",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -107,13 +112,21 @@ static gint
 wsf_detect(const GwyFileDetectInfo *fileinfo,
            gboolean only_name)
 {
+    const guchar *p;
+
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
 
-    if (strstr(fileinfo->head, MAGIC1) && strstr(fileinfo->head, MAGIC2))
-        return 100;
+    if (!(p = strstr(fileinfo->head, MAGIC1))
+        || p == fileinfo->head
+        || (*(p-1) != '\n' && *(p-1) != '\r'))
+        return 0;
+    if (!(p = strstr(fileinfo->head, MAGIC2))
+        || p == fileinfo->head
+        || (*(p-1) != '\n' && *(p-1) != '\r'))
+        return 0;
 
-    return 0;
+    return 100;
 }
 
 static GwyContainer*
@@ -150,7 +163,9 @@ wsf_load(const gchar *filename,
     while (g_ascii_isspace(*p))
         p++;
     // Then there's the header, followed by empty lines again.
-    if (!(header_end = strstr(p, "\r\n\r\n"))) {
+    if (!(header_end = strstr(p, "\r\n\r\n"))
+        && !(header_end = strstr(p, "\n\n"))
+        && !(header_end = strstr(p, "\r\r"))) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                     _("File header does not end with an empty line."));
         goto fail;
