@@ -1,22 +1,18 @@
 /*
- *  $Id: wsxmfile.c 22981 2020-11-30 15:32:03Z yeti-dn $
- *  Copyright (C) 2005-2020 David Necas (Yeti), Petr Klapetek.
+ *  $Id: wsxmfile.c 24544 2021-11-25 14:39:33Z yeti-dn $
+ *  Copyright (C) 2005-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -120,7 +116,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Nanotec WSxM data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.20",
+    "0.21",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -176,8 +172,7 @@ wsxmfile_check_magic(const char *head)
         return NULL;
 
     slen = strlen(imgstring);
-    if (strncmp(rest, imgstring, slen) == 0
-        && (rest = skip_to_next_line(rest + slen)))
+    if (strncmp(rest, imgstring, slen) == 0 && (rest = skip_to_next_line(rest + slen)))
         return rest;
 
     return NULL;
@@ -195,8 +190,7 @@ wsxmfile_detect(const GwyFileDetectInfo *fileinfo,
                 || g_str_has_suffix(fileinfo->name_lowercase, ".top")
                 ? 20 : 0);
 
-    if (fileinfo->buffer_len > MAGIC_SIZE
-        && wsxmfile_check_magic(fileinfo->head))
+    if (fileinfo->buffer_len > MAGIC_SIZE && wsxmfile_check_magic(fileinfo->head))
         score = 100;
 
     return score;
@@ -227,8 +221,7 @@ wsxmcurve_check_magic(const char *head, const gchar **which)
     if (i == G_N_ELEMENTS(curvenames))
         return NULL;
     cflen = strlen(cfstring);
-    if (strncmp(rest + len, cfstring, cflen) == 0
-        && (rest = skip_to_next_line(rest + len + cflen))) {
+    if (strncmp(rest + len, cfstring, cflen) == 0 && (rest = skip_to_next_line(rest + len + cflen))) {
         if (which)
             *which = curvenames[i];
         return rest;
@@ -246,8 +239,7 @@ wsxmcurve_detect(const GwyFileDetectInfo *fileinfo,
     if (only_name)
         return (g_str_has_suffix(fileinfo->name_lowercase, ".cur") ? 20 : 0);
 
-    if (fileinfo->buffer_len > MAGIC_SIZE
-        && wsxmcurve_check_magic(fileinfo->head, NULL))
+    if (fileinfo->buffer_len > MAGIC_SIZE && wsxmcurve_check_magic(fileinfo->head, NULL))
         score = 100;
 
     return score;
@@ -279,9 +271,7 @@ convert_to_utf8(G_GNUC_UNUSED const GwyTextHeaderContext *context,
                 G_GNUC_UNUSED gpointer user_data,
                 G_GNUC_UNUSED GError **error)
 {
-    g_hash_table_replace(hash, key,
-                         g_convert(value, strlen(value), "UTF-8", "ISO-8859-1",
-                                   NULL, NULL, NULL));
+    g_hash_table_replace(hash, key, g_convert(value, strlen(value), "UTF-8", "ISO-8859-1", NULL, NULL, NULL));
     return TRUE;
 }
 
@@ -313,8 +303,7 @@ parse_header_common(guchar *buffer, gsize size,
     p = gwy_memmem(buffer, size, HEADER_END, sizeof(HEADER_END)-1);
     if (!p) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Expected header end marker ‘%s’ was not found."),
-                    HEADER_END);
+                    _("Expected header end marker ‘%s’ was not found."), HEADER_END);
         gwy_file_abandon_contents(buffer, size, NULL);
         return NULL;
     }
@@ -365,15 +354,11 @@ wsxmfile_load(const gchar *filename,
         gwy_file_abandon_contents(buffer, size, NULL);
         return NULL;
     }
-    if (!(meta = parse_header_common(buffer, size, rest, "WSxM",
-                                     &header, &header_size,
-                                     error)))
+    if (!(meta = parse_header_common(buffer, size, rest, "WSxM", &header, &header_size, error)))
         return NULL;
 
-    xres = read_positive_integer(meta, "General Info::Number of columns",
-                                 &ok, error);
-    yres = read_positive_integer(meta, "General Info::Number of rows",
-                                 &ok, error);
+    xres = read_positive_integer(meta, "General Info::Number of columns", &ok, error);
+    yres = read_positive_integer(meta, "General Info::Number of rows", &ok, error);
 
     if (ok
         && (p = g_hash_table_lookup(meta, "General Info::Image Data Type"))) {
@@ -384,24 +369,21 @@ wsxmfile_load(const gchar *filename,
         else if (gwy_stramong(p, "integer", "short", NULL))
             type = GWY_RAW_DATA_SINT16;
         else {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                         _("Unknown data type `%s'."), p);
             ok = FALSE;
         }
     }
 
     if (ok)
-        ok = !err_SIZE_MISMATCH(error, 2*xres*yres, (guint)size - header_size,
-                                FALSE);
+        ok = !err_SIZE_MISMATCH(error, 2*xres*yres, (guint)size - header_size, FALSE);
     if (ok)
         dfield = read_data_field(buffer + header_size, xres, yres, type);
     gwy_file_abandon_contents(buffer, size, NULL);
 
     if (dfield) {
         container = gwy_container_new();
-        gwy_container_set_object(container,
-                                 gwy_app_get_data_key_for_id(0), dfield);
+        gwy_container_set_object(container, gwy_app_get_data_key_for_id(0), dfield);
         g_object_unref(dfield);
         process_metadata(meta, container);
 
@@ -423,6 +405,7 @@ wsxmcurve_load(const gchar *filename,
 {
     GwyContainer *container = NULL;
     guchar *buffer = NULL;
+    const gchar *xlabel = NULL, *ylabel = NULL, *xunitstr = NULL, *yunitstr = NULL;
     const gchar *which, *rest = NULL;
     gsize size = 0;
     GError *err = NULL;
@@ -431,8 +414,6 @@ wsxmcurve_load(const gchar *filename,
     guint header_size;
     gchar *header;
     gboolean ok = TRUE;
-    const gchar *xlabel = NULL, *ylabel = NULL;
-    const gchar *xunitstr = NULL, *yunitstr = NULL;
     guint ncurves, npts;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
@@ -446,23 +427,19 @@ wsxmcurve_load(const gchar *filename,
         gwy_file_abandon_contents(buffer, size, NULL);
         return NULL;
     }
-    if (!(meta = parse_header_common(buffer, size, rest, "WSxM curve",
-                                     &header, &header_size,
-                                     error)))
+    if (!(meta = parse_header_common(buffer, size, rest, "WSxM curve", &header, &header_size, error)))
         return NULL;
 
     /*
-    Number of lines: 2
-    Number of points: 512
-    X axis text: Bias [#x]
-    X axis unit: V
-    Y axis text: Amplitude [#y]
-    Y axis unit: V
-    */
-    ncurves = read_positive_integer(meta, "General Info::Number of lines",
-                                    &ok, error);
-    npts = read_positive_integer(meta, "General Info::Number of points",
-                                 &ok, error);
+     * Number of lines: 2
+     * Number of points: 512
+     * X axis text: Bias [#x]
+     * X axis unit: V
+     * Y axis text: Amplitude [#y]
+     * Y axis unit: V
+     */
+    ncurves = read_positive_integer(meta, "General Info::Number of lines", &ok, error);
+    npts = read_positive_integer(meta, "General Info::Number of points", &ok, error);
 
     if (ok) {
         xlabel = g_hash_table_lookup(meta, "General Info::X axis text");
@@ -472,16 +449,14 @@ wsxmcurve_load(const gchar *filename,
     }
 
     if (ok) {
-        gmodel = read_graph_model(buffer + header_size, ncurves, npts,
-                                  xlabel, ylabel, xunitstr, yunitstr, which,
+        gmodel = read_graph_model(buffer + header_size, ncurves, npts, xlabel, ylabel, xunitstr, yunitstr, which,
                                   error);
     }
     gwy_file_abandon_contents(buffer, size, NULL);
 
     if (gmodel) {
         container = gwy_container_new();
-        gwy_container_set_object(container,
-                                 gwy_app_get_graph_key_for_id(1), gmodel);
+        gwy_container_set_object(container, gwy_app_get_graph_key_for_id(1), gmodel);
         g_object_unref(gmodel);
     }
     g_hash_table_destroy(meta);
@@ -500,20 +475,45 @@ store_meta(gpointer key,
     gwy_container_set_string_by_name(meta, key, g_strdup(value));
 }
 
+static gboolean
+get_double_value(GHashTable *hash, const gchar *name, gdouble *value, gchar **pend)
+{
+    const gchar *s;
+    gchar *end;
+
+    if (!(s = g_hash_table_lookup(hash, name))) {
+        g_warning("Missing %s", name);
+        return FALSE;
+    }
+
+    *value = g_ascii_strtod(s, &end);
+    if (end == s) {
+        g_warning("Invalid value of %s", name);
+        return FALSE;
+    }
+    if (*value <= 0.0) {
+        g_warning("Invalid value of %s", name);
+        return FALSE;
+    }
+    if (pend)
+        *pend = end;
+
+    return TRUE;
+}
+
 static void
 process_metadata(GHashTable *wsxmmeta,
                  GwyContainer *container)
 {
     const gchar *nometa[] = {
-        "General Info::Z Amplitude",
-        "Control::X Amplitude", "Control::Y Amplitude",
+        "General Info::Z Amplitude", "Control::X Amplitude", "Control::Y Amplitude",
         "General Info::Number of rows", "General Info::Number of columns",
     };
     GwyDataField *dfield;
     GwySIUnit *siunit;
     GwyContainer *meta;
     gdouble r;
-    gchar *p, *end;
+    gchar *end;
     gint power10;
     guint i;
     gdouble min, max;
@@ -523,23 +523,21 @@ process_metadata(GHashTable *wsxmmeta,
     dfield = GWY_DATA_FIELD(gwy_container_get_object(container, quark));
 
     /* Fix value scale */
-    if (!(p = g_hash_table_lookup(wsxmmeta, "General Info::Z Amplitude"))
-        || (r = g_ascii_strtod(p, &end)) <= 0
-        || end == p) {
-        g_warning("Missing or invalid Z Amplitude");
-        gwy_data_field_multiply(dfield, 1e-9);
-    }
-    else {
+    if (get_double_value(wsxmmeta, "General Info::Z Amplitude", &r, &end)) {
         /* import `arbitrary units' as unit-less */
         while (g_ascii_isspace(*end))
             end++;
         if (gwy_strequal(end, "a.u."))
             siunit = gwy_si_unit_new(NULL);
+        else if (gwy_strequal(end, "Pi")) {
+            siunit = gwy_si_unit_new(NULL);
+            r *= G_PI;
+        }
         else {
             siunit = gwy_si_unit_new_parse(end, &power10);
             r *= pow10(power10);
         }
-        gwy_data_field_set_si_unit_z(dfield, siunit);
+        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), siunit);
         g_object_unref(siunit);
 
         gwy_data_field_get_min_max(dfield, &min, &max);
@@ -547,32 +545,24 @@ process_metadata(GHashTable *wsxmmeta,
 
         gwy_app_channel_title_fall_back(container, 0);
     }
+    else
+        gwy_data_field_multiply(dfield, 1e-9);
 
     /* Fix lateral scale */
-    if (!(p = g_hash_table_lookup(wsxmmeta, "Control::X Amplitude"))
-        || (r = g_ascii_strtod(p, &end)) <= 0
-        || end == p) {
-        g_warning("Missing or invalid X Amplitude");
-    }
-    else {
+    if (get_double_value(wsxmmeta, "Control::X Amplitude", &r, NULL)) {
         siunit = gwy_si_unit_new_parse(end, &power10);
-        gwy_data_field_set_si_unit_xy(dfield, siunit);
+        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), siunit);
         g_object_unref(siunit);
-
         gwy_data_field_set_xreal(dfield, r*pow10(power10));
     }
 
-    if (!(p = g_hash_table_lookup(wsxmmeta, "Control::Y Amplitude"))
-        || (r = g_ascii_strtod(p, &end)) <= 0
-        || end == p) {
-        g_warning("Missing or invalid Y Amplitude");
-        gwy_data_field_set_yreal(dfield, gwy_data_field_get_xreal(dfield));
-    }
-    else {
+    if (get_double_value(wsxmmeta, "Control::Y Amplitude", &r, NULL)) {
         siunit = gwy_si_unit_new_parse(end, &power10);
         g_object_unref(siunit);
         gwy_data_field_set_yreal(dfield, r*pow10(power10));
     }
+    else
+        gwy_data_field_set_yreal(dfield, gwy_data_field_get_xreal(dfield));
 
     /* And store everything else as metadata */
     for (i = 0; i < G_N_ELEMENTS(nometa); i++)
@@ -589,16 +579,14 @@ process_metadata(GHashTable *wsxmmeta,
 
 static GwyDataField*
 read_data_field(const guchar *buffer,
-                gint xres,
-                gint yres,
+                gint xres, gint yres,
                 GwyRawDataType type)
 {
     GwyDataField *dfield;
 
     dfield = gwy_data_field_new(xres, yres, 1e-6, 1e-6, FALSE);
     /* The conversion is probably wrong for the SINT16 type. */
-    gwy_convert_raw_data(buffer, xres*yres, 1,
-                         type, GWY_BYTE_ORDER_LITTLE_ENDIAN,
+    gwy_convert_raw_data(buffer, xres*yres, 1, type, GWY_BYTE_ORDER_LITTLE_ENDIAN,
                          gwy_data_field_get_data(dfield), 1.0, 0.0);
     gwy_data_field_invert(dfield, TRUE, TRUE, FALSE);
 
@@ -678,8 +666,7 @@ read_graph_model(const guchar *p,
                      "color", gwy_graph_get_preset_color(j),
                      "description", str->str,
                      NULL);
-        gwy_graph_curve_model_set_data_interleaved(gcmodel,
-                                                   data + j*2*npts, npts);
+        gwy_graph_curve_model_set_data_interleaved(gcmodel, data + j*2*npts, npts);
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
     }
@@ -695,8 +682,7 @@ fail:
     g_object_unref(xunit);
     g_object_unref(yunit);
     g_free(data);
-    g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                _("File is truncated."));
+    g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA, _("File is truncated."));
 
     return NULL;
 }
@@ -763,10 +749,8 @@ wsxmfile_export_double(GwyContainer *data,
     yres = gwy_data_field_get_yres(dfield);
     gwy_data_field_get_min_max(dfield, &min, &max);
 
-    xyunit = gwy_si_unit_get_string(gwy_data_field_get_si_unit_xy(dfield),
-                                    GWY_SI_UNIT_FORMAT_PLAIN);
-    zunit = gwy_si_unit_get_string(gwy_data_field_get_si_unit_z(dfield),
-                                   GWY_SI_UNIT_FORMAT_PLAIN);
+    xyunit = gwy_si_unit_get_string(gwy_data_field_get_si_unit_xy(dfield), GWY_SI_UNIT_FORMAT_PLAIN);
+    zunit = gwy_si_unit_get_string(gwy_data_field_get_si_unit_z(dfield), GWY_SI_UNIT_FORMAT_PLAIN);
     title = gwy_app_get_data_field_title(data, id);
     header = g_strdup_printf(header_template,
                              gwy_data_field_get_xreal(dfield), xyunit,
@@ -798,8 +782,7 @@ wsxmfile_export_double(GwyContainer *data,
             *drow = *r;
 
         if (G_BYTE_ORDER == G_BIG_ENDIAN) {
-            gwy_memcpy_byte_swap((const guint8*)dbuf, bytes,
-                                 sizeof(gdouble), xres, sizeof(gdouble)-1);
+            gwy_memcpy_byte_swap((const guint8*)dbuf, bytes, sizeof(gdouble), xres, sizeof(gdouble)-1);
             written = fwrite(bytes, 8, xres, fh);
         }
         else
@@ -823,4 +806,4 @@ fail:
     return ok;
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
