@@ -22,10 +22,16 @@ GwyModuleInfo* _gwy_module_query__graph_export_vector(void);
 GwyModuleInfo* _gwy_module_query__graph_fdfit(void);
 GwyModuleInfo* _gwy_module_query__graph_filter(void);
 GwyModuleInfo* _gwy_module_query__graph_fit(void);
+GwyModuleInfo* _gwy_module_query__graph_flip(void);
+GwyModuleInfo* _gwy_module_query__graph_fztofd(void);
+GwyModuleInfo* _gwy_module_query__graph_invert(void);
 GwyModuleInfo* _gwy_module_query__graph_level(void);
 GwyModuleInfo* _gwy_module_query__graph_logscale(void);
 GwyModuleInfo* _gwy_module_query__graph_peaks(void);
+GwyModuleInfo* _gwy_module_query__graph_polylevel(void);
 GwyModuleInfo* _gwy_module_query__graph_sfuncs(void);
+GwyModuleInfo* _gwy_module_query__graph_simplemech(void);
+GwyModuleInfo* _gwy_module_query__graph_sinebg(void);
 GwyModuleInfo* _gwy_module_query__graph_stats(void);
 GwyModuleInfo* _gwy_module_query__graph_terraces(void);
 
@@ -40,10 +46,16 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__graph_fdfit, "graph_fdfit", },
   { _gwy_module_query__graph_filter, "graph_filter", },
   { _gwy_module_query__graph_fit, "graph_fit", },
+  { _gwy_module_query__graph_flip, "graph_flip", },
+  { _gwy_module_query__graph_fztofd, "graph_fztofd", },
+  { _gwy_module_query__graph_invert, "graph_invert", },
   { _gwy_module_query__graph_level, "graph_level", },
   { _gwy_module_query__graph_logscale, "graph_logscale", },
   { _gwy_module_query__graph_peaks, "graph_peaks", },
+  { _gwy_module_query__graph_polylevel, "graph_polylevel", },
   { _gwy_module_query__graph_sfuncs, "graph_sfuncs", },
+  { _gwy_module_query__graph_simplemech, "graph_simplemech", },
+  { _gwy_module_query__graph_sinebg, "graph_sinebg", },
   { _gwy_module_query__graph_stats, "graph_stats", },
   { _gwy_module_query__graph_terraces, "graph_terraces", },
   { NULL, NULL, },
