@@ -1,5 +1,5 @@
 /*
- *  $Id: graph_cut.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: graph_cut.c 24414 2021-10-22 16:05:29Z yeti-dn $
  *  Copyright (C) 2003-2016 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -86,7 +86,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Cut graph"),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "1.3",
+    "1.4",
     "David Nečas (Yeti) & Petr Klapetek",
     "2007",
 };
@@ -98,9 +98,9 @@ module_register(void)
 {
     gwy_graph_func_register("graph_cut",
                             (GwyGraphFunc)&cut,
-                            N_("/_Cut..."),
+                            N_("/_Basic Operations/_Cut..."),
                             GWY_STOCK_GRAPH_CUT,
-                            GWY_MENU_FLAG_GRAPH,
+                            GWY_MENU_FLAG_GRAPH_CURVE,
                             N_("Extract part of graph into new one"));
 
     return TRUE;
