@@ -1,5 +1,5 @@
 /*
- *  $Id: graph_fdfit.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: graph_fdfit.c 24083 2021-09-06 16:37:12Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -168,7 +168,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Fit force-distance data"),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "1.5",
+    "1.6",
     "David Nečas (Yeti) & Petr Klapetek",
     "2007",
 };
@@ -180,9 +180,9 @@ module_register(void)
 {
     gwy_graph_func_register("graph_fdfit",
                             (GwyGraphFunc)&fit,
-                            N_("/_Fit FD Curve..."),
+                            N_("/_Force Distance/_Fit FD Curve..."),
                             GWY_STOCK_GRAPH_FD,
-                            GWY_MENU_FLAG_GRAPH,
+                            GWY_MENU_FLAG_GRAPH_CURVE,
                             N_("Fit a force-distance curve"));
 
     return TRUE;
