@@ -1,5 +1,5 @@
 /*
- *  $Id: ellipse.c 22239 2019-07-13 19:19:07Z yeti-dn $
+ *  $Id: ellipse.c 24732 2022-03-23 20:01:21Z yeti-dn $
  *  Copyright (C) 2005-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -232,10 +232,10 @@ gwy_selection_ellipse_crop_object(GwySelection *selection,
     gdouble xy[OBJECT_SIZE];
 
     gwy_selection_get_object(selection, i, xy);
-    return (MIN(xy[0], xy[2]) >= minmax[0]
-            && MIN(xy[1], xy[3]) >= minmax[1]
-            && MAX(xy[0], xy[2]) <= minmax[2]
-            && MAX(xy[1], xy[3]) <= minmax[3]);
+    return (fmin(xy[0], xy[2]) >= minmax[0]
+            && fmin(xy[1], xy[3]) >= minmax[1]
+            && fmax(xy[0], xy[2]) <= minmax[2]
+            && fmax(xy[1], xy[3]) <= minmax[3]);
 }
 
 static void
