@@ -1,6 +1,6 @@
 /*
- *  $Id: projective.c 23112 2021-01-28 15:55:09Z yeti-dn $
- *  Copyright (C) 2021 David Necas (Yeti).
+ *  $Id: projective.c 24731 2022-03-23 20:01:03Z yeti-dn $
+ *  Copyright (C) 2021-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -84,6 +84,17 @@ struct _GwySelectionProjectiveClass {
 static gboolean module_register                     (void);
 static GType    gwy_layer_projective_get_type       (void)                       G_GNUC_CONST;
 static GType    gwy_selection_projective_get_type   (void)                       G_GNUC_CONST;
+static gboolean gwy_selection_projective_crop_object(GwySelection *selection,
+                                                     gint i,
+                                                     gpointer user_data);
+static void     gwy_selection_projective_crop       (GwySelection *selection,
+                                                     gdouble xmin,
+                                                     gdouble ymin,
+                                                     gdouble xmax,
+                                                     gdouble ymax);
+static void     gwy_selection_projective_move       (GwySelection *selection,
+                                                     gdouble vx,
+                                                     gdouble vy);
 static void     gwy_layer_projective_set_property   (GObject *object,
                                                      guint prop_id,
                                                      const GValue *value,
@@ -130,7 +141,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Layer allowing selection of a projective plane."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "1.1",
     "David Nečas (Yeti)",
     "2021",
 };
@@ -153,6 +164,8 @@ gwy_selection_projective_class_init(GwySelectionProjectiveClass *klass)
     GwySelectionClass *sel_class = GWY_SELECTION_CLASS(klass);
 
     sel_class->object_size = OBJECT_SIZE;
+    sel_class->crop = gwy_selection_projective_crop;
+    sel_class->move = gwy_selection_projective_move;
 }
 
 static void
@@ -203,6 +216,53 @@ gwy_selection_projective_init(GwySelectionProjective *selection)
     g_array_set_size(GWY_SELECTION(selection)->objects, OBJECT_SIZE);
 }
 
+static gboolean
+gwy_selection_projective_crop_object(GwySelection *selection,
+                                     gint i,
+                                     gpointer user_data)
+{
+    const gdouble *minmax = (const gdouble*)user_data;
+    gdouble xy[OBJECT_SIZE];
+
+    gwy_selection_get_object(selection, i, xy);
+    return (fmin(fmin(xy[0], xy[2]), fmin(xy[4], xy[6])) >= minmax[0]
+            && fmin(fmin(xy[1], xy[3]), fmin(xy[5], xy[7])) >= minmax[1]
+            && fmax(fmax(xy[0], xy[2]), fmax(xy[4], xy[6])) <= minmax[2]
+            && fmax(fmax(xy[1], xy[3]), fmax(xy[5], xy[7])) <= minmax[3]);
+}
+
+static void
+gwy_selection_projective_crop(GwySelection *selection,
+                              gdouble xmin,
+                              gdouble ymin,
+                              gdouble xmax,
+                              gdouble ymax)
+{
+    gdouble minmax[4] = { xmin, ymin, xmax, ymax };
+
+    gwy_selection_filter(selection, gwy_selection_projective_crop_object, minmax);
+}
+
+static void
+gwy_selection_projective_move(GwySelection *selection,
+                              gdouble vx,
+                              gdouble vy)
+{
+    gdouble *data = (gdouble*)selection->objects->data;
+    guint i, n = selection->objects->len/OBJECT_SIZE;
+
+    for (i = 0; i < n; i++) {
+        data[OBJECT_SIZE*i + 0] += vx;
+        data[OBJECT_SIZE*i + 1] += vy;
+        data[OBJECT_SIZE*i + 2] += vx;
+        data[OBJECT_SIZE*i + 3] += vy;
+        data[OBJECT_SIZE*i + 4] += vx;
+        data[OBJECT_SIZE*i + 5] += vy;
+        data[OBJECT_SIZE*i + 6] += vx;
+        data[OBJECT_SIZE*i + 7] += vy;
+    }
+}
+
 static void
 gwy_layer_projective_init(GwyLayerProjective *layer)
 {
