@@ -1,5 +1,5 @@
 /*
- *  $Id: calcoefs_simple.c 21408 2018-09-07 13:53:53Z yeti-dn $
+ *  $Id: calcoefs_simple.c 23557 2021-04-22 07:34:37Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -225,7 +225,9 @@ simple(GwyContainer *data, GwyRunType run)
     if (simple_dialog(&args, dfield)) {
         if (args.mask == NULL) {
             gwy_app_undo_qcheckpointv(data, 1, &mquark);
-            args.mask = create_mask_field(dfield);
+            args.mask = gwy_data_field_new_alike(dfield, TRUE);
+            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.mask),
+                                        NULL);
             gwy_container_set_object(data, mquark, args.mask);
             g_object_unref(args.mask);
         }
