@@ -1,5 +1,5 @@
 /*
- *  $Id: calibrate.c 24652 2022-03-08 15:04:52Z yeti-dn $
+ *  $Id: calibrate.c 24975 2022-08-31 12:05:41Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -22,8 +22,6 @@
 #include <libgwyddion/gwymath.h>
 #include <gtk/gtk.h>
 #include <libprocess/stats.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwydgets/gwycombobox.h>
 #include <libgwydgets/gwystock.h>
 #include <libgwymodule/gwymodule-process.h>
 #include <libgwymodule/gwymodule-cmap.h>
@@ -141,7 +139,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Recalibrates scan lateral dimensions or value range."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "3.1",
+    "3.2",
     "David Nečas (Yeti) & Petr Klapetek",
     "2003",
 };
@@ -491,14 +489,16 @@ run_gui(ModuleArgs *args)
     gwy_dialog_add_param_table(dialog, table);
 
     if (args->lawn)
-        vf = gwy_lawn_get_value_format_xy(args->lawn, GWY_SI_UNIT_FORMAT_VFMARKUP, vf);
+        vf = gui.xyvf = gwy_lawn_get_value_format_xy(args->lawn, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
     else
-        vf = gwy_data_field_get_value_format_xy(args->field, GWY_SI_UNIT_FORMAT_VFMARKUP, vf);
+        vf = gui.xyvf = gwy_data_field_get_value_format_xy(args->field, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
+
     buf = g_strdup_printf("%.*f%s%s × %.*f%s%s",
                           vf->precision, args->xreal/vf->magnitude, *vf->units ? " " : "", vf->units,
                           vf->precision, args->yreal/vf->magnitude, *vf->units ? " " : "", vf->units);
     gwy_param_table_info_set_valuestr(gui.table_dims, LABEL_DIMS, buf);
     g_free(buf);
+
     buf = g_strdup_printf("(%.*f%s%s, %.*f%s%s)",
                           vf->precision, args->xoffset/vf->magnitude, *vf->units ? " " : "", vf->units,
                           vf->precision, args->yoffset/vf->magnitude, *vf->units ? " " : "", vf->units);
@@ -506,20 +506,20 @@ run_gui(ModuleArgs *args)
     g_free(buf);
 
     if (gui.table_value) {
-        vf = gwy_data_field_get_value_format_z(args->field, GWY_SI_UNIT_FORMAT_VFMARKUP, vf);
+        vf = gui.zvf = gwy_data_field_get_value_format_z(args->field, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
         buf = g_strdup_printf("[%.*f%s%s, %.*f%s%s]",
                               vf->precision, args->zmin/vf->magnitude, *vf->units ? " " : "", vf->units,
                               vf->precision, args->zmax/vf->magnitude, *vf->units ? " " : "", vf->units);
         gwy_param_table_info_set_valuestr(gui.table_value, LABEL_VALUES, buf);
         g_free(buf);
     }
-    gwy_si_unit_value_format_free(vf);
 
     g_signal_connect_swapped(gui.table_dims, "param-changed", G_CALLBACK(param_changed), &gui);
     if (gui.table_value)
         g_signal_connect_swapped(gui.table_value, "param-changed", G_CALLBACK(param_changed), &gui);
     g_signal_connect_swapped(gui.table_offsets, "param-changed", G_CALLBACK(param_changed), &gui);
     g_signal_connect_after(dialog, "response", G_CALLBACK(dialog_response), &gui);
+
     outcome = gwy_dialog_run(dialog);
 
     GWY_SI_VALUE_FORMAT_FREE(gui.xyvf);
@@ -537,7 +537,6 @@ make_table_dims(ModuleArgs *args)
         PARAM_XREAL, PARAM_YREAL, PARAM_XRATIO, PARAM_YRATIO, PARAM_SQUARE, PARAM_TEMPLATE, PARAM_XYUNIT
     };
     GwyParamTable *table;
-    guint i;
 
     table = gwy_param_table_new(args->params);
     gwy_param_table_append_header(table, -1, _("Dimensions"));
@@ -566,8 +565,7 @@ make_table_dims(ModuleArgs *args)
     gwy_param_table_append_separator(table);
     gwy_param_table_append_checkbox(table, PARAM_SQUARE);
     gwy_param_table_append_unit_chooser(table, PARAM_XYUNIT);
-    for (i = 0; i < G_N_ELEMENTS(noreset); i++)
-        gwy_param_table_set_no_reset(table, noreset[i], TRUE);
+    gwy_param_table_set_no_resetv(table, noreset, G_N_ELEMENTS(noreset), TRUE);
     return table;
 }
 
@@ -576,7 +574,6 @@ make_table_offsets(ModuleArgs *args)
 {
     static const gint noreset[] = { PARAM_XOFFSET, PARAM_YOFFSET };
     GwyParamTable *table;
-    guint i;
 
     table = gwy_param_table_new(args->params);
     gwy_param_table_append_header(table, -1, _("Offsets"));
@@ -588,8 +585,7 @@ make_table_offsets(ModuleArgs *args)
     gwy_param_table_append_radio_item(table, PARAM_OFFSETS_MODE, MODE_SET_RANGE);
     gwy_param_table_append_entry(table, PARAM_XOFFSET);
     gwy_param_table_append_entry(table, PARAM_YOFFSET);
-    for (i = 0; i < G_N_ELEMENTS(noreset); i++)
-        gwy_param_table_set_no_reset(table, noreset[i], TRUE);
+    gwy_param_table_set_no_resetv(table, noreset, G_N_ELEMENTS(noreset), TRUE);
     return table;
 }
 
@@ -598,7 +594,6 @@ make_table_value(ModuleArgs *args)
 {
     static const gint noreset[] = { PARAM_ZRANGE, PARAM_ZMIN, PARAM_ZSHIFT, PARAM_ZRATIO, PARAM_ZUNIT };
     GwyParamTable *table;
-    guint i;
 
     table = gwy_param_table_new(args->params);
     gwy_param_table_append_header(table, -1, _("Value Range"));
@@ -614,8 +609,7 @@ make_table_value(ModuleArgs *args)
     gwy_param_table_append_entry(table, PARAM_ZSHIFT);
     gwy_param_table_append_separator(table);
     gwy_param_table_append_unit_chooser(table, PARAM_ZUNIT);
-    for (i = 0; i < G_N_ELEMENTS(noreset); i++)
-        gwy_param_table_set_no_reset(table, noreset[i], TRUE);
+    gwy_param_table_set_no_resetv(table, noreset, G_N_ELEMENTS(noreset), TRUE);
     return table;
 }
 
@@ -836,7 +830,7 @@ param_changed_z(ModuleGUI *gui, gint id)
         unit = gwy_params_get_unit(params, PARAM_ZUNIT, &power10);
         unitcal = gwy_si_unit_divide(unit, args->zunit, NULL);
         gui->zvf = gwy_si_unit_get_format_for_power10(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, power10, gui->zvf);
-        gui->xyvf->precision = 4;
+        gui->zvf->precision = 4;
         m = 5.0*gwy_params_get_double(params, PARAM_ZRATIO);
         gui->zcalvf = gwy_si_unit_get_format_with_digits(unitcal, GWY_SI_UNIT_FORMAT_VFMARKUP, m, 6, gui->zcalvf);
         gwy_debug("Z %g (%d) [%s] for %g", gui->zcalvf->magnitude, gui->zcalvf->precision, gui->zcalvf->units, m);
