@@ -1,5 +1,5 @@
 /*
- *  $Id: convolution_filter.c 23556 2021-04-21 08:00:51Z yeti-dn $
+ *  $Id: convolution_filter.c 24707 2022-03-21 17:09:03Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -130,7 +130,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Generic convolution filter with a user-defined matrix."),
     "Yeti <yeti@gwyddion.net>",
-    "1.6",
+    "1.7",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -518,7 +518,6 @@ convolution_filter_create_preset_tab(ConvolutionControls *controls)
     GtkTreeViewColumn *column;
     GtkWidget *vbox, *treeview, *hbox, *image, *button;
     GtkTreeIter iter;
-    GtkTooltips *tooltips;
     const gchar *name;
     guint i;
 
@@ -584,7 +583,6 @@ convolution_filter_create_preset_tab(ConvolutionControls *controls)
                      G_CALLBACK(convolution_filter_switch_preset), controls);
 
     /* Controls */
-    tooltips = gwy_app_get_tooltips();
     hbox = gtk_hbox_new(TRUE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     for (i = 0; i < G_N_ELEMENTS(toolbar_buttons); i++) {
@@ -593,8 +591,7 @@ convolution_filter_create_preset_tab(ConvolutionControls *controls)
         button = gtk_button_new();
         gtk_container_add(GTK_CONTAINER(button), image);
         gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-        gtk_tooltips_set_tip(tooltips, button,
-                             toolbar_buttons[i].tooltip, NULL);
+        gtk_widget_set_tooltip_text(button, toolbar_buttons[i].tooltip);
         g_signal_connect_swapped(button, "clicked",
                                  toolbar_buttons[i].callback, controls);
 
