@@ -1,5 +1,5 @@
 /*
- *  $Id: correct_perspective.c 24796 2022-04-28 15:20:59Z yeti-dn $
+ *  $Id: correct_perspective.c 24975 2022-08-31 12:05:41Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -79,8 +79,9 @@ static GwyDialogOutcome run_gui                        (ModuleArgs *args,
                                                         gint id);
 static void             param_changed                  (ModuleGUI *gui,
                                                         gint id);
-static void             dialog_response                (ModuleGUI *gui,
-                                                        gint response);
+static void             dialog_response                (GwyDialog *dialog,
+                                                        gint response,
+                                                        ModuleGUI *gui);
 static void             preview                        (gpointer user_data);
 static void             init_coordinates               (GwyDataField *field,
                                                         gdouble *xy);
@@ -368,7 +369,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_dialog_add_param_table(dialog, table);
 
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
-    g_signal_connect_swapped(dialog, "response", G_CALLBACK(dialog_response), &gui);
+    g_signal_connect_after(dialog, "response", G_CALLBACK(dialog_response), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
     outcome = gwy_dialog_run(dialog);
@@ -403,7 +404,7 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static void
-dialog_response(ModuleGUI *gui, gint response)
+dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
 
