@@ -1,5 +1,5 @@
 /*
- *  $Id: crosscor.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: crosscor.c 24416 2021-10-24 07:25:04Z yeti-dn $
  *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -164,7 +164,7 @@ define_module_params(void)
     gwy_param_def_add_boolean(paramdef, PARAM_GAUSSIAN, "gaussian", NULL, FALSE);
     gwy_param_def_add_boolean(paramdef, PARAM_EXTEND, "extend", _("Extend results to borders"), TRUE);
     gwy_param_def_add_boolean(paramdef, PARAM_MULTIPLE, "multiple", _("Multichannel cross-corelation"), FALSE);
-    gwy_param_def_add_image_id(paramdef, PARAM_SECOND_SOURCE, "second_source", _("Second _source data:"));
+    gwy_param_def_add_image_id(paramdef, PARAM_SECOND_SOURCE, "second_source", _("Second _source data"));
     gwy_param_def_add_image_id(paramdef, PARAM_SECOND_OTHER, "second_other", _("Co_rrelate with"));
     return paramdef;
 }
