@@ -1,6 +1,6 @@
 /*
- *  $Id: dimensions.h 21240 2018-07-17 12:37:01Z yeti-dn $
- *  Copyright (C) 2009 David Necas (Yeti).
+ *  $Id: dimensions.h 23211 2021-03-11 11:14:21Z yeti-dn $
+ *  Copyright (C) 2009-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -49,6 +49,7 @@ typedef struct {
     /* These are provided for the user callbacks to that he can easily update
      * unit labels */
     GwySIValueFormat *xyvf;
+    GwySIValueFormat *sxyvf;  /* Small xy: for objects typically << image. */
     GwySIValueFormat *zvf;
     GwySIUnit *xysiunit;
     GwySIUnit *zsiunit;
@@ -205,8 +206,7 @@ gwy_dimensions_units_changed(GtkWidget *widget,
     }
 
     unit = gtk_entry_get_text(entry);
-    g_free(*unitstr);
-    *unitstr = g_strdup(unit);
+    gwy_assign_string(unitstr, unit);
     gwy_dimensions_set_combo_from_unit(combo, unit);
 
     gtk_widget_destroy(dialog);
@@ -302,11 +302,20 @@ static void
 gwy_dimensions_xypow10_changed(GwyDimensions *dims,
                                GtkComboBox *combo)
 {
-    dims->args->xypow10 = gwy_enum_combo_box_get_active(combo);
-    gwy_si_unit_set_from_string(dims->xysiunit, dims->args->xyunits);
+    GwyDimensionArgs *args = dims->args;
+    gdouble size;
+    guint res;
+
+    args->xypow10 = gwy_enum_combo_box_get_active(combo);
+    gwy_si_unit_set_from_string(dims->xysiunit, args->xyunits);
     gwy_si_unit_get_format_for_power10(dims->xysiunit,
                                        GWY_SI_UNIT_FORMAT_VFMARKUP,
-                                       dims->args->xypow10, dims->xyvf);
+                                       args->xypow10, dims->xyvf);
+    res = MAX(args->xres, args->yres);
+    size = res*args->measure * pow10(args->xypow10);
+    gwy_si_unit_get_format_with_resolution(dims->xysiunit,
+                                           GWY_SI_UNIT_FORMAT_VFMARKUP,
+                                           size, size/res, dims->sxyvf);
     /* Update our parameter unit labels. */
     gtk_label_set_markup(GTK_LABEL(dims->xunitslab), dims->xyvf->units);
     gtk_label_set_markup(GTK_LABEL(dims->yunitslab), dims->xyvf->units);
@@ -356,12 +365,10 @@ gwy_dimensions_use_template(GwyDimensions *dims)
     gtk_adjustment_set_value(dims->yres, yres);
     gtk_adjustment_set_value(dims->xreal, xreal/xyvf->magnitude);
     gtk_adjustment_set_value(dims->yreal, yreal/xyvf->magnitude);
-    g_free(dims->args->xyunits);
-    dims->args->xyunits = g_strdup(xyvf->units);
+    gwy_assign_string(&dims->args->xyunits, xyvf->units);
     gwy_dimensions_set_combo_from_unit(GTK_COMBO_BOX(dims->xypow10),
                                        xyvf->units);
-    g_free(dims->args->zunits);
-    dims->args->zunits = g_strdup(zvf->units);
+    gwy_assign_string(&dims->args->zunits, zvf->units);
     gwy_dimensions_set_combo_from_unit(GTK_COMBO_BOX(dims->zpow10),
                                        zvf->units);
     dims->in_update = FALSE;
@@ -412,6 +419,8 @@ gwy_dimensions_new(GwyDimensionArgs *args,
     GwySensitivityGroup *sensgroup;
     GtkWidget *label, *button;
     GtkTable *table;
+    gdouble size;
+    guint res;
     gint row;
 
     dims->args = args;
@@ -422,6 +431,11 @@ gwy_dimensions_new(GwyDimensionArgs *args,
                                                     GWY_SI_UNIT_FORMAT_VFMARKUP,
                                                     args->xypow10, NULL);
     dims->xyvf->precision = 3;
+    res = MAX(args->xres, args->yres);
+    size = res*args->measure * pow10(args->xypow10);
+    dims->sxyvf = gwy_si_unit_get_format_with_resolution(dims->xysiunit,
+                                                         GWY_SI_UNIT_FORMAT_VFMARKUP,
+                                                         size, size/res, NULL);
     dims->zsiunit = gwy_si_unit_new(dims->args->zunits);
     dims->zvf = gwy_si_unit_get_format_for_power10(dims->zsiunit,
                                                    GWY_SI_UNIT_FORMAT_VFMARKUP,
@@ -568,6 +582,7 @@ gwy_dimensions_free(GwyDimensions *dims)
     g_object_unref(dims->sensgroup);
     g_object_unref(dims->xysiunit);
     g_object_unref(dims->zsiunit);
+    gwy_si_unit_value_format_free(dims->sxyvf);
     gwy_si_unit_value_format_free(dims->xyvf);
     gwy_si_unit_value_format_free(dims->zvf);
     gwy_object_unref(dims->template_);
@@ -631,14 +646,12 @@ gwy_dimensions_load_args(GwyDimensionArgs *args,
 
     g_string_append(g_string_truncate(key, len), "xyunits");
     if (gwy_container_gis_string_by_name(settings, key->str, &s)) {
-        g_free(args->xyunits);
-        args->xyunits = g_strdup(s);
+        gwy_assign_string(&args->xyunits, s);
     }
 
     g_string_append(g_string_truncate(key, len), "zunits");
     if (gwy_container_gis_string_by_name(settings, key->str, &s)) {
-        g_free(args->zunits);
-        args->zunits = g_strdup(s);
+        gwy_assign_string(&args->zunits, s);
     }
 
     g_string_append(g_string_truncate(key, len), "replace");
