@@ -1,5 +1,5 @@
 /*
- *  $Id: displfield.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: displfield.c 24796 2022-04-28 15:20:59Z yeti-dn $
  *  Copyright (C) 2019-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -122,7 +122,7 @@ module_register(void)
     gwy_process_func_register("displfield",
                               (GwyProcessFunc)&displ_field,
                               N_("/_Distortion/Displacement _Field..."),
-                              NULL,
+                              GWY_STOCK_DISPLACEMENT_FIELD,
                               RUN_MODES,
                               GWY_MENU_FLAG_DATA,
                               _("Deform image or scan lines in plane"));
