@@ -1,5 +1,5 @@
 /*
- *  $Id: domain_synth.c 24658 2022-03-10 13:16:56Z yeti-dn $
+ *  $Id: domain_synth.c 25125 2022-10-21 14:51:26Z yeti-dn $
  *  Copyright (C) 2014-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -121,11 +121,13 @@ static const GwyEnum quantity_types[OUTPUT_NTYPES] = {
 };
 
 static const SimulationPreset presets[] = {
-    { "Alien biology", 1000, 2.0,  44.0, 2.0,  -0.5, 2.0,  },
-    { "Brain waves",   1200, 1.8,  42.0, 2.0,  0.3,  3.0,  },
-    { "Chaos",         500,  2.0,  9.0,  1.0,  0.0,  90.0, },
-    { "Islands",       500,  0.7,  8.0,  1.0,  0.0,  0.05, },
-    { "Mixed spirals", 1200, 0.12, 0.85, 27.0, 0.0,  45.0, },
+    { "Alien biology",       1000, 2.0,  44.0, 2.0,  -0.5,  2.0,  },
+    { "Angular",             700,  0.1,  20.0, 0.2,   0.0,  10.0, },
+    { "Brain waves",         1200, 1.8,  42.0, 2.0,   0.3,  3.0,  },
+    { "Chaos",               500,  2.0,  9.0,  1.0,   0.0,  90.0, },
+    { "Islands",             500,  0.7,  8.0,  1.0,   0.0,  0.05, },
+    { "Mixed spirals",       1200, 0.12, 0.85, 27.0,  0.0,  45.0, },
+    { "Multiscale blotches", 740,  0.8,  6.0,  1.2,  -0.4,  7.0,  },
 };
 
 static GwyModuleInfo module_info = {
@@ -133,7 +135,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Generates domain images using a hybrid Ising model."),
     "Yeti <yeti@gwyddion.net>",
-    "3.0",
+    "3.1",
     "David Nečas (Yeti)",
     "2014",
 };
