@@ -1,5 +1,5 @@
 /*
- *  $Id: drift.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: drift.c 24760 2022-04-14 09:36:59Z yeti-dn $
  *  Copyright (C) 2007-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -162,7 +162,7 @@ define_module_params(void)
     gwy_param_def_add_boolean(paramdef, PARAM_DO_CORRECT, "do-correct", _("Correct _data"), TRUE);
     gwy_param_def_add_boolean(paramdef, PARAM_DO_PLOT, "do-plot", _("Plot drift _graph"), TRUE);
     gwy_param_def_add_boolean(paramdef, PARAM_EXCLUDE_LINEAR, "exclude-linear", _("_Exclude linear skew"), FALSE);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", _("Preview"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", gwy_sgettext("verb|Display"),
                               previews, G_N_ELEMENTS(previews), PREVIEW_MASK);
     gwy_param_def_add_target_graph(paramdef, PARAM_TARGET_GRAPH, "target_graph", NULL);
     gwy_param_def_add_mask_color(paramdef, PARAM_MASK_COLOR, NULL, NULL);
@@ -179,7 +179,7 @@ compensate_drift(GwyContainer *data, GwyRunType runtype)
     GwyDataField *field, *mask, *sfield;
     gboolean do_plot, do_correct, new_image, distribute;
     GArray *undo_quarks;
-    gint *image_ids = NULL;
+    gint *image_ids;
     gint id, i;
 
     g_return_if_fail(runtype & RUN_MODES);
@@ -264,9 +264,9 @@ compensate_drift(GwyContainer *data, GwyRunType runtype)
 
         apply_correction_to_one_image(&args, data, image_ids[i]);
     }
+    g_free(image_ids);
 
 end:
-    g_free(image_ids);
     g_object_unref(params);
     g_object_unref(args.result);
     g_object_unref(args.mask);
