@@ -1,5 +1,5 @@
 /*
- *  $Id: facet_analysis.c 24626 2022-03-03 12:59:05Z yeti-dn $
+ *  $Id: facet_analysis.c 24952 2022-08-26 15:24:56Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1306,7 +1306,7 @@ facet_view_selection_updated(GwySelection *selection,
 {
     GtkTreeSelection *treesel;
     GtkTreeIter iter;
-    gint i, n, nold;
+    gint n, nold;
 
     n = gwy_selection_get_data(selection, NULL);
     nold = gwy_null_store_get_n_rows(controls->store);
@@ -1315,8 +1315,8 @@ facet_view_selection_updated(GwySelection *selection,
         if (n == nold+1)
             hint = n-1;
         n = MIN(n, nold);
-        for (i = 0; i < n; i++)
-            gwy_null_store_row_changed(controls->store, i);
+        if (n > 0)
+            gwy_null_store_rows_changed(controls->store, 0, n-1);
     }
     else {
         g_return_if_fail(hint >= 0);
@@ -1729,8 +1729,7 @@ gcd(gint a, gint b)
 {
     a = ABS(a);
     b = ABS(b);
-    if (a < b)
-        GWY_SWAP(gint, a, b);
+    GWY_ORDER(gint, b, a);
 
     /* This also handles that gcd(x, 0) = x, by definition. */
     while (b) {
