@@ -1,6 +1,6 @@
 /*
- *  $Id: facet_measure.c 23766 2021-05-24 15:48:55Z yeti-dn $
- *  Copyright (C) 2019-2021 David Necas (Yeti).
+ *  $Id: facet_measure.c 24626 2022-03-03 12:59:05Z yeti-dn $
+ *  Copyright (C) 2019-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -212,7 +212,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Visualizes, marks and measures facet orientation."),
     "Yeti <yeti@gwyddion.net>",
-    "2.0",
+    "2.1",
     "David Nečas (Yeti)",
     "2019",
 };
@@ -362,8 +362,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gtk_container_add(GTK_CONTAINER(alignment), dataview);
     gtk_box_pack_start(GTK_BOX(hbox), alignment, FALSE, FALSE, 0);
 
-    gui.iselection = create_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", TRUE);
-    gwy_selection_set_max_objects(gui.iselection, 1);
+    gui.iselection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
 
     table = gui.table = gwy_param_table_new(args->params);
     gwy_param_table_append_slider(table, PARAM_KERNEL_SIZE);
@@ -405,8 +404,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     dataview = gwy_create_preview(gui.fdata, 0, FACETVIEW_SIZE, TRUE);
     gtk_box_pack_start(GTK_BOX(vbox), dataview, FALSE, FALSE, 0);
-    gui.fselection = create_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", TRUE);
-    gwy_selection_set_max_objects(gui.fselection, 1);
+    gui.fselection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
     g_object_ref(gui.fselection);
 
     auxbox = gwy_hbox_new(0);
