@@ -1,5 +1,5 @@
 /*
- *  $Id: fit-shape.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: fit-shape.c 24761 2022-04-14 09:37:10Z yeti-dn $
  *  Copyright (C) 2016-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -236,7 +236,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Fits predefined geometrical shapes to data."),
     "Yeti <yeti@gwyddion.net>",
-    "2.0",
+    "2.2",
     "David Nečas (Yeti)",
     "2016",
 };
@@ -288,7 +288,7 @@ define_module_params(void)
     gwy_param_def_add_resource(paramdef, PARAM_FUNCTION, "function", _("_Function"),
                                gwy_shape_fit_presets(), "Grating (simple)");
     gwy_param_def_add_enum(paramdef, PARAM_MASKING, "masking", NULL, GWY_TYPE_MASKING_TYPE, GWY_MASK_IGNORE);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", _("Preview"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", gwy_sgettext("verb|Display"),
                               displays, G_N_ELEMENTS(displays), FIT_SHAPE_DISPLAY_DIFF);
     gwy_param_def_add_gwyenum(paramdef, PARAM_OUTPUT, "output", _("Output _type"),
                               outputs, G_N_ELEMENTS(outputs), FIT_SHAPE_OUTPUT_BOTH);
@@ -336,7 +336,9 @@ fit_shape(GwyContainer *data, GwyRunType runtype, const gchar *name)
     sanitise_params(&args);
     outcome = run_gui(&args, data, id);
     gwy_params_save_to_settings(args.params);
-    if (outcome != GWY_DIALOG_HAVE_RESULT)
+    /* The user can press OK and we will produce images of whatever currently exists.  This allows just running it and
+     * creating an image with the model shape, for instance. */
+    if (outcome == GWY_DIALOG_CANCEL)
         goto end;
 
     if (args.pageno == GWY_PAGE_XYZS)
@@ -550,10 +552,8 @@ dialog_response(ModuleGUI *gui, gint response)
 {
     if (response == RESPONSE_REFINE) {
         fit_shape_full_fit(gui);
-        if (gui->state == FIT_SHAPE_FITTED) {
+        if (gui->state == FIT_SHAPE_FITTED)
             fill_results(gui);
-            gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
-        }
     }
     else if (response == RESPONSE_CALCULATE)
         fit_shape_quick_fit(gui);
@@ -779,8 +779,13 @@ fit_param_table_resize(ModuleGUI *gui)
     for (i = 0; i < nparams; i++) {
         FitParamControl *cntrl = &g_array_index(gui->param_controls, FitParamControl, i);
         const gchar *name = gwy_shape_fit_preset_get_param_name(gui->preset, i);
+        const gchar *desc = gwy_shape_fit_preset_get_param_description(gui->preset, i);
 
         gtk_label_set_markup(GTK_LABEL(cntrl->name), name);
+        /* Set it on all three. */
+        gtk_widget_set_tooltip_markup(cntrl->name, desc);
+        gtk_widget_set_tooltip_markup(cntrl->equals, desc);
+        gtk_widget_set_tooltip_markup(cntrl->value, desc);
     }
 
     gtk_widget_show_all(gui->fit_param_table);
@@ -949,8 +954,13 @@ fit_secondary_table_resize(ModuleGUI *gui)
     for (i = 0; i < nsecondary; i++) {
         FitParamControl *cntrl = &g_array_index(gui->secondary_controls, FitParamControl, i);
         const gchar *name = gwy_shape_fit_preset_get_secondary_name(gui->preset, i);
+        const gchar *desc = gwy_shape_fit_preset_get_secondary_description(gui->preset, i);
 
         gtk_label_set_markup(GTK_LABEL(cntrl->name), name);
+        /* Set it on all three. */
+        gtk_widget_set_tooltip_markup(cntrl->name, desc);
+        gtk_widget_set_tooltip_markup(cntrl->equals, desc);
+        gtk_widget_set_tooltip_markup(cntrl->value, desc);
     }
 
     gtk_widget_show_all(gui->secondary_table);
@@ -1281,8 +1291,7 @@ fit_shape_estimate(ModuleGUI *gui)
     else
         gui->state = FIT_SHAPE_ESTIMATE_FAILED;
 
-    /* XXX: We honour fixed parameters by reverting to previous values and
-     * pretending nothing happened.  Is it OK? */
+    /* XXX: We honour fixed parameters by reverting to previous values and pretending nothing happened.  Is it OK? */
     for (i = 0; i < nparams; i++) {
         gwy_debug("[%u] %g", i, gui->param[i]);
         if (ctx->param_fixed[i])
