@@ -1,5 +1,5 @@
 /*
- *  $Id: good_profile.c 23766 2021-05-24 15:48:55Z yeti-dn $
+ *  $Id: good_profile.c 24365 2021-10-13 12:45:57Z yeti-dn $
  *  Copyright (C) 2020-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -134,7 +134,7 @@ define_module_params(void)
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, gwy_process_func_current());
     gwy_param_def_add_boolean(paramdef, PARAM_CREATE_MASK, "create_mask", _("Create _mask"), TRUE);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", _("Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", gwy_sgettext("verb|Display"),
                               displays, G_N_ELEMENTS(displays), GOOD_PROF_GRAPH);
     gwy_param_def_add_gwyenum(paramdef, PARAM_MODE, "mode", _("Mode"),
                               modes, G_N_ELEMENTS(modes), GOOD_PROF_SINGLE);
