@@ -1,5 +1,5 @@
 /*
- *  $Id: gradient.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: gradient.c 21784 2019-01-03 13:00:48Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -144,7 +144,7 @@ gradient(GwyContainer *data,
     gwy_app_undo_qcheckpointv(data, 1, &squark);
     if (!showfield) {
         showfield = gwy_data_field_new_alike(dfield, FALSE);
-        siunit = gwy_si_unit_new("");
+        siunit = gwy_si_unit_new(NULL);
         gwy_data_field_set_si_unit_z(showfield, siunit);
         g_object_unref(siunit);
         gwy_container_set_object(data, squark, showfield);
