@@ -1,5 +1,5 @@
 /*
- *  $Id: grain_filter.c 20835 2018-03-13 10:54:57Z yeti-dn $
+ *  $Id: grain_filter.c 23556 2021-04-21 08:00:51Z yeti-dn $
  *  Copyright (C) 2014-2017 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -339,7 +339,7 @@ gfilter_dialog(GFilterArgs *args,
                             GWY_DATA_ITEM_RANGE,
                             GWY_DATA_ITEM_REAL_SQUARE,
                             0);
-    controls.view = create_preview(controls.mydata, 0, PREVIEW_SIZE, TRUE);
+    controls.view = gwy_create_preview(controls.mydata, 0, PREVIEW_SIZE, TRUE);
     gtk_box_pack_start(GTK_BOX(vbox), controls.view, FALSE, FALSE, 0);
 
     controls.update = gtk_check_button_new_with_mnemonic(_("I_nstant updates"));
