@@ -1,5 +1,5 @@
 /*
- *  $Id: lat_synth.c 23801 2021-06-04 15:55:10Z yeti-dn $
+ *  $Id: lat_synth.c 24975 2022-08-31 12:05:41Z yeti-dn $
  *  Copyright (C) 2014-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -217,8 +217,9 @@ static GwyParamTable*   make_surface_param_table    (ModuleGUI *gui,
                                                      LatSynthSurfaceType i);
 static void             param_changed               (ModuleGUI *gui,
                                                      gint id);
-static void             dialog_response             (ModuleGUI *gui,
-                                                     gint response);
+static void             dialog_response             (GwyDialog *dialog,
+                                                     gint response,
+                                                     ModuleGUI *gui);
 static void             preview                     (gpointer user_data);
 static void             update_surface_sensitivity  (ModuleGUI *gui,
                                                      LatSynthSurfaceType i);
@@ -463,7 +464,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(gui.table_lattice, "param-changed", G_CALLBACK(param_changed), &gui);
     for (i = 0; i < LAT_NSURFACES; i++)
         g_signal_connect_swapped(gui.table_surface[i], "param-changed", G_CALLBACK(param_changed), &gui);
-    g_signal_connect_swapped(dialog, "response", G_CALLBACK(dialog_response), &gui);
+    g_signal_connect_after(dialog, "response", G_CALLBACK(dialog_response), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
     outcome = gwy_dialog_run(dialog);
@@ -631,7 +632,7 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static void
-dialog_response(ModuleGUI *gui, gint response)
+dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     guint i;
