@@ -1,22 +1,18 @@
 /*
- *  $Id: maskops.c 21784 2019-01-03 13:00:48Z yeti-dn $
- *  Copyright (C) 2003-2004 David Necas (Yeti), Petr Klapetek.
+ *  $Id: maskops.c 23632 2021-05-03 23:01:27Z yeti-dn $
+ *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -28,24 +24,26 @@
 #include <app/gwymoduleutils.h>
 #include <app/gwyapp.h>
 
-#define MASKOPS_RUN_MODES GWY_RUN_IMMEDIATE
+#define RUN_MODES GWY_RUN_IMMEDIATE
 
 static gboolean module_register(void);
 static void     mask_remove    (GwyContainer *data,
-                                GwyRunType run);
+                                GwyRunType runtype);
 static void     mask_invert    (GwyContainer *data,
-                                GwyRunType run);
+                                GwyRunType runtype);
 static void     mask_extract   (GwyContainer *data,
-                                GwyRunType run);
+                                GwyRunType runtype);
 static void     remove_touching(GwyContainer *data,
-                                GwyRunType run);
+                                GwyRunType runtype);
+static void     mask_thin      (GwyContainer *data,
+                                GwyRunType runtype);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Basic operations with mask: inversion, removal, extraction."),
     "Yeti <yeti@gwyddion.net>",
-    "1.5",
+    "1.6",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -59,42 +57,49 @@ module_register(void)
                               (GwyProcessFunc)&mask_remove,
                               N_("/_Mask/_Remove Mask"),
                               GWY_STOCK_MASK_REMOVE,
-                              MASKOPS_RUN_MODES,
+                              RUN_MODES,
                               GWY_MENU_FLAG_DATA_MASK | GWY_MENU_FLAG_DATA,
                               N_("Remove mask from data"));
     gwy_process_func_register("mask_invert",
                               (GwyProcessFunc)&mask_invert,
                               N_("/_Mask/_Invert Mask"),
                               GWY_STOCK_MASK_INVERT,
-                              MASKOPS_RUN_MODES,
+                              RUN_MODES,
                               GWY_MENU_FLAG_DATA_MASK | GWY_MENU_FLAG_DATA,
                               N_("Invert mask"));
     gwy_process_func_register("mask_extract",
                               (GwyProcessFunc)&mask_extract,
                               N_("/_Mask/_Extract Mask"),
                               GWY_STOCK_MASK_EXTRACT,
-                              MASKOPS_RUN_MODES,
+                              RUN_MODES,
                               GWY_MENU_FLAG_DATA_MASK | GWY_MENU_FLAG_DATA,
                               N_("Extract mask to a new image"));
     gwy_process_func_register("grain_rem_touching",
                               (GwyProcessFunc)&remove_touching,
                               N_("/_Grains/_Remove Edge-Touching"),
                               GWY_STOCK_GRAINS_EDGE_REMOVE,
-                              MASKOPS_RUN_MODES,
+                              RUN_MODES,
                               GWY_MENU_FLAG_DATA | GWY_MENU_FLAG_DATA_MASK,
                               N_("Remove grains touching image edges"));
+    gwy_process_func_register("mask_thin",
+                              (GwyProcessFunc)&mask_thin,
+                              N_("/_Mask/Thi_n"),
+                              GWY_STOCK_MASK_THIN,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_DATA_MASK | GWY_MENU_FLAG_DATA,
+                              N_("Thin mask"));
 
     return TRUE;
 }
 
 static void
-mask_invert(GwyContainer *data, GwyRunType run)
+mask_invert(GwyContainer *data, GwyRunType runtype)
 {
     GwyDataField *mfield;
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(run & MASKOPS_RUN_MODES);
+    g_return_if_fail(runtype & RUN_MODES);
     gwy_app_data_browser_get_current(GWY_APP_MASK_FIELD, &mfield,
                                      GWY_APP_MASK_FIELD_KEY, &mquark,
                                      GWY_APP_DATA_FIELD_ID, &id,
@@ -108,12 +113,12 @@ mask_invert(GwyContainer *data, GwyRunType run)
 }
 
 static void
-mask_remove(GwyContainer *data, GwyRunType run)
+mask_remove(GwyContainer *data, GwyRunType runtype)
 {
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(run & MASKOPS_RUN_MODES);
+    g_return_if_fail(runtype & RUN_MODES);
     gwy_app_data_browser_get_current(GWY_APP_MASK_FIELD_KEY, &mquark,
                                      GWY_APP_DATA_FIELD_ID, &id,
                                      0);
@@ -125,40 +130,39 @@ mask_remove(GwyContainer *data, GwyRunType run)
 }
 
 static void
-mask_extract(GwyContainer *data, GwyRunType run)
+mask_extract(GwyContainer *data, GwyRunType runtype)
 {
-    GwyDataField *dfield;
-    GwySIUnit *siunit;
-    gint oldid, newid;
+    GwyDataField *field;
+    gint id, newid;
 
-    g_return_if_fail(run & MASKOPS_RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_MASK_FIELD, &dfield,
-                                     GWY_APP_DATA_FIELD_ID, &oldid,
+    g_return_if_fail(runtype & RUN_MODES);
+    gwy_app_data_browser_get_current(GWY_APP_MASK_FIELD, &field,
+                                     GWY_APP_DATA_FIELD_ID, &id,
                                      0);
-    g_return_if_fail(dfield);
+    g_return_if_fail(field);
 
-    dfield = gwy_data_field_duplicate(dfield);
-    gwy_data_field_clamp(dfield, 0.0, 1.0);
+    field = gwy_data_field_duplicate(field);
+    gwy_data_field_clamp(field, 0.0, 1.0);
+    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), NULL);
 
-    /* Other functions should set the units correctly, but do not trust them */
-    siunit = gwy_si_unit_new(NULL);
-    gwy_data_field_set_si_unit_z(dfield, siunit);
-    g_object_unref(siunit);
-
-    newid = gwy_app_data_browser_add_data_field(dfield, data, TRUE);
-    g_object_unref(dfield);
+    newid = gwy_app_data_browser_add_data_field(field, data, TRUE);
+    g_object_unref(field);
     gwy_app_set_data_field_title(data, newid, _("Mask"));
-    gwy_app_channel_log_add_proc(data, oldid, newid);
+    gwy_app_sync_data_items(data, data, id, newid, FALSE,
+                            GWY_DATA_ITEM_GRADIENT,
+                            GWY_DATA_ITEM_REAL_SQUARE,
+                            0);
+    gwy_app_channel_log_add_proc(data, id, newid);
 }
 
 static void
-remove_touching(GwyContainer *data, GwyRunType run)
+remove_touching(GwyContainer *data, GwyRunType runtype)
 {
     GwyDataField *mfield;
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(run & MASKOPS_RUN_MODES);
+    g_return_if_fail(runtype & RUN_MODES);
     gwy_app_data_browser_get_current(GWY_APP_MASK_FIELD, &mfield,
                                      GWY_APP_MASK_FIELD_KEY, &mquark,
                                      GWY_APP_DATA_FIELD_ID, &id,
@@ -171,4 +175,24 @@ remove_touching(GwyContainer *data, GwyRunType run)
     gwy_app_channel_log_add_proc(data, id, id);
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+static void
+mask_thin(GwyContainer *data, GwyRunType runtype)
+{
+    GwyDataField *mask;
+    GQuark quark;
+    gint id;
+
+    g_return_if_fail(runtype & RUN_MODES);
+    gwy_app_data_browser_get_current(GWY_APP_MASK_FIELD, &mask,
+                                     GWY_APP_MASK_FIELD_KEY, &quark,
+                                     GWY_APP_DATA_FIELD_ID, &id,
+                                     0);
+    g_return_if_fail(mask);
+
+    gwy_app_undo_qcheckpointv(data, 1, &quark);
+    gwy_data_field_grains_thin(mask);
+    gwy_data_field_data_changed(mask);
+    gwy_app_channel_log_add_proc(data, id, id);
+}
+
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
