@@ -1,5 +1,5 @@
 /*
- *  $Id: measure_lattice.c 24754 2022-03-29 14:19:00Z yeti-dn $
+ *  $Id: measure_lattice.c 24975 2022-08-31 12:05:41Z yeti-dn $
  *  Copyright (C) 2015-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -72,6 +72,7 @@ enum {
     PARAM_MASKING,
     PARAM_IMAGE_MODE,
     PARAM_SELECTION_MODE,
+    PARAM_SINGLE_VECTOR,
     PARAM_SHOW_NUMBERS,
     PARAM_REPORT_STYLE,
 
@@ -114,8 +115,9 @@ static GwyResults*      create_results        (ModuleArgs *args,
                                                gint id);
 static void             param_changed         (ModuleGUI *gui,
                                                gint id);
-static void             dialog_response       (ModuleGUI *gui,
-                                               gint response);
+static void             dialog_response       (GwyDialog *dialog,
+                                               gint response,
+                                               ModuleGUI *gui);
 static void             preview               (gpointer user_data);
 static GtkWidget*       create_lattice_table  (gpointer user_data);
 static void             set_selection         (ModuleGUI *gui);
@@ -139,7 +141,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Measures parameters of two-dimensional lattices."),
     "Yeti <yeti@gwyddion.net>",
-    "3.2",
+    "3.4",
     "David Nečas (Yeti)",
     "2015",
 };
@@ -196,6 +198,7 @@ define_module_params(void)
                               image_modes, G_N_ELEMENTS(image_modes), IMAGE_DATA);
     gwy_param_def_add_gwyenum(paramdef, PARAM_SELECTION_MODE, "selection_mode", _("Show lattice as"),
                               selection_modes, G_N_ELEMENTS(selection_modes), SELECTION_LATTICE);
+    gwy_param_def_add_boolean(paramdef, PARAM_SINGLE_VECTOR, "single_vector", _("Measure single vector"), FALSE);
     gwy_param_def_add_boolean(paramdef, PARAM_SHOW_NUMBERS, "show_numbers", _("Show vector numbers"), FALSE);
     gwy_param_def_add_report_type(paramdef, PARAM_REPORT_STYLE, "report_style", _("Save Parameters"),
                                   GWY_RESULTS_EXPORT_PARAMETERS, GWY_RESULTS_REPORT_COLON);
@@ -296,6 +299,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     if (args->mask)
         gwy_param_table_append_combo(table, PARAM_MASKING);
     gwy_param_table_append_checkbox(table, PARAM_FIX_HACF);
+    gwy_param_table_append_checkbox(table, PARAM_SINGLE_VECTOR);
     gwy_param_table_append_header(table, -1, _("Lattice Vectors"));
     gwy_param_table_append_foreign(table, WIDGET_VECTORS, create_lattice_table, &gui, NULL);
     gwy_param_table_append_report(table, PARAM_REPORT_STYLE);
@@ -304,7 +308,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), TRUE, TRUE, 0);
     gwy_dialog_add_param_table(dialog, table);
     g_signal_connect_swapped(gui.table, "param-changed", G_CALLBACK(param_changed), &gui);
-    g_signal_connect_swapped(dialog, "response", G_CALLBACK(dialog_response), &gui);
+    g_signal_connect_after(dialog, "response", G_CALLBACK(dialog_response), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
     outcome = gwy_dialog_run(dialog);
@@ -426,6 +430,7 @@ set_selection(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     ImageMode image_mode = gwy_params_get_enum(args->params, PARAM_IMAGE_MODE);
+    gboolean single_vector = gwy_params_get_boolean(args->params, PARAM_SINGLE_VECTOR);
     GwyVectorLayer *vlayer = gwy_data_view_get_top_layer(GWY_DATA_VIEW(gui->dataview));
     GwySelection *selection = gwy_vector_layer_ensure_selection(vlayer);
     GwyDataField *field = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(image_mode));
@@ -442,7 +447,7 @@ set_selection(ModuleGUI *gui)
     else if (g_type_is_a(G_OBJECT_TYPE(selection), g_type_from_name("GwySelectionPoint"))) {
         /* Point selections have origin of real coordinates in the top left corner. */
         center_selection(field, xy, 2, 1);
-        gwy_selection_set_data(selection, 2, xy);
+        gwy_selection_set_data(selection, single_vector ? 1 : 2, xy);
     }
 }
 
@@ -451,6 +456,7 @@ get_selection(ModuleGUI *gui, gdouble *xy)
 {
     ModuleArgs *args = gui->args;
     ImageMode image_mode = gwy_params_get_enum(args->params, PARAM_IMAGE_MODE);
+    gboolean single_vector = gwy_params_get_boolean(args->params, PARAM_SINGLE_VECTOR);
     GwyVectorLayer *vlayer = gwy_data_view_get_top_layer(GWY_DATA_VIEW(gui->dataview));
     GwySelection *selection = gwy_vector_layer_ensure_selection(vlayer);
     GwyDataField *field = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(image_mode));
@@ -462,7 +468,13 @@ get_selection(ModuleGUI *gui, gdouble *xy)
     gwy_debug("image-space sel: (%g, %g) (%g, %g)", xy[0], xy[1], xy[2], xy[3]);
     /* Point selections have origin of real coordinates in the top left corner. */
     if (g_type_is_a(G_OBJECT_TYPE(selection), g_type_from_name("GwySelectionPoint")))
-        center_selection(field, xy, 2, -1);
+        center_selection(field, xy, single_vector ? 1 : 2, -1);
+    /* Keep an orthogonal vector of the same length (which we do not show) in the single vector mode.  It makes life
+     * easier because we can basically pretend we have a lattice then. */
+    if (single_vector) {
+        xy[2] = xy[1];
+        xy[3] = -xy[0];
+    }
     if (image_mode == IMAGE_PSDF)
         transform_selection(xy);
 
@@ -495,6 +507,7 @@ switch_selection_mode(ModuleGUI *gui)
     ModuleArgs *args = gui->args;
     SelectionMode sel_mode = gwy_params_get_enum(args->params, PARAM_SELECTION_MODE);
     gboolean show_numbers = gwy_params_get_boolean(args->params, PARAM_SHOW_NUMBERS);
+    gboolean single_vector = gwy_params_get_boolean(args->params, PARAM_SINGLE_VECTOR);
     GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
     GwyVectorLayer *vlayer = gwy_data_view_get_top_layer(dataview);
     GwySelection *selection = gwy_vector_layer_ensure_selection(vlayer);
@@ -505,12 +518,12 @@ switch_selection_mode(ModuleGUI *gui)
         gwy_param_table_set_sensitive(gui->table, PARAM_SHOW_NUMBERS, FALSE);
     }
     else {
-        selection = gwy_create_preview_vector_layer(dataview, 0, "Point", 2, TRUE);
+        selection = gwy_create_preview_vector_layer(dataview, 0, "Point", single_vector ? 1 : 2, TRUE);
         g_object_set(gwy_data_view_get_top_layer(dataview),
                      "draw-as-vector", TRUE,
-                     "point-numbers", show_numbers,
+                     "point-numbers", show_numbers && !single_vector,
                      NULL);
-        gwy_param_table_set_sensitive(gui->table, PARAM_SHOW_NUMBERS, TRUE);
+        gwy_param_table_set_sensitive(gui->table, PARAM_SHOW_NUMBERS, !single_vector);
     }
     set_selection(gui);
     gui->selection_id = g_signal_connect_swapped(selection, "changed", G_CALLBACK(selection_changed), gui);
@@ -566,8 +579,7 @@ calculate_psdf(ModuleGUI *gui)
 
     field = gwy_data_field_duplicate(field);
     gwy_data_field_add(field, -gwy_data_field_get_avg(field));
-    gwy_fft_window_data_field(field, GWY_ORIENTATION_HORIZONTAL, windowing);
-    gwy_fft_window_data_field(field, GWY_ORIENTATION_VERTICAL, windowing);
+    gwy_data_field_fft_window(field, windowing);
     extfield = gwy_data_field_extend(field, 0, extxres - xres, 0, extyres - yres,
                                      GWY_EXTERIOR_FIXED_VALUE, 0.0, FALSE);
     GWY_OBJECT_UNREF(field);
@@ -641,7 +653,7 @@ calculate_zoomed_field(ModuleGUI *gui)
         if (height < 17)
             height = MAX(height, MIN(17, yres));
 
-        zoomed = gwy_data_field_area_extract(field, (xres - width)/2, (yres - height)/2, width, height);
+        zoomed = gwy_data_field_area_extract(field, (xres - width + 1)/2, (yres - height + 1)/2, width, height);
         gwy_data_field_set_xoffset(zoomed, -0.5*gwy_data_field_get_xreal(zoomed));
         gwy_data_field_set_yoffset(zoomed, -0.5*gwy_data_field_get_yreal(zoomed));
     }
@@ -655,6 +667,7 @@ param_changed(ModuleGUI *gui, gint id)
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
     ImageMode image_mode = gwy_params_get_enum(params, PARAM_IMAGE_MODE);
+    gboolean single_vector = gwy_params_get_boolean(params, PARAM_SINGLE_VECTOR);
 
     if (id < 0 || id == PARAM_IMAGE_MODE) {
         gwy_param_table_set_sensitive(gui->table, PARAM_ZOOM, image_mode != IMAGE_DATA);
@@ -664,6 +677,11 @@ param_changed(ModuleGUI *gui, gint id)
             gwy_param_table_set_enum(gui->table, PARAM_ZOOM, gwy_params_get_enum(params, PARAM_ZOOM_PSDF));
         switch_display(gui);
     }
+    if (id < 0 || id == PARAM_SINGLE_VECTOR) {
+        gwy_param_table_set_sensitive(gui->table, PARAM_SELECTION_MODE, !single_vector);
+        if (single_vector)
+            gwy_param_table_set_enum(gui->table, PARAM_SELECTION_MODE, SELECTION_POINT);
+    }
     if (id < 0 || id == PARAM_ZOOM) {
         if (image_mode == IMAGE_ACF)
             gwy_params_set_enum(params, PARAM_ZOOM_ACF, gwy_params_get_enum(params, PARAM_ZOOM));
@@ -674,13 +692,13 @@ param_changed(ModuleGUI *gui, gint id)
         gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SIZE);
         set_selection(gui);
     }
-    if (id < 0 || id == PARAM_SELECTION_MODE)
+    if (id < 0 || id == PARAM_SELECTION_MODE || id == PARAM_SINGLE_VECTOR)
         switch_selection_mode(gui);
     if (id < 0 || id == PARAM_SHOW_NUMBERS) {
         GwyVectorLayer *vlayer = gwy_data_view_get_top_layer(GWY_DATA_VIEW(gui->dataview));
         if (g_type_is_a(G_OBJECT_TYPE(vlayer), g_type_from_name("GwyLayerPoint"))) {
             gboolean show_numbers = gwy_params_get_boolean(args->params, PARAM_SHOW_NUMBERS);
-            g_object_set(vlayer, "point-numbers", show_numbers, NULL);
+            g_object_set(vlayer, "point-numbers", show_numbers && !single_vector, NULL);
         }
     }
 
@@ -699,13 +717,13 @@ preview(gpointer user_data)
     gwy_data_field_data_changed(gui->acf);
     gwy_data_field_data_changed(gui->psdf);
     if (!gui->args->have_xy)
-        dialog_response(gui, RESPONSE_ESTIMATE);
+        dialog_response(GWY_DIALOG(gui->dialog), RESPONSE_ESTIMATE, gui);
     /* Does not do anything really useful. */
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
 static void
-dialog_response(ModuleGUI *gui, gint response)
+dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     ImageMode image_mode = gwy_params_get_enum(args->params, PARAM_IMAGE_MODE);
@@ -761,13 +779,16 @@ static void
 selection_changed(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
+    gboolean single_vector = gwy_params_get_boolean(args->params, PARAM_SINGLE_VECTOR);
     gdouble a1, a2, phi1, phi2, phi, A;
     GString *str;
     gdouble xy[4];
+    gint i;
 
     if (!get_selection(gui, xy)) {
         gwy_results_set_na(gui->results, "a1x", "a1y", "a1", "phi1", "a2x", "a2y", "a2", "phi2", "phi", "A", NULL);
-        /* FIXME: We should also clear labels, right? */
+        for (i = 0; i < VALUE_NVALUES; i++)
+            gtk_label_set_text(GTK_LABEL(gui->value_labels[i]), "");
         return;
     }
     gwy_assign(args->xy, xy, 4);
@@ -779,22 +800,34 @@ selection_changed(ModuleGUI *gui)
     phi = gwy_canonicalize_angle(phi2 - phi1, TRUE, TRUE);
     A = fabs(matrix_det(xy));
 
-    gwy_results_fill_values(gui->results,
-                            "a1x", xy[0], "a1y", -xy[1], "a1", a1, "phi1", phi1,
-                            "a2x", xy[2], "a2y", -xy[3], "a2", a2, "phi2", phi2,
-                            "phi", phi, "A", A,
-                            NULL);
+    if (single_vector) {
+        gwy_results_fill_values(gui->results, "a1x", xy[0], "a1y", -xy[1], "a1", a1, "phi1", phi1, NULL);
+        gwy_results_set_na(gui->results, "a2x", "a2y", "a2", "phi2", "phi", "A", NULL);
+    }
+    else {
+        gwy_results_fill_values(gui->results,
+                                "a1x", xy[0], "a1y", -xy[1], "a1", a1, "phi1", phi1,
+                                "a2x", xy[2], "a2y", -xy[3], "a2", a2, "phi2", phi2,
+                                "phi", phi, "A", A,
+                                NULL);
+    }
 
     str = g_string_new(NULL);
     update_value_label(gui->value_labels[VALUE_A1_X], xy[0], gui->xyvf, str);
     update_value_label(gui->value_labels[VALUE_A1_Y], -xy[1], gui->xyvf, str);
     update_value_label(gui->value_labels[VALUE_A1], a1, gui->xyvf, str);
     update_value_label(gui->value_labels[VALUE_PHI1], phi1, gui->phivf, str);
-    update_value_label(gui->value_labels[VALUE_A2_X], xy[2], gui->xyvf, str);
-    update_value_label(gui->value_labels[VALUE_A2_Y], -xy[3], gui->xyvf, str);
-    update_value_label(gui->value_labels[VALUE_A2], a2, gui->xyvf, str);
-    update_value_label(gui->value_labels[VALUE_PHI2], phi2, gui->phivf, str);
-    update_value_label(gui->value_labels[VALUE_PHI], phi, gui->phivf, str);
+    if (single_vector) {
+        for (i = VALUE_A2_X; i < VALUE_NVALUES; i++)
+            gtk_label_set_text(GTK_LABEL(gui->value_labels[i]), "");
+    }
+    else {
+        update_value_label(gui->value_labels[VALUE_A2_X], xy[2], gui->xyvf, str);
+        update_value_label(gui->value_labels[VALUE_A2_Y], -xy[3], gui->xyvf, str);
+        update_value_label(gui->value_labels[VALUE_A2], a2, gui->xyvf, str);
+        update_value_label(gui->value_labels[VALUE_PHI2], phi2, gui->phivf, str);
+        update_value_label(gui->value_labels[VALUE_PHI], phi, gui->phivf, str);
+    }
     g_string_free(str, TRUE);
 }
 
