@@ -1,22 +1,18 @@
 /*
- *  $Id: neural.c 23556 2021-04-21 08:00:51Z yeti-dn $
+ *  $Id: neural.c 24995 2022-09-02 14:40:53Z yeti-dn $
  *  Copyright (C) 2012 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -137,9 +133,6 @@ static gboolean can_select_network          (GtkTreeSelection *selection,
                                              gboolean path_currently_selected,
                                              gpointer data);
 static void     neural_train_update_controls(NeuralTrainControls *controls);
-static gboolean network_is_visible          (GtkTreeModel *model,
-                                             GtkTreeIter *iter,
-                                             gpointer user_data);
 static void     preview_type_changed        (GtkToggleButton *button,
                                              NeuralTrainControls *controls);
 static void     train_data_changed          (NeuralTrainControls *controls,
@@ -230,16 +223,14 @@ module_register(void)
                               GWY_STOCK_NEURAL_TRAIN,
                               NEURAL_TRAIN_RUN_MODES,
                               GWY_MENU_FLAG_DATA,
-                              N_("Train a neural network for image "
-                                 "processing"));
+                              N_("Train a neural network for image processing"));
     gwy_process_func_register("neural_apply",
                               (GwyProcessFunc)&neural_apply,
                               N_("/M_ultidata/Apply _Neural Network..."),
                               GWY_STOCK_NEURAL_APPLY,
                               NEURAL_APPLY_RUN_MODES,
                               GWY_MENU_FLAG_DATA,
-                              N_("Process data using a trained "
-                                 "neural network"));
+                              N_("Process data using a trained neural network"));
 
     return TRUE;
 }
@@ -331,8 +322,7 @@ setup_container(GwyContainer *mydata,
     diff = gwy_data_field_new_alike(tsignal, TRUE);
 
     gwy_container_set_object_by_name(mydata, "/0/data", tmodel);
-    gwy_app_sync_data_items(modeldata, mydata, args->tmodel.id, 0,
-                            FALSE,
+    gwy_app_sync_data_items(modeldata, mydata, args->tmodel.id, 0, FALSE,
                             GWY_DATA_ITEM_REAL_SQUARE,
                             GWY_DATA_ITEM_GRADIENT,
                             GWY_DATA_ITEM_RANGE_TYPE,
@@ -340,8 +330,7 @@ setup_container(GwyContainer *mydata,
                             0);
 
     gwy_container_set_object_by_name(mydata, "/1/data", tsignal);
-    gwy_app_sync_data_items(signaldata, mydata, args->tsignal.id, 1,
-                            FALSE,
+    gwy_app_sync_data_items(signaldata, mydata, args->tsignal.id, 1, FALSE,
                             GWY_DATA_ITEM_REAL_SQUARE,
                             GWY_DATA_ITEM_GRADIENT,
                             GWY_DATA_ITEM_RANGE_TYPE,
@@ -350,8 +339,7 @@ setup_container(GwyContainer *mydata,
 
     gwy_container_set_object_by_name(mydata, "/2/data", result);
     g_object_unref(result);
-    gwy_app_sync_data_items(signaldata, mydata, args->tsignal.id, 2,
-                            FALSE,
+    gwy_app_sync_data_items(signaldata, mydata, args->tsignal.id, 2, FALSE,
                             GWY_DATA_ITEM_REAL_SQUARE,
                             GWY_DATA_ITEM_GRADIENT,
                             GWY_DATA_ITEM_RANGE_TYPE,
@@ -360,8 +348,7 @@ setup_container(GwyContainer *mydata,
 
     gwy_container_set_object_by_name(mydata, "/3/data", diff);
     g_object_unref(diff);
-    gwy_app_sync_data_items(signaldata, mydata, args->tsignal.id, 3,
-                            FALSE,
+    gwy_app_sync_data_items(signaldata, mydata, args->tsignal.id, 3, FALSE,
                             GWY_DATA_ITEM_REAL_SQUARE,
                             GWY_DATA_ITEM_GRADIENT,
                             0);
@@ -429,20 +416,14 @@ create_network_list(GtkTreeModel *model,
 
     for (i = 0; i < G_N_ELEMENTS(columns); i++) {
         renderer = gtk_cell_renderer_text_new();
-        column = gtk_tree_view_column_new_with_attributes(_(columns[i].name),
-                                                          renderer,
-                                                          NULL);
-        gtk_tree_view_column_set_cell_data_func
-                                         (column, renderer,
-                                          network_cell_renderer,
-                                          GUINT_TO_POINTER(columns[i].value),
-                                          NULL);  /* destroy notify */
+        column = gtk_tree_view_column_new_with_attributes(_(columns[i].name), renderer, NULL);
+        gtk_tree_view_column_set_cell_data_func(column, renderer,
+                                                network_cell_renderer, GUINT_TO_POINTER(columns[i].value), NULL);
         gtk_tree_view_append_column(GTK_TREE_VIEW(networklist), column);
     }
 
     *scroll = gtk_scrolled_window_new(NULL, NULL);
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(*scroll),
-                                   GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(*scroll), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
     gtk_container_add(GTK_CONTAINER(*scroll), networklist);
 
     return networklist;
@@ -682,13 +663,9 @@ neural_train_dialog(NeuralTrainArgs *args)
                              gtk_label_new(_("Networks")));
 
     controls.store = gwy_inventory_store_new(gwy_neural_networks());
-    filtermodel = gtk_tree_model_filter_new(GTK_TREE_MODEL(controls.store),
-                                            NULL);
-    g_object_unref(controls.store);
-    gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(filtermodel),
-                                           &network_is_visible,
-                                           NULL, NULL);
+    filtermodel = gwy_create_inventory_model_without_default(controls.store);
     controls.networklist = create_network_list(filtermodel, &scroll);
+    g_object_unref(controls.store);
     g_object_unref(filtermodel);
     gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);
 
@@ -878,17 +855,6 @@ neural_train_update_controls(NeuralTrainControls *controls)
     controls->in_update = FALSE;
 }
 
-static gboolean
-network_is_visible(GtkTreeModel *model,
-                   GtkTreeIter *iter,
-                   G_GNUC_UNUSED gpointer user_data)
-{
-    GwyNeuralNetwork *network;
-    gtk_tree_model_get(model, iter, 0, &network, -1);
-    return !gwy_strequal(gwy_resource_get_name(GWY_RESOURCE(network)),
-                         GWY_NEURAL_NETWORK_UNTITLED);
-}
-
 static void
 preview_type_changed(G_GNUC_UNUSED GtkToggleButton *button,
                      NeuralTrainControls *controls)
@@ -940,8 +906,7 @@ train_data_changed(NeuralTrainControls *controls,
             || gwy_data_field_get_xres(tsignal) <= nndata->width
             || gwy_data_field_get_yres(tsignal) <= nndata->height) {
             ok = FALSE;
-            message = _("A field dimension is too small for chosen window "
-                        "size.");
+            message = _("A field dimension is too small for chosen window size.");
         }
     }
 
@@ -962,12 +927,9 @@ train_data_changed(NeuralTrainControls *controls,
         g_critical("Chooser lacks id");
     }
 
-    gwy_set_data_preview_size(GWY_DATA_VIEW(controls->view),
-                              PREVIEW_SMALL_SIZE);
-    gtk_widget_set_sensitive(gwy_radio_buttons_find(group, PREVIEW_RESULT),
-                             FALSE);
-    gtk_widget_set_sensitive(gwy_radio_buttons_find(group, PREVIEW_DIFFERENCE),
-                             FALSE);
+    gwy_set_data_preview_size(GWY_DATA_VIEW(controls->view), PREVIEW_SMALL_SIZE);
+    gtk_widget_set_sensitive(gwy_radio_buttons_find(group, PREVIEW_RESULT), FALSE);
+    gtk_widget_set_sensitive(gwy_radio_buttons_find(group, PREVIEW_DIFFERENCE), FALSE);
 
     if (gwy_strequal(id, "signal")) {
         GSList *l;
@@ -1127,10 +1089,8 @@ train_do(GwyNeuralNetwork *nn, GwyDataLine *errors, GwyGraphCurveModel *gcmodel,
     for (row = height/2; row < yres + height/2 - height; row++) {
         for (col = width/2; col < xres + width/2 - width; col++) {
             if (!dtmask
-                || (masking == GWY_MASK_INCLUDE
-                    && dtmask[row*xres + col] >= 1.0)
-                || (masking == GWY_MASK_EXCLUDE
-                    && dtmask[row*xres + col] <= 0.0))
+                || (masking == GWY_MASK_INCLUDE && dtmask[row*xres + col] >= 1.0)
+                || (masking == GWY_MASK_EXCLUDE && dtmask[row*xres + col] <= 0.0))
                 indices[npixels++] = (row - height/2)*xres + col - width/2;
         }
     }
@@ -1141,20 +1101,14 @@ train_do(GwyNeuralNetwork *nn, GwyDataLine *errors, GwyGraphCurveModel *gcmodel,
         /* shuffle(indices, (xres - width)*(yres - height), rng); even though
          * it may improve convergence. */
         for (k = 0; k < npixels; k++) {
-            for (irow = 0; irow < height; irow++) {
-                gwy_assign(nn->input + irow*width,
-                           dtmodel + indices[k] + irow*xres,
-                           width);
-            }
-            nn->target[0] = sfactor*(dtsignal[indices[k]
-                                              + height/2*xres + width/2]
-                                              - sshift);
+            for (irow = 0; irow < height; irow++)
+                gwy_assign(nn->input + irow*width, dtmodel + indices[k] + irow*xres, width);
+            nn->target[0] = sfactor*(dtsignal[indices[k] + height/2*xres + width/2] - sshift);
             gwy_neural_network_train_step(nn, 0.3, 0.3, &eo, &eh);
         }
         gwy_data_line_set_val(errors, n, eo + eh);
         if (g_timer_elapsed(timer, NULL) - lasttime >= 0.2) {
-            gwy_graph_curve_model_set_data_from_dataline(gcmodel, errors,
-                                                         0, n+1);
+            gwy_graph_curve_model_set_data_from_dataline(gcmodel, errors, 0, n+1);
             if (!gwy_app_wait_set_fraction((gdouble)n/trainsteps))
                 goto fail;
             lasttime = g_timer_elapsed(timer, NULL);
@@ -1197,12 +1151,8 @@ evaluate_do(GwyNeuralNetwork *nn,
 
     for (row = height/2; row < yres + height/2 - height; row++) {
         for (col = width/2; col < xres + width/2 - width; col++) {
-            for (irow = 0; irow < height; irow++) {
-                gwy_assign(nn->input + irow*width,
-                           drmodel + ((row + irow - height/2)*xres
-                                      + col - width/2),
-                           width);
-            }
+            for (irow = 0; irow < height; irow++)
+                gwy_assign(nn->input + irow*width, drmodel + ((row + irow - height/2)*xres + col - width/2), width);
             gwy_neural_network_forward_feed(nn);
             dresult[row*xres + col] = nn->output[0]/sfactor + sshift;
         }
@@ -1212,29 +1162,16 @@ evaluate_do(GwyNeuralNetwork *nn,
     ok = TRUE;
     unit = gwy_data_field_get_si_unit_z(result);
     gwy_si_unit_set_from_string(unit, nndata->outunits);
-    gwy_si_unit_power_multiply(unit, 1,
-                               gwy_data_field_get_si_unit_xy(model),
-                               nndata->inpowerxy,
-                               unit);
-    gwy_si_unit_power_multiply(unit, 1,
-                               gwy_data_field_get_si_unit_z(model),
-                               nndata->inpowerz,
-                               unit);
+    gwy_si_unit_power_multiply(unit, 1, gwy_data_field_get_si_unit_xy(model), nndata->inpowerxy, unit);
+    gwy_si_unit_power_multiply(unit, 1, gwy_data_field_get_si_unit_z(model), nndata->inpowerz, unit);
 
     /* Fill the borders with the average of result. */
     avg = gwy_data_field_area_get_avg_mask(result, NULL, GWY_MASK_IGNORE,
-                                           width/2, height/2,
-                                           xres - width, yres - height);
+                                           width/2, height/2, xres - width, yres - height);
     gwy_data_field_area_fill(result, 0, 0, xres, height/2, avg);
     gwy_data_field_area_fill(result, 0, height/2, width/2, yres - height, avg);
-    gwy_data_field_area_fill(result,
-                             xres + width/2 - width, height/2,
-                             width - width/2, yres - height,
-                             avg);
-    gwy_data_field_area_fill(result,
-                             0, yres + height/2 - height,
-                             xres, height - height/2,
-                             avg);
+    gwy_data_field_area_fill(result, xres + width/2 - width, height/2, width - width/2, yres - height, avg);
+    gwy_data_field_area_fill(result, 0, yres + height/2 - height, xres, height - height/2, avg);
 
 fail:
     g_object_unref(scaled);
@@ -1276,14 +1213,10 @@ train_network(NeuralTrainControls *controls)
 
     gwy_app_wait_start(GTK_WINDOW(controls->dialog), _("Initializing..."));
 
-    tmodel = GWY_DATA_FIELD(gwy_container_get_object_by_name(controls->mydata,
-                                                             "/0/data"));
-    tsignal = GWY_DATA_FIELD(gwy_container_get_object_by_name(controls->mydata,
-                                                              "/1/data"));
-    result = GWY_DATA_FIELD(gwy_container_get_object_by_name(controls->mydata,
-                                                             "/2/data"));
-    diff = GWY_DATA_FIELD(gwy_container_get_object_by_name(controls->mydata,
-                                                           "/3/data"));
+    tmodel = GWY_DATA_FIELD(gwy_container_get_object_by_name(controls->mydata, "/0/data"));
+    tsignal = GWY_DATA_FIELD(gwy_container_get_object_by_name(controls->mydata, "/1/data"));
+    result = GWY_DATA_FIELD(gwy_container_get_object_by_name(controls->mydata, "/2/data"));
+    diff = GWY_DATA_FIELD(gwy_container_get_object_by_name(controls->mydata, "/3/data"));
 
     signaldata = gwy_app_data_browser_get(args->tsignal.datano);
     quark = gwy_app_get_mask_key_for_id(args->tsignal.id);
@@ -1297,8 +1230,7 @@ train_network(NeuralTrainControls *controls)
     neural_network_data_copy(&nn->data, &backup);
 
     gwy_resource_use(GWY_RESOURCE(nn));
-    ok = (train_do(nn, errors, gcmodel, tmodel, tsignal, mask, args->masking,
-                   sfactor, sshift, args->trainsteps)
+    ok = (train_do(nn, errors, gcmodel, tmodel, tsignal, mask, args->masking, sfactor, sshift, args->trainsteps)
           && evaluate_do(nn, tmodel, result, sfactor, sshift));
     gwy_resource_release(GWY_RESOURCE(nn));
 
@@ -1310,8 +1242,7 @@ train_network(NeuralTrainControls *controls)
     g_object_unref(errors);
 
     if (!ok) {
-        gtk_label_set_text(GTK_LABEL(controls->message),
-                           _("Training was canceled."));
+        gtk_label_set_text(GTK_LABEL(controls->message), _("Training was canceled."));
         return;
     }
 
@@ -1319,12 +1250,9 @@ train_network(NeuralTrainControls *controls)
     gwy_data_field_data_changed(result);
     gwy_data_field_data_changed(diff);
 
-    vf = gwy_data_field_get_value_format_z(diff, GWY_SI_UNIT_FORMAT_VFMARKUP,
-                                           NULL);
+    vf = gwy_data_field_get_value_format_z(diff, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
     s = g_strdup_printf(_("Mean difference: %.*f %s"),
-                        vf->precision,
-                        calculate_mean_error(diff)/vf->magnitude,
-                        vf->units);
+                        vf->precision, calculate_mean_error(diff)/vf->magnitude, vf->units);
     gwy_si_unit_value_format_free(vf);
     gtk_label_set_markup(GTK_LABEL(controls->message), s);
     g_free(s);
@@ -1350,8 +1278,7 @@ neural_apply_do(NeuralApplyArgs *args,
     GwyDataField *result;
     gboolean ok;
 
-    gwy_app_wait_start(gwy_app_find_window_for_channel(data, id),
-                       _("Evaluating..."));
+    gwy_app_wait_start(gwy_app_find_window_for_channel(data, id), _("Evaluating..."));
 
     network = gwy_inventory_get_item(gwy_neural_networks(), args->name);
     g_assert(network);
@@ -1374,11 +1301,8 @@ neural_apply_do(NeuralApplyArgs *args,
     if (ok) {
         gint newid = gwy_app_data_browser_add_data_field(result, data, TRUE);
         gwy_app_set_data_field_title(data, newid, _("Evaluated signal"));
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_GRADIENT, 0);
-        gwy_app_channel_log_add(data, id, newid, "proc::neural_apply",
-                                "settings-name", "neural",
-                                NULL);
+        gwy_app_sync_data_items(data, data, id, newid, FALSE, GWY_DATA_ITEM_GRADIENT, 0);
+        gwy_app_channel_log_add(data, id, newid, "proc::neural_apply", "settings-name", "neural", NULL);
     }
     g_object_unref(result);
 }
@@ -1408,10 +1332,6 @@ network_store(NeuralTrainControls *controls)
 {
     GwyNeuralNetwork *network;
     NeuralNetworkData *nndata;
-    GtkTreeModelFilter *filter;
-    GtkTreeModel *model, *imodel;
-    GtkTreeSelection *tselect;
-    GtkTreeIter iter, iiter;
     const gchar *name;
 
     nndata = &controls->args->nn->data;
@@ -1431,16 +1351,10 @@ network_store(NeuralTrainControls *controls)
         neural_network_data_copy(nndata, &network->data);
         gwy_resource_data_changed(GWY_RESOURCE(network));
     }
-    GWY_RESOURCE(network)->is_modified = TRUE;
+    gwy_resource_data_changed(GWY_RESOURCE(network));
     gwy_neural_network_save(network);
 
-    model = gtk_tree_view_get_model(GTK_TREE_VIEW(controls->networklist));
-    tselect = gtk_tree_view_get_selection(GTK_TREE_VIEW(controls->networklist));
-    filter = GTK_TREE_MODEL_FILTER(model);
-    imodel = gtk_tree_model_filter_get_model(filter);
-    gwy_inventory_store_get_iter(GWY_INVENTORY_STORE(imodel), name, &iiter);
-    gtk_tree_model_filter_convert_child_iter_to_iter(filter, &iter, &iiter);
-    gtk_tree_selection_select_iter(tselect, &iter);
+    gwy_select_in_filtered_inventory_treeeview(GTK_TREE_VIEW(controls->networklist), name);
 }
 
 static void
@@ -1463,11 +1377,10 @@ static void
 network_rename(NeuralTrainControls *controls)
 {
     GwyNeuralNetwork *network;
-    GtkTreeModelFilter *filter;
     GwyInventory *inventory;
-    GtkTreeModel *model, *imodel;
+    GtkTreeModel *model;
     GtkTreeSelection *tselect;
-    GtkTreeIter iter, iiter;
+    GtkTreeIter iter;
     const gchar *newname, *oldname;
 
     tselect = gtk_tree_view_get_selection(GTK_TREE_VIEW(controls->networklist));
@@ -1486,11 +1399,7 @@ network_rename(NeuralTrainControls *controls)
     gwy_debug("Now I will rename `%s' to `%s'", oldname, newname);
     gwy_resource_rename(GWY_RESOURCE(network), newname);
 
-    filter = GTK_TREE_MODEL_FILTER(model);
-    imodel = gtk_tree_model_filter_get_model(filter);
-    gwy_inventory_store_get_iter(GWY_INVENTORY_STORE(imodel), newname, &iiter);
-    gtk_tree_model_filter_convert_child_iter_to_iter(filter, &iter, &iiter);
-    gtk_tree_selection_select_iter(tselect, &iter);
+    gwy_select_in_filtered_inventory_treeeview(GTK_TREE_VIEW(controls->networklist), newname);
 }
 
 static void
@@ -1561,16 +1470,12 @@ network_validate_name(NeuralTrainControls *controls,
 
     parent = controls->dialog;
     dialog = gtk_message_dialog_new(GTK_WINDOW(parent),
-                                    GTK_DIALOG_MODAL
-                                        | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_MESSAGE_INFO,
                                     GTK_BUTTONS_CLOSE,
                                     _("The name `%s' is invalid."),
                                     name);
-    gtk_window_set_modal(GTK_WINDOW(parent), FALSE);  /* Bug #66 workaround. */
-    gtk_dialog_run(GTK_DIALOG(dialog));
-    gtk_widget_destroy(dialog);
-    gtk_window_set_modal(GTK_WINDOW(parent), TRUE);  /* Bug #66 workaround. */
+    gwy_run_subdialog(GTK_DIALOG(dialog));
 
     return FALSE;
 }
@@ -1603,8 +1508,7 @@ static void
 neural_train_sanitize_args(NeuralTrainArgs *args)
 {
     args->trainsteps = MIN(args->trainsteps, 10000);
-    args->masking = gwy_enum_sanitize_value(args->masking,
-                                            GWY_TYPE_MASKING_TYPE);
+    args->masking = gwy_enum_sanitize_value(args->masking, GWY_TYPE_MASKING_TYPE);
     gwy_app_data_id_verify_channel(&args->tsignal);
 }
 
@@ -1619,10 +1523,8 @@ neural_train_load_args(GwyContainer *settings,
     args->trainsteps = trainsteps_default;
     args->masking = masking_default;
 
-    gwy_container_gis_int32_by_name(settings, trainsteps_key,
-                                    &args->trainsteps);
-    gwy_container_gis_enum_by_name(settings, masking_key,
-                                   &args->masking);
+    gwy_container_gis_int32_by_name(settings, trainsteps_key, &args->trainsteps);
+    gwy_container_gis_enum_by_name(settings, masking_key, &args->masking);
     args->tsignal = nnsignal_id;
     neural_train_sanitize_args(args);
 
@@ -1638,10 +1540,8 @@ neural_train_save_args(GwyContainer *settings,
                        NeuralTrainArgs *args)
 {
     nnsignal_id = args->tsignal;
-    gwy_container_set_int32_by_name(settings, trainsteps_key,
-                                    args->trainsteps);
-    gwy_container_set_enum_by_name(settings, masking_key,
-                                   args->masking);
+    gwy_container_set_int32_by_name(settings, trainsteps_key, args->trainsteps);
+    gwy_container_set_enum_by_name(settings, masking_key, args->masking);
     gwy_neural_network_save(args->nn);
 }
 
@@ -1660,10 +1560,8 @@ neural_apply_load_args(GwyContainer *settings,
 {
     *args = neural_apply_defaults;
     args->name = GWY_NEURAL_NETWORK_UNTITLED;
-    gwy_container_gis_string_by_name(settings, name_key,
-                                     (const guchar**)&args->name);
-    gwy_container_gis_boolean_by_name(settings, scale_output_key,
-                                      &args->scale_output);
+    gwy_container_gis_string_by_name(settings, name_key, (const guchar**)&args->name);
+    gwy_container_gis_boolean_by_name(settings, scale_output_key, &args->scale_output);
     args->name = g_strdup(args->name);
     neural_apply_sanitize_args(args);
 }
@@ -1672,48 +1570,31 @@ static void
 neural_apply_save_args(GwyContainer *settings,
                        NeuralApplyArgs *args)
 {
-    gwy_container_set_string_by_name(settings, name_key,
-                                     g_strdup(args->name));
-    gwy_container_set_boolean_by_name(settings, scale_output_key,
-                                      args->scale_output);
+    gwy_container_set_string_by_name(settings, name_key, g_strdup(args->name));
+    gwy_container_set_boolean_by_name(settings, scale_output_key, args->scale_output);
 }
 
 static gboolean
 gwy_neural_network_save(GwyNeuralNetwork *nn)
 {
-    GwyResource *resource;
-    GString *str;
-    gchar *filename;
-    FILE *fh;
+    GwyResource *resource = GWY_RESOURCE(nn);
+    GError *error = NULL;
 
-    resource = GWY_RESOURCE(nn);
     if (!resource->is_modified)
         return TRUE;
 
     if (!gwy_resource_get_is_modifiable(resource)) {
-        g_warning("Non-modifiable resource was modified and is about to be "
-                  "saved");
+        g_warning("Non-modifiable resource was modified and is about to be saved");
         return FALSE;
     }
-
-    filename = gwy_resource_build_filename(resource);
-    fh = gwy_fopen(filename, "w");
-    if (!fh) {
+    if (!gwy_resource_save(resource, &error)) {
         /* FIXME: GUIze this */
-        g_warning("Cannot save resource file: %s", filename);
-        g_free(filename);
+        g_warning("Cannot save resource %s: %s", gwy_resource_get_name(resource), error->message);
+        g_clear_error(&error);
         return FALSE;
     }
-    g_free(filename);
-
-    str = gwy_resource_dump(resource);
-    fwrite(str->str, 1, str->len, fh);
-    fclose(fh);
-    g_string_free(str, TRUE);
-
-    gwy_resource_data_saved(resource);
 
     return TRUE;
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
