@@ -1,5 +1,5 @@
 /*
- *  $Id: polydistort.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: polydistort.c 23556 2021-04-21 08:00:51Z yeti-dn $
  *  Copyright (C) 2007,2008 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -232,7 +232,7 @@ distort_dialog(DistortArgs *args,
                             GWY_DATA_ITEM_RANGE,
                             GWY_DATA_ITEM_REAL_SQUARE,
                             0);
-    controls.view = create_preview(controls.mydata, 0, PREVIEW_SIZE, FALSE);
+    controls.view = gwy_create_preview(controls.mydata, 0, PREVIEW_SIZE, FALSE);
     g_object_unref(controls.mydata);
     gtk_container_add(GTK_CONTAINER(align), controls.view);
 
