@@ -1,5 +1,5 @@
 /*
- *  $Id: polylevel.c 23985 2021-08-16 12:27:37Z yeti-dn $
+ *  $Id: polylevel.c 24707 2022-03-21 17:09:03Z yeti-dn $
  *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -107,7 +107,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Subtracts polynomial background."),
     "Yeti <yeti@gwyddion.net>",
-    "4.0",
+    "4.1",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -386,9 +386,6 @@ create_coeff_view(ModuleGUI *gui, GtkBox *hbox)
     GtkTreeViewColumn *column;
     GtkCellRenderer *renderer;
     GtkTreeSelection *selection;
-    GtkTooltips *tips;
-
-    tips = gwy_app_get_tooltips();
 
     coeffvbox = gtk_vbox_new(FALSE, 0);
     gtk_box_pack_start(hbox, coeffvbox, FALSE, FALSE, 0);
@@ -432,14 +429,14 @@ create_coeff_view(ModuleGUI *gui, GtkBox *hbox)
 
     button = gtk_button_new();
     gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-    gtk_tooltips_set_tip(tips, button, _("Save table to a file"), NULL);
+    gtk_widget_set_tooltip_text(button, _("Save table to a file"));
     gtk_container_add(GTK_CONTAINER(button), gtk_image_new_from_stock(GTK_STOCK_SAVE, GTK_ICON_SIZE_SMALL_TOOLBAR));
     gtk_box_pack_end(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
     g_signal_connect_swapped(button, "clicked", G_CALLBACK(save_coeffs), gui);
 
     button = gtk_button_new();
     gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-    gtk_tooltips_set_tip(tips, button, _("Copy table to clipboard"), NULL);
+    gtk_widget_set_tooltip_text(button, _("Copy table to clipboard"));
     gtk_container_add(GTK_CONTAINER(button), gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_SMALL_TOOLBAR));
     gtk_box_pack_end(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
     g_signal_connect_swapped(button, "clicked", G_CALLBACK(copy_coeffs), gui);
