@@ -152,6 +152,7 @@ GwyModuleInfo* _gwy_module_query__trimmed_mean(void);
 GwyModuleInfo* _gwy_module_query__volumize(void);
 GwyModuleInfo* _gwy_module_query__volumize_layers(void);
 GwyModuleInfo* _gwy_module_query__wave_synth(void);
+GwyModuleInfo* _gwy_module_query__wfr_synth(void);
 GwyModuleInfo* _gwy_module_query__wpour_mark(void);
 GwyModuleInfo* _gwy_module_query__wrapvalue(void);
 GwyModuleInfo* _gwy_module_query__xydenoise(void);
@@ -300,6 +301,7 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__volumize, "volumize", },
   { _gwy_module_query__volumize_layers, "volumize_layers", },
   { _gwy_module_query__wave_synth, "wave_synth", },
+  { _gwy_module_query__wfr_synth, "wfr_synth", },
   { _gwy_module_query__wpour_mark, "wpour_mark", },
   { _gwy_module_query__wrapvalue, "wrapvalue", },
   { _gwy_module_query__xydenoise, "xydenoise", },
