@@ -1,5 +1,5 @@
 /*
- *  $Id: psdf2d.c 24679 2022-03-15 14:54:58Z yeti-dn $
+ *  $Id: psdf2d.c 24813 2022-05-04 06:45:30Z yeti-dn $
  *  Copyright (C) 2009-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -123,7 +123,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Calculates two-dimensional power spectrum density function and extracts its linear profiles."),
     "Yeti <yeti@gwyddion.net>",
-    "2.0",
+    "2.1",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -605,7 +605,7 @@ cut_field_to_zoom(GwyDataField *field, gint zoom)
     if (width >= xres && height >= yres)
         return g_object_ref(field);
 
-    zoomed = gwy_data_field_area_extract(field, (xres - width)/2, (yres - height)/2, width, height);
+    zoomed = gwy_data_field_area_extract(field, (xres - width + 1)/2, (yres - height + 1)/2, width, height);
     gwy_data_field_set_xoffset(zoomed, -0.5*gwy_data_field_get_xreal(zoomed));
     gwy_data_field_set_yoffset(zoomed, -0.5*gwy_data_field_get_yreal(zoomed));
 
