@@ -1,5 +1,5 @@
 /*
- *  $Id: psf.c 24657 2022-03-10 13:16:25Z yeti-dn $
+ *  $Id: psf.c 24833 2022-05-19 13:45:20Z yeti-dn $
  *  Copyright (C) 2017-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -737,8 +737,7 @@ prepare_field(GwyDataField *field, GwyDataField *wfield, GwyWindowingType window
         gwy_data_field_copy(field, wfield, TRUE);
     }
     gwy_data_field_add(wfield, -gwy_data_field_get_avg(wfield));
-    gwy_fft_window_data_field(wfield, GWY_ORIENTATION_HORIZONTAL, window);
-    gwy_fft_window_data_field(wfield, GWY_ORIENTATION_VERTICAL, window);
+    gwy_data_field_fft_window(wfield, window);
 }
 
 static gboolean
