@@ -1,5 +1,5 @@
 /*
- *  $Id: roddeposit_synth.c 22360 2019-08-06 13:52:29Z yeti-dn $
+ *  $Id: roddeposit_synth.c 23556 2021-04-21 08:00:51Z yeti-dn $
  *  Copyright (C) 2007,2009,2010 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -417,7 +417,7 @@ rod_deposit_synth_dialog(RodDepositSynthArgs *args,
     }
 
     gwy_container_set_object_by_name(controls.mydata, "/0/data", dfield);
-    controls.view = create_preview(controls.mydata, 0, PREVIEW_SIZE, FALSE);
+    controls.view = gwy_create_preview(controls.mydata, 0, PREVIEW_SIZE, FALSE);
     gtk_box_pack_start(GTK_BOX(vbox), controls.view, FALSE, FALSE, 0);
 
     gtk_box_pack_start(GTK_BOX(vbox),
