@@ -1,5 +1,5 @@
 /*
- *  $Id: rotate.c 23766 2021-05-24 15:48:55Z yeti-dn $
+ *  $Id: rotate.c 24626 2022-03-03 12:59:05Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -88,7 +88,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Rotates data by arbitrary angle or to make characteristic directions parallel with x or y axis."),
     "Yeti <yeti@gwyddion.net>",
-    "3.0",
+    "3.1",
     "David Nečas (Yeti) & Petr Klapetek",
     "2003",
 };
@@ -264,8 +264,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     dataview = gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
     /* TODO: Add selection setup helper outside preview.h. */
-    gui.selection = create_vector_layer(GWY_DATA_VIEW(dataview), 0, "Lattice", FALSE);
-    gwy_selection_set_max_objects(gui.selection, 1);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Lattice", 1, FALSE);
 
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), TRUE);
 
