@@ -1,5 +1,5 @@
 /*
- *  $Id: stitch.c 23556 2021-04-21 08:00:51Z yeti-dn $
+ *  $Id: stitch.c 24707 2022-03-21 17:09:03Z yeti-dn $
  *  Copyright (C) 2017 David Necas (Yeti), Petr Klapetek, Petr Grolich.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, pgrolich@cmi.cz.
  *
@@ -248,7 +248,6 @@ stitch_dialog(GwyContainer *container, gint id, StitchArgs *args)
 {
     GQuark quark;
     GwyContainer *data;
-    GtkTooltips *tooltips;
     StitchControls controls;
     gint i;
     guint row, response;
@@ -265,7 +264,6 @@ stitch_dialog(GwyContainer *container, gint id, StitchArgs *args)
     gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield_current, 0);
     controls.args->format = gwy_data_field_get_value_format_xy
                             (dfield_current, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
-    tooltips = gwy_app_get_tooltips();
 
     dialog = gtk_dialog_new_with_buttons(_("Stitch"), NULL, 0, NULL);
     controls.dialog = dialog;
@@ -371,8 +369,7 @@ stitch_dialog(GwyContainer *container, gint id, StitchArgs *args)
         gtk_toggle_button_set_active
             (GTK_TOGGLE_BUTTON(controls.enabled[i]), is_active);
         args->enabled[i] = is_active;
-        gtk_tooltips_set_tip
-            (tooltips, controls.enabled[i], "Stitch data", NULL);
+        gtk_widget_set_tooltip_text(controls.enabled[i], _("Stitch data"));
         gtk_table_attach
             (GTK_TABLE(table), controls.enabled[i], 1, 2, row, row+1,
              0, 0, 0, 0);
