@@ -1,5 +1,5 @@
 /*
- *  $Id: zero_crossing.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: zero_crossing.c 24365 2021-10-13 12:45:57Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -121,7 +121,7 @@ define_module_params(void)
     /* Units: RMS */
     gwy_param_def_add_double(paramdef, PARAM_THRESHOLD, "threshold", _("_Threshold"), 0.0, 3.0, 0.1);
     gwy_param_def_add_double(paramdef, PARAM_GAUSSIAN_FWHM, "gaussian-fwhm", _("_Gaussian FWHM"), 0.0, 30.0, 3.0);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", _("Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", gwy_sgettext("verb|Display"),
                               displays, G_N_ELEMENTS(displays), DISPLAY_DATA);
     gwy_param_def_add_instant_updates(paramdef, PARAM_UPDATE, "update", NULL, FALSE);
     return paramdef;
