@@ -1,5 +1,5 @@
 #!/usr/bin/python2
-# $Id: pygwy-codegen.py 21284 2018-07-31 19:14:38Z yeti-dn $
+# $Id: pygwy-codegen.py 21886 2019-02-15 13:14:48Z yeti-dn $
 # Runtime for pygtk module codegen.py and overriden functions for Gwyddion data
 # types.
 # Public domain
@@ -341,6 +341,17 @@ class GwyConstStringArray(argtypes.ArgType):
         info.varlist.add('GArray', '*ret')
         info.codeafter.append('    return create_list_consume_const_string_garray(ret);\n')
 
+# Array argument types
+class GwyDataFieldArray(argtypes.ArgType):
+    def write_param(self, ptype, pname, pdflt, pnull, info):
+        # Input arrays of DataFields are not implemented because they do not
+        # occur.
+        assert not 'Reached'
+
+    def write_return(self, ptype, ownsreturn, info):
+        info.varlist.add('GArray', '*ret')
+        info.codeafter.append('    return create_list_consume_data_field_garray(ret);\n')
+
 # Pointer/boxed argument types
 class GwyConstPointerArg(argtypes.PointerArg):
     "Pointers that have const (like presets)."
@@ -406,6 +417,7 @@ argtypes.matcher.register('GwyIntArray*', GwyIntArray())
 argtypes.matcher.register('GwyIntArrayOutArg', GwyIntArrayOutArg())
 argtypes.matcher.register('GwyStringArray*', GwyStringArray())
 argtypes.matcher.register('GwyConstStringArray*', GwyConstStringArray())
+argtypes.matcher.register('GwyDataFieldArray*', GwyDataFieldArray())
 
 # Pointer/boxed stuff
 argtypes.matcher.register('const-GwyTipModelPreset*', GwyConstPointerArg('GwyTipModelPreset', 'GWY_TYPE_TIP_MODEL_PRESET'))
