@@ -1,5 +1,5 @@
 /*
- *  $Id: pygwy.c 21886 2019-02-15 13:14:48Z yeti-dn $
+ *  $Id: pygwy.c 22841 2020-06-16 12:04:36Z yeti-dn $
  *  Copyright (C) 2004-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -249,8 +249,10 @@ pygwy_initialize(void)
 
         gwy_debug("Init module gwy");
         m = Py_InitModule("gwy", (PyMethodDef*)pygwy_functions);
+        gwy_debug("gwy module = %p", m);
         gwy_debug("Get dict");
         pygwy_dict = PyModule_GetDict(m);
+        gwy_debug("dict = %p", pygwy_dict);
 
         gwy_debug("Register classes");
         pygwy_register_classes(pygwy_dict);
@@ -361,6 +363,7 @@ pygwy_create_environment(const gchar *filename, gboolean show_errors)
     argv[0] = NULL;
 
     d = PyDict_Copy(PyModule_GetDict(pygwy_module));
+    gwy_debug("copying dict from gwy %p as %p", pygwy_module, d);
     if (!d) {
         g_warning("Cannot create copy of Python dictionary.");
         return NULL;
