@@ -1,22 +1,18 @@
 /*
- *  $Id: grainmeasure.c 23305 2021-03-18 14:40:27Z yeti-dn $
- *  Copyright (C) 2007 David Necas (Yeti).
+ *  $Id: grainmeasure.c 24920 2022-08-23 15:40:11Z yeti-dn $
+ *  Copyright (C) 2007-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -31,6 +27,10 @@
 #include <libgwydgets/gwystock.h>
 #include <app/gwyapp.h>
 
+enum {
+    PARAM_EXPANDED,
+};
+
 #define GWY_TYPE_TOOL_GRAIN_MEASURE            (gwy_tool_grain_measure_get_type())
 #define GWY_TOOL_GRAIN_MEASURE(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), GWY_TYPE_TOOL_GRAIN_MEASURE, GwyToolGrainMeasure))
 #define GWY_IS_TOOL_GRAIN_MEASURE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), GWY_TYPE_TOOL_GRAIN_MEASURE))
@@ -39,14 +39,10 @@
 typedef struct _GwyToolGrainMeasure      GwyToolGrainMeasure;
 typedef struct _GwyToolGrainMeasureClass GwyToolGrainMeasureClass;
 
-typedef struct {
-    guint expanded;
-} ToolArgs;
-
 struct _GwyToolGrainMeasure {
     GwyPlainTool parent_instance;
 
-    ToolArgs args;
+    GwyParams *params;
 
     GtkTreeView *treeview;
     GPtrArray *values;
@@ -67,38 +63,32 @@ struct _GwyToolGrainMeasureClass {
     GwyPlainToolClass parent_class;
 };
 
-static gboolean module_register(void);
-
-static GType gwy_tool_grain_measure_get_type        (void) G_GNUC_CONST;
-static void gwy_tool_grain_measure_finalize         (GObject *object);
-static void gwy_tool_grain_measure_init_dialog      (GwyToolGrainMeasure *tool);
-static void gwy_tool_grain_measure_data_switched    (GwyTool *gwytool,
-                                                     GwyDataView *data_view);
-static void gwy_tool_grain_measure_data_changed     (GwyPlainTool *plain_tool);
-static void gwy_tool_grain_measure_mask_changed     (GwyPlainTool *plain_tool);
-static void gwy_tool_grain_measure_selection_changed(GwyPlainTool *plain_tool,
-                                                     gint hint);
-static void gwy_tool_grain_measure_invalidate       (GwyToolGrainMeasure *tool);
-static void gwy_tool_grain_measure_update_units     (GwyToolGrainMeasure *tool);
-static void gwy_tool_grain_measure_recalculate      (GwyToolGrainMeasure *tool);
-
-static const ToolArgs default_args = {
-    0,
-};
+static gboolean     module_register                         (void);
+static GwyParamDef* define_module_params                    (void);
+static GType        gwy_tool_grain_measure_get_type         (void)                       G_GNUC_CONST;
+static void         gwy_tool_grain_measure_finalize         (GObject *object);
+static void         gwy_tool_grain_measure_init_dialog      (GwyToolGrainMeasure *tool);
+static void         gwy_tool_grain_measure_data_switched    (GwyTool *gwytool,
+                                                             GwyDataView *data_view);
+static void         gwy_tool_grain_measure_data_changed     (GwyPlainTool *plain_tool);
+static void         gwy_tool_grain_measure_mask_changed     (GwyPlainTool *plain_tool);
+static void         gwy_tool_grain_measure_selection_changed(GwyPlainTool *plain_tool,
+                                                             gint hint);
+static void         invalidate                              (GwyToolGrainMeasure *tool);
+static void         update_units                            (GwyToolGrainMeasure *tool);
+static void         recalculate                             (GwyToolGrainMeasure *tool);
+static void         group_expanded_collapsed                (GwyToolGrainMeasure *tool);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
-    N_("Grain measurement tool, calculates characteristics of selected "
-       "continuous parts of mask."),
+    N_("Grain measurement tool, calculates characteristics of selected continuous parts of mask."),
     "Yeti <yeti@gwyddion.net>",
-    "1.6",
+    "2.0",
     "David Nečas (Yeti)",
     "2007",
 };
 
-static const gchar expanded_key[] = "/module/grainmeasure/expanded";
-
 GWY_MODULE_QUERY2(module_info, grainmeasure)
 
 G_DEFINE_TYPE(GwyToolGrainMeasure, gwy_tool_grain_measure, GWY_TYPE_PLAIN_TOOL)
@@ -111,6 +101,20 @@ module_register(void)
     return TRUE;
 }
 
+static GwyParamDef*
+define_module_params(void)
+{
+    static GwyParamDef *paramdef = NULL;
+
+    if (paramdef)
+        return paramdef;
+
+    paramdef = gwy_param_def_new();
+    gwy_param_def_set_function_name(paramdef, "grainmeasure");
+    gwy_param_def_add_grain_groups(paramdef, PARAM_EXPANDED, "expanded", NULL, 0);
+    return paramdef;
+}
+
 static void
 gwy_tool_grain_measure_class_init(GwyToolGrainMeasureClass *klass)
 {
@@ -122,8 +126,7 @@ gwy_tool_grain_measure_class_init(GwyToolGrainMeasureClass *klass)
 
     tool_class->stock_id = GWY_STOCK_GRAINS_MEASURE;
     tool_class->title = _("Grain Measure");
-    tool_class->tooltip = _("Measure individual grains "
-                            "(continuous parts of mask)");
+    tool_class->tooltip = _("Measure individual grains (continuous parts of mask)");
     tool_class->prefix = "/module/grainmeasure";
     tool_class->data_switched = gwy_tool_grain_measure_data_switched;
     tool_class->default_width = 240;
@@ -137,50 +140,36 @@ gwy_tool_grain_measure_class_init(GwyToolGrainMeasureClass *klass)
 static void
 gwy_tool_grain_measure_finalize(GObject *object)
 {
-    GwyToolGrainMeasure *tool;
-    GwyContainer *settings;
+    GwyToolGrainMeasure *tool = GWY_TOOL_GRAIN_MEASURE(object);
     guint i;
 
-    tool = GWY_TOOL_GRAIN_MEASURE(object);
-
-    settings = gwy_app_settings_get();
-    tool->args.expanded
-        = gwy_grain_value_tree_view_get_expanded_groups(tool->treeview);
-    gwy_container_set_int32_by_name(settings, expanded_key,
-                                    tool->args.expanded);
-
-    g_free(tool->grains);
+    gwy_params_save_to_settings(tool->params);
+    GWY_OBJECT_UNREF(tool->params);
     GWY_OBJECT_UNREF(tool->siunit);
+    g_free(tool->grains);
     if (tool->values) {
         for (i = 0; i < tool->values->len; i++)
             g_free(tool->values->pdata[i]);
         g_ptr_array_free(tool->values, TRUE);
     }
     GWY_SI_VALUE_FORMAT_FREE(tool->vf);
+
     G_OBJECT_CLASS(gwy_tool_grain_measure_parent_class)->finalize(object);
 }
 
 static void
 gwy_tool_grain_measure_init(GwyToolGrainMeasure *tool)
 {
-    GwyPlainTool *plain_tool;
-    GwyContainer *settings;
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    plain_tool = GWY_PLAIN_TOOL(tool);
-    tool->layer_type_point = gwy_plain_tool_check_layer_type(plain_tool,
-                                                             "GwyLayerPoint");
+    tool->layer_type_point = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerPoint");
     if (!tool->layer_type_point)
         return;
 
+    tool->params = gwy_params_new_from_settings(define_module_params());
+
     plain_tool->lazy_updates = TRUE;
-
-    settings = gwy_app_settings_get();
-    tool->args = default_args;
-    gwy_container_gis_int32_by_name(settings, expanded_key,
-                                    &tool->args.expanded);
-
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_point,
-                                     "pointer");
+    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_point, "pointer");
 
     gwy_tool_grain_measure_init_dialog(tool);
 }
@@ -201,9 +190,7 @@ render_value(G_GNUC_UNUSED GtkTreeViewColumn *column,
     const gchar *name;
     gint i;
 
-    gtk_tree_model_get(model, iter,
-                       GWY_GRAIN_VALUE_STORE_COLUMN_ITEM, &gvalue,
-                       -1);
+    gtk_tree_model_get(model, iter, GWY_GRAIN_VALUE_STORE_COLUMN_ITEM, &gvalue, -1);
     if (tool->gno <= 0 || !gvalue) {
         g_object_set(renderer, "text", "", NULL);
         GWY_OBJECT_UNREF(gvalue);
@@ -211,8 +198,7 @@ render_value(G_GNUC_UNUSED GtkTreeViewColumn *column,
     }
 
     g_object_unref(gvalue);
-    if (!tool->same_units
-        && (gwy_grain_value_get_flags(gvalue) & GWY_GRAIN_VALUE_SAME_UNITS)) {
+    if (!tool->same_units && (gwy_grain_value_get_flags(gvalue) & GWY_GRAIN_VALUE_SAME_UNITS)) {
         g_object_set(renderer, "text", _("N.A."), NULL);
         return;
     }
@@ -246,20 +232,16 @@ render_value(G_GNUC_UNUSED GtkTreeViewColumn *column,
     }
     else {
         GwySIUnit *siunitxy, *siunitz;
-        GwyDataField *dfield;
+        GwyDataField *field;
         GwySIUnitFormatStyle style = GWY_SI_UNIT_FORMAT_VFMARKUP;
 
-        dfield = GWY_PLAIN_TOOL(tool)->data_field;
-        siunitxy = gwy_data_field_get_si_unit_xy(dfield);
-        siunitz = gwy_data_field_get_si_unit_z(dfield);
-        tool->siunit
-            = gwy_si_unit_power_multiply(siunitxy,
-                                         gwy_grain_value_get_power_xy(gvalue),
-                                         siunitz,
-                                         gwy_grain_value_get_power_z(gvalue),
-                                         tool->siunit);
-        tool->vf = gwy_si_unit_get_format_with_digits(tool->siunit, style,
-                                                      value, 3, tool->vf);
+        field = GWY_PLAIN_TOOL(tool)->data_field;
+        siunitxy = gwy_data_field_get_si_unit_xy(field);
+        siunitz = gwy_data_field_get_si_unit_z(field);
+        tool->siunit = gwy_si_unit_power_multiply(siunitxy, gwy_grain_value_get_power_xy(gvalue),
+                                                  siunitz, gwy_grain_value_get_power_z(gvalue),
+                                                  tool->siunit);
+        tool->vf = gwy_si_unit_get_format_with_digits(tool->siunit, style, value, 3, tool->vf);
         g_snprintf(buf, sizeof(buf), "%.*f%s%s",
                    tool->vf->precision, value/tool->vf->magnitude,
                    *tool->vf->units ? " " : "", tool->vf->units);
@@ -270,21 +252,17 @@ render_value(G_GNUC_UNUSED GtkTreeViewColumn *column,
 static void
 gwy_tool_grain_measure_init_dialog(GwyToolGrainMeasure *tool)
 {
-    GtkDialog *dialog;
+    GtkDialog *dialog = GTK_DIALOG(GWY_TOOL(tool)->dialog);
     GtkWidget *treeview, *scwin;
     GtkTreeSelection *selection;
     GtkTreeViewColumn *column;
     GtkCellRenderer *renderer;
 
-    dialog = GTK_DIALOG(GWY_TOOL(tool)->dialog);
-
     scwin = gtk_scrolled_window_new(NULL, NULL);
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin),
-                                   GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
     gtk_box_pack_start(GTK_BOX(dialog->vbox), scwin, TRUE, TRUE, 0);
 
-    treeview = gwy_grain_value_tree_view_new(TRUE,
-                                             "name", "symbol_markup", NULL);
+    treeview = gwy_grain_value_tree_view_new(TRUE, "name", "symbol_markup", NULL);
     tool->treeview = GTK_TREE_VIEW(treeview);
     gtk_tree_view_set_headers_visible(tool->treeview, FALSE);
     gtk_container_add(GTK_CONTAINER(scwin), treeview);
@@ -295,14 +273,14 @@ gwy_tool_grain_measure_init_dialog(GwyToolGrainMeasure *tool)
     renderer = gtk_cell_renderer_text_new();
     g_object_set(renderer, "xalign", 1.0, NULL);
     gtk_tree_view_column_pack_start(column, renderer, TRUE);
-    gtk_tree_view_column_set_cell_data_func(column, renderer,
-                                            render_value, tool, NULL);
+    gtk_tree_view_column_set_cell_data_func(column, renderer, render_value, tool, NULL);
 
     selection = gtk_tree_view_get_selection(tool->treeview);
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_NONE);
+    g_signal_connect_swapped(tool->treeview, "row-expanded", G_CALLBACK(group_expanded_collapsed), tool);
+    g_signal_connect_swapped(tool->treeview, "row-collapsed", G_CALLBACK(group_expanded_collapsed), tool);
 
-    gwy_grain_value_tree_view_set_expanded_groups(tool->treeview,
-                                                  tool->args.expanded);
+    gwy_grain_value_tree_view_set_expanded_groups(tool->treeview, gwy_params_get_flags(tool->params, PARAM_EXPANDED));
 
     gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
     gwy_tool_add_hide_button(GWY_TOOL(tool), TRUE);
@@ -315,47 +293,41 @@ static void
 gwy_tool_grain_measure_data_switched(GwyTool *gwytool,
                                      GwyDataView *data_view)
 {
-    GwyPlainTool *plain_tool;
-    GwyToolGrainMeasure *tool;
-    gboolean ignore;
-
-    tool = GWY_TOOL_GRAIN_MEASURE(gwytool);
-    plain_tool = GWY_PLAIN_TOOL(gwytool);
-    ignore = (data_view == plain_tool->data_view);
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(gwytool);
+    GwyToolGrainMeasure *tool = GWY_TOOL_GRAIN_MEASURE(gwytool);
+    gboolean ignore = (data_view == plain_tool->data_view);
 
     if (!ignore)
-        gwy_tool_grain_measure_invalidate(tool);
+        invalidate(tool);
 
-    GWY_TOOL_CLASS(gwy_tool_grain_measure_parent_class)->data_switched(gwytool,
-                                                                    data_view);
+    GWY_TOOL_CLASS(gwy_tool_grain_measure_parent_class)->data_switched(gwytool, data_view);
 
     if (ignore || plain_tool->init_failed)
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer,
-                                tool->layer_type_point,
+        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_point,
                                 "draw-marker", TRUE,
                                 "marker-radius", 0,
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
         gwy_selection_set_max_objects(plain_tool->selection, 1);
-        gwy_tool_grain_measure_update_units(tool);
+        update_units(tool);
     }
 }
 
 static void
 gwy_tool_grain_measure_data_changed(GwyPlainTool *plain_tool)
 {
-    gwy_tool_grain_measure_invalidate(GWY_TOOL_GRAIN_MEASURE(plain_tool));
+    invalidate(GWY_TOOL_GRAIN_MEASURE(plain_tool));
     gwy_tool_grain_measure_selection_changed(plain_tool, -1);
 }
 
 static void
 gwy_tool_grain_measure_mask_changed(GwyPlainTool *plain_tool)
 {
-    gwy_tool_grain_measure_invalidate(GWY_TOOL_GRAIN_MEASURE(plain_tool));
+    invalidate(GWY_TOOL_GRAIN_MEASURE(plain_tool));
     gwy_tool_grain_measure_selection_changed(plain_tool, -1);
 }
 
@@ -373,24 +345,20 @@ static void
 gwy_tool_grain_measure_selection_changed(GwyPlainTool *plain_tool,
                                          gint hint)
 {
-    GwyToolGrainMeasure *tool;
+    GwyToolGrainMeasure *tool = GWY_TOOL_GRAIN_MEASURE(plain_tool);
+    gint col, row, xres, oldgno = tool->gno;
     gdouble point[2];
-    gint col, row, xres, oldgno;
 
     g_return_if_fail(hint <= 0);
 
-    tool = GWY_TOOL_GRAIN_MEASURE(plain_tool);
-    oldgno = tool->gno;
-
-    if (!plain_tool->mask_field
-        || !gwy_selection_get_object(plain_tool->selection, 0, point))
+    if (!plain_tool->mask_field || !gwy_selection_get_object(plain_tool->selection, 0, point))
         tool->gno = 0;
     else {
         row = floor(gwy_data_field_rtoi(plain_tool->mask_field, point[1]));
         col = floor(gwy_data_field_rtoj(plain_tool->mask_field, point[0]));
         if (gwy_data_field_get_val(plain_tool->mask_field, col, row)) {
             if (!tool->grains)
-                gwy_tool_grain_measure_recalculate(tool);
+                recalculate(tool);
 
             xres = gwy_data_field_get_xres(plain_tool->mask_field);
             tool->gno = tool->grains[row*xres + col];
@@ -408,7 +376,13 @@ gwy_tool_grain_measure_selection_changed(GwyPlainTool *plain_tool,
 }
 
 static void
-gwy_tool_grain_measure_invalidate(GwyToolGrainMeasure *tool)
+group_expanded_collapsed(GwyToolGrainMeasure *tool)
+{
+    gwy_params_set_flags(tool->params, PARAM_EXPANDED, gwy_grain_value_tree_view_get_expanded_groups(tool->treeview));
+}
+
+static void
+invalidate(GwyToolGrainMeasure *tool)
 {
     GWY_FREE(tool->grains);
     tool->ngrains = 0;
@@ -416,34 +390,27 @@ gwy_tool_grain_measure_invalidate(GwyToolGrainMeasure *tool)
 }
 
 static void
-gwy_tool_grain_measure_update_units(GwyToolGrainMeasure *tool)
+update_units(GwyToolGrainMeasure *tool)
 {
-    GwyDataField *dfield;
-    GwySIUnit *siunitxy, *siunitz;
+    GwyDataField *field = GWY_PLAIN_TOOL(tool)->data_field;
+    GwySIUnit *siunitxy = gwy_data_field_get_si_unit_xy(field);
+    GwySIUnit *siunitz = gwy_data_field_get_si_unit_z(field);
 
-    dfield = GWY_PLAIN_TOOL(tool)->data_field;
-    siunitxy = gwy_data_field_get_si_unit_xy(dfield);
-    siunitz = gwy_data_field_get_si_unit_z(dfield);
     tool->same_units = gwy_si_unit_equal(siunitxy, siunitz);
 }
 
 static void
-gwy_tool_grain_measure_recalculate(GwyToolGrainMeasure *tool)
+recalculate(GwyToolGrainMeasure *tool)
 {
-    GwyPlainTool *plain_tool;
-    GwyDataField *dfield, *mask;
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
+    GwyDataField *field = plain_tool->data_field;
+    GwyDataField *mask = plain_tool->mask_field;
     GwyInventory *inventory;
     GwyGrainValue **gvalues;
     guint n, i;
 
-    plain_tool = GWY_PLAIN_TOOL(tool);
-    dfield = plain_tool->data_field;
-    mask = plain_tool->mask_field;
-
     if (!tool->grains) {
-        tool->grains = g_new0(gint,
-                              gwy_data_field_get_xres(dfield)
-                              *gwy_data_field_get_yres(dfield));
+        tool->grains = g_new0(gint, gwy_data_field_get_xres(field) *gwy_data_field_get_yres(field));
         tool->ngrains = gwy_data_field_number_grains(mask, tool->grains);
     }
 
@@ -458,14 +425,11 @@ gwy_tool_grain_measure_recalculate(GwyToolGrainMeasure *tool)
     gvalues = g_new(GwyGrainValue*, n);
     for (i = 0; i < n; i++) {
         gvalues[i] = gwy_inventory_get_nth_item(inventory, i);
-        g_ptr_array_index(tool->values, i)
-             = g_renew(gdouble, g_ptr_array_index(tool->values, i),
-                       tool->ngrains+1);
+        g_ptr_array_index(tool->values, i) = g_renew(gdouble, g_ptr_array_index(tool->values, i), tool->ngrains+1);
     }
 
-    gwy_grain_values_calculate(n, gvalues, (gdouble**)tool->values->pdata,
-                               dfield, tool->ngrains, tool->grains);
+    gwy_grain_values_calculate(n, gvalues, (gdouble**)tool->values->pdata, field, tool->ngrains, tool->grains);
     g_free(gvalues);
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
