@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_fdfit.c 21408 2018-09-07 13:53:53Z yeti-dn $
+ *  $Id: volume_fdfit.c 24423 2021-10-24 14:55:31Z klapetek $
  *  Copyright (C) 2003-2007,2013 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -209,12 +209,12 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_fdfit",
-                            (GwyVolumeFunc)&volfit,
-                            N_("/_Evaluate FD Data..."),
-                            GWY_STOCK_VOLUME_FD,
-                            VOLFIT_RUN_MODES,
-                            GWY_MENU_FLAG_VOLUME,
-                            N_("Evaluate force-distance volume data"));
+                             (GwyVolumeFunc)&volfit,
+                             N_("/_Evaluate FD Data..."),
+                             GWY_STOCK_VOLUME_FD,
+                             VOLFIT_RUN_MODES,
+                             GWY_MENU_FLAG_VOLUME,
+                             N_("Evaluate force-distance volume data"));
 
     return TRUE;
 }
@@ -450,7 +450,7 @@ volfit_dialog(VolfitArgs *args)
     /* Correlation matrix */
     expander = gtk_expander_new(NULL);
     gtk_expander_set_label_widget(GTK_EXPANDER(expander),
-                                 gwy_label_new_header(_("Correlation Matrix")));
+                                  gwy_label_new_header(_("Correlation Matrix")));
     gtk_table_attach(GTK_TABLE(table), expander,
                      0, 2, row, row+1, GTK_FILL, 0, 0, 0);
     gtk_table_set_row_spacing(GTK_TABLE(table), row, 8);
@@ -631,7 +631,7 @@ load_curve(VolfitControls *controls)
 
 static void
 xpos_changed_cb(GtkAdjustment *adj,
-                 VolfitControls *controls)
+                VolfitControls *controls)
 {
     VolfitArgs *args = controls->args;
 
@@ -646,7 +646,7 @@ xpos_changed_cb(GtkAdjustment *adj,
 
 static void
 ypos_changed_cb(GtkAdjustment *adj,
-                 VolfitControls *controls)
+                VolfitControls *controls)
 {
     VolfitArgs *args = controls->args;
 
@@ -696,30 +696,29 @@ graph_selection_finished_cb(GwySelection *selection,
     gdouble sel[2];
     VolfitArgs *args = controls->args;
 
-    if (!gwy_selection_get_object(selection, 0, sel)) return;
+    if (!gwy_selection_get_object(selection, 0, sel))
+        return;
 
     controls->in_update = TRUE;
     args->xpos = gwy_data_field_rtoi(args->dfield, sel[0]);
     args->ypos = gwy_data_field_rtoj(args->dfield, sel[1]);
 
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->xpos),
-                                                     args->xpos);
+                             args->xpos);
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->ypos),
-                                                      args->ypos);
+                             args->ypos);
 
     controls->in_update = FALSE;
     load_curve(controls);
-
-
 }
 
 
 
 static void
 volfit_param_row_create(VolfitControls *controls,
-                     gint i,
-                     GtkTable *table,
-                     gint row)
+                        gint i,
+                        GtkTable *table,
+                        gint row)
 {
     FitParamControl *cntrl;
 
@@ -797,7 +796,7 @@ volfit_param_row_create(VolfitControls *controls,
 
 static void
 volfit_param_row_destroy(VolfitControls *controls,
-                      gint i)
+                         gint i)
 {
     FitParamControl *cntrl;
 
@@ -1017,7 +1016,7 @@ volfit_do(VolfitControls *controls)
 
     max = gwy_data_field_get_max(chresult);
     data = gwy_data_field_get_data(chresult);
-    for (i = 0; i< xres*yres; i++) {
+    for (i = 0; i < xres*yres; i++) {
         if (data[i] == -1)
             data[i] = max;
     }
@@ -1091,7 +1090,7 @@ volfit_single(VolfitControls *controls)
     param = g_newa(gdouble, nparams);
     error = g_newa(gdouble, nparams);
     for (i = 0; i < nparams; i++)
-        param[i] =g_array_index(args->param, FitParamArg, i).value;
+        param[i] = g_array_index(args->param, FitParamArg, i).value;
     args->fitter
         = gwy_nlfit_preset_fit(args->volfitfunc, NULL,
                                gwy_data_line_get_res(args->xdata),
@@ -1166,7 +1165,7 @@ function_changed(GtkComboBox *combo, VolfitControls *controls)
 
     args->function_type = gtk_combo_box_get_active(combo);
     args->volfitfunc = gwy_inventory_get_nth_item(gwy_fd_curve_presets(),
-                                               args->function_type);
+                                                  args->function_type);
     nparams = gwy_nlfit_preset_get_nparams(args->volfitfunc);
     gtk_label_set_markup(GTK_LABEL(controls->formula),
                          gwy_nlfit_preset_get_formula(args->volfitfunc));
@@ -1184,7 +1183,7 @@ function_changed(GtkComboBox *combo, VolfitControls *controls)
     gtk_table_resize(GTK_TABLE(controls->param_table), 1+nparams, 10);
     for (i = oldnparams; i < nparams; i++) {
         volfit_param_row_create(controls, i, GTK_TABLE(controls->param_table),
-                             i+1);
+                                i+1);
         for (j = 0; j <= i; j++) {
             GtkWidget *label;
 
@@ -1212,8 +1211,8 @@ function_changed(GtkComboBox *combo, VolfitControls *controls)
 
 static void
 volfit_set_state(VolfitControls *controls,
-              gboolean is_volfitted,
-              gboolean is_estimated)
+                 gboolean is_volfitted,
+                 gboolean is_estimated)
 {
     VolfitArgs *args;
     gint i, j, nparams;
@@ -1266,7 +1265,7 @@ volfit_estimate(VolfitControls *controls)
 
     param = g_newa(gdouble, nparams);
     for (i = 0; i < nparams; i++)
-        param[i] =g_array_index(args->param, FitParamArg, i).init;
+        param[i] = g_array_index(args->param, FitParamArg, i).init;
 
     if (!normalize_data(args))
         return;
@@ -1295,8 +1294,8 @@ volfit_estimate(VolfitControls *controls)
 
 static void
 volfit_param_row_update_value(VolfitControls *controls,
-                           gint i,
-                           gboolean errorknown)
+                              gint i,
+                              gboolean errorknown)
 {
     FitParamControl *cntrl;
     FitParamArg *arg;
@@ -1430,7 +1429,7 @@ range_changed(GtkWidget *entry,
 
 static void
 volfit_limit_selection(VolfitControls *controls,
-                    gboolean curve_switch)
+                       gboolean curve_switch)
 {
     GwySelection *selection;
     GwyGraphArea *area;
@@ -1461,8 +1460,8 @@ volfit_limit_selection(VolfitControls *controls,
 
 static void
 volfit_get_full_x_range(VolfitControls *controls,
-                     gdouble *xmin,
-                     gdouble *xmax)
+                        gdouble *xmin,
+                        gdouble *xmax)
 {
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
@@ -1567,7 +1566,6 @@ normalize_data(VolfitArgs *args)
         if (((args->from == args->to)
              || (xs[i] >= args->from && xs[i] <= args->to))
             && !(skip_first_point && i == 0)) {
-
             xd[j] = xs[i];
             yd[j] = ys[i];
             j++;
@@ -1616,7 +1614,6 @@ pick_and_normalize_data(VolfitArgs *args, gint xpos, gint ypos)
         if (((args->from == args->to)
              || ((i*ratio) >= args->from && (i*ratio) <= args->to))
             && !(skip_first_point && i == 0)) {
-
             xd[j] = i*ratio;
             yd[j] = ys[i];
             j++;
