@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_linestat.c 24289 2021-10-07 15:12:05Z yeti-dn $
+ *  $Id: volume_linestat.c 24975 2022-08-31 12:05:41Z yeti-dn $
  *  Copyright (C) 2015-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -115,8 +115,9 @@ static gboolean                    quantity_filter        (const GwyEnum *enumva
                                                            gpointer user_data);
 static void                        param_changed          (ModuleGUI *gui,
                                                            gint id);
-static void                        dialog_response        (ModuleGUI *gui,
-                                                           gint response);
+static void                        dialog_response        (GwyDialog *dialog,
+                                                           gint response,
+                                                           ModuleGUI *gui);
 static void                        preview                (gpointer user_data);
 static void                        set_image_selection    (ModuleGUI *gui);
 static void                        set_graph_selection    (ModuleGUI *gui);
@@ -357,7 +358,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(gui.table_options, "param-changed", G_CALLBACK(param_changed), &gui);
     g_signal_connect_swapped(gui.image_selection, "changed", G_CALLBACK(point_selection_changed), &gui);
     g_signal_connect_swapped(gui.graph_selection, "changed", G_CALLBACK(graph_selection_changed), &gui);
-    g_signal_connect_swapped(dialog, "response", G_CALLBACK(dialog_response), &gui);
+    g_signal_connect_after(dialog, "response", G_CALLBACK(dialog_response), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
     outcome = gwy_dialog_run(dialog);
@@ -402,7 +403,7 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static void
-dialog_response(ModuleGUI *gui, gint response)
+dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
 {
     if (response == GWY_RESPONSE_RESET)
         gwy_selection_clear(gui->graph_selection);
@@ -515,8 +516,7 @@ graph_selection_changed(ModuleGUI *gui, G_GNUC_UNUSED gint id, GwySelection *sel
     if (gwy_selection_get_object(selection, 0, z1z2)) {
         zfrom = CLAMP(gwy_brick_rtok_cal(brick, z1z2[0])+0.49, 0, zres-1);
         zto = CLAMP(gwy_brick_rtok_cal(brick, z1z2[1])+0.5, 0, zres-1);
-        if (zto < zfrom)
-            GWY_SWAP(gint, zfrom, zto);
+        GWY_ORDER(gint, zfrom, zto);
         if (zto - zfrom < 2)
             zfrom = zto = -1;
     }
