@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_psf.c 24683 2022-03-15 18:41:03Z yeti-dn $
+ *  $Id: volume_psf.c 24833 2022-05-19 13:45:20Z yeti-dn $
  *  Copyright (C) 2017-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1337,8 +1337,7 @@ prepare_field(GwyDataField *field, GwyDataField *wfield,
         gwy_data_field_copy(field, wfield, TRUE);
     }
     gwy_data_field_add(wfield, -gwy_data_field_get_avg(wfield));
-    gwy_fft_window_data_field(wfield, GWY_ORIENTATION_HORIZONTAL, window);
-    gwy_fft_window_data_field(wfield, GWY_ORIENTATION_VERTICAL, window);
+    gwy_data_field_fft_window(wfield, window);
 }
 
 static void
