@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_strayfield.c 22157 2019-06-10 13:03:11Z klapetek $
+ *  $Id: volume_strayfield.c 24952 2022-08-26 15:24:56Z yeti-dn $
  *  Copyright (C) 2015-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -508,8 +508,7 @@ graph_selection_changed(StrayfieldControls *controls,
     else {
         args->zfrom = CLAMP(gwy_brick_rtok_cal(brick, z[0])+0.49, 0, brick->zres);
         args->zto = CLAMP(gwy_brick_rtok_cal(brick, z[1])+0.5, 0, brick->zres);
-        if (args->zto < args->zfrom)
-            GWY_SWAP(gint, args->zfrom, args->zto);
+        GWY_ORDER(gint, args->zfrom, args->zto);
         if (args->zto - args->zfrom < 2)
             args->zfrom = args->zto = -1;
     }
