@@ -16,12 +16,14 @@ GwyModuleInfo* _gwy_module_query__xyzops(void);
 GwyModuleInfo* _gwy_module_query__xyz_drift(void);
 GwyModuleInfo* _gwy_module_query__xyz_level(void);
 GwyModuleInfo* _gwy_module_query__xyz_raster(void);
+GwyModuleInfo* _gwy_module_query__xyz_split(void);
 
 static const GwyModuleRecord modules[] = {
   { _gwy_module_query__xyzops, "xyzops", },
   { _gwy_module_query__xyz_drift, "xyz_drift", },
   { _gwy_module_query__xyz_level, "xyz_level", },
   { _gwy_module_query__xyz_raster, "xyz_raster", },
+  { _gwy_module_query__xyz_split, "xyz_split", },
   { NULL, NULL, },
 };
 
