@@ -1,6 +1,6 @@
 /*
- *  $Id: xyz_raster.c 21301 2018-08-08 14:30:28Z yeti-dn $
- *  Copyright (C) 2016-2018 David Necas (Yeti).
+ *  $Id: xyz_raster.c 22564 2019-10-10 15:48:53Z yeti-dn $
+ *  Copyright (C) 2016-2019 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,8 +24,10 @@
 #include <gtk/gtk.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
+#include <libgwyddion/gwythreads.h>
 #include <libgwyddion/gwyutils.h>
 #include <libprocess/stats.h>
+#include <libprocess/filters.h>
 #include <libprocess/grains.h>
 #include <libprocess/triangulation.h>
 #include <libgwydgets/gwydataview.h>
@@ -36,6 +38,7 @@
 #include <libgwymodule/gwymodule-xyz.h>
 #include <app/gwymoduleutils.h>
 #include <app/gwyapp.h>
+#include "libgwyddion/gwyomp.h"
 
 #define XYZRAS_RUN_MODES (GWY_RUN_INTERACTIVE | GWY_RUN_IMMEDIATE)
 
@@ -66,6 +69,7 @@ typedef struct {
     GwyExteriorType exterior;
     gint xres;
     gint yres;
+    gboolean mask_empty;
     /* Interface only. */
     gdouble xmin;
     gdouble xmax;
@@ -78,6 +82,7 @@ typedef struct {
     GwyTriangulation *triangulation;
     GwyDataField *regular;
     GwyDataField *raster;
+    GwyDataField *nilmask;
     GArray *points;
     guint norigpoints;
     guint nbasepoints;
@@ -99,6 +104,7 @@ typedef struct {
     GtkObject *yres;
     GtkWidget *interpolation;
     GtkWidget *exterior;
+    GtkWidget *mask_empty;
     GtkWidget *view;
     GtkWidget *do_preview;
     GtkWidget *error;
@@ -114,76 +120,80 @@ typedef struct {
     guint size;
 } WorkQueue;
 
-static gboolean      module_register          (void);
-static void          xyzras                   (GwyContainer *data,
-                                               GwyRunType run);
-static gboolean      xyzras_dialog            (XYZRasArgs *arg,
-                                               XYZRasData *rdata,
-                                               GwyContainer *data,
-                                               gint id);
-static void          add_dfield_to_data       (GwyDataField *dfield,
-                                               GwyContainer *data,
-                                               gint id);
-static gint          construct_physical_dims  (XYZRasControls *controls,
-                                               GtkTable *table,
-                                               gint row);
-static gint          construct_options        (XYZRasControls *controls,
-                                               GtkTable *table,
-                                               gint row);
-static void          make_pixels_square       (XYZRasControls *controls);
-static void          xres_changed             (XYZRasControls *controls,
-                                               GtkAdjustment *adj);
-static void          yres_changed             (XYZRasControls *controls,
-                                               GtkAdjustment *adj);
-static void          xmin_changed             (XYZRasControls *controls,
-                                               GtkEntry *entry);
-static void          xmax_changed             (XYZRasControls *controls,
-                                               GtkEntry *entry);
-static void          ymin_changed             (XYZRasControls *controls,
-                                               GtkEntry *entry);
-static void          ymax_changed             (XYZRasControls *controls,
-                                               GtkEntry *entry);
-static void          interpolation_changed    (XYZRasControls *controls,
-                                               GtkComboBox *combo);
-static void          exterior_changed         (XYZRasControls *controls,
-                                               GtkComboBox *combo);
-static void          reset_ranges             (XYZRasControls *controls);
-static void          update_selection         (XYZRasControls *controls);
-static void          selection_changed        (XYZRasControls *controls,
-                                               gint hint,
-                                               GwySelection *selection);
-static void          clear_selection          (XYZRasControls *controls);
-static void          preview                  (XYZRasControls *controls);
-static void          triangulation_info       (XYZRasControls *controls);
-static void          render_regular_directly  (XYZRasControls *controls);
-static GwyDataField* xyzras_do                (XYZRasData *rdata,
-                                               const XYZRasArgs *args,
-                                               GtkWindow *dialog,
-                                               gchar **error);
-static gboolean      interpolate_field        (guint npoints,
-                                               const GwyXYZ *points,
-                                               GwyDataField *dfield,
-                                               GwySetFractionFunc set_fraction,
-                                               GwySetMessageFunc set_message);
-static gboolean      extend_borders           (XYZRasData *rdata,
-                                               const XYZRasArgs *args,
-                                               gboolean check_for_changes,
-                                               gdouble epsrel);
-static void          xyzras_free              (XYZRasData *rdata);
-static void          initialize_ranges        (const XYZRasData *rdata,
-                                               XYZRasArgs *args);
-static void          invalidate_raster        (XYZRasData *rdata);
-static void          analyse_points           (XYZRasData *rdata,
-                                               double epsrel);
-static GwyDataField* check_regular_grid       (GwySurface *surface);
-static void          xyzras_load_args         (GwyContainer *container,
-                                               XYZRasArgs *args);
-static void          xyzras_save_args         (GwyContainer *container,
-                                               XYZRasArgs *args);
+static gboolean      module_register        (void);
+static void          xyzras                 (GwyContainer *data,
+                                             GwyRunType run);
+static gboolean      xyzras_dialog          (XYZRasArgs *arg,
+                                             XYZRasData *rdata,
+                                             GwyContainer *data,
+                                             gint id);
+static void          add_dfield_to_data     (GwyDataField *dfield,
+                                             GwyDataField *mask,
+                                             GwyContainer *data,
+                                             gint id);
+static gint          construct_physical_dims(XYZRasControls *controls,
+                                             GtkTable *table,
+                                             gint row);
+static gint          construct_options      (XYZRasControls *controls,
+                                             GtkTable *table,
+                                             gint row);
+static void          make_pixels_square     (XYZRasControls *controls);
+static void          xres_changed           (XYZRasControls *controls,
+                                             GtkAdjustment *adj);
+static void          yres_changed           (XYZRasControls *controls,
+                                             GtkAdjustment *adj);
+static void          xmin_changed           (XYZRasControls *controls,
+                                             GtkEntry *entry);
+static void          xmax_changed           (XYZRasControls *controls,
+                                             GtkEntry *entry);
+static void          ymin_changed           (XYZRasControls *controls,
+                                             GtkEntry *entry);
+static void          ymax_changed           (XYZRasControls *controls,
+                                             GtkEntry *entry);
+static void          interpolation_changed  (XYZRasControls *controls,
+                                             GtkComboBox *combo);
+static void          exterior_changed       (XYZRasControls *controls,
+                                             GtkComboBox *combo);
+static void          mask_empty_changed     (XYZRasControls *controls,
+                                             GtkToggleButton *button);
+static void          reset_ranges           (XYZRasControls *controls);
+static void          update_selection       (XYZRasControls *controls);
+static void          selection_changed      (XYZRasControls *controls,
+                                             gint hint,
+                                             GwySelection *selection);
+static void          clear_selection        (XYZRasControls *controls);
+static void          preview                (XYZRasControls *controls);
+static void          triangulation_info     (XYZRasControls *controls);
+static void          render_regular_directly(XYZRasControls *controls);
+static GwyDataField* xyzras_do              (XYZRasData *rdata,
+                                             const XYZRasArgs *args,
+                                             GwyDataField **mask,
+                                             GtkWindow *dialog,
+                                             gchar **error);
+static gboolean      interpolate_field      (guint npoints,
+                                             const GwyXYZ *points,
+                                             GwyDataField *dfield,
+                                             GwySetFractionFunc set_fraction,
+                                             GwySetMessageFunc set_message);
+static gboolean      extend_borders         (XYZRasData *rdata,
+                                             const XYZRasArgs *args,
+                                             gboolean check_for_changes,
+                                             gdouble epsrel);
+static void          xyzras_free            (XYZRasData *rdata);
+static void          initialize_ranges      (const XYZRasData *rdata,
+                                             XYZRasArgs *args);
+static void          invalidate_raster      (XYZRasData *rdata);
+static void          analyse_points         (XYZRasData *rdata,
+                                             double epsrel);
+static GwyDataField* check_regular_grid     (GwySurface *surface);
+static void          xyzras_load_args       (GwyContainer *container,
+                                             XYZRasArgs *args);
+static void          xyzras_save_args       (GwyContainer *container,
+                                             XYZRasArgs *args);
 
 static const XYZRasArgs xyzras_defaults = {
-    GWY_INTERPOLATION_LINEAR, GWY_EXTERIOR_MIRROR_EXTEND,
-    512, 512,
+    GWY_INTERPOLATION_AVERAGE, GWY_EXTERIOR_MIRROR_EXTEND,
+    512, 512, TRUE,
     /* Interface only. */
     0.0, 0.0, 0.0, 0.0,
 };
@@ -193,7 +203,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Rasterizes XYZ data to images."),
     "Yeti <yeti@gwyddion.net>",
-    "1.2",
+    "1.4",
     "David Nečas (Yeti)",
     "2016",
 };
@@ -222,7 +232,7 @@ xyzras(GwyContainer *data, GwyRunType run)
 
     GwyContainer *settings;
     GwySurface *surface = NULL;
-    GwyDataField *dfield;
+    GwyDataField *dfield, *mask;
     gboolean ok = TRUE;
     gint id;
 
@@ -236,7 +246,7 @@ xyzras(GwyContainer *data, GwyRunType run)
 
     dfield = check_regular_grid(surface);
     if (dfield && run == GWY_RUN_IMMEDIATE) {
-        add_dfield_to_data(dfield, data, id);
+        add_dfield_to_data(dfield, NULL, data, id);
         return;
     }
 
@@ -257,15 +267,17 @@ xyzras(GwyContainer *data, GwyRunType run)
     if (ok) {
         gchar *error = NULL;
 
-        if (rdata.raster)
+        if (rdata.raster) {
             dfield = g_object_ref(rdata.raster);
+            mask = rdata.nilmask ? g_object_ref(rdata.nilmask) : NULL;
+        }
         else {
             GtkWindow *window = gwy_app_find_window_for_xyz(data, id);
-            dfield = xyzras_do(&rdata, &args, window, &error);
+            dfield = xyzras_do(&rdata, &args, &mask, window, &error);
         }
 
         if (dfield) {
-            add_dfield_to_data(dfield, data, id);
+            add_dfield_to_data(dfield, mask, data, id);
         }
         else if (run == GWY_RUN_INTERACTIVE) {
             GtkWidget *dialog;
@@ -288,13 +300,19 @@ xyzras(GwyContainer *data, GwyRunType run)
 }
 
 static void
-add_dfield_to_data(GwyDataField *dfield, GwyContainer *data, gint id)
+add_dfield_to_data(GwyDataField *dfield, GwyDataField *mask,
+                   GwyContainer *data, gint id)
 {
     GQuark qsrc, qdest;
     const guchar *s;
     gint newid;
 
     newid = gwy_app_data_browser_add_data_field(dfield, data, TRUE);
+    if (mask) {
+        qdest = gwy_app_get_mask_key_for_id(newid);
+        gwy_container_set_object(data, qdest, mask);
+        g_object_unref(mask);
+    }
     gwy_app_channel_log_add(data, -1, newid, "xyz::xyz_raster", NULL);
 
     qsrc = gwy_app_get_surface_palette_key_for_id(id);
@@ -481,6 +499,8 @@ xyzras_dialog(XYZRasArgs *args,
                              G_CALLBACK(interpolation_changed), &controls);
     g_signal_connect_swapped(controls.exterior, "changed",
                              G_CALLBACK(exterior_changed), &controls);
+    g_signal_connect_swapped(controls.mask_empty, "toggled",
+                             G_CALLBACK(mask_empty_changed), &controls);
 
     controls.in_update = FALSE;
     reset_ranges(&controls);
@@ -636,6 +656,17 @@ construct_options(XYZRasControls *controls,
                      GTK_EXPAND | GTK_FILL, 0, 0, 0);
     row++;
 
+    controls->mask_empty
+        = gtk_check_button_new_with_mnemonic(_("_Mask empty regions"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls->mask_empty),
+                                 args->mask_empty);
+    gtk_widget_set_sensitive(controls->mask_empty,
+                             (gint)args->interpolation
+                             == GWY_INTERPOLATION_AVERAGE);
+    gtk_table_attach(table, controls->mask_empty, 0, 4, row, row+1,
+                     GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    row++;
+
     return row;
 }
 
@@ -776,6 +807,9 @@ interpolation_changed(XYZRasControls *controls,
                       GtkComboBox *combo)
 {
     controls->args->interpolation = gwy_enum_combo_box_get_active(combo);
+    gtk_widget_set_sensitive(controls->mask_empty,
+                             (gint)controls->args->interpolation
+                             == GWY_INTERPOLATION_AVERAGE);
     invalidate_raster(controls->rdata);
 }
 
@@ -787,6 +821,13 @@ exterior_changed(XYZRasControls *controls,
     invalidate_raster(controls->rdata);
 }
 
+static void
+mask_empty_changed(XYZRasControls *controls,
+                   GtkToggleButton *button)
+{
+    controls->args->mask_empty = gtk_toggle_button_get_active(button);
+}
+
 static void
 set_all_physical_dimensions(XYZRasControls *controls)
 {
@@ -879,7 +920,7 @@ static void
 preview(XYZRasControls *controls)
 {
     XYZRasArgs *args = controls->args;
-    GwyDataField *dfield;
+    GwyDataField *dfield, *mask;
     GtkWidget *entry;
     gchar *error = NULL;
 
@@ -887,12 +928,14 @@ preview(XYZRasControls *controls)
     if (entry && GTK_IS_ENTRY(entry))
         gtk_widget_activate(entry);
 
-    gwy_object_unref(controls->rdata->raster);
-    dfield = xyzras_do(controls->rdata, args,
+    GWY_OBJECT_UNREF(controls->rdata->raster);
+    GWY_OBJECT_UNREF(controls->rdata->nilmask);
+    dfield = xyzras_do(controls->rdata, args, &mask,
                        GTK_WINDOW(controls->dialog), &error);
     if (dfield) {
         triangulation_info(controls);
         controls->rdata->raster = g_object_ref(dfield);
+        controls->rdata->nilmask = mask;
     }
     else {
         gtk_label_set_text(GTK_LABEL(controls->error), error);
@@ -943,6 +986,7 @@ render_regular_directly(XYZRasControls *controls)
 static GwyDataField*
 xyzras_do(XYZRasData *rdata,
           const XYZRasArgs *args,
+          GwyDataField **mask,
           GtkWindow *window,
           gchar **error)
 {
@@ -954,6 +998,7 @@ xyzras_do(XYZRasData *rdata,
     GwySetFractionFunc set_fraction = (window ? gwy_app_wait_set_fraction : NULL);
     gboolean ok = TRUE, extended;
 
+    *mask = NULL;
     gwy_debug("%g %g :: %g %g", args->xmin, args->xmax, args->ymin, args->ymax);
     if (!(args->xmax > args->xmin) || !(args->ymax > args->ymin)) {
         *error = g_strdup(_("Physical dimensions are invalid."));
@@ -979,8 +1024,20 @@ xyzras_do(XYZRasData *rdata,
     }
     else if ((gint)args->interpolation == GWY_INTERPOLATION_AVERAGE) {
         extend_borders(rdata, args, FALSE, EPSREL);
-        gwy_data_field_average_xyz(dfield, NULL,
-                                   (const GwyXYZ*)points->data, points->len);
+        if (args->mask_empty) {
+            *mask = gwy_data_field_new_alike(dfield, FALSE);
+            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(*mask),
+                                        NULL);
+            gwy_data_field_average_xyz(dfield, *mask,
+                                       (const GwyXYZ*)points->data,
+                                       points->len);
+            gwy_data_field_threshold(*mask, G_MINDOUBLE, 1.0, 0.0);
+        }
+        else {
+            gwy_data_field_average_xyz(dfield, NULL,
+                                       (const GwyXYZ*)points->data,
+                                       points->len);
+        }
         ok = TRUE;
     }
     else {
@@ -1022,6 +1079,7 @@ xyzras_do(XYZRasData *rdata,
     if (!ok) {
         gwy_object_unref(rdata->triangulation);
         g_object_unref(dfield);
+        GWY_OBJECT_UNREF(*mask);
         *error = g_strdup(_("XYZ data regularization failed due to "
                             "numerical instability or was interrupted."));
         return NULL;
@@ -1037,8 +1095,9 @@ interpolate_field(guint npoints,
                   GwySetFractionFunc set_fraction,
                   GwySetMessageFunc set_message)
 {
+    gboolean cancelled = FALSE, *pcancelled = &cancelled;
     gdouble xoff, yoff, qx, qy;
-    guint xres, yres, i, j, k;
+    guint xres, yres;
     gdouble *d;
 
     xres = gwy_data_field_get_xres(dfield);
@@ -1052,39 +1111,50 @@ interpolate_field(guint npoints,
     if (set_message)
         set_message(_("Interpolating..."));
 
-    for (i = 0; i < yres; i++) {
-        gdouble y = yoff + qy*(i + 0.5);
+#ifdef _OPENMP
+#pragma omp parallel if (gwy_threads_are_enabled()) default(none) \
+            shared(d,xres,yres,points,npoints,xoff,yoff,qx,qy,set_fraction,pcancelled)
+#endif
+    {
+        gint ifrom = gwy_omp_chunk_start(yres), ito = gwy_omp_chunk_end(yres);
+        gint i, j, k;
 
-        for (j = 0; j < xres; j++) {
-            gdouble x = xoff + qx*(j + 0.5);
-            gdouble w = 0.0;
-            gdouble s = 0.0;
+        for (i = ifrom; i < ito; i++) {
+            gdouble y = yoff + qy*(i + 0.5);
+            gdouble *drow = d + i*xres;
 
-            for (k = 0; k < npoints; k++) {
-                const GwyXYZ *pt = points + k;
-                gdouble dx = x - pt->x;
-                gdouble dy = y - pt->y;
-                gdouble r2 = dx*dx + dy*dy;
+            for (j = 0; j < xres; j++) {
+                gdouble x = xoff + qx*(j + 0.5);
+                gdouble w = 0.0;
+                gdouble s = 0.0;
 
-                r2 *= r2;
-                if (G_UNLIKELY(r2 == 0.0)) {
-                    s = pt->z;
-                    w = 1.0;
-                    break;
+                for (k = 0; k < npoints; k++) {
+                    const GwyXYZ *pt = points + k;
+                    gdouble dx = x - pt->x;
+                    gdouble dy = y - pt->y;
+                    gdouble r2 = dx*dx + dy*dy;
+
+                    r2 *= r2;
+                    if (G_UNLIKELY(r2 == 0.0)) {
+                        s = pt->z;
+                        w = 1.0;
+                        break;
+                    }
+
+                    r2 = 1.0/r2;
+                    w += r2;
+                    s += r2*pt->z;
                 }
-
-                r2 = 1.0/r2;
-                w += r2;
-                s += r2*pt->z;
+                drow[j] = s/w;
             }
-            *(d++) = s/w;
-        }
 
-        if (set_fraction && !set_fraction(i/(gdouble)yres))
-            return FALSE;
+            if (gwy_omp_set_fraction_check_cancel(set_fraction,
+                                                  i, ifrom, ito, pcancelled))
+                break;
+        }
     }
 
-    return TRUE;
+    return !cancelled;
 }
 
 /* Return TRUE if extpoints have changed. */
@@ -1235,9 +1305,10 @@ extend_borders(XYZRasData *rdata,
 static void
 xyzras_free(XYZRasData *rdata)
 {
-    gwy_object_unref(rdata->triangulation);
-    gwy_object_unref(rdata->raster);
-    gwy_object_unref(rdata->regular);
+    GWY_OBJECT_UNREF(rdata->triangulation);
+    GWY_OBJECT_UNREF(rdata->raster);
+    GWY_OBJECT_UNREF(rdata->nilmask);
+    GWY_OBJECT_UNREF(rdata->regular);
     g_array_free(rdata->points, TRUE);
 }
 
@@ -1596,6 +1667,7 @@ check_regular_grid(GwySurface *surface)
 
 static const gchar exterior_key[]      = "/module/xyz_raster/exterior";
 static const gchar interpolation_key[] = "/module/xyz_raster/interpolation";
+static const gchar mask_empty_key[]    = "/module/xyz_raster/mask_empty";
 static const gchar xres_key[]          = "/module/xyz_raster/xres";
 static const gchar yres_key[]          = "/module/xyz_raster/yres";
 
@@ -1610,6 +1682,7 @@ xyzras_sanitize_args(XYZRasArgs *args)
     if (args->exterior != GWY_EXTERIOR_MIRROR_EXTEND
         && args->exterior != GWY_EXTERIOR_PERIODIC)
         args->exterior = GWY_EXTERIOR_BORDER_EXTEND;
+    args->mask_empty = !!args->mask_empty;
     args->xres = CLAMP(args->xres, 2, 16384);
     args->yres = CLAMP(args->yres, 2, 16384);
 }
@@ -1623,6 +1696,8 @@ xyzras_load_args(GwyContainer *container,
     gwy_container_gis_enum_by_name(container, interpolation_key,
                                    &args->interpolation);
     gwy_container_gis_enum_by_name(container, exterior_key, &args->exterior);
+    gwy_container_gis_boolean_by_name(container, mask_empty_key,
+                                      &args->mask_empty);
     gwy_container_gis_int32_by_name(container, xres_key, &args->xres);
     gwy_container_gis_int32_by_name(container, yres_key, &args->yres);
 
@@ -1636,6 +1711,8 @@ xyzras_save_args(GwyContainer *container,
     gwy_container_set_enum_by_name(container, interpolation_key,
                                    args->interpolation);
     gwy_container_set_enum_by_name(container, exterior_key, args->exterior);
+    gwy_container_set_boolean_by_name(container, mask_empty_key,
+                                      args->mask_empty);
     gwy_container_set_int32_by_name(container, xres_key, args->xres);
     gwy_container_set_int32_by_name(container, yres_key, args->yres);
 }
